@@ -28,12 +28,14 @@ import org.apache.hadoop.hdds.client.StandaloneReplicationConfig;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.ChunkInfo;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationFactor;
 import org.apache.hadoop.hdds.scm.XceiverClientFactory;
-import org.apache.hadoop.hdds.scm.container.ContainerNotFoundException;
+import org.apache.hadoop.hdds.scm.container.common.helpers.StorageContainerException;
 import org.apache.hadoop.hdds.scm.pipeline.Pipeline;
 import org.apache.hadoop.hdds.scm.pipeline.PipelineID;
 import org.apache.hadoop.hdds.security.token.OzoneBlockTokenIdentifier;
 import org.apache.hadoop.security.token.Token;
 
+import static org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.Result.CONTAINER_NOT_FOUND;
+
 /**
  * A dummy BlockInputStream with pipeline refresh function to mock read
  * block call to DN.
@@ -42,6 +44,7 @@ final class DummyBlockInputStreamWithRetry
     extends DummyBlockInputStream {
 
   private int getChunkInfoCount = 0;
+  private IOException ioException;
 
   @SuppressWarnings("parameternumber")
   DummyBlockInputStreamWithRetry(
@@ -53,25 +56,30 @@ final class DummyBlockInputStreamWithRetry
       XceiverClientFactory xceiverClientManager,
       List<ChunkInfo> chunkList,
       Map<String, byte[]> chunkMap,
-      AtomicBoolean isRerfreshed) {
+      AtomicBoolean isRerfreshed, IOException ioException) {
     super(blockId, blockLen, pipeline, token, verifyChecksum,
         xceiverClientManager, blockID -> {
           isRerfreshed.set(true);
           return Pipeline.newBuilder()
               .setState(Pipeline.PipelineState.OPEN)
               .setId(PipelineID.randomId())
-              .setReplicationConfig(new StandaloneReplicationConfig(
+              .setReplicationConfig(StandaloneReplicationConfig.getInstance(
                   ReplicationFactor.ONE))
               .setNodes(Collections.emptyList())
               .build();
         }, chunkList, chunkMap);
+    this.ioException  = ioException;
   }
 
   @Override
   protected List<ChunkInfo> getChunkInfos() throws IOException {
     if (getChunkInfoCount == 0) {
       getChunkInfoCount++;
-      throw new ContainerNotFoundException("Exception encountered");
+      if (ioException != null) {
+        throw  ioException;
+      }
+      throw new StorageContainerException("Exception encountered",
+          CONTAINER_NOT_FOUND);
     } else {
       return super.getChunkInfos();
     }
