@@ -32,6 +32,7 @@ import org.apache.hadoop.hdds.scm.pipeline.Pipeline;
 import org.apache.hadoop.ozone.common.Checksum;
 
 import org.apache.hadoop.ozone.common.OzoneChecksumException;
+import org.apache.ozone.test.GenericTestUtils;
 import org.apache.ozone.test.LambdaTestUtils;
 import org.junit.Assert;
 import org.junit.Before;
@@ -39,6 +40,8 @@ import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.mockito.Mock;
 import org.mockito.junit.MockitoJUnitRunner;
+import org.slf4j.LoggerFactory;
+import org.slf4j.event.Level;
 
 import java.io.EOFException;
 import java.io.IOException;
@@ -264,7 +267,7 @@ public class TestBlockInputStream {
     BlockInputStream blockInputStreamWithRetry =
         new DummyBlockInputStreamWithRetry(blockID, blockSize,
             MockPipeline.createSingleNodePipeline(), null,
-            false, null, chunks, chunkDataMap, isRefreshed);
+            false, null, chunks, chunkDataMap, isRefreshed, null);
 
     try {
       Assert.assertFalse(isRefreshed.get());
@@ -277,6 +280,33 @@ public class TestBlockInputStream {
     }
   }
 
+  @Test
+  public void testGetBlockInfoFailWithIOException() throws Exception {
+    GenericTestUtils.setLogLevel(BlockInputStream.getLog(), Level.DEBUG);
+    GenericTestUtils.LogCapturer logCapturer =
+        GenericTestUtils.LogCapturer.captureLogs(
+            LoggerFactory.getLogger(BlockInputStream.class));
+    BlockID blockID = new BlockID(new ContainerBlockID(1, 1));
+    AtomicBoolean isRefreshed = new AtomicBoolean();
+    createChunkList(5);
+    BlockInputStream blockInputStreamWithRetry =
+        new DummyBlockInputStreamWithRetry(blockID, blockSize,
+            MockPipeline.createSingleNodePipeline(), null,
+            false, null, chunks, chunkDataMap, isRefreshed,
+            new IOException("unavailable"));
+    try {
+      Assert.assertFalse(isRefreshed.get());
+      byte[] b = new byte[200];
+      blockInputStreamWithRetry.read(b, 0, 200);
+      // As in case of IOException we do not do do refresh.
+      Assert.assertFalse(isRefreshed.get());
+      Assert.assertTrue(logCapturer.getOutput().contains(
+          "Retry to get chunk info fail"));
+    } finally {
+      blockInputStreamWithRetry.close();
+    }
+  }
+
   @Test
   public void testRefreshOnReadFailure() throws Exception {
     // GIVEN
