@@ -31,8 +31,8 @@ import org.apache.hadoop.hdds.scm.XceiverClientSpi;
 import org.apache.hadoop.hdds.scm.pipeline.MockPipeline;
 import org.apache.hadoop.hdds.scm.pipeline.Pipeline;
 import org.apache.hadoop.ozone.common.Checksum;
-import org.apache.ozone.test.GenericTestUtils;
 
+import org.apache.ozone.test.GenericTestUtils;
 import org.apache.ratis.thirdparty.com.google.protobuf.ByteString;
 import org.junit.Assert;
 import org.junit.Before;
@@ -148,13 +148,12 @@ public class TestChunkInputStream {
     seekAndVerify(0);
 
     try {
-      seekAndVerify(CHUNK_SIZE);
-      Assert.fail("Seeking to Chunk Length should fail.");
+      seekAndVerify(CHUNK_SIZE + 1);
+      Assert.fail("Seeking to more than the length of Chunk should fail.");
     } catch (EOFException e) {
       GenericTestUtils.assertExceptionContains("EOF encountered at pos: "
-          + CHUNK_SIZE + " for chunk: " + CHUNK_NAME, e);
+          + (CHUNK_SIZE + 1) + " for chunk: " + CHUNK_NAME, e);
     }
-
     // Seek before read should update the ChunkInputStream#chunkPosition
     seekAndVerify(25);
     Assert.assertEquals(25, chunkStream.getChunkPosition());
