@@ -52,7 +52,7 @@ public final class ReconConfigKeys {
       "0.0.0.0";
   public static final int OZONE_RECON_DATANODE_PORT_DEFAULT = 9891;
   // Prometheus HTTP endpoint including port
-  // ex: http://localhost:9090
+  // ex: http://prometheus:9090
   public static final String OZONE_RECON_PROMETHEUS_HTTP_ENDPOINT =
       "ozone.recon.prometheus.http.endpoint";
   /**
@@ -65,4 +65,7 @@ public final class ReconConfigKeys {
    */
   public static final String OZONE_RECON_ADMINISTRATORS =
       "ozone.recon.administrators";
+
+  public static final String OZONE_RECON_ADMINISTRATORS_GROUPS =
+      "ozone.recon.administrators.groups";
 }
