@@ -25,6 +25,8 @@ import java.time.Instant;
 import java.util.Arrays;
 import java.util.Comparator;
 
+import com.fasterxml.jackson.annotation.JsonProperty;
+import org.apache.hadoop.hdds.client.ECReplicationConfig;
 import org.apache.hadoop.hdds.client.ReplicationConfig;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.hdds.scm.pipeline.PipelineID;
@@ -56,7 +58,12 @@ public class ContainerInfo implements Comparator<ContainerInfo>,
   // The wall-clock ms since the epoch at which the current state enters.
   private Instant stateEnterTime;
   private String owner;
-  private long containerID;
+  // This is JsonIgnored as originally this class held a long in instead of
+  // a containerID object. By emitting this in Json, it changes the JSON output.
+  // Therefore the method getContainerID is annotated to return the original
+  // field and hence maintain the original output.
+  @JsonIgnore
+  private ContainerID containerID;
   // Delete Transaction Id is updated when new transaction for a container
   // is stored in SCM delete Table.
   // TODO: Replication Manager should consider deleteTransactionId so that
@@ -86,7 +93,7 @@ public class ContainerInfo implements Comparator<ContainerInfo>,
       long deleteTransactionId,
       long sequenceId,
       ReplicationConfig repConfig) {
-    this.containerID = containerID;
+    this.containerID = ContainerID.valueOf(containerID);
     this.pipelineID = pipelineID;
     this.usedBytes = usedBytes;
     this.numberOfKeys = numberOfKeys;
@@ -108,7 +115,8 @@ public class ContainerInfo implements Comparator<ContainerInfo>,
   public static ContainerInfo fromProtobuf(HddsProtos.ContainerInfoProto info) {
     ContainerInfo.Builder builder = new ContainerInfo.Builder();
     final ReplicationConfig config = ReplicationConfig
-        .fromProto(info.getReplicationType(), info.getReplicationFactor());
+        .fromProto(info.getReplicationType(), info.getReplicationFactor(),
+            info.getEcReplicationConfig());
     builder.setUsedBytes(info.getUsedBytes())
         .setNumberOfKeys(info.getNumberOfKeys())
         .setState(info.getState())
@@ -128,12 +136,12 @@ public class ContainerInfo implements Comparator<ContainerInfo>,
   }
 
   /**
-   * This method is depricated, use {@code containerID()} which returns
-   * {@link ContainerID} object.
+   * Unless the long value of the ContainerID is needed, use the containerID()
+   * method to obtain the {@link ContainerID} object.
    */
-  @Deprecated
+  @JsonProperty
   public long getContainerID() {
-    return containerID;
+    return containerID.getId();
   }
 
   public HddsProtos.LifeCycleState getState() {
@@ -152,14 +160,42 @@ public class ContainerInfo implements Comparator<ContainerInfo>,
     return replicationConfig;
   }
 
+  @JsonIgnore
   public HddsProtos.ReplicationType getReplicationType() {
     return replicationConfig.getReplicationType();
   }
 
+  @JsonIgnore
+  public HddsProtos.ReplicationFactor getReplicationFactor() {
+    return ReplicationConfig.getLegacyFactor(replicationConfig);
+  }
+
   public PipelineID getPipelineID() {
     return pipelineID;
   }
 
+  /**
+   * Returns the usedBytes for the container. The value returned is derived
+   * from the replicas reported from the datanodes for the container.
+   *
+   * The size of a container can change over time. For an open container we
+   * assume the size of the container will grow, and hence the value will be
+   * the maximum of the values reported from its replicas.
+   *
+   * A closed container can only reduce in size as its blocks are removed. For
+   * a closed container, the value will be the minimum of the values reported
+   * from its replicas.
+   *
+   * An EC container, is made from a group data and parity containers where the
+   * first data and all parity containers should be the same size. The other
+   * data containers can be smaller or the same size. When calculating the EC
+   * container size, we use the min / max of the first data and parity
+   * containers,ignoring the others. For EC containers, this value actually
+   * represents the size of the largest container in the container group, rather
+   * than the total space used by all containers in the group.
+   *
+   * @return bytes used in the container.
+   */
   public long getUsedBytes() {
     return usedBytes;
   }
@@ -194,7 +230,7 @@ public class ContainerInfo implements Comparator<ContainerInfo>,
   }
 
   public ContainerID containerID() {
-    return ContainerID.valueOf(containerID);
+    return containerID;
   }
 
   /**
@@ -214,7 +250,6 @@ public class ContainerInfo implements Comparator<ContainerInfo>,
   public HddsProtos.ContainerInfoProto getProtobuf() {
     HddsProtos.ContainerInfoProto.Builder builder =
         HddsProtos.ContainerInfoProto.newBuilder();
-    Preconditions.checkState(containerID > 0);
     builder.setContainerID(getContainerID())
         .setUsedBytes(getUsedBytes())
         .setNumberOfKeys(getNumberOfKeys()).setState(getState())
@@ -222,10 +257,17 @@ public class ContainerInfo implements Comparator<ContainerInfo>,
         .setContainerID(getContainerID())
         .setDeleteTransactionId(getDeleteTransactionId())
         .setOwner(getOwner())
-        .setSequenceId(getSequenceId());
+        .setSequenceId(getSequenceId())
+        .setReplicationType(getReplicationType());
+
+    if (replicationConfig instanceof ECReplicationConfig) {
+      builder.setEcReplicationConfig(((ECReplicationConfig) replicationConfig)
+          .toProto());
+    } else {
+      builder.setReplicationFactor(
+          ReplicationConfig.getLegacyFactor(replicationConfig));
+    }
 
-    builder.setReplicationFactor(
-        ReplicationConfig.getLegacyFactor(replicationConfig));
     builder.setReplicationType(replicationConfig.getReplicationType());
 
     if (getPipelineID() != null) {
