@@ -39,6 +39,14 @@ public interface NetworkTopology {
    */
   void add(Node node);
 
+  /**
+   * Update a node. This will be called when a datanode needs to be updated.
+   * If the old datanode does not exist, then just add the new datanode.
+   * @param oldNode node to be updated; can be null
+   * @param newNode node to update to; cannot be null
+   */
+  void update(Node oldNode, Node newNode);
+
   /**
    * Remove a node from the network topology. This will be called when a
    * existing datanode is removed from the system.
