@@ -45,7 +45,7 @@ import static org.apache.hadoop.hdds.scm.net.NetConstants.ANCESTOR_GENERATION_DE
  * (computers) and inner nodes represent datacenter/core-switches/routers that
  * manages traffic in/out of data centers or racks.
  */
-public class NetworkTopologyImpl implements NetworkTopology{
+public class NetworkTopologyImpl implements NetworkTopology {
   public static final Logger LOG =
       LoggerFactory.getLogger(NetworkTopologyImpl.class);
 
@@ -91,7 +91,7 @@ public class NetworkTopologyImpl implements NetworkTopology{
     Preconditions.checkArgument(node != null, "node cannot be null");
     if (node instanceof InnerNode) {
       throw new IllegalArgumentException(
-          "Not allowed to add an inner node: "+ node.getNetworkFullPath());
+          "Not allowed to add an inner node: " + node.getNetworkFullPath());
     }
     int newDepth = NetUtils.locationToDepth(node.getNetworkLocation()) + 1;
 
@@ -104,7 +104,7 @@ public class NetworkTopologyImpl implements NetworkTopology{
     boolean add;
     try {
       add = clusterTree.add(node);
-    }finally {
+    } finally {
       netlock.writeLock().unlock();
     }
 
@@ -116,6 +116,59 @@ public class NetworkTopologyImpl implements NetworkTopology{
     }
   }
 
+  /**
+   * Update a leaf node. It is called when a datanode needs to be updated.
+   * If the old datanode does not exist, then just add the new datanode.
+   * @param oldNode node to be updated; can be null
+   * @param newNode node to update to; cannot be null
+   */
+  @Override
+  public void update(Node oldNode, Node newNode) {
+    Preconditions.checkArgument(newNode != null, "newNode cannot be null");
+    if (oldNode != null && oldNode instanceof InnerNode) {
+      throw new IllegalArgumentException(
+              "Not allowed to update an inner node: "
+                      + oldNode.getNetworkFullPath());
+    }
+
+    if (newNode instanceof InnerNode) {
+      throw new IllegalArgumentException(
+              "Not allowed to update a leaf node to an inner node: "
+                      + newNode.getNetworkFullPath());
+    }
+
+    int newDepth = NetUtils.locationToDepth(newNode.getNetworkLocation()) + 1;
+    // Check depth
+    if (maxLevel != newDepth) {
+      throw new InvalidTopologyException("Failed to update to " +
+              newNode.getNetworkFullPath()
+              + ": Its path depth is not "
+              + maxLevel);
+    }
+
+    netlock.writeLock().lock();
+    boolean add;
+    try {
+      boolean exist = false;
+      if (oldNode != null) {
+        exist = containsNode(oldNode);
+      }
+      if (exist) {
+        clusterTree.remove(oldNode);
+      }
+
+      add = clusterTree.add(newNode);
+    } finally {
+      netlock.writeLock().unlock();
+    }
+    if (add) {
+      LOG.info("Updated to the new node: {}", newNode.getNetworkFullPath());
+      if (LOG.isDebugEnabled()) {
+        LOG.debug("NetworkTopology became:\n{}", this);
+      }
+    }
+  }
+
   /**
    * Remove a node from the network topology. This will be called when a
    * existing datanode is removed from the system.
@@ -126,12 +179,12 @@ public class NetworkTopologyImpl implements NetworkTopology{
     Preconditions.checkArgument(node != null, "node cannot be null");
     if (node instanceof InnerNode) {
       throw new IllegalArgumentException(
-          "Not allowed to remove an inner node: "+ node.getNetworkFullPath());
+          "Not allowed to remove an inner node: " + node.getNetworkFullPath());
     }
     netlock.writeLock().lock();
     try {
       clusterTree.remove(node);
-    }finally {
+    } finally {
       netlock.writeLock().unlock();
     }
     LOG.info("Removed a node: {}", node.getNetworkFullPath());
@@ -150,16 +203,20 @@ public class NetworkTopologyImpl implements NetworkTopology{
     Preconditions.checkArgument(node != null, "node cannot be null");
     netlock.readLock().lock();
     try {
-      Node parent = node.getParent();
-      while (parent != null && parent != clusterTree) {
-        parent = parent.getParent();
-      }
-      if (parent == clusterTree) {
-        return true;
-      }
+      return containsNode(node);
     } finally {
       netlock.readLock().unlock();
     }
+  }
+
+  private boolean containsNode(Node node) {
+    Node parent = node.getParent();
+    while (parent != null && parent != clusterTree) {
+      parent = parent.getParent();
+    }
+    if (parent == clusterTree) {
+      return true;
+    }
     return false;
   }
 
@@ -511,6 +568,7 @@ public class NetworkTopologyImpl implements NetworkTopology{
     }
   }
 
+  @SuppressWarnings("java:S2245") // no need for secure random
   private Node chooseNodeInternal(String scope, int leafIndex,
       List<String> excludedScopes, Collection<? extends Node> excludedNodes,
       Node affinityNode, int ancestorGen) {
@@ -534,7 +592,7 @@ public class NetworkTopologyImpl implements NetworkTopology{
             " generation  " + ancestorGen);
       }
       // affinity ancestor should has overlap with scope
-      if (affinityAncestor.getNetworkFullPath().startsWith(scope)){
+      if (affinityAncestor.isDescendant(scope)) {
         finalScope = affinityAncestor.getNetworkFullPath();
       } else if (!scope.startsWith(affinityAncestor.getNetworkFullPath())) {
         return null;
@@ -542,6 +600,11 @@ public class NetworkTopologyImpl implements NetworkTopology{
       // reset ancestor generation since the new scope is identified now
       ancestorGen = 0;
     }
+    Node scopeNode = getNode(finalScope);
+    if (scopeNode == null) {
+      throw new IllegalArgumentException(String.format("No nodes with Scope: " +
+              "%s exists", finalScope));
+    }
 
     // check overlap of excludedScopes and finalScope
     List<String> mutableExcludedScopes = null;
@@ -549,12 +612,13 @@ public class NetworkTopologyImpl implements NetworkTopology{
       mutableExcludedScopes = new ArrayList<>();
       for (String s: excludedScopes) {
         // excludeScope covers finalScope
-        if (finalScope.startsWith(s)) {
+        if (scopeNode.isDescendant(s)) {
           return null;
         }
         // excludeScope and finalScope share nothing case
-        if (s.startsWith(finalScope)) {
-          if (mutableExcludedScopes.stream().noneMatch(s::startsWith)) {
+        if (scopeNode.isAncestor(s)) {
+          if (mutableExcludedScopes.stream().
+              noneMatch(n -> getNode(s).isDescendant(n))) {
             mutableExcludedScopes.add(s);
           }
         }
@@ -581,7 +645,6 @@ public class NetworkTopologyImpl implements NetworkTopology{
         ancestorGen);
 
     // calculate available node count
-    Node scopeNode = getNode(finalScope);
     int availableNodes = getAvailableNodesCount(
         scopeNode.getNetworkFullPath(), mutableExcludedScopes, mutableExNodes,
         ancestorGen);
@@ -655,21 +718,21 @@ public class NetworkTopologyImpl implements NetworkTopology{
       if (level1 > maxLevel || level2 > maxLevel) {
         return Integer.MAX_VALUE;
       }
-      while(level1 > level2 && node1 != null) {
+      while (level1 > level2 && node1 != null) {
         node1 = node1.getParent();
         level1--;
-        cost += node1 == null? 0 : node1.getCost();
+        cost += node1 == null ? 0 : node1.getCost();
       }
-      while(level2 > level1 && node2 != null) {
+      while (level2 > level1 && node2 != null) {
         node2 = node2.getParent();
         level2--;
-        cost += node2 == null? 0 : node2.getCost();
+        cost += node2 == null ? 0 : node2.getCost();
       }
-      while(node1 != null && node2 != null && node1 != node2) {
+      while (node1 != null && node2 != null && node1 != node2) {
         node1 = node1.getParent();
         node2 = node2.getParent();
-        cost += node1 == null? 0 : node1.getCost();
-        cost += node2 == null? 0 : node2.getCost();
+        cost += node1 == null ? 0 : node1.getCost();
+        cost += node2 == null ? 0 : node2.getCost();
       }
       return cost;
     } finally {
@@ -746,13 +809,12 @@ public class NetworkTopologyImpl implements NetworkTopology{
       return 0;
     }
     if (!CollectionUtils.isEmpty(mutableExcludedNodes)) {
-      mutableExcludedNodes.removeIf(
-          next -> !next.getNetworkFullPath().startsWith(scope));
+      mutableExcludedNodes.removeIf(next -> !next.isDescendant(scope));
     }
     List<Node> excludedAncestorList =
         NetUtils.getAncestorList(this, mutableExcludedNodes, ancestorGen);
     for (Node ancestor : excludedAncestorList) {
-      if (scope.startsWith(ancestor.getNetworkFullPath())){
+      if (ancestor.isAncestor(scope)) {
         return 0;
       }
     }
@@ -762,9 +824,9 @@ public class NetworkTopologyImpl implements NetworkTopology{
       for (String excludedScope: excludedScopes) {
         Node excludedScopeNode = getNode(excludedScope);
         if (excludedScopeNode != null) {
-          if (excludedScope.startsWith(scope)) {
+          if (excludedScopeNode.isDescendant(scope)) {
             excludedCount += excludedScopeNode.getNumOfLeaves();
-          } else if (scope.startsWith(excludedScope)) {
+          } else if (excludedScopeNode.isAncestor(scope)) {
             return 0;
           }
         }
@@ -780,7 +842,7 @@ public class NetworkTopologyImpl implements NetworkTopology{
         }
       } else {
         for (Node ancestor : excludedAncestorList) {
-          if (ancestor.getNetworkFullPath().startsWith(scope)) {
+          if (ancestor.isDescendant(scope)) {
             excludedCount += ancestor.getNumOfLeaves();
           }
         }
