@@ -48,7 +48,9 @@ public enum SCMAction implements AuditAction {
   STOP_CONTAINER_BALANCER,
   GET_CONTAINER_BALANCER_STATUS,
   GET_CONTAINER_WITH_PIPELINE_BATCH,
-  ADD_SCM;
+  ADD_SCM,
+  GET_REPLICATION_MANAGER_REPORT,
+  RESET_DELETED_BLOCK_RETRY_COUNT;
 
   @Override
   public String getAction() {
