@@ -43,5 +43,8 @@ public enum ConfigTag {
   S3GATEWAY,
   DATANODE,
   RECON,
-  DELETION
+  DELETION,
+  HA,
+  BALANCER,
+  UPGRADE
 }
