@@ -16,6 +16,7 @@
  */
 package org.apache.hadoop.ozone.container.common.statemachine;
 
+import java.io.IOException;
 import java.net.InetSocketAddress;
 import java.util.ArrayList;
 import java.util.HashMap;
@@ -61,13 +62,15 @@ import org.apache.hadoop.ozone.protocol.commands.DeleteBlockCommandStatus.Delete
 import org.apache.hadoop.ozone.protocol.commands.SCMCommand;
 
 import com.google.common.base.Preconditions;
-import com.google.protobuf.GeneratedMessage;
+import com.google.protobuf.Message;
 import static java.lang.Math.min;
 import static org.apache.hadoop.hdds.utils.HddsServerUtil.getLogWarnInterval;
+import static org.apache.hadoop.hdds.utils.HddsServerUtil.getReconHeartbeatInterval;
 import static org.apache.hadoop.hdds.utils.HddsServerUtil.getScmHeartbeatInterval;
 
 import org.apache.commons.collections.CollectionUtils;
 
+import org.apache.hadoop.util.Time;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -100,17 +103,17 @@ public class StateContext {
   private final Queue<SCMCommand> commandQueue;
   private final Map<Long, CommandStatus> cmdStatusMap;
   private final Lock lock;
-  private final DatanodeStateMachine parent;
+  private final DatanodeStateMachine parentDatanodeStateMachine;
   private final AtomicLong stateExecutionCount;
   private final ConfigurationSource conf;
   private final Set<InetSocketAddress> endpoints;
   // Only the latest full report of each type is kept
-  private final AtomicReference<GeneratedMessage> containerReports;
-  private final AtomicReference<GeneratedMessage> nodeReport;
-  private final AtomicReference<GeneratedMessage> pipelineReports;
-  private final AtomicReference<GeneratedMessage> crlStatusReport;
+  private final AtomicReference<Message> containerReports;
+  private final AtomicReference<Message> nodeReport;
+  private final AtomicReference<Message> pipelineReports;
+  private final AtomicReference<Message> crlStatusReport;
   // Incremental reports are queued in the map below
-  private final Map<InetSocketAddress, List<GeneratedMessage>>
+  private final Map<InetSocketAddress, List<Message>>
       incrementalReportsQueue;
   private final Map<InetSocketAddress, Queue<ContainerAction>> containerActions;
   private final Map<InetSocketAddress, Queue<PipelineAction>> pipelineActions;
@@ -118,14 +121,15 @@ public class StateContext {
   private boolean shutdownOnError = false;
   private boolean shutdownGracefully = false;
   private final AtomicLong threadPoolNotAvailableCount;
+  private final AtomicLong lastHeartbeatSent;
   // Endpoint -> ReportType -> Boolean of whether the full report should be
   //  queued in getFullReports call.
   private final Map<InetSocketAddress,
-      Map<String, AtomicBoolean>> fullReportSendIndicator;
+      Map<String, AtomicBoolean>> isFullReportReadyToBeSent;
   // List of supported full report types.
   private final List<String> fullReportTypeList;
   // ReportType -> Report.
-  private final Map<String, AtomicReference<GeneratedMessage>> type2Reports;
+  private final Map<String, AtomicReference<Message>> type2Reports;
 
   /**
    * term of latest leader SCM, extract from SCMCommand.
@@ -146,6 +150,7 @@ public class StateContext {
    */
   private final AtomicLong heartbeatFrequency = new AtomicLong(2000);
 
+  private final AtomicLong reconHeartbeatFrequency = new AtomicLong(2000);
   /**
    * Constructs a StateContext.
    *
@@ -158,21 +163,22 @@ public class StateContext {
           state, DatanodeStateMachine parent) {
     this.conf = conf;
     this.state = state;
-    this.parent = parent;
+    this.parentDatanodeStateMachine = parent;
     commandQueue = new LinkedList<>();
     cmdStatusMap = new ConcurrentHashMap<>();
     incrementalReportsQueue = new HashMap<>();
     containerReports = new AtomicReference<>();
     nodeReport = new AtomicReference<>();
     pipelineReports = new AtomicReference<>();
-    crlStatusReport = new AtomicReference<>();
+    crlStatusReport = new AtomicReference<>(); // Certificate Revocation List
     endpoints = new HashSet<>();
     containerActions = new HashMap<>();
     pipelineActions = new HashMap<>();
     lock = new ReentrantLock();
     stateExecutionCount = new AtomicLong(0);
     threadPoolNotAvailableCount = new AtomicLong(0);
-    fullReportSendIndicator = new HashMap<>();
+    lastHeartbeatSent = new AtomicLong(0);
+    isFullReportReadyToBeSent = new HashMap<>();
     fullReportTypeList = new ArrayList<>();
     type2Reports = new HashMap<>();
     initReportTypeCollection();
@@ -181,7 +187,7 @@ public class StateContext {
   /**
    * init related ReportType Collections.
    */
-  private void initReportTypeCollection(){
+  private void initReportTypeCollection() {
     fullReportTypeList.add(CONTAINER_REPORTS_PROTO_NAME);
     type2Reports.put(CONTAINER_REPORTS_PROTO_NAME, containerReports);
     fullReportTypeList.add(NODE_REPORT_PROTO_NAME);
@@ -193,12 +199,12 @@ public class StateContext {
   }
 
   /**
-   * Returns the ContainerStateMachine class that holds this state.
+   * Returns the DatanodeStateMachine class that holds this state.
    *
-   * @return ContainerStateMachine.
+   * @return DatanodeStateMachine.
    */
   public DatanodeStateMachine getParent() {
-    return parent;
+    return parentDatanodeStateMachine;
   }
 
   /**
@@ -218,7 +224,7 @@ public class StateContext {
    */
   boolean isExiting(DatanodeStateMachine.DatanodeStates newState) {
     boolean isExiting = state != newState && stateExecutionCount.get() > 0;
-    if(isExiting) {
+    if (isExiting) {
       stateExecutionCount.set(0);
     }
     return isExiting;
@@ -277,7 +283,7 @@ public class StateContext {
    *
    * @param report report to be added
    */
-  public void addIncrementalReport(GeneratedMessage report) {
+  public void addIncrementalReport(Message report) {
     if (report == null) {
       return;
     }
@@ -300,7 +306,7 @@ public class StateContext {
    *
    * @param report report to be refreshed
    */
-  public void refreshFullReport(GeneratedMessage report) {
+  public void refreshFullReport(Message report) {
     if (report == null) {
       return;
     }
@@ -313,8 +319,8 @@ public class StateContext {
           "not full report message type: " + reportType);
     }
     type2Reports.get(reportType).set(report);
-    if (fullReportSendIndicator != null) {
-      for (Map<String, AtomicBoolean> mp : fullReportSendIndicator.values()) {
+    if (isFullReportReadyToBeSent != null) {
+      for (Map<String, AtomicBoolean> mp : isFullReportReadyToBeSent.values()) {
         mp.get(reportType).set(true);
       }
     }
@@ -327,21 +333,21 @@ public class StateContext {
    * @param reportsToPutBack list of reports which failed to be sent by
    *                         heartbeat.
    */
-  public void putBackReports(List<GeneratedMessage> reportsToPutBack,
+  public void putBackReports(List<Message> reportsToPutBack,
                              InetSocketAddress endpoint) {
     if (LOG.isDebugEnabled()) {
       LOG.debug("endpoint: {}, size of reportsToPutBack: {}",
           endpoint, reportsToPutBack.size());
     }
     // We don't expect too much reports to be put back
-    for (GeneratedMessage report : reportsToPutBack) {
+    for (Message report : reportsToPutBack) {
       final Descriptor descriptor = report.getDescriptorForType();
       Preconditions.checkState(descriptor != null);
       final String reportType = descriptor.getFullName();
       Preconditions.checkState(reportType != null);
     }
     synchronized (incrementalReportsQueue) {
-      if (incrementalReportsQueue.containsKey(endpoint)){
+      if (incrementalReportsQueue.containsKey(endpoint)) {
         incrementalReportsQueue.get(endpoint).addAll(0, reportsToPutBack);
       }
     }
@@ -353,19 +359,68 @@ public class StateContext {
    *
    * @return List of reports
    */
-  public List<GeneratedMessage> getAllAvailableReports(
-      InetSocketAddress endpoint) {
-    return getReports(endpoint, Integer.MAX_VALUE);
+  public List<Message> getAllAvailableReports(
+      InetSocketAddress endpoint
+  ) {
+    int maxLimit = Integer.MAX_VALUE;
+    // TODO: It is highly unlikely that we will reach maxLimit for the number
+    //       for the number of reports, specially as it does not apply to the
+    //       number of entries in a report. But if maxLimit is hit, should a
+    //       heartbeat be scheduled ASAP? Should full reports not included be
+    //       dropped? Currently this code will keep the full reports not sent
+    //       and include it in the next heartbeat.
+    return getAllAvailableReportsUpToLimit(endpoint, maxLimit);
   }
 
-  List<GeneratedMessage> getIncrementalReports(
+  /**
+   * Gets a point in time snapshot of all containers, any pending incremental
+   * container reports (ICR) for containers will be included in this report
+   * and this call will drop any pending ICRs.
+   * @return Full Container Report
+   */
+  public ContainerReportsProto getFullContainerReportDiscardPendingICR()
+      throws IOException {
+
+    // Block ICRs from being generated
+    synchronized (parentDatanodeStateMachine
+        .getContainer()) {
+      synchronized (incrementalReportsQueue) {
+        for (Map.Entry<InetSocketAddress, List<Message>>
+            entry : incrementalReportsQueue.entrySet()) {
+          if (entry.getValue() != null) {
+            entry.getValue().removeIf(
+                generatedMessage ->
+                    generatedMessage instanceof
+                        IncrementalContainerReportProto);
+          }
+        }
+      }
+      return parentDatanodeStateMachine
+          .getContainer()
+          .getContainerSet()
+          .getContainerReport();
+    }
+  }
+
+  @VisibleForTesting
+  List<Message> getAllAvailableReportsUpToLimit(
+      InetSocketAddress endpoint,
+      int limit) {
+    List<Message> reports = getFullReports(endpoint, limit);
+    List<Message> incrementalReports = getIncrementalReports(endpoint,
+        limit - reports.size()); // get all (MAX_VALUE)
+    reports.addAll(incrementalReports);
+    return reports;
+  }
+
+  List<Message> getIncrementalReports(
       InetSocketAddress endpoint, int maxLimit) {
-    List<GeneratedMessage> reportsToReturn = new LinkedList<>();
+    List<Message> reportsToReturn = new LinkedList<>();
     synchronized (incrementalReportsQueue) {
-      List<GeneratedMessage> reportsForEndpoint =
+      List<Message> reportsForEndpoint =
           incrementalReportsQueue.get(endpoint);
       if (reportsForEndpoint != null) {
-        List<GeneratedMessage> tempList = reportsForEndpoint.subList(
+        List<Message> tempList = reportsForEndpoint.subList(
             0, min(reportsForEndpoint.size(), maxLimit));
         reportsToReturn.addAll(tempList);
         tempList.clear();
@@ -374,53 +429,37 @@ public class StateContext {
     return reportsToReturn;
   }
 
-  List<GeneratedMessage> getFullReports(
-      InetSocketAddress endpoint) {
-    Map<String, AtomicBoolean> mp = fullReportSendIndicator.get(endpoint);
-    List<GeneratedMessage> nonIncrementalReports = new LinkedList<>();
-    if (null != mp){
+  List<Message> getFullReports(
+      InetSocketAddress endpoint, int maxLimit) {
+    int count = 0;
+    Map<String, AtomicBoolean> mp = isFullReportReadyToBeSent.get(endpoint);
+    List<Message> fullReports = new LinkedList<>();
+    if (null != mp) {
       for (Map.Entry<String, AtomicBoolean> kv : mp.entrySet()) {
+        if (count == maxLimit) {
+          break;
+        }
         if (kv.getValue().get()) {
           String reportType = kv.getKey();
-          final AtomicReference<GeneratedMessage> ref =
+          final AtomicReference<Message> ref =
               type2Reports.get(reportType);
           if (ref == null) {
             throw new RuntimeException(reportType + " is not a valid full "
                 + "report type!");
           }
-          final GeneratedMessage msg = ref.get();
+          final Message msg = ref.get();
           if (msg != null) {
-            nonIncrementalReports.add(msg);
+            fullReports.add(msg);
+            // Mark the report as not ready to be sent, until another refresh.
             mp.get(reportType).set(false);
+            count++;
           }
         }
       }
     }
-    return nonIncrementalReports;
+    return fullReports;
   }
 
-  /**
-   * Returns available reports from the report queue with a max limit on
-   * list size, or empty list if the queue is empty.
-   *
-   * @return List of reports
-   */
-  public List<GeneratedMessage> getReports(InetSocketAddress endpoint,
-                                           int maxLimit) {
-    if (maxLimit < 0) {
-      throw new IllegalArgumentException("Illegal maxLimit value: " + maxLimit);
-    }
-    List<GeneratedMessage> reports = getFullReports(endpoint);
-    if (maxLimit <= reports.size()) {
-      return reports.subList(0, maxLimit);
-    } else {
-      reports.addAll(getIncrementalReports(endpoint,
-          maxLimit - reports.size()));
-      return reports;
-    }
-  }
-
-
   /**
    * Adds the ContainerAction to ContainerAction queue.
    *
@@ -449,17 +488,6 @@ public class StateContext {
     }
   }
 
-  /**
-   * Returns all the pending ContainerActions from the ContainerAction queue,
-   * or empty list if the queue is empty.
-   *
-   * @return {@literal List<ContainerAction>}
-   */
-  public List<ContainerAction> getAllPendingContainerActions(
-      InetSocketAddress endpoint) {
-    return getPendingContainerAction(endpoint, Integer.MAX_VALUE);
-  }
-
   /**
    * Returns pending ContainerActions from the ContainerAction queue with a
    * max limit on list size, or empty list if the queue is empty.
@@ -566,10 +594,12 @@ public class StateContext {
   public DatanodeState<DatanodeStateMachine.DatanodeStates> getTask() {
     switch (this.state) {
     case INIT:
-      return new InitDatanodeState(this.conf, parent.getConnectionManager(),
+      return new InitDatanodeState(this.conf,
+          parentDatanodeStateMachine.getConnectionManager(),
           this);
     case RUNNING:
-      return new RunningDatanodeState(this.conf, parent.getConnectionManager(),
+      return new RunningDatanodeState(this.conf,
+          parentDatanodeStateMachine.getConnectionManager(),
           this);
     case SHUTDOWN:
       return null;
@@ -616,16 +646,17 @@ public class StateContext {
       }
 
       if (!isThreadPoolAvailable(service)) {
-        long count = threadPoolNotAvailableCount.getAndIncrement();
-        if (count % getLogWarnInterval(conf) == 0) {
-          LOG.warn("No available thread in pool for past {} seconds.",
-              unit.toSeconds(time) * (count + 1));
+        long count = threadPoolNotAvailableCount.incrementAndGet();
+        long unavailableTime = Time.monotonicNow() - lastHeartbeatSent.get();
+        if (unavailableTime > time && count % getLogWarnInterval(conf) == 0) {
+          LOG.warn("No available thread in pool for the past {} seconds " +
+              "and {} times.", unit.toSeconds(unavailableTime), count);
         }
         return;
       }
-
       threadPoolNotAvailableCount.set(0);
       task.execute(service);
+      lastHeartbeatSent.set(Time.monotonicNow());
       DatanodeStateMachine.DatanodeStates newState = task.await(time, unit);
       if (this.state != newState) {
         if (LOG.isDebugEnabled()) {
@@ -755,6 +786,19 @@ public class StateContext {
     this.addCmdStatus(command);
   }
 
+  public Map<SCMCommandProto.Type, Integer> getCommandQueueSummary() {
+    Map<SCMCommandProto.Type, Integer> summary = new HashMap<>();
+    lock.lock();
+    try {
+      for (SCMCommand cmd : commandQueue) {
+        summary.put(cmd.getType(), summary.getOrDefault(cmd.getType(), 0) + 1);
+      }
+    } finally {
+      lock.unlock();
+    }
+    return summary;
+  }
+
   /**
    * Returns the count of the Execution.
    * @return long
@@ -813,14 +857,14 @@ public class StateContext {
    */
   public boolean updateCommandStatus(Long cmdId,
       Consumer<CommandStatus> cmdStatusUpdater) {
-    if(cmdStatusMap.containsKey(cmdId)) {
+    if (cmdStatusMap.containsKey(cmdId)) {
       cmdStatusUpdater.accept(cmdStatusMap.get(cmdId));
       return true;
     }
     return false;
   }
 
-  public void configureHeartbeatFrequency(){
+  public void configureHeartbeatFrequency() {
     heartbeatFrequency.set(getScmHeartbeatInterval(conf));
   }
 
@@ -841,27 +885,38 @@ public class StateContext {
       fullReportTypeList.forEach(e -> {
         mp.putIfAbsent(e, new AtomicBoolean(true));
       });
-      this.fullReportSendIndicator.putIfAbsent(endpoint, mp);
+      this.isFullReportReadyToBeSent.putIfAbsent(endpoint, mp);
     }
   }
 
   @VisibleForTesting
-  public GeneratedMessage getContainerReports() {
+  public Message getContainerReports() {
     return containerReports.get();
   }
 
   @VisibleForTesting
-  public GeneratedMessage getNodeReport() {
+  public Message getNodeReport() {
     return nodeReport.get();
   }
 
   @VisibleForTesting
-  public GeneratedMessage getPipelineReports() {
+  public Message getPipelineReports() {
     return pipelineReports.get();
   }
 
   @VisibleForTesting
-  public GeneratedMessage getCRLStatusReport() {
+  public Message getCRLStatusReport() {
     return crlStatusReport.get();
   }
+
+  public void configureReconHeartbeatFrequency() {
+    reconHeartbeatFrequency.set(getReconHeartbeatInterval(conf));
+  }
+
+  /**
+   * Return current Datanode to Recon heartbeat frequency in ms.
+   */
+  public long getReconHeartbeatFrequency() {
+    return reconHeartbeatFrequency.get();
+  }
 }
