@@ -62,14 +62,13 @@ public class CloseContainerCommandHandler implements CommandHandler {
    * Handles a given SCM command.
    *
    * @param command           - SCM Command
-   * @param ozoneContainer         - Ozone Container.
+   * @param ozoneContainer    - Ozone Container.
    * @param context           - Current Context.
    * @param connectionManager - The SCMs that we are talking to.
    */
   @Override
   public void handle(SCMCommand command, OzoneContainer ozoneContainer,
       StateContext context, SCMConnectionManager connectionManager) {
-    LOG.debug("Processing Close Container command.");
     invocationCount.incrementAndGet();
     final long startTime = Time.monotonicNow();
     final DatanodeDetails datanodeDetails = context.getParent()
@@ -78,6 +77,8 @@ public class CloseContainerCommandHandler implements CommandHandler {
         ((CloseContainerCommand)command).getProto();
     final ContainerController controller = ozoneContainer.getController();
     final long containerId = closeCommand.getContainerID();
+    LOG.debug("Processing Close Container command container #{}",
+        containerId);
     try {
       final Container container = controller.getContainer(containerId);
 
@@ -102,11 +103,13 @@ public class CloseContainerCommandHandler implements CommandHandler {
                   command.getEncodedToken());
           ozoneContainer.getWriteChannel()
               .submitRequest(request, closeCommand.getPipelineID());
+        } else if (closeCommand.getForce()) {
+          // Non-RATIS containers should have the force close flag set, so they
+          // are moved to CLOSED immediately rather than going to quasi-closed.
+          controller.closeContainer(containerId);
         } else {
-          // Container should not exist in CLOSING state without a pipeline
-          controller.markContainerUnhealthy(containerId);
-          LOG.info("Marking UNHEALTHY as Container should not be in " +
-              "CLOSING state without pipeline, ContainerID: {}", containerId);
+          controller.quasiCloseContainer(containerId);
+          LOG.info("Marking Container {} quasi closed", containerId);
         }
         break;
       case QUASI_CLOSED:
@@ -118,10 +121,8 @@ public class CloseContainerCommandHandler implements CommandHandler {
         break;
       case UNHEALTHY:
       case INVALID:
-        if (LOG.isDebugEnabled()) {
-          LOG.debug("Cannot close the container #{}, the container is"
-              + " in {} state.", containerId, container.getContainerState());
-        }
+        LOG.debug("Cannot close the container #{}, the container is"
+            + " in {} state.", containerId, container.getContainerState());
         break;
       default:
         break;
@@ -185,4 +186,9 @@ public class CloseContainerCommandHandler implements CommandHandler {
     }
     return 0;
   }
+
+  @Override
+  public int getQueuedCount() {
+    return 0;
+  }
 }
