@@ -20,10 +20,11 @@ package org.apache.hadoop.ozone.container.common.states.endpoint;
 
 import com.google.common.base.Preconditions;
 import com.google.protobuf.Descriptors;
-import com.google.protobuf.GeneratedMessage;
+import com.google.protobuf.Message;
 import org.apache.hadoop.hdds.conf.ConfigurationSource;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos.DatanodeDetailsProto;
+import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.CommandQueueReportProto;
 import org.apache.hadoop.hdds.protocol.proto
     .StorageContainerDatanodeProtocolProtos.LayoutVersionProto;
 import org.apache.hadoop.hdds.protocol.proto
@@ -54,8 +55,11 @@ import org.apache.hadoop.ozone.protocol.commands.CreatePipelineCommand;
 import org.apache.hadoop.ozone.protocol.commands.DeleteBlocksCommand;
 import org.apache.hadoop.ozone.protocol.commands.DeleteContainerCommand;
 import org.apache.hadoop.ozone.protocol.commands.FinalizeNewLayoutVersionCommand;
+import org.apache.hadoop.ozone.protocol.commands.ReconstructECContainersCommand;
+import org.apache.hadoop.ozone.protocol.commands.RefreshVolumeUsageCommand;
 import org.apache.hadoop.ozone.protocol.commands.ReplicateContainerCommand;
 
+import org.apache.hadoop.ozone.protocol.commands.SCMCommand;
 import org.apache.hadoop.ozone.protocol.commands.SetNodeOperationalStateCommand;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -64,6 +68,7 @@ import java.io.IOException;
 import java.time.ZonedDateTime;
 import java.util.LinkedList;
 import java.util.List;
+import java.util.Map;
 import java.util.concurrent.Callable;
 
 import static org.apache.hadoop.hdds.HddsConfigKeys
@@ -171,6 +176,7 @@ public class HeartbeatEndpointTask
       addReports(requestBuilder);
       addContainerActions(requestBuilder);
       addPipelineActions(requestBuilder);
+      addQueuedCommandCounts(requestBuilder);
       SCMHeartbeatRequestProto request = requestBuilder.build();
       if (LOG.isDebugEnabled()) {
         LOG.debug("Sending heartbeat message :: {}", request.toString());
@@ -194,7 +200,7 @@ public class HeartbeatEndpointTask
   // TODO: Make it generic.
   private void putBackIncrementalReports(
       SCMHeartbeatRequestProto.Builder requestBuilder) {
-    List<GeneratedMessage> reports = new LinkedList<>();
+    List<Message> reports = new LinkedList<>();
     // We only put back CommandStatusReports and IncrementalContainerReport
     // because those are incremental. Container/Node/PipelineReport are
     // accumulative so we can keep only the latest of each.
@@ -213,7 +219,7 @@ public class HeartbeatEndpointTask
    * @param requestBuilder builder to which the report has to be added.
    */
   private void addReports(SCMHeartbeatRequestProto.Builder requestBuilder) {
-    for (GeneratedMessage report :
+    for (Message report :
         context.getAllAvailableReports(rpcEndpoint.getAddress())) {
       String reportName = report.getDescriptorForType().getFullName();
       for (Descriptors.FieldDescriptor descriptor :
@@ -265,6 +271,24 @@ public class HeartbeatEndpointTask
     }
   }
 
+  /**
+   * Adds the count of all queued commands to the heartbeat.
+   * @param requestBuilder Builder to which the details will be added.
+   */
+  private void addQueuedCommandCounts(
+      SCMHeartbeatRequestProto.Builder requestBuilder) {
+    Map<SCMCommandProto.Type, Integer> commandCount =
+        context.getParent().getQueuedCommandCount();
+    CommandQueueReportProto.Builder reportProto =
+        CommandQueueReportProto.newBuilder();
+    for (Map.Entry<SCMCommandProto.Type, Integer> entry
+        : commandCount.entrySet()) {
+      reportProto.addCommand(entry.getKey())
+          .addCount(entry.getValue());
+    }
+    requestBuilder.setCommandQueueReport(reportProto.build());
+  }
+
   /**
    * Returns a builder class for HeartbeatEndpointTask task.
    * @return   Builder.
@@ -284,136 +308,112 @@ public class HeartbeatEndpointTask
             .equalsIgnoreCase(datanodeDetails.getUuid()),
         "Unexpected datanode ID in the response.");
     // Verify the response is indeed for this datanode.
-    for (SCMCommandProto commandResponseProto : response
-        .getCommandsList()) {
+    for (SCMCommandProto commandResponseProto : response.getCommandsList()) {
       switch (commandResponseProto.getCommandType()) {
       case reregisterCommand:
-        if (rpcEndpoint.getState() == EndPointStates.HEARTBEAT) {
-          if (LOG.isDebugEnabled()) {
-            LOG.debug("Received SCM notification to register."
-                + " Interrupt HEARTBEAT and transit to REGISTER state.");
-          }
-          rpcEndpoint.setState(EndPointStates.REGISTER);
-        } else {
-          if (LOG.isDebugEnabled()) {
-            LOG.debug("Illegal state {} found, expecting {}.",
-                rpcEndpoint.getState().name(), EndPointStates.HEARTBEAT);
-          }
-        }
+        processReregisterCommand();
         break;
       case deleteBlocksCommand:
-        DeleteBlocksCommand db = DeleteBlocksCommand
+        DeleteBlocksCommand deleteBlocksCommand = DeleteBlocksCommand
             .getFromProtobuf(
                 commandResponseProto.getDeleteBlocksCommandProto());
-        if (commandResponseProto.hasTerm()) {
-          db.setTerm(commandResponseProto.getTerm());
-        }
-        if (!db.blocksTobeDeleted().isEmpty()) {
+        if (!deleteBlocksCommand.blocksTobeDeleted().isEmpty()) {
           if (LOG.isDebugEnabled()) {
             LOG.debug(DeletedContainerBlocksSummary
-                .getFrom(db.blocksTobeDeleted())
+                .getFrom(deleteBlocksCommand.blocksTobeDeleted())
                 .toString());
           }
-          this.context.addCommand(db);
+          processCommonCommand(commandResponseProto, deleteBlocksCommand);
         }
         break;
       case closeContainerCommand:
         CloseContainerCommand closeContainer =
             CloseContainerCommand.getFromProtobuf(
                 commandResponseProto.getCloseContainerCommandProto());
-        if (commandResponseProto.hasTerm()) {
-          closeContainer.setTerm(commandResponseProto.getTerm());
-        }
-        if (commandResponseProto.hasEncodedToken()) {
-          closeContainer.setEncodedToken(
-              commandResponseProto.getEncodedToken());
-        }
         if (LOG.isDebugEnabled()) {
           LOG.debug("Received SCM container close request for container {}",
               closeContainer.getContainerID());
         }
-        this.context.addCommand(closeContainer);
+        processCommonCommand(commandResponseProto, closeContainer);
         break;
       case replicateContainerCommand:
         ReplicateContainerCommand replicateContainerCommand =
             ReplicateContainerCommand.getFromProtobuf(
                 commandResponseProto.getReplicateContainerCommandProto());
-        if (commandResponseProto.hasTerm()) {
-          replicateContainerCommand.setTerm(commandResponseProto.getTerm());
-        }
         if (LOG.isDebugEnabled()) {
           LOG.debug("Received SCM container replicate request for container {}",
               replicateContainerCommand.getContainerID());
         }
-        this.context.addCommand(replicateContainerCommand);
+        processCommonCommand(commandResponseProto, replicateContainerCommand);
+        break;
+      case reconstructECContainersCommand:
+        ReconstructECContainersCommand reccc =
+            ReconstructECContainersCommand.getFromProtobuf(
+                commandResponseProto.getReconstructECContainersCommandProto());
+        if (LOG.isDebugEnabled()) {
+          LOG.debug("Received SCM reconstruct request for container {}",
+              reccc.getContainerID());
+        }
+        processCommonCommand(commandResponseProto, reccc);
         break;
       case deleteContainerCommand:
         DeleteContainerCommand deleteContainerCommand =
             DeleteContainerCommand.getFromProtobuf(
                 commandResponseProto.getDeleteContainerCommandProto());
-        if (commandResponseProto.hasTerm()) {
-          deleteContainerCommand.setTerm(commandResponseProto.getTerm());
-        }
         if (LOG.isDebugEnabled()) {
           LOG.debug("Received SCM delete container request for container {}",
               deleteContainerCommand.getContainerID());
         }
-        this.context.addCommand(deleteContainerCommand);
+        processCommonCommand(commandResponseProto, deleteContainerCommand);
         break;
       case createPipelineCommand:
         CreatePipelineCommand createPipelineCommand =
             CreatePipelineCommand.getFromProtobuf(
                 commandResponseProto.getCreatePipelineCommandProto());
-        if (commandResponseProto.hasTerm()) {
-          createPipelineCommand.setTerm(commandResponseProto.getTerm());
-        }
         if (LOG.isDebugEnabled()) {
           LOG.debug("Received SCM create pipeline request {}",
               createPipelineCommand.getPipelineID());
         }
-        this.context.addCommand(createPipelineCommand);
+        processCommonCommand(commandResponseProto, createPipelineCommand);
         break;
       case closePipelineCommand:
         ClosePipelineCommand closePipelineCommand =
             ClosePipelineCommand.getFromProtobuf(
                 commandResponseProto.getClosePipelineCommandProto());
-        if (commandResponseProto.hasTerm()) {
-          closePipelineCommand.setTerm(commandResponseProto.getTerm());
-        }
         if (LOG.isDebugEnabled()) {
           LOG.debug("Received SCM close pipeline request {}",
               closePipelineCommand.getPipelineID());
         }
-        this.context.addCommand(closePipelineCommand);
+        processCommonCommand(commandResponseProto, closePipelineCommand);
         break;
       case setNodeOperationalStateCommand:
         SetNodeOperationalStateCommand setNodeOperationalStateCommand =
             SetNodeOperationalStateCommand.getFromProtobuf(
                 commandResponseProto.getSetNodeOperationalStateCommandProto());
-        if (commandResponseProto.hasTerm()) {
-          setNodeOperationalStateCommand.setTerm(
-              commandResponseProto.getTerm());
-        }
         if (LOG.isDebugEnabled()) {
           LOG.debug("Received SCM set operational state command. State: {} " +
               "Expiry: {}", setNodeOperationalStateCommand.getOpState(),
               setNodeOperationalStateCommand.getStateExpiryEpochSeconds());
         }
-        this.context.addCommand(setNodeOperationalStateCommand);
+        processCommonCommand(commandResponseProto,
+            setNodeOperationalStateCommand);
         break;
       case finalizeNewLayoutVersionCommand:
         FinalizeNewLayoutVersionCommand finalizeNewLayoutVersionCommand =
             FinalizeNewLayoutVersionCommand.getFromProtobuf(
                 commandResponseProto.getFinalizeNewLayoutVersionCommandProto());
-        if (commandResponseProto.hasTerm()) {
-          finalizeNewLayoutVersionCommand.setTerm(
-              commandResponseProto.getTerm());
-        }
         if (LOG.isDebugEnabled()) {
           LOG.debug("Received SCM finalize command {}",
               finalizeNewLayoutVersionCommand.getId());
         }
-        this.context.addCommand(finalizeNewLayoutVersionCommand);
+        processCommonCommand(commandResponseProto,
+            finalizeNewLayoutVersionCommand);
+        break;
+      case refreshVolumeUsageInfo:
+        RefreshVolumeUsageCommand refreshVolumeUsageCommand =
+            RefreshVolumeUsageCommand.getFromProtobuf(
+            commandResponseProto.getRefreshVolumeUsageCommandProto());
+        processCommonCommand(commandResponseProto, refreshVolumeUsageCommand);
         break;
       default:
         throw new IllegalArgumentException("Unknown response : "
@@ -422,6 +422,38 @@ public class HeartbeatEndpointTask
     }
   }
 
+  /**
+   * Common processing for SCM commands.
+   *  - set term
+   *  - set encoded token
+   *  - add to context's queue
+   */
+  private void processCommonCommand(
+      SCMCommandProto response, SCMCommand<?> cmd) {
+    if (response.hasTerm()) {
+      cmd.setTerm(response.getTerm());
+    }
+    if (response.hasEncodedToken()) {
+      cmd.setEncodedToken(response.getEncodedToken());
+    }
+    context.addCommand(cmd);
+  }
+
+  private void processReregisterCommand() {
+    if (rpcEndpoint.getState() == EndPointStates.HEARTBEAT) {
+      if (LOG.isDebugEnabled()) {
+        LOG.debug("Received SCM notification to register."
+            + " Interrupt HEARTBEAT and transit to REGISTER state.");
+      }
+      rpcEndpoint.setState(EndPointStates.REGISTER);
+    } else {
+      if (LOG.isDebugEnabled()) {
+        LOG.debug("Illegal state {} found, expecting {}.",
+            rpcEndpoint.getState().name(), EndPointStates.HEARTBEAT);
+      }
+    }
+  }
+
   /**
    * Builder class for HeartbeatEndpointTask.
    */
