@@ -121,6 +121,10 @@ public class MutableVolumeSet implements VolumeSet {
       this.volumeFactory = new MetadataVolumeFactory(conf, usageCheckFactory,
           this);
       maxVolumeFailuresTolerated = dnConf.getFailedMetadataVolumesTolerated();
+    } else if (volumeType == StorageVolume.VolumeType.DB_VOLUME) {
+      this.volumeFactory = new DbVolumeFactory(conf, usageCheckFactory,
+          this, datanodeUuid, clusterID);
+      maxVolumeFailuresTolerated = dnConf.getFailedDbVolumesTolerated();
     } else {
       this.volumeFactory = new HddsVolumeFactory(conf, usageCheckFactory,
           this, datanodeUuid, clusterID);
@@ -150,6 +154,8 @@ public class MutableVolumeSet implements VolumeSet {
     Collection<String> rawLocations;
     if (volumeType == StorageVolume.VolumeType.META_VOLUME) {
       rawLocations = HddsServerUtil.getOzoneDatanodeRatisDirectory(conf);
+    } else if (volumeType == StorageVolume.VolumeType.DB_VOLUME) {
+      rawLocations = HddsServerUtil.getDatanodeDbDirs(conf);
     } else {
       rawLocations = HddsServerUtil.getDatanodeStorageDirs(conf);
     }
@@ -289,6 +295,10 @@ public class MutableVolumeSet implements VolumeSet {
         });
   }
 
+  public void refreshAllVolumeUsage() {
+    volumeMap.forEach((k, v) -> v.refreshVolumeInfo());
+  }
+
   /**
    * Acquire Volume Set Read lock.
    */
@@ -444,6 +454,11 @@ public class MutableVolumeSet implements VolumeSet {
     return ImmutableMap.copyOf(volumeMap);
   }
 
+  @VisibleForTesting
+  public void setVolumeMap(Map<String, StorageVolume> map) {
+    this.volumeMap = map;
+  }
+
   @VisibleForTesting
   public Map<StorageType, List<StorageVolume>> getVolumeStateMap() {
     return ImmutableMap.copyOf(volumeStateMap);
