@@ -27,14 +27,11 @@ import org.apache.hadoop.hdds.conf.ConfigurationSource;
 import org.apache.hadoop.hdds.conf.StorageUnit;
 import org.apache.hadoop.hdds.scm.ScmConfigKeys;
 import org.apache.hadoop.hdds.scm.container.common.helpers.StorageContainerException;
-import org.apache.hadoop.hdds.utils.MetadataKeyFilters;
 import org.apache.hadoop.hdds.utils.db.BatchOperation;
 import org.apache.hadoop.hdds.utils.db.Table;
-import org.apache.hadoop.ozone.OzoneConsts;
 import org.apache.hadoop.ozone.container.common.helpers.BlockData;
 import org.apache.hadoop.ozone.container.common.interfaces.Container;
-import org.apache.hadoop.ozone.container.common.utils.ContainerCache;
-import org.apache.hadoop.ozone.container.common.utils.ReferenceCountedDB;
+import org.apache.hadoop.ozone.container.common.interfaces.DBHandle;
 import org.apache.hadoop.ozone.container.keyvalue.KeyValueContainer;
 import org.apache.hadoop.ozone.container.keyvalue.KeyValueContainerData;
 import org.apache.hadoop.ozone.container.keyvalue.helpers.BlockUtils;
@@ -94,40 +91,41 @@ public class BlockManagerImpl implements BlockManager {
    * Puts or overwrites a block.
    *
    * @param container - Container for which block need to be added.
-   * @param data     - BlockData.
-   * @param incrKeyCount - for FilePerBlockStrategy, increase key count only
-   *                     when the whole block file is written.
+   * @param data - BlockData.
+   * @param endOfBlock - The last putBlock call for this block (when
+   *                   all the chunks are written and stream is closed)
    * @return length of the block.
    * @throws IOException
    */
   @Override
   public long putBlock(Container container, BlockData data,
-      boolean incrKeyCount) throws IOException {
+      boolean endOfBlock) throws IOException {
     return persistPutBlock(
         (KeyValueContainer) container,
         data,
         config,
-        incrKeyCount);
+        endOfBlock);
   }
 
   public static long persistPutBlock(KeyValueContainer container,
-      BlockData data, ConfigurationSource config, boolean incrKeyCount)
+      BlockData data, ConfigurationSource config, boolean endOfBlock)
       throws IOException {
     Preconditions.checkNotNull(data, "BlockData cannot be null for put " +
         "operation.");
     Preconditions.checkState(data.getContainerID() >= 0, "Container Id " +
         "cannot be negative");
+
+    KeyValueContainerData containerData = container.getContainerData();
+
     // We are not locking the key manager since LevelDb serializes all actions
     // against a single DB. We rely on DB level locking to avoid conflicts.
-    try(ReferenceCountedDB db = BlockUtils.
-        getDB(container.getContainerData(), config)) {
+    try (DBHandle db = BlockUtils.getDB(containerData, config)) {
       // This is a post condition that acts as a hint to the user.
       // Should never fail.
       Preconditions.checkNotNull(db, DB_NULL_ERR_MSG);
 
       long bcsId = data.getBlockCommitSequenceId();
-      long containerBCSId = container.
-          getContainerData().getBlockCommitSequenceId();
+      long containerBCSId = containerData.getBlockCommitSequenceId();
 
       // default blockCommitSequenceId for any block is 0. It the putBlock
       // request is not coming via Ratis(for test scenarios), it will be 0.
@@ -143,14 +141,35 @@ public class BlockManagerImpl implements BlockManager {
             containerBCSId, bcsId);
         return data.getSize();
       }
+
+      // Check if the block is present in the pendingPutBlockCache for the
+      // container to determine whether the blockCount is already incremented
+      // for this block in the DB or not.
+      long localID = data.getLocalID();
+      boolean isBlockInCache = container.isBlockInPendingPutBlockCache(localID);
+      boolean incrBlockCount = false;
+
       // update the blockData as well as BlockCommitSequenceId here
       try (BatchOperation batch = db.getStore().getBatchHandler()
           .initBatchOperation()) {
+
+        // If the block does not exist in the pendingPutBlockCache of the
+        // container, then check the DB to ascertain if it exists or not.
+        // If block exists in cache, blockCount should not be incremented.
+        if (!isBlockInCache) {
+          if (db.getStore().getBlockDataTable().get(
+              containerData.blockKey(localID)) == null) {
+            // Block does not exist in DB => blockCount needs to be
+            // incremented when the block is added into DB.
+            incrBlockCount = true;
+          }
+        }
+
         db.getStore().getBlockDataTable().putWithBatch(
-            batch, Long.toString(data.getLocalID()), data);
+            batch, containerData.blockKey(localID), data);
         if (bcsId != 0) {
           db.getStore().getMetadataTable().putWithBatch(
-              batch, OzoneConsts.BLOCK_COMMIT_SEQUENCE_ID, bcsId);
+              batch, containerData.bcsIdKey(), bcsId);
         }
 
         // Set Bytes used, this bytes used will be updated for every write and
@@ -160,14 +179,14 @@ public class BlockManagerImpl implements BlockManager {
         // is only used to compute the bytes used. This is done to keep the
         // current behavior and avoid DB write during write chunk operation.
         db.getStore().getMetadataTable().putWithBatch(
-            batch, OzoneConsts.CONTAINER_BYTES_USED,
-            container.getContainerData().getBytesUsed());
+            batch, containerData.bytesUsedKey(),
+            containerData.getBytesUsed());
 
         // Set Block Count for a container.
-        if (incrKeyCount) {
+        if (incrBlockCount) {
           db.getStore().getMetadataTable().putWithBatch(
-              batch, OzoneConsts.BLOCK_COUNT,
-              container.getContainerData().getKeyCount() + 1);
+              batch, containerData.blockCountKey(),
+              containerData.getBlockCount() + 1);
         }
 
         db.getStore().getBatchHandler().commitBatchOperation(batch);
@@ -176,10 +195,24 @@ public class BlockManagerImpl implements BlockManager {
       if (bcsId != 0) {
         container.updateBlockCommitSequenceId(bcsId);
       }
-      // Increment block count finally here for in-memory.
-      if (incrKeyCount) {
-        container.getContainerData().incrKeyCount();
+
+      // Increment block count and add block to pendingPutBlockCache
+      // in-memory after the DB update.
+      if (incrBlockCount) {
+        containerData.incrBlockCount();
       }
+
+      // If the Block is not in PendingPutBlockCache (and it is not endOfBlock),
+      // add it there so that subsequent putBlock calls for this block do not
+      // have to read the DB to check for block existence
+      if (!isBlockInCache && !endOfBlock) {
+        container.addToPendingPutBlockCache(localID);
+      } else if (isBlockInCache && endOfBlock) {
+        // Remove the block from the PendingPutBlockCache as there would not
+        // be any more writes to this block
+        container.removeFromPendingPutBlockCache(localID);
+      }
+
       if (LOG.isDebugEnabled()) {
         LOG.debug(
             "Block " + data.getBlockID() + " successfully committed with bcsId "
@@ -212,15 +245,15 @@ public class BlockManagerImpl implements BlockManager {
     if (containerBCSId < bcsId) {
       throw new StorageContainerException(
           "Unable to find the block with bcsID " + bcsId + " .Container "
-              + container.getContainerData().getContainerID() + " bcsId is "
+              + containerData.getContainerID() + " bcsId is "
               + containerBCSId + ".", UNKNOWN_BCSID);
     }
 
-    try(ReferenceCountedDB db = BlockUtils.getDB(containerData, config)) {
+    try (DBHandle db = BlockUtils.getDB(containerData, config)) {
       // This is a post condition that acts as a hint to the user.
       // Should never fail.
       Preconditions.checkNotNull(db, DB_NULL_ERR_MSG);
-      BlockData blockData = getBlockByID(db, blockID);
+      BlockData blockData = getBlockByID(db, blockID, containerData);
       long id = blockData.getBlockID().getBlockCommitSequenceId();
       if (id < bcsId) {
         throw new StorageContainerException(
@@ -244,11 +277,11 @@ public class BlockManagerImpl implements BlockManager {
       throws IOException {
     KeyValueContainerData containerData = (KeyValueContainerData) container
         .getContainerData();
-    try(ReferenceCountedDB db = BlockUtils.getDB(containerData, config)) {
+    try (DBHandle db = BlockUtils.getDB(containerData, config)) {
       // This is a post condition that acts as a hint to the user.
       // Should never fail.
       Preconditions.checkNotNull(db, DB_NULL_ERR_MSG);
-      BlockData blockData = getBlockByID(db, blockID);
+      BlockData blockData = getBlockByID(db, blockID, containerData);
       return blockData.getSize();
     }
   }
@@ -260,51 +293,14 @@ public class BlockManagerImpl implements BlockManager {
 
   /**
    * Deletes an existing block.
-   *
-   * @param container - Container from which block need to be deleted.
-   * @param blockID - ID of the block.
-   * @throws StorageContainerException
    */
   @Override
   public void deleteBlock(Container container, BlockID blockID) throws
       IOException {
-    Preconditions.checkNotNull(blockID, "block ID cannot be null.");
-    Preconditions.checkState(blockID.getContainerID() >= 0,
-        "Container ID cannot be negative.");
-    Preconditions.checkState(blockID.getLocalID() >= 0,
-        "Local ID cannot be negative.");
-
-    KeyValueContainerData cData = (KeyValueContainerData) container
-        .getContainerData();
-    try(ReferenceCountedDB db = BlockUtils.getDB(cData, config)) {
-      // This is a post condition that acts as a hint to the user.
-      // Should never fail.
-      Preconditions.checkNotNull(db, DB_NULL_ERR_MSG);
-      // Note : There is a race condition here, since get and delete
-      // are not atomic. Leaving it here since the impact is refusing
-      // to delete a Block which might have just gotten inserted after
-      // the get check.
-
-      // Throw an exception if block data not found in the block data table.
-      getBlockByID(db, blockID);
-
-      // Update DB to delete block and set block count and bytes used.
-      try (BatchOperation batch = db.getStore().getBatchHandler()
-          .initBatchOperation()) {
-        String localID = Long.toString(blockID.getLocalID());
-        db.getStore().getBlockDataTable().deleteWithBatch(batch, localID);
-        // Update DB to delete block and set block count.
-        // No need to set bytes used here, as bytes used is taken care during
-        // delete chunk.
-        long blockCount = container.getContainerData().getKeyCount() - 1;
-        db.getStore().getMetadataTable()
-            .putWithBatch(batch, OzoneConsts.BLOCK_COUNT, blockCount);
-        db.getStore().getBatchHandler().commitBatchOperation(batch);
-      }
-
-      // Decrement block count here
-      container.getContainerData().decrKeyCount();
-    }
+    // Block/ Chunk Deletion is handled by BlockDeletingService.
+    // SCM sends Block Deletion commands directly to Datanodes and not
+    // through a Pipeline.
+    throw new UnsupportedOperationException();
   }
 
   /**
@@ -319,8 +315,8 @@ public class BlockManagerImpl implements BlockManager {
   public List<BlockData> listBlock(Container container, long startLocalID, int
       count) throws IOException {
     Preconditions.checkNotNull(container, "container cannot be null");
-    Preconditions.checkState(startLocalID >= 0, "startLocal ID cannot be " +
-        "negative");
+    Preconditions.checkState(startLocalID >= 0 || startLocalID == -1,
+        "startLocal ID cannot be negative");
     Preconditions.checkArgument(count > 0,
         "Count must be a positive number.");
     container.readLock();
@@ -328,15 +324,16 @@ public class BlockManagerImpl implements BlockManager {
       List<BlockData> result = null;
       KeyValueContainerData cData =
           (KeyValueContainerData) container.getContainerData();
-      try (ReferenceCountedDB db = BlockUtils.getDB(cData, config)) {
+      try (DBHandle db = BlockUtils.getDB(cData, config)) {
         result = new ArrayList<>();
+        String startKey = (startLocalID == -1) ? cData.startKeyEmpty()
+            : cData.blockKey(startLocalID);
         List<? extends Table.KeyValue<String, BlockData>> range =
             db.getStore().getBlockDataTable()
-                .getSequentialRangeKVs(Long.toString(startLocalID), count,
-                    MetadataKeyFilters.getUnprefixedKeyFilter());
+                .getSequentialRangeKVs(startKey, count,
+                    cData.containerPrefix(), cData.getUnprefixedKeyFilter());
         for (Table.KeyValue<String, BlockData> entry : range) {
-          BlockData data = new BlockData(entry.getValue().getBlockID());
-          result.add(data);
+          result.add(entry.getValue());
         }
         return result;
       }
@@ -350,12 +347,12 @@ public class BlockManagerImpl implements BlockManager {
    */
   @Override
   public void shutdown() {
-    BlockUtils.shutdownCache(ContainerCache.getInstance(config));
+    BlockUtils.shutdownCache(config);
   }
 
-  private BlockData getBlockByID(ReferenceCountedDB db, BlockID blockID)
-      throws IOException {
-    String blockKey = Long.toString(blockID.getLocalID());
+  private BlockData getBlockByID(DBHandle db, BlockID blockID,
+      KeyValueContainerData containerData) throws IOException {
+    String blockKey = containerData.blockKey(blockID.getLocalID());
 
     BlockData blockData = db.getStore().getBlockDataTable().get(blockKey);
     if (blockData == null) {
