@@ -49,7 +49,7 @@ import org.slf4j.LoggerFactory;
 /**
  * Client to read container data from gRPC.
  */
-public class GrpcReplicationClient implements AutoCloseable{
+public class GrpcReplicationClient implements AutoCloseable {
 
   private static final Logger LOG =
       LoggerFactory.getLogger(GrpcReplicationClient.class);
@@ -161,7 +161,16 @@ public class GrpcReplicationClient implements AutoCloseable{
       try {
         chunk.getData().writeTo(stream);
       } catch (IOException e) {
-        response.completeExceptionally(e);
+        LOG.error("Failed to write the stream buffer to {} for container {}",
+            outputPath, containerId, e);
+        try {
+          stream.close();
+        } catch (IOException ex) {
+          LOG.error("Failed to close OutputStream {}", outputPath, e);
+        } finally {
+          deleteOutputOnFailure();
+          response.completeExceptionally(e);
+        }
       }
     }
 
@@ -176,6 +185,7 @@ public class GrpcReplicationClient implements AutoCloseable{
       } catch (IOException e) {
         LOG.error("Failed to close {} for container {}",
             outputPath, containerId, e);
+        deleteOutputOnFailure();
         response.completeExceptionally(e);
       }
     }
@@ -189,9 +199,9 @@ public class GrpcReplicationClient implements AutoCloseable{
       } catch (IOException e) {
         LOG.error("Downloaded container {} OK, but failed to close {}",
             containerId, outputPath, e);
+        deleteOutputOnFailure();
         response.completeExceptionally(e);
       }
-
     }
 
     private void deleteOutputOnFailure() {
@@ -204,5 +214,4 @@ public class GrpcReplicationClient implements AutoCloseable{
       }
     }
   }
-
 }
