@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -22,7 +22,8 @@ import java.util.concurrent.TimeUnit;
 
 import org.apache.hadoop.hdds.conf.Config;
 import org.apache.hadoop.hdds.conf.ConfigGroup;
-import org.apache.hadoop.hdds.conf.ConfigTag;
+import org.apache.hadoop.hdds.conf.ConfigType;
+import org.apache.hadoop.hdds.conf.PostConstruct;
 import org.apache.hadoop.hdds.security.x509.SecurityConfig;
 import org.apache.hadoop.hdds.security.x509.certificate.client.CertificateClient;
 import org.apache.hadoop.hdds.tracing.GrpcServerInterceptor;
@@ -39,6 +40,9 @@ import org.apache.ratis.thirdparty.io.netty.handler.ssl.SslContextBuilder;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import static org.apache.hadoop.hdds.conf.ConfigTag.DATANODE;
+import static org.apache.hadoop.hdds.conf.ConfigTag.MANAGEMENT;
+
 /**
  * Separated network server for server2server container replication.
  */
@@ -128,12 +132,33 @@ public class ReplicationServer {
   /**
    * Replication-related configuration.
    */
-  @ConfigGroup(prefix = "hdds.datanode.replication")
+  @ConfigGroup(prefix = ReplicationConfig.PREFIX)
   public static final class ReplicationConfig {
 
-    @Config(key = "port", defaultValue = "9886", description = "Port used for"
-        + " the server2server replication server", tags = {
-        ConfigTag.MANAGEMENT})
+    public static final String PREFIX = "hdds.datanode.replication";
+    public static final String STREAMS_LIMIT_KEY = "streams.limit";
+
+    public static final String REPLICATION_STREAMS_LIMIT_KEY =
+        PREFIX + "." + STREAMS_LIMIT_KEY;
+
+    public static final int REPLICATION_MAX_STREAMS_DEFAULT = 10;
+
+    /**
+     * The maximum number of replication commands a single datanode can execute
+     * simultaneously.
+     */
+    @Config(key = STREAMS_LIMIT_KEY,
+        type = ConfigType.INT,
+        defaultValue = "10",
+        tags = {DATANODE},
+        description = "The maximum number of replication commands a single " +
+            "datanode can execute simultaneously"
+    )
+    private int replicationMaxStreams = REPLICATION_MAX_STREAMS_DEFAULT;
+
+    @Config(key = "port", defaultValue = "9886",
+        description = "Port used for the server2server replication server",
+        tags = {DATANODE, MANAGEMENT})
     private int port;
 
     public int getPort() {
@@ -144,6 +169,25 @@ public class ReplicationServer {
       this.port = portParam;
       return this;
     }
+
+    public int getReplicationMaxStreams() {
+      return replicationMaxStreams;
+    }
+
+    public void setReplicationMaxStreams(int replicationMaxStreams) {
+      this.replicationMaxStreams = replicationMaxStreams;
+    }
+
+    @PostConstruct
+    public void validate() {
+      if (replicationMaxStreams < 1) {
+        LOG.warn(REPLICATION_STREAMS_LIMIT_KEY + " must be greater than zero " +
+                "and was set to {}. Defaulting to {}",
+            replicationMaxStreams, REPLICATION_MAX_STREAMS_DEFAULT);
+        replicationMaxStreams = REPLICATION_MAX_STREAMS_DEFAULT;
+      }
+    }
+
   }
 
 }
