@@ -29,6 +29,7 @@ import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.ozone.container.common.impl.ContainerSet;
 import org.apache.hadoop.ozone.container.common.statemachine.StateContext;
+import org.apache.hadoop.ozone.container.replication.ReplicationServer.ReplicationConfig;
 import org.apache.hadoop.ozone.container.replication.ReplicationTask.Status;
 
 import com.google.common.annotations.VisibleForTesting;
@@ -71,6 +72,13 @@ public class ReplicationSupervisor {
     this.context = context;
   }
 
+  public ReplicationSupervisor(
+      ContainerSet containerSet, StateContext context,
+      ContainerReplicator replicator, ReplicationConfig replicationConfig) {
+    this(containerSet, context, replicator,
+        replicationConfig.getReplicationMaxStreams());
+  }
+
   public ReplicationSupervisor(
       ContainerSet containerSet, StateContext context,
       ContainerReplicator replicator, int poolSize) {
@@ -120,8 +128,7 @@ public class ReplicationSupervisor {
    *
    * @return Count of in-flight replications.
    */
-  @VisibleForTesting
-  int getInFlightReplications() {
+  public int getInFlightReplications() {
     return containersInFlight.size();
   }
 
@@ -184,6 +191,14 @@ public class ReplicationSupervisor {
     return requestCounter.get();
   }
 
+  public long getQueueSize() {
+    if (executor instanceof ThreadPoolExecutor) {
+      return ((ThreadPoolExecutor)executor).getQueue().size();
+    } else {
+      return 0;
+    }
+  }
+
   public long getReplicationSuccessCount() {
     return successCounter.get();
   }
