@@ -27,20 +27,18 @@ import org.apache.hadoop.ozone.container.common.impl.TestHddsDispatcher;
 import org.apache.hadoop.ozone.container.common.statemachine.DatanodeStateMachine;
 
 import org.apache.hadoop.ozone.container.common.volume.MutableVolumeSet;
-import org.junit.Test;
+import org.junit.jupiter.api.Test;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import java.io.IOException;
 import java.util.UUID;
 
-import static org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.Result.CONTAINER_UNHEALTHY;
 import static org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.Result.SUCCESS;
 import static org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.Result.UNKNOWN_BCSID;
 import static org.apache.hadoop.ozone.container.ContainerTestHelper.DATANODE_UUID;
 import static org.apache.hadoop.ozone.container.ContainerTestHelper.getDummyCommandRequestProto;
-import static org.hamcrest.core.Is.is;
-import static org.junit.Assert.assertThat;
+import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.when;
 
@@ -61,7 +59,7 @@ public class TestKeyValueHandlerWithUnhealthyContainer {
         handler.handleReadContainer(
             getDummyCommandRequestProto(ContainerProtos.Type.ReadContainer),
             container);
-    assertThat(response.getResult(), is(SUCCESS));
+    assertEquals(SUCCESS, response.getResult());
   }
 
   @Test
@@ -73,7 +71,7 @@ public class TestKeyValueHandlerWithUnhealthyContainer {
         handler.handleGetBlock(
             getDummyCommandRequestProto(ContainerProtos.Type.GetBlock),
             container);
-    assertThat(response.getResult(), is(UNKNOWN_BCSID));
+    assertEquals(UNKNOWN_BCSID, response.getResult());
   }
 
   @Test
@@ -86,7 +84,7 @@ public class TestKeyValueHandlerWithUnhealthyContainer {
             getDummyCommandRequestProto(
                 ContainerProtos.Type.GetCommittedBlockLength),
             container);
-    assertThat(response.getResult(), is(UNKNOWN_BCSID));
+    assertEquals(UNKNOWN_BCSID, response.getResult());
   }
 
   @Test
@@ -99,20 +97,7 @@ public class TestKeyValueHandlerWithUnhealthyContainer {
             getDummyCommandRequestProto(
                 ContainerProtos.Type.ReadChunk),
             container, null);
-    assertThat(response.getResult(), is(UNKNOWN_BCSID));
-  }
-
-  @Test
-  public void testDeleteChunk() throws IOException {
-    KeyValueContainer container = getMockUnhealthyContainer();
-    KeyValueHandler handler = getDummyHandler();
-
-    ContainerProtos.ContainerCommandResponseProto response =
-        handler.handleDeleteChunk(
-            getDummyCommandRequestProto(
-                ContainerProtos.Type.DeleteChunk),
-            container);
-    assertThat(response.getResult(), is(CONTAINER_UNHEALTHY));
+    assertEquals(UNKNOWN_BCSID, response.getResult());
   }
 
   @Test
@@ -125,7 +110,7 @@ public class TestKeyValueHandlerWithUnhealthyContainer {
             getDummyCommandRequestProto(
                 ContainerProtos.Type.GetSmallFile),
             container);
-    assertThat(response.getResult(), is(UNKNOWN_BCSID));
+    assertEquals(UNKNOWN_BCSID, response.getResult());
   }
 
   // -- Helper methods below.
