@@ -36,7 +36,7 @@ import org.apache.commons.compress.archivers.ArchiveOutputStream;
 import org.apache.commons.compress.archivers.tar.TarArchiveOutputStream;
 import org.apache.commons.compress.compressors.CompressorOutputStream;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
-import org.apache.hadoop.ozone.container.common.impl.ChunkLayOutVersion;
+import org.apache.hadoop.ozone.container.common.impl.ContainerLayoutVersion;
 import org.apache.hadoop.ozone.container.common.interfaces.ContainerPacker;
 
 import org.apache.commons.compress.archivers.tar.TarArchiveEntry;
@@ -87,15 +87,20 @@ public class TestTarContainerPacker {
 
   private static final AtomicInteger CONTAINER_ID = new AtomicInteger(1);
 
-  private final ChunkLayOutVersion layout;
+  private final ContainerLayoutVersion layout;
+  private final String schemaVersion;
+  private OzoneConfiguration conf;
 
-  public TestTarContainerPacker(ChunkLayOutVersion layout) {
-    this.layout = layout;
+  public TestTarContainerPacker(ContainerTestVersionInfo versionInfo) {
+    this.layout = versionInfo.getLayout();
+    this.schemaVersion = versionInfo.getSchemaVersion();
+    this.conf = new OzoneConfiguration();
+    ContainerTestVersionInfo.setTestSchemaVersion(schemaVersion, conf);
   }
 
   @Parameterized.Parameters
   public static Iterable<Object[]> parameters() {
-    return ChunkLayoutTestInfo.chunkLayoutParameters();
+    return ContainerTestVersionInfo.versionParameters();
   }
 
   @BeforeClass
@@ -125,14 +130,17 @@ public class TestTarContainerPacker {
     Path containerDir = dir.resolve("container" + id);
     Path dbDir = containerDir.resolve("db");
     Path dataDir = containerDir.resolve("data");
+    Path metaDir = containerDir.resolve("metadata");
+    Files.createDirectories(metaDir);
     Files.createDirectories(dbDir);
     Files.createDirectories(dataDir);
 
     KeyValueContainerData containerData = new KeyValueContainerData(
         id, layout,
         -1, UUID.randomUUID().toString(), UUID.randomUUID().toString());
+    containerData.setSchemaVersion(schemaVersion);
     containerData.setChunksPath(dataDir.toString());
-    containerData.setMetadataPath(dbDir.getParent().toString());
+    containerData.setMetadataPath(metaDir.toString());
     containerData.setDbFile(dbDir.toFile());
 
     return containerData;
@@ -140,10 +148,7 @@ public class TestTarContainerPacker {
 
   @Test
   public void pack() throws IOException, CompressorException {
-
     //GIVEN
-    OzoneConfiguration conf = new OzoneConfiguration();
-
     KeyValueContainerData sourceContainerData =
         createContainer(SOURCE_CONTAINER_ROOT);
 
@@ -211,7 +216,7 @@ public class TestTarContainerPacker {
     }
 
     assertExampleMetadataDbIsGood(
-        destinationContainerData.getDbFile().toPath(),
+        TarContainerPacker.getDbPath(destinationContainerData),
         TEST_DB_FILE_NAME);
     assertExampleChunkFileIsGood(
         Paths.get(destinationContainerData.getChunksPath()),
@@ -240,7 +245,8 @@ public class TestTarContainerPacker {
     KeyValueContainerData dest = unpackContainerData(containerFile);
 
     // THEN
-    assertExampleMetadataDbIsGood(dest.getDbFile().toPath(), fileName);
+    assertExampleMetadataDbIsGood(
+        TarContainerPacker.getDbPath(dest), fileName);
   }
 
   @Test
@@ -300,7 +306,6 @@ public class TestTarContainerPacker {
   private KeyValueContainerData unpackContainerData(File containerFile)
       throws IOException {
     try (FileInputStream input = new FileInputStream(containerFile)) {
-      OzoneConfiguration conf = new OzoneConfiguration();
       KeyValueContainerData data = createContainer(DEST_CONTAINER_ROOT);
       KeyValueContainer container = new KeyValueContainer(data, conf);
       packer.unpackContainerData(container, input);
@@ -320,29 +325,26 @@ public class TestTarContainerPacker {
   private File writeChunkFile(
       KeyValueContainerData containerData, String chunkFileName)
       throws IOException {
-    Path path = Paths.get(containerData.getChunksPath())
-        .resolve(chunkFileName);
-    Files.createDirectories(path.getParent());
-    File file = path.toFile();
-    FileOutputStream fileStream = new FileOutputStream(file);
-    try (OutputStreamWriter writer = new OutputStreamWriter(fileStream,
-        UTF_8)) {
-      IOUtils.write(TEST_CHUNK_FILE_CONTENT, writer);
-    }
-    return file;
+    return writeSingleFile(Paths.get(containerData.getChunksPath()),
+        chunkFileName, TEST_CHUNK_FILE_CONTENT);
   }
 
   private File writeDbFile(
       KeyValueContainerData containerData, String dbFileName)
       throws IOException {
-    Path path = containerData.getDbFile().toPath()
-        .resolve(dbFileName);
+    return writeSingleFile(TarContainerPacker.getDbPath(containerData),
+        dbFileName, TEST_DB_FILE_CONTENT);
+  }
+
+  private File writeSingleFile(Path parentPath, String fileName,
+      String content) throws IOException {
+    Path path = parentPath.resolve(fileName);
     Files.createDirectories(path.getParent());
     File file = path.toFile();
     FileOutputStream fileStream = new FileOutputStream(file);
     try (OutputStreamWriter writer = new OutputStreamWriter(fileStream,
         UTF_8)) {
-      IOUtils.write(TEST_DB_FILE_CONTENT, writer);
+      IOUtils.write(content, writer);
     }
     return file;
   }
@@ -361,35 +363,29 @@ public class TestTarContainerPacker {
 
   private void assertExampleMetadataDbIsGood(Path dbPath, String filename)
       throws IOException {
-
-    Path dbFile = dbPath.resolve(filename);
-
-    Assert.assertTrue(
-        "example DB file is missing after pack/unpackContainerData: " + dbFile,
-        Files.exists(dbFile));
-
-    try (FileInputStream testFile = new FileInputStream(dbFile.toFile())) {
-      List<String> strings = IOUtils.readLines(testFile, UTF_8);
-      Assert.assertEquals(1, strings.size());
-      Assert.assertEquals(TEST_DB_FILE_CONTENT, strings.get(0));
-    }
+    assertExampleFileIsGood(dbPath, filename, TEST_DB_FILE_CONTENT);
   }
 
   private void assertExampleChunkFileIsGood(Path chunkPath, String filename)
       throws IOException {
-
-    Path chunkFile = chunkPath.resolve(filename);
-
-    Assert.assertTrue(
-        "example chunk file is missing after pack/unpackContainerData: "
-            + chunkFile,
-        Files.exists(chunkFile));
-
-    try (FileInputStream testFile = new FileInputStream(chunkFile.toFile())) {
-      List<String> strings = IOUtils.readLines(testFile, UTF_8);
-      Assert.assertEquals(1, strings.size());
-      Assert.assertEquals(TEST_CHUNK_FILE_CONTENT, strings.get(0));
-    }
+    assertExampleFileIsGood(chunkPath, filename, TEST_CHUNK_FILE_CONTENT);
   }
 
+  private void assertExampleFileIsGood(Path parentPath, String filename,
+      String content) throws IOException {
+
+    Path exampleFile = parentPath.resolve(filename);
+
+    Assert.assertTrue(
+        "example file is missing after pack/unpackContainerData: "
+            + exampleFile,
+        Files.exists(exampleFile));
+
+    try (FileInputStream testFile =
+             new FileInputStream(exampleFile.toFile())) {
+      List<String> strings = IOUtils.readLines(testFile, UTF_8);
+      Assert.assertEquals(1, strings.size());
+      Assert.assertEquals(content, strings.get(0));
+    }
+  }
 }
