@@ -28,21 +28,19 @@ import org.apache.hadoop.ozone.OzoneSecurityUtil;
 import org.apache.hadoop.security.ssl.KeyStoreTestUtil;
 import org.apache.ozone.test.GenericTestUtils;
 import org.bouncycastle.cert.X509CertificateHolder;
-import org.junit.After;
-import org.junit.Before;
-import org.junit.Test;
-import org.junit.runner.RunWith;
-import org.junit.runners.Parameterized;
-import org.junit.runners.Parameterized.Parameter;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.Arguments;
+import org.junit.jupiter.params.provider.MethodSource;
 
 import java.nio.file.Files;
 import java.nio.file.Path;
 import java.nio.file.Paths;
 import java.security.KeyPair;
 import java.security.cert.X509Certificate;
-import java.util.Arrays;
-import java.util.Collection;
 import java.util.UUID;
+import java.util.stream.Stream;
 
 import static org.apache.hadoop.hdds.HddsConfigKeys.HDDS_METADATA_DIR_NAME;
 import static org.apache.hadoop.hdds.security.x509.certificate.client.CertificateClient.InitResponse;
@@ -50,13 +48,13 @@ import static org.apache.hadoop.hdds.security.x509.certificate.client.Certificat
 import static org.apache.hadoop.hdds.security.x509.certificate.client.CertificateClient.InitResponse.GETCERT;
 import static org.apache.hadoop.hdds.security.x509.certificate.client.CertificateClient.InitResponse.RECOVER;
 import static org.apache.hadoop.hdds.security.x509.certificate.client.CertificateClient.InitResponse.SUCCESS;
-import static org.junit.Assert.assertTrue;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.junit.jupiter.params.provider.Arguments.arguments;
 
 /**
  * Test class for {@link DefaultCertificateClient}.
  */
-@RunWith(Parameterized.class)
-@SuppressWarnings("visibilitymodifier")
 public class TestCertificateClientInit {
 
   private KeyPair keyPair;
@@ -72,29 +70,20 @@ public class TestCertificateClientInit {
   private static final String DN_COMPONENT = DNCertificateClient.COMPONENT_NAME;
   private static final String OM_COMPONENT = OMCertificateClient.COMPONENT_NAME;
 
-  @Parameter
-  public boolean pvtKeyPresent;
-  @Parameter(1)
-  public boolean pubKeyPresent;
-  @Parameter(2)
-  public boolean certPresent;
-  @Parameter(3)
-  public InitResponse expectedResult;
-
-  @Parameterized.Parameters
-  public static Collection<Object[]> initData() {
-    return Arrays.asList(new Object[][]{
-        {false, false, false, GETCERT},
-        {false, false, true, FAILURE},
-        {false, true, false, FAILURE},
-        {true, false, false, FAILURE},
-        {false, true, true, FAILURE},
-        {true, true, false, GETCERT},
-        {true, false, true, SUCCESS},
-        {true, true, true, SUCCESS}});
+  private static Stream<Arguments> parameters() {
+    return Stream.of(
+        arguments(false, false, false, GETCERT),
+        arguments(false, false, true, FAILURE),
+        arguments(false, true, false, FAILURE),
+        arguments(true, false, false, FAILURE),
+        arguments(false, true, true, FAILURE),
+        arguments(true, true, false, GETCERT),
+        arguments(true, false, true, SUCCESS),
+        arguments(true, true, true, SUCCESS)
+    );
   }
 
-  @Before
+  @BeforeEach
   public void setUp() throws Exception {
     OzoneConfiguration config = new OzoneConfiguration();
     final String path = GenericTestUtils
@@ -117,7 +106,7 @@ public class TestCertificateClientInit {
     Files.createDirectories(securityConfig.getKeyLocation(OM_COMPONENT));
   }
 
-  @After
+  @AfterEach
   public void tearDown() {
     dnCertificateClient = null;
     omCertificateClient = null;
@@ -125,8 +114,10 @@ public class TestCertificateClientInit {
   }
 
 
-  @Test
-  public void testInitDatanode() throws Exception {
+  @ParameterizedTest
+  @MethodSource("parameters")
+  public void testInitDatanode(boolean pvtKeyPresent, boolean pubKeyPresent,
+      boolean certPresent, InitResponse expectedResult) throws Exception {
     if (pvtKeyPresent) {
       dnKeyCodec.writePrivateKey(keyPair.getPrivate());
     } else {
@@ -157,7 +148,7 @@ public class TestCertificateClientInit {
     }
     InitResponse response = dnCertificateClient.init();
 
-    assertTrue(response.equals(expectedResult));
+    assertEquals(expectedResult, response);
 
     if (!response.equals(FAILURE)) {
       assertTrue(OzoneSecurityUtil.checkIfFileExist(
@@ -169,8 +160,10 @@ public class TestCertificateClientInit {
     }
   }
 
-  @Test
-  public void testInitOzoneManager() throws Exception {
+  @ParameterizedTest
+  @MethodSource("parameters")
+  public void testInitOzoneManager(boolean pvtKeyPresent, boolean pubKeyPresent,
+      boolean certPresent, InitResponse expectedResult) throws Exception {
     if (pvtKeyPresent) {
       omKeyCodec.writePrivateKey(keyPair.getPrivate());
     } else {
@@ -202,9 +195,9 @@ public class TestCertificateClientInit {
     InitResponse response = omCertificateClient.init();
 
     if (pvtKeyPresent && pubKeyPresent && !certPresent) {
-      assertTrue(response.equals(RECOVER));
+      assertEquals(RECOVER, response);
     } else {
-      assertTrue(response.equals(expectedResult));
+      assertEquals(expectedResult, response);
     }
 
     if (!response.equals(FAILURE)) {
@@ -219,6 +212,6 @@ public class TestCertificateClientInit {
 
   private X509Certificate getX509Certificate() throws Exception {
     return KeyStoreTestUtil.generateCertificate(
-        "CN=Test", keyPair, 10, securityConfig.getSignatureAlgo());
+        "CN=Test", keyPair, 365, securityConfig.getSignatureAlgo());
   }
 }
\ No newline at end of file
