@@ -21,9 +21,9 @@ package org.apache.hadoop.hdds.security.x509.certificate.utils;
 
 import static java.nio.charset.StandardCharsets.UTF_8;
 import static org.apache.hadoop.hdds.HddsConfigKeys.OZONE_METADATA_DIRS;
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertNotNull;
-import static org.junit.Assert.assertTrue;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 
 import java.io.BufferedReader;
 import java.io.ByteArrayInputStream;
@@ -33,6 +33,7 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.math.BigInteger;
+import java.nio.file.Path;
 import java.nio.file.Paths;
 import java.security.KeyPair;
 import java.security.NoSuchAlgorithmException;
@@ -60,12 +61,9 @@ import org.bouncycastle.cert.X509CertificateHolder;
 import org.bouncycastle.cert.X509v2CRLBuilder;
 import org.bouncycastle.operator.OperatorCreationException;
 import org.bouncycastle.operator.jcajce.JcaContentSignerBuilder;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Rule;
-import org.junit.Test;
-import org.junit.rules.TemporaryFolder;
-
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.io.TempDir;
 
 /**
  * Tests for the CRLCodec.
@@ -79,8 +77,6 @@ public class TestCRLCodec {
   private KeyPair keyPair;
   private static final String TMP_CERT_FILE_NAME = "pemcertificate.crt";
 
-  @Rule
-  public TemporaryFolder temporaryFolder = new TemporaryFolder();
   private File basePath;
   private static final String TMP_CRL_ENTRY =
       "-----BEGIN X509 CRL-----\n" +
@@ -95,12 +91,12 @@ public class TestCRLCodec {
       "tPiRCAUQLW9BACm17xc=\n" +
       "-----END X509 CRL-----\n";
 
-  @Before
-  public void init() throws NoSuchProviderException,
+  @BeforeEach
+  public void init(@TempDir Path tempDir) throws NoSuchProviderException,
       NoSuchAlgorithmException, IOException,
       CertificateException, OperatorCreationException {
 
-    conf.set(OZONE_METADATA_DIRS, temporaryFolder.newFolder().toString());
+    conf.set(OZONE_METADATA_DIRS, tempDir.toString());
     securityConfig = new SecurityConfig(conf);
     writeTempCert();
     x509CertificateHolder = readTempCert();
@@ -138,7 +134,7 @@ public class TestCRLCodec {
     assertTrue(crlFile.exists());
 
     try (BufferedReader reader = new BufferedReader(new InputStreamReader(
-        new FileInputStream(crlFile), UTF_8))){
+        new FileInputStream(crlFile), UTF_8))) {
 
       // Verify contents of the file
       String header = reader.readLine();
@@ -272,7 +268,7 @@ public class TestCRLCodec {
             securityConfig.getCertificateLocation("scm")));
 
     if (!basePath.exists()) {
-      Assert.assertTrue(basePath.mkdirs());
+      assertTrue(basePath.mkdirs());
     }
     codec.writeCertificate(basePath.toPath(), TMP_CERT_FILE_NAME,
                            pemString, false);
