@@ -20,96 +20,134 @@
 package org.apache.hadoop.hdds.utils.db.cache;
 
 import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.Collection;
 import java.util.List;
 import java.util.concurrent.CompletableFuture;
 
 import com.google.common.base.Optional;
 import org.apache.ozone.test.GenericTestUtils;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Test;
-import org.junit.runner.RunWith;
-import org.junit.runners.Parameterized;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.BeforeAll;
+import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.EnumSource;
 import org.slf4j.event.Level;
 
-import static org.junit.Assert.fail;
 
 /**
  * Class tests partial table cache.
  */
-@RunWith(value = Parameterized.class)
 public class TestTableCache {
+
   private TableCache<CacheKey<String>, CacheValue<String>> tableCache;
 
-  private final TableCache.CacheType cacheType;
-
-
-  @Parameterized.Parameters
-  public static Collection<Object[]> policy() {
-    Object[][] params = new Object[][] {
-        {TableCache.CacheType.FULL_CACHE},
-        {TableCache.CacheType.PARTIAL_CACHE}
-    };
-    return Arrays.asList(params);
-  }
-
-  public TestTableCache(
-      TableCache.CacheType cacheType) {
+  @BeforeAll
+  public static void setLogLevel() {
     GenericTestUtils.setLogLevel(FullTableCache.LOG, Level.DEBUG);
-    this.cacheType = cacheType;
   }
 
-
-  @Before
-  public void create() {
+  private void createTableCache(TableCache.CacheType cacheType) {
     if (cacheType == TableCache.CacheType.FULL_CACHE) {
       tableCache = new FullTableCache<>();
     } else {
       tableCache = new PartialTableCache<>();
     }
   }
-  @Test
-  public void testPartialTableCache() {
 
+  @ParameterizedTest
+  @EnumSource(TableCache.CacheType.class)
+  public void testPartialTableCache(TableCache.CacheType cacheType) {
 
-    for (int i = 0; i< 10; i++) {
+    createTableCache(cacheType);
+
+    for (int i = 0; i < 10; i++) {
       tableCache.put(new CacheKey<>(Integer.toString(i)),
           new CacheValue<>(Optional.of(Integer.toString(i)), i));
     }
 
 
-    for (int i=0; i < 10; i++) {
-      Assert.assertEquals(Integer.toString(i),
+    for (int i = 0; i < 10; i++) {
+      Assertions.assertEquals(Integer.toString(i),
           tableCache.get(new CacheKey<>(Integer.toString(i))).getCacheValue());
     }
 
-    ArrayList<Long> epochs = new ArrayList();
+    ArrayList<Long> epochs = new ArrayList<>();
     epochs.add(0L);
     epochs.add(1L);
     epochs.add(2L);
     epochs.add(3L);
     epochs.add(4L);
-    // On a full table cache if some one calls cleanup it is a no-op.
+    // On a full table cache if someone calls cleanup it is a no-op.
     tableCache.evictCache(epochs);
 
-    for (int i=5; i < 10; i++) {
-      Assert.assertEquals(Integer.toString(i),
+    for (int i = 5; i < 10; i++) {
+      Assertions.assertEquals(Integer.toString(i),
           tableCache.get(new CacheKey<>(Integer.toString(i))).getCacheValue());
     }
   }
 
+  @ParameterizedTest
+  @EnumSource(TableCache.CacheType.class)
+  public void testTableCacheWithRenameKey(TableCache.CacheType cacheType) {
+
+    createTableCache(cacheType);
+
+    // putting cache with same epoch and different keyNames
+    for (int i = 0; i < 3; i++) {
+      tableCache.put(new CacheKey<>(Integer.toString(i).concat("A")),
+              new CacheValue<>(Optional.absent(), i));
+      tableCache.put(new CacheKey<>(Integer.toString(i).concat("B")),
+              new CacheValue<>(Optional.of(Integer.toString(i)), i));
+    }
+
+    // Epoch entries should be like (long, (key1, key2, ...))
+    // (0, (0A, 0B))  (1, (1A, 1B))  (2, (2A, 1B))
+    Assertions.assertEquals(3, tableCache.getEpochEntries().size());
+    Assertions.assertEquals(2, tableCache.getEpochEntries().get(0L).size());
+    
+    // Cache should be like (key, (cacheValue, long))
+    // (0A, (null, 0))   (0B, (0, 0))
+    // (1A, (null, 1))   (1B, (0, 1))
+    // (2A, (null, 2))   (2B, (0, 2))
+    for (int i = 0; i < 3; i++) {
+      Assertions.assertNull(tableCache.get(new CacheKey<>(
+          Integer.toString(i).concat("A"))).getCacheValue());
+      Assertions.assertEquals(Integer.toString(i),
+          tableCache.get(new CacheKey<>(Integer.toString(i).concat("B")))
+              .getCacheValue());
+    }
+
+    ArrayList<Long> epochs = new ArrayList<>();
+    epochs.add(0L);
+    epochs.add(1L);
+    epochs.add(2L);
+    epochs.add(3L);
+    epochs.add(4L);
+
+    tableCache.evictCache(epochs);
+
+    Assertions.assertEquals(0, tableCache.getEpochEntries().size());
+
+    if (cacheType == TableCache.CacheType.PARTIAL_CACHE) {
+      Assertions.assertEquals(0, tableCache.size());
+    } else {
+      Assertions.assertEquals(3, tableCache.size());
+    }
+
+  }
+
+  @ParameterizedTest
+  @EnumSource(TableCache.CacheType.class)
+  public void testPartialTableCacheWithNotContinuousEntries(
+      TableCache.CacheType cacheType) {
+
+    createTableCache(cacheType);
 
-  @Test
-  public void testPartialTableCacheWithNotContinousEntries() throws Exception {
     int totalCount = 0;
     int insertedCount = 3000;
 
     int cleanupCount = 0;
 
-    ArrayList<Long> epochs = new ArrayList();
-    for (long i=0; i<insertedCount; i+=2) {
+    ArrayList<Long> epochs = new ArrayList<>();
+    for (long i = 0; i < insertedCount; i += 2) {
       if (cleanupCount++ < 1000) {
         epochs.add(i);
       }
@@ -118,7 +156,7 @@ public class TestTableCache {
       totalCount++;
     }
 
-    Assert.assertEquals(totalCount, tableCache.size());
+    Assertions.assertEquals(totalCount, tableCache.size());
 
     tableCache.evictCache(epochs);
 
@@ -126,30 +164,34 @@ public class TestTableCache {
 
     // If cleanup policy is manual entries should have been removed.
     if (cacheType == TableCache.CacheType.PARTIAL_CACHE) {
-      Assert.assertEquals(count - epochs.size(), tableCache.size());
+      Assertions.assertEquals(count - epochs.size(), tableCache.size());
 
       // Check remaining entries exist or not and deleted entries does not
       // exist.
       for (long i = 0; i < insertedCount; i += 2) {
         if (!epochs.contains(i)) {
-          Assert.assertEquals(Long.toString(i),
+          Assertions.assertEquals(Long.toString(i),
               tableCache.get(new CacheKey<>(Long.toString(i))).getCacheValue());
         } else {
-          Assert.assertEquals(null,
+          Assertions.assertNull(
               tableCache.get(new CacheKey<>(Long.toString(i))));
         }
       }
     } else {
       for (long i = 0; i < insertedCount; i += 2) {
-        Assert.assertEquals(Long.toString(i),
+        Assertions.assertEquals(Long.toString(i),
             tableCache.get(new CacheKey<>(Long.toString(i))).getCacheValue());
       }
     }
 
   }
 
-  @Test
-  public void testPartialTableCacheWithOverrideEntries() throws Exception {
+  @ParameterizedTest
+  @EnumSource(TableCache.CacheType.class)
+  public void testPartialTableCacheWithOverrideEntries(
+      TableCache.CacheType cacheType) {
+
+    createTableCache(cacheType);
 
     tableCache.put(new CacheKey<>(Long.toString(0)),
           new CacheValue<>(Optional.of(Long.toString(0)), 0));
@@ -170,10 +212,10 @@ public class TestTableCache {
 
 
 
-    Assert.assertEquals(3, tableCache.size());
+    Assertions.assertEquals(3, tableCache.size());
     // It will have 2 additional entries because we have 2 override entries.
-    Assert.assertEquals(3 + 2,
-        tableCache.getEpochEntrySet().size());
+    Assertions.assertEquals(3 + 2,
+        tableCache.getEpochEntries().size());
 
     // Now remove
 
@@ -188,9 +230,9 @@ public class TestTableCache {
 
       tableCache.evictCache(epochs);
 
-      Assert.assertEquals(0, tableCache.size());
+      Assertions.assertEquals(0, tableCache.size());
 
-      Assert.assertEquals(0, tableCache.getEpochEntrySet().size());
+      Assertions.assertEquals(0, tableCache.getEpochEntries().size());
     }
 
     // Add a new entry.
@@ -202,17 +244,21 @@ public class TestTableCache {
     if (cacheType == TableCache.CacheType.PARTIAL_CACHE) {
       tableCache.evictCache(epochs);
 
-      Assert.assertEquals(0, tableCache.size());
+      Assertions.assertEquals(0, tableCache.size());
 
-      // Overrided entries would have been deleted.
-      Assert.assertEquals(0, tableCache.getEpochEntrySet().size());
+      // Overridden entries would have been deleted.
+      Assertions.assertEquals(0, tableCache.getEpochEntries().size());
     }
 
 
   }
 
-  @Test
-  public void testPartialTableCacheWithOverrideAndDelete() throws Exception {
+  @ParameterizedTest
+  @EnumSource(TableCache.CacheType.class)
+  public void testPartialTableCacheWithOverrideAndDelete(
+      TableCache.CacheType cacheType) {
+
+    createTableCache(cacheType);
 
     tableCache.put(new CacheKey<>(Long.toString(0)),
         new CacheValue<>(Optional.of(Long.toString(0)), 0));
@@ -222,13 +268,13 @@ public class TestTableCache {
         new CacheValue<>(Optional.of(Long.toString(2)), 2));
 
 
-    //Override entries
+    // Override entries
     tableCache.put(new CacheKey<>(Long.toString(0)),
         new CacheValue<>(Optional.of(Long.toString(0)), 3));
     tableCache.put(new CacheKey<>(Long.toString(1)),
         new CacheValue<>(Optional.of(Long.toString(1)), 4));
 
-    // Finally mark them for deleted
+    // Finally, mark them for deleted
     tableCache.put(new CacheKey<>(Long.toString(0)),
         new CacheValue<>(Optional.absent(), 5));
     tableCache.put(new CacheKey<>(Long.toString(1)),
@@ -240,10 +286,10 @@ public class TestTableCache {
     // 0-5, 1-6, 2-2
 
 
-    Assert.assertEquals(3, tableCache.size());
+    Assertions.assertEquals(3, tableCache.size());
     // It will have 4 additional entries because we have 4 override entries.
-    Assert.assertEquals(3 + 4,
-        tableCache.getEpochEntrySet().size());
+    Assertions.assertEquals(3 + 4,
+        tableCache.getEpochEntries().size());
 
     // Now remove
 
@@ -260,16 +306,16 @@ public class TestTableCache {
     if (cacheType == TableCache.CacheType.PARTIAL_CACHE) {
       tableCache.evictCache(epochs);
 
-      Assert.assertEquals(0, tableCache.size());
+      Assertions.assertEquals(0, tableCache.size());
 
-      Assert.assertEquals(0, tableCache.getEpochEntrySet().size());
+      Assertions.assertEquals(0, tableCache.getEpochEntries().size());
     } else {
       tableCache.evictCache(epochs);
 
-      Assert.assertEquals(1, tableCache.size());
+      Assertions.assertEquals(1, tableCache.size());
 
-      // Epoch entries which are overrided also will be cleaned up.
-      Assert.assertEquals(0, tableCache.getEpochEntrySet().size());
+      // Epoch entries which are overridden also will be cleaned up.
+      Assertions.assertEquals(0, tableCache.getEpochEntries().size());
     }
 
     // Add a new entry, now old override entries will be cleaned up.
@@ -282,26 +328,30 @@ public class TestTableCache {
     if (cacheType == TableCache.CacheType.PARTIAL_CACHE) {
       tableCache.evictCache(epochs);
 
-      Assert.assertEquals(0, tableCache.size());
+      Assertions.assertEquals(0, tableCache.size());
 
-      // Epoch entries which are overrided now would have been deleted.
-      Assert.assertEquals(0, tableCache.getEpochEntrySet().size());
+      // Epoch entries which are overridden now would have been deleted.
+      Assertions.assertEquals(0, tableCache.getEpochEntries().size());
     } else {
       tableCache.evictCache(epochs);
 
       // 2 entries will be in cache, as 2 are not deleted.
-      Assert.assertEquals(2, tableCache.size());
+      Assertions.assertEquals(2, tableCache.size());
 
       // Epoch entries which are not marked for delete will also be cleaned up.
       // As they are override entries in full cache.
-      Assert.assertEquals(0, tableCache.getEpochEntrySet().size());
+      Assertions.assertEquals(0, tableCache.getEpochEntries().size());
     }
 
 
   }
 
-  @Test
-  public void testPartialTableCacheParallel() throws Exception {
+  @ParameterizedTest
+  @EnumSource(TableCache.CacheType.class)
+  public void testPartialTableCacheParallel(
+      TableCache.CacheType cacheType) throws Exception {
+
+    createTableCache(cacheType);
 
     int totalCount = 0;
     CompletableFuture<Integer> future =
@@ -309,12 +359,12 @@ public class TestTableCache {
           try {
             return writeToCache(10, 1, 0);
           } catch (InterruptedException ex) {
-            fail("writeToCache got interrupt exception");
+            Assertions.fail("writeToCache got interrupt exception");
           }
           return 0;
         });
     int value = future.get();
-    Assert.assertEquals(10, value);
+    Assertions.assertEquals(10, value);
 
     totalCount += value;
 
@@ -323,27 +373,27 @@ public class TestTableCache {
           try {
             return writeToCache(10, 11, 100);
           } catch (InterruptedException ex) {
-            fail("writeToCache got interrupt exception");
+            Assertions.fail("writeToCache got interrupt exception");
           }
           return 0;
         });
 
     // Check we have first 10 entries in cache.
-    for (int i=1; i <= 10; i++) {
-      Assert.assertEquals(Integer.toString(i),
+    for (int i = 1; i <= 10; i++) {
+      Assertions.assertEquals(Integer.toString(i),
           tableCache.get(new CacheKey<>(Integer.toString(i))).getCacheValue());
     }
 
 
     value = future.get();
-    Assert.assertEquals(10, value);
+    Assertions.assertEquals(10, value);
 
     totalCount += value;
 
     if (cacheType == TableCache.CacheType.PARTIAL_CACHE) {
       int deleted = 5;
 
-      // cleanup first 5 entires
+      // cleanup first 5 entries
 
       ArrayList<Long> epochs = new ArrayList<>();
       epochs.add(1L);
@@ -354,37 +404,39 @@ public class TestTableCache {
       tableCache.evictCache(epochs);
 
       // We should totalCount - deleted entries in cache.
-      final int tc = totalCount;
-      Assert.assertEquals(tc - deleted, tableCache.size());
+      Assertions.assertEquals(totalCount - deleted, tableCache.size());
       // Check if we have remaining entries.
-      for (int i=6; i <= totalCount; i++) {
-        Assert.assertEquals(Integer.toString(i), tableCache.get(
+      for (int i = 6; i <= totalCount; i++) {
+        Assertions.assertEquals(Integer.toString(i), tableCache.get(
             new CacheKey<>(Integer.toString(i))).getCacheValue());
       }
 
       epochs = new ArrayList<>();
-      for (long i=6; i<= totalCount; i++) {
+      for (long i = 6; i <= totalCount; i++) {
         epochs.add(i);
       }
 
       tableCache.evictCache(epochs);
 
       // Cleaned up all entries, so cache size should be zero.
-      Assert.assertEquals(0, tableCache.size());
+      Assertions.assertEquals(0, tableCache.size());
     } else {
       ArrayList<Long> epochs = new ArrayList<>();
-      for (long i=0; i<= totalCount; i++) {
+      for (long i = 0; i <= totalCount; i++) {
         epochs.add(i);
       }
       tableCache.evictCache(epochs);
-      Assert.assertEquals(totalCount, tableCache.size());
+      Assertions.assertEquals(totalCount, tableCache.size());
     }
 
 
   }
 
-  @Test
-  public void testTableCache() {
+  @ParameterizedTest
+  @EnumSource(TableCache.CacheType.class)
+  public void testTableCache(TableCache.CacheType cacheType) {
+
+    createTableCache(cacheType);
 
     // In non-HA epoch entries might be out of order.
     // Scenario is like create vol, set vol, set vol, delete vol
@@ -406,13 +458,17 @@ public class TestTableCache {
 
     tableCache.evictCache(epochs);
 
-    Assert.assertTrue(tableCache.size() == 0);
-    Assert.assertTrue(tableCache.getEpochEntrySet().size() == 0);
+    Assertions.assertEquals(0, tableCache.size());
+    Assertions.assertEquals(0, tableCache.getEpochEntries().size());
   }
 
 
-  @Test
-  public void testTableCacheWithNonConsecutiveEpochList() {
+  @ParameterizedTest
+  @EnumSource(TableCache.CacheType.class)
+  public void testTableCacheWithNonConsecutiveEpochList(
+      TableCache.CacheType cacheType) {
+
+    createTableCache(cacheType);
 
     // In non-HA epoch entries might be out of order.
     tableCache.put(new CacheKey<>(Long.toString(0)),
@@ -435,15 +491,15 @@ public class TestTableCache {
 
     tableCache.evictCache(epochs);
 
-    Assert.assertTrue(tableCache.size() == 2);
-    Assert.assertTrue(tableCache.getEpochEntrySet().size() == 2);
+    Assertions.assertEquals(2, tableCache.size());
+    Assertions.assertEquals(2, tableCache.getEpochEntries().size());
 
-    Assert.assertNotNull(tableCache.get(new CacheKey<>(Long.toString(0))));
-    Assert.assertEquals(2,
+    Assertions.assertNotNull(tableCache.get(new CacheKey<>(Long.toString(0))));
+    Assertions.assertEquals(2,
         tableCache.get(new CacheKey<>(Long.toString(0))).getEpoch());
 
-    Assert.assertNotNull(tableCache.get(new CacheKey<>(Long.toString(1))));
-    Assert.assertEquals(4,
+    Assertions.assertNotNull(tableCache.get(new CacheKey<>(Long.toString(1))));
+    Assertions.assertEquals(4,
         tableCache.get(new CacheKey<>(Long.toString(1))).getEpoch());
 
     // now evict 2,4
@@ -453,20 +509,22 @@ public class TestTableCache {
 
     tableCache.evictCache(epochs);
 
-    if(cacheType == TableCache.CacheType.PARTIAL_CACHE) {
-      Assert.assertTrue(tableCache.size() == 0);
-      Assert.assertTrue(tableCache.getEpochEntrySet().size() == 0);
+    if (cacheType == TableCache.CacheType.PARTIAL_CACHE) {
+      Assertions.assertEquals(0, tableCache.size());
+      Assertions.assertEquals(0, tableCache.getEpochEntries().size());
     } else {
-      Assert.assertTrue(tableCache.size() == 2);
-      Assert.assertTrue(tableCache.getEpochEntrySet().size() == 0);
+      Assertions.assertEquals(2, tableCache.size());
+      Assertions.assertEquals(0, tableCache.getEpochEntries().size());
 
       // Entries should exist, as the entries are not delete entries
-      Assert.assertNotNull(tableCache.get(new CacheKey<>(Long.toString(0))));
-      Assert.assertEquals(2,
+      Assertions.assertNotNull(
+          tableCache.get(new CacheKey<>(Long.toString(0))));
+      Assertions.assertEquals(2,
           tableCache.get(new CacheKey<>(Long.toString(0))).getEpoch());
 
-      Assert.assertNotNull(tableCache.get(new CacheKey<>(Long.toString(1))));
-      Assert.assertEquals(4,
+      Assertions.assertNotNull(
+          tableCache.get(new CacheKey<>(Long.toString(1))));
+      Assertions.assertEquals(4,
           tableCache.get(new CacheKey<>(Long.toString(1))).getEpoch());
     }
 
@@ -475,7 +533,7 @@ public class TestTableCache {
   private int writeToCache(int count, int startVal, long sleep)
       throws InterruptedException {
     int counter = 1;
-    while (counter <= count){
+    while (counter <= count) {
       tableCache.put(new CacheKey<>(Integer.toString(startVal)),
           new CacheValue<>(Optional.of(Integer.toString(startVal)), startVal));
       startVal++;
