@@ -53,6 +53,9 @@ import org.apache.hadoop.hdds.utils.db.Table.KeyValue;
 import org.apache.hadoop.hdds.utils.db.TableIterator;
 import org.apache.hadoop.ozone.common.statemachine.InvalidStateTransitionException;
 import org.apache.hadoop.ozone.common.statemachine.StateMachine;
+import org.apache.hadoop.ozone.lock.LockManager;
+import org.apache.hadoop.hdds.conf.ConfigurationSource;
+import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -71,6 +74,7 @@ import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.LifeCycleState.CL
 import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.LifeCycleState.DELETING;
 import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.LifeCycleState.DELETED;
 
+
 /**
  * Default implementation of ContainerStateManager. This implementation
  * holds the Container States in-memory which is backed by a persistent store.
@@ -79,7 +83,11 @@ import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.LifeCycleState.DE
  * This class is NOT thread safe. All the calls are idempotent.
  */
 public final class ContainerStateManagerImpl
-    implements ContainerStateManagerV2 {
+    implements ContainerStateManager {
+
+  private ConfigurationSource confSrc;
+
+  private final LockManager<ContainerID> lockManager;
 
   /**
    * Logger instance of ContainerStateManagerImpl.
@@ -125,7 +133,7 @@ public final class ContainerStateManagerImpl
 
   // Protect containers and containerStore against the potential
   // contentions between RaftServer and ContainerManager.
-  private final ReadWriteLock lock = new ReentrantReadWriteLock();
+  private final ReadWriteLock lock = new ReentrantReadWriteLock(true);
 
   /**
    * constructs ContainerStateManagerImpl instance and loads the containers
@@ -141,6 +149,7 @@ public final class ContainerStateManagerImpl
       final Table<ContainerID, ContainerInfo> containerStore,
       final DBTransactionBuffer buffer)
       throws IOException {
+    this.confSrc = OzoneConfiguration.of(conf);
     this.pipelineManager = pipelineManager;
     this.containerStore = containerStore;
     this.stateMachine = newStateMachine();
@@ -149,7 +158,8 @@ public final class ContainerStateManagerImpl
     this.lastUsedMap = new ConcurrentHashMap<>();
     this.containerStateChangeActions = getContainerStateChangeActions();
     this.transactionBuffer = buffer;
-
+    this.lockManager =
+        new LockManager<>(confSrc, true);
     initialize();
   }
 
@@ -220,27 +230,29 @@ public final class ContainerStateManagerImpl
    * @throws IOException in case of error while loading the containers
    */
   private void initialize() throws IOException {
-    TableIterator<ContainerID, ? extends KeyValue<ContainerID, ContainerInfo>>
-        iterator = containerStore.iterator();
+    try (TableIterator<ContainerID,
+        ? extends KeyValue<ContainerID, ContainerInfo>> iterator =
+             containerStore.iterator()) {
 
-    while (iterator.hasNext()) {
-      final ContainerInfo container = iterator.next().getValue();
-      Preconditions.checkNotNull(container);
-      containers.addContainer(container);
-      if (container.getState() == LifeCycleState.OPEN) {
-        try {
-          pipelineManager.addContainerToPipelineSCMStart(
-              container.getPipelineID(), container.containerID());
-        } catch (PipelineNotFoundException ex) {
-          LOG.warn("Found container {} which is in OPEN state with " +
-                  "pipeline {} that does not exist. Marking container for " +
-                  "closing.", container, container.getPipelineID());
+      while (iterator.hasNext()) {
+        final ContainerInfo container = iterator.next().getValue();
+        Preconditions.checkNotNull(container);
+        containers.addContainer(container);
+        if (container.getState() == LifeCycleState.OPEN) {
           try {
-            updateContainerState(container.containerID().getProtobuf(),
-                LifeCycleEvent.FINALIZE);
-          } catch (InvalidStateTransitionException e) {
-            // This cannot happen.
-            LOG.warn("Unable to finalize Container {}.", container);
+            pipelineManager.addContainerToPipelineSCMStart(
+                container.getPipelineID(), container.containerID());
+          } catch (PipelineNotFoundException ex) {
+            LOG.warn("Found container {} which is in OPEN state with " +
+                "pipeline {} that does not exist. Marking container for " +
+                "closing.", container, container.getPipelineID());
+            try {
+              updateContainerState(container.containerID().getProtobuf(),
+                  LifeCycleEvent.FINALIZE);
+            } catch (InvalidStateTransitionException e) {
+              // This cannot happen.
+              LOG.warn("Unable to finalize Container {}.", container);
+            }
           }
         }
       }
@@ -277,12 +289,12 @@ public final class ContainerStateManagerImpl
   }
 
   @Override
-  public ContainerInfo getContainer(final HddsProtos.ContainerID id) {
-    lock.readLock().lock();
+  public ContainerInfo getContainer(final ContainerID id) {
+    lockManager.readLock(id);
     try {
-      return containers.getContainerInfo(ContainerID.getFromProtobuf(id));
+      return containers.getContainerInfo(id);
     } finally {
-      lock.readLock().unlock();
+      lockManager.readUnlock(id);
     }
   }
 
@@ -301,24 +313,30 @@ public final class ContainerStateManagerImpl
 
     lock.writeLock().lock();
     try {
-      if (!containers.contains(containerID)) {
-        ExecutionUtil.create(() -> {
-          transactionBuffer.addToBuffer(containerStore, containerID, container);
-          containers.addContainer(container);
-          if (pipelineManager.containsPipeline(pipelineID)) {
-            pipelineManager.addContainerToPipeline(pipelineID, containerID);
-          } else if (containerInfo.getState().
-              equals(HddsProtos.LifeCycleState.OPEN)) {
-            // Pipeline should exist, but not
-            throw new PipelineNotFoundException();
-          }
-          //recon may receive report of closed container,
-          // no corresponding Pipeline can be synced for scm.
-          // just only add the container.
-        }).onException(() -> {
-          containers.removeContainer(containerID);
-          transactionBuffer.removeFromBuffer(containerStore, containerID);
-        }).execute();
+      lockManager.writeLock(containerID);
+      try {
+        if (!containers.contains(containerID)) {
+          ExecutionUtil.create(() -> {
+            transactionBuffer.addToBuffer(containerStore,
+                containerID, container);
+            containers.addContainer(container);
+            if (pipelineManager.containsPipeline(pipelineID)) {
+              pipelineManager.addContainerToPipeline(pipelineID, containerID);
+            } else if (containerInfo.getState().
+                equals(LifeCycleState.OPEN)) {
+              // Pipeline should exist, but not
+              throw new PipelineNotFoundException();
+            }
+            //recon may receive report of closed container,
+            // no corresponding Pipeline can be synced for scm.
+            // just only add the container.
+          }).onException(() -> {
+            containers.removeContainer(containerID);
+            transactionBuffer.removeFromBuffer(containerStore, containerID);
+          }).execute();
+        }
+      } finally {
+        lockManager.writeUnlock(containerID);
       }
     } finally {
       lock.writeLock().unlock();
@@ -326,13 +344,12 @@ public final class ContainerStateManagerImpl
   }
 
   @Override
-  public boolean contains(final HddsProtos.ContainerID id) {
-    lock.readLock().lock();
+  public boolean contains(ContainerID id) {
+    lockManager.readLock(id);
     try {
-      // TODO: Remove the protobuf conversion after fixing ContainerStateMap.
-      return containers.contains(ContainerID.getFromProtobuf(id));
+      return containers.contains(id);
     } finally {
-      lock.readLock().unlock();
+      lockManager.readUnlock(id);
     }
   }
 
@@ -343,7 +360,7 @@ public final class ContainerStateManagerImpl
     // TODO: Remove the protobuf conversion after fixing ContainerStateMap.
     final ContainerID id = ContainerID.getFromProtobuf(containerID);
 
-    lock.writeLock().lock();
+    lockManager.writeLock(id);
     try {
       if (containers.contains(id)) {
         final ContainerInfo oldInfo = containers.getContainerInfo(id);
@@ -364,62 +381,66 @@ public final class ContainerStateManagerImpl
         }
       }
     } finally {
-      lock.writeLock().unlock();
+      lockManager.writeUnlock(id);
     }
   }
 
 
   @Override
-  public Set<ContainerReplica> getContainerReplicas(
-      final HddsProtos.ContainerID id) {
-    lock.readLock().lock();
+  public Set<ContainerReplica> getContainerReplicas(final ContainerID id) {
+    lockManager.readLock(id);
     try {
-      return containers.getContainerReplicas(
-          ContainerID.getFromProtobuf(id));
+      return containers.getContainerReplicas(id);
     } finally {
-      lock.readLock().unlock();
+      lockManager.readUnlock(id);
     }
   }
 
   @Override
-  public void updateContainerReplica(final HddsProtos.ContainerID id,
+  public void updateContainerReplica(final ContainerID id,
                                      final ContainerReplica replica) {
-    lock.writeLock().lock();
+    lockManager.writeLock(id);
     try {
-      containers.updateContainerReplica(ContainerID.getFromProtobuf(id),
+      containers.updateContainerReplica(id, replica);
+    } finally {
+      lockManager.writeUnlock(id);
+    }
+  }
+
+  @Override
+  public void removeContainerReplica(final ContainerID id,
+                                     final ContainerReplica replica) {
+    lockManager.writeLock(id);
+    try {
+      containers.removeContainerReplica(id,
           replica);
     } finally {
-      lock.writeLock().unlock();
-    }
-  }
-
-  @Override
-  public void removeContainerReplica(final HddsProtos.ContainerID id,
-                                     final ContainerReplica replica) {
-    lock.writeLock().lock();
-    try {
-      containers.removeContainerReplica(ContainerID.getFromProtobuf(id),
-          replica);
-    } finally {
-      lock.writeLock().unlock();
+      lockManager.writeUnlock(id);
     }
   }
 
   @Override
   public void updateDeleteTransactionId(
       final Map<ContainerID, Long> deleteTransactionMap) throws IOException {
-    lock.writeLock().lock();
-    try {
-      // TODO: Refactor this. Error handling is not done.
-      for (Map.Entry<ContainerID, Long> transaction :
-          deleteTransactionMap.entrySet()) {
+
+    // TODO: Refactor this. Error handling is not done.
+    for (Map.Entry<ContainerID, Long> transaction :
+        deleteTransactionMap.entrySet()) {
+      ContainerID containerID = transaction.getKey();
+      try {
+        lockManager.writeLock(containerID);
         final ContainerInfo info = containers.getContainerInfo(
             transaction.getKey());
+        if (info == null) {
+          LOG.warn("Cannot find container {}, transaction id is {}",
+              transaction.getKey(), transaction.getValue());
+          continue;
+        }
         info.updateDeleteTransactionId(transaction.getValue());
         transactionBuffer.addToBuffer(containerStore, info.containerID(), info);
+      } finally {
+        lockManager.writeUnlock(containerID);
       }
-    } finally {
-      lock.writeLock().unlock();
     }
   }
 
@@ -435,48 +456,49 @@ public final class ContainerStateManagerImpl
     final ContainerID lastID =
         lastUsedMap.getOrDefault(key, containerIDs.first());
 
+
     // There is a small issue here. The first time, we will skip the first
     // container. But in most cases it will not matter.
     NavigableSet<ContainerID> resultSet = containerIDs.tailSet(lastID, false);
     if (resultSet.isEmpty()) {
       resultSet = containerIDs;
     }
+    ContainerInfo selectedContainer = findContainerWithSpace(size, resultSet);
+    if (selectedContainer == null) {
 
-    lock.readLock().lock();
-    try {
-      ContainerInfo selectedContainer = findContainerWithSpace(size, resultSet);
-      if (selectedContainer == null) {
+      // If we did not find any space in the tailSet, we need to look for
+      // space in the headset, we need to pass true to deal with the
+      // situation that we have a lone container that has space. That is we
+      // ignored the last used container under the assumption we can find
+      // other containers with space, but if have a single container that is
+      // not true. Hence we need to include the last used container as the
+      // last element in the sorted set.
 
-        // If we did not find any space in the tailSet, we need to look for
-        // space in the headset, we need to pass true to deal with the
-        // situation that we have a lone container that has space. That is we
-        // ignored the last used container under the assumption we can find
-        // other containers with space, but if have a single container that is
-        // not true. Hence we need to include the last used container as the
-        // last element in the sorted set.
-
-        resultSet = containerIDs.headSet(lastID, true);
-        selectedContainer = findContainerWithSpace(size, resultSet);
-      }
-
-      // TODO: cleanup entries in lastUsedMap
-      if (selectedContainer != null) {
-        lastUsedMap.put(key, selectedContainer.containerID());
-      }
-      return selectedContainer;
-    } finally {
-      lock.readLock().unlock();
+      resultSet = containerIDs.headSet(lastID, true);
+      selectedContainer = findContainerWithSpace(size, resultSet);
     }
+
+    // TODO: cleanup entries in lastUsedMap
+    if (selectedContainer != null) {
+      lastUsedMap.put(key, selectedContainer.containerID());
+    }
+    return selectedContainer;
   }
 
   private ContainerInfo findContainerWithSpace(final long size,
-      final NavigableSet<ContainerID> searchSet) {
-    // Get the container with space to meet our request.
+                                               final NavigableSet<ContainerID>
+                                                   searchSet) {
+      // Get the container with space to meet our request.
     for (ContainerID id : searchSet) {
-      final ContainerInfo containerInfo = containers.getContainerInfo(id);
-      if (containerInfo.getUsedBytes() + size <= this.containerSize) {
-        containerInfo.updateLastUsedTime();
-        return containerInfo;
+      try {
+        lockManager.readLock(id);
+        final ContainerInfo containerInfo = containers.getContainerInfo(id);
+        if (containerInfo.getUsedBytes() + size <= this.containerSize) {
+          containerInfo.updateLastUsedTime();
+          return containerInfo;
+        }
+      } finally {
+        lockManager.readUnlock(id);
       }
     }
     return null;
@@ -485,14 +507,19 @@ public final class ContainerStateManagerImpl
 
   public void removeContainer(final HddsProtos.ContainerID id)
       throws IOException {
+    final ContainerID cid = ContainerID.getFromProtobuf(id);
     lock.writeLock().lock();
     try {
-      final ContainerID cid = ContainerID.getFromProtobuf(id);
-      final ContainerInfo containerInfo = containers.getContainerInfo(cid);
-      ExecutionUtil.create(() -> {
-        transactionBuffer.removeFromBuffer(containerStore, cid);
-        containers.removeContainer(cid);
-      }).onException(() -> containerStore.put(cid, containerInfo)).execute();
+      lockManager.writeLock(cid);
+      try {
+        final ContainerInfo containerInfo = containers.getContainerInfo(cid);
+        ExecutionUtil.create(() -> {
+          transactionBuffer.removeFromBuffer(containerStore, cid);
+          containers.removeContainer(cid);
+        }).onException(() -> containerStore.put(cid, containerInfo)).execute();
+      } finally {
+        lockManager.writeUnlock(cid);
+      }
     } finally {
       lock.writeLock().unlock();
     }
@@ -561,21 +588,21 @@ public final class ContainerStateManagerImpl
       return this;
     }
 
-    public ContainerStateManagerV2 build() throws IOException {
+    public ContainerStateManager build() throws IOException {
       Preconditions.checkNotNull(conf);
       Preconditions.checkNotNull(pipelineMgr);
       Preconditions.checkNotNull(table);
 
-      final ContainerStateManagerV2 csm = new ContainerStateManagerImpl(
+      final ContainerStateManager csm = new ContainerStateManagerImpl(
           conf, pipelineMgr, table, transactionBuffer);
 
       final SCMHAInvocationHandler invocationHandler =
           new SCMHAInvocationHandler(RequestType.CONTAINER, csm,
               scmRatisServer);
 
-      return (ContainerStateManagerV2) Proxy.newProxyInstance(
+      return (ContainerStateManager) Proxy.newProxyInstance(
           SCMHAInvocationHandler.class.getClassLoader(),
-          new Class<?>[]{ContainerStateManagerV2.class}, invocationHandler);
+          new Class<?>[]{ContainerStateManager.class}, invocationHandler);
     }
 
   }
