@@ -21,7 +21,10 @@ import java.io.File;
 import java.io.IOException;
 import java.math.BigInteger;
 import java.security.cert.X509Certificate;
+import java.util.Map;
+import java.util.concurrent.ConcurrentHashMap;
 
+import com.google.protobuf.ByteString;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.DeletedBlocksTransaction;
 import org.apache.hadoop.hdds.scm.container.ContainerID;
@@ -48,10 +51,12 @@ import static org.apache.hadoop.hdds.scm.metadata.SCMDBDefinition.MOVE;
 import static org.apache.hadoop.hdds.scm.metadata.SCMDBDefinition.PIPELINES;
 import static org.apache.hadoop.hdds.scm.metadata.SCMDBDefinition.REVOKED_CERTS;
 import static org.apache.hadoop.hdds.scm.metadata.SCMDBDefinition.REVOKED_CERTS_V2;
+import static org.apache.hadoop.hdds.scm.metadata.SCMDBDefinition.STATEFUL_SERVICE_CONFIG;
 import static org.apache.hadoop.hdds.scm.metadata.SCMDBDefinition.TRANSACTIONINFO;
 import static org.apache.hadoop.hdds.scm.metadata.SCMDBDefinition.VALID_CERTS;
 import static org.apache.hadoop.hdds.scm.metadata.SCMDBDefinition.VALID_SCM_CERTS;
 import static org.apache.hadoop.hdds.scm.metadata.SCMDBDefinition.SEQUENCE_ID;
+import static org.apache.hadoop.hdds.scm.metadata.SCMDBDefinition.META;
 import static org.apache.hadoop.ozone.OzoneConsts.DB_TRANSIENT_MARKER;
 
 import org.apache.ratis.util.ExitUtils;
@@ -88,11 +93,18 @@ public class SCMMetadataStoreImpl implements SCMMetadataStore {
 
   private Table<ContainerID, MoveDataNodePair> moveTable;
 
+  private Table<String, String> metaTable;
+
+  private Table<String, ByteString> statefulServiceConfigTable;
+
   private static final Logger LOG =
       LoggerFactory.getLogger(SCMMetadataStoreImpl.class);
   private DBStore store;
   private final OzoneConfiguration configuration;
 
+  private SCMMetadataStoreMetrics metrics;
+  private Map<String, Table<?, ?>> tableMap = new ConcurrentHashMap<>();
+
   /**
    * Constructs the metadata store and starts the DB Services.
    *
@@ -132,48 +144,62 @@ public class SCMMetadataStoreImpl implements SCMMetadataStore {
       deletedBlocksTable =
           DELETED_BLOCKS.getTable(this.store);
 
-      checkTableStatus(deletedBlocksTable,
-          DELETED_BLOCKS.getName());
+      checkAndPopulateTable(deletedBlocksTable, DELETED_BLOCKS.getName());
 
       validCertsTable = VALID_CERTS.getTable(store);
 
-      checkTableStatus(validCertsTable, VALID_CERTS.getName());
+      checkAndPopulateTable(validCertsTable, VALID_CERTS.getName());
 
       validSCMCertsTable = VALID_SCM_CERTS.getTable(store);
 
-      checkTableStatus(validSCMCertsTable, VALID_SCM_CERTS.getName());
+      checkAndPopulateTable(validSCMCertsTable, VALID_SCM_CERTS.getName());
 
       revokedCertsTable = REVOKED_CERTS.getTable(store);
 
-      checkTableStatus(revokedCertsTable, REVOKED_CERTS.getName());
+      checkAndPopulateTable(revokedCertsTable, REVOKED_CERTS.getName());
 
       revokedCertsV2Table = REVOKED_CERTS_V2.getTable(store);
 
-      checkTableStatus(revokedCertsV2Table, REVOKED_CERTS_V2.getName());
+      checkAndPopulateTable(revokedCertsV2Table, REVOKED_CERTS_V2.getName());
 
       pipelineTable = PIPELINES.getTable(store);
 
-      checkTableStatus(pipelineTable, PIPELINES.getName());
+      checkAndPopulateTable(pipelineTable, PIPELINES.getName());
 
       containerTable = CONTAINERS.getTable(store);
 
-      checkTableStatus(containerTable, CONTAINERS.getName());
+      checkAndPopulateTable(containerTable, CONTAINERS.getName());
 
       transactionInfoTable = TRANSACTIONINFO.getTable(store);
 
-      checkTableStatus(transactionInfoTable, TRANSACTIONINFO.getName());
+      checkAndPopulateTable(transactionInfoTable, TRANSACTIONINFO.getName());
 
       crlInfoTable = CRLS.getTable(store);
 
+      checkAndPopulateTable(crlInfoTable, CRLS.getName());
+
       crlSequenceIdTable = CRL_SEQUENCE_ID.getTable(store);
 
+      checkAndPopulateTable(crlInfoTable, CRL_SEQUENCE_ID.getName());
+
       sequenceIdTable = SEQUENCE_ID.getTable(store);
 
-      checkTableStatus(sequenceIdTable, SEQUENCE_ID.getName());
+      checkAndPopulateTable(sequenceIdTable, SEQUENCE_ID.getName());
 
       moveTable = MOVE.getTable(store);
 
-      checkTableStatus(moveTable, MOVE.getName());
+      checkAndPopulateTable(moveTable, MOVE.getName());
+
+      metaTable = META.getTable(store);
+
+      checkAndPopulateTable(metaTable, META.getName());
+
+      statefulServiceConfigTable = STATEFUL_SERVICE_CONFIG.getTable(store);
+
+      checkAndPopulateTable(statefulServiceConfigTable,
+          STATEFUL_SERVICE_CONFIG.getName());
+
+      metrics = SCMMetadataStoreMetrics.create(this);
     }
   }
 
@@ -183,6 +209,10 @@ public class SCMMetadataStoreImpl implements SCMMetadataStore {
       store.close();
       store = null;
     }
+    if (metrics != null) {
+      metrics.unRegister();
+      metrics = null;
+    }
   }
 
   @Override
@@ -237,12 +267,13 @@ public class SCMMetadataStoreImpl implements SCMMetadataStore {
   }
 
   @Override
-  public TableIterator getAllCerts(CertificateStore.CertType certType) {
-    if(certType == CertificateStore.CertType.VALID_CERTS) {
+  public TableIterator getAllCerts(CertificateStore.CertType certType)
+      throws IOException {
+    if (certType == CertificateStore.CertType.VALID_CERTS) {
       return validCertsTable.iterator();
     }
 
-    if(certType == CertificateStore.CertType.REVOKED_CERTS) {
+    if (certType == CertificateStore.CertType.REVOKED_CERTS) {
       return revokedCertsTable.iterator();
     }
 
@@ -279,7 +310,17 @@ public class SCMMetadataStoreImpl implements SCMMetadataStore {
     return moveTable;
   }
 
-  private void checkTableStatus(Table table, String name) throws IOException {
+  @Override
+  public Table<String, String> getMetaTable() {
+    return metaTable;
+  }
+
+  public Table<String, ByteString> getStatefulServiceConfigTable() {
+    return statefulServiceConfigTable;
+  }
+
+  private void checkAndPopulateTable(Table table, String name)
+      throws IOException {
     String logMessage = "Unable to get a reference to %s table. Cannot " +
         "continue.";
     String errMsg = "Inconsistent DB state, Table - %s. Please check the" +
@@ -288,6 +329,14 @@ public class SCMMetadataStoreImpl implements SCMMetadataStore {
       LOG.error(String.format(logMessage, name));
       throw new IOException(String.format(errMsg, name));
     }
+    tableMap.put(name, table);
   }
 
+  Map<String, Table<?, ?>> getTableMap() {
+    return tableMap;
+  }
+
+  SCMMetadataStoreMetrics getMetrics() {
+    return metrics;
+  }
 }
