@@ -19,7 +19,7 @@
 package org.apache.hadoop.hdds.scm.node;
 
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
-import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
+import org.apache.hadoop.hdds.protocol.proto.HddsProtos.DatanodeUsageInfoProto;
 import org.apache.hadoop.hdds.scm.container.placement.metrics.SCMNodeStat;
 
 import java.util.Comparator;
@@ -180,17 +180,16 @@ public class DatanodeUsageInfo {
    *
    * @return Protobuf HddsProtos.DatanodeUsageInfo
    */
-  public HddsProtos.DatanodeUsageInfoProto toProto() {
-    return toProtoBuilder().build();
+  public DatanodeUsageInfoProto toProto(int clientVersion) {
+    return toProtoBuilder(clientVersion).build();
   }
 
-  private HddsProtos.DatanodeUsageInfoProto.Builder toProtoBuilder() {
-    HddsProtos.DatanodeUsageInfoProto.Builder builder =
-        HddsProtos.DatanodeUsageInfoProto.newBuilder();
+  private DatanodeUsageInfoProto.Builder toProtoBuilder(int clientVersion) {
+    DatanodeUsageInfoProto.Builder builder =
+        DatanodeUsageInfoProto.newBuilder();
 
     if (datanodeDetails != null) {
-      builder.setNode(
-          datanodeDetails.toProto(datanodeDetails.getCurrentVersion()));
+      builder.setNode(datanodeDetails.toProto(clientVersion));
     }
     if (scmNodeStat != null) {
       builder.setCapacity(scmNodeStat.getCapacity().get());
