@@ -24,7 +24,7 @@ import org.apache.hadoop.hdds.protocol.proto.HddsProtos.NodeOperationalState;
 import org.apache.hadoop.hdds.scm.DatanodeAdminError;
 import org.apache.hadoop.hdds.scm.ScmConfigKeys;
 import org.apache.hadoop.hdds.scm.container.ContainerManager;
-import org.apache.hadoop.hdds.scm.container.ReplicationManager;
+import org.apache.hadoop.hdds.scm.container.replication.ReplicationManager;
 import org.apache.hadoop.hdds.scm.ha.SCMContext;
 import org.apache.hadoop.hdds.scm.node.states.NodeNotFoundException;
 import org.apache.hadoop.hdds.server.events.EventPublisher;
@@ -60,6 +60,9 @@ public class NodeDecommissionManager {
   private boolean useHostnames;
   private long monitorInterval;
 
+  // Decommissioning and Maintenance mode progress related metrics.
+  private NodeDecommissionMetrics metrics;
+
   private static final Logger LOG =
       LoggerFactory.getLogger(NodeDecommissionManager.class);
 
@@ -85,20 +88,20 @@ public class NodeDecommissionManager {
       return port;
     }
 
-    private void parseHostname() throws InvalidHostStringException{
+    private void parseHostname() throws InvalidHostStringException {
       try {
         // A URI *must* have a scheme, so just create a fake one
-        URI uri = new URI("empty://"+rawHostname.trim());
+        URI uri = new URI("empty://" + rawHostname.trim());
         this.hostname = uri.getHost();
         this.port = uri.getPort();
 
         if (this.hostname == null) {
-          throw new InvalidHostStringException("The string "+rawHostname+
+          throw new InvalidHostStringException("The string " + rawHostname +
               " does not contain a value hostname or hostname:port definition");
         }
       } catch (URISyntaxException e) {
         throw new InvalidHostStringException(
-            "Unable to parse the hoststring "+rawHostname, e);
+            "Unable to parse the hoststring " + rawHostname, e);
       }
     }
   }
@@ -138,7 +141,7 @@ public class NodeDecommissionManager {
         results.add(found.get(0));
       } else if (found.size() > 1) {
         DatanodeDetails match = null;
-        for(DatanodeDetails dn : found) {
+        for (DatanodeDetails dn : found) {
           if (validateDNPortMatch(host.getPort(), dn)) {
             match = dn;
             break;
@@ -181,6 +184,7 @@ public class NodeDecommissionManager {
     this.scmContext = scmContext;
     this.eventQueue = eventQueue;
     this.replicationManager = rm;
+    this.metrics = null;
 
     executor = Executors.newScheduledThreadPool(1,
         new ThreadFactoryBuilder().setNameFormat("DatanodeAdminManager-%d")
@@ -208,7 +212,8 @@ public class NodeDecommissionManager {
 
     monitor = new DatanodeAdminMonitorImpl(conf, eventQueue, nodeManager,
         replicationManager);
-
+    this.metrics = NodeDecommissionMetrics.create();
+    monitor.setMetrics(this.metrics);
     executor.scheduleAtFixedRate(monitor, monitorInterval, monitorInterval,
         TimeUnit.SECONDS);
   }
@@ -231,7 +236,7 @@ public class NodeDecommissionManager {
         // NodeNotFoundException here expect if the node is remove in the
         // very short window between validation and starting decom. Therefore
         // log a warning and ignore the exception
-        LOG.warn("The host {} was not found in SCM. Ignoring the request to "+
+        LOG.warn("The host {} was not found in SCM. Ignoring the request to " +
             "decommission it", dn.getHostName());
         errors.add(new DatanodeAdminError(dn.getHostName(),
             "The host was not found in SCM"));
@@ -274,12 +279,12 @@ public class NodeDecommissionManager {
           dn, NodeOperationalState.DECOMMISSIONING);
       monitor.startMonitoring(dn);
     } else if (nodeStatus.isDecommission()) {
-      LOG.info("Start Decommission called on node {} in state {}. Nothing to "+
+      LOG.info("Start Decommission called on node {} in state {}. Nothing to " +
           "do.", dn, opState);
     } else {
       LOG.error("Cannot decommission node {} in state {}", dn, opState);
-      throw new InvalidNodeStateException("Cannot decommission node "+
-          dn +" in state "+ opState);
+      throw new InvalidNodeStateException("Cannot decommission node " +
+          dn + " in state " + opState);
     }
   }
 
@@ -296,7 +301,7 @@ public class NodeDecommissionManager {
         // NodeNotFoundException here expect if the node is remove in the
         // very short window between validation and starting decom. Therefore
         // log a warning and ignore the exception
-        LOG.warn("Host {} was not found in SCM. Ignoring the request to "+
+        LOG.warn("Host {} was not found in SCM. Ignoring the request to " +
             "recommission it.", dn.getHostName());
         errors.add(new DatanodeAdminError(dn.getHostName(),
             "The host was not found in SCM"));
@@ -306,7 +311,7 @@ public class NodeDecommissionManager {
   }
 
   public synchronized void recommission(DatanodeDetails dn)
-      throws NodeNotFoundException{
+      throws NodeNotFoundException {
     NodeStatus nodeStatus = getNodeStatus(dn);
     NodeOperationalState opState = nodeStatus.getOperationalState();
     if (opState != NodeOperationalState.IN_SERVICE) {
@@ -315,7 +320,7 @@ public class NodeDecommissionManager {
       monitor.stopMonitoring(dn);
       LOG.info("Queued node {} for recommission", dn);
     } else {
-      LOG.info("Recommission called on node {} with state {}. "+
+      LOG.info("Recommission called on node {} with state {}. " +
           "Nothing to do.", dn, opState);
     }
   }
@@ -333,7 +338,7 @@ public class NodeDecommissionManager {
         // NodeNotFoundException here expect if the node is remove in the
         // very short window between validation and starting decom. Therefore
         // log a warning and ignore the exception
-        LOG.warn("The host {} was not found in SCM. Ignoring the request to "+
+        LOG.warn("The host {} was not found in SCM. Ignoring the request to " +
             "start maintenance on it", dn.getHostName());
       } catch (InvalidNodeStateException e) {
         errors.add(new DatanodeAdminError(dn.getHostName(), e.getMessage()));
@@ -360,12 +365,12 @@ public class NodeDecommissionManager {
       monitor.startMonitoring(dn);
       LOG.info("Starting Maintenance for node {}", dn);
     } else if (nodeStatus.isMaintenance()) {
-      LOG.info("Starting Maintenance called on node {} with state {}. "+
+      LOG.info("Starting Maintenance called on node {} with state {}. " +
           "Nothing to do.", dn, opState);
     } else {
       LOG.error("Cannot start maintenance on node {} in state {}", dn, opState);
-      throw new InvalidNodeStateException("Cannot start maintenance on node "+
-          dn +" in state "+ opState);
+      throw new InvalidNodeStateException("Cannot start maintenance on node " +
+          dn + " in state " + opState);
     }
   }
 
@@ -373,6 +378,7 @@ public class NodeDecommissionManager {
    *  Stops the decommission monitor from running when SCM is shutdown.
    */
   public void stop() {
+    metrics.unRegister();
     if (executor != null) {
       executor.shutdown();
     }
