@@ -153,13 +153,14 @@ public class NodeStatus {
   }
 
   /**
-   * Returns true if the nodeStatus is healthy (ie not stale or dead) and false
-   * otherwise.
+   * Returns true if the nodeStatus is healthy or healthy_readonly (ie not stale
+   * or dead) and false otherwise.
    *
-   * @return True if the node is Healthy, false otherwise
+   * @return True if the node is healthy or healthy_readonly, false otherwise.
    */
   public boolean isHealthy() {
-    return health == HddsProtos.NodeState.HEALTHY;
+    return health == HddsProtos.NodeState.HEALTHY
+        || health == HddsProtos.NodeState.HEALTHY_READONLY;
   }
 
   /**
@@ -209,8 +210,8 @@ public class NodeStatus {
 
   @Override
   public String toString() {
-    return "OperationalState: "+operationalState+" Health: "+health+
-        " OperationStateExpiry: "+opStateExpiryEpochSeconds;
+    return "OperationalState: " + operationalState + " Health: " + health +
+        " OperationStateExpiry: " + opStateExpiryEpochSeconds;
   }
 
 }
\ No newline at end of file
