@@ -27,6 +27,8 @@ import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos.NodeOperationalState;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos.NodeState;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos;
+import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.SCMCommandProto;
+import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.CommandQueueReportProto;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.LayoutVersionProto;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.NodeReportProto;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.PipelineReportsProto;
@@ -48,6 +50,7 @@ import org.apache.hadoop.hdds.scm.pipeline.PipelineID;
 import org.apache.hadoop.hdds.scm.pipeline.PipelineManager;
 import org.apache.hadoop.hdds.scm.pipeline.PipelineNotFoundException;
 import org.apache.hadoop.hdds.scm.server.SCMStorageConfig;
+import org.apache.hadoop.hdds.scm.server.upgrade.FinalizationManager;
 import org.apache.hadoop.hdds.server.events.EventPublisher;
 import org.apache.hadoop.hdds.upgrade.HDDSLayoutVersionManager;
 import org.apache.hadoop.ipc.Server;
@@ -59,6 +62,7 @@ import org.apache.hadoop.ozone.OzoneConsts;
 import org.apache.hadoop.ozone.protocol.VersionResponse;
 import org.apache.hadoop.ozone.protocol.commands.CommandForDatanode;
 import org.apache.hadoop.ozone.protocol.commands.FinalizeNewLayoutVersionCommand;
+import org.apache.hadoop.ozone.protocol.commands.RefreshVolumeUsageCommand;
 import org.apache.hadoop.ozone.protocol.commands.RegisteredCommand;
 import org.apache.hadoop.ozone.protocol.commands.SCMCommand;
 import org.apache.hadoop.ozone.protocol.commands.SetNodeOperationalStateCommand;
@@ -85,6 +89,7 @@ import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ScheduledFuture;
 import java.util.stream.Collectors;
 
+import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.NodeOperationalState.IN_SERVICE;
 import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.NodeState.HEALTHY;
 import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.NodeState.HEALTHY_READONLY;
 
@@ -136,7 +141,7 @@ public class SCMNodeManager implements NodeManager {
                         HDDSLayoutVersionManager layoutVersionManager) {
     this.scmNodeEventPublisher = eventPublisher;
     this.nodeStateManager = new NodeStateManager(conf, eventPublisher,
-        layoutVersionManager);
+        layoutVersionManager, scmContext);
     this.version = VersionInfo.getLatestVersion();
     this.commandQueue = new CommandQueue();
     this.scmStorageConfig = scmStorageConfig;
@@ -268,7 +273,7 @@ public class SCMNodeManager implements NodeManager {
    */
   @Override
   public void setNodeOperationalState(DatanodeDetails datanodeDetails,
-      NodeOperationalState newState) throws NodeNotFoundException{
+      NodeOperationalState newState) throws NodeNotFoundException {
     setNodeOperationalState(datanodeDetails, newState, 0);
   }
 
@@ -283,7 +288,7 @@ public class SCMNodeManager implements NodeManager {
   @Override
   public void setNodeOperationalState(DatanodeDetails datanodeDetails,
       NodeOperationalState newState, long opStateExpiryEpocSec)
-      throws NodeNotFoundException{
+      throws NodeNotFoundException {
     nodeStateManager.setNodeOperationalState(
         datanodeDetails, newState, opStateExpiryEpocSec);
   }
@@ -357,33 +362,36 @@ public class SCMNodeManager implements NodeManager {
           .build();
     }
 
-    if (!isNodeRegistered(datanodeDetails)) {
-      InetAddress dnAddress = Server.getRemoteIp();
-      if (dnAddress != null) {
-        // Mostly called inside an RPC, update ip and peer hostname
+    InetAddress dnAddress = Server.getRemoteIp();
+    if (dnAddress != null) {
+      // Mostly called inside an RPC, update ip
+      if (!useHostname) {
         datanodeDetails.setHostName(dnAddress.getHostName());
-        datanodeDetails.setIpAddress(dnAddress.getHostAddress());
       }
-      try {
-        String dnsName;
-        String networkLocation;
-        datanodeDetails.setNetworkName(datanodeDetails.getUuidString());
-        if (useHostname) {
-          dnsName = datanodeDetails.getHostName();
-        } else {
-          dnsName = datanodeDetails.getIpAddress();
-        }
-        networkLocation = nodeResolve(dnsName);
-        if (networkLocation != null) {
-          datanodeDetails.setNetworkLocation(networkLocation);
-        }
+      datanodeDetails.setIpAddress(dnAddress.getHostAddress());
+    }
 
+    String dnsName;
+    String networkLocation;
+    datanodeDetails.setNetworkName(datanodeDetails.getUuidString());
+    if (useHostname) {
+      dnsName = datanodeDetails.getHostName();
+    } else {
+      dnsName = datanodeDetails.getIpAddress();
+    }
+    networkLocation = nodeResolve(dnsName);
+    if (networkLocation != null) {
+      datanodeDetails.setNetworkLocation(networkLocation);
+    }
+
+    if (!isNodeRegistered(datanodeDetails)) {
+      try {
         clusterMap.add(datanodeDetails);
         nodeStateManager.addNode(datanodeDetails, layoutInfo);
         // Check that datanode in nodeStateManager has topology parent set
         DatanodeDetails dn = nodeStateManager.getNode(datanodeDetails);
         Preconditions.checkState(dn.getParent() != null);
-        addEntryTodnsToUuidMap(dnsName, datanodeDetails.getUuidString());
+        addEntryToDnsToUuidMap(dnsName, datanodeDetails.getUuidString());
         // Updating Node Report, as registration is successful
         processNodeReport(datanodeDetails, nodeReport);
         LOG.info("Registered Data node : {}", datanodeDetails);
@@ -397,6 +405,42 @@ public class SCMNodeManager implements NodeManager {
         LOG.error("Cannot find datanode {} from nodeStateManager",
             datanodeDetails.toString());
       }
+    } else {
+      // Update datanode if it is registered but the ip or hostname changes
+      try {
+        final DatanodeInfo datanodeInfo =
+                nodeStateManager.getNode(datanodeDetails);
+        if (!datanodeInfo.getIpAddress().equals(datanodeDetails.getIpAddress())
+                || !datanodeInfo.getHostName()
+                .equals(datanodeDetails.getHostName())) {
+          LOG.info("Updating data node {} from {} to {}",
+                  datanodeDetails.getUuidString(),
+                  datanodeInfo,
+                  datanodeDetails);
+          clusterMap.update(datanodeInfo, datanodeDetails);
+
+          String oldDnsName;
+          if (useHostname) {
+            oldDnsName = datanodeInfo.getHostName();
+          } else {
+            oldDnsName = datanodeInfo.getIpAddress();
+          }
+          updateEntryFromDnsToUuidMap(oldDnsName,
+                  dnsName,
+                  datanodeDetails.getUuidString());
+
+          nodeStateManager.updateNode(datanodeDetails, layoutInfo);
+          DatanodeDetails dn = nodeStateManager.getNode(datanodeDetails);
+          Preconditions.checkState(dn.getParent() != null);
+          processNodeReport(datanodeDetails, nodeReport);
+          LOG.info("Updated Datanode to: {}", dn);
+          scmNodeEventPublisher
+                  .fireEvent(SCMEvents.NODE_ADDRESS_UPDATE, dn);
+        }
+      } catch (NodeNotFoundException e) {
+        LOG.error("Cannot find datanode {} from nodeStateManager",
+                datanodeDetails);
+      }
     }
 
     return RegisteredCommand.newBuilder().setErrorCode(ErrorCode.success)
@@ -413,11 +457,9 @@ public class SCMNodeManager implements NodeManager {
    * @param dnsName String representing the hostname or IP of the node
    * @param uuid    String representing the UUID of the registered node.
    */
-  @SuppressFBWarnings(value = "AT_OPERATION_SEQUENCE_ON_CONCURRENT_ABSTRACTION",
-      justification = "The method is synchronized and this is the only place " +
-          "dnsToUuidMap is modified")
-  private synchronized void addEntryTodnsToUuidMap(
-      String dnsName, String uuid) {
+  @SuppressFBWarnings(value = "AT_OPERATION_SEQUENCE_ON_CONCURRENT_ABSTRACTION")
+  private synchronized void addEntryToDnsToUuidMap(
+          String dnsName, String uuid) {
     Set<String> dnList = dnsToUuidMap.get(dnsName);
     if (dnList == null) {
       dnList = ConcurrentHashMap.newKeySet();
@@ -426,6 +468,26 @@ public class SCMNodeManager implements NodeManager {
     dnList.add(uuid);
   }
 
+  private synchronized void removeEntryFromDnsToUuidMap(String dnsName) {
+    if (!dnsToUuidMap.containsKey(dnsName)) {
+      return;
+    }
+    Set<String> dnSet = dnsToUuidMap.get(dnsName);
+    if (dnSet.contains(dnsName)) {
+      dnSet.remove(dnsName);
+    }
+    if (dnSet.isEmpty()) {
+      dnsToUuidMap.remove(dnsName);
+    }
+  }
+
+  private synchronized void updateEntryFromDnsToUuidMap(String oldDnsName,
+                                                        String newDnsName,
+                                                        String uuid) {
+    removeEntryFromDnsToUuidMap(oldDnsName);
+    addEntryToDnsToUuidMap(newDnsName, uuid);
+  }
+
   /**
    * Send heartbeat to indicate the datanode is alive and doing well.
    *
@@ -579,12 +641,13 @@ public class SCMNodeManager implements NodeManager {
           layoutVersionReport.toString().replaceAll("\n", "\\\\n"));
     }
 
+    // Software layout version is hardcoded to the SCM.
     int scmSlv = scmLayoutVersionManager.getSoftwareLayoutVersion();
-    int scmMlv = scmLayoutVersionManager.getMetadataLayoutVersion();
     int dnSlv = layoutVersionReport.getSoftwareLayoutVersion();
     int dnMlv = layoutVersionReport.getMetadataLayoutVersion();
 
-    // If the data node slv is > scm slv => log error condition
+    // A datanode with a larger software layout version is from a future
+    // version of ozone. It should not have been added to the cluster.
     if (dnSlv > scmSlv) {
       LOG.error("Invalid data node in the cluster : {}. " +
               "DataNode SoftwareLayoutVersion = {}, SCM " +
@@ -592,33 +655,86 @@ public class SCMNodeManager implements NodeManager {
           datanodeDetails.getHostName(), dnSlv, scmSlv);
     }
 
-    // If the datanode slv < scm slv, it can not be allowed to be part of
-    // any pipeline. However it can be allowed to join the cluster
-    if (dnMlv < scmMlv) {
-      LOG.warn("Data node {} can not be used in any pipeline in the " +
-              "cluster. " + "DataNode MetadataLayoutVersion = {}, SCM " +
-              "MetadataLayoutVersion = {}",
-          datanodeDetails.getHostName(), dnMlv, scmMlv);
+    if (FinalizationManager.shouldTellDatanodesToFinalize(
+        scmContext.getFinalizationCheckpoint())) {
+      // Because we have crossed the MLV_EQUALS_SLV checkpoint, SCM metadata
+      // layout version will not change. We can now compare it to the
+      // datanodes' metadata layout versions to tell them to finalize.
+      int scmMlv = scmLayoutVersionManager.getMetadataLayoutVersion();
 
-      FinalizeNewLayoutVersionCommand finalizeCmd =
-          new FinalizeNewLayoutVersionCommand(true,
-          LayoutVersionProto.newBuilder()
-              .setSoftwareLayoutVersion(dnSlv)
-              .setMetadataLayoutVersion(dnSlv).build());
-      try {
-        finalizeCmd.setTerm(scmContext.getTermOfLeader());
+      // If the datanode mlv < scm mlv, it can not be allowed to be part of
+      // any pipeline. However it can be allowed to join the cluster
+      if (dnMlv < scmMlv) {
+        LOG.warn("Data node {} can not be used in any pipeline in the " +
+                "cluster. " + "DataNode MetadataLayoutVersion = {}, SCM " +
+                "MetadataLayoutVersion = {}",
+            datanodeDetails.getHostName(), dnMlv, scmMlv);
 
-        // Send Finalize command to the data node. Its OK to
-        // send Finalize command multiple times.
-        scmNodeEventPublisher.fireEvent(SCMEvents.DATANODE_COMMAND,
-            new CommandForDatanode<>(datanodeDetails.getUuid(), finalizeCmd));
-      } catch(NotLeaderException ex) {
-        LOG.warn("Skip sending finalize upgrade command since current SCM is" +
-            "not leader.", ex);
+        FinalizeNewLayoutVersionCommand finalizeCmd =
+            new FinalizeNewLayoutVersionCommand(true,
+                LayoutVersionProto.newBuilder()
+                    .setSoftwareLayoutVersion(dnSlv)
+                    .setMetadataLayoutVersion(dnSlv).build());
+        if (scmContext.isLeader()) {
+          try {
+            finalizeCmd.setTerm(scmContext.getTermOfLeader());
+
+            // Send Finalize command to the data node. Its OK to
+            // send Finalize command multiple times.
+            scmNodeEventPublisher.fireEvent(SCMEvents.DATANODE_COMMAND,
+                new CommandForDatanode<>(datanodeDetails.getUuid(),
+                    finalizeCmd));
+          } catch (NotLeaderException ex) {
+            LOG.warn("Skip sending finalize upgrade command since current SCM" +
+                " is not leader.", ex);
+          }
+        }
       }
     }
   }
 
+  /**
+   * Process Command Queue Reports from the Datanode Heartbeat.
+   *
+   * @param datanodeDetails
+   * @param commandQueueReportProto
+   */
+  @Override
+  public void processNodeCommandQueueReport(DatanodeDetails datanodeDetails,
+      CommandQueueReportProto commandQueueReportProto) {
+    LOG.debug("Processing Command Queue Report from [datanode={}]",
+        datanodeDetails.getHostName());
+    if (LOG.isTraceEnabled()) {
+      LOG.trace("Command Queue Report is received from [datanode={}]: " +
+          "<json>{}</json>", datanodeDetails.getHostName(),
+          commandQueueReportProto.toString().replaceAll("\n", "\\\\n"));
+    }
+    try {
+      DatanodeInfo datanodeInfo = nodeStateManager.getNode(datanodeDetails);
+      if (commandQueueReportProto != null) {
+        datanodeInfo.setCommandCounts(commandQueueReportProto);
+        metrics.incNumNodeCommandQueueReportProcessed();
+      }
+    } catch (NodeNotFoundException e) {
+      metrics.incNumNodeCommandQueueReportProcessingFailed();
+      LOG.warn("Got Command Queue Report from unregistered datanode {}",
+          datanodeDetails);
+    }
+  }
+
+  /**
+   * Get the number of commands of the given type queued on the datanode at the
+   * last heartbeat. If the Datanode has not reported information for the given
+   * command type, -1 will be returned.
+   * @param cmdType
+   * @return The queued count or -1 if no data has been received from the DN.
+   */
+  public int getNodeQueuedCommandCount(DatanodeDetails datanodeDetails,
+      SCMCommandProto.Type cmdType) throws NodeNotFoundException {
+    DatanodeInfo datanodeInfo = nodeStateManager.getNode(datanodeDetails);
+    return datanodeInfo.getCommandCount(cmdType);
+  }
+
   /**
    * Returns the aggregated node stats.
    *
@@ -680,7 +796,7 @@ public class SCMNodeManager implements NodeManager {
   public List<DatanodeUsageInfo> getMostOrLeastUsedDatanodes(
       boolean mostUsed) {
     List<DatanodeDetails> healthyNodes =
-        getNodes(NodeOperationalState.IN_SERVICE, NodeState.HEALTHY);
+        getNodes(IN_SERVICE, NodeState.HEALTHY);
 
     List<DatanodeUsageInfo> datanodeUsageInfoList =
         new ArrayList<>(healthyNodes.size());
@@ -764,7 +880,7 @@ public class SCMNodeManager implements NodeManager {
     for (DatanodeInfo dni : nodeStateManager.getAllNodes()) {
       NodeStatus status = dni.getNodeStatus();
       nodes.get(status.getOperationalState().name())
-          .compute(status.getHealth().name(), (k, v) -> v+1);
+          .compute(status.getHealth().name(), (k, v) -> v + 1);
     }
     return nodes;
   }
@@ -972,6 +1088,13 @@ public class SCMNodeManager implements NodeManager {
     nodeStateManager.addContainer(datanodeDetails.getUuid(), containerId);
   }
 
+  @Override
+  public void removeContainer(final DatanodeDetails datanodeDetails,
+                           final ContainerID containerId)
+      throws NodeNotFoundException {
+    nodeStateManager.removeContainer(datanodeDetails.getUuid(), containerId);
+  }
+
   /**
    * Update set of containers available on a datanode.
    *
@@ -988,7 +1111,9 @@ public class SCMNodeManager implements NodeManager {
   }
 
   /**
-   * Return set of containerIDs available on a datanode.
+   * Return set of containerIDs available on a datanode. This is a copy of the
+   * set which resides inside NodeManager and hence can be modified without
+   * synchronization or side effects.
    *
    * @param datanodeDetails - DatanodeID
    * @return - set of containerIDs
@@ -1008,6 +1133,25 @@ public class SCMNodeManager implements NodeManager {
     this.commandQueue.addCommand(dnId, command);
   }
 
+  /**
+   * send refresh command to all the healthy datanodes to refresh
+   * volume usage info immediately.
+   */
+  @Override
+  public void refreshAllHealthyDnUsageInfo() {
+    RefreshVolumeUsageCommand refreshVolumeUsageCommand =
+        new RefreshVolumeUsageCommand();
+    try {
+      refreshVolumeUsageCommand.setTerm(scmContext.getTermOfLeader());
+    } catch (NotLeaderException nle) {
+      LOG.warn("Skip sending refreshVolumeUsage command,"
+          + " since current SCM is not leader.", nle);
+      return;
+    }
+    getNodes(IN_SERVICE, HEALTHY).forEach(datanode ->
+        addDatanodeCommand(datanode.getUuid(), refreshVolumeUsageCommand));
+  }
+
   /**
    * This method is called by EventQueue whenever someone adds a new
    * DATANODE_COMMAND to the Queue.
