@@ -29,6 +29,7 @@ import org.slf4j.LoggerFactory;
 
 import java.io.IOException;
 import java.util.Set;
+import java.util.concurrent.TimeoutException;
 
 /**
  * Handler which is fired when a datanode starts admin (decommission or
@@ -59,7 +60,7 @@ public class StartDatanodeAdminHandler
       try {
         Pipeline pipeline = pipelineManager.getPipeline(pipelineID);
         pipelineManager.closePipeline(pipeline, false);
-      } catch (IOException e) {
+      } catch (IOException | TimeoutException e) {
         LOG.info("Could not finalize pipeline={} for dn={}", pipelineID,
             datanodeDetails);
       }
