@@ -23,10 +23,13 @@ import java.io.IOException;
 import java.util.Collection;
 import java.util.List;
 import java.util.NavigableSet;
+import java.util.Set;
+import java.util.concurrent.TimeoutException;
 
 import org.apache.hadoop.hdds.client.ReplicationConfig;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.hdds.scm.container.ContainerID;
+import org.apache.hadoop.hdds.scm.container.ContainerReplica;
 import org.apache.hadoop.hdds.utils.db.Table;
 
 /**
@@ -34,16 +37,23 @@ import org.apache.hadoop.hdds.utils.db.Table;
  */
 public interface PipelineManager extends Closeable, PipelineManagerMXBean {
 
-  Pipeline createPipeline(
-      ReplicationConfig replicationConfig
-  )
-      throws IOException;
+  Pipeline createPipeline(ReplicationConfig replicationConfig)
+      throws IOException, TimeoutException;
+
+  Pipeline createPipeline(ReplicationConfig replicationConfig,
+                          List<DatanodeDetails> excludedNodes,
+                          List<DatanodeDetails> favoredNodes)
+      throws IOException, TimeoutException;
+
 
   Pipeline createPipeline(
       ReplicationConfig replicationConfig,
       List<DatanodeDetails> nodes
   );
 
+  Pipeline createPipelineForRead(
+      ReplicationConfig replicationConfig, Set<ContainerReplica> replicas);
+
   Pipeline getPipeline(PipelineID pipelineID) throws PipelineNotFoundException;
 
   boolean containsPipeline(PipelineID pipelineID);
@@ -65,6 +75,17 @@ public interface PipelineManager extends Closeable, PipelineManagerMXBean {
       Collection<PipelineID> excludePipelines
   );
 
+  /**
+   * Returns the count of pipelines meeting the given ReplicationConfig and
+   * state.
+   * @param replicationConfig The ReplicationConfig of the pipelines to count
+   * @param state The current state of the pipelines to count
+   * @return The count of pipelines meeting the above criteria
+   */
+  int getPipelineCount(
+      ReplicationConfig replicationConfig, Pipeline.PipelineState state
+  );
+
   void addContainerToPipeline(PipelineID pipelineID, ContainerID containerID)
       throws IOException;
 
@@ -86,12 +107,14 @@ public interface PipelineManager extends Closeable, PipelineManagerMXBean {
 
   int getNumberOfContainers(PipelineID pipelineID) throws IOException;
 
-  void openPipeline(PipelineID pipelineId) throws IOException;
+  void openPipeline(PipelineID pipelineId) throws IOException, TimeoutException;
 
-  void closePipeline(Pipeline pipeline, boolean onTimeout) throws IOException;
+  void closePipeline(Pipeline pipeline, boolean onTimeout)
+      throws IOException, TimeoutException;
 
-  void scrubPipeline(ReplicationConfig replicationConfig)
-      throws IOException;
+  void closeStalePipelines(DatanodeDetails datanodeDetails);
+
+  void scrubPipelines() throws IOException, TimeoutException;
 
   void startPipelineCreator();
 
@@ -109,7 +132,8 @@ public interface PipelineManager extends Closeable, PipelineManagerMXBean {
    * @param pipelineID ID of the pipeline to activate.
    * @throws IOException in case of any Exception
    */
-  void activatePipeline(PipelineID pipelineID) throws IOException;
+  void activatePipeline(PipelineID pipelineID)
+      throws IOException, TimeoutException;
 
   /**
    * Deactivates an active pipeline.
@@ -117,7 +141,8 @@ public interface PipelineManager extends Closeable, PipelineManagerMXBean {
    * @param pipelineID ID of the pipeline to deactivate.
    * @throws IOException in case of any Exception
    */
-  void deactivatePipeline(PipelineID pipelineID) throws IOException;
+  void deactivatePipeline(PipelineID pipelineID)
+      throws IOException, TimeoutException;
 
   /**
    * Wait a pipeline to be OPEN.
@@ -130,6 +155,19 @@ public interface PipelineManager extends Closeable, PipelineManagerMXBean {
       throws IOException {
   }
 
+  /**
+   * Wait one pipeline to be OPEN among a collection pipelines.
+   * @param pipelineIDs ID collection of the pipelines to wait for
+   * @param timeout wait timeout(millisecond), if 0, use default timeout
+   * @return Pipeline the pipeline which is OPEN
+   * @throws IOException in case of any Exception, such as timeout
+   */
+  default Pipeline waitOnePipelineReady(Collection<PipelineID> pipelineIDs,
+                                    long timeout)
+          throws IOException {
+    return null;
+  }
+
   /**
    * Get SafeMode status.
    * @return boolean
@@ -153,6 +191,8 @@ public interface PipelineManager extends Closeable, PipelineManagerMXBean {
    */
   void resumePipelineCreation();
 
+  boolean isPipelineCreationFrozen();
+
   /**
    * Acquire read lock.
    */
