@@ -27,6 +27,7 @@ import java.util.stream.Collectors;
 import org.apache.hadoop.hdds.client.ReplicationConfig;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.hdds.scm.SCMCommonPlacementPolicy;
+import org.apache.hadoop.hdds.scm.container.ContainerReplica;
 import org.apache.hadoop.hdds.scm.exceptions.SCMException;
 import org.apache.hadoop.hdds.scm.node.NodeManager;
 import org.apache.hadoop.hdds.scm.node.NodeStatus;
@@ -65,24 +66,34 @@ public abstract class PipelineProvider<REPLICATION_CONFIG
   protected abstract Pipeline create(REPLICATION_CONFIG replicationConfig)
       throws IOException;
 
+  protected abstract Pipeline create(REPLICATION_CONFIG replicationConfig,
+      List<DatanodeDetails> excludedNodes, List<DatanodeDetails> favoredNodes)
+      throws IOException;
+
   protected abstract Pipeline create(
       REPLICATION_CONFIG replicationConfig,
       List<DatanodeDetails> nodes
   );
 
+  protected abstract Pipeline createForRead(
+      REPLICATION_CONFIG replicationConfig,
+      Set<ContainerReplica> replicas
+  );
+
   protected abstract void close(Pipeline pipeline) throws IOException;
 
   protected abstract void shutdown();
 
   List<DatanodeDetails> pickNodesNotUsed(REPLICATION_CONFIG replicationConfig,
       long metadataSizeRequired, long dataSizeRequired) throws SCMException {
-    List<DatanodeDetails> healthyDNs = pickNodesNotUsed(replicationConfig);
+    int nodesRequired = replicationConfig.getRequiredNodes();
+    List<DatanodeDetails> healthyDNs = pickAllNodesNotUsed(replicationConfig);
     List<DatanodeDetails> healthyDNsWithSpace = healthyDNs.stream()
         .filter(dn -> SCMCommonPlacementPolicy
             .hasEnoughSpace(dn, metadataSizeRequired, dataSizeRequired))
+        .limit(nodesRequired)
         .collect(Collectors.toList());
 
-    int nodesRequired = replicationConfig.getRequiredNodes();
     if (healthyDNsWithSpace.size() < nodesRequired) {
       String msg = String.format("Unable to find enough nodes that meet the " +
               "space requirement of %d bytes for metadata and %d bytes for " +
@@ -99,6 +110,13 @@ public abstract class PipelineProvider<REPLICATION_CONFIG
 
   List<DatanodeDetails> pickNodesNotUsed(REPLICATION_CONFIG replicationConfig)
       throws SCMException {
+    return pickAllNodesNotUsed(replicationConfig).stream()
+        .limit(replicationConfig.getRequiredNodes())
+        .collect(Collectors.toList());
+  }
+
+  List<DatanodeDetails> pickAllNodesNotUsed(
+      REPLICATION_CONFIG replicationConfig) throws SCMException {
     Set<DatanodeDetails> dnsUsed = new HashSet<>();
     stateManager.getPipelines(replicationConfig).stream().filter(
         p -> p.getPipelineState().equals(Pipeline.PipelineState.OPEN) ||
@@ -111,7 +129,6 @@ public abstract class PipelineProvider<REPLICATION_CONFIG
         .getNodes(NodeStatus.inServiceHealthy())
         .parallelStream()
         .filter(dn -> !dnsUsed.contains(dn))
-        .limit(replicationConfig.getRequiredNodes())
         .collect(Collectors.toList());
     if (dns.size() < replicationConfig.getRequiredNodes()) {
       String e = String
