@@ -27,6 +27,7 @@ import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMGetCer
 import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMGetCrlsRequestProto;
 import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMGetCrlsResponseProto;
 import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMGetDataNodeCertRequestProto;
+import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMGetCertRequestProto;
 import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMGetLatestCrlIdRequestProto;
 import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMGetLatestCrlIdResponseProto;
 import org.apache.hadoop.hdds.protocol.proto.SCMSecurityProtocolProtos.SCMGetOMCertRequestProto;
@@ -149,6 +150,11 @@ public class SCMSecurityProtocolServerSideTranslatorPB
             .setRevokeCertificatesResponseProto(revokeCertificates(
                 request.getRevokeCertificatesRequest()))
             .build();
+      case GetCert:
+        return scmSecurityResponse.setGetCertResponseProto(
+            getCertificate(request.getGetCertRequest()))
+            .build();
+
       default:
         throw new IllegalArgumentException(
             "Unknown request type: " + request.getCmdType());
@@ -211,6 +217,28 @@ public class SCMSecurityProtocolServerSideTranslatorPB
 
   }
 
+  /**
+   * Get SCM signed certificate.
+   *
+   * @param request
+   * @return SCMGetCertResponseProto.
+   */
+  public SCMGetCertResponseProto getCertificate(
+      SCMGetCertRequestProto request) throws IOException {
+    String certificate = impl
+        .getCertificate(request.getNodeDetails(),
+            request.getCSR());
+    SCMGetCertResponseProto.Builder builder =
+        SCMGetCertResponseProto
+            .newBuilder()
+            .setResponseCode(ResponseCode.success)
+            .setX509Certificate(certificate)
+            .setX509CACertificate(impl.getCACertificate());
+    setRootCAIfNeeded(builder);
+
+    return builder.build();
+  }
+
   /**
    * Get signed certificate for SCM.
    *
