@@ -27,12 +27,14 @@ import org.apache.hadoop.hdds.scm.ha.SCMHAManager;
 import org.apache.hadoop.hdds.scm.net.NetworkTopology;
 import org.apache.hadoop.hdds.scm.pipeline.WritableContainerFactory;
 import org.apache.hadoop.hdds.scm.safemode.SCMSafeModeManager;
-import org.apache.hadoop.hdds.scm.container.ReplicationManager;
+import org.apache.hadoop.hdds.scm.container.replication.ReplicationManager;
 import org.apache.hadoop.hdds.scm.metadata.SCMMetadataStore;
 import org.apache.hadoop.hdds.scm.node.NodeManager;
 import org.apache.hadoop.hdds.scm.pipeline.PipelineManager;
+import org.apache.hadoop.hdds.scm.server.upgrade.SCMUpgradeFinalizationContext;
 import org.apache.hadoop.hdds.security.x509.certificate.authority
     .CertificateServer;
+import org.apache.hadoop.ozone.upgrade.UpgradeFinalizationExecutor;
 
 /**
  * This class acts as an SCM builder Class. This class is important for us
@@ -75,6 +77,8 @@ public final class SCMConfigurator {
   private SCMHAManager scmHAManager;
   private SCMContext scmContext;
   private WritableContainerFactory writableContainerFactory;
+  private UpgradeFinalizationExecutor<SCMUpgradeFinalizationContext>
+      finalizationExecutor;
 
   /**
    * Allows user to specify a version of Node manager to use with this SCM.
@@ -183,6 +187,15 @@ public final class SCMConfigurator {
     this.writableContainerFactory = writableContainerFactory;
   }
 
+  /**
+   * Allows user to set the executor for upgrade finalization.
+   * @param executor - Finalization executor to use.
+   */
+  public void setUpgradeFinalizationExecutor(
+      UpgradeFinalizationExecutor<SCMUpgradeFinalizationContext> executor) {
+    this.finalizationExecutor = executor;
+  }
+
   /**
    * Gets SCM Node Manager.
    * @return Node Manager.
@@ -279,4 +292,12 @@ public final class SCMConfigurator {
     return writableContainerFactory;
   }
 
+  /**
+   * Get the upgrade finalization executor.
+   * @return UpgradeFinalizationExecutor.
+   */
+  public UpgradeFinalizationExecutor<SCMUpgradeFinalizationContext>
+      getUpgradeFinalizationExecutor() {
+    return finalizationExecutor;
+  }
 }
