@@ -25,13 +25,13 @@ import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.hdds.protocol.proto
     .StorageContainerDatanodeProtocolProtos.ContainerReplicaProto;
 import org.apache.hadoop.hdds.scm.ScmConfigKeys;
-import org.apache.hadoop.hdds.scm.TestUtils;
+import org.apache.hadoop.hdds.scm.HddsTestUtils;
 import org.apache.hadoop.hdds.scm.container.ContainerID;
 import org.apache.hadoop.hdds.scm.container.ContainerManager;
 import org.apache.hadoop.hdds.scm.container.ContainerReplica;
 import org.apache.hadoop.hdds.scm.container.ContainerInfo;
 import org.apache.hadoop.hdds.scm.ha.SCMHADBTransactionBuffer;
-import org.apache.hadoop.hdds.scm.ha.MockSCMHADBTransactionBuffer;
+import org.apache.hadoop.hdds.scm.ha.SCMHADBTransactionBufferStub;
 import org.apache.hadoop.hdds.scm.pipeline.Pipeline;
 import org.apache.hadoop.hdds.scm.pipeline.PipelineID;
 import org.apache.hadoop.hdds.scm.server.StorageContainerManager;
@@ -46,10 +46,10 @@ import org.apache.hadoop.hdds.protocol.proto
 import org.apache.hadoop.hdds.utils.db.Table;
 import org.apache.hadoop.hdds.utils.db.TableIterator;
 import org.apache.ozone.test.GenericTestUtils;
-import org.junit.After;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Test;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
 import org.mockito.Mockito;
 
 import java.io.File;
@@ -65,10 +65,10 @@ import java.util.Optional;
 import java.util.Random;
 import java.util.Set;
 import java.util.UUID;
+import java.util.concurrent.TimeoutException;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.stream.Collectors;
 
-import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationFactor.THREE;
 import static org.apache.hadoop.hdds.scm.ScmConfigKeys
     .OZONE_SCM_BLOCK_DELETION_MAX_RETRY;
 import static org.mockito.Matchers.anyObject;
@@ -92,8 +92,10 @@ public class TestDeletedBlockLog {
   private Map<Long, ContainerInfo> containers = new HashMap<>();
   private Map<Long, Set<ContainerReplica>> replicas = new HashMap<>();
   private ScmBlockDeletingServiceMetrics metrics;
+  private static final int THREE = ReplicationFactor.THREE_VALUE;
+  private static final int ONE = ReplicationFactor.ONE_VALUE;
 
-  @Before
+  @BeforeEach
   public void setup() throws Exception {
     testDir = GenericTestUtils.getTestDir(
         TestDeletedBlockLog.class.getSimpleName());
@@ -101,11 +103,11 @@ public class TestDeletedBlockLog {
     conf.setBoolean(ScmConfigKeys.OZONE_SCM_HA_ENABLE_KEY, true);
     conf.setInt(OZONE_SCM_BLOCK_DELETION_MAX_RETRY, 20);
     conf.set(HddsConfigKeys.OZONE_METADATA_DIRS, testDir.getAbsolutePath());
-    scm = TestUtils.getScm(conf);
+    scm = HddsTestUtils.getScm(conf);
     containerManager = Mockito.mock(ContainerManager.class);
     containerTable = scm.getScmMetadataStore().getContainerTable();
     scmHADBTransactionBuffer =
-        new MockSCMHADBTransactionBuffer(scm.getScmMetadataStore().getStore());
+        new SCMHADBTransactionBufferStub(scm.getScmMetadataStore().getStore());
     metrics = Mockito.mock(ScmBlockDeletingServiceMetrics.class);
     deletedBlockLog = new DeletedBlockLogImpl(conf,
         containerManager,
@@ -148,7 +150,7 @@ public class TestDeletedBlockLog {
       for (Map.Entry<ContainerID, Long> e : map.entrySet()) {
         ContainerInfo info = containers.get(e.getKey().getId());
         try {
-          Assert.assertTrue(e.getValue() > info.getDeleteTransactionId());
+          Assertions.assertTrue(e.getValue() > info.getDeleteTransactionId());
         } catch (AssertionError err) {
           throw new Exception("New TxnId " + e.getValue() + " < " + info
               .getDeleteTransactionId());
@@ -164,7 +166,8 @@ public class TestDeletedBlockLog {
     final ContainerInfo container =
         new ContainerInfo.Builder()
             .setContainerID(cid)
-            .setReplicationConfig(new RatisReplicationConfig(THREE))
+            .setReplicationConfig(RatisReplicationConfig.getInstance(
+                ReplicationFactor.THREE))
             .setState(HddsProtos.LifeCycleState.CLOSED)
             .setOwner("TestDeletedBlockLog")
             .setPipelineID(PipelineID.randomId())
@@ -181,7 +184,7 @@ public class TestDeletedBlockLog {
     replicas.put(cid, replicaSet);
   }
 
-  @After
+  @AfterEach
   public void tearDown() throws Exception {
     deletedBlockLog.close();
     scm.stop();
@@ -209,16 +212,26 @@ public class TestDeletedBlockLog {
 
   private void addTransactions(Map<Long, List<Long>> containerBlocksMap,
       boolean shouldFlush)
-      throws IOException {
+      throws IOException, TimeoutException {
     deletedBlockLog.addTransactions(containerBlocksMap);
     if (shouldFlush) {
       scmHADBTransactionBuffer.flush();
     }
   }
 
-  private void incrementCount(List<Long> txIDs) throws IOException {
+  private void incrementCount(List<Long> txIDs)
+      throws IOException, TimeoutException {
     deletedBlockLog.incrementCount(txIDs);
     scmHADBTransactionBuffer.flush();
+    // mock scmHADBTransactionBuffer does not flush deletedBlockLog
+    deletedBlockLog.onFlush();
+  }
+
+  private void resetCount(List<Long> txIDs)
+      throws IOException, TimeoutException {
+    deletedBlockLog.resetCount(txIDs);
+    scmHADBTransactionBuffer.flush();
+    deletedBlockLog.onFlush();
   }
 
   private void commitTransactions(
@@ -254,6 +267,15 @@ public class TestDeletedBlockLog {
         .collect(Collectors.toList()));
   }
 
+  private void commitTransactions(DatanodeDeletedBlockTransactions
+      transactions) {
+    transactions.getDatanodeTransactionMap().forEach((uuid,
+        deletedBlocksTransactions) -> deletedBlockLog
+        .commitTransactions(deletedBlocksTransactions.stream()
+            .map(this::createDeleteBlockTransactionResult)
+            .collect(Collectors.toList()), uuid));
+  }
+
   private DeleteBlockTransactionResult createDeleteBlockTransactionResult(
       DeletedBlocksTransaction transaction) {
     return DeleteBlockTransactionResult.newBuilder()
@@ -261,8 +283,12 @@ public class TestDeletedBlockLog {
         .setTxID(transaction.getTxID()).build();
   }
 
+  private List<DeletedBlocksTransaction> getAllTransactions() throws Exception {
+    return getTransactions(Integer.MAX_VALUE);
+  }
+
   private List<DeletedBlocksTransaction> getTransactions(
-      int maximumAllowedBlocksNum) throws IOException {
+      int maximumAllowedBlocksNum) throws IOException, TimeoutException {
     DatanodeDeletedBlockTransactions transactions =
         deletedBlockLog.getTransactions(maximumAllowedBlocksNum);
     List<DeletedBlocksTransaction> txns = new LinkedList<>();
@@ -271,31 +297,31 @@ public class TestDeletedBlockLog {
           transactions.getDatanodeTransactionMap().get(dn.getUuid()))
           .orElseGet(LinkedList::new));
     }
-    return txns.stream().distinct().collect(Collectors.toList());
+    return txns;
   }
 
   @Test
   public void testContainerManagerTransactionId() throws Exception {
     // Initially all containers should have deleteTransactionId as 0
     for (ContainerInfo containerInfo : containerManager.getContainers()) {
-      Assert.assertEquals(0, containerInfo.getDeleteTransactionId());
+      Assertions.assertEquals(0, containerInfo.getDeleteTransactionId());
     }
 
     // Create 30 TXs
     addTransactions(generateData(30), false);
     // Since transactions are not yet flushed deleteTransactionId should be
     // 0 for all containers
-    Assert.assertEquals(0, getTransactions(1000).size());
+    Assertions.assertEquals(0, getAllTransactions().size());
     for (ContainerInfo containerInfo : containerManager.getContainers()) {
-      Assert.assertEquals(0, containerInfo.getDeleteTransactionId());
+      Assertions.assertEquals(0, containerInfo.getDeleteTransactionId());
     }
 
     scmHADBTransactionBuffer.flush();
     // After flush there should be 30 transactions in deleteTable
     // All containers should have positive deleteTransactionId
-    Assert.assertEquals(30, getTransactions(1000).size());
+    Assertions.assertEquals(30 * THREE, getAllTransactions().size());
     for (ContainerInfo containerInfo : containerManager.getContainers()) {
-      Assert.assertTrue(containerInfo.getDeleteTransactionId() > 0);
+      Assertions.assertTrue(containerInfo.getDeleteTransactionId() > 0);
     }
   }
 
@@ -307,10 +333,47 @@ public class TestDeletedBlockLog {
     addTransactions(generateData(30), true);
 
     // This will return all TXs, total num 30.
-    List<DeletedBlocksTransaction> blocks =
-        getTransactions(40 * BLOCKS_PER_TXN);
+    List<DeletedBlocksTransaction> blocks = getAllTransactions();
     List<Long> txIDs = blocks.stream().map(DeletedBlocksTransaction::getTxID)
-        .collect(Collectors.toList());
+        .distinct().collect(Collectors.toList());
+    Assertions.assertEquals(30, txIDs.size());
+
+    for (DeletedBlocksTransaction block : blocks) {
+      Assertions.assertEquals(0, block.getCount());
+    }
+
+    for (int i = 0; i < maxRetry; i++) {
+      incrementCount(txIDs);
+    }
+    blocks = getAllTransactions();
+    for (DeletedBlocksTransaction block : blocks) {
+      Assertions.assertEquals(maxRetry, block.getCount());
+    }
+
+    // Increment another time so it exceed the maxRetry.
+    // On this call, count will be set to -1 which means TX eventually fails.
+    incrementCount(txIDs);
+    blocks = getAllTransactions();
+    for (DeletedBlocksTransaction block : blocks) {
+      Assertions.assertEquals(-1, block.getCount());
+    }
+
+    // If all TXs are failed, getTransactions call will always return nothing.
+    blocks = getAllTransactions();
+    Assertions.assertEquals(0, blocks.size());
+  }
+
+  @Test
+  public void testResetCount() throws Exception {
+    int maxRetry = conf.getInt(OZONE_SCM_BLOCK_DELETION_MAX_RETRY, 20);
+
+    // Create 30 TXs in the log.
+    addTransactions(generateData(30), true);
+
+    // This will return all TXs, total num 30.
+    List<DeletedBlocksTransaction> blocks = getAllTransactions();
+    List<Long> txIDs = blocks.stream().map(DeletedBlocksTransaction::getTxID)
+        .distinct().collect(Collectors.toList());
 
     for (int i = 0; i < maxRetry; i++) {
       incrementCount(txIDs);
@@ -319,40 +382,84 @@ public class TestDeletedBlockLog {
     // Increment another time so it exceed the maxRetry.
     // On this call, count will be set to -1 which means TX eventually fails.
     incrementCount(txIDs);
-    blocks = getTransactions(40 * BLOCKS_PER_TXN);
+    blocks = getAllTransactions();
     for (DeletedBlocksTransaction block : blocks) {
-      Assert.assertEquals(-1, block.getCount());
+      Assertions.assertEquals(-1, block.getCount());
     }
 
     // If all TXs are failed, getTransactions call will always return nothing.
-    blocks = getTransactions(40 * BLOCKS_PER_TXN);
-    Assert.assertEquals(blocks.size(), 0);
+    blocks = getAllTransactions();
+    Assertions.assertEquals(0, blocks.size());
+
+    // Reset the retry count, these transactions should be accessible.
+    resetCount(txIDs);
+    blocks = getAllTransactions();
+    for (DeletedBlocksTransaction block : blocks) {
+      Assertions.assertEquals(0, block.getCount());
+    }
+
+    // Increment for the reset transactions.
+    incrementCount(txIDs);
+    blocks = getAllTransactions();
+    for (DeletedBlocksTransaction block : blocks) {
+      Assertions.assertEquals(1, block.getCount());
+    }
+
+    Assertions.assertEquals(30 * THREE, blocks.size());
   }
 
   @Test
   public void testCommitTransactions() throws Exception {
     addTransactions(generateData(50), true);
     List<DeletedBlocksTransaction> blocks =
-        getTransactions(20 * BLOCKS_PER_TXN);
+        getTransactions(20 * BLOCKS_PER_TXN * THREE);
     // Add an invalid txn.
     blocks.add(
         DeletedBlocksTransaction.newBuilder().setContainerID(1).setTxID(70)
             .setCount(0).addLocalID(0).build());
     commitTransactions(blocks);
-    blocks.remove(blocks.size() - 1);
 
-    blocks = getTransactions(50 * BLOCKS_PER_TXN);
-    Assert.assertEquals(30, blocks.size());
+    blocks = getTransactions(50 * BLOCKS_PER_TXN * THREE);
+    Assertions.assertEquals(30 * THREE, blocks.size());
     commitTransactions(blocks, dnList.get(1), dnList.get(2),
         DatanodeDetails.newBuilder().setUuid(UUID.randomUUID())
             .build());
 
-    blocks = getTransactions(50 * BLOCKS_PER_TXN);
-    Assert.assertEquals(30, blocks.size());
+    blocks = getTransactions(50 * BLOCKS_PER_TXN * THREE);
+    // only uncommitted dn have transactions
+    Assertions.assertEquals(30, blocks.size());
     commitTransactions(blocks, dnList.get(0));
 
-    blocks = getTransactions(50 * BLOCKS_PER_TXN);
-    Assert.assertEquals(0, blocks.size());
+    blocks = getTransactions(50 * BLOCKS_PER_TXN * THREE);
+    Assertions.assertEquals(0, blocks.size());
+  }
+
+  @Test
+  public void testInadequateReplicaCommit() throws Exception {
+    Map<Long, List<Long>> deletedBlocks = generateData(50);
+    addTransactions(deletedBlocks, true);
+    long containerID;
+    // let the first 30 container only consisting of only two unhealthy replicas
+    int count = 30;
+    for (Map.Entry<Long, List<Long>> entry :deletedBlocks.entrySet()) {
+      if (count <= 0) {
+        break;
+      }
+      containerID = entry.getKey();
+      mockInadequateReplicaUnhealthyContainerInfo(containerID);
+      count -= 1;
+    }
+    // getTransactions will get existing container replicas then add transaction
+    // to DN.
+    // For the first 30 txn, deletedBlockLog only has the txn from dn1 and dn2
+    // For the rest txn, txn will be got from all dns.
+    // Committed txn will be: 1-40. 1-40. 31-40
+    commitTransactions(deletedBlockLog.getTransactions(
+        30 * BLOCKS_PER_TXN * THREE));
+
+    // The rest txn shall be: 41-50. 41-50. 41-50
+    List<DeletedBlocksTransaction> blocks = getAllTransactions();
+    Assertions.assertEquals(30, blocks.size());
   }
 
   @Test
@@ -368,7 +475,7 @@ public class TestDeletedBlockLog {
         addTransactions(generateData(10), true);
         added += 10;
       } else if (state == 1) {
-        blocks = getTransactions(20);
+        blocks = getTransactions(20 * BLOCKS_PER_TXN * THREE);
         txIDs = new ArrayList<>();
         for (DeletedBlocksTransaction block : blocks) {
           txIDs.add(block.getTxID());
@@ -376,7 +483,7 @@ public class TestDeletedBlockLog {
         incrementCount(txIDs);
       } else if (state == 2) {
         commitTransactions(blocks);
-        committed += blocks.size();
+        committed += blocks.size() / THREE;
         blocks = new ArrayList<>();
       } else {
         // verify the number of added and committed.
@@ -385,11 +492,11 @@ public class TestDeletedBlockLog {
             scm.getScmMetadataStore().getDeletedBlocksTXTable().iterator()) {
           AtomicInteger count = new AtomicInteger();
           iter.forEachRemaining((keyValue) -> count.incrementAndGet());
-          Assert.assertEquals(added, count.get() + committed);
+          Assertions.assertEquals(added, count.get() + committed);
         }
       }
     }
-    blocks = getTransactions(1000);
+    blocks = getAllTransactions();
     commitTransactions(blocks);
   }
 
@@ -408,11 +515,11 @@ public class TestDeletedBlockLog {
         scm.getSequenceIdGen(),
         metrics);
     List<DeletedBlocksTransaction> blocks =
-        getTransactions(BLOCKS_PER_TXN * 10);
-    Assert.assertEquals(10, blocks.size());
+        getTransactions(10 * BLOCKS_PER_TXN * THREE);
+    Assertions.assertEquals(10 * THREE, blocks.size());
     commitTransactions(blocks);
-    blocks = getTransactions(BLOCKS_PER_TXN * 40);
-    Assert.assertEquals(40, blocks.size());
+    blocks = getTransactions(40 * BLOCKS_PER_TXN * THREE);
+    Assertions.assertEquals(40 * THREE, blocks.size());
     commitTransactions(blocks);
 
     // close db and reopen it again to make sure
@@ -426,13 +533,14 @@ public class TestDeletedBlockLog {
         scm.getScmContext(),
         scm.getSequenceIdGen(),
         metrics);
-    blocks = getTransactions(BLOCKS_PER_TXN * 40);
-    Assert.assertEquals(0, blocks.size());
-    //Assert.assertEquals((long)deletedBlockLog.getCurrentTXID(), 50L);
+    blocks = getTransactions(40 * BLOCKS_PER_TXN * THREE);
+    Assertions.assertEquals(0, blocks.size());
+    //Assertions.assertEquals((long)deletedBlockLog.getCurrentTXID(), 50L);
   }
 
   @Test
-  public void testDeletedBlockTransactions() throws IOException {
+  public void testDeletedBlockTransactions()
+      throws IOException, TimeoutException {
     int txNum = 10;
     List<DeletedBlocksTransaction> blocks;
     DatanodeDetails dnId1 = dnList.get(0), dnId2 = dnList.get(1);
@@ -446,43 +554,38 @@ public class TestDeletedBlockLog {
     for (Map.Entry<Long, List<Long>> entry :deletedBlocks.entrySet()) {
       count++;
       containerID = entry.getKey();
-
+      // let the container replication factor to be ONE
       if (count % 2 == 0) {
-        mockContainerInfo(containerID, dnId1);
+        mockStandAloneContainerInfo(containerID, dnId1);
       } else {
-        mockContainerInfo(containerID, dnId2);
+        mockStandAloneContainerInfo(containerID, dnId2);
       }
     }
 
     // fetch and delete 1 less txn Id
-    commitTransactions(getTransactions((txNum - 1) * BLOCKS_PER_TXN));
+    commitTransactions(getTransactions((txNum - 1) * BLOCKS_PER_TXN * ONE));
 
-    blocks = getTransactions(txNum * BLOCKS_PER_TXN);
+    blocks = getTransactions(txNum * BLOCKS_PER_TXN * ONE);
     // There should be one txn remaining
-    Assert.assertEquals(1, blocks.size());
+    Assertions.assertEquals(1, blocks.size());
 
     // add two transactions for same container
     containerID = blocks.get(0).getContainerID();
-    DeletedBlocksTransaction.Builder builder =
-        DeletedBlocksTransaction.newBuilder();
-    builder.setTxID(11);
-    builder.setContainerID(containerID);
-    builder.setCount(0);
     Map<Long, List<Long>> deletedBlocksMap = new HashMap<>();
     deletedBlocksMap.put(containerID, new LinkedList<>());
     addTransactions(deletedBlocksMap, true);
 
     // get should return two transactions for the same container
-    blocks = getTransactions(txNum);
-    Assert.assertEquals(2, blocks.size());
+    blocks = getTransactions(txNum * BLOCKS_PER_TXN * ONE);
+    Assertions.assertEquals(2, blocks.size());
   }
 
-  private void mockContainerInfo(long containerID, DatanodeDetails dd)
+  private void mockStandAloneContainerInfo(long containerID, DatanodeDetails dd)
       throws IOException {
     List<DatanodeDetails> dns = Collections.singletonList(dd);
     Pipeline pipeline = Pipeline.newBuilder()
         .setReplicationConfig(
-            new StandaloneReplicationConfig(ReplicationFactor.ONE))
+            StandaloneReplicationConfig.getInstance(ReplicationFactor.ONE))
         .setState(Pipeline.PipelineState.OPEN)
             .setId(PipelineID.randomId())
             .setNodes(dns)
@@ -508,4 +611,36 @@ public class TestDeletedBlockLog {
         ContainerID.valueOf(containerID)))
         .thenReturn(replicaSet);
   }
+
+  private void mockInadequateReplicaUnhealthyContainerInfo(long containerID)
+      throws IOException {
+    List<DatanodeDetails> dns = dnList.subList(0, 2);
+    Pipeline pipeline = Pipeline.newBuilder()
+        .setReplicationConfig(
+            RatisReplicationConfig.getInstance(ReplicationFactor.THREE))
+        .setState(Pipeline.PipelineState.OPEN)
+        .setId(PipelineID.randomId())
+        .setNodes(dnList)
+        .build();
+
+    ContainerInfo.Builder builder = new ContainerInfo.Builder();
+    builder.setContainerID(containerID)
+        .setPipelineID(pipeline.getId())
+        .setReplicationConfig(pipeline.getReplicationConfig());
+
+    ContainerInfo containerInfo = builder.build();
+    Mockito.doReturn(containerInfo).when(containerManager)
+        .getContainer(ContainerID.valueOf(containerID));
+
+    final Set<ContainerReplica> replicaSet = dns.stream()
+        .map(datanodeDetails -> ContainerReplica.newBuilder()
+            .setContainerID(containerInfo.containerID())
+            .setContainerState(ContainerReplicaProto.State.CLOSED)
+            .setDatanodeDetails(datanodeDetails)
+            .build())
+        .collect(Collectors.toSet());
+    when(containerManager.getContainerReplicas(
+        ContainerID.valueOf(containerID)))
+        .thenReturn(replicaSet);
+  }
 }
