@@ -18,15 +18,21 @@
 package org.apache.hadoop.hdds.scm.container;
 
 import java.io.File;
+import java.io.IOException;
 import java.util.UUID;
+import java.util.concurrent.TimeoutException;
 
 import org.apache.hadoop.fs.FileUtil;
 import org.apache.hadoop.hdds.HddsConfigKeys;
+import org.apache.hadoop.hdds.client.ECReplicationConfig;
 import org.apache.hadoop.hdds.client.RatisReplicationConfig;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
+import org.apache.hadoop.hdds.protocol.DatanodeDetails;
+import org.apache.hadoop.hdds.protocol.MockDatanodeDetails;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationFactor;
-import org.apache.hadoop.hdds.scm.ha.MockSCMHAManager;
+import org.apache.hadoop.hdds.scm.container.replication.ContainerReplicaPendingOps;
+import org.apache.hadoop.hdds.scm.ha.SCMHAManagerStub;
 import org.apache.hadoop.hdds.scm.ha.SCMHAManager;
 import org.apache.hadoop.hdds.scm.ha.SequenceIdGenerator;
 import org.apache.hadoop.hdds.scm.metadata.SCMDBDefinition;
@@ -37,11 +43,13 @@ import org.apache.hadoop.hdds.utils.db.DBStore;
 import org.apache.hadoop.hdds.utils.db.DBStoreBuilder;
 import org.apache.hadoop.ozone.container.common.SCMTestUtils;
 import org.apache.ozone.test.GenericTestUtils;
-import org.junit.After;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Test;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
+import org.mockito.Mockito;
 
+import static org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.ContainerReplicaProto.State.OPEN;
 
 
 /**
@@ -55,8 +63,9 @@ public class TestContainerManagerImpl {
   private SCMHAManager scmhaManager;
   private SequenceIdGenerator sequenceIdGen;
   private NodeManager nodeManager;
+  private ContainerReplicaPendingOps pendingOpsMock;
 
-  @Before
+  @BeforeEach
   public void setUp() throws Exception {
     final OzoneConfiguration conf = SCMTestUtils.getConf();
     testDir = GenericTestUtils.getTestDir(
@@ -64,22 +73,23 @@ public class TestContainerManagerImpl {
     conf.set(HddsConfigKeys.OZONE_METADATA_DIRS, testDir.getAbsolutePath());
     dbStore = DBStoreBuilder.createDBStore(
         conf, new SCMDBDefinition());
-    scmhaManager = MockSCMHAManager.getInstance(true);
+    scmhaManager = SCMHAManagerStub.getInstance(true);
     nodeManager = new MockNodeManager(true, 10);
     sequenceIdGen = new SequenceIdGenerator(
         conf, scmhaManager, SCMDBDefinition.SEQUENCE_ID.getTable(dbStore));
     final PipelineManager pipelineManager =
         new MockPipelineManager(dbStore, scmhaManager, nodeManager);
-    pipelineManager.createPipeline(new RatisReplicationConfig(
+    pipelineManager.createPipeline(RatisReplicationConfig.getInstance(
         ReplicationFactor.THREE));
+    pendingOpsMock = Mockito.mock(ContainerReplicaPendingOps.class);
     containerManager = new ContainerManagerImpl(conf,
         scmhaManager, sequenceIdGen, pipelineManager,
-        SCMDBDefinition.CONTAINERS.getTable(dbStore));
+        SCMDBDefinition.CONTAINERS.getTable(dbStore), pendingOpsMock);
   }
 
-  @After
+  @AfterEach
   public void cleanup() throws Exception {
-    if(containerManager != null) {
+    if (containerManager != null) {
       containerManager.close();
     }
 
@@ -91,68 +101,134 @@ public class TestContainerManagerImpl {
   }
 
   @Test
-  public void testAllocateContainer() throws Exception {
-    Assert.assertTrue(
+  void testAllocateContainer() throws Exception {
+    Assertions.assertTrue(
         containerManager.getContainers().isEmpty());
     final ContainerInfo container = containerManager.allocateContainer(
-        new RatisReplicationConfig(
+        RatisReplicationConfig.getInstance(
             ReplicationFactor.THREE), "admin");
-    Assert.assertEquals(1, containerManager.getContainers().size());
-    Assert.assertNotNull(containerManager.getContainer(
+    Assertions.assertEquals(1, containerManager.getContainers().size());
+    Assertions.assertNotNull(containerManager.getContainer(
         container.containerID()));
   }
 
   @Test
-  public void testUpdateContainerState() throws Exception {
+  void testUpdateContainerState() throws Exception {
     final ContainerInfo container = containerManager.allocateContainer(
-        new RatisReplicationConfig(
+        RatisReplicationConfig.getInstance(
             ReplicationFactor.THREE), "admin");
     final ContainerID cid = container.containerID();
-    Assert.assertEquals(HddsProtos.LifeCycleState.OPEN,
+    Assertions.assertEquals(HddsProtos.LifeCycleState.OPEN,
         containerManager.getContainer(cid).getState());
     containerManager.updateContainerState(cid,
         HddsProtos.LifeCycleEvent.FINALIZE);
-    Assert.assertEquals(HddsProtos.LifeCycleState.CLOSING,
+    Assertions.assertEquals(HddsProtos.LifeCycleState.CLOSING,
         containerManager.getContainer(cid).getState());
     containerManager.updateContainerState(cid,
         HddsProtos.LifeCycleEvent.QUASI_CLOSE);
-    Assert.assertEquals(HddsProtos.LifeCycleState.QUASI_CLOSED,
+    Assertions.assertEquals(HddsProtos.LifeCycleState.QUASI_CLOSED,
         containerManager.getContainer(cid).getState());
     containerManager.updateContainerState(cid,
         HddsProtos.LifeCycleEvent.FORCE_CLOSE);
-    Assert.assertEquals(HddsProtos.LifeCycleState.CLOSED,
+    Assertions.assertEquals(HddsProtos.LifeCycleState.CLOSED,
         containerManager.getContainer(cid).getState());
   }
 
   @Test
-  public void testGetContainers() throws Exception{
-    Assert.assertTrue(
+  void testGetContainers() throws Exception {
+    Assertions.assertTrue(
         containerManager.getContainers().isEmpty());
 
     ContainerID[] cidArray = new ContainerID[10];
-    for(int i = 0; i < 10; i++){
+    for (int i = 0; i < 10; i++) {
       ContainerInfo container = containerManager.allocateContainer(
-          new RatisReplicationConfig(
+          RatisReplicationConfig.getInstance(
               ReplicationFactor.THREE), "admin");
       cidArray[i] = container.containerID();
     }
 
-    Assert.assertEquals(10,
+    Assertions.assertEquals(10,
         containerManager.getContainers(cidArray[0], 10).size());
-    Assert.assertEquals(10,
+    Assertions.assertEquals(10,
         containerManager.getContainers(cidArray[0], 100).size());
 
     containerManager.updateContainerState(cidArray[0],
         HddsProtos.LifeCycleEvent.FINALIZE);
-    Assert.assertEquals(9,
+    Assertions.assertEquals(9,
         containerManager.getContainers(HddsProtos.LifeCycleState.OPEN).size());
-    Assert.assertEquals(1, containerManager
+    Assertions.assertEquals(1, containerManager
         .getContainers(HddsProtos.LifeCycleState.CLOSING).size());
     containerManager.updateContainerState(cidArray[1],
         HddsProtos.LifeCycleEvent.FINALIZE);
-    Assert.assertEquals(8,
+    Assertions.assertEquals(8,
         containerManager.getContainers(HddsProtos.LifeCycleState.OPEN).size());
-    Assert.assertEquals(2, containerManager
+    Assertions.assertEquals(2, containerManager
         .getContainers(HddsProtos.LifeCycleState.CLOSING).size());
+    containerManager.updateContainerState(cidArray[1],
+        HddsProtos.LifeCycleEvent.QUASI_CLOSE);
+    containerManager.updateContainerState(cidArray[2],
+        HddsProtos.LifeCycleEvent.FINALIZE);
+    containerManager.updateContainerState(cidArray[2],
+        HddsProtos.LifeCycleEvent.CLOSE);
+    Assertions.assertEquals(7, containerManager.
+        getContainerStateCount(HddsProtos.LifeCycleState.OPEN));
+    Assertions.assertEquals(1, containerManager
+        .getContainerStateCount(HddsProtos.LifeCycleState.CLOSING));
+    Assertions.assertEquals(1, containerManager
+        .getContainerStateCount(HddsProtos.LifeCycleState.QUASI_CLOSED));
+    Assertions.assertEquals(1, containerManager
+        .getContainerStateCount(HddsProtos.LifeCycleState.CLOSED));
   }
+
+  @Test
+  void testAllocateContainersWithECReplicationConfig() throws Exception {
+    final ContainerInfo admin = containerManager
+        .allocateContainer(new ECReplicationConfig(3, 2), "admin");
+    Assertions.assertEquals(1, containerManager.getContainers().size());
+    Assertions.assertNotNull(
+        containerManager.getContainer(admin.containerID()));
+  }
+
+  @Test
+  void testUpdateContainerReplicaInvokesPendingOp()
+      throws IOException, TimeoutException {
+    final ContainerInfo container = containerManager.allocateContainer(
+        RatisReplicationConfig.getInstance(
+            ReplicationFactor.THREE), "admin");
+    DatanodeDetails dn = MockDatanodeDetails.randomDatanodeDetails();
+    containerManager.updateContainerReplica(container.containerID(),
+        ContainerReplica.newBuilder()
+            .setContainerState(OPEN)
+            .setReplicaIndex(0)
+            .setContainerID(container.containerID())
+            .setDatanodeDetails(dn)
+            .setSequenceId(1)
+            .setBytesUsed(1234)
+            .setKeyCount(123)
+            .build());
+    Mockito.verify(pendingOpsMock, Mockito.times(1))
+        .completeAddReplica(container.containerID(), dn, 0);
+  }
+
+  @Test
+  void testRemoveContainerReplicaInvokesPendingOp()
+      throws IOException, TimeoutException {
+    final ContainerInfo container = containerManager.allocateContainer(
+        RatisReplicationConfig.getInstance(
+            ReplicationFactor.THREE), "admin");
+    DatanodeDetails dn = MockDatanodeDetails.randomDatanodeDetails();
+    containerManager.removeContainerReplica(container.containerID(),
+        ContainerReplica.newBuilder()
+            .setContainerState(OPEN)
+            .setReplicaIndex(0)
+            .setContainerID(container.containerID())
+            .setDatanodeDetails(dn)
+            .setSequenceId(1)
+            .setBytesUsed(1234)
+            .setKeyCount(123)
+            .build());
+    Mockito.verify(pendingOpsMock, Mockito.times(1))
+        .completeDeleteReplica(container.containerID(), dn, 0);
+  }
+
 }
