@@ -17,24 +17,38 @@
 
 package org.apache.hadoop.hdds.scm.container;
 
+import java.io.File;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Set;
+import java.util.UUID;
+import java.util.concurrent.TimeoutException;
 
+import org.apache.hadoop.fs.FileUtil;
+import org.apache.hadoop.hdds.HddsConfigKeys;
 import org.apache.hadoop.hdds.client.StandaloneReplicationConfig;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.hdds.protocol.MockDatanodeDetails;
+import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationFactor;
 import org.apache.hadoop.hdds.protocol.proto
     .StorageContainerDatanodeProtocolProtos.ContainerReplicaProto;
 
+import org.apache.hadoop.hdds.scm.ha.SCMHAManagerStub;
+import org.apache.hadoop.hdds.scm.ha.SCMHAManager;
+import org.apache.hadoop.hdds.scm.metadata.SCMDBDefinition;
 import org.apache.hadoop.hdds.scm.pipeline.Pipeline;
 import org.apache.hadoop.hdds.scm.pipeline.PipelineID;
 import org.apache.hadoop.hdds.scm.pipeline.PipelineManager;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Test;
+import org.apache.hadoop.hdds.utils.db.DBStore;
+import org.apache.hadoop.hdds.utils.db.DBStoreBuilder;
+import org.apache.hadoop.util.Time;
+import org.apache.ozone.test.GenericTestUtils;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
 import org.mockito.Mockito;
 
 import static org.mockito.Mockito.when;
@@ -45,16 +59,56 @@ import static org.mockito.Mockito.when;
 public class TestContainerStateManager {
 
   private ContainerStateManager containerStateManager;
+  private PipelineManager pipelineManager;
+  private SCMHAManager scmhaManager;
+  private File testDir;
+  private DBStore dbStore;
+  private Pipeline pipeline;
 
-  @Before
-  public void init() throws IOException {
+  @BeforeEach
+  public void init() throws IOException, TimeoutException {
     OzoneConfiguration conf = new OzoneConfiguration();
-    containerStateManager = new ContainerStateManager(conf);
+    scmhaManager = SCMHAManagerStub.getInstance(true);
+    testDir = GenericTestUtils.getTestDir(
+        TestContainerManagerImpl.class.getSimpleName() + UUID.randomUUID());
+    conf.set(HddsConfigKeys.OZONE_METADATA_DIRS, testDir.getAbsolutePath());
+    dbStore = DBStoreBuilder.createDBStore(
+        conf, new SCMDBDefinition());
+    pipelineManager = Mockito.mock(PipelineManager.class);
+    pipeline = Pipeline.newBuilder().setState(Pipeline.PipelineState.CLOSED)
+            .setId(PipelineID.randomId())
+            .setReplicationConfig(StandaloneReplicationConfig.getInstance(
+                ReplicationFactor.THREE))
+            .setNodes(new ArrayList<>()).build();
+    when(pipelineManager.createPipeline(StandaloneReplicationConfig.getInstance(
+        ReplicationFactor.THREE))).thenReturn(pipeline);
+    when(pipelineManager.containsPipeline(Mockito.any(PipelineID.class)))
+        .thenReturn(true);
+
+
+    containerStateManager = ContainerStateManagerImpl.newBuilder()
+        .setConfiguration(conf)
+        .setPipelineManager(pipelineManager)
+        .setRatisServer(scmhaManager.getRatisServer())
+        .setContainerStore(SCMDBDefinition.CONTAINERS.getTable(dbStore))
+        .setSCMDBTransactionBuffer(scmhaManager.getDBTransactionBuffer())
+        .build();
 
   }
 
+  @AfterEach
+  public void tearDown() throws Exception {
+    containerStateManager.close();
+    if (dbStore != null) {
+      dbStore.close();
+    }
+
+    FileUtil.fullyDelete(testDir);
+  }
+
   @Test
-  public void checkReplicationStateOK() throws IOException {
+  public void checkReplicationStateOK()
+      throws IOException, TimeoutException {
     //GIVEN
     ContainerInfo c1 = allocateContainer();
 
@@ -71,11 +125,12 @@ public class TestContainerStateManager {
         .getContainerReplicas(c1.containerID());
 
     //THEN
-    Assert.assertEquals(3, replicas.size());
+    Assertions.assertEquals(3, replicas.size());
   }
 
   @Test
-  public void checkReplicationStateMissingReplica() throws IOException {
+  public void checkReplicationStateMissingReplica()
+      throws IOException, TimeoutException {
     //GIVEN
 
     ContainerInfo c1 = allocateContainer();
@@ -90,12 +145,11 @@ public class TestContainerStateManager {
     Set<ContainerReplica> replicas = containerStateManager
         .getContainerReplicas(c1.containerID());
 
-    Assert.assertEquals(2, replicas.size());
-    Assert.assertEquals(3, c1.getReplicationConfig().getRequiredNodes());
+    Assertions.assertEquals(2, replicas.size());
+    Assertions.assertEquals(3, c1.getReplicationConfig().getRequiredNodes());
   }
 
-  private void addReplica(ContainerInfo cont, DatanodeDetails node)
-      throws ContainerNotFoundException {
+  private void addReplica(ContainerInfo cont, DatanodeDetails node) {
     ContainerReplica replica = ContainerReplica.newBuilder()
         .setContainerID(cont.containerID())
         .setContainerState(ContainerReplicaProto.State.CLOSED)
@@ -105,24 +159,23 @@ public class TestContainerStateManager {
         .updateContainerReplica(cont.containerID(), replica);
   }
 
-  private ContainerInfo allocateContainer() throws IOException {
+  private ContainerInfo allocateContainer()
+      throws IOException, TimeoutException {
 
-    PipelineManager pipelineManager = Mockito.mock(PipelineManager.class);
-
-    Pipeline pipeline =
-        Pipeline.newBuilder().setState(Pipeline.PipelineState.CLOSED)
-            .setId(PipelineID.randomId())
-            .setReplicationConfig(new StandaloneReplicationConfig(
-                ReplicationFactor.THREE))
-            .setNodes(new ArrayList<>()).build();
-
-    when(pipelineManager.createPipeline(new StandaloneReplicationConfig(
-        ReplicationFactor.THREE))).thenReturn(pipeline);
-
-    return containerStateManager.allocateContainer(pipelineManager,
-        new StandaloneReplicationConfig(
-            ReplicationFactor.THREE), "root");
+    final ContainerInfo containerInfo = new ContainerInfo.Builder()
+        .setState(HddsProtos.LifeCycleState.OPEN)
+        .setPipelineID(pipeline.getId())
+        .setUsedBytes(0)
+        .setNumberOfKeys(0)
+        .setStateEnterTime(Time.now())
+        .setOwner("root")
+        .setContainerID(1)
+        .setDeleteTransactionId(0)
+        .setReplicationConfig(pipeline.getReplicationConfig())
+        .build();
 
+    containerStateManager.addContainer(containerInfo.getProtobuf());
+    return containerInfo;
   }
 
 }
\ No newline at end of file
