@@ -17,7 +17,9 @@
  */
 package org.apache.hadoop.hdds.scm.container;
 
+import org.apache.hadoop.fs.FileUtil;
 import org.apache.hadoop.hdds.HddsConfigKeys;
+import org.apache.hadoop.hdds.client.ECReplicationConfig;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
@@ -27,12 +29,19 @@ import org.apache.hadoop.hdds.protocol.proto
     .StorageContainerDatanodeProtocolProtos.ContainerReplicaProto;
 import org.apache.hadoop.hdds.protocol.proto
     .StorageContainerDatanodeProtocolProtos.IncrementalContainerReportProto;
+import org.apache.hadoop.hdds.scm.HddsTestUtils;
+import org.apache.hadoop.hdds.scm.ha.SCMHAManagerStub;
 import org.apache.hadoop.hdds.scm.ha.SCMContext;
+import org.apache.hadoop.hdds.scm.ha.SCMHAManager;
+import org.apache.hadoop.hdds.scm.metadata.SCMDBDefinition;
 import org.apache.hadoop.hdds.scm.net.NetworkTopology;
 import org.apache.hadoop.hdds.scm.net.NetworkTopologyImpl;
 import org.apache.hadoop.hdds.scm.node.NodeManager;
 import org.apache.hadoop.hdds.scm.node.SCMNodeManager;
 import org.apache.hadoop.hdds.scm.node.states.NodeNotFoundException;
+import org.apache.hadoop.hdds.scm.pipeline.MockPipelineManager;
+import org.apache.hadoop.hdds.scm.pipeline.PipelineID;
+import org.apache.hadoop.hdds.scm.pipeline.PipelineManager;
 import org.apache.hadoop.hdds.scm.server.SCMDatanodeHeartbeatDispatcher.ContainerReportFromDatanode;
 import org.apache.hadoop.hdds.scm.server.SCMDatanodeHeartbeatDispatcher
     .IncrementalContainerReportFromDatanode;
@@ -40,28 +49,39 @@ import org.apache.hadoop.hdds.scm.server.SCMStorageConfig;
 import org.apache.hadoop.hdds.server.events.EventPublisher;
 import org.apache.hadoop.hdds.server.events.EventQueue;
 import org.apache.hadoop.hdds.upgrade.HDDSLayoutVersionManager;
+import org.apache.hadoop.hdds.utils.db.DBStore;
+import org.apache.hadoop.hdds.utils.db.DBStoreBuilder;
 import org.apache.hadoop.ozone.common.statemachine.InvalidStateTransitionException;
 import org.apache.ozone.test.GenericTestUtils;
-import org.junit.After;
 import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Test;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
 import org.mockito.Mockito;
 
+import java.io.File;
 import java.io.IOException;
 import java.nio.file.Path;
 import java.nio.file.Paths;
+import java.util.List;
+import java.util.Map;
 import java.util.Set;
 import java.util.UUID;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.Executors;
 import java.util.concurrent.Future;
 import java.util.concurrent.ThreadPoolExecutor;
+import java.util.concurrent.TimeoutException;
+import java.util.stream.Collectors;
+import java.util.stream.IntStream;
 
 import static org.apache.hadoop.hdds.protocol.MockDatanodeDetails.randomDatanodeDetails;
 import static org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.ContainerReplicaProto.State.CLOSED;
-import static org.apache.hadoop.hdds.scm.TestUtils.getContainer;
-import static org.apache.hadoop.hdds.scm.TestUtils.getReplicas;
+import static org.apache.hadoop.hdds.scm.HddsTestUtils.getContainer;
+import static org.apache.hadoop.hdds.scm.HddsTestUtils.getECContainer;
+import static org.apache.hadoop.hdds.scm.HddsTestUtils.getReplicas;
+import static org.apache.hadoop.hdds.scm.container.TestContainerReportHandler.getContainerReportsProto;
 import static org.apache.hadoop.hdds.upgrade.HDDSLayoutVersionManager.maxLayoutVersion;
 
 /**
@@ -75,9 +95,14 @@ public class TestIncrementalContainerReportHandler {
   private EventPublisher publisher;
   private HDDSLayoutVersionManager versionManager;
   private SCMContext scmContext = SCMContext.emptyContext();
+  private PipelineManager pipelineManager;
+  private File testDir;
+  private DBStore dbStore;
+  private SCMHAManager scmhaManager;
 
-  @Before
-  public void setup() throws IOException, InvalidStateTransitionException {
+  @BeforeEach
+  public void setup() throws IOException, InvalidStateTransitionException,
+      TimeoutException {
     final OzoneConfiguration conf = new OzoneConfiguration();
     final String path =
         GenericTestUtils.getTempPath(UUID.randomUUID().toString());
@@ -96,22 +121,40 @@ public class TestIncrementalContainerReportHandler {
     this.nodeManager =
         new SCMNodeManager(conf, storageConfig, eventQueue, clusterMap,
             scmContext, versionManager);
+    scmhaManager = SCMHAManagerStub.getInstance(true);
+    testDir = GenericTestUtils.getTestDir(
+        TestContainerManagerImpl.class.getSimpleName() + UUID.randomUUID());
+    dbStore = DBStoreBuilder.createDBStore(
+        conf, new SCMDBDefinition());
+
+    pipelineManager =
+        new MockPipelineManager(dbStore, scmhaManager, nodeManager);
+
+    this.containerStateManager = ContainerStateManagerImpl.newBuilder()
+        .setConfiguration(conf)
+        .setPipelineManager(pipelineManager)
+        .setRatisServer(scmhaManager.getRatisServer())
+        .setContainerStore(SCMDBDefinition.CONTAINERS.getTable(dbStore))
+        .setSCMDBTransactionBuffer(scmhaManager.getDBTransactionBuffer())
+        .build();
 
-    this.containerStateManager = new ContainerStateManager(conf);
     this.publisher = Mockito.mock(EventPublisher.class);
 
     Mockito.when(containerManager.getContainer(Mockito.any(ContainerID.class)))
         .thenAnswer(invocation -> containerStateManager
-            .getContainer((ContainerID)invocation.getArguments()[0]));
+            .getContainer(((ContainerID)invocation
+                .getArguments()[0])));
 
     Mockito.when(containerManager.getContainerReplicas(
         Mockito.any(ContainerID.class)))
         .thenAnswer(invocation -> containerStateManager
-            .getContainerReplicas((ContainerID)invocation.getArguments()[0]));
+            .getContainerReplicas(((ContainerID)invocation
+                .getArguments()[0])));
 
     Mockito.doAnswer(invocation -> {
       containerStateManager
-          .removeContainerReplica((ContainerID)invocation.getArguments()[0],
+          .removeContainerReplica(((ContainerID)invocation
+                  .getArguments()[0]),
               (ContainerReplica)invocation.getArguments()[1]);
       return null;
     }).when(containerManager).removeContainerReplica(
@@ -120,7 +163,8 @@ public class TestIncrementalContainerReportHandler {
 
     Mockito.doAnswer(invocation -> {
       containerStateManager
-          .updateContainerState((ContainerID)invocation.getArguments()[0],
+          .updateContainerState(((ContainerID)invocation
+                  .getArguments()[0]).getProtobuf(),
               (HddsProtos.LifeCycleEvent)invocation.getArguments()[1]);
       return null;
     }).when(containerManager).updateContainerState(
@@ -129,7 +173,8 @@ public class TestIncrementalContainerReportHandler {
 
     Mockito.doAnswer(invocation -> {
       containerStateManager
-          .updateContainerReplica((ContainerID)invocation.getArguments()[0],
+          .updateContainerReplica(((ContainerID)invocation
+                  .getArguments()[0]),
               (ContainerReplica) invocation.getArguments()[1]);
       return null;
     }).when(containerManager).updateContainerReplica(
@@ -138,14 +183,19 @@ public class TestIncrementalContainerReportHandler {
 
   }
 
-  @After
-  public void tearDown() throws IOException {
+  @AfterEach
+  public void tearDown() throws Exception {
     containerStateManager.close();
+    if (dbStore != null) {
+      dbStore.close();
+    }
+
+    FileUtil.fullyDelete(testDir);
   }
 
 
   @Test
-  public void testClosingToClosed() throws IOException {
+  public void testClosingToClosed() throws IOException, TimeoutException {
     final IncrementalContainerReportHandler reportHandler =
         new IncrementalContainerReportHandler(
             nodeManager, containerManager, scmContext);
@@ -161,15 +211,9 @@ public class TestIncrementalContainerReportHandler {
         ContainerReplicaProto.State.CLOSING,
         datanodeOne, datanodeTwo, datanodeThree);
 
-    containerStateManager.loadContainer(container);
-    containerReplicas.forEach(r -> {
-      try {
-        containerStateManager.updateContainerReplica(
-            container.containerID(), r);
-      } catch (ContainerNotFoundException ignored) {
-
-      }
-    });
+    containerStateManager.addContainer(container.getProtobuf());
+    containerReplicas.forEach(r -> containerStateManager.updateContainerReplica(
+        container.containerID(), r));
 
     final IncrementalContainerReportProto containerReport =
         getIncrementalContainerReportProto(container.containerID(),
@@ -179,11 +223,12 @@ public class TestIncrementalContainerReportHandler {
         new IncrementalContainerReportFromDatanode(
             datanodeOne, containerReport);
     reportHandler.onMessage(icrFromDatanode, publisher);
-    Assert.assertEquals(LifeCycleState.CLOSED, container.getState());
+    Assertions.assertEquals(LifeCycleState.CLOSED,
+        containerManager.getContainer(container.containerID()).getState());
   }
 
   @Test
-  public void testClosingToQuasiClosed() throws IOException {
+  public void testClosingToQuasiClosed() throws IOException, TimeoutException {
     final IncrementalContainerReportHandler reportHandler =
         new IncrementalContainerReportHandler(
             nodeManager, containerManager, scmContext);
@@ -199,15 +244,9 @@ public class TestIncrementalContainerReportHandler {
         ContainerReplicaProto.State.CLOSING,
         datanodeOne, datanodeTwo, datanodeThree);
 
-    containerStateManager.loadContainer(container);
-    containerReplicas.forEach(r -> {
-      try {
-        containerStateManager.updateContainerReplica(
-            container.containerID(), r);
-      } catch (ContainerNotFoundException ignored) {
-
-      }
-    });
+    containerStateManager.addContainer(container.getProtobuf());
+    containerReplicas.forEach(r -> containerStateManager.updateContainerReplica(
+        container.containerID(), r));
 
 
     final IncrementalContainerReportProto containerReport =
@@ -218,11 +257,12 @@ public class TestIncrementalContainerReportHandler {
         new IncrementalContainerReportFromDatanode(
             datanodeOne, containerReport);
     reportHandler.onMessage(icrFromDatanode, publisher);
-    Assert.assertEquals(LifeCycleState.QUASI_CLOSED, container.getState());
+    Assertions.assertEquals(LifeCycleState.QUASI_CLOSED,
+        containerManager.getContainer(container.containerID()).getState());
   }
 
   @Test
-  public void testQuasiClosedToClosed() throws IOException {
+  public void testQuasiClosedToClosed() throws IOException, TimeoutException {
     final IncrementalContainerReportHandler reportHandler =
         new IncrementalContainerReportHandler(
             nodeManager, containerManager, scmContext);
@@ -242,15 +282,9 @@ public class TestIncrementalContainerReportHandler {
         ContainerReplicaProto.State.QUASI_CLOSED,
         datanodeThree));
 
-    containerStateManager.loadContainer(container);
-    containerReplicas.forEach(r -> {
-      try {
-        containerStateManager.updateContainerReplica(
-            container.containerID(), r);
-      } catch (ContainerNotFoundException ignored) {
-
-      }
-    });
+    containerStateManager.addContainer(container.getProtobuf());
+    containerReplicas.forEach(r -> containerStateManager.updateContainerReplica(
+        container.containerID(), r));
 
     final IncrementalContainerReportProto containerReport =
         getIncrementalContainerReportProto(container.containerID(),
@@ -260,11 +294,12 @@ public class TestIncrementalContainerReportHandler {
         new IncrementalContainerReportFromDatanode(
             datanodeOne, containerReport);
     reportHandler.onMessage(icr, publisher);
-    Assert.assertEquals(LifeCycleState.CLOSED, container.getState());
+    Assertions.assertEquals(LifeCycleState.CLOSED,
+        containerManager.getContainer(container.containerID()).getState());
   }
 
   @Test
-  public void testDeleteContainer() throws IOException {
+  public void testDeleteContainer() throws IOException, TimeoutException {
     final IncrementalContainerReportHandler reportHandler =
         new IncrementalContainerReportHandler(
             nodeManager, containerManager, scmContext);
@@ -280,16 +315,10 @@ public class TestIncrementalContainerReportHandler {
         CLOSED,
         datanodeOne, datanodeTwo, datanodeThree);
 
-    containerStateManager.loadContainer(container);
-    containerReplicas.forEach(r -> {
-      try {
-        containerStateManager.updateContainerReplica(
-            container.containerID(), r);
-      } catch (ContainerNotFoundException ignored) {
-
-      }
-    });
-    Assert.assertEquals(3, containerStateManager
+    containerStateManager.addContainer(container.getProtobuf());
+    containerReplicas.forEach(r -> containerStateManager.updateContainerReplica(
+        container.containerID(), r));
+    Assertions.assertEquals(3, containerStateManager
         .getContainerReplicas(container.containerID()).size());
     final IncrementalContainerReportProto containerReport =
         getIncrementalContainerReportProto(container.containerID(),
@@ -299,7 +328,7 @@ public class TestIncrementalContainerReportHandler {
         new IncrementalContainerReportFromDatanode(
             datanodeOne, containerReport);
     reportHandler.onMessage(icr, publisher);
-    Assert.assertEquals(2, containerStateManager
+    Assertions.assertEquals(2, containerStateManager
         .getContainerReplicas(container.containerID()).size());
   }
 
@@ -307,7 +336,7 @@ public class TestIncrementalContainerReportHandler {
   // HDDS-5249 - This test reproduces the race condition mentioned in the Jira
   // until the code was changed to fix the race condition.
   public void testICRFCRRace() throws IOException, NodeNotFoundException,
-      ExecutionException, InterruptedException {
+      ExecutionException, InterruptedException, TimeoutException {
     final IncrementalContainerReportHandler reportHandler =
         new IncrementalContainerReportHandler(
             nodeManager, containerManager, scmContext);
@@ -319,10 +348,10 @@ public class TestIncrementalContainerReportHandler {
     final DatanodeDetails datanode = randomDatanodeDetails();
     nodeManager.register(datanode, null, null);
 
-    containerStateManager.loadContainer(container);
-    containerStateManager.loadContainer(containerTwo);
+    containerStateManager.addContainer(container.getProtobuf());
+    containerStateManager.addContainer(containerTwo.getProtobuf());
 
-    Assert.assertEquals(0, nodeManager.getContainers(datanode).size());
+    Assertions.assertEquals(0, nodeManager.getContainers(datanode).size());
 
     final IncrementalContainerReportProto containerReport =
         getIncrementalContainerReportProto(container.containerID(),
@@ -332,10 +361,9 @@ public class TestIncrementalContainerReportHandler {
         new IncrementalContainerReportFromDatanode(
             datanode, containerReport);
 
-    final ContainerReportsProto fullReport = TestContainerReportHandler
-        .getContainerReportsProto(containerTwo.containerID(), CLOSED,
-            datanode.getUuidString());
-    final ContainerReportFromDatanode fcr =new ContainerReportFromDatanode(
+    final ContainerReportsProto fullReport = getContainerReportsProto(
+            containerTwo.containerID(), CLOSED, datanode.getUuidString());
+    final ContainerReportFromDatanode fcr = new ContainerReportFromDatanode(
         datanode, fullReport);
 
     // We need to run the FCR and ICR at the same time via the executor so we
@@ -346,7 +374,7 @@ public class TestIncrementalContainerReportHandler {
       // Running this test 10 times to ensure the race condition we are testing
       // for does not occur. In local tests, before the code was fixed, this
       // test failed consistently every time (reproducing the issue).
-      for (int i=0; i<10; i++) {
+      for (int i = 0; i < 10; i++) {
         Future<?> t1 =
             executor.submit(() -> fullReportHandler.onMessage(fcr, publisher));
         Future<?> t2 =
@@ -358,49 +386,123 @@ public class TestIncrementalContainerReportHandler {
         if (nmContainers.contains(container.containerID())) {
           // If we find "container" in the NM, then we must also have it in
           // Container Manager.
-          Assert.assertEquals(1, containerStateManager
-              .getContainerReplicas(container.containerID()).size());
-          Assert.assertEquals(2, nmContainers.size());
+          Assertions.assertEquals(1, containerStateManager
+              .getContainerReplicas(container.containerID())
+              .size());
+          Assertions.assertEquals(2, nmContainers.size());
         } else {
           // If the race condition occurs as mentioned in HDDS-5249, then this
           // assert should fail. We will have found nothing for "container" in
           // NM, but have found something for it in ContainerManager, and that
           // should not happen. It should be in both, or neither.
-          Assert.assertEquals(0, containerStateManager
-              .getContainerReplicas(container.containerID()).size());
-          Assert.assertEquals(1, nmContainers.size());
+          Assertions.assertEquals(0, containerStateManager
+              .getContainerReplicas(container.containerID())
+              .size());
+          Assertions.assertEquals(1, nmContainers.size());
         }
-        Assert.assertEquals(1, containerStateManager
-            .getContainerReplicas(containerTwo.containerID()).size());
+        Assertions.assertEquals(1, containerStateManager
+            .getContainerReplicas(containerTwo.containerID())
+            .size());
       }
     } finally {
       executor.shutdown();
     }
   }
 
+  private static IncrementalContainerReportProto
+      getIncrementalContainerReportProto(ContainerReplicaProto replicaProto) {
+    final IncrementalContainerReportProto.Builder crBuilder =
+            IncrementalContainerReportProto.newBuilder();
+    return crBuilder.addReport(replicaProto).build();
+  }
+
+  private static IncrementalContainerReportProto
+      getIncrementalContainerReportProto(
+            final ContainerID containerId,
+            final ContainerReplicaProto.State state,
+            final String originNodeId,
+            final boolean hasReplicaIndex,
+            final int replicaIndex) {
+    final ContainerReplicaProto.Builder replicaProto =
+            ContainerReplicaProto.newBuilder()
+                    .setContainerID(containerId.getId())
+                    .setState(state)
+                    .setOriginNodeId(originNodeId)
+                    .setFinalhash("e16cc9d6024365750ed8dbd194ea46d2")
+                    .setSize(5368709120L)
+                    .setUsed(2000000000L)
+                    .setKeyCount(100000000L)
+                    .setReadCount(100000000L)
+                    .setWriteCount(100000000L)
+                    .setReadBytes(2000000000L)
+                    .setWriteBytes(2000000000L)
+                    .setBlockCommitSequenceId(10000L)
+                    .setDeleteTransactionId(0);
+    if (hasReplicaIndex) {
+      replicaProto.setReplicaIndex(replicaIndex);
+    }
+    return getIncrementalContainerReportProto(replicaProto.build());
+  }
+
   private static IncrementalContainerReportProto
       getIncrementalContainerReportProto(
           final ContainerID containerId,
           final ContainerReplicaProto.State state,
           final String originNodeId) {
-    final IncrementalContainerReportProto.Builder crBuilder =
-        IncrementalContainerReportProto.newBuilder();
-    final ContainerReplicaProto replicaProto =
-        ContainerReplicaProto.newBuilder()
-            .setContainerID(containerId.getId())
-            .setState(state)
-            .setOriginNodeId(originNodeId)
-            .setFinalhash("e16cc9d6024365750ed8dbd194ea46d2")
-            .setSize(5368709120L)
-            .setUsed(2000000000L)
-            .setKeyCount(100000000L)
-            .setReadCount(100000000L)
-            .setWriteCount(100000000L)
-            .setReadBytes(2000000000L)
-            .setWriteBytes(2000000000L)
-            .setBlockCommitSequenceId(10000L)
-            .setDeleteTransactionId(0)
-            .build();
-    return crBuilder.addReport(replicaProto).build();
+    return getIncrementalContainerReportProto(containerId, state, originNodeId,
+            false, 0);
+  }
+
+  private void testReplicaIndexUpdate(ContainerInfo container,
+         DatanodeDetails dn, int replicaIndex,
+         Map<DatanodeDetails, Integer> expectedReplicaMap) {
+    final IncrementalContainerReportProto containerReport =
+            getIncrementalContainerReportProto(container.containerID(),
+                    ContainerReplicaProto.State.CLOSED, dn.getUuidString(),
+                    true, replicaIndex);
+    final IncrementalContainerReportFromDatanode containerReportFromDatanode =
+            new IncrementalContainerReportFromDatanode(dn, containerReport);
+    final IncrementalContainerReportHandler reportHandler =
+            new IncrementalContainerReportHandler(nodeManager,
+                    containerManager, scmContext);
+    reportHandler.onMessage(containerReportFromDatanode, publisher);
+    Assert.assertEquals(containerStateManager
+            .getContainerReplicas(container.containerID()).stream()
+            .collect(Collectors.toMap(ContainerReplica::getDatanodeDetails,
+                    ContainerReplica::getReplicaIndex)), expectedReplicaMap);
+
+  }
+
+  @Test
+  public void testECReplicaIndexValidation() throws NodeNotFoundException,
+          IOException, TimeoutException {
+    List<DatanodeDetails> dns = IntStream.range(0, 5)
+        .mapToObj(i -> randomDatanodeDetails()).collect(Collectors.toList());
+    dns.stream().forEach(dn -> nodeManager.register(dn, null, null));
+    ECReplicationConfig replicationConfig = new ECReplicationConfig(3, 2);
+    final ContainerInfo container = getECContainer(LifeCycleState.CLOSED,
+            PipelineID.randomId(), replicationConfig);
+    nodeManager.addContainer(dns.get(0), container.containerID());
+    nodeManager.addContainer(dns.get(1), container.containerID());
+    nodeManager.addContainer(dns.get(2), container.containerID());
+    nodeManager.addContainer(dns.get(3), container.containerID());
+    nodeManager.addContainer(dns.get(4), container.containerID());
+    containerStateManager.addContainer(container.getProtobuf());
+    Set<ContainerReplica> replicas =
+            HddsTestUtils.getReplicasWithReplicaIndex(container.containerID(),
+                    ContainerReplicaProto.State.CLOSED,
+                    HddsTestUtils.CONTAINER_USED_BYTES_DEFAULT,
+                    HddsTestUtils.CONTAINER_NUM_KEYS_DEFAULT,
+                    1000000L,
+                    dns.toArray(new DatanodeDetails[0]));
+    Map<DatanodeDetails, Integer> replicaMap = replicas.stream()
+            .collect(Collectors.toMap(ContainerReplica::getDatanodeDetails,
+                    ContainerReplica::getReplicaIndex));
+    replicas.forEach(r -> containerStateManager.updateContainerReplica(
+            container.containerID(), r));
+    testReplicaIndexUpdate(container, dns.get(0), 0, replicaMap);
+    testReplicaIndexUpdate(container, dns.get(0), 6, replicaMap);
+    replicaMap.put(dns.get(0), 2);
+    testReplicaIndexUpdate(container, dns.get(0), 2, replicaMap);
   }
 }
\ No newline at end of file
