@@ -18,9 +18,9 @@ package org.apache.hadoop.hdds.scm.container.placement.algorithms;
 
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Collection;
 import java.util.List;
 import java.util.Random;
+import java.util.stream.IntStream;
 
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.conf.StorageUnit;
@@ -29,7 +29,7 @@ import org.apache.hadoop.hdds.protocol.MockDatanodeDetails;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.MetadataStorageReportProto;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.StorageReportProto;
 import org.apache.hadoop.hdds.scm.ContainerPlacementStatus;
-import org.apache.hadoop.hdds.scm.TestUtils;
+import org.apache.hadoop.hdds.scm.HddsTestUtils;
 import org.apache.hadoop.hdds.scm.exceptions.SCMException;
 import org.apache.hadoop.hdds.scm.net.NetConstants;
 import org.apache.hadoop.hdds.scm.net.NetworkTopology;
@@ -40,11 +40,13 @@ import org.apache.hadoop.hdds.scm.node.DatanodeInfo;
 import org.apache.hadoop.hdds.scm.node.NodeManager;
 import org.apache.hadoop.hdds.scm.node.NodeStatus;
 import org.apache.hadoop.ozone.container.upgrade.UpgradeUtils;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Test;
-import org.junit.runner.RunWith;
-import org.junit.runners.Parameterized;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.MethodSource;
+import org.junit.jupiter.params.provider.ValueSource;
 import org.mockito.Mockito;
 
 import org.apache.commons.lang3.StringUtils;
@@ -55,25 +57,21 @@ import static org.apache.hadoop.hdds.scm.ScmConfigKeys.OZONE_DATANODE_RATIS_VOLU
 import static org.apache.hadoop.hdds.scm.net.NetConstants.LEAF_SCHEMA;
 import static org.apache.hadoop.hdds.scm.net.NetConstants.RACK_SCHEMA;
 import static org.apache.hadoop.hdds.scm.net.NetConstants.ROOT_SCHEMA;
-import org.hamcrest.MatcherAssert;
 
-import static org.hamcrest.Matchers.greaterThanOrEqualTo;
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertFalse;
-import static org.junit.Assert.assertTrue;
-import static org.junit.Assert.fail;
-import static org.junit.Assume.assumeTrue;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.junit.jupiter.api.Assertions.fail;
+import static org.junit.jupiter.api.Assumptions.assumeTrue;
 import static org.mockito.Mockito.when;
 
 /**
  * Test for the scm container rack aware placement.
  */
-@RunWith(Parameterized.class)
 public class TestSCMContainerPlacementRackAware {
   private NetworkTopology cluster;
   private OzoneConfiguration conf;
   private NodeManager nodeManager;
-  private final Integer datanodeCount;
   private final List<DatanodeDetails> datanodes = new ArrayList<>();
   private final List<DatanodeInfo> dnInfos = new ArrayList<>();
   // policy with fallback capability
@@ -85,18 +83,12 @@ public class TestSCMContainerPlacementRackAware {
   private SCMContainerPlacementMetrics metrics;
   private static final int NODE_PER_RACK = 5;
 
-  public TestSCMContainerPlacementRackAware(Integer count) {
-    this.datanodeCount = count;
+
+  private static IntStream numDatanodes() {
+    return IntStream.rangeClosed(3, 15);
   }
 
-  @Parameterized.Parameters
-  public static Collection<Object[]> setupDatanodes() {
-    return Arrays.asList(new Object[][] {{3}, {4}, {5}, {6}, {7}, {8}, {9},
-        {10}, {11}, {12}, {13}, {14}, {15}});
-  }
-
-  @Before
-  public void setup() {
+  private void setup(int datanodeCount) {
     //initialize network topology instance
     conf = new OzoneConfiguration();
     // We are using small units here
@@ -115,61 +107,60 @@ public class TestSCMContainerPlacementRackAware {
       DatanodeDetails datanodeDetails =
           MockDatanodeDetails.createDatanodeDetails(
           hostname + i, rack + (i / NODE_PER_RACK));
+      datanodes.add(datanodeDetails);
+      cluster.add(datanodeDetails);
       DatanodeInfo datanodeInfo = new DatanodeInfo(
           datanodeDetails, NodeStatus.inServiceHealthy(),
           UpgradeUtils.defaultLayoutVersionProto());
 
-      StorageReportProto storage1 = TestUtils.createStorageReport(
+      StorageReportProto storage1 = HddsTestUtils.createStorageReport(
           datanodeInfo.getUuid(), "/data1-" + datanodeInfo.getUuidString(),
           STORAGE_CAPACITY, 0, 100L, null);
       MetadataStorageReportProto metaStorage1 =
-          TestUtils.createMetadataStorageReport(
+          HddsTestUtils.createMetadataStorageReport(
           "/metadata1-" + datanodeInfo.getUuidString(),
           STORAGE_CAPACITY, 0, 100L, null);
       datanodeInfo.updateStorageReports(
           new ArrayList<>(Arrays.asList(storage1)));
       datanodeInfo.updateMetaDataStorageReports(
           new ArrayList<>(Arrays.asList(metaStorage1)));
-
-      datanodes.add(datanodeDetails);
-      cluster.add(datanodeDetails);
       dnInfos.add(datanodeInfo);
     }
 
     if (datanodeCount > 4) {
-      StorageReportProto storage2 = TestUtils.createStorageReport(
+      StorageReportProto storage2 = HddsTestUtils.createStorageReport(
           dnInfos.get(2).getUuid(),
           "/data1-" + datanodes.get(2).getUuidString(),
           STORAGE_CAPACITY, 90L, 10L, null);
       dnInfos.get(2).updateStorageReports(
           new ArrayList<>(Arrays.asList(storage2)));
-      StorageReportProto storage3 = TestUtils.createStorageReport(
+      StorageReportProto storage3 = HddsTestUtils.createStorageReport(
           dnInfos.get(3).getUuid(),
           "/data1-" + dnInfos.get(3).getUuidString(),
           STORAGE_CAPACITY, 80L, 20L, null);
       dnInfos.get(3).updateStorageReports(
           new ArrayList<>(Arrays.asList(storage3)));
-      StorageReportProto storage4 = TestUtils.createStorageReport(
+      StorageReportProto storage4 = HddsTestUtils.createStorageReport(
           dnInfos.get(4).getUuid(),
           "/data1-" + dnInfos.get(4).getUuidString(),
           STORAGE_CAPACITY, 70L, 30L, null);
       dnInfos.get(4).updateStorageReports(
           new ArrayList<>(Arrays.asList(storage4)));
     } else if (datanodeCount > 3) {
-      StorageReportProto storage2 = TestUtils.createStorageReport(
+      StorageReportProto storage2 = HddsTestUtils.createStorageReport(
           dnInfos.get(2).getUuid(),
           "/data1-" + dnInfos.get(2).getUuidString(),
           STORAGE_CAPACITY, 90L, 10L, null);
       dnInfos.get(2).updateStorageReports(
           new ArrayList<>(Arrays.asList(storage2)));
-      StorageReportProto storage3 = TestUtils.createStorageReport(
+      StorageReportProto storage3 = HddsTestUtils.createStorageReport(
           dnInfos.get(3).getUuid(),
           "/data1-" + dnInfos.get(3).getUuidString(),
           STORAGE_CAPACITY, 80L, 20L, null);
       dnInfos.get(3).updateStorageReports(
           new ArrayList<>(Arrays.asList(storage3)));
     } else if (datanodeCount > 2) {
-      StorageReportProto storage2 = TestUtils.createStorageReport(
+      StorageReportProto storage2 = HddsTestUtils.createStorageReport(
           dnInfos.get(2).getUuid(),
           "/data1-" + dnInfos.get(2).getUuidString(),
           STORAGE_CAPACITY, 84L, 16L, null);
@@ -189,62 +180,77 @@ public class TestSCMContainerPlacementRackAware {
         .thenReturn(cluster);
 
     // create placement policy instances
-    metrics = SCMContainerPlacementMetrics.create();
     policy = new SCMContainerPlacementRackAware(
         nodeManager, conf, cluster, true, metrics);
     policyNoFallback = new SCMContainerPlacementRackAware(
         nodeManager, conf, cluster, false, metrics);
   }
 
-  @Test
-  public void chooseNodeWithNoExcludedNodes() throws SCMException {
+  @BeforeEach
+  public void init() {
+    metrics = SCMContainerPlacementMetrics.create();
+  }
+
+  @AfterEach
+  public void teardown() {
+    metrics.unRegister();
+  }
+
+  @ParameterizedTest
+  @MethodSource("numDatanodes")
+  public void chooseNodeWithNoExcludedNodes(int datanodeCount)
+      throws SCMException {
+    setup(datanodeCount);
     // test choose new datanodes for new pipeline cases
     // 1 replica
     int nodeNum = 1;
     List<DatanodeDetails> datanodeDetails =
         policy.chooseDatanodes(null, null, nodeNum, 0, 15);
-    Assert.assertEquals(nodeNum, datanodeDetails.size());
+    Assertions.assertEquals(nodeNum, datanodeDetails.size());
 
     // 2 replicas
     nodeNum = 2;
     datanodeDetails = policy.chooseDatanodes(null, null, nodeNum, 0, 15);
-    Assert.assertEquals(nodeNum, datanodeDetails.size());
-    Assert.assertTrue(cluster.isSameParent(datanodeDetails.get(0),
+    Assertions.assertEquals(nodeNum, datanodeDetails.size());
+    Assertions.assertTrue(cluster.isSameParent(datanodeDetails.get(0),
         datanodeDetails.get(1)) || (datanodeCount % NODE_PER_RACK == 1));
 
     //  3 replicas
     nodeNum = 3;
     datanodeDetails = policy.chooseDatanodes(null, null, nodeNum, 0, 15);
-    Assert.assertEquals(nodeNum, datanodeDetails.size());
+    Assertions.assertEquals(nodeNum, datanodeDetails.size());
     // requires at least 2 racks for following statement
     assumeTrue(datanodeCount > NODE_PER_RACK &&
         datanodeCount % NODE_PER_RACK > 1);
-    Assert.assertTrue(cluster.isSameParent(datanodeDetails.get(0),
+    Assertions.assertTrue(cluster.isSameParent(datanodeDetails.get(0),
         datanodeDetails.get(1)));
-    Assert.assertFalse(cluster.isSameParent(datanodeDetails.get(0),
+    Assertions.assertFalse(cluster.isSameParent(datanodeDetails.get(0),
         datanodeDetails.get(2)));
-    Assert.assertFalse(cluster.isSameParent(datanodeDetails.get(1),
+    Assertions.assertFalse(cluster.isSameParent(datanodeDetails.get(1),
         datanodeDetails.get(2)));
 
     //  4 replicas
     nodeNum = 4;
     datanodeDetails = policy.chooseDatanodes(null, null, nodeNum, 0, 15);
-    Assert.assertEquals(nodeNum, datanodeDetails.size());
+    Assertions.assertEquals(nodeNum, datanodeDetails.size());
     // requires at least 2 racks and enough datanodes for following statement
     assumeTrue(datanodeCount > NODE_PER_RACK + 1);
-    Assert.assertTrue(cluster.isSameParent(datanodeDetails.get(0),
+    Assertions.assertTrue(cluster.isSameParent(datanodeDetails.get(0),
         datanodeDetails.get(1)));
-    Assert.assertFalse(cluster.isSameParent(datanodeDetails.get(0),
+    Assertions.assertFalse(cluster.isSameParent(datanodeDetails.get(0),
         datanodeDetails.get(2)));
-    Assert.assertFalse(cluster.isSameParent(datanodeDetails.get(1),
+    Assertions.assertFalse(cluster.isSameParent(datanodeDetails.get(1),
         datanodeDetails.get(2)));
   }
 
-  @Test
-  public void chooseNodeWithExcludedNodes() throws SCMException {
+  @ParameterizedTest
+  @MethodSource("numDatanodes")
+  public void chooseNodeWithExcludedNodes(int datanodeCount)
+      throws SCMException {
     // test choose new datanodes for under replicated pipeline
     // 3 replicas, two existing datanodes on same rack
     assumeTrue(datanodeCount > NODE_PER_RACK);
+    setup(datanodeCount);
     int nodeNum = 1;
     List<DatanodeDetails> excludedNodes = new ArrayList<>();
 
@@ -252,10 +258,10 @@ public class TestSCMContainerPlacementRackAware {
     excludedNodes.add(datanodes.get(1));
     List<DatanodeDetails> datanodeDetails = policy.chooseDatanodes(
         excludedNodes, null, nodeNum, 0, 15);
-    Assert.assertEquals(nodeNum, datanodeDetails.size());
-    Assert.assertFalse(cluster.isSameParent(datanodeDetails.get(0),
+    Assertions.assertEquals(nodeNum, datanodeDetails.size());
+    Assertions.assertFalse(cluster.isSameParent(datanodeDetails.get(0),
         excludedNodes.get(0)));
-    Assert.assertFalse(cluster.isSameParent(datanodeDetails.get(0),
+    Assertions.assertFalse(cluster.isSameParent(datanodeDetails.get(0),
         excludedNodes.get(1)));
 
     // 3 replicas, one existing datanode
@@ -264,8 +270,8 @@ public class TestSCMContainerPlacementRackAware {
     excludedNodes.add(datanodes.get(0));
     datanodeDetails = policy.chooseDatanodes(
         excludedNodes, null, nodeNum, 0, 15);
-    Assert.assertEquals(nodeNum, datanodeDetails.size());
-    Assert.assertTrue(cluster.isSameParent(
+    Assertions.assertEquals(nodeNum, datanodeDetails.size());
+    Assertions.assertTrue(cluster.isSameParent(
         datanodeDetails.get(0), excludedNodes.get(0)) ||
         cluster.isSameParent(datanodeDetails.get(0), excludedNodes.get(1)));
 
@@ -276,47 +282,52 @@ public class TestSCMContainerPlacementRackAware {
     excludedNodes.add(datanodes.get(5));
     datanodeDetails = policy.chooseDatanodes(
         excludedNodes, null, nodeNum, 0, 15);
-    Assert.assertEquals(nodeNum, datanodeDetails.size());
-    Assert.assertTrue(cluster.isSameParent(
+    Assertions.assertEquals(nodeNum, datanodeDetails.size());
+    Assertions.assertTrue(cluster.isSameParent(
         datanodeDetails.get(0), excludedNodes.get(0)) ||
         cluster.isSameParent(datanodeDetails.get(0), excludedNodes.get(1)));
   }
 
-  @Test
-  public void testSingleNodeRack() throws SCMException {
+  @ParameterizedTest
+  @ValueSource(ints = {NODE_PER_RACK + 1, 2 * NODE_PER_RACK + 1})
+  public void testSingleNodeRack(int datanodeCount) throws SCMException {
     // make sure there is a single node rack
     assumeTrue(datanodeCount % NODE_PER_RACK == 1);
+    setup(datanodeCount);
     List<DatanodeDetails> excludeNodes = new ArrayList<>();
     excludeNodes.add(datanodes.get(datanodeCount - 1));
     excludeNodes.add(datanodes.get(0));
     List<DatanodeDetails> chooseDatanodes =
         policy.chooseDatanodes(excludeNodes, null, 1, 0, 0);
-    Assert.assertTrue(chooseDatanodes.size() == 1);
+    assertEquals(1, chooseDatanodes.size());
     // the selected node should be on the same rack as the second exclude node
-    Assert.assertTrue(chooseDatanodes.get(0).toString(),
-        cluster.isSameParent(chooseDatanodes.get(0), excludeNodes.get(1)));
+    Assertions.assertTrue(
+        cluster.isSameParent(chooseDatanodes.get(0), excludeNodes.get(1)),
+        chooseDatanodes.get(0).toString());
   }
 
-  @Test
-  public void testFallback() throws SCMException {
+  @ParameterizedTest
+  @ValueSource(ints = {12, 13, 14})
+  public void testFallback(int datanodeCount) throws SCMException {
     // 5 replicas. there are only 3 racks. policy with fallback should
     // allocate the 5th datanode though it will break the rack rule(first
     // 2 replicas on same rack, others on different racks).
     assumeTrue(datanodeCount > NODE_PER_RACK * 2 &&
         (datanodeCount % NODE_PER_RACK > 1));
+    setup(datanodeCount);
     int nodeNum = 5;
     List<DatanodeDetails> datanodeDetails =
         policy.chooseDatanodes(null, null, nodeNum, 0, 15);
-    Assert.assertEquals(nodeNum, datanodeDetails.size());
-    Assert.assertTrue(cluster.isSameParent(datanodeDetails.get(0),
+    Assertions.assertEquals(nodeNum, datanodeDetails.size());
+    Assertions.assertTrue(cluster.isSameParent(datanodeDetails.get(0),
         datanodeDetails.get(1)));
-    Assert.assertFalse(cluster.isSameParent(datanodeDetails.get(0),
+    Assertions.assertFalse(cluster.isSameParent(datanodeDetails.get(0),
         datanodeDetails.get(2)));
-    Assert.assertFalse(cluster.isSameParent(datanodeDetails.get(1),
+    Assertions.assertFalse(cluster.isSameParent(datanodeDetails.get(1),
         datanodeDetails.get(2)));
-    Assert.assertFalse(cluster.isSameParent(datanodeDetails.get(0),
+    Assertions.assertFalse(cluster.isSameParent(datanodeDetails.get(0),
         datanodeDetails.get(3)));
-    Assert.assertFalse(cluster.isSameParent(datanodeDetails.get(2),
+    Assertions.assertFalse(cluster.isSameParent(datanodeDetails.get(2),
         datanodeDetails.get(3)));
 
     // get metrics
@@ -326,16 +337,18 @@ public class TestSCMContainerPlacementRackAware {
     long compromiseCount = metrics.getDatanodeChooseFallbackCount();
 
     // verify metrics
-    Assert.assertEquals(totalRequest, nodeNum);
-    Assert.assertEquals(successCount, nodeNum);
-    Assert.assertTrue(tryCount > nodeNum);
-    Assert.assertTrue(compromiseCount >= 1);
+    Assertions.assertEquals(totalRequest, nodeNum);
+    Assertions.assertEquals(successCount, nodeNum);
+    Assertions.assertTrue(tryCount > nodeNum);
+    Assertions.assertTrue(compromiseCount >= 1);
   }
 
-  @Test
-  public void testNoFallback() throws SCMException {
+  @ParameterizedTest
+  @ValueSource(ints = {11, 12, 13, 14, 15})
+  public void testNoFallback(int datanodeCount) {
     assumeTrue(datanodeCount > (NODE_PER_RACK * 2) &&
         (datanodeCount <= NODE_PER_RACK * 3));
+    setup(datanodeCount);
     // 5 replicas. there are only 3 racks. policy prohibit fallback should fail.
     int nodeNum = 5;
     try {
@@ -351,16 +364,17 @@ public class TestSCMContainerPlacementRackAware {
     long tryCount = metrics.getDatanodeChooseAttemptCount();
     long compromiseCount = metrics.getDatanodeChooseFallbackCount();
 
-    Assert.assertEquals(totalRequest, nodeNum);
-    MatcherAssert.assertThat("Not enough success count", successCount,
-        greaterThanOrEqualTo(1L));
-    MatcherAssert.assertThat("Not enough try count", tryCount,
-        greaterThanOrEqualTo(1L));
-    Assert.assertEquals(compromiseCount, 0);
+    Assertions.assertEquals(nodeNum, totalRequest);
+    Assertions.assertTrue(successCount >= 1, "Not enough success count");
+    Assertions.assertTrue(tryCount >= 1, "Not enough try count");
+    Assertions.assertEquals(0, compromiseCount);
   }
 
-  @Test
-  public void chooseNodeWithFavoredNodes() throws SCMException {
+  @ParameterizedTest
+  @MethodSource("numDatanodes")
+  public void chooseNodeWithFavoredNodes(int datanodeCount)
+      throws SCMException {
+    setup(datanodeCount);
     int nodeNum = 1;
     List<DatanodeDetails> excludedNodes = new ArrayList<>();
     List<DatanodeDetails> favoredNodes = new ArrayList<>();
@@ -369,8 +383,8 @@ public class TestSCMContainerPlacementRackAware {
     favoredNodes.add(datanodes.get(0));
     List<DatanodeDetails> datanodeDetails = policy.chooseDatanodes(
         excludedNodes, favoredNodes, nodeNum, 0, 15);
-    Assert.assertEquals(nodeNum, datanodeDetails.size());
-    Assert.assertEquals(datanodeDetails.get(0).getNetworkFullPath(),
+    Assertions.assertEquals(nodeNum, datanodeDetails.size());
+    Assertions.assertEquals(datanodeDetails.get(0).getNetworkFullPath(),
         favoredNodes.get(0).getNetworkFullPath());
 
     // no overlap between excludedNodes and favoredNodes, favoredNodes can been
@@ -381,8 +395,8 @@ public class TestSCMContainerPlacementRackAware {
     favoredNodes.add(datanodes.get(2));
     datanodeDetails = policy.chooseDatanodes(
         excludedNodes, favoredNodes, nodeNum, 0, 15);
-    Assert.assertEquals(nodeNum, datanodeDetails.size());
-    Assert.assertEquals(datanodeDetails.get(0).getNetworkFullPath(),
+    Assertions.assertEquals(nodeNum, datanodeDetails.size());
+    Assertions.assertEquals(datanodeDetails.get(0).getNetworkFullPath(),
         favoredNodes.get(0).getNetworkFullPath());
 
     // there is overlap between excludedNodes and favoredNodes, favoredNodes
@@ -393,13 +407,15 @@ public class TestSCMContainerPlacementRackAware {
     favoredNodes.add(datanodes.get(0));
     datanodeDetails = policy.chooseDatanodes(
         excludedNodes, favoredNodes, nodeNum, 0, 15);
-    Assert.assertEquals(nodeNum, datanodeDetails.size());
-    Assert.assertFalse(datanodeDetails.get(0).getNetworkFullPath()
-        .equals(favoredNodes.get(0).getNetworkFullPath()));
+    Assertions.assertEquals(nodeNum, datanodeDetails.size());
+    Assertions.assertNotEquals(datanodeDetails.get(0).getNetworkFullPath(),
+        favoredNodes.get(0).getNetworkFullPath());
   }
 
-  @Test
-  public void testNoInfiniteLoop() throws SCMException {
+  @ParameterizedTest
+  @MethodSource("numDatanodes")
+  public void testNoInfiniteLoop(int datanodeCount) {
+    setup(datanodeCount);
     int nodeNum = 1;
 
     try {
@@ -416,15 +432,17 @@ public class TestSCMContainerPlacementRackAware {
     long tryCount = metrics.getDatanodeChooseAttemptCount();
     long compromiseCount = metrics.getDatanodeChooseFallbackCount();
 
-    Assert.assertEquals(totalRequest, nodeNum);
-    Assert.assertEquals(successCount, 0);
-    MatcherAssert.assertThat("Not enough try", tryCount,
-        greaterThanOrEqualTo((long) nodeNum));
-    Assert.assertEquals(compromiseCount, 0);
+    Assertions.assertEquals(totalRequest, nodeNum);
+    Assertions.assertEquals(successCount, 0);
+    Assertions.assertTrue(tryCount >= nodeNum, "Not enough try");
+    Assertions.assertEquals(compromiseCount, 0);
   }
 
-  @Test
-  public void testDatanodeWithDefaultNetworkLocation() throws SCMException {
+  @ParameterizedTest
+  @MethodSource("numDatanodes")
+  public void testDatanodeWithDefaultNetworkLocation(int datanodeCount)
+      throws SCMException {
+    setup(datanodeCount);
     String hostname = "node";
     List<DatanodeInfo> dnInfoList = new ArrayList<>();
     List<DatanodeDetails> dataList = new ArrayList<>();
@@ -438,11 +456,11 @@ public class TestSCMContainerPlacementRackAware {
           dn, NodeStatus.inServiceHealthy(),
           UpgradeUtils.defaultLayoutVersionProto());
 
-      StorageReportProto storage1 = TestUtils.createStorageReport(
+      StorageReportProto storage1 = HddsTestUtils.createStorageReport(
           dnInfo.getUuid(), "/data1-" + dnInfo.getUuidString(),
           STORAGE_CAPACITY, 0, 100L, null);
       MetadataStorageReportProto metaStorage1 =
-          TestUtils.createMetadataStorageReport(
+          HddsTestUtils.createMetadataStorageReport(
           "/metadata1-" + dnInfo.getUuidString(),
           STORAGE_CAPACITY, 0, 100L, null);
       dnInfo.updateStorageReports(
@@ -454,7 +472,7 @@ public class TestSCMContainerPlacementRackAware {
       clusterMap.add(dn);
       dnInfoList.add(dnInfo);
     }
-    Assert.assertEquals(dataList.size(), StringUtils.countMatches(
+    Assertions.assertEquals(dataList.size(), StringUtils.countMatches(
         clusterMap.toString(), NetConstants.DEFAULT_RACK));
     for (DatanodeInfo dn: dnInfoList) {
       when(nodeManager.getNodeByUuid(dn.getUuidString()))
@@ -468,19 +486,20 @@ public class TestSCMContainerPlacementRackAware {
             metrics);
     List<DatanodeDetails> datanodeDetails =
         newPolicy.chooseDatanodes(null, null, nodeNum, 0, 15);
-    Assert.assertEquals(nodeNum, datanodeDetails.size());
-    Assert.assertTrue(cluster.isSameParent(datanodeDetails.get(0),
+    Assertions.assertEquals(nodeNum, datanodeDetails.size());
+    Assertions.assertTrue(cluster.isSameParent(datanodeDetails.get(0),
         datanodeDetails.get(1)));
-    Assert.assertTrue(cluster.isSameParent(datanodeDetails.get(0),
+    Assertions.assertTrue(cluster.isSameParent(datanodeDetails.get(0),
         datanodeDetails.get(2)));
-    Assert.assertTrue(cluster.isSameParent(datanodeDetails.get(1),
+    Assertions.assertTrue(cluster.isSameParent(datanodeDetails.get(1),
         datanodeDetails.get(2)));
   }
 
   @Test
   public void testvalidateContainerPlacement() {
     // Only run this test for the full set of DNs. 5 DNs per rack on 3 racks.
-    assumeTrue(datanodeCount == 15);
+    final int datanodeCount = 15;
+    setup(datanodeCount);
     List<DatanodeDetails> dns = new ArrayList<>();
     // First 5 node are on the same rack
     dns.add(datanodes.get(0));
@@ -516,7 +535,8 @@ public class TestSCMContainerPlacementRackAware {
 
   @Test
   public void testvalidateContainerPlacementSingleRackCluster() {
-    assumeTrue(datanodeCount == 5);
+    final int datanodeCount = 5;
+    setup(datanodeCount);
 
     // All nodes are on the same rack in this test, and the cluster only has
     // one rack.
@@ -543,21 +563,23 @@ public class TestSCMContainerPlacementRackAware {
     assertEquals(0, stat.misReplicationCount());
   }
 
-  @Test
-  public void testOutOfServiceNodesNotSelected() {
+  @ParameterizedTest
+  @MethodSource("numDatanodes")
+  public void testOutOfServiceNodesNotSelected(int datanodeCount) {
+    setup(datanodeCount);
     // Set all the nodes to out of service
     for (DatanodeInfo dn : dnInfos) {
       dn.setNodeStatus(new NodeStatus(DECOMMISSIONED, HEALTHY));
     }
 
-    for (int i=0; i<10; i++) {
+    for (int i = 0; i < 10; i++) {
       // Set a random DN to in_service and ensure it is always picked
       int index = new Random().nextInt(dnInfos.size());
       dnInfos.get(index).setNodeStatus(NodeStatus.inServiceHealthy());
       try {
         List<DatanodeDetails> datanodeDetails =
             policy.chooseDatanodes(null, null, 1, 0, 0);
-        Assert.assertEquals(dnInfos.get(index), datanodeDetails.get(0));
+        Assertions.assertEquals(dnInfos.get(index), datanodeDetails.get(0));
       } catch (SCMException e) {
         // If we get SCMException: No satisfied datanode to meet the ... this is
         // ok, as there is only 1 IN_SERVICE node and with the retry logic we
