@@ -21,33 +21,27 @@ import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.hdds.protocol.MockDatanodeDetails;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
-import org.apache.hadoop.hdds.protocol.proto
-    .StorageContainerDatanodeProtocolProtos.ContainerReplicaProto;
-import org.apache.hadoop.hdds.scm.container.*;
-import org.apache.hadoop.hdds.scm.container.ReplicationManager;
+import org.apache.hadoop.hdds.scm.container.ContainerID;
+import org.apache.hadoop.hdds.scm.container.ContainerNotFoundException;
+import org.apache.hadoop.hdds.scm.container.replication.ReplicationManager;
+import org.apache.hadoop.hdds.scm.container.SimpleMockNodeManager;
 import org.apache.hadoop.hdds.scm.events.SCMEvents;
 import org.apache.hadoop.hdds.scm.node.states.NodeNotFoundException;
-import org.apache.hadoop.hdds.server.events.EventHandler;
-import org.apache.hadoop.hdds.server.events.EventPublisher;
 import org.apache.hadoop.hdds.server.events.EventQueue;
 import org.apache.hadoop.security.authentication.client.AuthenticationException;
-import org.junit.After;
-import org.junit.Before;
-import org.junit.Test;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
 import org.mockito.Mockito;
 import java.io.IOException;
 import java.util.HashSet;
 import java.util.Set;
-import java.util.concurrent.atomic.AtomicInteger;
 
-import static junit.framework.TestCase.assertEquals;
-import static junit.framework.TestCase.assertTrue;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.NodeOperationalState.DECOMMISSIONED;
 import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.NodeOperationalState.ENTERING_MAINTENANCE;
 import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.NodeOperationalState.IN_MAINTENANCE;
 import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.NodeOperationalState.IN_SERVICE;
-import static org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.ContainerReplicaProto.State.CLOSED;
-import static org.mockito.Mockito.reset;
 
 /**
  * Tests to ensure the DatanodeAdminMonitor is working correctly. This class
@@ -59,16 +53,18 @@ public class TestDatanodeAdminMonitor {
   private SimpleMockNodeManager nodeManager;
   private OzoneConfiguration conf;
   private DatanodeAdminMonitorImpl monitor;
-  private DatanodeAdminHandler startAdminHandler;
+  private DatanodeAdminMonitorTestUtil
+          .DatanodeAdminHandler startAdminHandler;
   private ReplicationManager repManager;
   private EventQueue eventQueue;
 
-  @Before
+  @BeforeEach
   public void setup() throws IOException, AuthenticationException {
     conf = new OzoneConfiguration();
 
     eventQueue = new EventQueue();
-    startAdminHandler = new DatanodeAdminHandler();
+    startAdminHandler = new DatanodeAdminMonitorTestUtil
+            .DatanodeAdminHandler();
     eventQueue.addHandler(SCMEvents.START_ADMIN_ON_NODE, startAdminHandler);
 
     nodeManager = new SimpleMockNodeManager();
@@ -77,10 +73,7 @@ public class TestDatanodeAdminMonitor {
 
     monitor =
         new DatanodeAdminMonitorImpl(conf, eventQueue, nodeManager, repManager);
-  }
-
-  @After
-  public void teardown() {
+    monitor.setMetrics(NodeDecommissionMetrics.create());
   }
 
   @Test
@@ -105,7 +98,7 @@ public class TestDatanodeAdminMonitor {
    */
   @Test
   public void testClosePipelinesEventFiredWhenAdminStarted()
-      throws NodeNotFoundException{
+      throws NodeNotFoundException {
     DatanodeDetails dn1 = MockDatanodeDetails.randomDatanodeDetails();
     nodeManager.register(dn1,
         new NodeStatus(HddsProtos.NodeOperationalState.DECOMMISSIONING,
@@ -172,11 +165,13 @@ public class TestDatanodeAdminMonitor {
     nodeManager.setContainers(dn1, generateContainers(3));
     // Mock Replication Manager to return ContainerReplicaCount's which
     // always have a DECOMMISSIONED replica.
-    mockGetContainerReplicaCount(
-        HddsProtos.LifeCycleState.CLOSED,
-        DECOMMISSIONED,
-        IN_SERVICE,
-        IN_SERVICE);
+    DatanodeAdminMonitorTestUtil
+            .mockGetContainerReplicaCount(
+                    repManager,
+                    HddsProtos.LifeCycleState.CLOSED,
+                    DECOMMISSIONED,
+                    IN_SERVICE,
+                    IN_SERVICE);
 
     // Run the monitor for the first time and the node will transition to
     // REPLICATE_CONTAINERS as there are no pipelines to close.
@@ -196,11 +191,13 @@ public class TestDatanodeAdminMonitor {
     // Now change the replicationManager mock to return 3 CLOSED replicas
     // and the node should complete the REPLICATE_CONTAINERS step, moving to
     // complete which will end the decommission workflow
-    mockGetContainerReplicaCount(
-        HddsProtos.LifeCycleState.CLOSED,
-        IN_SERVICE,
-        IN_SERVICE,
-        IN_SERVICE);
+    DatanodeAdminMonitorTestUtil
+            .mockGetContainerReplicaCount(
+                    repManager,
+                    HddsProtos.LifeCycleState.CLOSED,
+                    IN_SERVICE,
+                    IN_SERVICE,
+                    IN_SERVICE);
 
     monitor.run();
 
@@ -218,11 +215,13 @@ public class TestDatanodeAdminMonitor {
             HddsProtos.NodeState.HEALTHY));
 
     nodeManager.setContainers(dn1, generateContainers(3));
-    mockGetContainerReplicaCount(
-        HddsProtos.LifeCycleState.CLOSED,
-        DECOMMISSIONED,
-        IN_SERVICE,
-        IN_SERVICE);
+    DatanodeAdminMonitorTestUtil
+            .mockGetContainerReplicaCount(
+                    repManager,
+                    HddsProtos.LifeCycleState.CLOSED,
+                    DECOMMISSIONED,
+                    IN_SERVICE,
+                    IN_SERVICE);
 
     // Add the node to the monitor, it should have 3 under-replicated containers
     // after the first run
@@ -253,9 +252,13 @@ public class TestDatanodeAdminMonitor {
             HddsProtos.NodeState.HEALTHY));
 
     nodeManager.setContainers(dn1, generateContainers(3));
-    mockGetContainerReplicaCount(
-        HddsProtos.LifeCycleState.CLOSED,
-        DECOMMISSIONED, IN_SERVICE, IN_SERVICE);
+    DatanodeAdminMonitorTestUtil
+            .mockGetContainerReplicaCount(
+                    repManager,
+                    HddsProtos.LifeCycleState.CLOSED,
+                    DECOMMISSIONED,
+                    IN_SERVICE,
+                    IN_SERVICE);
 
     // Add the node to the monitor, it should have 3 under-replicated containers
     // after the first run
@@ -343,11 +346,13 @@ public class TestDatanodeAdminMonitor {
             HddsProtos.NodeState.HEALTHY));
 
     nodeManager.setContainers(dn1, generateContainers(3));
-    mockGetContainerReplicaCount(
-        HddsProtos.LifeCycleState.CLOSED,
-        IN_MAINTENANCE,
-        ENTERING_MAINTENANCE,
-        IN_MAINTENANCE);
+    DatanodeAdminMonitorTestUtil
+            .mockGetContainerReplicaCount(
+                    repManager,
+                    HddsProtos.LifeCycleState.CLOSED,
+                    IN_MAINTENANCE,
+                    ENTERING_MAINTENANCE,
+                    IN_MAINTENANCE);
 
     // Add the node to the monitor, it should transiting to
     // REPLICATE_CONTAINERS as the containers are under-replicated for
@@ -425,57 +430,12 @@ public class TestDatanodeAdminMonitor {
    */
   private Set<ContainerID> generateContainers(int count) {
     Set<ContainerID> containers = new HashSet<>();
-    for (int i=0; i<count; i++) {
+    for (int i = 0; i < count; i++) {
       containers.add(ContainerID.valueOf(i));
     }
     return containers;
   }
 
-  /**
-   * Create a ContainerReplicaCount object, including a container with the
-   * requested ContainerID and state, along with a set of replicas of the given
-   * states.
-   * @param containerID The ID of the container to create an included
-   * @param containerState The state of the container
-   * @param states Create a replica for each of the given states.
-   * @return A ContainerReplicaCount containing the generated container and
-   *         replica set
-   */
-  private ContainerReplicaCount generateReplicaCount(ContainerID containerID,
-      HddsProtos.LifeCycleState containerState,
-      HddsProtos.NodeOperationalState...states) {
-    Set<ContainerReplica> replicas = new HashSet<>();
-    for (HddsProtos.NodeOperationalState s : states) {
-      replicas.add(generateReplica(containerID, s, CLOSED));
-    }
-    ContainerInfo container = new ContainerInfo.Builder()
-        .setContainerID(containerID.getId())
-        .setState(containerState)
-        .build();
-
-    return new ContainerReplicaCount(container, replicas, 0, 0, 3, 2);
-  }
-
-  /**
-   * Generate a new ContainerReplica with the given containerID and State.
-   * @param containerID The ID the replica is associated with
-   * @param nodeState The persistedOpState stored in datanodeDetails.
-   * @param replicaState The state of the generated replica.
-   * @return A containerReplica with the given ID and state
-   */
-  private ContainerReplica generateReplica(ContainerID containerID,
-      HddsProtos.NodeOperationalState nodeState,
-      ContainerReplicaProto.State replicaState) {
-    DatanodeDetails dn = MockDatanodeDetails.randomDatanodeDetails();
-    dn.setPersistedOpState(nodeState);
-    return ContainerReplica.newBuilder()
-        .setContainerState(replicaState)
-        .setContainerID(containerID)
-        .setSequenceId(1)
-        .setDatanodeDetails(dn)
-        .build();
-  }
-
   /**
    * Helper method to get the first node from the set of trackedNodes within
    * the monitor.
@@ -485,47 +445,4 @@ public class TestDatanodeAdminMonitor {
     return
         monitor.getTrackedNodes().toArray(new DatanodeDetails[0])[0];
   }
-
-  /**
-   * The only interaction the DatanodeAdminMonitor has with the
-   * ReplicationManager, is to request a ContainerReplicaCount object for each
-   * container on nodes being deocmmissioned or moved to maintenance. This
-   * method mocks that interface to return a ContainerReplicaCount with a
-   * container in the given containerState and a set of replias in the given
-   * replicaStates.
-   * @param containerState
-   * @param replicaStates
-   * @throws ContainerNotFoundException
-   */
-  private void mockGetContainerReplicaCount(
-      HddsProtos.LifeCycleState containerState,
-      HddsProtos.NodeOperationalState...replicaStates)
-      throws ContainerNotFoundException {
-    reset(repManager);
-    Mockito.when(repManager.getContainerReplicaCount(
-        Mockito.any(ContainerID.class)))
-        .thenAnswer(invocation ->
-            generateReplicaCount((ContainerID)invocation.getArguments()[0],
-                containerState, replicaStates));
-  }
-
-  /**
-   * This simple internal class is used to track and handle any DatanodeAdmin
-   * events fired by the DatanodeAdminMonitor during tests.
-   */
-  private static class DatanodeAdminHandler implements
-      EventHandler<DatanodeDetails> {
-
-    private AtomicInteger invocation = new AtomicInteger(0);
-
-    @Override
-    public void onMessage(final DatanodeDetails dn,
-                          final EventPublisher publisher) {
-      invocation.incrementAndGet();
-    }
-
-    public int getInvocation() {
-      return invocation.get();
-    }
-  }
 }
\ No newline at end of file
