@@ -45,13 +45,14 @@ import org.apache.hadoop.hdds.protocol.proto
 import org.apache.hadoop.hdds.protocol.proto
     .StorageContainerDatanodeProtocolProtos.StorageReportProto;
 import org.apache.hadoop.hdds.scm.ScmConfigKeys;
-import org.apache.hadoop.hdds.scm.TestUtils;
+import org.apache.hadoop.hdds.scm.HddsTestUtils;
 import org.apache.hadoop.hdds.scm.container.ContainerID;
 import org.apache.hadoop.hdds.scm.container.ContainerManager;
 import org.apache.hadoop.hdds.scm.container.ContainerNotFoundException;
 import org.apache.hadoop.hdds.scm.container.ContainerReplica;
 import org.apache.hadoop.hdds.scm.container.ContainerInfo;
 import org.apache.hadoop.hdds.scm.events.SCMEvents;
+import org.apache.hadoop.hdds.scm.ha.SCMContext;
 import org.apache.hadoop.hdds.scm.node.states.NodeNotFoundException;
 import org.apache.hadoop.hdds.scm.pipeline.PipelineManager;
 import org.apache.hadoop.hdds.scm.pipeline.PipelineProvider;
@@ -66,10 +67,10 @@ import org.apache.hadoop.security.authentication.client
     .AuthenticationException;
 import org.apache.ozone.test.GenericTestUtils;
 import org.apache.ozone.test.LambdaTestUtils;
-import org.junit.After;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Test;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
 import org.mockito.Mockito;
 
 /**
@@ -86,8 +87,9 @@ public class TestDeadNodeHandler {
   private EventPublisher publisher;
   private EventQueue eventQueue;
   private String storageDir;
+  private SCMContext scmContext;
 
-  @Before
+  @BeforeEach
   public void setup() throws IOException, AuthenticationException {
     OzoneConfiguration conf = new OzoneConfiguration();
     conf.setTimeDuration(HddsConfigKeys.HDDS_SCM_WAIT_TIME_AFTER_SAFE_MODE_EXIT,
@@ -99,10 +101,14 @@ public class TestDeadNodeHandler {
         TestDeadNodeHandler.class.getSimpleName() + UUID.randomUUID());
     conf.set(HddsConfigKeys.OZONE_METADATA_DIRS, storageDir);
     eventQueue = new EventQueue();
-    scm = TestUtils.getScm(conf);
+    scm = HddsTestUtils.getScm(conf);
     nodeManager = (SCMNodeManager) scm.getScmNodeManager();
+    scmContext = new SCMContext.Builder().setIsInSafeMode(true)
+        .setLeader(true).setIsPreCheckComplete(true)
+        .setSCM(scm).build();
     pipelineManager =
         (PipelineManagerImpl)scm.getPipelineManager();
+    pipelineManager.setScmContext(scmContext);
     PipelineProvider mockRatisProvider =
         new MockRatisPipelineProvider(nodeManager,
             pipelineManager.getStateManager(), conf);
@@ -113,12 +119,12 @@ public class TestDeadNodeHandler {
         Mockito.mock(PipelineManager.class), containerManager);
     healthyReadOnlyNodeHandler =
         new HealthyReadOnlyNodeHandler(nodeManager,
-            pipelineManager, conf);
+            pipelineManager);
     eventQueue.addHandler(SCMEvents.DEAD_NODE, deadNodeHandler);
     publisher = Mockito.mock(EventPublisher.class);
   }
 
-  @After
+  @AfterEach
   public void teardown() {
     scm.stop();
     scm.join();
@@ -137,11 +143,11 @@ public class TestDeadNodeHandler {
     String metaStoragePath = GenericTestUtils.getRandomizedTempPath()
         .concat("/metadata-" + datanode1.getUuidString());
 
-    StorageReportProto storageOne = TestUtils.createStorageReport(
+    StorageReportProto storageOne = HddsTestUtils.createStorageReport(
         datanode1.getUuid(), storagePath, 100 * OzoneConsts.TB,
         10 * OzoneConsts.TB, 90 * OzoneConsts.TB, null);
     MetadataStorageReportProto metaStorageOne =
-        TestUtils.createMetadataStorageReport(metaStoragePath,
+        HddsTestUtils.createMetadataStorageReport(metaStoragePath,
             100 * OzoneConsts.GB, 10 * OzoneConsts.GB,
             90 * OzoneConsts.GB, null);
 
@@ -154,48 +160,49 @@ public class TestDeadNodeHandler {
     // test case happy.
 
     nodeManager.register(datanode1,
-        TestUtils.createNodeReport(Arrays.asList(storageOne),
+        HddsTestUtils.createNodeReport(Arrays.asList(storageOne),
             Arrays.asList(metaStorageOne)), null);
     nodeManager.register(datanode2,
-        TestUtils.createNodeReport(Arrays.asList(storageOne),
+        HddsTestUtils.createNodeReport(Arrays.asList(storageOne),
             Arrays.asList(metaStorageOne)), null);
     nodeManager.register(datanode3,
-        TestUtils.createNodeReport(Arrays.asList(storageOne),
+        HddsTestUtils.createNodeReport(Arrays.asList(storageOne),
             Arrays.asList(metaStorageOne)), null);
 
     nodeManager.register(MockDatanodeDetails.randomDatanodeDetails(),
-        TestUtils.createNodeReport(Arrays.asList(storageOne),
+        HddsTestUtils.createNodeReport(Arrays.asList(storageOne),
             Arrays.asList(metaStorageOne)), null);
     nodeManager.register(MockDatanodeDetails.randomDatanodeDetails(),
-        TestUtils.createNodeReport(Arrays.asList(storageOne),
+        HddsTestUtils.createNodeReport(Arrays.asList(storageOne),
             Arrays.asList(metaStorageOne)), null);
     nodeManager.register(MockDatanodeDetails.randomDatanodeDetails(),
-        TestUtils.createNodeReport(Arrays.asList(storageOne),
+        HddsTestUtils.createNodeReport(Arrays.asList(storageOne),
             Arrays.asList(metaStorageOne)), null);
 
     nodeManager.register(MockDatanodeDetails.randomDatanodeDetails(),
-        TestUtils.createNodeReport(Arrays.asList(storageOne),
+        HddsTestUtils.createNodeReport(Arrays.asList(storageOne),
             Arrays.asList(metaStorageOne)), null);
     nodeManager.register(MockDatanodeDetails.randomDatanodeDetails(),
-        TestUtils.createNodeReport(Arrays.asList(storageOne),
+        HddsTestUtils.createNodeReport(Arrays.asList(storageOne),
             Arrays.asList(metaStorageOne)), null);
     nodeManager.register(MockDatanodeDetails.randomDatanodeDetails(),
-        TestUtils.createNodeReport(Arrays.asList(storageOne),
+        HddsTestUtils.createNodeReport(Arrays.asList(storageOne),
             Arrays.asList(metaStorageOne)), null);
 
     LambdaTestUtils.await(120000, 1000,
-        () -> pipelineManager.getPipelines(new RatisReplicationConfig(THREE))
+        () -> pipelineManager.getPipelines(RatisReplicationConfig
+                .getInstance(THREE))
             .size() > 3);
-    TestUtils.openAllRatisPipelines(pipelineManager);
+    HddsTestUtils.openAllRatisPipelines(pipelineManager);
 
     ContainerInfo container1 =
-        TestUtils.allocateContainer(containerManager);
+        HddsTestUtils.allocateContainer(containerManager);
     ContainerInfo container2 =
-        TestUtils.allocateContainer(containerManager);
+        HddsTestUtils.allocateContainer(containerManager);
     ContainerInfo container3 =
-        TestUtils.allocateContainer(containerManager);
+        HddsTestUtils.allocateContainer(containerManager);
     ContainerInfo container4 =
-        TestUtils.allocateContainer(containerManager);
+        HddsTestUtils.allocateContainer(containerManager);
 
     registerContainers(datanode1, container1, container2, container4);
     registerContainers(datanode2, container1, container2);
@@ -206,35 +213,36 @@ public class TestDeadNodeHandler {
     registerReplicas(containerManager, container3, datanode3);
     registerReplicas(containerManager, container4, datanode1);
 
-    TestUtils.closeContainer(containerManager, container1.containerID());
-    TestUtils.closeContainer(containerManager, container2.containerID());
-    TestUtils.quasiCloseContainer(containerManager, container3.containerID());
+    HddsTestUtils.closeContainer(containerManager, container1.containerID());
+    HddsTestUtils.closeContainer(containerManager, container2.containerID());
+    HddsTestUtils.quasiCloseContainer(containerManager,
+        container3.containerID());
 
     // First set the node to IN_MAINTENANCE and ensure the container replicas
     // are not removed on the dead event
     datanode1 = nodeManager.getNodeByUuid(datanode1.getUuidString());
-    Assert.assertTrue(
+    Assertions.assertTrue(
         nodeManager.getClusterNetworkTopologyMap().contains(datanode1));
     nodeManager.setNodeOperationalState(datanode1,
         HddsProtos.NodeOperationalState.IN_MAINTENANCE);
     deadNodeHandler.onMessage(datanode1, publisher);
     // make sure the node is removed from
     // ClusterNetworkTopology when it is considered as dead
-    Assert.assertFalse(
+    Assertions.assertFalse(
         nodeManager.getClusterNetworkTopologyMap().contains(datanode1));
 
     Set<ContainerReplica> container1Replicas = containerManager
         .getContainerReplicas(ContainerID.valueOf(container1.getContainerID()));
-    Assert.assertEquals(2, container1Replicas.size());
+    Assertions.assertEquals(2, container1Replicas.size());
 
     Set<ContainerReplica> container2Replicas = containerManager
         .getContainerReplicas(ContainerID.valueOf(container2.getContainerID()));
-    Assert.assertEquals(2, container2Replicas.size());
+    Assertions.assertEquals(2, container2Replicas.size());
 
     Set<ContainerReplica> container3Replicas = containerManager
             .getContainerReplicas(
                 ContainerID.valueOf(container3.getContainerID()));
-    Assert.assertEquals(1, container3Replicas.size());
+    Assertions.assertEquals(1, container3Replicas.size());
 
 
     // Now set the node to anything other than IN_MAINTENANCE and the relevant
@@ -244,30 +252,30 @@ public class TestDeadNodeHandler {
     deadNodeHandler.onMessage(datanode1, publisher);
     //datanode1 has been removed from ClusterNetworkTopology, another
     //deadNodeHandler.onMessage call will not change this
-    Assert.assertFalse(
+    Assertions.assertFalse(
         nodeManager.getClusterNetworkTopologyMap().contains(datanode1));
 
     container1Replicas = containerManager
         .getContainerReplicas(ContainerID.valueOf(container1.getContainerID()));
-    Assert.assertEquals(1, container1Replicas.size());
-    Assert.assertEquals(datanode2,
+    Assertions.assertEquals(1, container1Replicas.size());
+    Assertions.assertEquals(datanode2,
         container1Replicas.iterator().next().getDatanodeDetails());
 
     container2Replicas = containerManager
         .getContainerReplicas(ContainerID.valueOf(container2.getContainerID()));
-    Assert.assertEquals(1, container2Replicas.size());
-    Assert.assertEquals(datanode2,
+    Assertions.assertEquals(1, container2Replicas.size());
+    Assertions.assertEquals(datanode2,
         container2Replicas.iterator().next().getDatanodeDetails());
 
     container3Replicas = containerManager
         .getContainerReplicas(ContainerID.valueOf(container3.getContainerID()));
-    Assert.assertEquals(1, container3Replicas.size());
-    Assert.assertEquals(datanode3,
+    Assertions.assertEquals(1, container3Replicas.size());
+    Assertions.assertEquals(datanode3,
         container3Replicas.iterator().next().getDatanodeDetails());
 
     //datanode will be added back to ClusterNetworkTopology if it resurrects
     healthyReadOnlyNodeHandler.onMessage(datanode1, publisher);
-    Assert.assertTrue(
+    Assertions.assertTrue(
         nodeManager.getClusterNetworkTopologyMap().contains(datanode1));
 
   }
