@@ -22,7 +22,7 @@ import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.hdds.protocol.MockDatanodeDetails;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
-import org.apache.hadoop.hdds.scm.TestUtils;
+import org.apache.hadoop.hdds.scm.HddsTestUtils;
 import org.apache.hadoop.hdds.scm.DatanodeAdminError;
 import org.apache.hadoop.hdds.scm.ha.SCMContext;
 import org.apache.hadoop.hdds.scm.node.states.NodeNotFoundException;
@@ -30,16 +30,16 @@ import org.apache.hadoop.hdds.scm.server.StorageContainerManager;
 import org.apache.hadoop.hdds.server.events.EventQueue;
 import org.apache.hadoop.security.authentication.client.AuthenticationException;
 import org.apache.ozone.test.GenericTestUtils;
-import org.junit.Before;
-import org.junit.Test;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
 import java.io.IOException;
 import java.util.List;
 import java.util.UUID;
 import java.util.Arrays;
 import java.util.ArrayList;
-import static junit.framework.TestCase.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.assertj.core.api.Fail.fail;
-import static org.junit.Assert.assertNotEquals;
+import static org.junit.jupiter.api.Assertions.assertNotEquals;
 
 /**
  * Unit tests for the decommision manager.
@@ -53,7 +53,7 @@ public class TestNodeDecommissionManager {
   private OzoneConfiguration conf;
   private String storageDir;
 
-  @Before
+  @BeforeEach
   public void setup() throws Exception {
     conf = new OzoneConfiguration();
     storageDir = GenericTestUtils.getTempPath(
@@ -94,12 +94,12 @@ public class TestNodeDecommissionManager {
 
   @Test
   public void testAnyInvalidHostThrowsException()
-      throws InvalidHostStringException{
+      throws InvalidHostStringException {
     List<DatanodeDetails> dns = generateDatanodes();
 
     // Try to decommission a host that does exist, but give incorrect port
     try {
-      decom.decommissionNodes(Arrays.asList(dns.get(1).getIpAddress()+":10"));
+      decom.decommissionNodes(Arrays.asList(dns.get(1).getIpAddress() + ":10"));
       fail("InvalidHostStringException expected");
     } catch (InvalidHostStringException e) {
     }
@@ -131,7 +131,7 @@ public class TestNodeDecommissionManager {
     // that does not exist
     try {
       decom.decommissionNodes(Arrays.asList(
-          dns.get(0).getIpAddress()+":10"));
+          dns.get(0).getIpAddress() + ":10"));
       fail("InvalidHostStringException expected");
     } catch (InvalidHostStringException e) {
     }
@@ -159,7 +159,7 @@ public class TestNodeDecommissionManager {
     // and we hardcoded ports to 3456, 4567, 5678
     DatanodeDetails multiDn = dns.get(10);
     String multiAddr =
-        multiDn.getIpAddress()+":"+multiDn.getPorts().get(0).getValue();
+        multiDn.getIpAddress() + ":" + multiDn.getPorts().get(0).getValue();
     decom.decommissionNodes(Arrays.asList(multiAddr));
     assertEquals(HddsProtos.NodeOperationalState.DECOMMISSIONING,
         nodeManager.getNodeStatus(multiDn).getOperationalState());
@@ -202,7 +202,7 @@ public class TestNodeDecommissionManager {
     // and we hardcoded ports to 3456, 4567, 5678
     DatanodeDetails multiDn = dns.get(10);
     String multiAddr =
-        multiDn.getIpAddress()+":"+multiDn.getPorts().get(0).getValue();
+        multiDn.getIpAddress() + ":" + multiDn.getPorts().get(0).getValue();
     decom.startMaintenanceNodes(Arrays.asList(multiAddr), 100);
     assertEquals(HddsProtos.NodeOperationalState.ENTERING_MAINTENANCE,
         nodeManager.getNodeStatus(multiDn).getOperationalState());
@@ -282,7 +282,7 @@ public class TestNodeDecommissionManager {
 
   private SCMNodeManager createNodeManager(OzoneConfiguration config)
       throws IOException, AuthenticationException {
-    scm = TestUtils.getScm(config);
+    scm = HddsTestUtils.getScm(config);
     return (SCMNodeManager) scm.getScmNodeManager();
   }
 
@@ -296,7 +296,7 @@ public class TestNodeDecommissionManager {
    */
   private List<DatanodeDetails> generateDatanodes() {
     List<DatanodeDetails> dns = new ArrayList<>();
-    for (int i=0; i<10; i++) {
+    for (int i = 0; i < 10; i++) {
       DatanodeDetails dn = MockDatanodeDetails.randomDatanodeDetails();
       dns.add(dn);
       nodeManager.register(dn, null, null);
