@@ -29,10 +29,10 @@ import org.apache.hadoop.hdds.protocol.proto.HddsProtos.NodeState;
 import org.apache.hadoop.hdds.scm.container.ContainerID;
 import org.apache.hadoop.hdds.scm.node.NodeStatus;
 
-import static junit.framework.TestCase.assertEquals;
-import org.junit.After;
-import org.junit.Before;
-import org.junit.Test;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
 
 /**
  * Class to test the NodeStateMap class, which is an internal class used by
@@ -43,12 +43,12 @@ public class TestNodeStateMap {
 
   private NodeStateMap map;
 
-  @Before
+  @BeforeEach
   public void setUp() {
     map = new NodeStateMap();
   }
 
-  @After
+  @AfterEach
   public void tearDown() {
   }
 
@@ -98,8 +98,8 @@ public class TestNodeStateMap {
       throws NodeAlreadyExistsException {
     // Add one node for all possible states
     int nodeCount = 0;
-    for(NodeOperationalState op : NodeOperationalState.values()) {
-      for(NodeState health : NodeState.values()) {
+    for (NodeOperationalState op : NodeOperationalState.values()) {
+      for (NodeState health : NodeState.values()) {
         addRandomNodeWithState(op, health);
         nodeCount++;
       }
