@@ -27,31 +27,29 @@ import org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationFactor;
 import org.apache.hadoop.hdds.scm.container.MockNodeManager;
 import org.apache.hadoop.hdds.scm.container.TestContainerManagerImpl;
 import org.apache.hadoop.hdds.scm.exceptions.SCMException;
-import org.apache.hadoop.hdds.scm.ha.MockSCMHAManager;
+import org.apache.hadoop.hdds.scm.ha.SCMHAManagerStub;
 import org.apache.hadoop.hdds.scm.ha.SCMHAManager;
 import org.apache.hadoop.hdds.scm.metadata.SCMDBDefinition;
 import org.apache.hadoop.hdds.scm.node.NodeManager;
 import org.apache.hadoop.hdds.scm.node.NodeStatus;
 import org.apache.hadoop.hdds.utils.db.DBStore;
 import org.apache.hadoop.hdds.utils.db.DBStoreBuilder;
-import org.apache.hadoop.ozone.ClientVersions;
+import org.apache.hadoop.ozone.ClientVersion;
 import org.apache.hadoop.ozone.container.common.SCMTestUtils;
 import org.apache.ozone.test.GenericTestUtils;
-import org.junit.After;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Test;
-import org.junit.runner.RunWith;
-import org.junit.runners.Parameterized;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.CsvSource;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import java.io.File;
 import java.io.IOException;
-import java.util.Arrays;
-import java.util.Collection;
 import java.util.List;
 import java.util.UUID;
+import java.util.concurrent.TimeoutException;
 
 import static org.apache.hadoop.hdds.scm.ScmConfigKeys.OZONE_DATANODE_PIPELINE_LIMIT;
 import static org.apache.hadoop.hdds.scm.ScmConfigKeys.OZONE_SCM_PIPELINE_AUTO_CREATE_FACTOR_ONE;
@@ -59,19 +57,16 @@ import static org.apache.hadoop.hdds.scm.ScmConfigKeys.OZONE_SCM_PIPELINE_AUTO_C
 /**
  * Test for pipeline datanodes intersection.
  */
-@RunWith(Parameterized.class)
 public class TestPipelineDatanodesIntersection {
   private static final Logger LOG = LoggerFactory
       .getLogger(TestPipelineDatanodesIntersection.class.getName());
 
-  private int nodeCount;
-  private int nodeHeaviness;
   private OzoneConfiguration conf;
   private boolean end;
   private File testDir;
   private DBStore dbStore;
 
-  @Before
+  @BeforeEach
   public void initialize() throws IOException {
     conf = SCMTestUtils.getConf();
     end = false;
@@ -82,7 +77,7 @@ public class TestPipelineDatanodesIntersection {
         conf, new SCMDBDefinition());
   }
 
-  @After
+  @AfterEach
   public void cleanup() throws Exception {
     if (dbStore != null) {
       dbStore.close();
@@ -91,29 +86,21 @@ public class TestPipelineDatanodesIntersection {
     FileUtil.fullyDelete(testDir);
   }
 
-  public TestPipelineDatanodesIntersection(int nodeCount, int nodeHeaviness) {
-    this.nodeCount = nodeCount;
-    this.nodeHeaviness = nodeHeaviness;
-  }
-
-  @Parameterized.Parameters
-  public static Collection inputParams() {
-    return Arrays.asList(new Object[][] {
-        {4, 5},
-        {10, 5},
-        {20, 5},
-        {50, 5},
-        {100, 5},
-        {100, 10}
-    });
-  }
-
-  @Test
-  public void testPipelineDatanodesIntersection() throws IOException {
-    NodeManager nodeManager= new MockNodeManager(true, nodeCount);
+  @ParameterizedTest
+  @CsvSource({
+      "4, 5",
+      "10, 5",
+      "20, 5",
+      "50, 5",
+      "100, 5",
+      "100, 10"
+  })
+  public void testPipelineDatanodesIntersection(int nodeCount,
+      int nodeHeaviness) throws IOException {
+    NodeManager nodeManager = new MockNodeManager(true, nodeCount);
     conf.setInt(OZONE_DATANODE_PIPELINE_LIMIT, nodeHeaviness);
     conf.setBoolean(OZONE_SCM_PIPELINE_AUTO_CREATE_FACTOR_ONE, false);
-    SCMHAManager scmhaManager = MockSCMHAManager.getInstance(true);
+    SCMHAManager scmhaManager = SCMHAManagerStub.getInstance(true);
 
     PipelineStateManager stateManager = PipelineStateManagerImpl.newBuilder()
         .setPipelineStore(SCMDBDefinition.PIPELINES.getTable(dbStore))
@@ -123,8 +110,8 @@ public class TestPipelineDatanodesIntersection {
         .build();
 
 
-    PipelineProvider provider = new MockRatisPipelineProvider(nodeManager,
-        stateManager, conf);
+    PipelineProvider<RatisReplicationConfig> provider =
+        new MockRatisPipelineProvider(nodeManager, stateManager, conf);
 
     int healthyNodeCount = nodeManager
         .getNodeCount(NodeStatus.inServiceHealthy());
@@ -132,16 +119,16 @@ public class TestPipelineDatanodesIntersection {
     int createdPipelineCount = 0;
     while (!end && createdPipelineCount <= healthyNodeCount * nodeHeaviness) {
       try {
-        Pipeline pipeline = provider.create(new RatisReplicationConfig(
+        Pipeline pipeline = provider.create(RatisReplicationConfig.getInstance(
             ReplicationFactor.THREE));
         HddsProtos.Pipeline pipelineProto = pipeline.getProtobufMessage(
-            ClientVersions.CURRENT_VERSION);
+            ClientVersion.CURRENT_VERSION);
         stateManager.addPipeline(pipelineProto);
         nodeManager.addPipeline(pipeline);
         List<Pipeline> overlapPipelines = RatisPipelineUtils
             .checkPipelineContainSameDatanodes(stateManager, pipeline);
 
-        if (overlapPipelines.isEmpty()) {
+        if (!overlapPipelines.isEmpty()) {
           intersectionCount++;
           for (Pipeline overlapPipeline : overlapPipelines) {
             LOG.info("This pipeline: " + pipeline.getId().toString() +
@@ -157,12 +144,12 @@ public class TestPipelineDatanodesIntersection {
           }
         }
         createdPipelineCount++;
-      } catch(SCMException e) {
+      } catch (SCMException e) {
         end = true;
-      } catch (IOException e) {
+      } catch (IOException | TimeoutException e) {
         end = true;
         // Should not throw regular IOException.
-        Assert.fail();
+        Assertions.fail();
       }
     }
 
@@ -170,7 +157,8 @@ public class TestPipelineDatanodesIntersection {
 
     LOG.info("Among total " +
         stateManager
-            .getPipelines(new RatisReplicationConfig(ReplicationFactor.THREE))
+            .getPipelines(RatisReplicationConfig
+                .getInstance(ReplicationFactor.THREE))
             .size() + " created pipelines" +
         " with " + healthyNodeCount + " healthy datanodes and " +
         nodeHeaviness + " as node heaviness, " +
