@@ -31,20 +31,20 @@ import org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationType;
 import org.apache.hadoop.hdds.scm.container.ContainerID;
 import org.apache.hadoop.hdds.scm.container.MockNodeManager;
 import org.apache.hadoop.hdds.scm.container.TestContainerManagerImpl;
-import org.apache.hadoop.hdds.scm.ha.MockSCMHAManager;
+import org.apache.hadoop.hdds.scm.ha.SCMHAManagerStub;
 import org.apache.hadoop.hdds.scm.ha.SCMHAManager;
 import org.apache.hadoop.hdds.scm.metadata.SCMDBDefinition;
 import org.apache.hadoop.hdds.scm.node.NodeManager;
 import org.apache.hadoop.hdds.utils.db.DBStore;
 import org.apache.hadoop.hdds.utils.db.DBStoreBuilder;
-import org.apache.hadoop.ozone.ClientVersions;
+import org.apache.hadoop.ozone.ClientVersion;
 import org.apache.hadoop.ozone.container.common.SCMTestUtils;
 import org.apache.ozone.test.GenericTestUtils;
 import org.apache.ratis.protocol.exceptions.StateMachineException;
-import org.junit.After;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Test;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
 
 import java.io.File;
 import java.io.IOException;
@@ -53,6 +53,7 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
 import java.util.UUID;
+import java.util.concurrent.TimeoutException;
 
 /**
  * Test for PipelineStateManagerImpl.
@@ -63,7 +64,7 @@ public class TestPipelineStateManagerImpl {
   private File testDir;
   private DBStore dbStore;
 
-  @Before
+  @BeforeEach
   public void init() throws Exception {
     final OzoneConfiguration conf = SCMTestUtils.getConf();
     testDir = GenericTestUtils.getTestDir(
@@ -72,7 +73,7 @@ public class TestPipelineStateManagerImpl {
     dbStore = DBStoreBuilder.createDBStore(
         conf, new SCMDBDefinition());
 
-    SCMHAManager scmhaManager = MockSCMHAManager.getInstance(true);
+    SCMHAManager scmhaManager = SCMHAManagerStub.getInstance(true);
     NodeManager nodeManager = new MockNodeManager(true, 10);
 
     stateManager = PipelineStateManagerImpl.newBuilder()
@@ -83,7 +84,7 @@ public class TestPipelineStateManagerImpl {
         .build();
   }
 
-  @After
+  @AfterEach
   public void cleanup() throws Exception {
     if (dbStore != null) {
       dbStore.close();
@@ -104,7 +105,8 @@ public class TestPipelineStateManagerImpl {
       nodes.add(MockDatanodeDetails.randomDatanodeDetails());
     }
     return Pipeline.newBuilder()
-        .setReplicationConfig(ReplicationConfig.fromTypeAndFactor(type, factor))
+        .setReplicationConfig(
+            ReplicationConfig.fromProtoTypeAndFactor(type, factor))
         .setNodes(nodes)
         .setState(Pipeline.PipelineState.ALLOCATED)
         .setId(PipelineID.randomId())
@@ -112,35 +114,34 @@ public class TestPipelineStateManagerImpl {
   }
 
   @Test
-  public void testAddAndGetPipeline() throws IOException {
+  public void testAddAndGetPipeline() throws IOException, TimeoutException {
     Pipeline pipeline = createDummyPipeline(0);
     HddsProtos.Pipeline pipelineProto = pipeline.getProtobufMessage(
-        ClientVersions.CURRENT_VERSION);
+        ClientVersion.CURRENT_VERSION);
     try {
       stateManager.addPipeline(pipelineProto);
-      Assert.fail("Pipeline should not have been added");
+      Assertions.fail("Pipeline should not have been added");
     } catch (StateMachineException e) {
       // replication factor and number of nodes in the pipeline do not match
-      Assert.assertTrue(e.getMessage().contains("do not match"));
+      Assertions.assertTrue(e.getMessage().contains("do not match"));
     }
 
     // add a pipeline
     pipeline = createDummyPipeline(1);
-    pipelineProto = pipeline.getProtobufMessage(
-        ClientVersions.CURRENT_VERSION);
+    pipelineProto = pipeline.getProtobufMessage(ClientVersion.CURRENT_VERSION);
     stateManager.addPipeline(pipelineProto);
 
     try {
       stateManager.addPipeline(pipelineProto);
-      Assert.fail("Pipeline should not have been added");
+      Assertions.fail("Pipeline should not have been added");
     } catch (IOException e) {
       // Can not add a pipeline twice
-      Assert.assertTrue(e.getMessage().contains("Duplicate pipeline ID"));
+      Assertions.assertTrue(e.getMessage().contains("Duplicate pipeline ID"));
     }
 
     // verify pipeline returned is same
     Pipeline pipeline1 = stateManager.getPipeline(pipeline.getId());
-    Assert.assertTrue(pipeline.getId().equals(pipeline1.getId()));
+    Assertions.assertTrue(pipeline.getId().equals(pipeline1.getId()));
 
     // clean up
     finalizePipeline(pipelineProto);
@@ -148,26 +149,27 @@ public class TestPipelineStateManagerImpl {
   }
 
   @Test
-  public void testGetPipelines() throws IOException {
+  public void testGetPipelines() throws IOException, TimeoutException {
     // In start there should be no pipelines
-    Assert.assertTrue(stateManager.getPipelines().isEmpty());
+    Assertions.assertTrue(stateManager.getPipelines().isEmpty());
 
     Set<HddsProtos.Pipeline> pipelines = new HashSet<>();
     HddsProtos.Pipeline pipeline = createDummyPipeline(1).getProtobufMessage(
-        ClientVersions.CURRENT_VERSION);
+        ClientVersion.CURRENT_VERSION);
     stateManager.addPipeline(pipeline);
     pipelines.add(pipeline);
     pipeline = createDummyPipeline(1).getProtobufMessage(
-        ClientVersions.CURRENT_VERSION);
+        ClientVersion.CURRENT_VERSION);
     stateManager.addPipeline(pipeline);
     pipelines.add(pipeline);
 
     Set<Pipeline> pipelines1 = new HashSet<>(stateManager
-        .getPipelines(new RatisReplicationConfig(ReplicationFactor.ONE)));
-    Assert.assertEquals(pipelines1.size(), pipelines.size());
+        .getPipelines(RatisReplicationConfig
+            .getInstance(ReplicationFactor.ONE)));
+    Assertions.assertEquals(pipelines1.size(), pipelines.size());
 
     pipelines1 = new HashSet<>(stateManager.getPipelines());
-    Assert.assertEquals(pipelines1.size(), pipelines.size());
+    Assertions.assertEquals(pipelines1.size(), pipelines.size());
 
     // clean up
     for (HddsProtos.Pipeline pipeline1 : pipelines) {
@@ -177,7 +179,8 @@ public class TestPipelineStateManagerImpl {
   }
 
   @Test
-  public void testGetPipelinesByTypeAndFactor() throws IOException {
+  public void testGetPipelinesByTypeAndFactor()
+      throws IOException, TimeoutException {
     Set<HddsProtos.Pipeline> pipelines = new HashSet<>();
     for (HddsProtos.ReplicationType type : new ReplicationType[] {
         ReplicationType.RATIS, ReplicationType.STAND_ALONE}) {
@@ -187,19 +190,19 @@ public class TestPipelineStateManagerImpl {
           // 5 pipelines in allocated state for each type and factor
           HddsProtos.Pipeline pipeline =
               createDummyPipeline(type, factor, factor.getNumber())
-                  .getProtobufMessage(ClientVersions.CURRENT_VERSION);
+                  .getProtobufMessage(ClientVersion.CURRENT_VERSION);
           stateManager.addPipeline(pipeline);
           pipelines.add(pipeline);
 
           // 5 pipelines in open state for each type and factor
           pipeline = createDummyPipeline(type, factor, factor.getNumber())
-              .getProtobufMessage(ClientVersions.CURRENT_VERSION);
+              .getProtobufMessage(ClientVersion.CURRENT_VERSION);
           stateManager.addPipeline(pipeline);
           pipelines.add(pipeline);
 
           // 5 pipelines in closed state for each type and factor
           pipeline = createDummyPipeline(type, factor, factor.getNumber())
-              .getProtobufMessage(ClientVersions.CURRENT_VERSION);
+              .getProtobufMessage(ClientVersion.CURRENT_VERSION);
           stateManager.addPipeline(pipeline);
           pipelines.add(pipeline);
         }
@@ -213,10 +216,10 @@ public class TestPipelineStateManagerImpl {
         // verify pipelines received
         List<Pipeline> pipelines1 =
             stateManager.getPipelines(
-                ReplicationConfig.fromTypeAndFactor(type, factor));
-        Assert.assertEquals(15, pipelines1.size());
+                ReplicationConfig.fromProtoTypeAndFactor(type, factor));
+        Assertions.assertEquals(15, pipelines1.size());
         pipelines1.stream().forEach(p -> {
-          Assert.assertEquals(type, p.getType());
+          Assertions.assertEquals(type, p.getType());
         });
       }
     }
@@ -229,7 +232,8 @@ public class TestPipelineStateManagerImpl {
   }
 
   @Test
-  public void testGetPipelinesByTypeFactorAndState() throws IOException {
+  public void testGetPipelinesByTypeFactorAndState()
+      throws IOException, TimeoutException {
     Set<HddsProtos.Pipeline> pipelines = new HashSet<>();
     for (HddsProtos.ReplicationType type : new ReplicationType[] {
         ReplicationType.RATIS, ReplicationType.STAND_ALONE}) {
@@ -239,20 +243,20 @@ public class TestPipelineStateManagerImpl {
           // 5 pipelines in allocated state for each type and factor
           HddsProtos.Pipeline pipeline =
               createDummyPipeline(type, factor, factor.getNumber())
-                  .getProtobufMessage(ClientVersions.CURRENT_VERSION);
+                  .getProtobufMessage(ClientVersion.CURRENT_VERSION);
           stateManager.addPipeline(pipeline);
           pipelines.add(pipeline);
 
           // 5 pipelines in open state for each type and factor
           pipeline = createDummyPipeline(type, factor, factor.getNumber())
-              .getProtobufMessage(ClientVersions.CURRENT_VERSION);
+              .getProtobufMessage(ClientVersion.CURRENT_VERSION);
           stateManager.addPipeline(pipeline);
           openPipeline(pipeline);
           pipelines.add(pipeline);
 
           // 5 pipelines in dormant state for each type and factor
           pipeline = createDummyPipeline(type, factor, factor.getNumber())
-              .getProtobufMessage(ClientVersions.CURRENT_VERSION);
+              .getProtobufMessage(ClientVersion.CURRENT_VERSION);
           stateManager.addPipeline(pipeline);
           openPipeline(pipeline);
           deactivatePipeline(pipeline);
@@ -260,7 +264,7 @@ public class TestPipelineStateManagerImpl {
 
           // 5 pipelines in closed state for each type and factor
           pipeline = createDummyPipeline(type, factor, factor.getNumber())
-              .getProtobufMessage(ClientVersions.CURRENT_VERSION);
+              .getProtobufMessage(ClientVersion.CURRENT_VERSION);
           stateManager.addPipeline(pipeline);
           finalizePipeline(pipeline);
           pipelines.add(pipeline);
@@ -276,11 +280,12 @@ public class TestPipelineStateManagerImpl {
           // verify pipelines received
           List<Pipeline> pipelines1 =
               stateManager.getPipelines(
-                  ReplicationConfig.fromTypeAndFactor(type, factor), state);
-          Assert.assertEquals(5, pipelines1.size());
+                  ReplicationConfig.fromProtoTypeAndFactor(type, factor),
+                  state);
+          Assertions.assertEquals(5, pipelines1.size());
           pipelines1.forEach(p -> {
-            Assert.assertEquals(type, p.getType());
-            Assert.assertEquals(state, p.getPipelineState());
+            Assertions.assertEquals(type, p.getType());
+            Assertions.assertEquals(state, p.getPipelineState());
           });
         }
       }
@@ -294,11 +299,11 @@ public class TestPipelineStateManagerImpl {
   }
 
   @Test
-  public void testAddAndGetContainer() throws IOException {
+  public void testAddAndGetContainer() throws IOException, TimeoutException {
     long containerID = 0;
     Pipeline pipeline = createDummyPipeline(1);
     HddsProtos.Pipeline pipelineProto = pipeline
-        .getProtobufMessage(ClientVersions.CURRENT_VERSION);
+        .getProtobufMessage(ClientVersion.CURRENT_VERSION);
     stateManager.addPipeline(pipelineProto);
     pipeline = stateManager.getPipeline(pipeline.getId());
     stateManager.addContainerToPipeline(pipeline.getId(),
@@ -314,25 +319,25 @@ public class TestPipelineStateManagerImpl {
     //verify the number of containers returned
     Set<ContainerID> containerIDs =
         stateManager.getContainers(pipeline.getId());
-    Assert.assertEquals(containerIDs.size(), containerID);
+    Assertions.assertEquals(containerIDs.size(), containerID);
 
     finalizePipeline(pipelineProto);
     removePipeline(pipelineProto);
     try {
       stateManager.addContainerToPipeline(pipeline.getId(),
           ContainerID.valueOf(++containerID));
-      Assert.fail("Container should not have been added");
+      Assertions.fail("Container should not have been added");
     } catch (IOException e) {
       // Can not add a container to removed pipeline
-      Assert.assertTrue(e.getMessage().contains("not found"));
+      Assertions.assertTrue(e.getMessage().contains("not found"));
     }
   }
 
   @Test
-  public void testRemovePipeline() throws IOException {
+  public void testRemovePipeline() throws IOException, TimeoutException {
     Pipeline pipeline = createDummyPipeline(1);
     HddsProtos.Pipeline pipelineProto = pipeline
-        .getProtobufMessage(ClientVersions.CURRENT_VERSION);
+        .getProtobufMessage(ClientVersion.CURRENT_VERSION);
     stateManager.addPipeline(pipelineProto);
     // close the pipeline
     openPipeline(pipelineProto);
@@ -341,10 +346,10 @@ public class TestPipelineStateManagerImpl {
 
     try {
       removePipeline(pipelineProto);
-      Assert.fail("Pipeline should not have been removed");
+      Assertions.fail("Pipeline should not have been removed");
     } catch (IOException e) {
       // can not remove a pipeline which already has containers
-      Assert.assertTrue(e.getMessage().contains("not yet closed"));
+      Assertions.assertTrue(e.getMessage().contains("not yet closed"));
     }
 
     // close the pipeline
@@ -354,28 +359,31 @@ public class TestPipelineStateManagerImpl {
   }
 
   @Test
-  public void testRemoveContainer() throws IOException {
+  public void testRemoveContainer() throws IOException, TimeoutException {
     long containerID = 1;
     Pipeline pipeline = createDummyPipeline(1);
     HddsProtos.Pipeline pipelineProto = pipeline
-        .getProtobufMessage(ClientVersions.CURRENT_VERSION);
+        .getProtobufMessage(ClientVersion.CURRENT_VERSION);
     // create an open pipeline in stateMap
     stateManager.addPipeline(pipelineProto);
     openPipeline(pipelineProto);
 
     stateManager.addContainerToPipeline(pipeline.getId(),
         ContainerID.valueOf(containerID));
-    Assert.assertEquals(1, stateManager.getContainers(pipeline.getId()).size());
+    Assertions.assertEquals(1,
+        stateManager.getContainers(pipeline.getId()).size());
     stateManager.removeContainerFromPipeline(pipeline.getId(),
         ContainerID.valueOf(containerID));
-    Assert.assertEquals(0, stateManager.getContainers(pipeline.getId()).size());
+    Assertions.assertEquals(0,
+        stateManager.getContainers(pipeline.getId()).size());
 
     // add two containers in the pipeline
     stateManager.addContainerToPipeline(pipeline.getId(),
         ContainerID.valueOf(++containerID));
     stateManager.addContainerToPipeline(pipeline.getId(),
         ContainerID.valueOf(++containerID));
-    Assert.assertEquals(2, stateManager.getContainers(pipeline.getId()).size());
+    Assertions.assertEquals(2,
+        stateManager.getContainers(pipeline.getId()).size());
 
     // move pipeline to closing state
     finalizePipeline(pipelineProto);
@@ -384,65 +392,66 @@ public class TestPipelineStateManagerImpl {
         ContainerID.valueOf(containerID));
     stateManager.removeContainerFromPipeline(pipeline.getId(),
         ContainerID.valueOf(--containerID));
-    Assert.assertEquals(0, stateManager.getContainers(pipeline.getId()).size());
+    Assertions.assertEquals(0,
+        stateManager.getContainers(pipeline.getId()).size());
 
     // clean up
     removePipeline(pipelineProto);
   }
 
   @Test
-  public void testFinalizePipeline() throws IOException {
+  public void testFinalizePipeline() throws IOException, TimeoutException {
     Pipeline pipeline = createDummyPipeline(1);
     HddsProtos.Pipeline pipelineProto = pipeline
-        .getProtobufMessage(ClientVersions.CURRENT_VERSION);
+        .getProtobufMessage(ClientVersion.CURRENT_VERSION);
     stateManager.addPipeline(pipelineProto);
     // finalize on ALLOCATED pipeline
     finalizePipeline(pipelineProto);
-    Assert.assertEquals(Pipeline.PipelineState.CLOSED,
+    Assertions.assertEquals(Pipeline.PipelineState.CLOSED,
         stateManager.getPipeline(pipeline.getId()).getPipelineState());
     // clean up
     removePipeline(pipelineProto);
 
     pipeline = createDummyPipeline(1);
     pipelineProto = pipeline
-        .getProtobufMessage(ClientVersions.CURRENT_VERSION);
+        .getProtobufMessage(ClientVersion.CURRENT_VERSION);
     stateManager.addPipeline(pipelineProto);
     openPipeline(pipelineProto);
     // finalize on OPEN pipeline
     finalizePipeline(pipelineProto);
-    Assert.assertEquals(Pipeline.PipelineState.CLOSED,
+    Assertions.assertEquals(Pipeline.PipelineState.CLOSED,
         stateManager.getPipeline(pipeline.getId()).getPipelineState());
     // clean up
     removePipeline(pipelineProto);
 
     pipeline = createDummyPipeline(1);
     pipelineProto = pipeline
-        .getProtobufMessage(ClientVersions.CURRENT_VERSION);
+        .getProtobufMessage(ClientVersion.CURRENT_VERSION);
     stateManager.addPipeline(pipelineProto);
     openPipeline(pipelineProto);
     finalizePipeline(pipelineProto);
     // finalize should work on already closed pipeline
     finalizePipeline(pipelineProto);
-    Assert.assertEquals(Pipeline.PipelineState.CLOSED,
+    Assertions.assertEquals(Pipeline.PipelineState.CLOSED,
         stateManager.getPipeline(pipeline.getId()).getPipelineState());
     // clean up
     removePipeline(pipelineProto);
   }
 
   @Test
-  public void testOpenPipeline() throws IOException {
+  public void testOpenPipeline() throws IOException, TimeoutException {
     Pipeline pipeline = createDummyPipeline(1);
     HddsProtos.Pipeline pipelineProto = pipeline
-        .getProtobufMessage(ClientVersions.CURRENT_VERSION);
+        .getProtobufMessage(ClientVersion.CURRENT_VERSION);
     stateManager.addPipeline(pipelineProto);
     // open on ALLOCATED pipeline
     openPipeline(pipelineProto);
-    Assert.assertEquals(Pipeline.PipelineState.OPEN,
+    Assertions.assertEquals(Pipeline.PipelineState.OPEN,
         stateManager.getPipeline(pipeline.getId()).getPipelineState());
 
     openPipeline(pipelineProto);
     // open should work on already open pipeline
-    Assert.assertEquals(Pipeline.PipelineState.OPEN,
+    Assertions.assertEquals(Pipeline.PipelineState.OPEN,
         stateManager.getPipeline(pipeline.getId()).getPipelineState());
     // clean up
     finalizePipeline(pipelineProto);
@@ -450,22 +459,24 @@ public class TestPipelineStateManagerImpl {
   }
 
   @Test
-  public void testQueryPipeline() throws IOException {
+  public void testQueryPipeline() throws IOException, TimeoutException {
     Pipeline pipeline = createDummyPipeline(HddsProtos.ReplicationType.RATIS,
         HddsProtos.ReplicationFactor.THREE, 3);
     // pipeline in allocated state should not be reported
     HddsProtos.Pipeline pipelineProto = pipeline
-        .getProtobufMessage(ClientVersions.CURRENT_VERSION);
+        .getProtobufMessage(ClientVersion.CURRENT_VERSION);
     stateManager.addPipeline(pipelineProto);
-    Assert.assertEquals(0, stateManager
-        .getPipelines(new RatisReplicationConfig(ReplicationFactor.THREE),
+    Assertions.assertEquals(0, stateManager
+        .getPipelines(RatisReplicationConfig
+            .getInstance(ReplicationFactor.THREE),
             Pipeline.PipelineState.OPEN)
         .size());
 
     // pipeline in open state should be reported
     openPipeline(pipelineProto);
-    Assert.assertEquals(1, stateManager
-        .getPipelines(new RatisReplicationConfig(ReplicationFactor.THREE),
+    Assertions.assertEquals(1, stateManager
+        .getPipelines(RatisReplicationConfig
+            .getInstance(ReplicationFactor.THREE),
             Pipeline.PipelineState.OPEN)
         .size());
 
@@ -475,18 +486,20 @@ public class TestPipelineStateManagerImpl {
         .setState(Pipeline.PipelineState.OPEN)
         .build();
     HddsProtos.Pipeline pipelineProto2 = pipeline2
-        .getProtobufMessage(ClientVersions.CURRENT_VERSION);
+        .getProtobufMessage(ClientVersion.CURRENT_VERSION);
     // pipeline in open state should be reported
     stateManager.addPipeline(pipelineProto2);
-    Assert.assertEquals(2, stateManager
-        .getPipelines(new RatisReplicationConfig(ReplicationFactor.THREE),
+    Assertions.assertEquals(2, stateManager
+        .getPipelines(RatisReplicationConfig
+            .getInstance(ReplicationFactor.THREE),
             Pipeline.PipelineState.OPEN)
         .size());
 
     // pipeline in closed state should not be reported
     finalizePipeline(pipelineProto2);
-    Assert.assertEquals(1, stateManager
-        .getPipelines(new RatisReplicationConfig(ReplicationFactor.THREE),
+    Assertions.assertEquals(1, stateManager
+        .getPipelines(RatisReplicationConfig
+            .getInstance(ReplicationFactor.THREE),
             Pipeline.PipelineState.OPEN)
         .size());
 
@@ -497,23 +510,25 @@ public class TestPipelineStateManagerImpl {
     removePipeline(pipelineProto2);
   }
 
-  private void removePipeline(HddsProtos.Pipeline pipeline) throws IOException {
+  private void removePipeline(HddsProtos.Pipeline pipeline)
+      throws IOException, TimeoutException {
     stateManager.removePipeline(pipeline.getId());
   }
 
-  private void openPipeline(HddsProtos.Pipeline pipeline) throws IOException {
+  private void openPipeline(HddsProtos.Pipeline pipeline)
+      throws IOException, TimeoutException {
     stateManager.updatePipelineState(pipeline.getId(),
         HddsProtos.PipelineState.PIPELINE_OPEN);
   }
 
   private void finalizePipeline(HddsProtos.Pipeline pipeline)
-      throws IOException {
+      throws IOException, TimeoutException {
     stateManager.updatePipelineState(pipeline.getId(),
         HddsProtos.PipelineState.PIPELINE_CLOSED);
   }
 
   private void deactivatePipeline(HddsProtos.Pipeline pipeline)
-      throws IOException {
+      throws IOException, TimeoutException {
     stateManager.updatePipelineState(pipeline.getId(),
         HddsProtos.PipelineState.PIPELINE_DORMANT);
   }
