@@ -19,6 +19,8 @@ package org.apache.hadoop.hdds.scm.safemode;
 
 import java.io.File;
 import java.io.IOException;
+import java.nio.file.Path;
+import java.time.ZoneOffset;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
@@ -39,7 +41,7 @@ import org.apache.hadoop.hdds.scm.container.ContainerInfo;
 import org.apache.hadoop.hdds.scm.container.MockNodeManager;
 import org.apache.hadoop.hdds.scm.events.SCMEvents;
 import org.apache.hadoop.hdds.scm.exceptions.SCMException;
-import org.apache.hadoop.hdds.scm.ha.MockSCMHAManager;
+import org.apache.hadoop.hdds.scm.ha.SCMHAManagerStub;
 import org.apache.hadoop.hdds.scm.ha.SCMContext;
 import org.apache.hadoop.hdds.scm.ha.SCMServiceManager;
 import org.apache.hadoop.hdds.scm.metadata.SCMMetadataStore;
@@ -53,23 +55,25 @@ import org.apache.hadoop.hdds.scm.server.SCMDatanodeHeartbeatDispatcher;
 import org.apache.hadoop.hdds.server.events.EventHandler;
 import org.apache.hadoop.hdds.server.events.EventPublisher;
 import org.apache.hadoop.hdds.server.events.EventQueue;
+import org.apache.hadoop.ozone.common.MonotonicClock;
 import org.apache.ozone.test.GenericTestUtils;
 
-import org.junit.After;
-import org.junit.Assert;
-import static org.junit.Assert.assertFalse;
-import static org.junit.Assert.assertTrue;
-import static org.junit.Assert.fail;
-import org.junit.Before;
-import org.junit.Ignore;
-import org.junit.Rule;
-import org.junit.Test;
-import org.junit.rules.TemporaryFolder;
-import org.junit.rules.Timeout;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Disabled;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.Timeout;
+import org.junit.jupiter.api.io.TempDir;
 import org.mockito.Mockito;
 
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.junit.jupiter.api.Assertions.fail;
+
 /** Test class for SCMSafeModeManager.
  */
+@Timeout(300)
 public class TestSCMSafeModeManager {
 
   private EventQueue queue;
@@ -79,32 +83,22 @@ public class TestSCMSafeModeManager {
   private OzoneConfiguration config;
   private List<ContainerInfo> containers = Collections.emptyList();
 
-  @Rule
-  public Timeout timeout = Timeout.seconds(300);
-
-  @Rule
-  public final TemporaryFolder tempDir = new TemporaryFolder();
-
   private SCMMetadataStore scmMetadataStore;
 
-  @Before
-  public void setUp() {
+  @BeforeEach
+  public void setUp(@TempDir Path tempDir) throws IOException {
     queue = new EventQueue();
     scmContext = SCMContext.emptyContext();
     serviceManager = new SCMServiceManager();
     config = new OzoneConfiguration();
     config.setBoolean(HddsConfigKeys.HDDS_SCM_SAFEMODE_PIPELINE_CREATION,
         false);
-  }
-
-  @Before
-  public void initDbStore() throws IOException {
     config.set(HddsConfigKeys.OZONE_METADATA_DIRS,
-        tempDir.newFolder().getAbsolutePath());
+        tempDir.toAbsolutePath().toString());
     scmMetadataStore = new SCMMetadataStoreImpl(config);
   }
 
-  @After
+  @AfterEach
   public void destroyDbStore() throws Exception {
     if (scmMetadataStore.getStore() != null) {
       scmMetadataStore.getStore().close();
@@ -130,7 +124,7 @@ public class TestSCMSafeModeManager {
     containers = new ArrayList<>();
     containers.addAll(HddsTestUtils.getContainerInfo(numContainers));
 
-    // Currently only considered containers which are not in open state.
+    // Currently, only considered containers which are not in open state.
     for (ContainerInfo container : containers) {
       container.setState(HddsProtos.LifeCycleState.CLOSED);
     }
@@ -146,14 +140,13 @@ public class TestSCMSafeModeManager {
         HddsConfigKeys.HDDS_SCM_SAFEMODE_THRESHOLD_PCT,
         HddsConfigKeys.HDDS_SCM_SAFEMODE_THRESHOLD_PCT_DEFAULT));
 
-    Assert.assertEquals(cutOff, scmSafeModeManager.getSafeModeMetrics()
+    assertEquals(cutOff, scmSafeModeManager.getSafeModeMetrics()
         .getNumContainerWithOneReplicaReportedThreshold().value());
 
-    GenericTestUtils.waitFor(() -> {
-      return !scmSafeModeManager.getInSafeMode();
-    }, 100, 1000 * 5);
+    GenericTestUtils.waitFor(() -> !scmSafeModeManager.getInSafeMode(),
+        100, 1000 * 5);
 
-    Assert.assertEquals(cutOff, scmSafeModeManager.getSafeModeMetrics()
+    assertEquals(cutOff, scmSafeModeManager.getSafeModeMetrics()
         .getCurrentContainersWithOneReplicaReportedCount().value());
 
   }
@@ -176,38 +169,35 @@ public class TestSCMSafeModeManager {
         HddsConfigKeys.HDDS_SCM_SAFEMODE_THRESHOLD_PCT,
         HddsConfigKeys.HDDS_SCM_SAFEMODE_THRESHOLD_PCT_DEFAULT));
 
-    Assert.assertEquals(cutOff, scmSafeModeManager.getSafeModeMetrics()
+    assertEquals(cutOff, scmSafeModeManager.getSafeModeMetrics()
         .getNumContainerWithOneReplicaReportedThreshold().value());
 
     assertTrue(scmSafeModeManager.getInSafeMode());
 
     testContainerThreshold(containers.subList(0, 25), 0.25);
-    Assert.assertEquals(25, scmSafeModeManager.getSafeModeMetrics()
+    assertEquals(25, scmSafeModeManager.getSafeModeMetrics()
         .getCurrentContainersWithOneReplicaReportedCount().value());
     assertTrue(scmSafeModeManager.getInSafeMode());
     testContainerThreshold(containers.subList(25, 50), 0.50);
-    Assert.assertEquals(50, scmSafeModeManager.getSafeModeMetrics()
+    assertEquals(50, scmSafeModeManager.getSafeModeMetrics()
         .getCurrentContainersWithOneReplicaReportedCount().value());
     assertTrue(scmSafeModeManager.getInSafeMode());
     testContainerThreshold(containers.subList(50, 75), 0.75);
-    Assert.assertEquals(75, scmSafeModeManager.getSafeModeMetrics()
+    assertEquals(75, scmSafeModeManager.getSafeModeMetrics()
         .getCurrentContainersWithOneReplicaReportedCount().value());
     assertTrue(scmSafeModeManager.getInSafeMode());
     testContainerThreshold(containers.subList(75, 100), 1.0);
-    Assert.assertEquals(100, scmSafeModeManager.getSafeModeMetrics()
+    assertEquals(100, scmSafeModeManager.getSafeModeMetrics()
         .getCurrentContainersWithOneReplicaReportedCount().value());
 
-    GenericTestUtils.waitFor(() -> {
-      return !scmSafeModeManager.getInSafeMode();
-    }, 100, 1000 * 5);
+    GenericTestUtils.waitFor(() -> !scmSafeModeManager.getInSafeMode(),
+        100, 1000 * 5);
   }
 
 
   private OzoneConfiguration createConf(double healthyPercent,
-      double oneReplicaPercent) throws Exception {
-    OzoneConfiguration conf = new OzoneConfiguration();
-    conf.set(HddsConfigKeys.OZONE_METADATA_DIRS,
-        tempDir.newFolder().toString());
+      double oneReplicaPercent) {
+    OzoneConfiguration conf = new OzoneConfiguration(config);
     conf.setBoolean(
         HddsConfigKeys.HDDS_SCM_SAFEMODE_PIPELINE_AVAILABILITY_CHECK,
         true);
@@ -215,7 +205,6 @@ public class TestSCMSafeModeManager {
         HDDS_SCM_SAFEMODE_HEALTHY_PIPELINE_THRESHOLD_PCT, healthyPercent);
     conf.setDouble(HddsConfigKeys.
         HDDS_SCM_SAFEMODE_ONE_NODE_REPORTED_PIPELINE_PCT, oneReplicaPercent);
-    conf.setBoolean(HddsConfigKeys.HDDS_SCM_SAFEMODE_PIPELINE_CREATION, false);
     return conf;
   }
 
@@ -259,12 +248,13 @@ public class TestSCMSafeModeManager {
       PipelineManager pipelineManager =
           PipelineManagerImpl.newPipelineManager(
               conf,
-              MockSCMHAManager.getInstance(true),
+              SCMHAManagerStub.getInstance(true),
               mockNodeManager,
               scmMetadataStore.getPipelineTable(),
               queue,
               scmContext,
-              serviceManager);
+              serviceManager,
+              new MonotonicClock(ZoneOffset.UTC));
       scmSafeModeManager = new SCMSafeModeManager(
           conf, containers, null, pipelineManager, queue, serviceManager,
           scmContext);
@@ -285,12 +275,13 @@ public class TestSCMSafeModeManager {
       PipelineManager pipelineManager =
           PipelineManagerImpl.newPipelineManager(
               conf,
-              MockSCMHAManager.getInstance(true),
+              SCMHAManagerStub.getInstance(true),
               mockNodeManager,
               scmMetadataStore.getPipelineTable(),
               queue,
               scmContext,
-              serviceManager);
+              serviceManager,
+              new MonotonicClock(ZoneOffset.UTC));
       scmSafeModeManager = new SCMSafeModeManager(
           conf, containers, null, pipelineManager, queue, serviceManager,
           scmContext);
@@ -310,12 +301,13 @@ public class TestSCMSafeModeManager {
       PipelineManager pipelineManager =
           PipelineManagerImpl.newPipelineManager(
               conf,
-              MockSCMHAManager.getInstance(true),
+              SCMHAManagerStub.getInstance(true),
               mockNodeManager,
               scmMetadataStore.getPipelineTable(),
               queue,
               scmContext,
-              serviceManager);
+              serviceManager,
+              new MonotonicClock(ZoneOffset.UTC));
       scmSafeModeManager = new SCMSafeModeManager(
           conf, containers, null, pipelineManager, queue, serviceManager,
           scmContext);
@@ -342,23 +334,25 @@ public class TestSCMSafeModeManager {
     PipelineManagerImpl pipelineManager =
         PipelineManagerImpl.newPipelineManager(
             conf,
-            MockSCMHAManager.getInstance(true),
+            SCMHAManagerStub.getInstance(true),
             mockNodeManager,
             scmMetadataStore.getPipelineTable(),
             queue,
             scmContext,
-            serviceManager);
-    PipelineProvider mockRatisProvider =
+            serviceManager,
+            new MonotonicClock(ZoneOffset.UTC));
+    PipelineProvider<RatisReplicationConfig> mockRatisProvider =
         new MockRatisPipelineProvider(mockNodeManager,
             pipelineManager.getStateManager(), config);
     pipelineManager.setPipelineProvider(HddsProtos.ReplicationType.RATIS,
         mockRatisProvider);
     pipelineManager.getBackgroundPipelineCreator().stop();
+    pipelineManager.getBackgroundPipelineScrubber().stop();
 
     for (int i = 0; i < pipelineCount; i++) {
       // Create pipeline
       Pipeline pipeline = pipelineManager.createPipeline(
-          new RatisReplicationConfig(
+          RatisReplicationConfig.getInstance(
               ReplicationFactor.THREE));
 
       pipelineManager.openPipeline(pipeline.getId());
@@ -387,11 +381,11 @@ public class TestSCMSafeModeManager {
         scmSafeModeManager.getOneReplicaPipelineSafeModeRule()
             .getThresholdCount();
 
-    Assert.assertEquals(healthyPipelineThresholdCount,
+    assertEquals(healthyPipelineThresholdCount,
         scmSafeModeManager.getSafeModeMetrics()
             .getNumHealthyPipelinesThreshold().value());
 
-    Assert.assertEquals(oneReplicaThresholdCount,
+    assertEquals(oneReplicaThresholdCount,
         scmSafeModeManager.getSafeModeMetrics()
             .getNumPipelinesWithAtleastOneReplicaReportedThreshold().value());
 
@@ -408,34 +402,33 @@ public class TestSCMSafeModeManager {
 
       if (i < healthyPipelineThresholdCount) {
         checkHealthy(i + 1);
-        Assert.assertEquals(i + 1,
+        assertEquals(i + 1,
             scmSafeModeManager.getSafeModeMetrics()
                 .getCurrentHealthyPipelinesCount().value());
       }
 
       if (i < oneReplicaThresholdCount) {
         checkOpen(i + 1);
-        Assert.assertEquals(i + 1,
+        assertEquals(i + 1,
             scmSafeModeManager.getSafeModeMetrics()
                 .getCurrentPipelinesWithAtleastOneReplicaCount().value());
       }
     }
 
-    Assert.assertEquals(healthyPipelineThresholdCount,
+    assertEquals(healthyPipelineThresholdCount,
         scmSafeModeManager.getSafeModeMetrics()
             .getCurrentHealthyPipelinesCount().value());
 
-    Assert.assertEquals(oneReplicaThresholdCount,
+    assertEquals(oneReplicaThresholdCount,
         scmSafeModeManager.getSafeModeMetrics()
             .getCurrentPipelinesWithAtleastOneReplicaCount().value());
 
 
-    GenericTestUtils.waitFor(() -> {
-      return !scmSafeModeManager.getInSafeMode();
-    }, 100, 1000 * 5);
+    GenericTestUtils.waitFor(() -> !scmSafeModeManager.getInSafeMode(),
+        100, 1000 * 5);
   }
 
-  private void checkHealthy(int expectedCount) throws Exception{
+  private void checkHealthy(int expectedCount) throws Exception {
     GenericTestUtils.waitFor(() -> scmSafeModeManager
             .getHealthyPipelineSafeModeRule()
             .getCurrentHealthyPipelineCount() == expectedCount,
@@ -479,7 +472,7 @@ public class TestSCMSafeModeManager {
 
 
   @Test
-  public void testDisableSafeMode() throws IOException {
+  public void testDisableSafeMode() {
     OzoneConfiguration conf = new OzoneConfiguration(config);
     conf.setBoolean(HddsConfigKeys.HDDS_SCM_SAFEMODE_ENABLED, false);
     PipelineManager pipelineManager = Mockito.mock(PipelineManager.class);
@@ -507,7 +500,7 @@ public class TestSCMSafeModeManager {
     containers = new ArrayList<>();
     // Add 100 containers to the list of containers in SCM
     containers.addAll(HddsTestUtils.getContainerInfo(25 * 4));
-    // Assign CLOSED state to first 25 containers and OPEM state to rest
+    // Assign CLOSED state to first 25 containers and OPEN state to rest
     // of the containers
     for (ContainerInfo container : containers.subList(0, 25)) {
       container.setState(HddsProtos.LifeCycleState.CLOSED);
@@ -532,9 +525,8 @@ public class TestSCMSafeModeManager {
     // threshold should be (10+15)/25.
     testContainerThreshold(containers.subList(10, 25), 1.0);
 
-    GenericTestUtils.waitFor(() -> {
-      return !scmSafeModeManager.getInSafeMode();
-    }, 100, 1000 * 5);
+    GenericTestUtils.waitFor(() -> !scmSafeModeManager.getInSafeMode(),
+        100, 1000 * 5);
   }
 
   private void testSafeModeDataNodes(int numOfDns) throws Exception {
@@ -548,25 +540,23 @@ public class TestSCMSafeModeManager {
     assertTrue(scmSafeModeManager.getInSafeMode());
 
     // Register all DataNodes except last one and assert SCM is in safe mode.
-    for (int i = 0; i < numOfDns-1; i++) {
+    for (int i = 0; i < numOfDns - 1; i++) {
       queue.fireEvent(SCMEvents.NODE_REGISTRATION_CONT_REPORT,
           HddsTestUtils.createNodeRegistrationContainerReport(containers));
       assertTrue(scmSafeModeManager.getInSafeMode());
-      assertTrue(scmSafeModeManager.getCurrentContainerThreshold() == 1);
+      assertEquals(1, scmSafeModeManager.getCurrentContainerThreshold());
     }
 
-    if(numOfDns == 0){
-      GenericTestUtils.waitFor(() -> {
-        return scmSafeModeManager.getInSafeMode();
-      }, 10, 1000 * 10);
+    if (numOfDns == 0) {
+      GenericTestUtils.waitFor(() -> scmSafeModeManager.getInSafeMode(),
+          10, 1000 * 10);
       return;
     }
     // Register last DataNode and check that SCM is out of Safe mode.
     queue.fireEvent(SCMEvents.NODE_REGISTRATION_CONT_REPORT,
         HddsTestUtils.createNodeRegistrationContainerReport(containers));
-    GenericTestUtils.waitFor(() -> {
-      return !scmSafeModeManager.getInSafeMode();
-    }, 10, 1000 * 10);
+    GenericTestUtils.waitFor(() -> !scmSafeModeManager.getInSafeMode(),
+        10, 1000 * 10);
   }
 
   private void testContainerThreshold(List<ContainerInfo> dnContainers,
@@ -586,7 +576,7 @@ public class TestSCMSafeModeManager {
     containers.addAll(HddsTestUtils.getContainerInfo(25 * 4));
     String storageDir = GenericTestUtils.getTempPath(
         TestSCMSafeModeManager.class.getName() + UUID.randomUUID());
-    try{
+    try {
       MockNodeManager nodeManager = new MockNodeManager(true, 3);
       config.set(HddsConfigKeys.OZONE_METADATA_DIRS, storageDir);
       // enable pipeline check
@@ -596,21 +586,22 @@ public class TestSCMSafeModeManager {
       PipelineManagerImpl pipelineManager =
           PipelineManagerImpl.newPipelineManager(
               config,
-              MockSCMHAManager.getInstance(true),
+              SCMHAManagerStub.getInstance(true),
               nodeManager,
               scmMetadataStore.getPipelineTable(),
               queue,
               scmContext,
-              serviceManager);
+              serviceManager,
+              new MonotonicClock(ZoneOffset.UTC));
 
-      PipelineProvider mockRatisProvider =
+      PipelineProvider<RatisReplicationConfig> mockRatisProvider =
           new MockRatisPipelineProvider(nodeManager,
               pipelineManager.getStateManager(), config);
       pipelineManager.setPipelineProvider(HddsProtos.ReplicationType.RATIS,
           mockRatisProvider);
 
       Pipeline pipeline = pipelineManager.createPipeline(
-          new RatisReplicationConfig(
+          RatisReplicationConfig.getInstance(
               ReplicationFactor.THREE));
 
       pipeline = pipelineManager.getPipeline(pipeline.getId());
@@ -626,9 +617,8 @@ public class TestSCMSafeModeManager {
 
       firePipelineEvent(pipelineManager, pipeline);
 
-      GenericTestUtils.waitFor(() -> {
-        return !scmSafeModeManager.getInSafeMode();
-      }, 100, 1000 * 10);
+      GenericTestUtils.waitFor(() -> !scmSafeModeManager.getInSafeMode(),
+          100, 1000 * 10);
       pipelineManager.close();
     } finally {
       config.setBoolean(
@@ -639,7 +629,7 @@ public class TestSCMSafeModeManager {
   }
 
   @Test
-  @Ignore("The test is failing, enable after fixing it")
+  @Disabled("The test is failing, enable after fixing it")
   public void testPipelinesNotCreatedUntilPreCheckPasses()
       throws Exception {
     int numOfDns = 5;
@@ -661,14 +651,15 @@ public class TestSCMSafeModeManager {
     PipelineManagerImpl pipelineManager =
         PipelineManagerImpl.newPipelineManager(
             config,
-            MockSCMHAManager.getInstance(true),
+            SCMHAManagerStub.getInstance(true),
             nodeManager,
             scmMetadataStore.getPipelineTable(),
             queue,
             scmContext,
-            serviceManager);
+            serviceManager,
+            new MonotonicClock(ZoneOffset.UTC));
 
-    PipelineProvider mockRatisProvider =
+    PipelineProvider<RatisReplicationConfig> mockRatisProvider =
         new MockRatisPipelineProvider(nodeManager,
             pipelineManager.getStateManager(), config);
     pipelineManager.setPipelineProvider(HddsProtos.ReplicationType.RATIS,
@@ -691,17 +682,17 @@ public class TestSCMSafeModeManager {
       assertFalse(scmSafeModeManager.getPreCheckComplete());
     }
     queue.processAll(5000);
-    Assert.assertEquals(0, smHandler.getInvokedCount());
+    assertEquals(0, smHandler.getInvokedCount());
 
     // Register last DataNode and check that the SafeModeEvent gets fired, but
-    // safemode is still enabled with preCheck completed.
+    // SafeMode is still enabled with preCheck completed.
     queue.fireEvent(SCMEvents.NODE_REGISTRATION_CONT_REPORT,
         HddsTestUtils.createNodeRegistrationContainerReport(containers));
     queue.processAll(5000);
 
-    Assert.assertEquals(1, smHandler.getInvokedCount());
-    Assert.assertEquals(true, smHandler.getPreCheckComplete());
-    Assert.assertEquals(true, smHandler.getIsInSafeMode());
+    assertEquals(1, smHandler.getInvokedCount());
+    assertTrue(smHandler.getPreCheckComplete());
+    assertTrue(smHandler.getIsInSafeMode());
 
     /* There is a race condition where the background pipeline creation
      * task creates the pipeline before the following create call.
@@ -710,11 +701,11 @@ public class TestSCMSafeModeManager {
     Pipeline pipeline;
     try {
       pipeline = pipelineManager.createPipeline(
-          new RatisReplicationConfig(
+          RatisReplicationConfig.getInstance(
               ReplicationFactor.THREE));
     } catch (SCMException ex) {
       pipeline = pipelineManager.getPipelines(
-          new RatisReplicationConfig(
+          RatisReplicationConfig.getInstance(
               ReplicationFactor.THREE)).get(0);
     }
 
@@ -725,9 +716,9 @@ public class TestSCMSafeModeManager {
     firePipelineEvent(pipelineManager, pipeline);
 
     queue.processAll(5000);
-    Assert.assertEquals(2, smHandler.getInvokedCount());
-    Assert.assertEquals(true, smHandler.getPreCheckComplete());
-    Assert.assertEquals(false, smHandler.getIsInSafeMode());
+    assertEquals(2, smHandler.getInvokedCount());
+    assertTrue(smHandler.getPreCheckComplete());
+    assertFalse(smHandler.getIsInSafeMode());
   }
 
   private static class SafeModeEventHandler
