@@ -41,7 +41,7 @@ import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolPro
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.SCMVersionResponseProto;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.MetadataStorageReportProto;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.StorageReportProto;
-import org.apache.hadoop.hdds.scm.TestUtils;
+import org.apache.hadoop.hdds.scm.HddsTestUtils;
 import org.apache.hadoop.hdds.scm.VersionInfo;
 import org.apache.hadoop.hdds.scm.pipeline.PipelineID;
 import org.apache.hadoop.hdds.upgrade.HDDSLayoutVersionManager;
@@ -69,14 +69,13 @@ import static org.apache.hadoop.hdds.protocol.MockDatanodeDetails.randomDatanode
 import static org.apache.hadoop.ozone.container.upgrade.UpgradeUtils.defaultLayoutVersionProto;
 import static org.apache.hadoop.hdds.upgrade.HDDSLayoutVersionManager.maxLayoutVersion;
 import static org.apache.hadoop.ozone.container.common.ContainerTestUtils.createEndpoint;
-import static org.hamcrest.Matchers.lessThanOrEqualTo;
-import org.junit.AfterClass;
-import org.junit.Assert;
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertNotNull;
-import static org.junit.Assert.assertTrue;
-import org.junit.BeforeClass;
-import org.junit.Test;
+import org.junit.jupiter.api.AfterAll;
+import org.junit.jupiter.api.Assertions;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+import org.junit.jupiter.api.BeforeAll;
+import org.junit.jupiter.api.Test;
 import org.mockito.Mockito;
 import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.when;
@@ -91,7 +90,7 @@ public class TestEndPoint {
   private static File testDir;
   private static OzoneConfiguration config;
 
-  @AfterClass
+  @AfterAll
   public static void tearDown() throws Exception {
     if (scmServer != null) {
       scmServer.stop();
@@ -99,7 +98,7 @@ public class TestEndPoint {
     FileUtil.fullyDelete(testDir);
   }
 
-  @BeforeClass
+  @BeforeAll
   public static void setUp() throws Exception {
     serverAddress = SCMTestUtils.getReuseableAddress();
     scmServerImpl = new ScmTestMock();
@@ -126,10 +125,10 @@ public class TestEndPoint {
             serverAddress, 1000)) {
       SCMVersionResponseProto responseProto = rpcEndPoint.getEndPoint()
           .getVersion(null);
-      Assert.assertNotNull(responseProto);
-      Assert.assertEquals(VersionInfo.DESCRIPTION_KEY,
+      Assertions.assertNotNull(responseProto);
+      Assertions.assertEquals(VersionInfo.DESCRIPTION_KEY,
           responseProto.getKeys(0).getKey());
-      Assert.assertEquals(VersionInfo.getLatestVersion().getDescription(),
+      Assertions.assertEquals(VersionInfo.getLatestVersion().getDescription(),
           responseProto.getKeys(0).getValue());
     }
   }
@@ -154,11 +153,11 @@ public class TestEndPoint {
 
       // if version call worked the endpoint should automatically move to the
       // next state.
-      Assert.assertEquals(EndpointStateMachine.EndPointStates.REGISTER,
+      Assertions.assertEquals(EndpointStateMachine.EndPointStates.REGISTER,
           newState);
 
       // Now rpcEndpoint should remember the version it got from SCM
-      Assert.assertNotNull(rpcEndPoint.getVersion());
+      Assertions.assertNotNull(rpcEndPoint.getVersion());
     }
   }
 
@@ -184,11 +183,11 @@ public class TestEndPoint {
 
       // if version call worked the endpoint should automatically move to the
       // next state.
-      Assert.assertEquals(EndpointStateMachine.EndPointStates.REGISTER,
+      Assertions.assertEquals(EndpointStateMachine.EndPointStates.REGISTER,
           newState);
 
       // Now rpcEndpoint should remember the version it got from SCM
-      Assert.assertNotNull(rpcEndPoint.getVersion());
+      Assertions.assertNotNull(rpcEndPoint.getVersion());
 
       // Now change server cluster ID, so datanode cluster ID will be
       // different from SCM server response cluster ID.
@@ -196,18 +195,17 @@ public class TestEndPoint {
       scmServerImpl.setClusterId(newClusterId);
       rpcEndPoint.setState(EndpointStateMachine.EndPointStates.GETVERSION);
       newState = versionTask.call();
-      Assert.assertEquals(EndpointStateMachine.EndPointStates.SHUTDOWN,
+      Assertions.assertEquals(EndpointStateMachine.EndPointStates.SHUTDOWN,
           newState);
       List<HddsVolume> volumesList = StorageVolumeUtil.getHddsVolumesList(
           ozoneContainer.getVolumeSet().getFailedVolumesList());
-      Assert.assertTrue(volumesList.size() == 1);
-      Assert.assertTrue(logCapturer.getOutput()
+      assertEquals(1, volumesList.size());
+      Assertions.assertTrue(logCapturer.getOutput()
           .contains("org.apache.hadoop.ozone.common" +
               ".InconsistentStorageStateException: Mismatched ClusterIDs"));
-      Assert.assertTrue(ozoneContainer.getVolumeSet().getVolumesList().size()
-          == 0);
-      Assert.assertTrue(ozoneContainer.getVolumeSet().getFailedVolumesList()
-          .size() == 1);
+      assertEquals(0, ozoneContainer.getVolumeSet().getVolumesList().size());
+      assertEquals(1, ozoneContainer.getVolumeSet().getFailedVolumesList()
+          .size());
 
     }
   }
@@ -233,7 +231,7 @@ public class TestEndPoint {
 
       // This version call did NOT work, so endpoint should remain in the same
       // state.
-      Assert.assertEquals(EndpointStateMachine.EndPointStates.GETVERSION,
+      Assertions.assertEquals(EndpointStateMachine.EndPointStates.GETVERSION,
           newState);
     }
   }
@@ -263,8 +261,8 @@ public class TestEndPoint {
       EndpointStateMachine.EndPointStates newState = versionTask.call();
       long end = Time.monotonicNow();
       scmServerImpl.setRpcResponseDelay(0);
-      Assert.assertThat(end - start, lessThanOrEqualTo(rpcTimeout + tolerance));
-      Assert.assertEquals(EndpointStateMachine.EndPointStates.GETVERSION,
+      Assertions.assertTrue(end - start <= rpcTimeout + tolerance);
+      Assertions.assertEquals(EndpointStateMachine.EndPointStates.GETVERSION,
           newState);
     }
   }
@@ -275,33 +273,35 @@ public class TestEndPoint {
     try (EndpointStateMachine rpcEndPoint = createEndpoint(
         SCMTestUtils.getConf(), serverAddress, 1000)) {
       SCMRegisteredResponseProto responseProto = rpcEndPoint.getEndPoint()
-          .register(nodeToRegister.getExtendedProtoBufMessage(), TestUtils
+          .register(nodeToRegister.getExtendedProtoBufMessage(), HddsTestUtils
                   .createNodeReport(
                       Arrays.asList(getStorageReports(
                           nodeToRegister.getUuid())),
                       Arrays.asList(getMetadataStorageReports(
                           nodeToRegister.getUuid()))),
-              TestUtils.getRandomContainerReports(10),
-              TestUtils.getRandomPipelineReports(),
+              HddsTestUtils.getRandomContainerReports(10),
+              HddsTestUtils.getRandomPipelineReports(),
               defaultLayoutVersionProto());
-      Assert.assertNotNull(responseProto);
-      Assert.assertEquals(nodeToRegister.getUuidString(),
+      Assertions.assertNotNull(responseProto);
+      Assertions.assertEquals(nodeToRegister.getUuidString(),
           responseProto.getDatanodeUUID());
-      Assert.assertNotNull(responseProto.getClusterID());
-      Assert.assertEquals(10, scmServerImpl.
+      Assertions.assertNotNull(responseProto.getClusterID());
+      Assertions.assertEquals(10, scmServerImpl.
           getContainerCountsForDatanode(nodeToRegister));
-      Assert.assertEquals(1, scmServerImpl.getNodeReportsCount(nodeToRegister));
+      Assertions.assertEquals(1,
+          scmServerImpl.getNodeReportsCount(nodeToRegister));
     }
   }
 
   private StorageReportProto getStorageReports(UUID id) {
     String storagePath = testDir.getAbsolutePath() + "/data-" + id;
-    return TestUtils.createStorageReport(id, storagePath, 100, 10, 90, null);
+    return HddsTestUtils.createStorageReport(id, storagePath, 100, 10, 90,
+        null);
   }
 
   private MetadataStorageReportProto getMetadataStorageReports(UUID id) {
     String storagePath = testDir.getAbsolutePath() + "/metadata-" + id;
-    return TestUtils.createMetadataStorageReport(storagePath, 100, 10, 90,
+    return HddsTestUtils.createMetadataStorageReport(storagePath, 100, 10, 90,
         null);
   }
 
@@ -315,15 +315,15 @@ public class TestEndPoint {
     rpcEndPoint.setState(EndpointStateMachine.EndPointStates.REGISTER);
     OzoneContainer ozoneContainer = mock(OzoneContainer.class);
     UUID datanodeID = UUID.randomUUID();
-    when(ozoneContainer.getNodeReport()).thenReturn(TestUtils
+    when(ozoneContainer.getNodeReport()).thenReturn(HddsTestUtils
         .createNodeReport(Arrays.asList(getStorageReports(datanodeID)),
             Arrays.asList(getMetadataStorageReports(datanodeID))));
     ContainerController controller = Mockito.mock(ContainerController.class);
     when(controller.getContainerReport()).thenReturn(
-        TestUtils.getRandomContainerReports(10));
+        HddsTestUtils.getRandomContainerReports(10));
     when(ozoneContainer.getController()).thenReturn(controller);
     when(ozoneContainer.getPipelineReport()).thenReturn(
-        TestUtils.getRandomPipelineReports());
+        HddsTestUtils.getRandomPipelineReports());
     HDDSLayoutVersionManager versionManager =
         Mockito.mock(HDDSLayoutVersionManager.class);
     when(versionManager.getMetadataLayoutVersion())
@@ -346,7 +346,7 @@ public class TestEndPoint {
     try (EndpointStateMachine rpcEndpoint =
         registerTaskHelper(serverAddress, 1000, false)) {
       // Successful register should move us to Heartbeat state.
-      Assert.assertEquals(EndpointStateMachine.EndPointStates.HEARTBEAT,
+      Assertions.assertEquals(EndpointStateMachine.EndPointStates.HEARTBEAT,
           rpcEndpoint.getState());
     }
   }
@@ -356,7 +356,7 @@ public class TestEndPoint {
     InetSocketAddress address = SCMTestUtils.getReuseableAddress();
     try (EndpointStateMachine rpcEndpoint =
         registerTaskHelper(address, 1000, false)) {
-      Assert.assertEquals(EndpointStateMachine.EndPointStates.REGISTER,
+      Assertions.assertEquals(EndpointStateMachine.EndPointStates.REGISTER,
           rpcEndpoint.getState());
     }
   }
@@ -368,7 +368,7 @@ public class TestEndPoint {
         registerTaskHelper(address, 1000, true)) {
       // No Container ID, therefore we tell the datanode that we would like to
       // shutdown.
-      Assert.assertEquals(EndpointStateMachine.EndPointStates.SHUTDOWN,
+      Assertions.assertEquals(EndpointStateMachine.EndPointStates.SHUTDOWN,
           rpcEndpoint.getState());
     }
   }
@@ -382,7 +382,7 @@ public class TestEndPoint {
     registerTaskHelper(serverAddress, 1000, false).close();
     long end = Time.monotonicNow();
     scmServerImpl.setRpcResponseDelay(0);
-    Assert.assertThat(end - start, lessThanOrEqualTo(rpcTimeout + tolerance));
+    Assertions.assertTrue(end - start <= rpcTimeout + tolerance);
   }
 
   @Test
@@ -393,15 +393,15 @@ public class TestEndPoint {
             serverAddress, 1000)) {
       SCMHeartbeatRequestProto request = SCMHeartbeatRequestProto.newBuilder()
           .setDatanodeDetails(dataNode.getProtoBufMessage())
-          .setNodeReport(TestUtils.createNodeReport(
+          .setNodeReport(HddsTestUtils.createNodeReport(
               Arrays.asList(getStorageReports(dataNode.getUuid())),
               Arrays.asList(getMetadataStorageReports(dataNode.getUuid()))))
           .build();
 
       SCMHeartbeatResponseProto responseProto = rpcEndPoint.getEndPoint()
           .sendHeartbeat(request);
-      Assert.assertNotNull(responseProto);
-      Assert.assertEquals(0, responseProto.getCommandsCount());
+      Assertions.assertNotNull(responseProto);
+      Assertions.assertEquals(0, responseProto.getCommandsCount());
     }
   }
 
@@ -416,7 +416,7 @@ public class TestEndPoint {
 
       SCMHeartbeatRequestProto request = SCMHeartbeatRequestProto.newBuilder()
           .setDatanodeDetails(dataNode.getProtoBufMessage())
-          .setNodeReport(TestUtils.createNodeReport(
+          .setNodeReport(HddsTestUtils.createNodeReport(
               Arrays.asList(getStorageReports(dataNode.getUuid())),
               Arrays.asList(getMetadataStorageReports(dataNode.getUuid()))))
           .build();
@@ -432,7 +432,7 @@ public class TestEndPoint {
           serverAddress, 3000);
       Map<Long, CommandStatus> map = stateContext.getCommandStatusMap();
       assertNotNull(map);
-      assertEquals("Should have 1 objects", 1, map.size());
+      assertEquals(1, map.size(), "Should have 1 objects");
       assertTrue(map.containsKey(3L));
       assertEquals(Type.deleteBlocksCommand, map.get(3L).getType());
       assertEquals(Status.PENDING, map.get(3L).getStatus());
@@ -506,9 +506,9 @@ public class TestEndPoint {
               stateMachine.getLayoutVersionManager());
       endpointTask.setDatanodeDetailsProto(datanodeDetailsProto);
       endpointTask.call();
-      Assert.assertNotNull(endpointTask.getDatanodeDetailsProto());
+      Assertions.assertNotNull(endpointTask.getDatanodeDetailsProto());
 
-      Assert.assertEquals(EndpointStateMachine.EndPointStates.HEARTBEAT,
+      Assertions.assertEquals(EndpointStateMachine.EndPointStates.HEARTBEAT,
           rpcEndPoint.getState());
       return stateContext;
     }
@@ -535,8 +535,8 @@ public class TestEndPoint {
     heartbeatTaskHelper(invalidAddress, 1000);
     long end = Time.monotonicNow();
     scmServerImpl.setRpcResponseDelay(0);
-    Assert.assertThat(end - start,
-        lessThanOrEqualTo(rpcTimeout + tolerance));
+    // 6s is introduced by DeleteBlocksCommandHandler#stop
+    Assertions.assertTrue(end - start <= rpcTimeout + tolerance + 6000);
   }
 
   private StateContext getContext(DatanodeDetails datanodeDetails) {
