@@ -18,8 +18,10 @@
 package org.apache.hadoop.hdds.cli;
 
 import com.google.common.annotations.VisibleForTesting;
+import java.io.IOException;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.tracing.TracingUtil;
+import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.hadoop.util.NativeCodeLoader;
 
 import org.apache.log4j.ConsoleAppender;
@@ -43,6 +45,8 @@ public class OzoneAdmin extends GenericCli {
 
   private OzoneConfiguration ozoneConf;
 
+  private UserGroupInformation user;
+
   public OzoneAdmin() {
     super(OzoneAdmin.class);
   }
@@ -60,6 +64,13 @@ public class OzoneAdmin extends GenericCli {
     return ozoneConf;
   }
 
+  public UserGroupInformation getUser() throws IOException {
+    if (user == null) {
+      user = UserGroupInformation.getCurrentUser();
+    }
+    return user;
+  }
+
   /**
    * Main for the Ozone Admin shell Command handling.
    *
@@ -76,12 +87,9 @@ public class OzoneAdmin extends GenericCli {
   }
 
   @Override
-  public void execute(String[] argv) {
+  public int execute(String[] argv) {
     TracingUtil.initTracing("shell", createOzoneConfiguration());
-    TracingUtil.executeInNewSpan("main",
-        (Supplier<Void>) () -> {
-          super.execute(argv);
-          return null;
-        });
+    return TracingUtil.executeInNewSpan("main",
+        (Supplier<Integer>) () -> super.execute(argv));
   }
 }
