@@ -22,6 +22,9 @@ import com.fasterxml.jackson.annotation.JsonIgnore;
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
 import org.apache.commons.lang3.StringUtils;
+import org.apache.commons.lang3.tuple.ImmutablePair;
+import org.apache.commons.lang3.tuple.Pair;
+import org.apache.hadoop.hdds.client.DefaultReplicationConfig;
 import org.apache.hadoop.hdds.client.OzoneQuota;
 import org.apache.hadoop.hdds.client.ReplicationConfig;
 import org.apache.hadoop.hdds.conf.ConfigurationSource;
@@ -58,6 +61,7 @@ import java.util.NoSuchElementException;
 
 import static org.apache.hadoop.ozone.OzoneConsts.QUOTA_RESET;
 import static org.apache.hadoop.ozone.OzoneConsts.OZONE_URI_DELIMITER;
+import static org.apache.hadoop.ozone.om.exceptions.OMException.ResultCodes.FILE_NOT_FOUND;
 
 /**
  * A class that encapsulates OzoneBucket.
@@ -80,7 +84,7 @@ public class OzoneBucket extends WithMetadata {
   /**
    * Default replication factor to be used while creating keys.
    */
-  private final ReplicationConfig defaultReplication;
+  private ReplicationConfig defaultReplication;
 
   /**
    * Type of storage to be used for this bucket.
@@ -140,6 +144,10 @@ public class OzoneBucket extends WithMetadata {
    * Bucket Layout.
    */
   private BucketLayout bucketLayout = BucketLayout.DEFAULT;
+  /**
+   * Bucket Owner.
+   */
+  private String owner;
 
   private OzoneBucket(ConfigurationSource conf, String volumeName,
       String bucketName, ClientProtocol proxy) {
@@ -147,7 +155,8 @@ public class OzoneBucket extends WithMetadata {
     this.volumeName = volumeName;
     this.name = bucketName;
 
-    this.defaultReplication = ReplicationConfig.getDefault(conf);
+    // Bucket level replication is not configured by default.
+    this.defaultReplication = null;
 
     this.proxy = proxy;
     this.ozoneObj = OzoneObjInfo.Builder.newBuilder()
@@ -221,6 +230,48 @@ public class OzoneBucket extends WithMetadata {
     this.bucketLayout = bucketLayout;
   }
 
+  @SuppressWarnings("parameternumber")
+  public OzoneBucket(ConfigurationSource conf, ClientProtocol proxy,
+      String volumeName, String bucketName, StorageType storageType,
+      Boolean versioning, long creationTime, long modificationTime,
+      Map<String, String> metadata, String encryptionKeyName,
+      String sourceVolume, String sourceBucket, long usedBytes,
+      long usedNamespace, long quotaInBytes, long quotaInNamespace,
+      BucketLayout bucketLayout, String owner,
+      DefaultReplicationConfig defaultReplicationConfig) {
+    this(conf, proxy, volumeName, bucketName, storageType, versioning,
+        creationTime, modificationTime, metadata, encryptionKeyName,
+        sourceVolume, sourceBucket, usedBytes, usedNamespace, quotaInBytes,
+        quotaInNamespace, bucketLayout, owner);
+    this.bucketLayout = bucketLayout;
+    if (defaultReplicationConfig != null) {
+      this.defaultReplication =
+          defaultReplicationConfig.getType() == ReplicationType.EC ?
+              defaultReplicationConfig.getEcReplicationConfig() :
+              ReplicationConfig
+                  .fromTypeAndFactor(defaultReplicationConfig.getType(),
+                      defaultReplicationConfig.getFactor());
+    } else {
+      // Bucket level replication is not configured by default.
+      this.defaultReplication = null;
+    }
+  }
+
+  @SuppressWarnings("checkstyle:ParameterNumber")
+  public OzoneBucket(ConfigurationSource conf, ClientProtocol proxy,
+       String volumeName, String bucketName, StorageType storageType,
+       Boolean versioning, long creationTime, long modificationTime,
+       Map<String, String> metadata, String encryptionKeyName,
+       String sourceVolume, String sourceBucket, long usedBytes,
+       long usedNamespace, long quotaInBytes, long quotaInNamespace,
+       BucketLayout bucketLayout, String owner) {
+    this(conf, proxy, volumeName, bucketName, storageType, versioning,
+        creationTime, modificationTime, metadata, encryptionKeyName,
+        sourceVolume, sourceBucket, usedBytes, usedNamespace, quotaInBytes,
+        quotaInNamespace, bucketLayout);
+    this.owner = owner;
+  }
+
   /**
    * Constructs OzoneBucket instance.
    * @param conf Configuration object.
@@ -385,6 +436,15 @@ public class OzoneBucket extends WithMetadata {
     return quotaInNamespace;
   }
 
+  /**
+   * Returns the owner of the Bucket.
+   *
+   * @return owner
+   */
+  public String getOwner() {
+    return owner;
+  }
+
   /**
    * Builder for OmBucketInfo.
   /**
@@ -478,6 +538,17 @@ public class OzoneBucket extends WithMetadata {
     quotaInNamespace = quota.getQuotaInNamespace();
   }
 
+  /**
+   * Sets/Changes the default replication config of this Bucket.
+   *
+   * @param replicationConfig Replication config that can be applied to bucket.
+   * @throws IOException
+   */
+  public void setReplicationConfig(ReplicationConfig replicationConfig)
+      throws IOException {
+    proxy.setReplicationConfig(volumeName, name, replicationConfig);
+  }
+
   /**
    * Creates a new key in the bucket, with default replication type RATIS and
    * with replication factor THREE.
@@ -582,7 +653,7 @@ public class OzoneBucket extends WithMetadata {
    * @return {@code Iterator<OzoneKey>}
    */
   public Iterator<? extends OzoneKey> listKeys(String keyPrefix)
-      throws IOException{
+      throws IOException {
     return listKeys(keyPrefix, null);
   }
 
@@ -603,6 +674,14 @@ public class OzoneBucket extends WithMetadata {
         .getKeyIterator(keyPrefix, prevKey, bucketLayout);
   }
 
+  /**
+   * Checks if the bucket is a Link Bucket.
+   * @return True if bucket is a link, False otherwise.
+   */
+  public boolean isLink() {
+    return sourceVolume != null && sourceBucket != null;
+  }
+
   /**
    * Deletes key from the bucket.
    * @param key Name of the key to be deleted.
@@ -651,6 +730,7 @@ public class OzoneBucket extends WithMetadata {
    * @param keyMap The key is original key name nad value is new key name.
    * @throws IOException
    */
+  @Deprecated
   public void renameKeys(Map<String, String> keyMap)
       throws IOException {
     proxy.renameKeys(volumeName, name, keyMap);
@@ -810,6 +890,7 @@ public class OzoneBucket extends WithMetadata {
    * @throws IOException if there is error in the db
    *                     invalid arguments
    */
+  @Deprecated
   public OzoneOutputStream createFile(String keyName, long size,
       ReplicationType type, ReplicationFactor factor, boolean overWrite,
       boolean recursive) throws IOException {
@@ -857,6 +938,28 @@ public class OzoneBucket extends WithMetadata {
         .listStatus(volumeName, name, keyName, recursive, startKey, numEntries);
   }
 
+  /**
+   * List the status for a file or a directory and its contents.
+   *
+   * @param keyName    Absolute path of the entry to be listed
+   * @param recursive  For a directory if true all the descendants of a
+   *                   particular directory are listed
+   * @param startKey   Key from which listing needs to start. If startKey exists
+   *                   its status is included in the final list.
+   * @param numEntries Number of entries to list from the start key
+   * @param allowPartialPrefix allow partial prefixes during listStatus,
+   *                           this is used in context of listKeys calling
+   *                           listStatus
+   * @return list of file status
+   */
+  public List<OzoneFileStatus> listStatus(String keyName, boolean recursive,
+      String startKey, long numEntries, boolean allowPartialPrefix)
+      throws IOException {
+    return proxy
+        .listStatus(volumeName, name, keyName, recursive, startKey,
+            numEntries, allowPartialPrefix);
+  }
+
   /**
    * Return with the list of the in-flight multipart uploads.
    *
@@ -867,6 +970,17 @@ public class OzoneBucket extends WithMetadata {
     return proxy.listMultipartUploads(volumeName, getName(), prefix);
   }
 
+  /**
+   * Sets/Changes the owner of this Bucket.
+   * @param userName new owner
+   * @throws IOException
+   */
+  public boolean setOwner(String userName) throws IOException {
+    boolean result = proxy.setBucketOwner(volumeName, name, userName);
+    this.owner = userName;
+    return result;
+  }
+
   /**
    * An Iterator to iterate over {@link OzoneKey} list.
    */
@@ -890,7 +1004,7 @@ public class OzoneBucket extends WithMetadata {
      * The returned keys match key prefix.
      * @param keyPrefix
      */
-    KeyIterator(String keyPrefix, String prevKey) throws IOException{
+    KeyIterator(String keyPrefix, String prevKey) throws IOException {
       setKeyPrefix(keyPrefix);
       this.currentValue = null;
       this.currentIterator = getNextListOfKeys(prevKey).iterator();
@@ -898,7 +1012,7 @@ public class OzoneBucket extends WithMetadata {
 
     @Override
     public boolean hasNext() {
-      if(!currentIterator.hasNext() && currentValue != null) {
+      if (!currentIterator.hasNext() && currentValue != null) {
         try {
           currentIterator =
               getNextListOfKeys(currentValue.getName()).iterator();
@@ -911,7 +1025,7 @@ public class OzoneBucket extends WithMetadata {
 
     @Override
     public OzoneKey next() {
-      if(hasNext()) {
+      if (hasNext()) {
         currentValue = currentIterator.next();
         return currentValue;
       }
@@ -959,11 +1073,11 @@ public class OzoneBucket extends WithMetadata {
    *
    * Note: Does not guarantee to return the list of keys in a sorted order.
    */
-  private class KeyIteratorWithFSO extends KeyIterator{
+  private class KeyIteratorWithFSO extends KeyIterator {
 
-    private Stack<String> stack;
-    private List<OzoneKey> pendingItemsToBeBatched;
+    private Stack<Pair<String, String>> stack;
     private boolean addedKeyPrefix;
+    private String removeStartKey = "";
 
     /**
      * Creates an Iterator to iterate over all keys after prevKey in the bucket.
@@ -977,11 +1091,41 @@ public class OzoneBucket extends WithMetadata {
       super(keyPrefix, prevKey);
     }
 
+    /**
+     * keyPrefix="a1" and startKey="a1/b2/d2/f3/f31.tx".
+     * Now, this function will prepare and return a list :
+     * <"a1/b2/d2", "a1/b2/d2/f3">
+     * <"a1/b2", "a1/b2/d2">
+     * <"a1", "a1/b2">
+     *
+     * @param keyPrefix keyPrefix
+     * @param startKey  startKey
+     * @param seekPaths list of seek paths between keyPrefix and startKey
+     */
+    private void getSeekPathsBetweenKeyPrefixAndStartKey(String keyPrefix,
+        String startKey, List<Pair<String, String>> seekPaths) {
+
+      String parentStartKeyPath = OzoneFSUtils.getParentDir(startKey);
+
+      if (StringUtils.isNotBlank(startKey)) {
+        if (StringUtils.compare(parentStartKeyPath, keyPrefix) >= 0) {
+          seekPaths.add(new ImmutablePair<>(parentStartKeyPath, startKey));
+
+          // recursively fetch all the sub-paths between keyPrefix and prevKey
+          getSeekPathsBetweenKeyPrefixAndStartKey(keyPrefix, parentStartKeyPath,
+              seekPaths);
+        } else if (StringUtils.compare(startKey, keyPrefix) >= 0) {
+          // Both keyPrefix and startKey reached at the same level.
+          // Adds partial keyPrefix and startKey for seek.
+          seekPaths.add(new ImmutablePair<>(keyPrefix, startKey));
+        }
+      }
+    }
+
     @Override
     List<OzoneKey> getNextListOfKeys(String prevKey) throws IOException {
       if (stack == null) {
         stack = new Stack();
-        pendingItemsToBeBatched = new ArrayList<>();
       }
 
       // normalize paths
@@ -992,18 +1136,97 @@ public class OzoneBucket extends WithMetadata {
           keyPrefixName = OmUtils.normalizeKey(getKeyPrefix(), true);
         }
         setKeyPrefix(keyPrefixName);
+
+        if (StringUtils.isNotBlank(prevKey)) {
+          if (StringUtils.startsWith(prevKey, getKeyPrefix())) {
+            // 1. Prepare all the seekKeys after the prefixKey.
+            // Example case: prefixKey="a1", startKey="a1/b2/d2/f3/f31.tx"
+            // Now, stack should be build with all the levels after prefixKey
+            // Stack format => <keyPrefix and startKey>, startKey should be an
+            // immediate child of keyPrefix.
+            //             _______________________________________
+            // Stack=> top | < a1/b2/d2/f3, a1/b2/d2/f3/f31.tx > |
+            //             |-------------------------------------|
+            //             | < a1/b2/d2, a1/b2/d2/f3 >           |
+            //             |-------------------------------------|
+            //             | < a1/b2, a1/b2/d2 >                 |
+            //             |-------------------------------------|
+            //      bottom | < a1, a1/b2 >                       |
+            //             --------------------------------------|
+            List<Pair<String, String>> seekPaths = new ArrayList<>();
+
+            if (StringUtils.isNotBlank(getKeyPrefix())) {
+              // If the prev key is a dir then seek its sub-paths
+              // Say, prevKey="a1/b2/d2"
+              addPrevDirectoryToSeekPath(prevKey, seekPaths);
+            }
+
+            // Key Prefix is Blank. The seek all the keys with startKey.
+            removeStartKey = prevKey;
+            getSeekPathsBetweenKeyPrefixAndStartKey(getKeyPrefix(), prevKey,
+                seekPaths);
+
+            // 2. Push elements in reverse order so that the FS tree traversal
+            // will occur in left-to-right fashion[Depth-First Search]
+            for (int index = seekPaths.size() - 1; index >= 0; index--) {
+              Pair<String, String> seekDirPath = seekPaths.get(index);
+              stack.push(seekDirPath);
+            }
+          } else if (StringUtils.isNotBlank(getKeyPrefix())) {
+            if (!OzoneFSUtils.isAncestorPath(getKeyPrefix(), prevKey)) {
+              // Case-1 - sibling: keyPrefix="a1/b2", startKey="a0/b123Invalid"
+              // Skip traversing, if the startKey is not a sibling.
+              // "a1/b", "a1/b1/e/"
+              return new ArrayList<>();
+            } else if (StringUtils.compare(prevKey, getKeyPrefix()) < 0) {
+              // Case-2 - compare: keyPrefix="a1/b2", startKey="a1/b123Invalid"
+              // Since startKey is lexographically behind keyPrefix,
+              // the seek precedence goes to keyPrefix.
+              stack.push(new ImmutablePair<>(getKeyPrefix(), ""));
+            }
+          }
+        }
       }
 
-      // Get immediate children
+      // 1. Pop out top pair and get its immediate children
       List<OzoneKey> keysResultList = new ArrayList<>();
-      getChildrenKeys(getKeyPrefix(), prevKey, keysResultList);
-
-      // TODO: Back and Forth seek all the files & dirs, starting from
-      //  startKey till keyPrefix.
+      if (stack.isEmpty()) {
+        // case: startKey is empty
+        if (getChildrenKeys(getKeyPrefix(), prevKey, keysResultList)) {
+          return keysResultList;
+        }
+      }
 
+      // 2. Pop element and seek for its sub-child path(s). Basically moving
+      // seek pointer to next level(depth) in FS tree.
+      // case: startKey is non-empty
+      while (!stack.isEmpty()) {
+        Pair<String, String> keyPrefixPath = stack.pop();
+        if (getChildrenKeys(keyPrefixPath.getLeft(), keyPrefixPath.getRight(),
+            keysResultList)) {
+          // reached limit batch size.
+          break;
+        }
+      }
       return keysResultList;
     }
 
+    private void addPrevDirectoryToSeekPath(String prevKey,
+        List<Pair<String, String>> seekPaths)
+        throws IOException {
+      try {
+        OzoneFileStatus prevStatus =
+            proxy.getOzoneFileStatus(volumeName, name, prevKey);
+        if (prevStatus != null) {
+          if (prevStatus.isDirectory()) {
+            seekPaths.add(new ImmutablePair<>(prevKey, ""));
+          }
+        }
+      } catch (OMException ome) {
+        // ignore exception
+      }
+    }
+
     /**
      * List children under the given keyPrefix and startKey path. It does
      * recursive #listStatus calls to list all the sub-keys resultList.
@@ -1050,81 +1273,60 @@ public class OzoneBucket extends WithMetadata {
       // listStatus API expects a not null 'startKey' value
       startKey = startKey == null ? "" : startKey;
 
-      // 1. Add pending items to the user key resultList
-      if (addAllPendingItemsToResultList(keysResultList)) {
-        // reached limit batch size.
-        return true;
-      }
-
-      // 2. Get immediate children of keyPrefix, starting with startKey
+      // 1. Get immediate children of keyPrefix, starting with startKey
       List<OzoneFileStatus> statuses = proxy.listStatus(volumeName, name,
-              keyPrefix, false, startKey, listCacheSize);
+          keyPrefix, false, startKey, listCacheSize, true);
+      boolean reachedLimitCacheSize = statuses.size() == listCacheSize;
 
-      // 3. Special case: ListKey expects keyPrefix element should present in
+      // 2. Special case: ListKey expects keyPrefix element should present in
       // the resultList, only if startKey is blank. If startKey is not blank
       // then resultList shouldn't contain the startKey element.
       // Since proxy#listStatus API won't return keyPrefix element in the
       // resultList. So, this is to add user given keyPrefix to the return list.
       addKeyPrefixInfoToResultList(keyPrefix, startKey, keysResultList);
 
-      // 4. Special case: ListKey expects startKey shouldn't present in the
+      // 3. Special case: ListKey expects startKey shouldn't present in the
       // resultList. Since proxy#listStatus API returns startKey element to
       // the returnList, this function is to remove the startKey element.
       removeStartKeyIfExistsInStatusList(startKey, statuses);
 
-      boolean reachedLimitCacheSize = false;
-      // This dirList is used to store paths elements in left-to-right order.
-      List<String> dirList = new ArrayList<>();
-
-      // 5. Iterating over the resultStatuses list and add each key to the
-      // resultList. If the listCacheSize reaches then it will add the rest
-      // of the statuses to pendingItemsToBeBatched
+      // 4. Iterating over the resultStatuses list and add each key to the
+      // resultList.
       for (int indx = 0; indx < statuses.size(); indx++) {
         OzoneFileStatus status = statuses.get(indx);
         OmKeyInfo keyInfo = status.getKeyInfo();
         String keyName = keyInfo.getKeyName();
 
+        OzoneKey ozoneKey;
         // Add dir to the dirList
         if (status.isDirectory()) {
-          dirList.add(keyInfo.getKeyName());
           // add trailing slash to represent directory
           keyName = OzoneFSUtils.addTrailingSlashIfNeeded(keyName);
         }
+        ozoneKey = new OzoneKey(keyInfo.getVolumeName(),
+            keyInfo.getBucketName(), keyName,
+            keyInfo.getDataSize(), keyInfo.getCreationTime(),
+            keyInfo.getModificationTime(),
+            keyInfo.getReplicationConfig());
 
-        OzoneKey ozoneKey = new OzoneKey(keyInfo.getVolumeName(),
-                keyInfo.getBucketName(), keyName,
-                keyInfo.getDataSize(), keyInfo.getCreationTime(),
-                keyInfo.getModificationTime(),
-                keyInfo.getReplicationConfig());
+        keysResultList.add(ozoneKey);
 
-        // 5.1) Add to the resultList till it reaches limit batch size.
-        // Once it reaches limit, then add rest of the items to
-        // pendingItemsToBeBatched and this will picked in next batch iteration
-        if (!reachedLimitCacheSize && listCacheSize > keysResultList.size()) {
-          keysResultList.add(ozoneKey);
-          reachedLimitCacheSize = listCacheSize <= keysResultList.size();
-        } else {
-          pendingItemsToBeBatched.add(ozoneKey);
-        }
-      }
-
-      // 6. Push elements in reverse order so that the FS tree traversal will
-      // occur in left-to-right fashion.
-      for (int indx = dirList.size() - 1; indx >= 0; indx--) {
-        String dirPathComponent = dirList.get(indx);
-        stack.push(dirPathComponent);
-      }
-
-      if (reachedLimitCacheSize) {
-        return true;
-      }
-
-      // 7. Pop element and seek for its sub-child path(s). Basically moving
-      // seek pointer to next level(depth) in FS tree.
-      while (!stack.isEmpty()) {
-        keyPrefix = stack.pop();
-        if (getChildrenKeys(keyPrefix, "", keysResultList)) {
-          // reached limit batch size.
+        if (status.isDirectory()) {
+          // Adding in-progress keyPath back to the stack to make sure
+          // all the siblings will be fetched.
+          stack.push(new ImmutablePair<>(keyPrefix, keyInfo.getKeyName()));
+          // Adding current directory to the stack, so that this dir will be
+          // the top element. Moving seek pointer to fetch sub-paths
+          stack.push(new ImmutablePair<>(keyInfo.getKeyName(), ""));
+          // Return it so that the next iteration will be
+          // started using the stacked items.
+          return true;
+        } else if (reachedLimitCacheSize && indx == statuses.size() - 1) {
+          // The last element is a FILE and reaches the listCacheSize.
+          // Now, sets next seek key to this element
+          stack.push(new ImmutablePair<>(keyPrefix, keyInfo.getKeyName()));
+          // Return it so that the next iteration will be
+          // started using the stacked items.
           return true;
         }
       }
@@ -1135,33 +1337,26 @@ public class OzoneBucket extends WithMetadata {
     private void removeStartKeyIfExistsInStatusList(String startKey,
         List<OzoneFileStatus> statuses) {
 
-      if (StringUtils.isNotBlank(startKey) && !statuses.isEmpty()) {
+      if (!statuses.isEmpty()) {
+        String firstElement = statuses.get(0).getKeyInfo().getKeyName();
         String startKeyPath = startKey;
-        if (startKey.endsWith(OZONE_URI_DELIMITER)) {
-          startKeyPath = OzoneFSUtils.removeTrailingSlashIfNeeded(startKey);
+        if (StringUtils.isNotBlank(startKey)) {
+          if (startKey.endsWith(OZONE_URI_DELIMITER)) {
+            startKeyPath = OzoneFSUtils.removeTrailingSlashIfNeeded(startKey);
+          }
         }
-        if (StringUtils.equals(statuses.get(0).getKeyInfo().getKeyName(),
-                startKeyPath)) {
-          // remove the duplicateKey from the list.
+
+        // case-1) remove the startKey from the list as it should be skipped.
+        // case-2) removeStartKey - as the startKey is a placeholder, which is
+        //  managed internally to traverse leaf node's sub-paths.
+        if (StringUtils.equals(firstElement, startKeyPath) ||
+            StringUtils.equals(firstElement, removeStartKey)) {
+
           statuses.remove(0);
         }
       }
     }
 
-    private boolean addAllPendingItemsToResultList(List<OzoneKey> keys) {
-
-      Iterator<OzoneKey> ozoneKeyItr = pendingItemsToBeBatched.iterator();
-      while (ozoneKeyItr.hasNext()) {
-        if (listCacheSize <= keys.size()) {
-          // reached limit batch size.
-          return true;
-        }
-        keys.add(ozoneKeyItr.next());
-        ozoneKeyItr.remove();
-      }
-      return false;
-    }
-
     private void addKeyPrefixInfoToResultList(String keyPrefix,
         String startKey, List<OzoneKey> keysResultList) throws IOException {
 
@@ -1179,21 +1374,33 @@ public class OzoneBucket extends WithMetadata {
         return;
       }
 
-      // TODO: HDDS-4859 will fix the case where startKey not started with
-      //  keyPrefix.
-
-      OzoneFileStatus status = proxy.getOzoneFileStatus(volumeName, name,
-          keyPrefix);
+      OzoneFileStatus status = null;
+      try {
+        status = proxy.getOzoneFileStatus(volumeName, name,
+            keyPrefix);
+      } catch (OMException ome) {
+        if (ome.getResult() == FILE_NOT_FOUND) {
+          // keyPrefix path can't be found and skip adding it to result list
+          return;
+        }
+      }
 
       if (status != null) {
         OmKeyInfo keyInfo = status.getKeyInfo();
         String keyName = keyInfo.getKeyName();
+
         if (status.isDirectory()) {
           // add trailing slash to represent directory
           keyName =
               OzoneFSUtils.addTrailingSlashIfNeeded(keyInfo.getKeyName());
         }
 
+        // removeStartKey - as the startKey is a placeholder, which is
+        // managed internally to traverse leaf node's sub-paths.
+        if (StringUtils.equals(keyName, removeStartKey)) {
+          return;
+        }
+
         OzoneKey ozoneKey = new OzoneKey(keyInfo.getVolumeName(),
             keyInfo.getBucketName(), keyName,
             keyInfo.getDataSize(), keyInfo.getCreationTime(),
@@ -1208,7 +1415,7 @@ public class OzoneBucket extends WithMetadata {
   private class KeyIteratorFactory {
     KeyIterator getKeyIterator(String keyPrefix, String prevKey,
         BucketLayout bType) throws IOException {
-      if (bType.equals(BucketLayout.FILE_SYSTEM_OPTIMIZED)) {
+      if (bType.isFileSystemOptimized()) {
         return new KeyIteratorWithFSO(keyPrefix, prevKey);
       } else {
         return new KeyIterator(keyPrefix, prevKey);
@@ -1219,4 +1426,8 @@ public class OzoneBucket extends WithMetadata {
   public BucketLayout getBucketLayout() {
     return bucketLayout;
   }
+
+  public ReplicationConfig getReplicationConfig() {
+    return this.defaultReplication;
+  }
 }
