@@ -93,6 +93,19 @@ public class OzoneVolume extends WithMetadata {
   private int listCacheSize;
 
   private OzoneObj ozoneObj;
+  /**
+   * Reference count on this Ozone volume.
+   *
+   * When reference count is larger than zero, it indicates that at least one
+   * "lock" is held on the volume by some Ozone feature (e.g. multi-tenancy).
+   * Volume delete operation will be denied in this case, and user should be
+   * prompted to release the lock first via the interface provided by that
+   * feature.
+   *
+   * Volumes created using CLI, ObjectStore API or upgraded from older OM DB
+   * will have reference count set to zero by default.
+   */
+  private long refCount;
 
   /**
    * Constructs OzoneVolume instance.
@@ -133,6 +146,20 @@ public class OzoneVolume extends WithMetadata {
         .setStoreType(OzoneObj.StoreType.OZONE).build();
   }
 
+  /**
+   * @param refCount volume reference count.
+   */
+  @SuppressWarnings("parameternumber")
+  public OzoneVolume(ConfigurationSource conf, ClientProtocol proxy,
+      String name, String admin, String owner, long quotaInBytes,
+      long quotaInNamespace, long usedNamespace, long creationTime,
+      long modificationTime, List<OzoneAcl> acls,
+      Map<String, String> metadata, long refCount) {
+    this(conf, proxy, name, admin, owner, quotaInBytes, quotaInNamespace,
+        usedNamespace, creationTime, modificationTime, acls, metadata);
+    this.refCount = refCount;
+  }
+
   /**
    * @param modificationTime modification time of the volume.
    */
@@ -444,6 +471,9 @@ public class OzoneVolume extends WithMetadata {
     proxy.deleteBucket(name, bucketName);
   }
 
+  public long getRefCount() {
+    return refCount;
+  }
 
   /**
    * An Iterator to iterate over {@link OzoneBucket} list.
@@ -480,7 +510,7 @@ public class OzoneVolume extends WithMetadata {
 
     @Override
     public OzoneBucket next() {
-      if(hasNext()) {
+      if (hasNext()) {
         currentValue = currentIterator.next();
         return currentValue;
       }
