@@ -26,6 +26,7 @@ import java.util.ListIterator;
 import org.apache.hadoop.hdds.client.ReplicationConfig;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.scm.ByteStringConversion;
+import org.apache.hadoop.hdds.scm.ContainerClientMetrics;
 import org.apache.hadoop.hdds.scm.OzoneClientConfig;
 import org.apache.hadoop.hdds.scm.XceiverClientFactory;
 import org.apache.hadoop.hdds.scm.container.common.helpers.ExcludeList;
@@ -44,25 +45,42 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 /**
- * This class manages the stream entries list and handles block allocation
- * from OzoneManager.
+ * A BlockOutputStreamEntryPool manages the communication with OM during writing
+ * a Key to Ozone with {@link KeyOutputStream}.
+ * Block allocation, handling of pre-allocated blocks, and managing stream
+ * entries that represent a writing channel towards DataNodes are the main
+ * responsibility of this class.
  */
 public class BlockOutputStreamEntryPool {
 
   public static final Logger LOG =
       LoggerFactory.getLogger(BlockOutputStreamEntryPool.class);
 
+  /**
+   * List of stream entries that are used to write a block of data.
+   */
   private final List<BlockOutputStreamEntry> streamEntries;
   private final OzoneClientConfig config;
+  /**
+   * The actual stream entry we are writing into. Note that a stream entry is
+   * allowed to manage more streams, as for example in the EC write case, where
+   * an entry represents an EC block group.
+   */
   private int currentStreamIndex;
   private final OzoneManagerProtocol omClient;
   private final OmKeyArgs keyArgs;
   private final XceiverClientFactory xceiverClientFactory;
   private final String requestID;
+  /**
+   * A {@link BufferPool} shared between all
+   * {@link org.apache.hadoop.hdds.scm.storage.BlockOutputStream}s managed by
+   * the entries in the pool.
+   */
   private final BufferPool bufferPool;
   private OmMultipartCommitUploadPartInfo commitUploadPartInfo;
   private final long openID;
   private final ExcludeList excludeList;
+  private final ContainerClientMetrics clientMetrics;
 
   @SuppressWarnings({"parameternumber", "squid:S00107"})
   public BlockOutputStreamEntryPool(
@@ -72,7 +90,8 @@ public class BlockOutputStreamEntryPool {
       String uploadID, int partNumber,
       boolean isMultipart, OmKeyInfo info,
       boolean unsafeByteBufferConversion,
-      XceiverClientFactory xceiverClientFactory, long openID
+      XceiverClientFactory xceiverClientFactory, long openID,
+      ContainerClientMetrics clientMetrics
   ) {
     this.config = config;
     this.xceiverClientFactory = xceiverClientFactory;
@@ -86,7 +105,7 @@ public class BlockOutputStreamEntryPool {
         .setMultipartUploadPartNumber(partNumber).build();
     this.requestID = requestId;
     this.openID = openID;
-    this.excludeList = new ExcludeList();
+    this.excludeList = createExcludeList();
 
     this.bufferPool =
         new BufferPool(config.getStreamBufferSize(),
@@ -94,15 +113,14 @@ public class BlockOutputStreamEntryPool {
                 .getStreamBufferSize()),
             ByteStringConversion
                 .createByteBufferConversion(unsafeByteBufferConversion));
+    this.clientMetrics = clientMetrics;
   }
 
-  /**
-   * A constructor for testing purpose only.
-   *
-   * @see KeyOutputStream#KeyOutputStream()
-   */
-  @VisibleForTesting
-  BlockOutputStreamEntryPool() {
+  ExcludeList createExcludeList() {
+    return new ExcludeList();
+  }
+
+  BlockOutputStreamEntryPool(ContainerClientMetrics clientMetrics) {
     streamEntries = new ArrayList<>();
     omClient = null;
     keyArgs = null;
@@ -120,6 +138,7 @@ public class BlockOutputStreamEntryPool {
     currentStreamIndex = 0;
     openID = -1;
     excludeList = new ExcludeList();
+    this.clientMetrics = clientMetrics;
   }
 
   /**
@@ -145,9 +164,17 @@ public class BlockOutputStreamEntryPool {
     }
   }
 
-  private void addKeyLocationInfo(OmKeyLocationInfo subKeyInfo) {
-    Preconditions.checkNotNull(subKeyInfo.getPipeline());
-    BlockOutputStreamEntry.Builder builder =
+  /**
+   * Method to create a stream entry instance based on the
+   * {@link OmKeyLocationInfo}.
+   * If implementations require additional data to create the entry, they need
+   * to get that data before starting to create entries.
+   * @param subKeyInfo the {@link OmKeyLocationInfo} object that describes the
+   *                   key to be written.
+   * @return a BlockOutputStreamEntry instance that handles how data is written.
+   */
+  BlockOutputStreamEntry createStreamEntry(OmKeyLocationInfo subKeyInfo) {
+    return
         new BlockOutputStreamEntry.Builder()
             .setBlockID(subKeyInfo.getBlockID())
             .setKey(keyArgs.getKeyName())
@@ -156,22 +183,45 @@ public class BlockOutputStreamEntryPool {
             .setConfig(config)
             .setLength(subKeyInfo.getLength())
             .setBufferPool(bufferPool)
-            .setToken(subKeyInfo.getToken());
-    streamEntries.add(builder.build());
+            .setToken(subKeyInfo.getToken())
+            .setClientMetrics(clientMetrics)
+            .build();
   }
 
-  public List<OmKeyLocationInfo> getLocationInfoList()  {
-    List<OmKeyLocationInfo> locationInfoList = new ArrayList<>();
-    for (BlockOutputStreamEntry streamEntry : streamEntries) {
-      long length = streamEntry.getCurrentPosition();
+  private void addKeyLocationInfo(OmKeyLocationInfo subKeyInfo) {
+    Preconditions.checkNotNull(subKeyInfo.getPipeline());
+    streamEntries.add(createStreamEntry(subKeyInfo));
+  }
 
+  /**
+   * Returns the list of {@link OmKeyLocationInfo} object that describes to OM
+   * where the blocks of the key have been written.
+   * @return the location info list of written blocks.
+   */
+  @VisibleForTesting
+  public List<OmKeyLocationInfo> getLocationInfoList() {
+    List<OmKeyLocationInfo> locationInfoList;
+    List<OmKeyLocationInfo> currBlocksLocationInfoList =
+        getOmKeyLocationInfos(streamEntries);
+    locationInfoList = currBlocksLocationInfoList;
+    return locationInfoList;
+  }
+
+  private List<OmKeyLocationInfo> getOmKeyLocationInfos(
+      List<BlockOutputStreamEntry> streams) {
+    List<OmKeyLocationInfo> locationInfoList = new ArrayList<>();
+    for (BlockOutputStreamEntry streamEntry : streams) {
+      long length = streamEntry.getCurrentPosition();
       // Commit only those blocks to OzoneManager which are not empty
       if (length != 0) {
         OmKeyLocationInfo info =
-            new OmKeyLocationInfo.Builder().setBlockID(streamEntry.getBlockID())
-                .setLength(streamEntry.getCurrentPosition()).setOffset(0)
+            new OmKeyLocationInfo.Builder()
+                .setBlockID(streamEntry.getBlockID())
+                .setLength(streamEntry.getCurrentPosition())
+                .setOffset(0)
                 .setToken(streamEntry.getToken())
-                .setPipeline(streamEntry.getPipeline()).build();
+                .setPipeline(streamEntry.getPipeline())
+                .build();
         locationInfoList.add(info);
       }
       if (LOG.isDebugEnabled()) {
@@ -184,6 +234,23 @@ public class BlockOutputStreamEntryPool {
     return locationInfoList;
   }
 
+  /**
+   * Retrieves the {@link BufferPool} instance shared between managed block
+   * output stream entries.
+   * @return the shared buffer pool.
+   */
+  public BufferPool getBufferPool() {
+    return this.bufferPool;
+  }
+
+  OzoneClientConfig getConfig() {
+    return config;
+  }
+
+  ContainerClientMetrics getClientMetrics() {
+    return clientMetrics;
+  }
+
   /**
    * Discards the subsequent pre allocated blocks and removes the streamEntries
    * from the streamEntries list for the container which is closed.
@@ -211,10 +278,12 @@ public class BlockOutputStreamEntryPool {
     }
   }
 
+  @VisibleForTesting
   List<BlockOutputStreamEntry> getStreamEntries() {
     return streamEntries;
   }
 
+  @VisibleForTesting
   XceiverClientFactory getXceiverClientFactory() {
     return xceiverClientFactory;
   }
@@ -224,8 +293,8 @@ public class BlockOutputStreamEntryPool {
   }
 
   long getKeyLength() {
-    return streamEntries.stream().mapToLong(
-        BlockOutputStreamEntry::getCurrentPosition).sum();
+    return streamEntries.stream()
+        .mapToLong(BlockOutputStreamEntry::getCurrentPosition).sum();
   }
   /**
    * Contact OM to get a new block. Set the new block with the index (e.g.
@@ -244,12 +313,22 @@ public class BlockOutputStreamEntryPool {
     addKeyLocationInfo(subKeyInfo);
   }
 
-
+  /**
+   * Commits the keys with Ozone Manager(s).
+   * At the end of the write committing the key from client side lets the OM
+   * know that the data has been written and to where. With this info OM can
+   * register the metadata stored in OM and SCM about the key that was written.
+   * @param offset the offset on which the key writer stands at the time of
+   *               finishing data writes. (Has to be equal and checked against
+   *               the actual length written by the stream entries.)
+   * @throws IOException in case there is an I/O problem during communication.
+   */
   void commitKey(long offset) throws IOException {
     if (keyArgs != null) {
       // in test, this could be null
       long length = getKeyLength();
-      Preconditions.checkArgument(offset == length);
+      Preconditions.checkArgument(offset == length,
+          "Epected offset: " + offset + " expected len: " + length);
       keyArgs.setDataSize(length);
       keyArgs.setLocationInfoList(getLocationInfoList());
       // When the key is multipart upload part file upload, we should not
@@ -266,7 +345,7 @@ public class BlockOutputStreamEntryPool {
     }
   }
 
-  public BlockOutputStreamEntry getCurrentStreamEntry() {
+  BlockOutputStreamEntry getCurrentStreamEntry() {
     if (streamEntries.isEmpty() || streamEntries.size() <= currentStreamIndex) {
       return null;
     } else {
@@ -274,6 +353,12 @@ public class BlockOutputStreamEntryPool {
     }
   }
 
+  /**
+   * Allocates a new block with OM if the current stream is closed, and new
+   * writes are to be handled.
+   * @return the new current open stream to write to
+   * @throws IOException if the block allocation failed.
+   */
   BlockOutputStreamEntry allocateBlockIfNeeded() throws IOException {
     BlockOutputStreamEntry streamEntry = getCurrentStreamEntry();
     if (streamEntry != null && streamEntry.isClosed()) {
