@@ -56,6 +56,10 @@ public enum OMAction implements AuditAction {
   LIST_MULTIPART_UPLOAD_PARTS,
   LIST_MULTIPART_UPLOADS,
   ABORT_MULTIPART_UPLOAD,
+  GET_DELEGATION_TOKEN,
+  RENEW_DELEGATION_TOKEN,
+  CANCEL_DELEGATION_TOKEN,
+  GET_SERVICE_LIST,
 
   //ACL Actions
   ADD_ACL,
@@ -71,7 +75,20 @@ public enum OMAction implements AuditAction {
   LIST_STATUS,
 
   GET_S3_SECRET,
-  REVOKE_S3_SECRET;
+  SET_S3_SECRET,
+  REVOKE_S3_SECRET,
+
+  CREATE_TENANT,
+  DELETE_TENANT,
+  LIST_TENANT,
+
+  TENANT_GET_USER_INFO,
+  TENANT_ASSIGN_USER_ACCESSID,
+  TENANT_REVOKE_USER_ACCESSID,
+
+  TENANT_ASSIGN_ADMIN,
+  TENANT_REVOKE_ADMIN,
+  TENANT_LIST_USER;
 
   @Override
   public String getAction() {
