@@ -242,6 +242,21 @@ public class OMException extends IOException {
     UPDATE_LAYOUT_VERSION_FAILED,
     LAYOUT_FEATURE_FINALIZATION_FAILED,
     PREPARE_FAILED,
-    NOT_SUPPORTED_OPERATION_WHEN_PREPARED
+    NOT_SUPPORTED_OPERATION_WHEN_PREPARED,
+    NOT_SUPPORTED_OPERATION_PRIOR_FINALIZATION,
+
+    TENANT_NOT_FOUND,
+    TENANT_ALREADY_EXISTS, INVALID_TENANT_ID,
+
+    ACCESS_ID_NOT_FOUND,
+    TENANT_USER_ACCESS_ID_ALREADY_EXISTS,
+    INVALID_TENANT_USERNAME,
+    INVALID_ACCESS_ID,
+    TENANT_AUTHORIZER_ERROR,
+
+    VOLUME_IS_REFERENCED,
+    TENANT_NOT_EMPTY,
+
+    FEATURE_NOT_ENABLED
   }
 }
