@@ -27,7 +27,13 @@ import java.util.Map;
 import java.util.Objects;
 import java.util.stream.Collectors;
 
+import org.apache.hadoop.hdds.client.DefaultReplicationConfig;
+import org.apache.hadoop.hdds.client.ECReplicationConfig;
+import org.apache.hadoop.hdds.client.ReplicationConfig;
+import org.apache.hadoop.hdds.client.ReplicationFactor;
+import org.apache.hadoop.hdds.client.ReplicationType;
 import org.apache.hadoop.hdds.protocol.StorageType;
+import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.ozone.OzoneAcl;
 import org.apache.hadoop.ozone.OzoneConsts;
 import org.apache.hadoop.ozone.audit.Auditable;
@@ -75,6 +81,11 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
    */
   private BucketEncryptionKeyInfo bekInfo;
 
+  /**
+   * Optional default replication for bucket.
+   */
+  private DefaultReplicationConfig defaultReplicationConfig;
+
   private final String sourceVolume;
 
   private final String sourceBucket;
@@ -91,6 +102,8 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
    */
   private BucketLayout bucketLayout;
 
+  private String owner;
+
   /**
    * Private constructor, constructed via builder.
    * @param volumeName - Volume name.
@@ -107,6 +120,9 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
    * @param usedBytes - Bucket Quota Usage in bytes.
    * @param quotaInBytes Bucket quota in bytes.
    * @param quotaInNamespace Bucket quota in counts.
+   * @param bucketLayout bucket layout.
+   * @param owner owner of the bucket.
+   * @param defaultReplicationConfig default replication config.
    * @param bucketLayout Bucket Layout.
    */
   @SuppressWarnings("checkstyle:ParameterNumber")
@@ -127,7 +143,9 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
       long usedNamespace,
       long quotaInBytes,
       long quotaInNamespace,
-      BucketLayout bucketLayout) {
+      BucketLayout bucketLayout,
+      String owner,
+      DefaultReplicationConfig defaultReplicationConfig) {
     this.volumeName = volumeName;
     this.bucketName = bucketName;
     this.acls = acls;
@@ -146,6 +164,8 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
     this.quotaInBytes = quotaInBytes;
     this.quotaInNamespace = quotaInNamespace;
     this.bucketLayout = bucketLayout;
+    this.owner = owner;
+    this.defaultReplicationConfig = defaultReplicationConfig;
   }
 
   /**
@@ -245,12 +265,22 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
 
   /**
    * Returns the Bucket Layout.
+   *
    * @return BucketLayout.
    */
   public BucketLayout getBucketLayout() {
     return bucketLayout;
   }
 
+  /**
+   * Returns bucket EC replication config.
+   *
+   * @return EC replication config.
+   */
+  public DefaultReplicationConfig getDefaultReplicationConfig() {
+    return defaultReplicationConfig;
+  }
+
   public String getSourceVolume() {
     return sourceVolume;
   }
@@ -288,6 +318,18 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
     return sourceVolume != null && sourceBucket != null;
   }
 
+  public String getOwner() {
+    return owner;
+  }
+
+  public void setModificationTime(long modificationTime) {
+    this.modificationTime = modificationTime;
+  }
+
+  public void setOwner(String ownerName) {
+    this.owner = ownerName;
+  }
+
   /**
    * Returns new builder class that builds a OmBucketInfo.
    *
@@ -302,6 +344,7 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
     Map<String, String> auditMap = new LinkedHashMap<>();
     auditMap.put(OzoneConsts.VOLUME, this.volumeName);
     auditMap.put(OzoneConsts.BUCKET, this.bucketName);
+    auditMap.put(OzoneConsts.BUCKET_LAYOUT, String.valueOf(this.bucketLayout));
     auditMap.put(OzoneConsts.GDPR_FLAG,
         this.metadata.get(OzoneConsts.GDPR_FLAG));
     auditMap.put(OzoneConsts.ACLS,
@@ -340,6 +383,10 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
         acl.getName(), (BitSet) acl.getAclBitSet().clone(),
         acl.getAclScope())));
 
+    if (defaultReplicationConfig != null) {
+      builder.setDefaultReplicationConfig(defaultReplicationConfig.copy());
+    }
+
     return builder.build();
   }
 
@@ -362,7 +409,22 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
         .setUsedNamespace(usedNamespace)
         .setQuotaInBytes(quotaInBytes)
         .setQuotaInNamespace(quotaInNamespace)
-        .setBucketLayout(bucketLayout);
+        .setBucketLayout(bucketLayout)
+        .setOwner(owner)
+        .setDefaultReplicationConfig(defaultReplicationConfig);
+  }
+
+  public void setDefaultReplicationConfig(ReplicationConfig replicationConfig) {
+    this.defaultReplicationConfig = new DefaultReplicationConfig(
+        ReplicationType.fromProto(replicationConfig.getReplicationType()),
+        replicationConfig
+            .getReplicationType() == HddsProtos.ReplicationType.EC ?
+            null :
+            ReplicationFactor.valueOf(replicationConfig.getRequiredNodes()),
+        replicationConfig
+            .getReplicationType() == HddsProtos.ReplicationType.EC ?
+            ((ECReplicationConfig) replicationConfig) :
+            null);
   }
 
   /**
@@ -387,6 +449,8 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
     private long quotaInBytes;
     private long quotaInNamespace;
     private BucketLayout bucketLayout;
+    private String owner;
+    private DefaultReplicationConfig defaultReplicationConfig;
 
     public Builder() {
       //Default values
@@ -510,6 +574,17 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
       return this;
     }
 
+    public Builder setOwner(String ownerName) {
+      this.owner = ownerName;
+      return this;
+    }
+
+    public Builder setDefaultReplicationConfig(
+        DefaultReplicationConfig defaultReplConfig) {
+      this.defaultReplicationConfig = defaultReplConfig;
+      return this;
+    }
+
     /**
      * Constructs the OmBucketInfo.
      * @return instance of OmBucketInfo.
@@ -520,11 +595,11 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
       Preconditions.checkNotNull(acls);
       Preconditions.checkNotNull(isVersionEnabled);
       Preconditions.checkNotNull(storageType);
-
       return new OmBucketInfo(volumeName, bucketName, acls, isVersionEnabled,
           storageType, creationTime, modificationTime, objectID, updateID,
           metadata, bekInfo, sourceVolume, sourceBucket, usedBytes,
-          usedNamespace, quotaInBytes, quotaInNamespace, bucketLayout);
+          usedNamespace, quotaInBytes, quotaInNamespace, bucketLayout, owner,
+          defaultReplicationConfig);
     }
   }
 
@@ -553,12 +628,18 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
     if (bekInfo != null && bekInfo.getKeyName() != null) {
       bib.setBeinfo(OMPBHelper.convert(bekInfo));
     }
+    if (defaultReplicationConfig != null) {
+      bib.setDefaultReplicationConfig(defaultReplicationConfig.toProto());
+    }
     if (sourceVolume != null) {
       bib.setSourceVolume(sourceVolume);
     }
     if (sourceBucket != null) {
       bib.setSourceBucket(sourceBucket);
     }
+    if (owner != null) {
+      bib.setOwner(owner);
+    }
     return bib.build();
   }
 
@@ -598,6 +679,10 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
       obib.setBucketLayout(
           BucketLayout.fromProto(bucketInfo.getBucketLayout()));
     }
+    if (bucketInfo.hasDefaultReplicationConfig()) {
+      obib.setDefaultReplicationConfig(
+          OMPBHelper.convert(bucketInfo.getDefaultReplicationConfig()));
+    }
     if (bucketInfo.hasObjectID()) {
       obib.setObjectID(bucketInfo.getObjectID());
     }
@@ -617,6 +702,9 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
     if (bucketInfo.hasSourceBucket()) {
       obib.setSourceBucket(bucketInfo.getSourceBucket());
     }
+    if (bucketInfo.hasOwner()) {
+      obib.setOwner(bucketInfo.getOwner());
+    }
     return obib.build();
   }
 
@@ -637,6 +725,7 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
         ", quotaInBytes='" + quotaInBytes + "'" +
         ", quotaInNamespace='" + quotaInNamespace + "'" +
         ", bucketLayout='" + bucketLayout + '\'' +
+        ", defaultReplicationConfig='" + defaultReplicationConfig + '\'' +
         sourceInfo +
         '}';
   }
@@ -664,7 +753,9 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
         Objects.equals(sourceVolume, that.sourceVolume) &&
         Objects.equals(sourceBucket, that.sourceBucket) &&
         Objects.equals(metadata, that.metadata) &&
-        Objects.equals(bekInfo, that.bekInfo);
+        Objects.equals(bekInfo, that.bekInfo) &&
+        Objects.equals(owner, that.owner) &&
+        Objects.equals(defaultReplicationConfig, this.defaultReplicationConfig);
   }
 
   @Override
@@ -692,6 +783,8 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
         ", quotaInBytes=" + quotaInBytes +
         ", quotaInNamespace=" + quotaInNamespace +
         ", bucketLayout=" + bucketLayout +
+        ", owner=" + owner +
+        ", defaultReplicationConfig=" + defaultReplicationConfig +
         '}';
   }
 }
