@@ -27,6 +27,8 @@ import org.apache.hadoop.ozone.OzoneAcl;
 import org.apache.hadoop.ozone.om.OMConfigKeys;
 import org.apache.hadoop.ozone.om.exceptions.OMException;
 import org.apache.hadoop.ozone.om.helpers.DBUpdates;
+import org.apache.hadoop.ozone.om.helpers.DeleteTenantState;
+import org.apache.hadoop.ozone.om.helpers.KeyInfoWithVolumeContext;
 import org.apache.hadoop.ozone.om.helpers.OmBucketArgs;
 import org.apache.hadoop.ozone.om.helpers.OmBucketInfo;
 import org.apache.hadoop.ozone.om.helpers.OmDeleteKeys;
@@ -40,18 +42,24 @@ import org.apache.hadoop.ozone.om.helpers.OmMultipartUploadCompleteList;
 import org.apache.hadoop.ozone.om.helpers.OmMultipartUploadList;
 import org.apache.hadoop.ozone.om.helpers.OmMultipartUploadListParts;
 import org.apache.hadoop.ozone.om.helpers.OmRenameKeys;
+import org.apache.hadoop.ozone.om.helpers.OmTenantArgs;
 import org.apache.hadoop.ozone.om.helpers.OmVolumeArgs;
 import org.apache.hadoop.ozone.om.helpers.OpenKeySession;
 import org.apache.hadoop.ozone.om.helpers.OzoneFileStatus;
 import org.apache.hadoop.ozone.om.helpers.RepeatedOmKeyInfo;
 import org.apache.hadoop.ozone.om.helpers.S3SecretValue;
+import org.apache.hadoop.ozone.om.helpers.S3VolumeContext;
 import org.apache.hadoop.ozone.om.helpers.ServiceInfo;
 import org.apache.hadoop.ozone.om.helpers.ServiceInfoEx;
+import org.apache.hadoop.ozone.om.helpers.TenantStateList;
+import org.apache.hadoop.ozone.om.helpers.TenantUserInfoValue;
+import org.apache.hadoop.ozone.om.helpers.TenantUserList;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.OzoneAclInfo;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.PrepareStatusResponse;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.PrepareStatusResponse.PrepareStatus;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.CancelPrepareResponse;
+import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.EchoRPCResponse;
 import org.apache.hadoop.ozone.security.OzoneDelegationTokenSelector;
 import org.apache.hadoop.ozone.security.acl.OzoneObj;
 import org.apache.hadoop.ozone.upgrade.UpgradeFinalizer.StatusAndMessages;
@@ -78,7 +86,11 @@ public interface OzoneManagerProtocol
    * @param args - Arguments to create Volume.
    * @throws IOException
    */
-  void createVolume(OmVolumeArgs args) throws IOException;
+  default void createVolume(OmVolumeArgs args) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
+
 
   /**
    * Changes the owner of a volume.
@@ -88,7 +100,11 @@ public interface OzoneManagerProtocol
    *         already the owner.
    * @throws IOException
    */
-  boolean setOwner(String volume, String owner) throws IOException;
+  default boolean setOwner(String volume, String owner) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
+
 
   /**
    * Changes the Quota on a volume.
@@ -97,8 +113,12 @@ public interface OzoneManagerProtocol
    * @param quotaInBytes - Volume quota in bytes.
    * @throws IOException
    */
-  void setQuota(String volume, long quotaInNamespace, long quotaInBytes)
-      throws IOException;
+  default void setQuota(String volume, long quotaInNamespace, long quotaInBytes)
+      throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
+
 
   /**
    * Checks if the specified user can access this volume.
@@ -108,8 +128,11 @@ public interface OzoneManagerProtocol
    *         false otherwise
    * @throws IOException
    */
-  boolean checkVolumeAccess(String volume, OzoneAclInfo userAcl)
-      throws IOException;
+  default boolean checkVolumeAccess(String volume, OzoneAclInfo userAcl)
+      throws IOException {
+    throw new UnsupportedOperationException("This operation is not supported.");
+  }
+
 
   /**
    * Gets the volume information.
@@ -124,7 +147,11 @@ public interface OzoneManagerProtocol
    * @param volume - Name of the volume.
    * @throws IOException
    */
-  void deleteVolume(String volume) throws IOException;
+  default void deleteVolume(String volume) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
+
 
   /**
    * Lists volumes accessible by a specific user.
@@ -154,7 +181,11 @@ public interface OzoneManagerProtocol
    * @param bucketInfo - BucketInfo to create Bucket.
    * @throws IOException
    */
-  void createBucket(OmBucketInfo bucketInfo) throws IOException;
+  default void createBucket(OmBucketInfo bucketInfo) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
+
 
   /**
    * Gets the bucket information.
@@ -171,7 +202,23 @@ public interface OzoneManagerProtocol
    * @param args - BucketArgs.
    * @throws IOException
    */
-  void setBucketProperty(OmBucketArgs args) throws IOException;
+  default void setBucketProperty(OmBucketArgs args) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
+
+  /**
+   * Changes the owner of a bucket.
+   * @param args  - OMBucketArgs
+   * @return true if operation succeeded, false if specified user is
+   *         already the owner.
+   * @throws IOException
+   */
+  default boolean setBucketOwner(OmBucketArgs args) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
+
 
   /**
    * Open the given key and return an open key session.
@@ -180,7 +227,10 @@ public interface OzoneManagerProtocol
    * @return OpenKeySession instance that client uses to talk to container.
    * @throws IOException
    */
-  OpenKeySession openKey(OmKeyArgs args) throws IOException;
+  default OpenKeySession openKey(OmKeyArgs args) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
 
   /**
    * Commit a key. This will make the change from the client visible. The client
@@ -190,7 +240,12 @@ public interface OzoneManagerProtocol
    * @param clientID the client identification
    * @throws IOException
    */
-  void commitKey(OmKeyArgs args, long clientID) throws IOException;
+  default void commitKey(OmKeyArgs args, long clientID)
+      throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
+
 
   /**
    * Allocate a new block, it is assumed that the client is having an open key
@@ -203,9 +258,11 @@ public interface OzoneManagerProtocol
    * @return an allocated block
    * @throws IOException
    */
-  OmKeyLocationInfo allocateBlock(OmKeyArgs args, long clientID,
-      ExcludeList excludeList) throws IOException;
-
+  default OmKeyLocationInfo allocateBlock(OmKeyArgs args, long clientID,
+      ExcludeList excludeList) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
 
   /**
    * Look up for the container of an existing key.
@@ -216,13 +273,29 @@ public interface OzoneManagerProtocol
    */
   OmKeyInfo lookupKey(OmKeyArgs args) throws IOException;
 
+  /**
+   * Lookup for the container of an existing key.
+   *
+   * @param args the args of the key.
+   * @param assumeS3Context if true OM will automatically lookup the S3
+   *                        volume context and includes in the response.
+   * @return KeyInfoWithVolumeContext includes info that client uses to talk
+   *         to containers and S3 volume context info if assumeS3Context is set.
+   */
+  KeyInfoWithVolumeContext getKeyInfo(OmKeyArgs args, boolean assumeS3Context)
+      throws IOException;
+
   /**
    * Rename an existing key within a bucket.
    * @param args the args of the key.
    * @param toKeyName New name to be used for the Key
    * @throws IOException
    */
-  void renameKey(OmKeyArgs args, String toKeyName) throws IOException;
+  default void renameKey(OmKeyArgs args, String toKeyName) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
+
 
   /**
    * Rename existing keys within a bucket.
@@ -230,7 +303,11 @@ public interface OzoneManagerProtocol
    *                     and fromKey name toKey info Map.
    * @throws IOException
    */
-  void renameKeys(OmRenameKeys omRenameKeys) throws IOException;
+  default void renameKeys(OmRenameKeys omRenameKeys) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
+
 
   /**
    * Deletes an existing key.
@@ -238,7 +315,11 @@ public interface OzoneManagerProtocol
    * @param args the args of the key.
    * @throws IOException
    */
-  void deleteKey(OmKeyArgs args) throws IOException;
+  default void deleteKey(OmKeyArgs args) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
+
 
   /**
    * Deletes existing key/keys. This interface supports delete
@@ -248,7 +329,11 @@ public interface OzoneManagerProtocol
    * @param deleteKeys
    * @throws IOException
    */
-  void deleteKeys(OmDeleteKeys deleteKeys) throws IOException;
+  default void deleteKeys(OmDeleteKeys deleteKeys) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
+
 
   /**
    * Deletes an existing empty bucket from volume.
@@ -256,7 +341,11 @@ public interface OzoneManagerProtocol
    * @param bucket - Name of the bucket.
    * @throws IOException
    */
-  void deleteBucket(String volume, String bucket) throws IOException;
+  default void deleteBucket(String volume, String bucket) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
+
 
   /**
    * Returns a list of buckets represented by {@link OmBucketInfo}
@@ -316,6 +405,18 @@ public interface OzoneManagerProtocol
 
   ServiceInfoEx getServiceInfo() throws IOException;
 
+  /**
+   * Triggers Ranger background sync task immediately.
+   *
+   * Requires Ozone administrator privilege.
+   *
+   * @param noWait set to true if client won't wait for the result.
+   * @return true if noWait is true or when task completed successfully,
+   *         false otherwise.
+   * @throws IOException OMException (e.g. PERMISSION_DENIED)
+   */
+  boolean triggerRangerBGSync(boolean noWait) throws IOException;
+
   /**
    * Initiate metadata upgrade finalization.
    * This method when called, initiates finalization of Ozone Manager metadata
@@ -389,7 +490,12 @@ public interface OzoneManagerProtocol
    * @return MultipartInfo
    * @throws IOException
    */
-  OmMultipartInfo initiateMultipartUpload(OmKeyArgs keyArgs) throws IOException;
+  default OmMultipartInfo initiateMultipartUpload(OmKeyArgs keyArgs)
+      throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
+
 
 
   /**
@@ -399,8 +505,12 @@ public interface OzoneManagerProtocol
    * @return OmMultipartCommitUploadPartInfo
    * @throws IOException
    */
-  OmMultipartCommitUploadPartInfo commitMultipartUploadPart(
-      OmKeyArgs omKeyArgs, long clientID) throws IOException;
+  default OmMultipartCommitUploadPartInfo commitMultipartUploadPart(
+      OmKeyArgs omKeyArgs, long clientID) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
+
 
   /**
    * Complete Multipart upload Request.
@@ -409,16 +519,24 @@ public interface OzoneManagerProtocol
    * @return OmMultipartUploadCompleteInfo
    * @throws IOException
    */
-  OmMultipartUploadCompleteInfo completeMultipartUpload(
+  default OmMultipartUploadCompleteInfo completeMultipartUpload(
       OmKeyArgs omKeyArgs, OmMultipartUploadCompleteList multipartUploadList)
-      throws IOException;
+      throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
+
 
   /**
    * Abort multipart upload.
    * @param omKeyArgs
    * @throws IOException
    */
-  void abortMultipartUpload(OmKeyArgs omKeyArgs) throws IOException;
+  default void abortMultipartUpload(OmKeyArgs omKeyArgs) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
+
 
   /**
    * Returns list of parts of a multipart upload key.
@@ -446,14 +564,142 @@ public interface OzoneManagerProtocol
    * @return S3SecretValue
    * @throws IOException
    */
-  S3SecretValue getS3Secret(String kerberosID) throws IOException;
+  default S3SecretValue getS3Secret(String kerberosID) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
+
+  /**
+   * Gets s3Secret for given kerberos user.
+   * @param kerberosID
+   * @param createIfNotExist
+   * @return S3SecretValue
+   * @throws IOException
+   */
+  default S3SecretValue getS3Secret(String kerberosID, boolean createIfNotExist)
+      throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach");
+  };
+
+  /**
+   * Set secret key for accessId.
+   * @param accessId
+   * @param secretKey
+   * @return S3SecretValue
+   * @throws IOException
+   */
+  default S3SecretValue setS3Secret(String accessId, String secretKey)
+      throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach");
+  }
 
   /**
    * Revokes s3Secret of given kerberos user.
    * @param kerberosID
    * @throws IOException
    */
-  void revokeS3Secret(String kerberosID) throws IOException;
+  default void revokeS3Secret(String kerberosID) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
+
+  /**
+   * Create a tenant.
+   * @param omTenantArgs OmTenantArgs
+   * @throws IOException
+   */
+  default void createTenant(OmTenantArgs omTenantArgs) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach");
+  }
+
+  /**
+   * Delete a tenant.
+   * @param tenantId tenant name.
+   * @return DeleteTenantResponse
+   * @throws IOException
+   */
+  default DeleteTenantState deleteTenant(String tenantId)
+      throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach");
+  }
+
+  /**
+   * Assign user to a tenant.
+   * @param username user name to be assigned.
+   * @param tenantId tenant name.
+   * @param accessId access ID.
+   * @return S3SecretValue
+   * @throws IOException
+   */
+  default S3SecretValue tenantAssignUserAccessId(String username,
+                                                 String tenantId,
+                                                 String accessId)
+      throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach");
+  }
+
+  S3VolumeContext getS3VolumeContext() throws IOException;
+
+  /**
+   * Revoke user accessId to a tenant.
+   * @param accessId accessId to be revoked.
+   * @throws IOException
+   */
+  default void tenantRevokeUserAccessId(String accessId) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach");
+  }
+
+  /**
+   * Assign admin role to a user identified by an accessId in a tenant.
+   * @param accessId access ID.
+   * @param tenantId tenant name.
+   * @param delegated true if making delegated admin.
+   * @throws IOException
+   */
+  default void tenantAssignAdmin(String accessId,
+                                 String tenantId,
+                                 boolean delegated)
+      throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach");
+  }
+
+  /**
+   * Revoke admin role of an accessId in a tenant.
+   * @param accessId access ID.
+   * @param tenantId tenant name.
+   * @throws IOException
+   */
+  default void tenantRevokeAdmin(String accessId,
+                                 String tenantId) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach");
+  }
+
+  /**
+   * Get tenant info for a user.
+   * @param userPrincipal Kerberos principal of a user.
+   * @return TenantUserInfo
+   * @throws IOException
+   */
+  TenantUserInfoValue tenantGetUserInfo(String userPrincipal)
+      throws IOException;
+
+  TenantUserList listUsersInTenant(String tenantId, String prefix)
+      throws IOException;
+
+  /**
+   * List tenants.
+   * @return TenantStateList
+   * @throws IOException
+   */
+  TenantStateList listTenant() throws IOException;
 
   /**
    * OzoneFS api to get file status for an entry.
@@ -476,7 +722,11 @@ public interface OzoneManagerProtocol
    * @throws IOException if there is error in the db
    *                     invalid arguments
    */
-  void createDirectory(OmKeyArgs args) throws IOException;
+  default void createDirectory(OmKeyArgs args) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
+
 
   /**
    * OzoneFS api to creates an output stream for a file.
@@ -492,8 +742,12 @@ public interface OzoneManagerProtocol
    * @throws IOException if there is error in the db
    *                     invalid arguments
    */
-  OpenKeySession createFile(OmKeyArgs keyArgs, boolean overWrite,
-      boolean recursive) throws IOException;
+  default OpenKeySession createFile(OmKeyArgs keyArgs, boolean overWrite,
+      boolean recursive) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
+
 
   /**
    * OzoneFS api to lookup for a file.
@@ -520,6 +774,24 @@ public interface OzoneManagerProtocol
   List<OzoneFileStatus> listStatus(OmKeyArgs keyArgs, boolean recursive,
       String startKey, long numEntries) throws IOException;
 
+  /**
+   * List the status for a file or a directory and its contents.
+   *
+   * @param keyArgs    Key args
+   * @param recursive  For a directory if true all the descendants of a
+   *                   particular directory are listed
+   * @param startKey   Key from which listing needs to start. If startKey exists
+   *                   its status is included in the final list.
+   * @param numEntries Number of entries to list from the start key
+   * @param allowPartialPrefixes if partial prefixes should be allowed,
+   *                             this is needed in context of ListKeys
+   * @return list of file status
+   */
+  List<OzoneFileStatus> listStatus(OmKeyArgs keyArgs, boolean recursive,
+                                   String startKey, long numEntries,
+                                   boolean allowPartialPrefixes)
+      throws IOException;
+
   /**
    * Add acl for Ozone object. Return true if acl is added successfully else
    * false.
@@ -528,7 +800,11 @@ public interface OzoneManagerProtocol
    *
    * @throws IOException if there is error.
    * */
-  boolean addAcl(OzoneObj obj, OzoneAcl acl) throws IOException;
+  default boolean addAcl(OzoneObj obj, OzoneAcl acl) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
+
 
   /**
    * Remove acl for Ozone object. Return true if acl is removed successfully
@@ -538,7 +814,11 @@ public interface OzoneManagerProtocol
    *
    * @throws IOException if there is error.
    * */
-  boolean removeAcl(OzoneObj obj, OzoneAcl acl) throws IOException;
+  default boolean removeAcl(OzoneObj obj, OzoneAcl acl) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
+
 
   /**
    * Acls to be set for given Ozone object. This operations reset ACL for
@@ -548,7 +828,11 @@ public interface OzoneManagerProtocol
    *
    * @throws IOException if there is error.
    * */
-  boolean setAcl(OzoneObj obj, List<OzoneAcl> acls) throws IOException;
+  default boolean setAcl(OzoneObj obj, List<OzoneAcl> acls) throws IOException {
+    throw new UnsupportedOperationException("OzoneManager does not require " +
+        "this to be implemented, as write requests use a new approach.");
+  }
+
 
   /**
    * Returns list of ACLs for given Ozone object.
@@ -638,4 +922,17 @@ public interface OzoneManagerProtocol
   default CancelPrepareResponse cancelOzoneManagerPrepare() throws IOException {
     return CancelPrepareResponse.newBuilder().build();
   }
+
+  /**
+   * Send RPC request with or without payload to OM
+   * to benchmark RPC communication performance.
+   * @param payloadReq payload in request.
+   * @param payloadSizeResp payload size of response.
+   * @throws IOException if there is error in the RPC communication.
+   * @return EchoRPCResponse.
+   */
+  EchoRPCResponse echoRPCReq(byte[] payloadReq,
+                             int payloadSizeResp)
+          throws IOException;
+
 }
