@@ -21,13 +21,20 @@ source "${DIR}/_lib.sh"
 
 install_bats
 
+git clone https://github.com/bats-core/bats-assert dev-support/ci/bats-assert
+git clone https://github.com/bats-core/bats-support dev-support/ci/bats-support
+
 REPORT_DIR=${OUTPUT_DIR:-"${DIR}/../../../target/bats"}
 mkdir -p "${REPORT_DIR}"
 REPORT_FILE="${REPORT_DIR}/summary.txt"
 
 rm -f "${REPORT_DIR}/output.log"
 
-find * -path '*/src/test/shell/*' -name '*.bats' -print0 \
+find * \( \
+    -path '*/src/test/shell/*' -name '*.bats' \
+    -or -path dev-support/ci/selective_ci_checks.bats \
+    -or -path dev-support/ci/pr_title_check.bats \
+    \) -print0 \
   | xargs -0 -n1 bats --formatter tap \
   | tee -a "${REPORT_DIR}/output.log"
 
