@@ -21,14 +21,15 @@ mkdir -p "$REPORT_DIR"
 
 REPORT_FILE="$REPORT_DIR/summary.txt"
 
-cd hadoop-hdds || exit 1
-mvn -B -fn org.apache.rat:apache-rat-plugin:0.13:check
-cd ../hadoop-ozone || exit 1
-mvn -B -fn org.apache.rat:apache-rat-plugin:0.13:check
+dirs="hadoop-hdds hadoop-ozone"
 
-cd "$DIR/../../.." || exit 1
+for d in $dirs; do
+  pushd "$d" || exit 1
+  mvn -B --no-transfer-progress -fn org.apache.rat:apache-rat-plugin:0.13:check
+  popd
+done
 
-grep -r --include=rat.txt "!????" hadoop-hdds hadoop-ozone | tee "$REPORT_FILE"
+grep -r --include=rat.txt "!????" $dirs | tee "$REPORT_FILE"
 
 wc -l "$REPORT_FILE" | awk '{print $1}'> "$REPORT_DIR/failures"
 
