@@ -26,29 +26,24 @@ import org.apache.hadoop.hdds.scm.container.ContainerInfo;
 import org.apache.hadoop.hdds.scm.container.ContainerManager;
 import org.apache.hadoop.hdds.scm.server.StorageContainerManager;
 import org.apache.hadoop.ozone.MiniOzoneCluster;
-import org.junit.AfterClass;
-import org.junit.Assert;
-import org.junit.BeforeClass;
-import org.junit.Test;
+import org.junit.jupiter.api.AfterAll;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.BeforeAll;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.Timeout;
 
 import java.io.IOException;
 import java.util.concurrent.TimeUnit;
+import java.util.concurrent.TimeoutException;
 
 import static org.apache.hadoop.hdds.HddsConfigKeys.HDDS_PIPELINE_REPORT_INTERVAL;
-import org.junit.Rule;
-import org.junit.rules.Timeout;
 
 /**
  * Test SCM restart and recovery wrt pipelines.
  */
+@Timeout(300)
 public class TestSCMRestart {
 
-  /**
-    * Set a timeout for each test.
-    */
-  @Rule
-  public Timeout timeout = Timeout.seconds(300);
-
   private static MiniOzoneCluster cluster;
   private static OzoneConfiguration conf;
   private static Pipeline ratisPipeline1;
@@ -62,7 +57,7 @@ public class TestSCMRestart {
    *
    * @throws IOException
    */
-  @BeforeClass
+  @BeforeAll
   public static void init() throws Exception {
     conf = new OzoneConfiguration();
     conf.setTimeDuration(HDDS_PIPELINE_REPORT_INTERVAL, 1000,
@@ -82,12 +77,12 @@ public class TestSCMRestart {
     pipelineManager = scm.getPipelineManager();
     ratisPipeline1 = pipelineManager.getPipeline(
         containerManager.allocateContainer(
-            new RatisReplicationConfig(
+            RatisReplicationConfig.getInstance(
                 ReplicationFactor.THREE), "Owner1").getPipelineID());
     pipelineManager.openPipeline(ratisPipeline1.getId());
     ratisPipeline2 = pipelineManager.getPipeline(
         containerManager.allocateContainer(
-            new RatisReplicationConfig(
+            RatisReplicationConfig.getInstance(
                 ReplicationFactor.ONE), "Owner2").getPipelineID());
     pipelineManager.openPipeline(ratisPipeline2.getId());
     // At this stage, there should be 2 pipeline one with 1 open container
@@ -102,7 +97,7 @@ public class TestSCMRestart {
   /**
    * Shutdown MiniDFSCluster.
    */
-  @AfterClass
+  @AfterAll
   public static void shutdown() {
     if (cluster != null) {
       cluster.shutdown();
@@ -110,22 +105,24 @@ public class TestSCMRestart {
   }
 
   @Test
-  public void testPipelineWithScmRestart() throws IOException {
+  public void testPipelineWithScmRestart()
+      throws IOException, TimeoutException {
     // After restart make sure that the pipeline are still present
     Pipeline ratisPipeline1AfterRestart =
         pipelineManager.getPipeline(ratisPipeline1.getId());
     Pipeline ratisPipeline2AfterRestart =
         pipelineManager.getPipeline(ratisPipeline2.getId());
-    Assert.assertNotSame(ratisPipeline1AfterRestart, ratisPipeline1);
-    Assert.assertNotSame(ratisPipeline2AfterRestart, ratisPipeline2);
-    Assert.assertEquals(ratisPipeline1AfterRestart, ratisPipeline1);
-    Assert.assertEquals(ratisPipeline2AfterRestart, ratisPipeline2);
+    Assertions.assertNotSame(ratisPipeline1AfterRestart, ratisPipeline1);
+    Assertions.assertNotSame(ratisPipeline2AfterRestart, ratisPipeline2);
+    Assertions.assertEquals(ratisPipeline1AfterRestart, ratisPipeline1);
+    Assertions.assertEquals(ratisPipeline2AfterRestart, ratisPipeline2);
 
     // Try creating a new container, it should be from the same pipeline
     // as was before restart
     ContainerInfo containerInfo = newContainerManager
-        .allocateContainer(new RatisReplicationConfig(
+        .allocateContainer(RatisReplicationConfig.getInstance(
             ReplicationFactor.THREE), "Owner1");
-    Assert.assertEquals(containerInfo.getPipelineID(), ratisPipeline1.getId());
+    Assertions.assertEquals(ratisPipeline1.getId(),
+        containerInfo.getPipelineID());
   }
 }
\ No newline at end of file
