@@ -30,6 +30,7 @@ import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.hdds.scm.protocolPB.StorageContainerLocationProtocolClientSideTranslatorPB;
+import org.apache.hadoop.hdds.scm.server.SCMConfigurator;
 import org.apache.hadoop.hdds.scm.server.StorageContainerManager;
 import org.apache.hadoop.hdds.security.x509.certificate.client.CertificateClient;
 import org.apache.hadoop.ozone.client.OzoneClient;
@@ -37,6 +38,7 @@ import org.apache.hadoop.ozone.om.OzoneManager;
 import org.apache.hadoop.ozone.recon.ReconServer;
 import org.apache.hadoop.security.authentication.client.AuthenticationException;
 import org.apache.ozone.test.GenericTestUtils;
+import org.apache.ratis.util.ExitUtils;
 
 /**
  * Interface used for MiniOzoneClusters.
@@ -62,7 +64,7 @@ public interface MiniOzoneCluster {
    * @return MiniOzoneCluster builder
    */
   static Builder newOMHABuilder(OzoneConfiguration conf) {
-    return new MiniOzoneOMHAClusterImpl.Builder(conf);
+    return new MiniOzoneHAClusterImpl.Builder(conf);
   }
 
   static Builder newHABuilder(OzoneConfiguration conf) {
@@ -159,6 +161,8 @@ public interface MiniOzoneCluster {
    */
   List<HddsDatanodeService> getHddsDatanodes();
 
+  HddsDatanodeService getHddsDatanode(DatanodeDetails dn) throws IOException;
+
   /**
    * Returns a {@link ReconServer} instance.
    *
@@ -293,7 +297,7 @@ public interface MiniOzoneCluster {
     protected static final int DEFAULT_HB_PROCESSOR_INTERVAL_MS = 100;
     protected static final int ACTIVE_OMS_NOT_SET = -1;
     protected static final int ACTIVE_SCMS_NOT_SET = -1;
-    protected static final int DEFAULT_PIPELIME_LIMIT = 3;
+    protected static final int DEFAULT_PIPELINE_LIMIT = 3;
     protected static final int DEFAULT_RATIS_RPC_TIMEOUT_SEC = 1;
 
     protected OzoneConfiguration conf;
@@ -307,6 +311,7 @@ public interface MiniOzoneCluster {
     protected String scmServiceId;
     protected int numOfSCMs;
     protected int numOfActiveSCMs = ACTIVE_SCMS_NOT_SET;
+    protected SCMConfigurator scmConfigurator;
 
     protected Optional<Boolean> enableTrace = Optional.of(false);
     protected Optional<Integer> hbInterval = Optional.empty();
@@ -336,11 +341,14 @@ public interface MiniOzoneCluster {
     protected int numDataVolumes = 1;
     protected boolean  startDataNodes = true;
     protected CertificateClient certClient;
-    protected int pipelineNumLimit = DEFAULT_PIPELIME_LIMIT;
+    protected int pipelineNumLimit = DEFAULT_PIPELINE_LIMIT;
 
     protected Builder(OzoneConfiguration conf) {
       this.conf = conf;
       setClusterId(UUID.randomUUID().toString());
+      // Use default SCM configurations if no override is provided.
+      setSCMConfigurator(new SCMConfigurator());
+      ExitUtils.disableSystemExit();
     }
 
     public Builder setConf(OzoneConfiguration config) {
@@ -348,6 +356,11 @@ public interface MiniOzoneCluster {
       return this;
     }
 
+    public Builder setSCMConfigurator(SCMConfigurator configurator) {
+      this.scmConfigurator = configurator;
+      return this;
+    }
+
     /**
      * Sets the cluster Id.
      *
@@ -626,4 +639,4 @@ public interface MiniOzoneCluster {
      */
     public abstract MiniOzoneCluster build() throws IOException;
   }
-}
\ No newline at end of file
+}
