@@ -19,16 +19,19 @@
 package org.apache.hadoop.ozone;
 
 import com.google.common.base.Preconditions;
+import com.google.common.base.Supplier;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
+import org.apache.hadoop.hdds.ExitManager;
+import org.apache.hadoop.hdds.conf.ConfigurationTarget;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.scm.ScmConfigKeys;
-import org.apache.hadoop.hdds.scm.TestUtils;
+import org.apache.hadoop.hdds.scm.HddsTestUtils;
 import org.apache.hadoop.hdds.scm.ha.CheckedConsumer;
 import org.apache.hadoop.hdds.scm.safemode.HealthyPipelineSafeModeRule;
+import org.apache.hadoop.hdds.scm.server.SCMConfigurator;
 import org.apache.hadoop.hdds.scm.server.StorageContainerManager;
 import org.apache.hadoop.metrics2.lib.DefaultMetricsSystem;
-import org.apache.hadoop.net.NetUtils;
 import org.apache.hadoop.ozone.client.OzoneClient;
 import org.apache.hadoop.ozone.client.OzoneClientFactory;
 import org.apache.hadoop.ozone.ha.ConfUtils;
@@ -38,22 +41,30 @@ import org.apache.hadoop.ozone.om.ratis.OzoneManagerRatisServer;
 import org.apache.hadoop.ozone.recon.ReconServer;
 import org.apache.hadoop.security.authentication.client.AuthenticationException;
 import org.apache.ozone.test.GenericTestUtils;
+import org.apache.ratis.util.IOUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import java.io.Closeable;
 import java.io.IOException;
 import java.net.BindException;
+import java.net.ServerSocket;
 import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.Iterator;
+import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
-import java.util.Random;
+import java.util.PrimitiveIterator;
+import java.util.Queue;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.TimeoutException;
 import java.util.function.Function;
-import java.util.stream.Collectors;
 
+import static java.util.Collections.singletonList;
 import static org.apache.hadoop.hdds.HddsConfigKeys.OZONE_METADATA_DIRS;
 import static org.apache.hadoop.hdds.scm.ScmConfig.ConfigStrings.HDDS_SCM_INIT_DEFAULT_LAYOUT_VERSION;
+import static org.apache.hadoop.ozone.OzoneTestUtils.reservePorts;
 import static org.apache.hadoop.ozone.om.OmUpgradeConfig.ConfigStrings.OZONE_OM_INIT_DEFAULT_LAYOUT_VERSION;
 
 /**
@@ -64,7 +75,7 @@ import static org.apache.hadoop.ozone.om.OmUpgradeConfig.ConfigStrings.OZONE_OM_
  */
 public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
 
-  private static final Logger LOG =
+  public static final Logger LOG =
       LoggerFactory.getLogger(MiniOzoneHAClusterImpl.class);
 
   private final OMHAService omhaService;
@@ -74,51 +85,28 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
 
   private int waitForClusterToBeReadyTimeout = 120000; // 2 min
 
-  private static final Random RANDOM = new Random();
   private static final int RATIS_RPC_TIMEOUT = 1000; // 1 second
-  private static final int NODE_FAILURE_TIMEOUT = 2000; // 2 seconds
+  public static final int NODE_FAILURE_TIMEOUT = 2000; // 2 seconds
 
   /**
    * Creates a new MiniOzoneCluster.
    *
    * @throws IOException if there is an I/O error
    */
-  @SuppressWarnings("checkstyle:ParameterNumber")
   public MiniOzoneHAClusterImpl(
       OzoneConfiguration conf,
-      List<OzoneManager> activeOMList,
-      List<OzoneManager> inactiveOMList,
-      List<StorageContainerManager> activeSCMList,
-      List<StorageContainerManager> inactiveSCMList,
+      SCMConfigurator scmConfigurator,
+      OMHAService omhaService,
+      SCMHAService scmhaService,
       List<HddsDatanodeService> hddsDatanodes,
-      String omServiceId,
-      String scmServiceId,
       String clusterPath,
       ReconServer reconServer) {
-    super(conf, hddsDatanodes, reconServer);
-    omhaService =
-        new OMHAService(activeOMList, inactiveOMList, omServiceId);
-    scmhaService =
-        new SCMHAService(activeSCMList, inactiveSCMList, scmServiceId);
+    super(conf, scmConfigurator, hddsDatanodes, reconServer);
+    this.omhaService = omhaService;
+    this.scmhaService = scmhaService;
     this.clusterMetaPath = clusterPath;
   }
 
-  /**
-   * Creates a new MiniOzoneCluster with all OMs active.
-   * This is used by MiniOzoneChaosCluster.
-   */
-  protected MiniOzoneHAClusterImpl(
-      OzoneConfiguration conf,
-      List<OzoneManager> omList,
-      List<StorageContainerManager> scmList,
-      List<HddsDatanodeService> hddsDatanodes,
-      String omServiceId,
-      String scmServiceId,
-      String clusterPath) {
-    this(conf, omList, null, scmList, null, hddsDatanodes,
-        omServiceId, scmServiceId, clusterPath, null);
-  }
-
   @Override
   public String getOMServiceId() {
     return omhaService.getServiceId();
@@ -154,8 +142,8 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
     return omhaService.isServiceActive(omNodeId);
   }
 
-  public boolean isSCMActive(String scmNodeId) {
-    return scmhaService.isServiceActive(scmNodeId);
+  public Iterator<StorageContainerManager> getInactiveSCM() {
+    return scmhaService.inactiveServices();
   }
 
   public StorageContainerManager getSCM(String scmNodeId) {
@@ -182,6 +170,23 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
     return this.scmhaService.getServiceByIndex(index);
   }
 
+  private OzoneManager getOMLeader(boolean waitForLeaderElection)
+      throws TimeoutException, InterruptedException {
+    if (waitForLeaderElection) {
+      final OzoneManager[] om = new OzoneManager[1];
+      GenericTestUtils.waitFor(new Supplier<Boolean>() {
+        @Override
+        public Boolean get() {
+          om[0] = getOMLeader();
+          return om[0] != null;
+        }
+      }, 200, waitForClusterToBeReadyTimeout);
+      return om[0];
+    } else {
+      return getOMLeader();
+    }
+  }
+
   /**
    * Get OzoneManager leader object.
    * @return OzoneManager object, null if there isn't one or more than one
@@ -246,23 +251,25 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
     LOG.info("Shutting down StorageContainerManager " + scm.getScmId());
 
     scm.stop();
-    scmhaService.deactivate(scm);
+    scmhaService.removeInstance(scm);
   }
 
-  public void restartStorageContainerManager(
+  public StorageContainerManager restartStorageContainerManager(
       StorageContainerManager scm, boolean waitForSCM)
       throws IOException, TimeoutException,
       InterruptedException, AuthenticationException {
     LOG.info("Restarting SCM in cluster " + this.getClass());
+    scmhaService.removeInstance(scm);
     OzoneConfiguration scmConf = scm.getConfiguration();
     shutdownStorageContainerManager(scm);
     scm.join();
-    scm = TestUtils.getScmSimple(scmConf);
-    scmhaService.activate(scm);
+    scm = HddsTestUtils.getScmSimple(scmConf, getSCMConfigurator());
+    scmhaService.addInstance(scm, true);
     scm.start();
     if (waitForSCM) {
       waitForClusterToBeReady();
     }
+    return scm;
   }
 
   public String getClusterId() throws IOException {
@@ -291,6 +298,13 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
     }, 1000, waitForClusterToBeReadyTimeout);
   }
 
+  @Override
+  public void shutdown() {
+    super.shutdown();
+    omhaService.releasePorts();
+    scmhaService.releasePorts();
+  }
+
   @Override
   public void stop() {
     for (OzoneManager ozoneManager : this.omhaService.getServices()) {
@@ -325,6 +339,29 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
     omhaService.deactivate(om);
   }
 
+  private static void configureOMPorts(ConfigurationTarget conf,
+      String omServiceId, String omNodeId,
+      ReservedPorts omPorts, ReservedPorts omRpcPorts) {
+
+    String omAddrKey = ConfUtils.addKeySuffixes(
+        OMConfigKeys.OZONE_OM_ADDRESS_KEY, omServiceId, omNodeId);
+    String omHttpAddrKey = ConfUtils.addKeySuffixes(
+        OMConfigKeys.OZONE_OM_HTTP_ADDRESS_KEY, omServiceId, omNodeId);
+    String omHttpsAddrKey = ConfUtils.addKeySuffixes(
+        OMConfigKeys.OZONE_OM_HTTPS_ADDRESS_KEY, omServiceId, omNodeId);
+    String omRatisPortKey = ConfUtils.addKeySuffixes(
+        OMConfigKeys.OZONE_OM_RATIS_PORT_KEY, omServiceId, omNodeId);
+
+    PrimitiveIterator.OfInt nodePorts = omPorts.assign(omNodeId);
+    PrimitiveIterator.OfInt rpcPorts = omRpcPorts.assign(omNodeId);
+    conf.set(omAddrKey, "127.0.0.1:" + rpcPorts.nextInt());
+    conf.set(omHttpAddrKey, "127.0.0.1:" + nodePorts.nextInt());
+    conf.set(omHttpsAddrKey, "127.0.0.1:" + nodePorts.nextInt());
+    conf.setInt(omRatisPortKey, nodePorts.nextInt());
+
+    omRpcPorts.release(omNodeId);
+  }
+
   /**
    * Builder for configuring the MiniOzoneCluster to run.
    */
@@ -338,6 +375,15 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
     private List<StorageContainerManager> activeSCMs = new ArrayList<>();
     private List<StorageContainerManager> inactiveSCMs = new ArrayList<>();
 
+    // These port reservations are for servers started when the component
+    // (OM or SCM) is started.  These are Ratis, HTTP and HTTPS.  We also have
+    // another set of ports for RPC endpoints, which are started as soon as
+    // the component is created (in methods called by OzoneManager and
+    // StorageContainerManager constructors respectively).  So we need to manage
+    // them separately, see initOMHAConfig() and initSCMHAConfig().
+    private final ReservedPorts omPorts = new ReservedPorts(3);
+    private final ReservedPorts scmPorts = new ReservedPorts(3);
+
     /**
      * Creates a new Builder.
      *
@@ -347,14 +393,6 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
       super(conf);
     }
 
-    public List<OzoneManager> getActiveOMs() {
-      return activeOMs;
-    }
-
-    public List<OzoneManager> getInactiveOMs() {
-      return inactiveOMs;
-    }
-
     @Override
     public MiniOzoneCluster build() throws IOException {
       if (numOfActiveOMs > numOfOMs) {
@@ -380,11 +418,12 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
       DefaultMetricsSystem.setMiniClusterMode(true);
       initializeConfiguration();
       initOMRatisConf();
-      StorageContainerManager scm;
+      SCMHAService scmService;
+      OMHAService omService;
       ReconServer reconServer = null;
       try {
-        createSCMService();
-        createOMService();
+        scmService = createSCMService();
+        omService = createOMService();
         if (includeRecon) {
           configureRecon();
           reconServer = new ReconServer();
@@ -395,11 +434,11 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
       }
 
       final List<HddsDatanodeService> hddsDatanodes = createHddsDatanodes(
-          activeSCMs, reconServer);
+          scmService.getActiveServices(), reconServer);
 
       MiniOzoneHAClusterImpl cluster = new MiniOzoneHAClusterImpl(conf,
-          activeOMs, inactiveOMs, activeSCMs, inactiveSCMs,
-          hddsDatanodes, omServiceId, scmServiceId, path, reconServer);
+          scmConfigurator, omService, scmService, hddsDatanodes, path,
+          reconServer);
 
       if (startDataNodes) {
         cluster.startHddsDatanodes();
@@ -440,20 +479,23 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
     /**
      * Start OM service with multiple OMs.
      */
-    protected List<OzoneManager> createOMService() throws IOException,
+    protected OMHAService createOMService() throws IOException,
         AuthenticationException {
+      if (omServiceId == null) {
+        OzoneManager om = createOM();
+        om.start();
+        return new OMHAService(singletonList(om), null, null, null);
+      }
 
       List<OzoneManager> omList = Lists.newArrayList();
 
       int retryCount = 0;
-      int basePort;
 
       while (true) {
         try {
-          basePort = 10000 + RANDOM.nextInt(1000) * 4;
-          initOMHAConfig(basePort);
+          initOMHAConfig();
 
-          for (int i = 1; i<= numOfOMs; i++) {
+          for (int i = 1; i <= numOfOMs; i++) {
             // Set nodeId
             String nodeId = OM_NODE_ID_PREFIX + i;
             OzoneConfiguration config = new OzoneConfiguration(conf);
@@ -491,11 +533,6 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
             }
           }
 
-          // Set default OM address to point to the first OM. Clients would
-          // try connecting to this address by default
-          conf.set(OMConfigKeys.OZONE_OM_ADDRESS_KEY,
-              NetUtils.getHostPortString(omList.get(0).getOmRpcServerAddr()));
-
           break;
         } catch (BindException e) {
           for (OzoneManager om : omList) {
@@ -507,28 +544,32 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
           omList.clear();
           ++retryCount;
           LOG.info("MiniOzoneHACluster port conflicts, retried {} times",
-              retryCount);
+              retryCount, e);
         }
       }
-      return omList;
+      return new OMHAService(activeOMs, inactiveOMs, omServiceId, omPorts);
     }
 
     /**
      * Start OM service with multiple OMs.
      */
-    protected List<StorageContainerManager> createSCMService()
+    protected SCMHAService createSCMService()
         throws IOException, AuthenticationException {
+      if (scmServiceId == null) {
+        StorageContainerManager scm = createSCM();
+        scm.start();
+        return new SCMHAService(singletonList(scm), null, null, null);
+      }
+
       List<StorageContainerManager> scmList = Lists.newArrayList();
 
       int retryCount = 0;
-      int basePort = 12000;
 
       while (true) {
         try {
-          basePort = 12000 + RANDOM.nextInt(1000) * 4;
-          initSCMHAConfig(basePort);
+          initSCMHAConfig();
 
-          for (int i = 1; i<= numOfSCMs; i++) {
+          for (int i = 1; i <= numOfSCMs; i++) {
             // Set nodeId
             String nodeId = SCM_NODE_ID_PREFIX + i;
             String metaDirPath = path + "/" + nodeId;
@@ -547,7 +588,8 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
             } else {
               StorageContainerManager.scmBootstrap(scmConfig);
             }
-            StorageContainerManager scm = TestUtils.getScmSimple(scmConfig);
+            StorageContainerManager scm =
+                HddsTestUtils.getScmSimple(scmConfig, scmConfigurator);
             HealthyPipelineSafeModeRule rule =
                 scm.getScmSafeModeManager().getHealthyPipelineSafeModeRule();
             if (rule != null) {
@@ -562,11 +604,11 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
               scm.start();
               activeSCMs.add(scm);
               LOG.info("Started SCM RPC server at {}",
-                  scm.getClientProtocolServer());
+                  scm.getClientRpcAddress());
             } else {
               inactiveSCMs.add(scm);
               LOG.info("Intialized SCM at {}. This SCM is currently "
-                  + "inactive (not running).", scm.getClientProtocolServer());
+                  + "inactive (not running).", scm.getClientRpcAddress());
             }
           }
 
@@ -577,21 +619,22 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
             scm.stop();
             scm.join();
             LOG.info("Stopping StorageContainerManager server at {}",
-                scm.getClientProtocolServer());
+                scm.getClientRpcAddress());
           }
           scmList.clear();
           ++retryCount;
           LOG.info("MiniOzoneHACluster port conflicts, retried {} times",
-              retryCount);
+              retryCount, e);
         }
       }
-      return scmList;
+
+      return new SCMHAService(activeSCMs, inactiveSCMs, scmServiceId, scmPorts);
     }
 
     /**
      * Initialize HA related configurations.
      */
-    private void initSCMHAConfig(int basePort) throws IOException {
+    private void initSCMHAConfig() {
       // Set configurations required for starting OM HA service, because that
       // is the serviceID being passed to start Ozone HA cluster.
       // Here setting internal service and OZONE_OM_SERVICE_IDS_KEY, in this
@@ -603,11 +646,14 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
       StringBuilder scmNodesKeyValue = new StringBuilder();
       StringBuilder scmNames = new StringBuilder();
 
-      int port = basePort;
+      scmPorts.reserve(numOfSCMs);
+      ReservedPorts scmRpcPorts = new ReservedPorts(4);
+      scmRpcPorts.reserve(numOfSCMs);
 
-      for (int i = 1; i <= numOfSCMs; i++, port+=10) {
+      for (int i = 1; i <= numOfSCMs; i++) {
         String scmNodeId = SCM_NODE_ID_PREFIX + i;
         scmNodesKeyValue.append(",").append(scmNodeId);
+
         String scmAddrKey = ConfUtils.addKeySuffixes(
             ScmConfigKeys.OZONE_SCM_ADDRESS_KEY, scmServiceId, scmNodeId);
         String scmHttpAddrKey = ConfUtils.addKeySuffixes(
@@ -628,18 +674,32 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
         String scmGrpcPortKey = ConfUtils.addKeySuffixes(
             ScmConfigKeys.OZONE_SCM_GRPC_PORT_KEY, scmServiceId, scmNodeId);
 
+        PrimitiveIterator.OfInt nodePorts = scmPorts.assign(scmNodeId);
+        PrimitiveIterator.OfInt rpcPorts = scmRpcPorts.assign(scmNodeId);
         conf.set(scmAddrKey, "127.0.0.1");
-        conf.set(scmHttpAddrKey, "127.0.0.1:" + (port + 2));
-        conf.set(scmHttpsAddrKey, "127.0.0.1:" + (port + 3));
-        conf.setInt(scmRatisPortKey, port + 4);
-        //conf.setInt("ozone.scm.ha.ratis.bind.port", port + 4);
-        conf.set(dnPortKey, "127.0.0.1:" + (port + 5));
-        conf.set(blockClientKey, "127.0.0.1:" + (port + 6));
-        conf.set(ssClientKey, "127.0.0.1:" + (port + 7));
-        conf.setInt(scmGrpcPortKey, port + 8);
-        scmNames.append(",").append("localhost:" + (port + 5));
-        conf.set(ScmConfigKeys.
-            OZONE_SCM_BLOCK_CLIENT_ADDRESS_KEY, "127.0.0.1:" + (port + 6));
+        conf.set(scmHttpAddrKey, "127.0.0.1:" + nodePorts.nextInt());
+        conf.set(scmHttpsAddrKey, "127.0.0.1:" + nodePorts.nextInt());
+
+        int ratisPort = nodePorts.nextInt();
+        conf.setInt(scmRatisPortKey, ratisPort);
+        //conf.setInt("ozone.scm.ha.ratis.bind.port", ratisPort);
+
+        int dnPort = rpcPorts.nextInt();
+        conf.set(dnPortKey, "127.0.0.1:" + dnPort);
+        scmNames.append(",localhost:").append(dnPort);
+
+        conf.set(ssClientKey, "127.0.0.1:" + rpcPorts.nextInt());
+        conf.setInt(scmGrpcPortKey, rpcPorts.nextInt());
+
+        int blockPort = rpcPorts.nextInt();
+        conf.set(blockClientKey, "127.0.0.1:" + blockPort);
+        conf.set(ScmConfigKeys.OZONE_SCM_BLOCK_CLIENT_ADDRESS_KEY,
+            "127.0.0.1:" + blockPort);
+
+        if (i <= numOfActiveSCMs) {
+          scmPorts.release(scmNodeId);
+        }
+        scmRpcPorts.release(scmNodeId);
       }
 
       conf.set(scmNodesKey, scmNodesKeyValue.substring(1));
@@ -649,7 +709,7 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
     /**
      * Initialize HA related configurations.
      */
-    private void initOMHAConfig(int basePort) throws IOException {
+    private void initOMHAConfig() {
       // Set configurations required for starting OM HA service, because that
       // is the serviceID being passed to start Ozone HA cluster.
       // Here setting internal service and OZONE_OM_SERVICE_IDS_KEY, in this
@@ -660,25 +720,19 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
           OMConfigKeys.OZONE_OM_NODES_KEY, omServiceId);
       List<String> omNodeIds = new ArrayList<>();
 
-      int port = basePort;
+      omPorts.reserve(numOfOMs);
+      ReservedPorts omRpcPorts = new ReservedPorts(1);
+      omRpcPorts.reserve(numOfOMs);
 
-      for (int i = 1; i <= numOfOMs; i++, port+=6) {
+      for (int i = 1; i <= numOfOMs; i++) {
         String omNodeId = OM_NODE_ID_PREFIX + i;
         omNodeIds.add(omNodeId);
 
-        String omAddrKey = ConfUtils.addKeySuffixes(
-            OMConfigKeys.OZONE_OM_ADDRESS_KEY, omServiceId, omNodeId);
-        String omHttpAddrKey = ConfUtils.addKeySuffixes(
-            OMConfigKeys.OZONE_OM_HTTP_ADDRESS_KEY, omServiceId, omNodeId);
-        String omHttpsAddrKey = ConfUtils.addKeySuffixes(
-            OMConfigKeys.OZONE_OM_HTTPS_ADDRESS_KEY, omServiceId, omNodeId);
-        String omRatisPortKey = ConfUtils.addKeySuffixes(
-            OMConfigKeys.OZONE_OM_RATIS_PORT_KEY, omServiceId, omNodeId);
+        configureOMPorts(conf, omServiceId, omNodeId, omPorts, omRpcPorts);
 
-        conf.set(omAddrKey, "127.0.0.1:" + port);
-        conf.set(omHttpAddrKey, "127.0.0.1:" + (port + 2));
-        conf.set(omHttpsAddrKey, "127.0.0.1:" + (port + 3));
-        conf.setInt(omRatisPortKey, port + 4);
+        if (i <= numOfActiveOMs) {
+          omPorts.release(omNodeId);
+        }
       }
 
       conf.set(omNodesKey, String.join(",", omNodeIds));
@@ -686,25 +740,44 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
   }
 
   /**
-   * Bootstrap new OM and add to existing OM HA service ring.
-   * @return new OM nodeId
+   * Bootstrap new OM by updating existing OM configs.
    */
   public void bootstrapOzoneManager(String omNodeId) throws Exception {
+    bootstrapOzoneManager(omNodeId, true, false);
+  }
+
+  /**
+   * Bootstrap new OM and add to existing OM HA service ring.
+   * @param omNodeId nodeId of new OM
+   * @param updateConfigs if true, update the old OM configs with new node
+   *                      information
+   * @param force if true, start new OM with FORCE_BOOTSTRAP option.
+   *              Otherwise, start new OM with BOOTSTRAP option.
+   */
+  public void bootstrapOzoneManager(String omNodeId,
+      boolean updateConfigs, boolean force) throws Exception {
+
+    // Set testReloadConfigFlag to true so that
+    // OzoneManager#reloadConfiguration does not reload config as it will
+    // return the default configurations.
+    OzoneManager.setTestReloadConfigFlag(true);
 
-    int basePort;
     int retryCount = 0;
-
     OzoneManager om = null;
 
-    long leaderSnapshotIndex = getOMLeader().getRatisSnapshotIndex();
+    OzoneManager omLeader = getOMLeader(true);
+    long leaderSnapshotIndex = omLeader.getRatisSnapshotIndex();
 
     while (true) {
       try {
-        List<Integer> portSet = getFreePortList(4);
         OzoneConfiguration newConf = addNewOMToConfig(getOMServiceId(),
-            omNodeId, portSet);
+            omNodeId);
 
-        om = bootstrapNewOM(omNodeId);
+        if (updateConfigs) {
+          updateOMConfigs(newConf);
+        }
+
+        om = bootstrapNewOM(omNodeId, newConf, force);
 
         LOG.info("Bootstrapped OzoneManager {} RPC server at {}", omNodeId,
             om.getOmRpcServerAddr());
@@ -712,7 +785,6 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
         // Add new OMs to cluster's in memory map and update existing OMs conf.
         setConf(newConf);
 
-        omhaService.addInstance(om, true);
         break;
       } catch (IOException e) {
         // Existing OM config could have been updated with new conf. Reset it.
@@ -723,7 +795,7 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
             e.getCause() instanceof BindException) {
           ++retryCount;
           LOG.info("MiniOzoneHACluster port conflicts, retried {} times",
-              retryCount);
+              retryCount, e);
         } else {
           throw e;
         }
@@ -731,69 +803,73 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
     }
 
     waitForBootstrappedNodeToBeReady(om, leaderSnapshotIndex);
-    waitForConfigUpdateOnAllOMs(omNodeId);
+    if (updateConfigs) {
+      waitForConfigUpdateOnActiveOMs(omNodeId);
+    }
   }
 
   /**
    * Set the configs for new OMs.
    */
   private OzoneConfiguration addNewOMToConfig(String omServiceId,
-      String omNodeId, List<Integer> portList) {
-    OzoneConfiguration newConf = getConf();
+      String omNodeId) {
+
+    ReservedPorts omPorts = omhaService.getPorts();
+    omPorts.reserve(1);
+    ReservedPorts omRpcPorts = new ReservedPorts(1);
+    omRpcPorts.reserve(1);
+
+    OzoneConfiguration newConf = new OzoneConfiguration(getConf());
+    configureOMPorts(newConf, omServiceId, omNodeId, omPorts, omRpcPorts);
+
     String omNodesKey = ConfUtils.addKeySuffixes(
         OMConfigKeys.OZONE_OM_NODES_KEY, omServiceId);
-    StringBuilder omNodesKeyValue = new StringBuilder();
-    omNodesKeyValue.append(newConf.get(omNodesKey))
-        .append(",").append(omNodeId);
-
-    String omAddrKey = ConfUtils.addKeySuffixes(
-        OMConfigKeys.OZONE_OM_ADDRESS_KEY, omServiceId, omNodeId);
-    String omHttpAddrKey = ConfUtils.addKeySuffixes(
-        OMConfigKeys.OZONE_OM_HTTP_ADDRESS_KEY, omServiceId, omNodeId);
-    String omHttpsAddrKey = ConfUtils.addKeySuffixes(
-        OMConfigKeys.OZONE_OM_HTTPS_ADDRESS_KEY, omServiceId, omNodeId);
-    String omRatisPortKey = ConfUtils.addKeySuffixes(
-        OMConfigKeys.OZONE_OM_RATIS_PORT_KEY, omServiceId, omNodeId);
-
-    newConf.set(omAddrKey, "127.0.0.1:" + portList.get(0));
-    newConf.set(omHttpAddrKey, "127.0.0.1:" + portList.get(1));
-    newConf.set(omHttpsAddrKey, "127.0.0.1:" + portList.get(2));
-    newConf.setInt(omRatisPortKey, portList.get(3));
-
-    newConf.set(omNodesKey, omNodesKeyValue.toString());
+    newConf.set(omNodesKey, newConf.get(omNodesKey) + "," + omNodeId);
 
     return newConf;
   }
 
   /**
-   * Start a new OM in Bootstrap mode. Configs for the new OM must already be
-   * set.
+   * Update the configurations of the given list of OMs.
    */
-  private OzoneManager bootstrapNewOM(String nodeId)
-      throws IOException, AuthenticationException {
-    OzoneConfiguration config = new OzoneConfiguration(getConf());
+  public void updateOMConfigs(OzoneConfiguration newConf) {
+    for (OzoneManager om : omhaService.getActiveServices()) {
+      om.setConfiguration(newConf);
+    }
+  }
+
+  /**
+   * Start a new OM in Bootstrap mode. Configs (address and ports) for the new
+   * OM must already be set in the newConf.
+   */
+  private OzoneManager bootstrapNewOM(String nodeId, OzoneConfiguration newConf,
+      boolean force) throws IOException, AuthenticationException {
+
+    OzoneConfiguration config = new OzoneConfiguration(newConf);
+
+    // For bootstrapping node, set the nodeId config also.
     config.set(OMConfigKeys.OZONE_OM_NODE_ID_KEY, nodeId);
-    // Set the OM rpc and http(s) address to null so that the cluster picks
-    // up the address set with service ID and node ID
-    config.set(OMConfigKeys.OZONE_OM_ADDRESS_KEY, "");
-    config.set(OMConfigKeys.OZONE_OM_HTTP_ADDRESS_KEY, "");
-    config.set(OMConfigKeys.OZONE_OM_HTTPS_ADDRESS_KEY, "");
 
     // Set metadata/DB dir base path
     String metaDirPath = clusterMetaPath + "/" + nodeId;
     config.set(OZONE_METADATA_DIRS, metaDirPath);
 
-    // Update existing OMs config
-    for (OzoneManager existingOM : omhaService.getServices()) {
-      existingOM.setConfiguration(config);
+    OzoneManager.omInit(config);
+    OzoneManager om;
+
+    if (force) {
+      om = OzoneManager.createOm(config,
+          OzoneManager.StartupOption.FORCE_BOOTSTRAP);
+    } else {
+      om = OzoneManager.createOm(config, OzoneManager.StartupOption.BOOTSTRAP);
     }
 
-    OzoneManager.omInit(config);
-    OzoneManager om = OzoneManager.createOm(config,
-        OzoneManager.StartupOption.BOOTSTRAP);
-    om.start();
-    return om;
+    ExitManagerForOM exitManager = new ExitManagerForOM(this, nodeId);
+    om.setExitManagerForTesting(exitManager);
+    omhaService.addInstance(om, false);
+    startInactiveOM(nodeId);
 
+    return om;
   }
 
   /**
@@ -814,7 +890,7 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
     }, 1000, waitForClusterToBeReadyTimeout);
   }
 
-  private void waitForConfigUpdateOnAllOMs(String newOMNodeId)
+  private void waitForConfigUpdateOnActiveOMs(String newOMNodeId)
       throws Exception {
     OzoneManager newOMNode = omhaService.getServiceById(newOMNodeId);
     OzoneManagerRatisServer newOMRatisServer = newOMNode.getOmRatisServer();
@@ -837,6 +913,12 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
     }, 1000, waitForClusterToBeReadyTimeout);
   }
 
+  public void setupExitManagerForTesting() {
+    for (OzoneManager om : omhaService.getServices()) {
+      om.setExitManagerForTesting(new ExitManagerForOM(this, om.getOMNodeId()));
+    }
+  }
+
   /**
    * MiniOzoneHAService is a helper class used for both SCM and OM HA.
    * This class keeps track of active and inactive OM/SCM services
@@ -847,6 +929,7 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
     private List<Type> services;
     private String serviceId;
     private String serviceName;
+    private final ReservedPorts ports;
 
     // Active services s denote OM/SCM services which are up and running
     private List<Type> activeServices;
@@ -856,9 +939,10 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
     private Function<Type, String> serviceIdProvider;
 
     MiniOzoneHAService(String name, List<Type> activeList,
-                       List<Type> inactiveList, String serviceId,
-                       Function<Type, String> idProvider) {
+        List<Type> inactiveList, String serviceId,
+        ReservedPorts ports, Function<Type, String> idProvider) {
       this.serviceName = name;
+      this.ports = ports != null ? ports : new ReservedPorts(0);
       this.serviceMap = Maps.newHashMap();
       this.serviceIdProvider = idProvider;
       if (activeList != null) {
@@ -890,12 +974,20 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
       return services;
     }
 
+    public void releasePorts() {
+      ports.releaseAll();
+    }
+
     public List<Type> getActiveServices() {
       return activeServices;
     }
 
     public boolean removeInstance(Type t) {
-      return services.remove(t);
+      boolean result =  services.remove(t);
+      serviceMap.remove(serviceIdProvider.apply(t));
+      activeServices.remove(t);
+      inactiveServices.remove(t);
+      return result;
     }
 
     public void addInstance(Type t, boolean isActive) {
@@ -903,6 +995,8 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
       serviceMap.put(serviceIdProvider.apply(t), t);
       if (isActive) {
         activeServices.add(t);
+      } else {
+        inactiveServices.add(t);
       }
     }
 
@@ -920,6 +1014,10 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
       return activeServices.contains(serviceMap.get(id));
     }
 
+    public Iterator<Type> inactiveServices() {
+      return new ArrayList<>(inactiveServices).iterator();
+    }
+
     public Type getServiceByIndex(int index) {
       return this.services.get(index);
     }
@@ -934,17 +1032,22 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
       if (!inactiveServices.contains(service)) {
         throw new IOException(serviceName + " is already active.");
       } else {
+        ports.release(id);
         serviceStarter.execute(service);
         activeServices.add(service);
         inactiveServices.remove(service);
       }
     }
+
+    public ReservedPorts getPorts() {
+      return ports;
+    }
   }
 
   static class OMHAService extends MiniOzoneHAService<OzoneManager> {
     OMHAService(List<OzoneManager> activeList, List<OzoneManager> inactiveList,
-                String serviceId) {
-      super("OM", activeList, inactiveList, serviceId,
+        String serviceId, ReservedPorts omPorts) {
+      super("OM", activeList, inactiveList, serviceId, omPorts,
           OzoneManager::getOMNodeId);
     }
   }
@@ -952,25 +1055,118 @@ public class MiniOzoneHAClusterImpl extends MiniOzoneClusterImpl {
   static class SCMHAService extends
       MiniOzoneHAService<StorageContainerManager> {
     SCMHAService(List<StorageContainerManager> activeList,
-                 List<StorageContainerManager> inactiveList,
-                 String serviceId) {
+        List<StorageContainerManager> inactiveList,
+        String serviceId, ReservedPorts scmPorts) {
       super("SCM", activeList, inactiveList, serviceId,
-          StorageContainerManager::getScmId);
+          scmPorts, StorageContainerManager::getSCMNodeId);
     }
   }
 
   public List<StorageContainerManager> getStorageContainerManagers() {
-    return this.scmhaService.getServices();
+    return new ArrayList<>(this.scmhaService.getServices());
   }
 
   public StorageContainerManager getStorageContainerManager() {
     return getStorageContainerManagers().get(0);
   }
 
-  private List<Integer> getFreePortList(int size) {
-    return org.apache.ratis.util.NetUtils.createLocalServerAddress(size)
-        .stream()
-        .map(inetSocketAddress -> inetSocketAddress.getPort())
-        .collect(Collectors.toList());
+  private static final class ExitManagerForOM extends ExitManager {
+
+    private MiniOzoneHAClusterImpl cluster;
+    private String omNodeId;
+
+    private ExitManagerForOM(MiniOzoneHAClusterImpl cluster, String nodeId) {
+      this.cluster = cluster;
+      this.omNodeId = nodeId;
+    }
+
+    @Override
+    public void exitSystem(int status, String message, Throwable throwable,
+        Logger log) throws IOException {
+      LOG.error(omNodeId + " - System Exit: " + message, throwable);
+      cluster.stopOzoneManager(omNodeId);
+      throw new IOException(throwable);
+    }
+
+    @Override
+    public void exitSystem(int status, String message, Logger log)
+        throws IOException {
+      LOG.error(omNodeId + " - System Exit: " + message);
+      cluster.stopOzoneManager(omNodeId);
+      throw new IOException(message);
+    }
   }
-}
\ No newline at end of file
+
+  /**
+   * Reserves a number of ports for services.
+   */
+  private static class ReservedPorts {
+
+    private final Queue<ServerSocket> allPorts = new LinkedList<>();
+    private final Map<String, List<ServerSocket>> assignedPorts =
+        new HashMap<>();
+    private final int portsPerNode;
+
+    ReservedPorts(int portsPerNode) {
+      this.portsPerNode = portsPerNode;
+    }
+
+    /**
+     * Reserve {@code portsPerNode * nodes} ports by binding server sockets
+     * to random free ports.  The sockets are kept open until
+     * {@link #release(String)} or {@link #releaseAll} is called.
+     */
+    public void reserve(int nodes) {
+      Preconditions.checkState(allPorts.isEmpty());
+      allPorts.addAll(reservePorts(portsPerNode * nodes));
+    }
+
+    /**
+     * Assign {@code portsPerNode} ports to a service identified by {@code id}.
+     * This set of ports should be released right before starting the service
+     * by calling {@link #release(String)}.
+     *
+     * @return iterator of the ports assigned
+     */
+    public PrimitiveIterator.OfInt assign(String id) {
+      Preconditions.checkState(allPorts.size() >= portsPerNode);
+      List<ServerSocket> nodePorts = new LinkedList<>();
+      for (int i = 0; i < portsPerNode; i++) {
+        nodePorts.add(allPorts.remove());
+      }
+      assignedPorts.put(id, nodePorts);
+      LOG.debug("assign ports for {}: {}", id, nodePorts);
+
+      return nodePorts.stream().mapToInt(ServerSocket::getLocalPort).iterator();
+    }
+
+    /**
+     * Release the ports assigned to the service identified by {@code id}.
+     *
+     * This closes the server sockets, making the same ports available for
+     * the service.  Note: there is a race condition with other processes
+     * running on the host, but that's OK since this is for tests.
+     *
+     * If no ports are assigned to the service, this is a no-op.
+     */
+    public void release(String id) {
+      List<ServerSocket> ports = assignedPorts.remove(id);
+      LOG.debug("release ports for {}: {}", id, ports);
+      if (ports != null) {
+        IOUtils.cleanup(LOG, ports.toArray(new Closeable[0]));
+      }
+    }
+
+    /**
+     * Release all reserved ports, assigned or not.
+     */
+    public void releaseAll() {
+      IOUtils.cleanup(LOG, allPorts.toArray(new Closeable[0]));
+      allPorts.clear();
+
+      for (String id : new ArrayList<>(assignedPorts.keySet())) {
+        release(id);
+      }
+    }
+  }
+}
