@@ -53,6 +53,11 @@ import org.apache.hadoop.ozone.client.io.KeyOutputStream;
 import org.apache.hadoop.ozone.client.io.OzoneOutputStream;
 import org.apache.hadoop.ozone.container.ContainerTestHelper;
 import org.apache.hadoop.ozone.container.TestHelper;
+import org.apache.hadoop.ozone.container.common.helpers.BlockData;
+import org.apache.hadoop.ozone.container.common.interfaces.DBHandle;
+import org.apache.hadoop.ozone.container.keyvalue.KeyValueContainer;
+import org.apache.hadoop.ozone.container.keyvalue.KeyValueContainerData;
+import org.apache.hadoop.ozone.container.keyvalue.helpers.BlockUtils;
 import org.apache.hadoop.ozone.om.helpers.OmKeyArgs;
 import org.apache.hadoop.ozone.om.helpers.OmKeyInfo;
 import org.apache.hadoop.ozone.om.helpers.OmKeyLocationInfo;
@@ -60,24 +65,19 @@ import org.apache.hadoop.ozone.om.helpers.OmKeyLocationInfo;
 import static org.apache.hadoop.fs.CommonConfigurationKeysPublic.NET_TOPOLOGY_NODE_SWITCH_MAPPING_IMPL_KEY;
 import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationFactor.THREE;
 import static org.apache.hadoop.hdds.scm.ScmConfigKeys.OZONE_SCM_STALENODE_INTERVAL;
-import org.junit.After;
+
+import org.apache.ozone.test.tag.Flaky;
 import org.junit.Assert;
-import org.junit.Ignore;
-import org.junit.Rule;
-import org.junit.Test;
-import org.junit.rules.Timeout;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.Timeout;
 
 /**
  * Tests Exception handling by Ozone Client.
  */
+@Timeout(300)
 public class TestFailureHandlingByClient {
 
-  /**
-    * Set a timeout for each test.
-    */
-  @Rule
-  public Timeout timeout = Timeout.seconds(300);
-
   private MiniOzoneCluster cluster;
   private OzoneConfiguration conf;
   private OzoneClient client;
@@ -155,7 +155,7 @@ public class TestFailureHandlingByClient {
   /**
    * Shutdown MiniDFSCluster.
    */
-  @After
+  @AfterEach
   public void shutdown() {
     if (cluster != null) {
       cluster.shutdown();
@@ -167,10 +167,8 @@ public class TestFailureHandlingByClient {
     startCluster();
     String keyName = UUID.randomUUID().toString();
     OzoneOutputStream key = createKey(keyName, ReplicationType.RATIS, 0);
-    byte[] data =
-        ContainerTestHelper
-        .getFixedLengthString(keyString, chunkSize + chunkSize / 2).getBytes(
-            UTF_8);
+    byte[] data = ContainerTestHelper.getFixedLengthString(
+        keyString, 2 * chunkSize + chunkSize / 2).getBytes(UTF_8);
     key.write(data);
 
     // get the name of a valid container
@@ -196,13 +194,114 @@ public class TestFailureHandlingByClient {
     //get the name of a valid container
     OmKeyArgs keyArgs = new OmKeyArgs.Builder().setVolumeName(volumeName)
         .setBucketName(bucketName)
-        .setReplicationConfig(new RatisReplicationConfig(THREE))
+        .setReplicationConfig(RatisReplicationConfig.getInstance(THREE))
         .setKeyName(keyName)
-        .setRefreshPipeline(true)
         .build();
     OmKeyInfo keyInfo = cluster.getOzoneManager().lookupKey(keyArgs);
+
     Assert.assertEquals(data.length, keyInfo.getDataSize());
     validateData(keyName, data);
+
+    // Verify that the block information is updated correctly in the DB on
+    // failures
+    testBlockCountOnFailures(keyInfo);
+  }
+
+  /**
+   * Test whether blockData and Container metadata (block count and used
+   * bytes) is updated correctly when there is a write failure.
+   * We can combine this test with {@link #testBlockWritesWithDnFailures()}
+   * as that test also simulates a write failure and client writes failed
+   * chunk writes to a new block.
+   */
+  private void testBlockCountOnFailures(OmKeyInfo omKeyInfo) throws Exception {
+    // testBlockWritesWithDnFailures writes chunkSize*2.5 size of data into
+    // KeyOutputStream. But before closing the outputStream, 2 of the DNs in
+    // the pipeline being written to are closed. So the key will be written
+    // to 2 blocks as atleast the last 0.5 chunk would not be committed to the
+    // first block before the stream is closed.
+    /**
+     * There are 3 possible scenarios:
+     * 1. Block1 has 2 chunks and OMKeyInfo also has 2 chunks against this block
+     *    => Block2 should have 1 chunk
+     *    (2 chunks were written to Block1, committed and acknowledged by
+     *    CommitWatcher)
+     * 2. Block1 has 1 chunk and OMKeyInfo has 1 chunk against this block
+     *    => Block2 should have 2 chunks
+     *    (Possibly 2 chunks were written but only 1 was committed to the
+     *    block)
+     * 3. Block1 has 2 chunks but OMKeyInfo has only 1 chunk against this block
+     *    => Block2 should have 2 chunks
+     *    (This happens when the 2nd chunk has been committed to Block1 but
+     *    not acknowledged by CommitWatcher before pipeline shutdown)
+     */
+
+    // Get information about the first and second block (in different pipelines)
+    List<OmKeyLocationInfo> locationList = omKeyInfo.getLatestVersionLocations()
+        .getLocationList();
+    long containerId1 = locationList.get(0).getContainerID();
+    List<DatanodeDetails> block1DNs = locationList.get(0).getPipeline()
+        .getNodes();
+    long containerId2 = locationList.get(1).getContainerID();
+    List<DatanodeDetails> block2DNs = locationList.get(1).getPipeline()
+        .getNodes();
+
+
+    int block2ExpectedChunkCount;
+    if (locationList.get(0).getLength() == 2 * chunkSize) {
+      // Scenario 1
+      block2ExpectedChunkCount = 1;
+    } else {
+      // Scenario 2
+      block2ExpectedChunkCount = 2;
+    }
+
+    // For the first block, first 2 DNs in the pipeline are shutdown (to
+    // simulate a failure). It should have 1 or 2 chunks (depending on
+    // whether the DN CommitWatcher successfully acknowledged the 2nd chunk
+    // write or not). The 3rd chunk would not exist on the first pipeline as
+    // the pipeline would be closed before the last 0.5 chunk was committed
+    // to the block.
+    KeyValueContainerData containerData1 =
+        ((KeyValueContainer) cluster.getHddsDatanode(block1DNs.get(2))
+            .getDatanodeStateMachine().getContainer().getContainerSet()
+            .getContainer(containerId1)).getContainerData();
+    try (DBHandle containerDb1 = BlockUtils.getDB(containerData1, conf)) {
+      BlockData blockData1 = containerDb1.getStore().getBlockDataTable().get(
+          containerData1.blockKey(locationList.get(0).getBlockID()
+              .getLocalID()));
+      // The first Block could have 1 or 2 chunkSize of data
+      int block1NumChunks = blockData1.getChunks().size();
+      Assert.assertTrue(block1NumChunks >= 1);
+
+      Assert.assertEquals(chunkSize * block1NumChunks, blockData1.getSize());
+      Assert.assertEquals(1, containerData1.getBlockCount());
+      Assert.assertEquals(chunkSize * block1NumChunks,
+          containerData1.getBytesUsed());
+    }
+
+    // Verify that the second block has the remaining 0.5*chunkSize of data
+    KeyValueContainerData containerData2 =
+        ((KeyValueContainer) cluster.getHddsDatanode(block2DNs.get(0))
+            .getDatanodeStateMachine().getContainer().getContainerSet()
+            .getContainer(containerId2)).getContainerData();
+    try (DBHandle containerDb2 = BlockUtils.getDB(containerData2, conf)) {
+      BlockData blockData2 = containerDb2.getStore().getBlockDataTable().get(
+          containerData2.blockKey(locationList.get(1).getBlockID()
+              .getLocalID()));
+      // The second Block should have 0.5 chunkSize of data
+      Assert.assertEquals(block2ExpectedChunkCount,
+          blockData2.getChunks().size());
+      Assert.assertEquals(1, containerData2.getBlockCount());
+      int expectedBlockSize;
+      if (block2ExpectedChunkCount == 1) {
+        expectedBlockSize = chunkSize / 2;
+      } else {
+        expectedBlockSize = chunkSize + chunkSize / 2;
+      }
+      Assert.assertEquals(expectedBlockSize, blockData2.getSize());
+      Assert.assertEquals(expectedBlockSize, containerData2.getBytesUsed());
+    }
   }
 
   @Test
@@ -212,7 +311,7 @@ public class TestFailureHandlingByClient {
     OzoneOutputStream key =
         createKey(keyName, ReplicationType.RATIS, 0);
     String data = ContainerTestHelper
-        .getFixedLengthString(keyString,  chunkSize/2);
+        .getFixedLengthString(keyString,  chunkSize / 2);
     key.write(data.getBytes(UTF_8));
     // get the name of a valid container
     Assert.assertTrue(key.getOutputStream() instanceof KeyOutputStream);
@@ -237,9 +336,8 @@ public class TestFailureHandlingByClient {
     // will be discarded and write a new block
     OmKeyArgs keyArgs = new OmKeyArgs.Builder().setVolumeName(volumeName)
         .setBucketName(bucketName)
-        .setReplicationConfig(new RatisReplicationConfig(THREE))
+        .setReplicationConfig(RatisReplicationConfig.getInstance(THREE))
         .setKeyName(keyName)
-        .setRefreshPipeline(true)
         .build();
     OmKeyInfo keyInfo = cluster.getOzoneManager().lookupKey(keyArgs);
 
@@ -299,9 +397,8 @@ public class TestFailureHandlingByClient {
     key.close();
     OmKeyArgs keyArgs = new OmKeyArgs.Builder().setVolumeName(volumeName)
         .setBucketName(bucketName)
-        .setReplicationConfig(new RatisReplicationConfig(THREE))
+        .setReplicationConfig(RatisReplicationConfig.getInstance(THREE))
         .setKeyName(keyName)
-        .setRefreshPipeline(true)
         .build();
     OmKeyInfo keyInfo = cluster.getOzoneManager().lookupKey(keyArgs);
 
@@ -315,7 +412,7 @@ public class TestFailureHandlingByClient {
   }
 
   @Test
-  @Ignore("HDDS-3298")
+  @Flaky("HDDS-3298")
   public void testDatanodeExclusionWithMajorityCommit() throws Exception {
     startCluster();
     String keyName = UUID.randomUUID().toString();
@@ -364,9 +461,8 @@ public class TestFailureHandlingByClient {
 
     OmKeyArgs keyArgs = new OmKeyArgs.Builder().setVolumeName(volumeName)
         .setBucketName(bucketName)
-        .setReplicationConfig(new RatisReplicationConfig(THREE))
+        .setReplicationConfig(RatisReplicationConfig.getInstance(THREE))
         .setKeyName(keyName)
-        .setRefreshPipeline(true)
         .build();
     OmKeyInfo keyInfo = cluster.getOzoneManager().lookupKey(keyArgs);
 
@@ -428,9 +524,8 @@ public class TestFailureHandlingByClient {
 
     OmKeyArgs keyArgs = new OmKeyArgs.Builder().setVolumeName(volumeName)
         .setBucketName(bucketName)
-        .setReplicationConfig(new RatisReplicationConfig(THREE))
+        .setReplicationConfig(RatisReplicationConfig.getInstance(THREE))
         .setKeyName(keyName)
-        .setRefreshPipeline(true)
         .build();
     OmKeyInfo keyInfo = cluster.getOzoneManager().lookupKey(keyArgs);
 
