@@ -28,6 +28,7 @@ import org.apache.hadoop.hdds.scm.ScmConfigKeys;
 import org.apache.hadoop.hdds.scm.container.ContainerInfo;
 import org.apache.hadoop.hdds.scm.protocolPB.StorageContainerLocationProtocolClientSideTranslatorPB;
 import org.apache.hadoop.hdds.security.x509.SecurityConfig;
+import org.apache.hadoop.ozone.ClientVersion;
 import org.apache.hadoop.ozone.MiniOzoneCluster;
 import org.apache.hadoop.ozone.OzoneConfigKeys;
 import org.apache.hadoop.ozone.client.CertificateClientTestImpl;
@@ -43,14 +44,24 @@ import org.apache.hadoop.ozone.om.OzoneManager;
 import org.apache.hadoop.ozone.om.helpers.OmKeyArgs;
 import org.apache.hadoop.ozone.om.helpers.OmKeyInfo;
 import org.apache.hadoop.ozone.om.helpers.OmKeyLocationInfo;
-import org.apache.hadoop.ozone.security.OzoneBlockTokenSecretManager;
+import org.apache.hadoop.ozone.om.helpers.S3SecretValue;
+import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos;
+import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.CreateVolumeRequest;
+import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.InfoVolumeRequest;
+import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.OMRequest;
+import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.OMResponse;
+import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.S3Authentication;
+import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.Status;
+import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.VolumeInfo;
+import org.apache.hadoop.hdds.security.token.OzoneBlockTokenSecretManager;
+import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.ozone.test.GenericTestUtils;
 import org.apache.ozone.test.LambdaTestUtils;
-import org.junit.AfterClass;
+import org.junit.jupiter.api.AfterAll;
 import org.junit.Assert;
-import org.junit.BeforeClass;
-import org.junit.Ignore;
-import org.junit.Test;
+import org.junit.jupiter.api.BeforeAll;
+import org.junit.jupiter.api.Disabled;
+import org.junit.jupiter.api.Test;
 
 import java.io.File;
 import java.io.IOException;
@@ -86,7 +97,7 @@ public class TestSecureOzoneRpcClient extends TestOzoneRpcClient {
    *
    * @throws IOException
    */
-  @BeforeClass
+  @BeforeAll
   public static void init() throws Exception {
     testDir = GenericTestUtils.getTestDir(
         TestSecureOzoneRpcClient.class.getSimpleName());
@@ -102,13 +113,13 @@ public class TestSecureOzoneRpcClient extends TestOzoneRpcClient {
     CertificateClientTestImpl certificateClientTest =
         new CertificateClientTestImpl(conf);
     cluster = MiniOzoneCluster.newBuilder(conf)
-        .setNumDatanodes(10)
+        .setNumDatanodes(14)
         .setScmId(SCM_ID)
         .setClusterId(CLUSTER_ID)
         .setCertificateClient(certificateClientTest)
         .build();
     secretManager = new OzoneBlockTokenSecretManager(new SecurityConfig(conf),
-        60 *60, certificateClientTest.getCertificate().
+        60 * 60, certificateClientTest.getCertificate().
         getSerialNumber().toString());
     secretManager.start(certificateClientTest);
     cluster.getOzoneManager().startSecretManager();
@@ -148,22 +159,26 @@ public class TestSecureOzoneRpcClient extends TestOzoneRpcClient {
     for (int i = 0; i < 10; i++) {
       String keyName = UUID.randomUUID().toString();
 
+      long committedBytes = ozoneManager.getMetrics().getDataCommittedBytes();
       try (OzoneOutputStream out = bucket.createKey(keyName,
-          value.getBytes(UTF_8).length, ReplicationType.STAND_ALONE,
+          value.getBytes(UTF_8).length, ReplicationType.RATIS,
           ReplicationFactor.ONE, new HashMap<>())) {
         out.write(value.getBytes(UTF_8));
       }
 
+      Assert.assertEquals(committedBytes + value.getBytes(UTF_8).length,
+              ozoneManager.getMetrics().getDataCommittedBytes());
+
       OzoneKey key = bucket.getKey(keyName);
       Assert.assertEquals(keyName, key.getName());
       byte[] fileContent;
-      try(OzoneInputStream is = bucket.readKey(keyName)) {
+      try (OzoneInputStream is = bucket.readKey(keyName)) {
         fileContent = new byte[value.getBytes(UTF_8).length];
         is.read(fileContent);
       }
 
       Assert.assertTrue(verifyRatisReplication(volumeName, bucketName,
-          keyName, ReplicationType.STAND_ALONE,
+          keyName, ReplicationType.RATIS,
           ReplicationFactor.ONE));
       Assert.assertEquals(value, new String(fileContent, UTF_8));
       Assert.assertFalse(key.getCreationTime().isBefore(testStartTime));
@@ -178,7 +193,7 @@ public class TestSecureOzoneRpcClient extends TestOzoneRpcClient {
    * 2. writeChunk
    * */
   @Test
-  @Ignore("Needs to be moved out of this class as  client setup is static")
+  @Disabled("Needs to be moved out of this class as  client setup is static")
   public void testKeyOpFailureWithoutBlockToken() throws Exception {
     String volumeName = UUID.randomUUID().toString();
     String bucketName = UUID.randomUUID().toString();
@@ -192,7 +207,7 @@ public class TestSecureOzoneRpcClient extends TestOzoneRpcClient {
       String keyName = UUID.randomUUID().toString();
 
       try (OzoneOutputStream out = bucket.createKey(keyName,
-          value.getBytes(UTF_8).length, ReplicationType.STAND_ALONE,
+          value.getBytes(UTF_8).length, ReplicationType.RATIS,
           ReplicationFactor.ONE, new HashMap<>())) {
         LambdaTestUtils.intercept(IOException.class, "UNAUTHENTICATED: Fail " +
                 "to find any token ",
@@ -207,6 +222,92 @@ public class TestSecureOzoneRpcClient extends TestOzoneRpcClient {
     }
   }
 
+
+  @Test
+  public void testS3Auth() throws Exception {
+
+    String volumeName = UUID.randomUUID().toString();
+
+    String strToSign = "AWS4-HMAC-SHA256\n" +
+        "20150830T123600Z\n" +
+        "20150830/us-east-1/iam/aws4_request\n" +
+        "f536975d06c0309214f805bb90ccff089219ecd68b2" +
+        "577efef23edd43b7e1a59";
+
+    String signature =  "5d672d79c15b13162d9279b0855cfba" +
+        "6789a8edb4c82c400e06b5924a6f2b5d7";
+
+    String secret = "wJalrXUtnFEMI/K7MDENG+bPxRfiCYEXAMPLEKEY";
+
+
+    String accessKey = UserGroupInformation.getCurrentUser().getUserName();
+
+    // Add secret to S3Secret table.
+    cluster.getOzoneManager().getMetadataManager().getS3SecretTable().put(
+        accessKey, new S3SecretValue(accessKey, secret));
+
+    OMRequest writeRequest = OMRequest.newBuilder()
+        .setCmdType(OzoneManagerProtocolProtos.Type.CreateVolume)
+        .setVersion(ClientVersion.CURRENT_VERSION)
+        .setClientId(UUID.randomUUID().toString())
+        .setCreateVolumeRequest(CreateVolumeRequest.newBuilder().
+            setVolumeInfo(VolumeInfo.newBuilder().setVolume(volumeName)
+                .setAdminName(accessKey).setOwnerName(accessKey).build())
+            .build())
+        .setS3Authentication(S3Authentication.newBuilder()
+            .setAccessId(accessKey)
+            .setSignature(signature).setStringToSign(strToSign))
+        .build();
+
+    GenericTestUtils.waitFor(() -> cluster.getOzoneManager().isLeaderReady(),
+        100, 120000);
+    OMResponse omResponse = cluster.getOzoneManager().getOmServerProtocol()
+        .submitRequest(null, writeRequest);
+
+    // Verify response.
+    Assert.assertTrue(omResponse.getStatus() == Status.OK);
+
+    // Read Request
+    OMRequest readRequest = OMRequest.newBuilder()
+        .setCmdType(OzoneManagerProtocolProtos.Type.InfoVolume)
+        .setVersion(ClientVersion.CURRENT_VERSION)
+        .setClientId(UUID.randomUUID().toString())
+        .setInfoVolumeRequest(InfoVolumeRequest.newBuilder()
+            .setVolumeName(volumeName).build())
+        .setS3Authentication(S3Authentication.newBuilder()
+            .setAccessId(accessKey)
+            .setSignature(signature).setStringToSign(strToSign))
+        .build();
+
+    omResponse = cluster.getOzoneManager().getOmServerProtocol()
+        .submitRequest(null, readRequest);
+
+    // Verify response.
+    Assert.assertTrue(omResponse.getStatus() == Status.OK);
+
+    VolumeInfo volumeInfo = omResponse.getInfoVolumeResponse().getVolumeInfo();
+    Assert.assertNotNull(volumeInfo);
+    Assert.assertEquals(volumeName, volumeInfo.getVolume());
+    Assert.assertEquals(accessKey, volumeInfo.getAdminName());
+    Assert.assertEquals(accessKey, volumeInfo.getOwnerName());
+
+    // Override secret to S3Secret table with some dummy value
+    cluster.getOzoneManager().getMetadataManager().getS3SecretTable().put(
+        accessKey, new S3SecretValue(accessKey, "dummy"));
+
+    // Write request with invalid credentials.
+    omResponse = cluster.getOzoneManager().getOmServerProtocol()
+        .submitRequest(null, writeRequest);
+    Assert.assertTrue(omResponse.getStatus() == Status.INVALID_TOKEN);
+
+  // Read request with invalid credentials.
+    omResponse = cluster.getOzoneManager().getOmServerProtocol()
+        .submitRequest(null, readRequest);
+    Assert.assertTrue(omResponse.getStatus() == Status.INVALID_TOKEN);
+
+
+  }
+
   private boolean verifyRatisReplication(String volumeName, String bucketName,
       String keyName, ReplicationType type, ReplicationFactor factor)
       throws IOException {
@@ -214,7 +315,6 @@ public class TestSecureOzoneRpcClient extends TestOzoneRpcClient {
         .setVolumeName(volumeName)
         .setBucketName(bucketName)
         .setKeyName(keyName)
-        .setRefreshPipeline(true)
         .build();
     HddsProtos.ReplicationType replicationType =
         HddsProtos.ReplicationType.valueOf(type.toString());
@@ -237,14 +337,15 @@ public class TestSecureOzoneRpcClient extends TestOzoneRpcClient {
   @Test
   @Override
   // Restart DN doesn't work with security enabled.
-  public void testZReadKeyWithUnhealthyContainerReplia() throws Exception {
+  public void testZReadKeyWithUnhealthyContainerReplica() {
   }
+
   /**
    * Close OzoneClient and shutdown MiniOzoneCluster.
    */
-  @AfterClass
+  @AfterAll
   public static void shutdown() throws IOException {
-    if(ozClient != null) {
+    if (ozClient != null) {
       ozClient.close();
     }
 
