@@ -19,10 +19,11 @@ package org.apache.hadoop.ozone.client.rpc.read;
 
 import java.io.IOException;
 import java.nio.ByteBuffer;
+
 import org.apache.hadoop.hdds.scm.storage.BlockInputStream;
 import org.apache.hadoop.hdds.scm.storage.ChunkInputStream;
 import org.apache.hadoop.ozone.client.io.KeyInputStream;
-import org.apache.hadoop.ozone.container.common.impl.ChunkLayOutVersion;
+import org.apache.hadoop.ozone.container.common.impl.ContainerLayoutVersion;
 import org.junit.Assert;
 import org.junit.Test;
 
@@ -31,7 +32,7 @@ import org.junit.Test;
  */
 public class TestChunkInputStream extends TestInputStreamBase {
 
-  public TestChunkInputStream(ChunkLayOutVersion layout) {
+  public TestChunkInputStream(ContainerLayoutVersion layout) {
     super(layout);
   }
 
@@ -43,6 +44,7 @@ public class TestChunkInputStream extends TestInputStreamBase {
   public void testAll() throws Exception {
     testChunkReadBuffers();
     testBufferRelease();
+    testCloseReleasesBuffers();
   }
 
 
@@ -57,7 +59,8 @@ public class TestChunkInputStream extends TestInputStreamBase {
 
     KeyInputStream keyInputStream = getKeyInputStream(keyName);
 
-    BlockInputStream block0Stream = keyInputStream.getBlockStreams().get(0);
+    BlockInputStream block0Stream =
+        (BlockInputStream)keyInputStream.getBlockStreams().get(0);
     block0Stream.initialize();
 
     ChunkInputStream chunk0Stream = block0Stream.getChunkStreams().get(0);
@@ -110,6 +113,25 @@ public class TestChunkInputStream extends TestInputStreamBase {
         "reaching EOF.", chunk0Stream.getCachedBuffers());
   }
 
+  private void testCloseReleasesBuffers() throws Exception {
+    String keyName = getNewKeyName();
+    writeRandomBytes(keyName, CHUNK_SIZE);
+
+    try (KeyInputStream keyInputStream = getKeyInputStream(keyName)) {
+      BlockInputStream block0Stream =
+          (BlockInputStream) keyInputStream.getBlockStreams().get(0);
+      block0Stream.initialize();
+
+      ChunkInputStream chunk0Stream = block0Stream.getChunkStreams().get(0);
+      readDataFromChunk(chunk0Stream, 0, 1);
+      Assert.assertNotNull(chunk0Stream.getCachedBuffers());
+
+      chunk0Stream.close();
+
+      Assert.assertNull(chunk0Stream.getCachedBuffers());
+    }
+  }
+
   /**
    * Test that ChunkInputStream buffers are released as soon as the last byte
    * of the buffer is read.
@@ -121,7 +143,8 @@ public class TestChunkInputStream extends TestInputStreamBase {
 
     try (KeyInputStream keyInputStream = getKeyInputStream(keyName)) {
 
-      BlockInputStream block0Stream = keyInputStream.getBlockStreams().get(0);
+      BlockInputStream block0Stream =
+          (BlockInputStream)keyInputStream.getBlockStreams().get(0);
       block0Stream.initialize();
 
       ChunkInputStream chunk0Stream = block0Stream.getChunkStreams().get(0);
