@@ -23,8 +23,10 @@ import java.nio.ByteBuffer;
 import java.util.Arrays;
 
 import java.util.List;
+import java.util.Random;
 import java.util.concurrent.TimeoutException;
 
+import org.apache.hadoop.hdds.client.ECReplicationConfig;
 import org.apache.hadoop.hdds.client.RatisReplicationConfig;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos;
@@ -33,12 +35,13 @@ import org.apache.hadoop.hdds.scm.XceiverClientManager;
 import org.apache.hadoop.hdds.scm.XceiverClientMetrics;
 import org.apache.hadoop.hdds.scm.node.NodeManager;
 import org.apache.hadoop.hdds.scm.node.states.NodeNotFoundException;
+import org.apache.hadoop.hdds.scm.storage.BlockExtendedInputStream;
 import org.apache.hadoop.hdds.scm.storage.BlockInputStream;
 import org.apache.hadoop.hdds.scm.storage.ChunkInputStream;
 import org.apache.hadoop.ozone.client.io.KeyInputStream;
 import org.apache.hadoop.ozone.common.utils.BufferUtils;
 import org.apache.hadoop.ozone.container.TestHelper;
-import org.apache.hadoop.ozone.container.common.impl.ChunkLayOutVersion;
+import org.apache.hadoop.ozone.container.common.impl.ContainerLayoutVersion;
 import org.apache.hadoop.ozone.om.helpers.OmKeyArgs;
 import org.apache.hadoop.ozone.om.helpers.OmKeyInfo;
 import org.apache.hadoop.ozone.om.helpers.OmKeyLocationInfo;
@@ -50,6 +53,7 @@ import org.junit.Test;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import static org.apache.hadoop.hdds.client.ECReplicationConfig.EcCodec.RS;
 import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationFactor.THREE;
 import static org.apache.hadoop.ozone.container.TestHelper.countReplicas;
 import static org.junit.Assert.fail;
@@ -62,7 +66,7 @@ public class TestKeyInputStream extends TestInputStreamBase {
   private static final Logger LOG =
       LoggerFactory.getLogger(TestKeyInputStream.class);
 
-  public TestKeyInputStream(ChunkLayOutVersion layout) {
+  public TestKeyInputStream(ContainerLayoutVersion layout) {
     super(layout);
   }
 
@@ -77,23 +81,41 @@ public class TestKeyInputStream extends TestInputStreamBase {
   private void randomSeek(int dataLength, KeyInputStream keyInputStream,
       byte[] inputData) throws Exception {
     // Do random seek.
-    for (int i=0; i<dataLength - 300; i+=20) {
+    for (int i = 0; i < dataLength - 300; i += 20) {
       validate(keyInputStream, inputData, i, 200);
     }
 
     // Seek to end and read in reverse order. And also this is partial chunks
     // as readLength is 20, chunk length is 100.
-    for (int i=dataLength - 100; i>=100; i-=20) {
+    for (int i = dataLength - 100; i >= 100; i -= 20) {
       validate(keyInputStream, inputData, i, 20);
     }
 
     // Start from begin and seek such that we read partially chunks.
-    for (int i=0; i<dataLength - 300; i+=20) {
+    for (int i = 0; i < dataLength - 300; i += 20) {
       validate(keyInputStream, inputData, i, 90);
     }
 
   }
 
+  /**
+   * This method does random seeks and reads and validates the reads are
+   * correct or not.
+   * @param dataLength
+   * @param keyInputStream
+   * @param inputData
+   * @param readSize
+   * @throws Exception
+   */
+  private void randomPositionSeek(int dataLength, KeyInputStream keyInputStream,
+      byte[] inputData, int readSize) throws Exception {
+    Random rand = new Random();
+    for (int i = 0; i < 100; i++) {
+      int position = rand.nextInt(dataLength - readSize);
+      validate(keyInputStream, inputData, position, readSize);
+    }
+  }
+
   /**
    * This method seeks to specified seek value and read the data specified by
    * readLength and validate the read is correct or not.
@@ -125,6 +147,7 @@ public class TestKeyInputStream extends TestInputStreamBase {
     testReadChunkWithByteArray();
     testReadChunkWithByteBuffer();
     testSkip();
+    testECSeek();
   }
 
   public void testInputStreams() throws Exception {
@@ -137,11 +160,13 @@ public class TestKeyInputStream extends TestInputStreamBase {
     // Verify BlockStreams and ChunkStreams
     int expectedNumBlockStreams = BufferUtils.getNumberOfBins(
         dataLength, BLOCK_SIZE);
-    List<BlockInputStream> blockStreams = keyInputStream.getBlockStreams();
+    List<BlockExtendedInputStream> blockStreams =
+        keyInputStream.getBlockStreams();
     Assert.assertEquals(expectedNumBlockStreams, blockStreams.size());
 
     int readBlockLength = 0;
-    for (BlockInputStream blockStream : blockStreams) {
+    for (BlockExtendedInputStream stream : blockStreams) {
+      BlockInputStream blockStream = (BlockInputStream) stream;
       int blockStreamLength = Math.min(BLOCK_SIZE,
           dataLength - readBlockLength);
       Assert.assertEquals(blockStreamLength, blockStream.getLength());
@@ -173,7 +198,7 @@ public class TestKeyInputStream extends TestInputStreamBase {
     KeyInputStream keyInputStream = getKeyInputStream(keyName);
 
     // Seek to some where end.
-    validate(keyInputStream, inputData, dataLength-200, 100);
+    validate(keyInputStream, inputData, dataLength - 200, 100);
 
     // Now seek to start.
     validate(keyInputStream, inputData, 0, 140);
@@ -195,6 +220,36 @@ public class TestKeyInputStream extends TestInputStreamBase {
     keyInputStream.close();
   }
 
+  public void testECSeek() throws Exception {
+    int ecChunkSize = 1024 * 1024;
+    ECReplicationConfig repConfig = new ECReplicationConfig(3, 2, RS,
+        ecChunkSize);
+    String keyName = getNewKeyName();
+    // 3 full EC blocks plus one chunk
+    int dataLength = (9 * BLOCK_SIZE + ecChunkSize);
+
+    byte[] inputData = writeRandomBytes(keyName, repConfig, dataLength);
+    try (KeyInputStream keyInputStream = getKeyInputStream(keyName)) {
+
+      validate(keyInputStream, inputData, 0, ecChunkSize + 1234);
+
+      validate(keyInputStream, inputData, 200, ecChunkSize);
+
+      validate(keyInputStream, inputData, BLOCK_SIZE * 4, ecChunkSize);
+
+      validate(keyInputStream, inputData, BLOCK_SIZE * 4 + 200, ecChunkSize);
+
+      validate(keyInputStream, inputData, dataLength - ecChunkSize - 100,
+          ecChunkSize + 50);
+
+      randomPositionSeek(dataLength, keyInputStream, inputData,
+          ecChunkSize + 200);
+
+      // Read entire key.
+      validate(keyInputStream, inputData, 0, dataLength);
+    }
+  }
+
   public void testSeek() throws Exception {
     XceiverClientManager.resetXceiverClientMetrics();
     XceiverClientMetrics metrics = XceiverClientManager
@@ -353,7 +408,7 @@ public class TestKeyInputStream extends TestInputStreamBase {
     OmKeyArgs keyArgs = new OmKeyArgs.Builder().setVolumeName(getVolumeName())
         .setBucketName(getBucketName())
         .setKeyName(keyName)
-        .setReplicationConfig(new RatisReplicationConfig(THREE))
+        .setReplicationConfig(RatisReplicationConfig.getInstance(THREE))
         .build();
     OmKeyInfo keyInfo = getCluster().getOzoneManager().lookupKey(keyArgs);
 
