@@ -53,6 +53,7 @@ import java.util.UUID;
 
 import org.junit.Rule;
 import org.junit.rules.Timeout;
+import org.slf4j.LoggerFactory;
 
 import static java.nio.charset.StandardCharsets.UTF_8;
 import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationFactor.ONE;
@@ -87,6 +88,7 @@ public class TestDeleteContainerHandler {
     cluster = MiniOzoneCluster.newBuilder(conf)
         .setNumDatanodes(1).build();
     cluster.waitForClusterToBeReady();
+    cluster.waitForPipelineTobeReady(ONE, 30000);
 
     OzoneClient client = OzoneClientFactory.getRpcClient(conf);
     objectStore = client.getObjectStore();
@@ -126,7 +128,7 @@ public class TestDeleteContainerHandler {
         .getPipelineManager().getPipeline(container.getPipelineID());
 
     // We need to close the container because delete container only happens
-    // on closed containers with force flag set to false.
+    // on closed containers when force flag is set to false.
 
     HddsDatanodeService hddsDatanodeService =
         cluster.getHddsDatanodes().get(0);
@@ -164,16 +166,33 @@ public class TestDeleteContainerHandler {
         cluster.getStorageContainerManager().getScmContext().getTermOfLeader());
     nodeManager.addDatanodeCommand(datanodeDetails.getUuid(), command);
 
+    // Deleting a non-empty container should fail on DN when the force flag
+    // is false.
+    // Check the log for the error message when deleting non-empty containers
+    GenericTestUtils.LogCapturer logCapturer =
+        GenericTestUtils.LogCapturer.captureLogs(
+            LoggerFactory.getLogger(DeleteContainerCommandHandler.class));
+    GenericTestUtils.waitFor(() -> logCapturer.getOutput().contains("Non" +
+        "-force deletion of non-empty container is not allowed"), 500,
+        5 * 1000);
+
+    // Set container blockCount to 0 to mock that it is empty
+    getContainerfromDN(hddsDatanodeService, containerId.getId())
+        .getContainerData().setBlockCount(0);
+
+    // Send the delete command again. It should succeed this time.
+    command.setTerm(
+        cluster.getStorageContainerManager().getScmContext().getTermOfLeader());
+    nodeManager.addDatanodeCommand(datanodeDetails.getUuid(), command);
+
     GenericTestUtils.waitFor(() ->
             isContainerDeleted(hddsDatanodeService, containerId.getId()),
         500, 5 * 1000);
 
     Assert.assertTrue(isContainerDeleted(hddsDatanodeService,
         containerId.getId()));
-
   }
 
-
   @Test
   public void testDeleteContainerRequestHandlerOnOpenContainer()
       throws Exception {
@@ -243,7 +262,7 @@ public class TestDeleteContainerHandler {
   private void createKey(String keyName) throws IOException {
     OzoneOutputStream key = objectStore.getVolume(volumeName)
         .getBucket(bucketName)
-        .createKey(keyName, 1024, ReplicationType.STAND_ALONE,
+        .createKey(keyName, 1024, ReplicationType.RATIS,
             ReplicationFactor.ONE, new HashMap<>());
     key.write("test".getBytes(UTF_8));
     key.close();
@@ -259,9 +278,8 @@ public class TestDeleteContainerHandler {
     OmKeyArgs keyArgs =
         new OmKeyArgs.Builder().setVolumeName(volumeName)
             .setBucketName(bucketName)
-            .setReplicationConfig(new StandaloneReplicationConfig(ONE))
+            .setReplicationConfig(StandaloneReplicationConfig.getInstance(ONE))
             .setKeyName(keyName)
-            .setRefreshPipeline(true)
             .build();
 
     OmKeyLocationInfo omKeyLocationInfo =
@@ -281,9 +299,8 @@ public class TestDeleteContainerHandler {
   private Boolean isContainerClosed(HddsDatanodeService hddsDatanodeService,
       long containerID) {
     ContainerData containerData;
-    containerData =hddsDatanodeService
-        .getDatanodeStateMachine().getContainer().getContainerSet()
-        .getContainer(containerID).getContainerData();
+    containerData = getContainerfromDN(hddsDatanodeService, containerID)
+        .getContainerData();
     return !containerData.isOpen();
   }
 
@@ -297,9 +314,16 @@ public class TestDeleteContainerHandler {
       long containerID) {
     Container container;
     // if container is not in container set, it means container got deleted.
-    container = hddsDatanodeService
-        .getDatanodeStateMachine().getContainer().getContainerSet()
-        .getContainer(containerID);
+    container = getContainerfromDN(hddsDatanodeService, containerID);
     return container == null;
   }
+
+  /**
+   * Return the container for the given containerID from the given DN.
+   */
+  private Container getContainerfromDN(HddsDatanodeService hddsDatanodeService,
+      long containerID) {
+    return hddsDatanodeService.getDatanodeStateMachine().getContainer()
+        .getContainerSet().getContainer(containerID);
+  }
 }
