@@ -18,7 +18,6 @@
 
 package org.apache.hadoop.ozone.container.ozoneimpl;
 
-import org.apache.hadoop.hdds.HddsConfigKeys;
 import org.apache.hadoop.hdds.client.BlockID;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos;
@@ -33,29 +32,30 @@ import org.apache.hadoop.hdds.scm.pipeline.Pipeline;
 import org.apache.hadoop.ozone.container.common.statemachine.DatanodeStateMachine;
 import org.apache.hadoop.ozone.container.common.statemachine.StateContext;
 import org.junit.Assert;
-import org.junit.Ignore;
 import org.junit.Rule;
-import org.junit.Test;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.Timeout;
+import org.junit.jupiter.migrationsupport.rules.EnableRuleMigrationSupport;
 import org.junit.rules.TemporaryFolder;
-import org.junit.rules.Timeout;
 import org.mockito.Mockito;
 
-import java.util.*;
+import java.util.HashMap;
+import java.util.LinkedList;
+import java.util.List;
+import java.util.Map;
+import java.util.UUID;
 import java.util.concurrent.CompletableFuture;
 
+import static org.apache.hadoop.hdds.HddsConfigKeys.OZONE_METADATA_DIRS;
 import static org.apache.hadoop.hdds.protocol.MockDatanodeDetails.randomDatanodeDetails;
 import static org.apache.hadoop.hdds.scm.ScmConfigKeys.HDDS_DATANODE_DIR_KEY;
 
 /**
  * Tests ozone containers.
  */
-@Ignore
+@EnableRuleMigrationSupport
+@Timeout(300)
 public class TestOzoneContainer {
-  /**
-   * Set the timeout for every test.
-   */
-  @Rule
-  public Timeout testTimeout = Timeout.seconds(300);
 
   @Rule
   public TemporaryFolder tempFolder = new TemporaryFolder();
@@ -65,19 +65,15 @@ public class TestOzoneContainer {
     long containerID = ContainerTestHelper.getTestContainerID();
     OzoneConfiguration conf = newOzoneConfiguration();
     OzoneContainer container = null;
-    MiniOzoneCluster cluster = null;
     try {
-      cluster = MiniOzoneCluster.newBuilder(conf).build();
-      cluster.waitForClusterToBeReady();
       // We don't start Ozone Container via data node, we will do it
       // independently in our test path.
       Pipeline pipeline = MockPipeline.createSingleNodePipeline();
-      conf.set(HDDS_DATANODE_DIR_KEY, tempFolder.getRoot().getPath());
+      conf.set(OZONE_METADATA_DIRS, tempFolder.newFolder().getPath());
+      conf.set(HDDS_DATANODE_DIR_KEY, tempFolder.newFolder().getPath());
       conf.setInt(OzoneConfigKeys.DFS_CONTAINER_IPC_PORT,
           pipeline.getFirstNode()
               .getPort(DatanodeDetails.Port.Name.STANDALONE).getValue());
-      conf.setBoolean(
-          OzoneConfigKeys.DFS_CONTAINER_IPC_RANDOM_PORT, false);
 
       DatanodeDetails datanodeDetails = randomDatanodeDetails();
       StateContext context = Mockito.mock(StateContext.class);
@@ -95,30 +91,21 @@ public class TestOzoneContainer {
       if (container != null) {
         container.stop();
       }
-      if (cluster != null) {
-        cluster.shutdown();
-      }
     }
   }
 
   @Test
   public void testOzoneContainerStart() throws Exception {
     OzoneConfiguration conf = newOzoneConfiguration();
-    MiniOzoneCluster cluster = null;
     OzoneContainer container = null;
 
     try {
-      cluster = MiniOzoneCluster.newBuilder(conf).build();
-      cluster.waitForClusterToBeReady();
-
       Pipeline pipeline = MockPipeline.createSingleNodePipeline();
-      conf.set(HDDS_DATANODE_DIR_KEY, tempFolder.getRoot().getPath());
+      conf.set(OZONE_METADATA_DIRS, tempFolder.newFolder().getPath());
+      conf.set(HDDS_DATANODE_DIR_KEY, tempFolder.newFolder().getPath());
       conf.setInt(OzoneConfigKeys.DFS_CONTAINER_IPC_PORT,
           pipeline.getFirstNode()
               .getPort(DatanodeDetails.Port.Name.STANDALONE).getValue());
-      conf.setBoolean(
-          OzoneConfigKeys.DFS_CONTAINER_IPC_RANDOM_PORT, false);
-
 
       DatanodeDetails datanodeDetails = randomDatanodeDetails();
       StateContext context = Mockito.mock(StateContext.class);
@@ -147,9 +134,6 @@ public class TestOzoneContainer {
       if (container != null) {
         container.stop();
       }
-      if (cluster != null) {
-        cluster.shutdown();
-      }
     }
   }
 
@@ -168,20 +152,13 @@ public class TestOzoneContainer {
       OzoneConfiguration conf = newOzoneConfiguration();
 
       // Start ozone container Via Datanode create.
-
-      Pipeline pipeline =
-          MockPipeline.createSingleNodePipeline();
-      conf.setInt(OzoneConfigKeys.DFS_CONTAINER_IPC_PORT,
-          pipeline.getFirstNode()
-              .getPort(DatanodeDetails.Port.Name.STANDALONE).getValue());
-
       cluster = MiniOzoneCluster.newBuilder(conf)
-          .setRandomContainerPort(false)
+          .setNumDatanodes(1)
           .build();
       cluster.waitForClusterToBeReady();
 
       // This client talks to ozone container via datanode.
-      XceiverClientGrpc client = new XceiverClientGrpc(pipeline, conf);
+      XceiverClientGrpc client = createClientForTesting(cluster);
 
       runTestOzoneContainerViaDataNode(containerID, client);
     } finally {
@@ -231,22 +208,9 @@ public class TestOzoneContainer {
           getChunksCount();
       ContainerTestHelper.verifyGetBlock(request, response, chunksCount);
 
-
-      // Delete Block
-      request =
-          ContainerTestHelper.getDeleteBlockRequest(
-              pipeline, putBlockRequest.getPutBlock());
-      response = client.sendCommand(request);
-      Assert.assertNotNull(response);
-      Assert.assertEquals(ContainerProtos.Result.SUCCESS, response.getResult());
-
-      //Delete Chunk
-      request = ContainerTestHelper.getDeleteChunkRequest(
-          pipeline, writeChunkRequest.getWriteChunk());
-
-      response = client.sendCommand(request);
-      Assert.assertNotNull(response);
-      Assert.assertEquals(ContainerProtos.Result.SUCCESS, response.getResult());
+      // Delete Block and Delete Chunk are handled by BlockDeletingService
+      // ContainerCommandRequestProto DeleteBlock and DeleteChunk requests
+      // are deprecated
 
       //Update an existing container
       Map<String, String> containerUpdate = new HashMap<String, String>();
@@ -279,14 +243,15 @@ public class TestOzoneContainer {
     XceiverClientGrpc client = null;
     try {
       OzoneConfiguration conf = newOzoneConfiguration();
-      conf.set(HddsConfigKeys.OZONE_METADATA_DIRS,
-          tempFolder.getRoot().getPath());
-      client = createClientForTesting(conf);
+      conf.set(OZONE_METADATA_DIRS, tempFolder.newFolder().getPath());
+      conf.set(HDDS_DATANODE_DIR_KEY, tempFolder.newFolder().getPath());
       cluster = MiniOzoneCluster.newBuilder(conf)
-          .setRandomContainerPort(false)
+          .setNumDatanodes(1)
           .build();
       cluster.waitForClusterToBeReady();
       long containerID = ContainerTestHelper.getTestContainerID();
+
+      client = createClientForTesting(cluster);
       runTestBothGetandPutSmallFile(containerID, client);
     } finally {
       if (cluster != null) {
@@ -306,6 +271,8 @@ public class TestOzoneContainer {
       final ContainerProtos.ContainerCommandRequestProto smallFileRequest
           = ContainerTestHelper.getWriteSmallFileRequest(
           client.getPipeline(), blockId, 1024);
+      final byte[] requestBytes = smallFileRequest.getPutSmallFile().getData()
+          .toByteArray();
       ContainerProtos.ContainerCommandResponseProto response
           = client.sendCommand(smallFileRequest);
       Assert.assertNotNull(response);
@@ -314,9 +281,18 @@ public class TestOzoneContainer {
           = ContainerTestHelper.getReadSmallFileRequest(client.getPipeline(),
           smallFileRequest.getPutSmallFile().getBlock());
       response = client.sendCommand(getSmallFileRequest);
-      Assert.assertArrayEquals(
-          smallFileRequest.getPutSmallFile().getData().toByteArray(),
-          response.getGetSmallFile().getData().getData().toByteArray());
+
+      Assert.assertEquals(ContainerProtos.Result.SUCCESS, response.getResult());
+
+      ContainerProtos.ReadChunkResponseProto chunkResponse =
+          response.getGetSmallFile().getData();
+      if (chunkResponse.hasDataBuffers()) {
+        Assert.assertArrayEquals(requestBytes,
+            chunkResponse.getDataBuffers().toByteArray());
+      } else {
+        Assert.assertArrayEquals(requestBytes,
+            chunkResponse.getData().toByteArray());
+      }
     } finally {
       if (client != null) {
         client.close();
@@ -336,13 +312,14 @@ public class TestOzoneContainer {
     try {
 
       OzoneConfiguration conf = newOzoneConfiguration();
-      conf.set(HddsConfigKeys.OZONE_METADATA_DIRS,
-          tempFolder.getRoot().getPath());
-      client = createClientForTesting(conf);
+      conf.set(OZONE_METADATA_DIRS, tempFolder.newFolder().getPath());
+      conf.set(HDDS_DATANODE_DIR_KEY, tempFolder.newFolder().getPath());
       cluster = MiniOzoneCluster.newBuilder(conf)
-          .setRandomContainerPort(false)
+          .setNumDatanodes(1)
           .build();
       cluster.waitForClusterToBeReady();
+
+      client = createClientForTesting(cluster);
       client.connect();
 
       long containerID = ContainerTestHelper.getTestContainerID();
@@ -396,15 +373,6 @@ public class TestOzoneContainer {
       int chunksCount = putBlockRequest.getPutBlock().getBlockData()
           .getChunksCount();
       ContainerTestHelper.verifyGetBlock(request, response, chunksCount);
-
-      // Delete block must fail on a closed container.
-      request =
-          ContainerTestHelper.getDeleteBlockRequest(client.getPipeline(),
-              putBlockRequest.getPutBlock());
-      response = client.sendCommand(request);
-      Assert.assertNotNull(response);
-      Assert.assertEquals(ContainerProtos.Result.CLOSED_CONTAINER_IO,
-          response.getResult());
     } finally {
       if (client != null) {
         client.close();
@@ -424,13 +392,14 @@ public class TestOzoneContainer {
         writeChunkRequest, putBlockRequest;
     try {
       OzoneConfiguration conf = newOzoneConfiguration();
-      conf.set(HddsConfigKeys.OZONE_METADATA_DIRS,
-          tempFolder.getRoot().getPath());
-      client = createClientForTesting(conf);
+      conf.set(OZONE_METADATA_DIRS, tempFolder.newFolder().getPath());
+      conf.set(HDDS_DATANODE_DIR_KEY, tempFolder.newFolder().getPath());
       cluster = MiniOzoneCluster.newBuilder(conf)
-          .setRandomContainerPort(false)
+          .setNumDatanodes(1)
           .build();
       cluster.waitForClusterToBeReady();
+
+      client = createClientForTesting(cluster);
       client.connect();
 
       long containerID = ContainerTestHelper.getTestContainerID();
@@ -487,7 +456,7 @@ public class TestOzoneContainer {
       final List<CompletableFuture> computeResults = new LinkedList<>();
       int requestCount = 1000;
       // Create a bunch of Async calls from this test.
-      for(int x = 0; x <requestCount; x++) {
+      for (int x = 0; x < requestCount; x++) {
         BlockID blockID = ContainerTestHelper.getTestBlockID(containerID);
         final ContainerProtos.ContainerCommandRequestProto smallFileRequest
             = ContainerTestHelper.getWriteSmallFileRequest(
@@ -520,14 +489,15 @@ public class TestOzoneContainer {
     XceiverClientGrpc client = null;
     try {
       OzoneConfiguration conf = newOzoneConfiguration();
-      conf.set(HddsConfigKeys.OZONE_METADATA_DIRS,
-          tempFolder.getRoot().getPath());
-      client = createClientForTesting(conf);
+      conf.set(OZONE_METADATA_DIRS, tempFolder.newFolder().getPath());
+      conf.set(HDDS_DATANODE_DIR_KEY, tempFolder.newFolder().getPath());
       cluster = MiniOzoneCluster.newBuilder(conf)
-          .setRandomContainerPort(false)
+          .setNumDatanodes(1)
           .build();
       cluster.waitForClusterToBeReady();
       long containerID = ContainerTestHelper.getTestContainerID();
+
+      client = createClientForTesting(cluster);
       runAsyncTests(containerID, client);
     } finally {
       if (cluster != null) {
@@ -537,16 +507,10 @@ public class TestOzoneContainer {
   }
 
   private static XceiverClientGrpc createClientForTesting(
-      OzoneConfiguration conf) throws Exception {
-    // Start ozone container Via Datanode create.
-    Pipeline pipeline =
-        MockPipeline.createSingleNodePipeline();
-    conf.setInt(OzoneConfigKeys.DFS_CONTAINER_IPC_PORT,
-        pipeline.getFirstNode()
-            .getPort(DatanodeDetails.Port.Name.STANDALONE).getValue());
-
-    // This client talks to ozone container via datanode.
-    return new XceiverClientGrpc(pipeline, conf);
+      MiniOzoneCluster cluster) {
+    Pipeline pipeline = cluster.getStorageContainerManager()
+        .getPipelineManager().getPipelines().iterator().next();
+    return new XceiverClientGrpc(pipeline, cluster.getConf());
   }
 
   public static void createContainerForTesting(XceiverClientSpi client,
@@ -567,7 +531,7 @@ public class TestOzoneContainer {
     BlockID blockID = ContainerTestHelper.getTestBlockID(containerID);
     ContainerProtos.ContainerCommandRequestProto writeChunkRequest =
         ContainerTestHelper.getWriteChunkRequest(client.getPipeline(),
-            blockID, dataLen, null);
+            blockID, dataLen);
     ContainerProtos.ContainerCommandResponseProto response =
         client.sendCommand(writeChunkRequest);
     Assert.assertNotNull(response);
