@@ -45,7 +45,6 @@ import org.apache.hadoop.ozone.om.exceptions.OMException;
 import org.apache.hadoop.ozone.om.helpers.BucketLayout;
 import org.apache.hadoop.ozone.om.helpers.OmDirectoryInfo;
 import org.apache.hadoop.ozone.om.helpers.OmKeyInfo;
-import org.apache.hadoop.ozone.om.request.TestOMRequestUtils;
 import org.apache.hadoop.util.StringUtils;
 import org.apache.ozone.test.GenericTestUtils;
 import org.junit.Assert;
@@ -68,9 +67,8 @@ import java.util.UUID;
 import java.util.concurrent.TimeoutException;
 
 import static org.apache.hadoop.hdds.client.ReplicationFactor.ONE;
-import static org.apache.hadoop.hdds.client.ReplicationType.STAND_ALONE;
+import static org.apache.hadoop.hdds.client.ReplicationType.RATIS;
 import static org.apache.hadoop.ozone.OzoneConfigKeys.OZONE_FS_ITERATE_BATCH_SIZE;
-import static org.apache.hadoop.ozone.OzoneConsts.OM_KEY_PREFIX;
 import static org.apache.hadoop.ozone.OzoneConsts.OZONE_URI_SCHEME;
 import static org.apache.hadoop.ozone.om.exceptions.OMException.ResultCodes.KEY_ALREADY_EXISTS;
 import static org.apache.hadoop.ozone.om.exceptions.OMException.ResultCodes.KEY_NOT_FOUND;
@@ -100,13 +98,14 @@ public class TestObjectStoreWithFSO {
    *
    * @throws IOException
    */
-  @BeforeClass public static void init() throws Exception {
+  @BeforeClass
+  public static void init() throws Exception {
     conf = new OzoneConfiguration();
     clusterId = UUID.randomUUID().toString();
     scmId = UUID.randomUUID().toString();
     omId = UUID.randomUUID().toString();
-    TestOMRequestUtils.configureFSOptimizedPaths(conf, true,
-        OMConfigKeys.OZONE_OM_METADATA_LAYOUT_PREFIX);
+    conf.set(OMConfigKeys.OZONE_DEFAULT_BUCKET_LAYOUT,
+        BucketLayout.FILE_SYSTEM_OPTIMIZED.name());
     cluster = MiniOzoneCluster.newBuilder(conf).setClusterId(clusterId)
         .setScmId(scmId).setOmId(omId).build();
     cluster.waitForClusterToBeReady();
@@ -196,7 +195,8 @@ public class TestObjectStoreWithFSO {
     ozoneOutputStream.close();
 
     Table<String, OmKeyInfo> fileTable =
-            cluster.getOzoneManager().getMetadataManager().getKeyTable();
+        cluster.getOzoneManager().getMetadataManager()
+            .getKeyTable(getBucketLayout());
     // After closing the file. File entry should be removed from openFileTable
     // and it should be added to fileTable.
     verifyKeyInFileTable(fileTable, file, dirPathC.getObjectID(), false);
@@ -211,6 +211,87 @@ public class TestObjectStoreWithFSO {
             dirPathC.getObjectID(), true);
   }
 
+  /**
+   * Tests bucket deletion behaviour. Buckets should not be allowed to be
+   * deleted if they contain files or directories under them.
+   *
+   * @throws Exception
+   */
+  @Test
+  public void testDeleteBucketWithKeys() throws Exception {
+    // Create temporary volume and bucket for this test.
+    OzoneBucket testBucket = TestDataUtil
+        .createVolumeAndBucket(cluster, BucketLayout.FILE_SYSTEM_OPTIMIZED);
+    String testVolumeName = testBucket.getVolumeName();
+    String testBucketName = testBucket.getName();
+
+    String parent = "a/b/c/";
+    String file = "key" + RandomStringUtils.randomNumeric(5);
+    String key = parent + file;
+
+    OzoneClient client = cluster.getClient();
+
+    ObjectStore objectStore = client.getObjectStore();
+    OzoneVolume ozoneVolume = objectStore.getVolume(testVolumeName);
+    assertEquals(ozoneVolume.getName(), testVolumeName);
+    OzoneBucket ozoneBucket = ozoneVolume.getBucket(testBucketName);
+    assertEquals(ozoneBucket.getName(), testBucketName);
+
+    Table<String, OmKeyInfo> openFileTable =
+        cluster.getOzoneManager().getMetadataManager()
+            .getOpenKeyTable(getBucketLayout());
+
+    // before file creation
+    verifyKeyInFileTable(openFileTable, file, 0, true);
+
+    // Create a key.
+    ozoneBucket.createKey(key, 10).close();
+    Assert.assertFalse(cluster.getOzoneManager().getMetadataManager()
+        .isBucketEmpty(testVolumeName, testBucketName));
+
+    try {
+      // Try to delete the bucket while a key is present under it.
+      ozoneVolume.deleteBucket(testBucketName);
+      fail("Bucket Deletion should fail, since bucket is not empty.");
+    } catch (IOException ioe) {
+      // Do nothing
+    }
+
+    // Delete the key (this only deletes the file)
+    ozoneBucket.deleteKey(key);
+    Assert.assertFalse(cluster.getOzoneManager().getMetadataManager()
+        .isBucketEmpty(testVolumeName, testBucketName));
+    try {
+      // Try to delete the bucket while intermediate dirs are present under it.
+      ozoneVolume.deleteBucket(testBucketName);
+      fail("Bucket Deletion should fail, since bucket still contains " +
+          "intermediate directories");
+    } catch (IOException ioe) {
+      // Do nothing
+    }
+
+    // Delete last level of directories.
+    ozoneBucket.deleteDirectory(parent, true);
+    Assert.assertFalse(cluster.getOzoneManager().getMetadataManager()
+        .isBucketEmpty(testVolumeName, testBucketName));
+    try {
+      // Try to delete the bucket while dirs are present under it.
+      ozoneVolume.deleteBucket(testBucketName);
+      fail("Bucket Deletion should fail, since bucket still contains " +
+          "intermediate directories");
+    } catch (IOException ioe) {
+      // Do nothing
+    }
+
+    // Delete all the intermediate directories
+    ozoneBucket.deleteDirectory("a/", true);
+    Assert.assertTrue(cluster.getOzoneManager().getMetadataManager()
+        .isBucketEmpty(testVolumeName, testBucketName));
+    ozoneVolume.deleteBucket(testBucketName);
+    // Cleanup the Volume.
+    cluster.getClient().getObjectStore().deleteVolume(testVolumeName);
+  }
+
   @Test
   public void testLookupKey() throws Exception {
     String parent = "a/b/c/";
@@ -264,7 +345,8 @@ public class TestObjectStoreWithFSO {
     Assert.assertEquals(key, keyDetails.getName());
 
     Table<String, OmKeyInfo> fileTable =
-            cluster.getOzoneManager().getMetadataManager().getKeyTable();
+        cluster.getOzoneManager().getMetadataManager()
+            .getKeyTable(getBucketLayout());
 
     // When closing the key, entry should be removed from openFileTable
     // and it should be added to fileTable.
@@ -368,11 +450,11 @@ public class TestObjectStoreWithFSO {
     expectedKeys = new LinkedList<>();
     expectedKeys.add("a/b2/");
     expectedKeys.add("a/b2/d1/");
-    expectedKeys.add("a/b2/d2/");
-    expectedKeys.add("a/b2/d3/");
     expectedKeys.add("a/b2/d1/d11.tx");
+    expectedKeys.add("a/b2/d2/");
     expectedKeys.add("a/b2/d2/d21.tx");
     expectedKeys.add("a/b2/d2/d22.tx");
+    expectedKeys.add("a/b2/d3/");
     expectedKeys.add("a/b2/d3/d31.tx");
     checkKeyList(ozoneKeyIterator, expectedKeys);
 
@@ -402,24 +484,24 @@ public class TestObjectStoreWithFSO {
     LinkedList<String> expectedKeys = new LinkedList<>();
     expectedKeys.add("a/");
     expectedKeys.add("a/b1/");
-    expectedKeys.add("a/b2/");
-    expectedKeys.add("a/b3/");
     expectedKeys.add("a/b1/c1/");
-    expectedKeys.add("a/b1/c2/");
     expectedKeys.add("a/b1/c1/c1.tx");
+    expectedKeys.add("a/b1/c2/");
     expectedKeys.add("a/b1/c2/c2.tx");
+    expectedKeys.add("a/b2/");
     expectedKeys.add("a/b2/d1/");
-    expectedKeys.add("a/b2/d2/");
-    expectedKeys.add("a/b2/d3/");
     expectedKeys.add("a/b2/d1/d11.tx");
+    expectedKeys.add("a/b2/d2/");
     expectedKeys.add("a/b2/d2/d21.tx");
     expectedKeys.add("a/b2/d2/d22.tx");
+    expectedKeys.add("a/b2/d3/");
     expectedKeys.add("a/b2/d3/d31.tx");
+    expectedKeys.add("a/b3/");
     expectedKeys.add("a/b3/e1/");
-    expectedKeys.add("a/b3/e2/");
-    expectedKeys.add("a/b3/e3/");
     expectedKeys.add("a/b3/e1/e11.tx");
+    expectedKeys.add("a/b3/e2/");
     expectedKeys.add("a/b3/e2/e21.tx");
+    expectedKeys.add("a/b3/e3/");
     expectedKeys.add("a/b3/e3/e31.tx");
     checkKeyList(keyItr, expectedKeys);
   }
@@ -649,7 +731,8 @@ public class TestObjectStoreWithFSO {
     Assert.assertEquals(sampleBucketName, bucket.getName());
     Assert.assertEquals(BucketLayout.OBJECT_STORE, bucket.getBucketLayout());
 
-    // Case 3: Bucket layout: Empty and OM Metadata layout: PREFIX
+    // Case 3: Bucket layout: Empty and
+    // OM default bucket layout: FILE_SYSTEM_OPTIMIZED
     builder = BucketArgs.newBuilder();
     sampleBucketName = UUID.randomUUID().toString();
     volume.createBucket(sampleBucketName, builder.build());
@@ -658,9 +741,9 @@ public class TestObjectStoreWithFSO {
     Assert.assertEquals(BucketLayout.FILE_SYSTEM_OPTIMIZED,
         bucket.getBucketLayout());
 
-    // Case 4: Bucket layout: DEFAULT
+    // Case 4: Bucket layout: Empty
     sampleBucketName = UUID.randomUUID().toString();
-    builder.setBucketLayout(BucketLayout.DEFAULT);
+    builder = BucketArgs.newBuilder();
     volume.createBucket(sampleBucketName, builder.build());
     bucket = volume.getBucket(sampleBucketName);
     Assert.assertEquals(sampleBucketName, bucket.getName());
@@ -673,7 +756,7 @@ public class TestObjectStoreWithFSO {
     volume.createBucket(sampleBucketName, builder.build());
     bucket = volume.getBucket(sampleBucketName);
     Assert.assertEquals(sampleBucketName, bucket.getName());
-    Assert.assertNotEquals(BucketLayout.LEGACY, bucket.getBucketLayout());
+    Assert.assertEquals(BucketLayout.LEGACY, bucket.getBucketLayout());
   }
 
   private void assertKeyRenamedEx(OzoneBucket bucket, String keyName)
@@ -689,7 +772,7 @@ public class TestObjectStoreWithFSO {
   private void createTestKey(OzoneBucket bucket, String keyName,
       String keyValue) throws IOException {
     OzoneOutputStream out = bucket.createKey(keyName,
-            keyValue.getBytes(StandardCharsets.UTF_8).length, STAND_ALONE,
+            keyValue.getBytes(StandardCharsets.UTF_8).length, RATIS,
             ONE, new HashMap<>());
     out.write(keyValue.getBytes(StandardCharsets.UTF_8));
     out.close();
@@ -700,17 +783,16 @@ public class TestObjectStoreWithFSO {
   private OmDirectoryInfo getDirInfo(String parentKey) throws Exception {
     OMMetadataManager omMetadataManager =
             cluster.getOzoneManager().getMetadataManager();
-    long bucketId = TestOMRequestUtils.getBucketId(volumeName, bucketName,
-            omMetadataManager);
+    long volumeId = omMetadataManager.getVolumeId(volumeName);
+    long bucketId = omMetadataManager.getBucketId(volumeName, bucketName);
     String[] pathComponents = StringUtils.split(parentKey, '/');
     long parentId = bucketId;
     OmDirectoryInfo dirInfo = null;
     for (int indx = 0; indx < pathComponents.length; indx++) {
       String pathElement = pathComponents[indx];
-      String dbKey = omMetadataManager.getOzonePathKey(parentId,
-              pathElement);
-      dirInfo =
-              omMetadataManager.getDirectoryTable().get(dbKey);
+      String dbKey = omMetadataManager.getOzonePathKey(volumeId,
+              bucketId, parentId, pathElement);
+      dirInfo = omMetadataManager.getDirectoryTable().get(dbKey);
       parentId = dirInfo.getObjectID();
     }
     return dirInfo;
@@ -719,7 +801,12 @@ public class TestObjectStoreWithFSO {
   private void verifyKeyInFileTable(Table<String, OmKeyInfo> fileTable,
       String fileName, long parentID, boolean isEmpty) throws IOException {
 
-    String dbFileKey = parentID + OM_KEY_PREFIX + fileName;
+    final OMMetadataManager omMetadataManager =
+            cluster.getOzoneManager().getMetadataManager();
+    final String dbFileKey = omMetadataManager.getOzonePathKey(
+            omMetadataManager.getVolumeId(volumeName),
+            omMetadataManager.getBucketId(volumeName, bucketName),
+            parentID, fileName);
     OmKeyInfo omKeyInfo = fileTable.get(dbFileKey);
     if (isEmpty) {
       Assert.assertNull("Table is not empty!", omKeyInfo);
@@ -737,8 +824,12 @@ public class TestObjectStoreWithFSO {
   private void verifyKeyInOpenFileTable(Table<String, OmKeyInfo> openFileTable,
       long clientID, String fileName, long parentID, boolean isEmpty)
           throws IOException, TimeoutException, InterruptedException {
-    String dbOpenFileKey =
-            parentID + OM_KEY_PREFIX + fileName + OM_KEY_PREFIX + clientID;
+    final OMMetadataManager omMetadataManager =
+            cluster.getOzoneManager().getMetadataManager();
+    final String dbOpenFileKey = omMetadataManager.getOpenFileName(
+            omMetadataManager.getVolumeId(volumeName),
+            omMetadataManager.getBucketId(volumeName, bucketName),
+            parentID, fileName, clientID);
 
     if (isEmpty) {
       // wait for DB updates
