@@ -31,6 +31,7 @@ import org.apache.hadoop.ozone.om.helpers.OmKeyInfo;
 import org.apache.hadoop.ozone.om.helpers.OmKeyLocationInfo;
 import org.apache.hadoop.ozone.om.helpers.OmKeyLocationInfoGroup;
 import org.apache.hadoop.ozone.om.helpers.OpenKeySession;
+import org.apache.hadoop.ozone.om.protocol.OzoneManagerProtocol;
 
 import org.apache.commons.lang3.RandomStringUtils;
 import org.junit.AfterClass;
@@ -57,6 +58,7 @@ public class TestOmBlockVersioning {
   private static MiniOzoneCluster cluster = null;
   private static OzoneConfiguration conf;
   private static OzoneManager ozoneManager;
+  private static OzoneManagerProtocol writeClient;
 
   @Rule
   public ExpectedException exception = ExpectedException.none();
@@ -74,6 +76,8 @@ public class TestOmBlockVersioning {
     cluster = MiniOzoneCluster.newBuilder(conf).build();
     cluster.waitForClusterToBeReady();
     ozoneManager = cluster.getOzoneManager();
+    writeClient = cluster.getRpcClient().getObjectStore()
+        .getClientProxy().getOzoneManagerClient();
   }
 
   /**
@@ -92,24 +96,26 @@ public class TestOmBlockVersioning {
     String bucketName = "bucket" + RandomStringUtils.randomNumeric(5);
     String keyName = "key" + RandomStringUtils.randomNumeric(5);
 
-    TestDataUtil.createVolumeAndBucket(cluster, volumeName, bucketName);
+    OzoneBucket bucket =
+        TestDataUtil.createVolumeAndBucket(cluster, volumeName, bucketName);
+    // Versioning isn't supported currently, but just preserving old behaviour
+    bucket.setVersioning(true);
 
     OmKeyArgs keyArgs = new OmKeyArgs.Builder()
         .setVolumeName(volumeName)
         .setBucketName(bucketName)
         .setKeyName(keyName)
         .setDataSize(1000)
-        .setRefreshPipeline(true)
         .setAcls(new ArrayList<>())
-        .setReplicationConfig(new StandaloneReplicationConfig(ONE))
+        .setReplicationConfig(StandaloneReplicationConfig.getInstance(ONE))
         .build();
 
     // 1st update, version 0
-    OpenKeySession openKey = ozoneManager.openKey(keyArgs);
+    OpenKeySession openKey = writeClient.openKey(keyArgs);
     // explicitly set the keyLocation list before committing the key.
     keyArgs.setLocationInfoList(openKey.getKeyInfo().getLatestVersionLocations()
         .getBlocksLatestVersionOnly());
-    ozoneManager.commitKey(keyArgs, openKey.getId());
+    writeClient.commitKey(keyArgs, openKey.getId());
 
     OmKeyInfo keyInfo = ozoneManager.lookupKey(keyArgs);
     OmKeyLocationInfoGroup highestVersion =
@@ -118,13 +124,13 @@ public class TestOmBlockVersioning {
     assertEquals(1, highestVersion.getLocationList().size());
 
     // 2nd update, version 1
-    openKey = ozoneManager.openKey(keyArgs);
+    openKey = writeClient.openKey(keyArgs);
     //OmKeyLocationInfo locationInfo =
-    //    ozoneManager.allocateBlock(keyArgs, openKey.getId());
+    //    writeClient.allocateBlock(keyArgs, openKey.getId());
     // explicitly set the keyLocation list before committing the key.
     keyArgs.setLocationInfoList(openKey.getKeyInfo().getLatestVersionLocations()
         .getBlocksLatestVersionOnly());
-    ozoneManager.commitKey(keyArgs, openKey.getId());
+    writeClient.commitKey(keyArgs, openKey.getId());
 
     keyInfo = ozoneManager.lookupKey(keyArgs);
     highestVersion = checkVersions(keyInfo.getKeyLocationVersions());
@@ -132,11 +138,11 @@ public class TestOmBlockVersioning {
     assertEquals(1, highestVersion.getLocationList().size());
 
     // 3rd update, version 2
-    openKey = ozoneManager.openKey(keyArgs);
+    openKey = writeClient.openKey(keyArgs);
 
     // this block will be appended to the latest version of version 2.
     OmKeyLocationInfo locationInfo =
-        ozoneManager.allocateBlock(keyArgs, openKey.getId(),
+        writeClient.allocateBlock(keyArgs, openKey.getId(),
             new ExcludeList());
     List<OmKeyLocationInfo> locationInfoList =
         openKey.getKeyInfo().getLatestVersionLocations()
@@ -144,7 +150,7 @@ public class TestOmBlockVersioning {
     Assert.assertTrue(locationInfoList.size() == 1);
     locationInfoList.add(locationInfo);
     keyArgs.setLocationInfoList(locationInfoList);
-    ozoneManager.commitKey(keyArgs, openKey.getId());
+    writeClient.commitKey(keyArgs, openKey.getId());
 
     keyInfo = ozoneManager.lookupKey(keyArgs);
     highestVersion = checkVersions(keyInfo.getKeyLocationVersions());
@@ -179,7 +185,6 @@ public class TestOmBlockVersioning {
         .setBucketName(bucketName)
         .setKeyName(keyName)
         .setDataSize(1000)
-        .setRefreshPipeline(true)
         .build();
 
     String dataString = RandomStringUtils.randomAlphabetic(100);
@@ -191,14 +196,13 @@ public class TestOmBlockVersioning {
     assertEquals(1,
         keyInfo.getLatestVersionLocations().getLocationList().size());
 
-    // this write will create 2nd version, 2nd version will contain block from
-    // version 1, and add a new block
+    // When bucket versioning is disabled, overwriting a key doesn't increment
+    // its version count. Rather it always resets the version to 0
     TestDataUtil.createKey(bucket, keyName, dataString);
 
-
     keyInfo = ozoneManager.lookupKey(omKeyArgs);
     assertEquals(dataString, TestDataUtil.getKey(bucket, keyName));
-    assertEquals(1, keyInfo.getLatestVersionLocations().getVersion());
+    assertEquals(0, keyInfo.getLatestVersionLocations().getVersion());
     assertEquals(1,
         keyInfo.getLatestVersionLocations().getLocationList().size());
 
@@ -207,8 +211,8 @@ public class TestOmBlockVersioning {
 
     keyInfo = ozoneManager.lookupKey(omKeyArgs);
     assertEquals(dataString, TestDataUtil.getKey(bucket, keyName));
-    assertEquals(2, keyInfo.getLatestVersionLocations().getVersion());
+    assertEquals(0, keyInfo.getLatestVersionLocations().getVersion());
     assertEquals(1,
         keyInfo.getLatestVersionLocations().getLocationList().size());
   }
-}
\ No newline at end of file
+}
