@@ -41,17 +41,20 @@ import org.apache.hadoop.ozone.client.OzoneClientFactory;
 import org.apache.hadoop.ozone.client.OzoneVolume;
 import org.apache.hadoop.ozone.client.io.OzoneOutputStream;
 import org.apache.hadoop.ozone.om.OMConfigKeys;
-import org.junit.After;
+import org.apache.ozone.test.tag.Flaky;
+import org.junit.jupiter.api.AfterEach;
 import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Ignore;
-import org.junit.Test;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.Timeout;
 
 import java.io.IOException;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.UUID;
+import java.util.concurrent.TimeUnit;
+import java.util.concurrent.TimeoutException;
 import java.util.stream.Collectors;
 
 import static java.nio.charset.StandardCharsets.UTF_8;
@@ -64,7 +67,7 @@ import static org.apache.hadoop.test.MetricsAsserts.getMetrics;
 /**
  * Test cases to verify the metrics exposed by SCMPipelineManager.
  */
-@Ignore("HDDS-2961")
+@Flaky("HDDS-2961")
 public class TestSCMContainerPlacementPolicyMetrics {
 
   private MiniOzoneCluster cluster;
@@ -72,7 +75,7 @@ public class TestSCMContainerPlacementPolicyMetrics {
   private OzoneClient ozClient = null;
   private ObjectStore store = null;
 
-  @Before
+  @BeforeEach
   public void setup() throws Exception {
     OzoneConfiguration conf = new OzoneConfiguration();
     conf.set(ScmConfigKeys.OZONE_SCM_CONTAINER_PLACEMENT_IMPL_KEY,
@@ -98,8 +101,8 @@ public class TestSCMContainerPlacementPolicyMetrics {
   /**
    * Verifies container placement metric.
    */
-  @Test(timeout = 60000)
-  public void test() throws IOException {
+  @Test @Timeout(unit = TimeUnit.MILLISECONDS, value = 60000)
+  public void test() throws IOException, TimeoutException {
     String volumeName = UUID.randomUUID().toString();
     String bucketName = UUID.randomUUID().toString();
 
@@ -155,7 +158,7 @@ public class TestSCMContainerPlacementPolicyMetrics {
     Assert.assertTrue(compromiseCount == 0);
   }
 
-  @After
+  @AfterEach
   public void teardown() {
     cluster.shutdown();
   }
