@@ -25,10 +25,11 @@ import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.hdds.scm.cli.ContainerOperationClient;
 import org.apache.ozone.test.GenericTestUtils;
-import org.junit.After;
-import org.junit.Before;
-import org.junit.Ignore;
-import org.junit.Test;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Disabled;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.Timeout;
 
 import java.util.List;
 import java.util.concurrent.TimeUnit;
@@ -62,19 +63,19 @@ import static org.apache.hadoop.hdds.scm.ScmConfigKeys
     .OZONE_SCM_HEARTBEAT_PROCESS_INTERVAL;
 import static org.apache.hadoop.hdds.scm.ScmConfigKeys
     .OZONE_SCM_STALENODE_INTERVAL;
-import static org.junit.Assert.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertEquals;
 
 /**
  * Test Query Node Operation.
  */
-@Ignore
+@Disabled
 public class TestQueryNode {
   private static int numOfDatanodes = 5;
   private MiniOzoneCluster cluster;
 
   private ContainerOperationClient scmClient;
 
-  @Before
+  @BeforeEach
   public void setUp() throws Exception {
     OzoneConfiguration conf = new OzoneConfiguration();
     final int interval = 100;
@@ -92,13 +93,13 @@ public class TestQueryNode {
 
     cluster = MiniOzoneCluster.newBuilder(conf)
         .setNumDatanodes(numOfDatanodes)
-        .setTotalPipelineNumLimit(numOfDatanodes + numOfDatanodes/2)
+        .setTotalPipelineNumLimit(numOfDatanodes + numOfDatanodes / 2)
         .build();
     cluster.waitForClusterToBeReady();
     scmClient = new ContainerOperationClient(conf);
   }
 
-  @After
+  @AfterEach
   public void tearDown() throws Exception {
     if (cluster != null) {
       cluster.shutdown();
@@ -109,11 +110,11 @@ public class TestQueryNode {
   public void testHealthyNodesCount() throws Exception {
     List<HddsProtos.Node> nodes = scmClient.queryNode(null, HEALTHY,
         HddsProtos.QueryScope.CLUSTER, "");
-    assertEquals("Expected  live nodes", numOfDatanodes,
-        nodes.size());
+    assertEquals(numOfDatanodes, nodes.size(), "Expected live nodes");
   }
 
-  @Test(timeout = 10 * 1000L)
+  @Test
+  @Timeout(10)
   public void testStaleNodesCount() throws Exception {
     cluster.shutdownHddsDatanode(0);
     cluster.shutdownHddsDatanode(1);
@@ -124,7 +125,7 @@ public class TestQueryNode {
 
     int nodeCount = scmClient.queryNode(null, STALE,
         HddsProtos.QueryScope.CLUSTER, "").size();
-    assertEquals("Mismatch of expected nodes count", 2, nodeCount);
+    assertEquals(2, nodeCount, "Mismatch of expected nodes count");
 
     GenericTestUtils.waitFor(() ->
             cluster.getStorageContainerManager().getNodeCount(DEAD) == 2,
@@ -133,12 +134,12 @@ public class TestQueryNode {
     // Assert that we don't find any stale nodes.
     nodeCount = scmClient.queryNode(null, STALE,
         HddsProtos.QueryScope.CLUSTER, "").size();
-    assertEquals("Mismatch of expected nodes count", 0, nodeCount);
+    assertEquals(0, nodeCount, "Mismatch of expected nodes count");
 
     // Assert that we find the expected number of dead nodes.
     nodeCount = scmClient.queryNode(null, DEAD,
         HddsProtos.QueryScope.CLUSTER, "").size();
-    assertEquals("Mismatch of expected nodes count", 2, nodeCount);
+    assertEquals(2, nodeCount, "Mismatch of expected nodes count");
   }
 
   @Test
