@@ -17,6 +17,7 @@
 package org.apache.hadoop.ozone.om;
 
 import java.io.IOException;
+import java.time.Duration;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
@@ -29,16 +30,20 @@ import org.apache.hadoop.hdds.utils.db.cache.CacheKey;
 import org.apache.hadoop.hdds.utils.db.cache.CacheValue;
 import org.apache.hadoop.ozone.common.BlockGroup;
 import org.apache.hadoop.ozone.om.helpers.OmBucketInfo;
+import org.apache.hadoop.ozone.om.helpers.OmDBAccessIdInfo;
+import org.apache.hadoop.ozone.om.helpers.OmDBUserPrincipalInfo;
 import org.apache.hadoop.ozone.om.helpers.OmDirectoryInfo;
 import org.apache.hadoop.ozone.om.helpers.OmKeyInfo;
 import org.apache.hadoop.ozone.om.helpers.OmMultipartKeyInfo;
 import org.apache.hadoop.ozone.om.helpers.OmPrefixInfo;
+import org.apache.hadoop.ozone.om.helpers.OmDBTenantState;
 import org.apache.hadoop.ozone.om.helpers.OmVolumeArgs;
 import org.apache.hadoop.ozone.om.helpers.RepeatedOmKeyInfo;
 import org.apache.hadoop.ozone.om.helpers.S3SecretValue;
 import org.apache.hadoop.ozone.om.helpers.BucketLayout;
 import org.apache.hadoop.ozone.om.lock.OzoneManagerLock;
 import org.apache.hadoop.hdds.utils.TransactionInfo;
+import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.OpenKeyBucket;
 import org.apache.hadoop.ozone.storage.proto.
     OzoneManagerStorageProtos.PersistedUserVolumeInfo;
 import org.apache.hadoop.ozone.security.OzoneTokenIdentifier;
@@ -256,14 +261,17 @@ public interface OMMetadataManager extends DBStoreHAManager {
   List<BlockGroup> getPendingDeletionKeys(int count) throws IOException;
 
   /**
-   * Returns the names of up to {@code count} open keys that are older than
-   * the configured expiration age.
+   * Returns the names of up to {@code count} open keys whose age is
+   * greater than or equal to {@code expireThreshold}.
    *
    * @param count The maximum number of open keys to return.
-   * @return a list of {@link String} representing names of open expired keys.
+   * @param expireThreshold The threshold of open key expire age.
+   * @param bucketLayout The type of open keys to get (e.g. DEFAULT or FSO).
+   * @return a {@link List} of {@link OpenKeyBucket}, the expired open keys.
    * @throws IOException
    */
-  List<String> getExpiredOpenKeys(int count) throws IOException;
+  List<OpenKeyBucket> getExpiredOpenKeys(Duration expireThreshold, int count,
+      BucketLayout bucketLayout) throws IOException;
 
   /**
    * Returns the user Table.
@@ -291,7 +299,8 @@ public interface OMMetadataManager extends DBStoreHAManager {
    *
    * @return KeyTable.
    */
-  Table<String, OmKeyInfo> getKeyTable();
+
+  Table<String, OmKeyInfo> getKeyTable(BucketLayout bucketLayout);
 
   /**
    * Returns the FileTable.
@@ -355,6 +364,12 @@ public interface OMMetadataManager extends DBStoreHAManager {
 
   Table<String, TransactionInfo> getTransactionInfoTable();
 
+  Table<String, OmDBAccessIdInfo> getTenantAccessIdTable();
+
+  Table<String, OmDBUserPrincipalInfo> getPrincipalToAccessIdsTable();
+
+  Table<String, OmDBTenantState> getTenantStateTable();
+
   /**
    * Gets the OM Meta table.
    * @return meta table reference.
@@ -418,38 +433,48 @@ public interface OMMetadataManager extends DBStoreHAManager {
       getBucketIterator();
 
   TableIterator<String, ? extends Table.KeyValue<String, OmKeyInfo>>
-      getKeyIterator();
+      getKeyIterator() throws IOException;
 
   /**
    * Given parent object id and path component name, return the corresponding
    * DB 'prefixKey' key.
    *
+   * @param volumeId - ID of the volume
+   * @param bucketId - ID of the bucket
    * @param parentObjectId - parent object Id
    * @param pathComponentName   - path component name
    * @return DB directory key as String.
    */
-  String getOzonePathKey(long parentObjectId, String pathComponentName);
+  String getOzonePathKey(long volumeId, long bucketId,
+                         long parentObjectId, String pathComponentName);
 
   /**
    * Returns DB key name of an open file in OM metadata store. Should be
    * #open# prefix followed by actual leaf node name.
    *
+   * @param volumeId       - ID of the volume
+   * @param bucketId       - ID of the bucket
    * @param parentObjectId - parent object Id
    * @param fileName       - file name
    * @param id             - client id for this open request
    * @return DB directory key as String.
    */
-  String getOpenFileName(long parentObjectId, String fileName, long id);
+  String getOpenFileName(long volumeId, long bucketId,
+                         long parentObjectId, String fileName, long id);
 
   /**
    * Returns the DB key name of a multipart upload key in OM metadata store.
    *
+   * @param volumeId       - ID of the volume
+   * @param bucketId       - ID of the bucket
    * @param parentObjectId - parent object Id
    * @param fileName       - file name
    * @param uploadId       - the upload id for this key
    * @return bytes of DB key.
    */
-  String getMultipartKey(long parentObjectId, String fileName, String uploadId);
+  String getMultipartKey(long volumeId, long bucketId,
+                         long parentObjectId, String fileName,
+                         String uploadId);
 
   /**
    * Get Deleted Directory Table.
@@ -458,4 +483,23 @@ public interface OMMetadataManager extends DBStoreHAManager {
    */
   Table<String, OmKeyInfo> getDeletedDirTable();
 
+  /**
+   * Get the ID of given volume.
+   *
+   * @param volume volume name
+   * @return ID of the volume
+   * @throws IOException
+   */
+  long getVolumeId(String volume) throws IOException;
+
+  /**
+   * Get the ID of given bucket.
+   *
+   * @param volume volume name
+   * @param bucket bucket name
+   * @return ID of the bucket
+   * @throws IOException
+   */
+  long getBucketId(String volume, String bucket) throws IOException;
+
 }
