@@ -18,10 +18,11 @@
 
 package org.apache.hadoop.ozone.om.codec;
 
+import com.google.common.util.concurrent.ThreadFactoryBuilder;
 import org.apache.hadoop.hdds.client.BlockID;
 import org.apache.hadoop.hdds.client.RatisReplicationConfig;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
-import org.apache.hadoop.hdds.scm.TestUtils;
+import org.apache.hadoop.hdds.scm.HddsTestUtils;
 import org.apache.hadoop.hdds.scm.pipeline.Pipeline;
 import org.apache.hadoop.ozone.om.helpers.OmKeyInfo;
 import org.apache.hadoop.ozone.om.helpers.OmKeyLocationInfo;
@@ -34,7 +35,10 @@ import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
+import java.util.concurrent.ThreadFactory;
+import java.util.concurrent.atomic.AtomicBoolean;
 
+import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertNull;
 import static org.junit.Assert.fail;
@@ -51,7 +55,7 @@ public class TestRepeatedOmKeyInfoCodec {
 
   private OmKeyInfo getKeyInfo(int chunkNum) {
     List<OmKeyLocationInfo> omKeyLocationInfoList = new ArrayList<>();
-    Pipeline pipeline = TestUtils.getRandomPipeline();
+    Pipeline pipeline = HddsTestUtils.getRandomPipeline();
     for (int i = 0; i < chunkNum; i++) {
       BlockID blockID = new BlockID(i, i);
       OmKeyLocationInfo keyLocationInfo = new OmKeyLocationInfo.Builder()
@@ -66,7 +70,8 @@ public class TestRepeatedOmKeyInfoCodec {
         .setCreationTime(Time.now())
         .setModificationTime(Time.now())
         .setReplicationConfig(
-                new RatisReplicationConfig(HddsProtos.ReplicationFactor.THREE))
+                RatisReplicationConfig
+                    .getInstance(HddsProtos.ReplicationFactor.THREE))
         .setVolumeName(VOLUME)
         .setBucketName(BUCKET)
         .setKeyName(KEYNAME)
@@ -79,7 +84,8 @@ public class TestRepeatedOmKeyInfoCodec {
   }
 
   @Test
-  public void test() {
+  public void test() throws InterruptedException {
+    threadSafety();
     testWithoutPipeline(1);
     testWithoutPipeline(2);
     testCompatibility(1);
@@ -120,4 +126,33 @@ public class TestRepeatedOmKeyInfoCodec {
       fail("Should success");
     }
   }
+
+  public void threadSafety() throws InterruptedException {
+    final OmKeyInfo key = getKeyInfo(1);
+    final RepeatedOmKeyInfo subject = new RepeatedOmKeyInfo(key);
+    final RepeatedOmKeyInfoCodec codec = new RepeatedOmKeyInfoCodec(true);
+    final AtomicBoolean failed = new AtomicBoolean();
+    ThreadFactory threadFactory = new ThreadFactoryBuilder().setDaemon(true)
+        .build();
+    threadFactory.newThread(() -> {
+      for (int i = 0; i < 1000000; i++) {
+        try {
+          codec.toPersistedFormat(subject.copyObject());
+        } catch (Exception e) {
+          e.printStackTrace();
+          failed.set(true);
+        }
+      }
+    }).start();
+    threadFactory.newThread(() -> {
+      for (int i = 0; i < 10000; i++) {
+        subject.addOmKeyInfo(key);
+      }
+    }).start();
+    final long start = Time.monotonicNow();
+    while (!failed.get() && (Time.monotonicNow() - start < 5000)) {
+      Thread.sleep(100);
+    }
+    assertFalse(failed.get());
+  }
 }
