@@ -17,24 +17,21 @@
 package org.apache.hadoop.ozone.om;
 
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
-import org.apache.hadoop.hdds.scm.container.common.helpers.ExcludeList;
+import org.apache.hadoop.hdds.utils.db.Table;
 import org.apache.hadoop.ozone.common.BlockGroup;
 import org.apache.hadoop.ozone.om.exceptions.OMException;
+import org.apache.hadoop.ozone.om.helpers.BucketLayout;
 import org.apache.hadoop.ozone.om.helpers.OmKeyArgs;
 import org.apache.hadoop.ozone.om.helpers.OmKeyInfo;
-import org.apache.hadoop.ozone.om.helpers.OmKeyLocationInfo;
-import org.apache.hadoop.ozone.om.helpers.OmMultipartCommitUploadPartInfo;
-import org.apache.hadoop.ozone.om.helpers.OmMultipartInfo;
-import org.apache.hadoop.ozone.om.helpers.OmMultipartUploadCompleteInfo;
-import org.apache.hadoop.ozone.om.helpers.OmMultipartUploadCompleteList;
 import org.apache.hadoop.ozone.om.helpers.OmMultipartUploadList;
 import org.apache.hadoop.ozone.om.helpers.OmMultipartUploadListParts;
-import org.apache.hadoop.ozone.om.helpers.OpenKeySession;
 import org.apache.hadoop.ozone.om.fs.OzoneManagerFS;
 import org.apache.hadoop.hdds.utils.BackgroundService;
 import org.apache.hadoop.ozone.om.helpers.RepeatedOmKeyInfo;
+import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.OpenKeyBucket;
 
 import java.io.IOException;
+import java.time.Duration;
 import java.util.List;
 
 /**
@@ -55,44 +52,6 @@ public interface KeyManager extends OzoneManagerFS, IOzoneAcl {
    */
   void stop() throws IOException;
 
-  /**
-   * After calling commit, the key will be made visible. There can be multiple
-   * open key writes in parallel (identified by client id). The most recently
-   * committed one will be the one visible.
-   *
-   * @param args the key to commit.
-   * @param clientID the client that is committing.
-   * @throws IOException
-   */
-  void commitKey(OmKeyArgs args, long clientID) throws IOException;
-
-  /**
-   * A client calls this on an open key, to request to allocate a new block,
-   * and appended to the tail of current block list of the open client.
-   *
-   * @param args the key to append
-   * @param clientID the client requesting block.
-   * @param excludeList List of datanodes/containers to exclude during block
-   *                    allocation.
-   * @return the reference to the new block.
-   * @throws IOException
-   */
-  OmKeyLocationInfo allocateBlock(OmKeyArgs args, long clientID,
-      ExcludeList excludeList) throws IOException;
-
-  /**
-   * Given the args of a key to put, write an open key entry to meta data.
-   *
-   * In case that the container creation or key write failed on
-   * DistributedStorageHandler, this key's metadata will still stay in OM.
-   * TODO garbage collect the open keys that never get closed
-   *
-   * @param args the args of the key provided by client.
-   * @return a OpenKeySession instance client uses to talk to container.
-   * @throws IOException
-   */
-  OpenKeySession openKey(OmKeyArgs args) throws IOException;
-
   /**
    * Look up an existing key. Return the info of the key to client side, which
    * DistributedStorageHandler will use to access the data on datanode.
@@ -106,25 +65,16 @@ public interface KeyManager extends OzoneManagerFS, IOzoneAcl {
   OmKeyInfo lookupKey(OmKeyArgs args, String clientAddress) throws IOException;
 
   /**
-   * Renames an existing key within a bucket.
-   *
+   * Return info of an existing key to client side to access to data on
+   * datanodes.
    * @param args the args of the key provided by client.
-   * @param toKeyName New name to be used for the key
-   * @throws IOException if specified key doesn't exist or
-   * some other I/O errors while renaming the key.
+   * @param clientAddress a hint to key manager, order the datanode in returned
+   *                      pipeline by distance between client and datanode.
+   * @return a OmKeyInfo instance client uses to talk to container.
+   * @throws IOException
    */
-  void renameKey(OmKeyArgs args, String toKeyName) throws IOException;
+  OmKeyInfo getKeyInfo(OmKeyArgs args, String clientAddress) throws IOException;
 
-  /**
-   * Deletes an object by an object key. The key will be immediately removed
-   * from OM namespace and become invisible to clients. The object data
-   * will be removed in async manner that might retain for some time.
-   *
-   * @param args the args of the key provided by client.
-   * @throws IOException if specified key doesn't exist or
-   * some other I/O errors while deleting an object.
-   */
-  void deleteKey(OmKeyArgs args) throws IOException;
 
   /**
    * Returns a list of keys represented by {@link OmKeyInfo}
@@ -182,25 +132,17 @@ public interface KeyManager extends OzoneManagerFS, IOzoneAcl {
   List<BlockGroup> getPendingDeletionKeys(int count) throws IOException;
 
   /**
-   * Returns the names of up to {@code count} open keys that are older than
-   * the configured expiration age.
+   * Returns the names of up to {@code count} open keys whose age is
+   * greater than or equal to {@code expireThreshold}.
    *
    * @param count The maximum number of expired open keys to return.
-   * @return a list of {@link String} representing the names of expired
-   * open keys.
+   * @param expireThreshold The threshold of open key expiration age.
+   * @param bucketLayout The type of open keys to get (e.g. DEFAULT or FSO).
+   * @return a {@link List} of {@link OpenKeyBucket}, the expired open keys.
    * @throws IOException
    */
-  List<String> getExpiredOpenKeys(int count) throws IOException;
-
-  /**
-   * Deletes a expired open key by its name. Called when a hanging key has been
-   * lingering for too long. Once called, the open key entries gets removed
-   * from OM mdata data.
-   *
-   * @param objectKeyName object key name with #open# prefix.
-   * @throws IOException if specified key doesn't exist or other I/O errors.
-   */
-  void deleteExpiredOpenKey(String objectKeyName) throws IOException;
+  List<OpenKeyBucket> getExpiredOpenKeys(Duration expireThreshold, int count,
+      BucketLayout bucketLayout) throws IOException;
 
   /**
    * Returns the metadataManager.
@@ -215,42 +157,6 @@ public interface KeyManager extends OzoneManagerFS, IOzoneAcl {
   BackgroundService getDeletingService();
 
 
-  /**
-   * Initiate multipart upload for the specified key.
-   * @param keyArgs
-   * @return MultipartInfo
-   * @throws IOException
-   */
-  OmMultipartInfo initiateMultipartUpload(OmKeyArgs keyArgs) throws IOException;
-
-  /**
-   * Commit Multipart upload part file.
-   * @param omKeyArgs
-   * @param clientID
-   * @return OmMultipartCommitUploadPartInfo
-   * @throws IOException
-   */
-
-  OmMultipartCommitUploadPartInfo commitMultipartUploadPart(
-      OmKeyArgs omKeyArgs, long clientID) throws IOException;
-
-  /**
-   * Complete Multipart upload Request.
-   * @param omKeyArgs
-   * @param multipartUploadList
-   * @return OmMultipartUploadCompleteInfo
-   * @throws IOException
-   */
-  OmMultipartUploadCompleteInfo completeMultipartUpload(OmKeyArgs omKeyArgs,
-      OmMultipartUploadCompleteList multipartUploadList) throws IOException;
-
-  /**
-   * Abort multipart upload request.
-   * @param omKeyArgs
-   * @throws IOException
-   */
-  void abortMultipartUpload(OmKeyArgs omKeyArgs) throws IOException;
-
   OmMultipartUploadList listMultipartUploads(String volumeName,
       String bucketName, String prefix) throws OMException;
 
@@ -300,7 +206,7 @@ public interface KeyManager extends OzoneManagerFS, IOzoneAcl {
    * @return OmKeyInfo
    * @throws IOException
    */
-  OmKeyInfo getPendingDeletionDir() throws IOException;
+  Table.KeyValue<String, OmKeyInfo> getPendingDeletionDir() throws IOException;
 
   /**
    * Returns all sub directories under the given parent directory.
@@ -310,8 +216,8 @@ public interface KeyManager extends OzoneManagerFS, IOzoneAcl {
    * @return list of dirs
    * @throws IOException
    */
-  List<OmKeyInfo> getPendingDeletionSubDirs(OmKeyInfo parentInfo,
-      long numEntries) throws IOException;
+  List<OmKeyInfo> getPendingDeletionSubDirs(long volumeId, long bucketId,
+      OmKeyInfo parentInfo, long numEntries) throws IOException;
 
   /**
    * Returns all sub files under the given parent directory.
@@ -321,12 +227,19 @@ public interface KeyManager extends OzoneManagerFS, IOzoneAcl {
    * @return list of files
    * @throws IOException
    */
-  List<OmKeyInfo> getPendingDeletionSubFiles(OmKeyInfo parentInfo,
-      long numEntries) throws IOException;
+  List<OmKeyInfo> getPendingDeletionSubFiles(long volumeId,
+      long bucketId, OmKeyInfo parentInfo, long numEntries)
+          throws IOException;
 
   /**
    * Returns the instance of Directory Deleting Service.
    * @return Background service.
    */
   BackgroundService getDirDeletingService();
+
+  /**
+   * Returns the instance of Open Key Cleanup Service.
+   * @return Background service.
+   */
+  BackgroundService getOpenKeyCleanupService();
 }
