@@ -28,4 +28,11 @@ import org.apache.hadoop.hdds.server.ServiceRuntimeInfo;
 public interface OMMXBean extends ServiceRuntimeInfo {
 
   String getRpcPort();
+
+  String getRatisRoles();
+
+  String getRatisLogDirectory();
+
+  String getRocksDbDirectory();
+
 }
