@@ -31,7 +31,7 @@ import org.apache.hadoop.metrics2.lib.MutableCounterLong;
  * This class is for maintaining Ozone Manager statistics.
  */
 @InterfaceAudience.Private
-@Metrics(about="Ozone Manager Metrics", context="dfs")
+@Metrics(about = "Ozone Manager Metrics", context = "dfs")
 public class OMMetrics {
   private static final String SOURCE_NAME =
       OMMetrics.class.getSimpleName();
@@ -48,10 +48,12 @@ public class OMMetrics {
   private @Metric MutableCounterLong numVolumeInfos;
   private @Metric MutableCounterLong numVolumeCheckAccesses;
   private @Metric MutableCounterLong numBucketCreates;
+  private @Metric MutableCounterLong numFSOBucketCreates;
   private @Metric MutableCounterLong numVolumeDeletes;
   private @Metric MutableCounterLong numBucketInfos;
   private @Metric MutableCounterLong numBucketUpdates;
   private @Metric MutableCounterLong numBucketDeletes;
+  private @Metric MutableCounterLong numFSOBucketDeletes;
   private @Metric MutableCounterLong numKeyAllocate;
   private @Metric MutableCounterLong numKeyLookup;
   private @Metric MutableCounterLong numKeyRenames;
@@ -81,6 +83,7 @@ public class OMMetrics {
   private @Metric MutableCounterLong numSetAcl;
   private @Metric MutableCounterLong numGetAcl;
   private @Metric MutableCounterLong numRemoveAcl;
+  private @Metric MutableCounterLong numGetKeyInfo;
 
   // Failure Metrics
   private @Metric MutableCounterLong numVolumeCreateFails;
@@ -114,11 +117,44 @@ public class OMMetrics {
   private @Metric MutableCounterLong numListMultipartUploadPartFails;
   private @Metric MutableCounterLong numOpenKeyDeleteRequestFails;
 
+  // Number of tenant operations attempted
+  private @Metric MutableCounterLong numTenantOps;
+  // Metrics for a total number of tenants
+  private @Metric MutableCounterLong numTenants;
+  // Metrics for tenant create operation
+  private @Metric MutableCounterLong numTenantCreates;
+  private @Metric MutableCounterLong numTenantCreateFails;
+  // Metrics for tenant delete operation
+  private @Metric MutableCounterLong numTenantDeletes;
+  private @Metric MutableCounterLong numTenantDeleteFails;
+  // Metrics for tenant assign user operation
+  private @Metric MutableCounterLong numTenantAssignUsers;
+  private @Metric MutableCounterLong numTenantAssignUserFails;
+  // Metrics for tenant revoke user operation
+  private @Metric MutableCounterLong numTenantRevokeUsers;
+  private @Metric MutableCounterLong numTenantRevokeUserFails;
+  // Metrics for tenant assign admin operation
+  private @Metric MutableCounterLong numTenantAssignAdmins;
+  private @Metric MutableCounterLong numTenantAssignAdminFails;
+  // Metrics for tenant revoke admin operation
+  private @Metric MutableCounterLong numTenantRevokeAdmins;
+  private @Metric MutableCounterLong numTenantRevokeAdminFails;
+  // Metric for tenant list operation
+  private @Metric MutableCounterLong numTenantLists;
+  // Metric for tenant get user info
+  private @Metric MutableCounterLong numTenantGetUserInfos;
+  // Metric for list users in tenant operation
+  private @Metric MutableCounterLong numTenantTenantUserLists;
+
   private @Metric MutableCounterLong numGetFileStatusFails;
   private @Metric MutableCounterLong numCreateDirectoryFails;
   private @Metric MutableCounterLong numCreateFileFails;
   private @Metric MutableCounterLong numLookupFileFails;
   private @Metric MutableCounterLong numListStatusFails;
+  private @Metric MutableCounterLong getNumGetKeyInfoFails;
+
+  // Metrics for total amount of data written
+  private @Metric MutableCounterLong totalDataCommitted;
 
   // Metrics for total number of volumes, buckets and keys
 
@@ -157,6 +193,10 @@ public class OMMetrics {
   private @Metric MutableCounterLong numTrashAtomicDirRenames;
   private @Metric MutableCounterLong numTrashAtomicDirDeletes;
 
+  //FSO Metrics
+  private @Metric MutableCounterLong numDirs;
+  private @Metric MutableCounterLong numFiles;
+
   private final DBCheckpointMetrics dbCheckpointMetrics;
 
   public OMMetrics() {
@@ -193,6 +233,9 @@ public class OMMetrics {
     numBucketS3DeleteFails.incr();
   }
 
+  public void incDataCommittedBytes(long bytesWritten) {
+    totalDataCommitted.incr(bytesWritten);
+  }
 
   public void incNumS3Buckets() {
     numS3Buckets.incr();
@@ -242,7 +285,17 @@ public class OMMetrics {
 
   public void setNumKeys(long val) {
     long oldVal = this.numKeys.value();
-    this.numKeys.incr(val- oldVal);
+    this.numKeys.incr(val - oldVal);
+  }
+
+  public void setNumDirs(long val) {
+    long oldVal = this.numDirs.value();
+    this.numDirs.incr(val - oldVal);
+  }
+
+  public void setNumFiles(long val) {
+    long oldVal = this.numDirs.value();
+    this.numDirs.incr(val - oldVal);
   }
 
   public void decNumKeys(long val) {
@@ -292,6 +345,10 @@ public class OMMetrics {
     numBucketCreates.incr();
   }
 
+  public void incNumFSOBucketCreates() {
+    numFSOBucketCreates.incr();
+  }
+
   public void incNumBucketInfos() {
     numBucketOps.incr();
     numBucketInfos.incr();
@@ -307,6 +364,10 @@ public class OMMetrics {
     numBucketDeletes.incr();
   }
 
+  public void incNumFSOBucketDeletes() {
+    numFSOBucketDeletes.incr();
+  }
+
   public void incNumBucketLists() {
     numBucketOps.incr();
     numBucketLists.incr();
@@ -386,6 +447,83 @@ public class OMMetrics {
     numListMultipartUploadParts.incr();
   }
 
+  public void incNumTenants() {
+    numTenants.incr();
+  }
+
+  public void decNumTenants() {
+    numTenants.incr(-1);
+  }
+
+  public void incNumTenantCreates() {
+    numTenantOps.incr();
+    numTenantCreates.incr();
+  }
+
+  public void incNumTenantCreateFails() {
+    numTenantCreateFails.incr();
+  }
+
+  public void incNumTenantDeletes() {
+    numTenantOps.incr();
+    numTenantDeletes.incr();
+  }
+
+  public void incNumTenantDeleteFails() {
+    numTenantDeleteFails.incr();
+  }
+
+  public void incNumTenantAssignUsers() {
+    numTenantOps.incr();
+    numTenantAssignUsers.incr();
+  }
+
+  public void incNumTenantAssignUserFails() {
+    numTenantAssignUserFails.incr();
+  }
+
+  public void incNumTenantRevokeUsers() {
+    numTenantOps.incr();
+    numTenantRevokeUsers.incr();
+  }
+
+  public void incNumTenantRevokeUserFails() {
+    numTenantRevokeUserFails.incr();
+  }
+
+  public void incNumTenantAssignAdmins() {
+    numTenantOps.incr();
+    numTenantAssignAdmins.incr();
+  }
+
+  public void incNumTenantAssignAdminFails() {
+    numTenantAssignAdminFails.incr();
+  }
+
+  public void incNumTenantRevokeAdmins() {
+    numTenantOps.incr();
+    numTenantRevokeAdmins.incr();
+  }
+
+  public void incNumTenantRevokeAdminFails() {
+    numTenantRevokeAdminFails.incr();
+  }
+
+  public void incNumTenantLists() {
+    numTenantOps.incr();
+    numTenantLists.incr();
+  }
+
+  public void incNumTenantGetUserInfos() {
+    numTenantOps.incr();
+    numTenantGetUserInfos.incr();
+  }
+
+  public void incNumTenantUserLists() {
+    numTenantOps.incr();
+    numTenantTenantUserLists.incr();
+  }
+
   public void incNumGetFileStatus() {
     numKeyOps.incr();
     numFSOps.incr();
@@ -585,6 +723,15 @@ public class OMMetrics {
     numRemoveAcl.incr();
   }
 
+  public void incNumGetKeyInfo() {
+    numGetKeyInfo.incr();
+    numKeyOps.incr();
+  }
+
+  public void incNumGetKeyInfoFails() {
+    getNumGetKeyInfoFails.incr();
+  }
+
   @VisibleForTesting
   public long getNumVolumeCreates() {
     return numVolumeCreates.value();
@@ -615,6 +762,11 @@ public class OMMetrics {
     return numBucketCreates.value();
   }
 
+  @VisibleForTesting
+  public long getNumFSOBucketCreates() {
+    return numFSOBucketCreates.value();
+  }
+
   @VisibleForTesting
   public long getNumBucketInfos() {
     return numBucketInfos.value();
@@ -630,6 +782,11 @@ public class OMMetrics {
     return numBucketDeletes.value();
   }
 
+  @VisibleForTesting
+  public long getNumFSOBucketDeletes() {
+    return numFSOBucketDeletes.value();
+  }
+
   @VisibleForTesting
   public long getNumBucketLists() {
     return numBucketLists.value();
@@ -810,6 +967,11 @@ public class OMMetrics {
     return numBucketS3ListFails.value();
   }
 
+  @VisibleForTesting
+  public long getDataCommittedBytes() {
+    return totalDataCommitted.value();
+  }
+
   public long getNumInitiateMultipartUploads() {
     return numInitiateMultipartUploads.value();
   }
@@ -858,6 +1020,74 @@ public class OMMetrics {
     return numRemoveAcl.value();
   }
 
+  public long getNumTenantOps() {
+    return numTenantOps.value();
+  }
+
+  public long getNumTenants() {
+    return numTenants.value();
+  }
+
+  public long getNumTenantCreates() {
+    return numTenantCreates.value();
+  }
+
+  public long getNumTenantCreateFails() {
+    return numTenantCreateFails.value();
+  }
+
+  public long getNumTenantDeletes() {
+    return numTenantDeletes.value();
+  }
+
+  public long getNumTenantDeleteFails() {
+    return numTenantDeleteFails.value();
+  }
+
+  public long getNumTenantAssignUsers() {
+    return numTenantAssignUsers.value();
+  }
+
+  public long getNumTenantAssignUserFails() {
+    return numTenantAssignUserFails.value();
+  }
+
+  public long getNumTenantRevokeUsers() {
+    return numTenantRevokeUsers.value();
+  }
+
+  public long getNumTenantRevokeUserFails() {
+    return numTenantRevokeUserFails.value();
+  }
+
+  public long getNumTenantAssignAdmins() {
+    return numTenantAssignAdmins.value();
+  }
+
+  public long getNumTenantAssignAdminFails() {
+    return numTenantAssignAdminFails.value();
+  }
+
+  public long getNumTenantRevokeAdmins() {
+    return numTenantRevokeAdmins.value();
+  }
+
+  public long getNumTenantRevokeAdminFails() {
+    return numTenantRevokeAdminFails.value();
+  }
+
+  public long getNumTenantLists() {
+    return numTenantLists.value();
+  }
+
+  public long getNumTenantGetUserInfos() {
+    return numTenantGetUserInfos.value();
+  }
+
+  public long getNumTenantTenantUserLists() {
+    return numTenantTenantUserLists.value();
+  }
+
   public void incNumTrashRenames() {
     numTrashRenames.incr();
   }
