@@ -18,6 +18,7 @@ package org.apache.hadoop.ozone.om;
 
 import com.google.common.base.Preconditions;
 import com.google.protobuf.RpcController;
+import org.apache.hadoop.ozone.ClientVersion;
 import org.apache.hadoop.ozone.om.exceptions.OMException;
 import org.apache.hadoop.fs.FSDataInputStream;
 import org.apache.hadoop.fs.permission.FsPermission;
@@ -35,7 +36,6 @@ import org.apache.hadoop.ozone.om.helpers.OmBucketInfo;
 import org.apache.hadoop.ozone.om.helpers.OmKeyArgs;
 import org.apache.hadoop.ozone.om.helpers.OmKeyInfo;
 import org.apache.hadoop.ozone.om.helpers.OzoneFileStatus;
-import org.apache.hadoop.ozone.om.helpers.OzoneFSUtils;
 import org.apache.hadoop.ozone.om.helpers.OMRatisHelper;
 import org.apache.hadoop.ozone.om.ratis.utils.OzoneManagerRatisUtils;
 import org.apache.hadoop.ozone.om.request.OMClientRequest;
@@ -48,6 +48,7 @@ import org.apache.ratis.protocol.RaftClientRequest;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import java.io.Closeable;
 import java.io.IOException;
 import java.net.InetAddress;
 import java.net.URI;
@@ -58,6 +59,7 @@ import java.util.Map;
 import java.util.Iterator;
 import java.util.concurrent.atomic.AtomicLong;
 
+import static org.apache.hadoop.ozone.OzoneConsts.OZONE_O3TRASH_URI_SCHEME;
 import static org.apache.hadoop.ozone.OzoneConsts.OZONE_URI_DELIMITER;
 import static org.apache.hadoop.ozone.om.helpers.OzoneFSUtils.addTrailingSlashIfNeeded;
 import static org.apache.hadoop.ozone.om.helpers.OzoneFSUtils.pathToKey;
@@ -122,8 +124,7 @@ public class TrashOzoneFileSystem extends FileSystem {
 
   @Override
   public URI getUri() {
-    throw new UnsupportedOperationException(
-        "fs.getUri() not implemented in TrashOzoneFileSystem");
+    return URI.create(OZONE_O3TRASH_URI_SCHEME + ":///");
   }
 
   @Override
@@ -136,7 +137,7 @@ public class TrashOzoneFileSystem extends FileSystem {
   public FSDataOutputStream create(Path path,
       FsPermission fsPermission,
       boolean b, int i, short i1,
-      long l, Progressable progressable){
+      long l, Progressable progressable) {
     throw new UnsupportedOperationException(
         "fs.create() not implemented in TrashOzoneFileSystem");
   }
@@ -157,15 +158,16 @@ public class TrashOzoneFileSystem extends FileSystem {
     OFSPath dstPath = new OFSPath(dst);
     OmBucketInfo bucket = ozoneManager.getBucketInfo(srcPath.getVolumeName(),
         srcPath.getBucketName());
-    if (OzoneFSUtils.isFSOptimizedBucket(bucket.getBucketLayout())) {
+    if (bucket.getBucketLayout().isFileSystemOptimized()) {
       return renameFSO(srcPath, dstPath);
     }
     Preconditions.checkArgument(srcPath.getBucketName().
         equals(dstPath.getBucketName()));
     Preconditions.checkArgument(srcPath.getTrashRoot().
         toString().equals(dstPath.getTrashRoot().toString()));
-    RenameIterator iterator = new RenameIterator(src, dst);
-    iterator.iterate();
+    try (RenameIterator iterator = new RenameIterator(src, dst)) {
+      iterator.iterate();
+    }
     return true;
   }
 
@@ -174,12 +176,12 @@ public class TrashOzoneFileSystem extends FileSystem {
     OzoneManagerProtocolProtos.OMRequest omRequest =
         getRenameKeyRequest(srcPath, dstPath);
     try {
-      if(omRequest != null) {
+      if (omRequest != null) {
         submitRequest(omRequest);
         return true;
       }
       return false;
-    } catch (Exception e){
+    } catch (Exception e) {
       LOG.error("Couldn't send rename request", e);
       return false;
     }
@@ -191,11 +193,12 @@ public class TrashOzoneFileSystem extends FileSystem {
     OFSPath srcPath = new OFSPath(path);
     OmBucketInfo bucket = ozoneManager.getBucketInfo(srcPath.getVolumeName(),
         srcPath.getBucketName());
-    if (OzoneFSUtils.isFSOptimizedBucket(bucket.getBucketLayout())) {
+    if (bucket.getBucketLayout().isFileSystemOptimized()) {
       return deleteFSO(srcPath);
     }
-    DeleteIterator iterator = new DeleteIterator(path, true);
-    iterator.iterate();
+    try (DeleteIterator iterator = new DeleteIterator(path, true)) {
+      iterator.iterate();
+    }
     return true;
   }
 
@@ -204,7 +207,7 @@ public class TrashOzoneFileSystem extends FileSystem {
     OzoneManagerProtocolProtos.OMRequest omRequest =
         getDeleteKeyRequest(srcPath);
     try {
-      if(omRequest != null) {
+      if (omRequest != null) {
         submitRequest(omRequest);
         return true;
       }
@@ -288,6 +291,7 @@ public class TrashOzoneFileSystem extends FileSystem {
         .setVolumeName(volume)
         .setBucketName(bucket)
         .setKeyName(key)
+        .setHeadOp(true)
         .build();
     return keyArgs;
   }
@@ -300,7 +304,7 @@ public class TrashOzoneFileSystem extends FileSystem {
         CacheValue<OmBucketInfo>>> bucketIterator =
         ozoneManager.getMetadataManager().getBucketIterator();
     List<FileStatus> ret = new ArrayList<>();
-    while (bucketIterator.hasNext()){
+    while (bucketIterator.hasNext()) {
       Map.Entry<CacheKey<String>, CacheValue<OmBucketInfo>> entry =
           bucketIterator.next();
       OmBucketInfo omBucketInfo = entry.getValue().getCacheValue();
@@ -317,7 +321,7 @@ public class TrashOzoneFileSystem extends FileSystem {
             }
           }
         }
-      } catch (Exception e){
+      } catch (Exception e) {
         LOG.error("Couldn't perform fs operation " +
             "fs.listStatus()/fs.exists()", e);
       }
@@ -341,7 +345,7 @@ public class TrashOzoneFileSystem extends FileSystem {
     }
   }
 
-  private abstract class OzoneListingIterator {
+  private abstract class OzoneListingIterator implements Closeable {
     private final Path path;
     private final FileStatus status;
     private String pathKey;
@@ -419,6 +423,11 @@ public class TrashOzoneFileSystem extends FileSystem {
     FileStatus getStatus() {
       return status;
     }
+
+    @Override
+    public void close() throws IOException {
+      keyIterator.close();
+    }
   }
 
 
@@ -446,6 +455,7 @@ public class TrashOzoneFileSystem extends FileSystem {
     try {
       omRequest = OzoneManagerProtocolProtos.OMRequest.newBuilder()
               .setClientId(CLIENT_ID.toString())
+              .setVersion(ClientVersion.CURRENT_VERSION)
               .setUserInfo(getUserInfo())
               .setRenameKeyRequest(renameKeyRequest)
               .setCmdType(OzoneManagerProtocolProtos.Type.RenameKey)
@@ -510,6 +520,7 @@ public class TrashOzoneFileSystem extends FileSystem {
       omRequest =
           OzoneManagerProtocolProtos.OMRequest.newBuilder()
               .setClientId(CLIENT_ID.toString())
+              .setVersion(ClientVersion.CURRENT_VERSION)
               .setUserInfo(getUserInfo())
               .setDeleteKeyRequest(deleteKeyRequest)
               .setCmdType(OzoneManagerProtocolProtos.Type.DeleteKey)
@@ -577,6 +588,7 @@ public class TrashOzoneFileSystem extends FileSystem {
       try {
         omRequest = OzoneManagerProtocolProtos.OMRequest.newBuilder()
             .setClientId(CLIENT_ID.toString())
+            .setVersion(ClientVersion.CURRENT_VERSION)
             .setUserInfo(getUserInfo())
             .setDeleteKeysRequest(deleteKeysRequest)
             .setCmdType(OzoneManagerProtocolProtos.Type.DeleteKeys)
