@@ -20,7 +20,7 @@ package org.apache.hadoop.ozone.om.ratis;
 
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
-import com.google.common.collect.Lists;
+import com.google.common.collect.Maps;
 import com.google.protobuf.InvalidProtocolBufferException;
 import com.google.protobuf.ServiceException;
 
@@ -35,11 +35,13 @@ import java.util.Collections;
 import java.util.List;
 import java.util.Map;
 import java.util.UUID;
+import java.util.concurrent.ExecutionException;
 import java.util.concurrent.atomic.AtomicLong;
 import java.util.concurrent.TimeUnit;
 
 import org.apache.hadoop.hdds.conf.ConfigurationSource;
 import org.apache.hadoop.hdds.conf.StorageUnit;
+import org.apache.hadoop.hdds.ratis.RatisHelper;
 import org.apache.hadoop.hdds.security.x509.SecurityConfig;
 import org.apache.hadoop.hdds.security.x509.certificate.client.CertificateClient;
 import org.apache.hadoop.hdds.tracing.TracingUtil;
@@ -58,7 +60,6 @@ import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.OMReque
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.OMResponse;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.Status;
 
-import org.apache.ratis.RaftConfigKeys;
 import org.apache.ratis.conf.Parameters;
 import org.apache.ratis.conf.RaftProperties;
 import org.apache.ratis.grpc.GrpcConfigKeys;
@@ -81,6 +82,7 @@ import org.apache.ratis.rpc.SupportedRpcType;
 import org.apache.ratis.server.RaftServer;
 import org.apache.ratis.server.RaftServerConfigKeys;
 import org.apache.ratis.server.protocol.TermIndex;
+import org.apache.ratis.thirdparty.com.google.protobuf.ByteString;
 import org.apache.ratis.util.LifeCycle;
 import org.apache.ratis.util.SizeInBytes;
 import org.apache.ratis.util.StringUtils;
@@ -105,7 +107,7 @@ public final class OzoneManagerRatisServer {
   private final RaftGroupId raftGroupId;
   private final RaftGroup raftGroup;
   private final RaftPeerId raftPeerId;
-  private final List<RaftPeer> raftPeers;
+  private final Map<String, RaftPeer> raftPeerMap;
 
   private final OzoneManager ozoneManager;
   private final OzoneManagerStateMachine omStateMachine;
@@ -143,8 +145,8 @@ public final class OzoneManagerRatisServer {
     this.raftPeerId = localRaftPeerId;
     this.raftGroupId = RaftGroupId.valueOf(
         getRaftGroupIdFromOmServiceId(raftGroupIdStr));
-    this.raftPeers = Lists.newArrayList();
-    this.raftPeers.addAll(peers);
+    this.raftPeerMap = Maps.newHashMap();
+    peers.forEach(e -> raftPeerMap.put(e.getId().toString(), e));
     this.raftGroup = RaftGroup.valueOf(raftGroupId, peers);
 
     if (isBootstrapping) {
@@ -281,7 +283,10 @@ public final class OzoneManagerRatisServer {
     try {
       return server.submitClientRequestAsync(raftClientRequest)
           .get();
-    } catch (Exception ex) {
+    } catch (ExecutionException | IOException ex) {
+      throw new ServiceException(ex.getMessage(), ex);
+    } catch (InterruptedException ex) {
+      Thread.currentThread().interrupt();
       throw new ServiceException(ex.getMessage(), ex);
     }
   }
@@ -307,28 +312,55 @@ public final class OzoneManagerRatisServer {
         newRaftPeer, raftGroup);
 
     List<RaftPeer> newPeersList = new ArrayList<>();
-    newPeersList.addAll(raftPeers);
+    newPeersList.addAll(raftPeerMap.values());
     newPeersList.add(newRaftPeer);
 
     checkLeaderStatus();
     SetConfigurationRequest request = new SetConfigurationRequest(clientId,
         server.getId(), raftGroupId, nextCallId(), newPeersList);
 
-    try {
-      RaftClientReply raftClientReply = server.setConfiguration(request);
-      if (raftClientReply.isSuccess()) {
-        LOG.info("Added OM {} to Ratis group {}.", newOMNodeId, raftGroupId);
-      } else {
-        LOG.error("Failed to add OM {} to Ratis group {}. Ratis " +
-                "SetConfiguration reply: {}", newOMNodeId, raftGroupId,
-            raftClientReply);
-        throw new IOException("Failed to add OM " + newOMNodeId + " to Ratis " +
-            "ring.");
-      }
-    } catch (IOException e) {
-      LOG.error("Failed to update Ratis configuration and add OM {} to " +
-          "Ratis group {}", newOMNodeId, raftGroupId, e);
-      throw e;
+    RaftClientReply raftClientReply = server.setConfiguration(request);
+    if (raftClientReply.isSuccess()) {
+      LOG.info("Added OM {} to Ratis group {}.", newOMNodeId, raftGroupId);
+    } else {
+      LOG.error("Failed to add OM {} to Ratis group {}. Ratis " +
+              "SetConfiguration reply: {}", newOMNodeId, raftGroupId,
+          raftClientReply);
+      throw new IOException("Failed to add OM " + newOMNodeId + " to Ratis " +
+          "ring.");
+    }
+  }
+
+  /**
+   * Remove decommissioned OM node from Ratis ring.
+   */
+  public void removeOMFromRatisRing(OMNodeDetails removeOMNode)
+      throws IOException {
+    Preconditions.checkNotNull(removeOMNode);
+
+    String removeNodeId = removeOMNode.getNodeId();
+    LOG.info("{}: Submitting SetConfiguration request to Ratis server to " +
+            "remove OM peer {} from Ratis group {}", ozoneManager.getOMNodeId(),
+        removeNodeId, raftGroup);
+
+    List<RaftPeer> newPeersList = new ArrayList<>();
+    newPeersList.addAll(raftPeerMap.values());
+    newPeersList.remove(raftPeerMap.get(removeNodeId));
+
+    checkLeaderStatus();
+    SetConfigurationRequest request = new SetConfigurationRequest(clientId,
+        server.getId(), raftGroupId, nextCallId(), newPeersList);
+
+    RaftClientReply raftClientReply = server.setConfiguration(request);
+    if (raftClientReply.isSuccess()) {
+      LOG.info("Removed OM {} from Ratis group {}.", removeNodeId,
+          raftGroupId);
+    } else {
+      LOG.error("Failed to remove OM {} from Ratis group {}. Ratis " +
+              "SetConfiguration reply: {}", removeNodeId, raftGroupId,
+          raftClientReply);
+      throw new IOException("Failed to remove OM " + removeNodeId + " from " +
+          "Ratis ring.");
     }
   }
 
@@ -337,9 +369,7 @@ public final class OzoneManagerRatisServer {
    */
   public List<String> getPeerIds() {
     List<String> peerIds = new ArrayList<>();
-    for (RaftPeer raftPeer : raftPeers) {
-      peerIds.add(raftPeer.getId().toString());
-    }
+    peerIds.addAll(raftPeerMap.keySet());
     return peerIds;
   }
 
@@ -350,12 +380,7 @@ public final class OzoneManagerRatisServer {
    */
   @VisibleForTesting
   public boolean doesPeerExist(String peerId) {
-    for (RaftPeer raftPeer : raftPeers) {
-      if (raftPeer.getId().toString().equals(peerId)) {
-        return true;
-      }
-    }
-    return false;
+    return raftPeerMap.containsKey(peerId);
   }
 
   /**
@@ -365,12 +390,24 @@ public final class OzoneManagerRatisServer {
     InetSocketAddress newOMRatisAddr = new InetSocketAddress(
         omNodeDetails.getHostAddress(), omNodeDetails.getRatisPort());
 
-    raftPeers.add(RaftPeer.newBuilder()
-        .setId(RaftPeerId.valueOf(omNodeDetails.getNodeId()))
+    String newNodeId = omNodeDetails.getNodeId();
+    RaftPeerId newPeerId = RaftPeerId.valueOf(newNodeId);
+    RaftPeer raftPeer = RaftPeer.newBuilder()
+        .setId(newPeerId)
         .setAddress(newOMRatisAddr)
-        .build());
+        .build();
+    raftPeerMap.put(newNodeId, raftPeer);
 
-    LOG.info("Added OM {} to Ratis Peers list.", omNodeDetails.getNodeId());
+    LOG.info("Added OM {} to Ratis Peers list.", newNodeId);
+  }
+
+  /**
+   * Remove given node from list of RaftPeers.
+   */
+  public void removeRaftPeer(OMNodeDetails omNodeDetails) {
+    String removeNodeID = omNodeDetails.getNodeId();
+    raftPeerMap.remove(removeNodeID);
+    LOG.info("{}: Removed OM {} from Ratis Peers list.", this, removeNodeID);
   }
 
   /**
@@ -380,8 +417,10 @@ public final class OzoneManagerRatisServer {
    * ratis server.
    */
   private RaftClientRequest createWriteRaftClientRequest(OMRequest omRequest) {
-    Preconditions.checkArgument(Server.getClientId() != DUMMY_CLIENT_ID);
-    Preconditions.checkArgument(Server.getCallId() != INVALID_CALL_ID);
+    if (!ozoneManager.isTestSecureOmFlag()) {
+      Preconditions.checkArgument(Server.getClientId() != DUMMY_CLIENT_ID);
+      Preconditions.checkArgument(Server.getCallId() != INVALID_CALL_ID);
+    }
     return RaftClientRequest.newBuilder()
         .setClientId(
             ClientId.valueOf(UUID.nameUUIDFromBytes(Server.getClientId())))
@@ -521,7 +560,7 @@ public final class OzoneManagerRatisServer {
   public void stop() {
     try {
       server.close();
-      omStateMachine.stop();
+      omStateMachine.close();
     } catch (IOException e) {
       throw new RuntimeException(e);
     }
@@ -530,14 +569,12 @@ public final class OzoneManagerRatisServer {
   //TODO simplify it to make it shorter
   @SuppressWarnings("methodlength")
   private RaftProperties newRaftProperties(ConfigurationSource conf) {
-    final RaftProperties properties = new RaftProperties();
-
     // Set RPC type
     final String rpcType = conf.get(
         OMConfigKeys.OZONE_OM_RATIS_RPC_TYPE_KEY,
         OMConfigKeys.OZONE_OM_RATIS_RPC_TYPE_DEFAULT);
     final RpcType rpc = SupportedRpcType.valueOfIgnoreCase(rpcType);
-    RaftConfigKeys.Rpc.setType(properties, rpc);
+    final RaftProperties properties = RatisHelper.newRaftProperties(rpc);
 
     // Set the ratis port number
     if (rpc == SupportedRpcType.GRPC) {
@@ -688,7 +725,20 @@ public final class OzoneManagerRatisServer {
 
   private static Map<String, String> getOMHAConfigs(
       ConfigurationSource configuration) {
-    return configuration.getPropsWithPrefix(OZONE_OM_HA_PREFIX + ".");
+    return configuration
+        .getPropsMatchPrefixAndTrimPrefix(OZONE_OM_HA_PREFIX + ".");
+  }
+
+  public RaftPeer getLeader() throws IOException {
+    RaftServer.Division division = server.getDivision(raftGroupId);
+    if (division.getInfo().isLeader()) {
+      return division.getPeer();
+    } else {
+      ByteString leaderId = division.getInfo().getRoleInfoProto()
+          .getFollowerInfo().getLeaderInfo().getId().getId();
+      return leaderId.isEmpty() ? null :
+          division.getRaftConf().getPeer(RaftPeerId.valueOf(leaderId));
+    }
   }
 
   /**
@@ -775,21 +825,16 @@ public final class OzoneManagerRatisServer {
 
   private static Parameters createServerTlsParameters(SecurityConfig conf,
       CertificateClient caClient) throws IOException {
-    Parameters parameters = new Parameters();
-
     if (conf.isSecurityEnabled() && conf.isGrpcTlsEnabled()) {
       List<X509Certificate> caList = HAUtils.buildCAX509List(caClient,
           conf.getConfiguration());
       GrpcTlsConfig config = new GrpcTlsConfig(
           caClient.getPrivateKey(), caClient.getCertificate(),
           caList, true);
-      GrpcConfigKeys.Server.setTlsConf(parameters, config);
-      GrpcConfigKeys.Admin.setTlsConf(parameters, config);
-      GrpcConfigKeys.Client.setTlsConf(parameters, config);
-      GrpcConfigKeys.TLS.setConf(parameters, config);
+      return RatisHelper.setServerTlsConf(config);
     }
 
-    return parameters;
+    return null;
   }
 
 }
