@@ -22,6 +22,7 @@ import com.google.common.base.Strings;
 import com.google.protobuf.ServiceException;
 import java.io.File;
 import java.nio.file.Paths;
+
 import org.apache.hadoop.hdds.conf.ConfigurationSource;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.server.ServerUtils;
@@ -32,35 +33,20 @@ import org.apache.hadoop.ozone.om.codec.OMDBDefinition;
 import org.apache.hadoop.ozone.om.exceptions.OMException;
 import org.apache.hadoop.hdds.utils.TransactionInfo;
 import org.apache.hadoop.ozone.om.helpers.BucketLayout;
-import org.apache.hadoop.ozone.om.helpers.OmBucketInfo;
 import org.apache.hadoop.ozone.om.exceptions.OMLeaderNotReadyException;
 import org.apache.hadoop.ozone.om.exceptions.OMNotLeaderException;
-import org.apache.hadoop.ozone.om.ratis.OzoneManagerRatisServer.RaftServerStatus;
+import org.apache.hadoop.ozone.om.request.BucketLayoutAwareOMKeyRequestFactory;
 import org.apache.hadoop.ozone.om.request.bucket.OMBucketCreateRequest;
 import org.apache.hadoop.ozone.om.request.bucket.OMBucketDeleteRequest;
+import org.apache.hadoop.ozone.om.request.bucket.OMBucketSetOwnerRequest;
 import org.apache.hadoop.ozone.om.request.bucket.OMBucketSetPropertyRequest;
 import org.apache.hadoop.ozone.om.request.OMClientRequest;
 import org.apache.hadoop.ozone.om.request.bucket.acl.OMBucketAddAclRequest;
 import org.apache.hadoop.ozone.om.request.bucket.acl.OMBucketRemoveAclRequest;
 import org.apache.hadoop.ozone.om.request.bucket.acl.OMBucketSetAclRequest;
-import org.apache.hadoop.ozone.om.request.file.OMDirectoryCreateRequest;
-import org.apache.hadoop.ozone.om.request.file.OMDirectoryCreateRequestWithFSO;
-import org.apache.hadoop.ozone.om.request.file.OMFileCreateRequest;
-import org.apache.hadoop.ozone.om.request.file.OMFileCreateRequestWithFSO;
-import org.apache.hadoop.ozone.om.request.key.OMKeysDeleteRequest;
-import org.apache.hadoop.ozone.om.request.key.OMAllocateBlockRequest;
-import org.apache.hadoop.ozone.om.request.key.OMAllocateBlockRequestWithFSO;
-import org.apache.hadoop.ozone.om.request.key.OMKeyCommitRequest;
-import org.apache.hadoop.ozone.om.request.key.OMKeyCommitRequestWithFSO;
-import org.apache.hadoop.ozone.om.request.key.OMKeyCreateRequest;
-import org.apache.hadoop.ozone.om.request.key.OMKeyCreateRequestWithFSO;
-import org.apache.hadoop.ozone.om.request.key.OMKeyDeleteRequest;
-import org.apache.hadoop.ozone.om.request.key.OMKeyDeleteRequestWithFSO;
 import org.apache.hadoop.ozone.om.request.key.OMKeyPurgeRequest;
-import org.apache.hadoop.ozone.om.request.key.OMKeyRenameRequest;
-import org.apache.hadoop.ozone.om.request.key.OMKeyRenameRequestWithFSO;
-import org.apache.hadoop.ozone.om.request.key.OMKeysRenameRequest;
-import org.apache.hadoop.ozone.om.request.key.OMPathsPurgeRequestWithFSO;
+import org.apache.hadoop.ozone.om.request.key.OMDirectoriesPurgeRequestWithFSO;
+import org.apache.hadoop.ozone.om.request.key.OMOpenKeysDeleteRequest;
 import org.apache.hadoop.ozone.om.request.key.OMTrashRecoverRequest;
 import org.apache.hadoop.ozone.om.request.key.acl.OMKeyAddAclRequest;
 import org.apache.hadoop.ozone.om.request.key.acl.OMKeyAddAclRequestWithFSO;
@@ -71,16 +57,16 @@ import org.apache.hadoop.ozone.om.request.key.acl.OMKeySetAclRequestWithFSO;
 import org.apache.hadoop.ozone.om.request.key.acl.prefix.OMPrefixAddAclRequest;
 import org.apache.hadoop.ozone.om.request.key.acl.prefix.OMPrefixRemoveAclRequest;
 import org.apache.hadoop.ozone.om.request.key.acl.prefix.OMPrefixSetAclRequest;
-import org.apache.hadoop.ozone.om.request.s3.multipart.S3InitiateMultipartUploadRequest;
-import org.apache.hadoop.ozone.om.request.s3.multipart.S3InitiateMultipartUploadRequestWithFSO;
-import org.apache.hadoop.ozone.om.request.s3.multipart.S3MultipartUploadAbortRequest;
-import org.apache.hadoop.ozone.om.request.s3.multipart.S3MultipartUploadAbortRequestWithFSO;
-import org.apache.hadoop.ozone.om.request.s3.multipart.S3MultipartUploadCommitPartRequest;
-import org.apache.hadoop.ozone.om.request.s3.multipart.S3MultipartUploadCommitPartRequestWithFSO;
-import org.apache.hadoop.ozone.om.request.s3.multipart.S3MultipartUploadCompleteRequest;
-import org.apache.hadoop.ozone.om.request.s3.multipart.S3MultipartUploadCompleteRequestWithFSO;
+import org.apache.hadoop.ozone.om.request.s3.security.OMSetSecretRequest;
 import org.apache.hadoop.ozone.om.request.s3.security.S3GetSecretRequest;
 import org.apache.hadoop.ozone.om.request.s3.security.S3RevokeSecretRequest;
+import org.apache.hadoop.ozone.om.request.s3.tenant.OMTenantAssignUserAccessIdRequest;
+import org.apache.hadoop.ozone.om.request.s3.tenant.OMSetRangerServiceVersionRequest;
+import org.apache.hadoop.ozone.om.request.s3.tenant.OMTenantAssignAdminRequest;
+import org.apache.hadoop.ozone.om.request.s3.tenant.OMTenantCreateRequest;
+import org.apache.hadoop.ozone.om.request.s3.tenant.OMTenantDeleteRequest;
+import org.apache.hadoop.ozone.om.request.s3.tenant.OMTenantRevokeAdminRequest;
+import org.apache.hadoop.ozone.om.request.s3.tenant.OMTenantRevokeUserAccessIdRequest;
 import org.apache.hadoop.ozone.om.request.security.OMCancelDelegationTokenRequest;
 import org.apache.hadoop.ozone.om.request.security.OMGetDelegationTokenRequest;
 import org.apache.hadoop.ozone.om.request.security.OMRenewDelegationTokenRequest;
@@ -94,16 +80,17 @@ import org.apache.hadoop.ozone.om.request.volume.OMVolumeSetQuotaRequest;
 import org.apache.hadoop.ozone.om.request.volume.acl.OMVolumeAddAclRequest;
 import org.apache.hadoop.ozone.om.request.volume.acl.OMVolumeRemoveAclRequest;
 import org.apache.hadoop.ozone.om.request.volume.acl.OMVolumeSetAclRequest;
+import org.apache.hadoop.ozone.om.request.OMEchoRPCRequest;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.OMRequest;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.OzoneObj.ObjectType;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.Status;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.Type;
-import org.apache.ratis.protocol.RaftPeerId;
 import org.rocksdb.RocksDBException;
 
 import java.io.IOException;
 import java.nio.file.Path;
+
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -115,35 +102,21 @@ import static org.apache.hadoop.ozone.om.OMConfigKeys.OZONE_OM_RATIS_SNAPSHOT_DI
  * Utility class used by OzoneManager HA.
  */
 public final class OzoneManagerRatisUtils {
-
-  // TODO: Temporary workaround for OM upgrade path and will be replaced once
-  //  upgrade HDDS-3698 story reaches consensus.
-  private static boolean isBucketFSOptimized = false;
-
   private static final Logger LOG = LoggerFactory
       .getLogger(OzoneManagerRatisUtils.class);
 
   private OzoneManagerRatisUtils() {
   }
 
-  /**
-   * Sets enabled/disabled file system optimized path property. A true value
-   * represents enabled, false represents disabled.
-   *
-   * @param enabledFSO enabled/disabled file system optimized
-   */
-  public static void setBucketFSOptimized(boolean enabledFSO) {
-    OzoneManagerRatisUtils.isBucketFSOptimized = enabledFSO;
-  }
-
   /**
    * Create OMClientRequest which encapsulates the OMRequest.
    * @param omRequest
    * @return OMClientRequest
    * @throws IOException
    */
+  @SuppressWarnings("checkstyle:methodlength")
   public static OMClientRequest createClientRequest(OMRequest omRequest,
-      OzoneManager ozoneManager) {
+      OzoneManager ozoneManager) throws IOException {
 
     // Handling of exception by createClientRequest(OMRequest, OzoneManger):
     // Either the code will take FSO or non FSO path, both classes has a
@@ -154,6 +127,10 @@ public final class OzoneManagerRatisUtils {
     // handles it appropriately.
     Type cmdType = omRequest.getCmdType();
     OzoneManagerProtocolProtos.KeyArgs keyArgs;
+    BucketLayout bucketLayout = BucketLayout.DEFAULT;
+    String volumeName = "";
+    String bucketName = "";
+
     switch (cmdType) {
     case CreateVolume:
       return new OMVolumeCreateRequest(omRequest);
@@ -178,92 +155,13 @@ public final class OzoneManagerRatisUtils {
     case DeleteBucket:
       return new OMBucketDeleteRequest(omRequest);
     case SetBucketProperty:
-      return new OMBucketSetPropertyRequest(omRequest);
-    case AllocateBlock:
-      keyArgs = omRequest.getAllocateBlockRequest().getKeyArgs();
-      if (isBucketFSOptimized(keyArgs.getVolumeName(), keyArgs.getBucketName(),
-          ozoneManager)) {
-        return new OMAllocateBlockRequestWithFSO(omRequest);
+      boolean hasBucketOwner = omRequest.getSetBucketPropertyRequest()
+          .getBucketArgs().hasOwnerName();
+      if (hasBucketOwner) {
+        return new OMBucketSetOwnerRequest(omRequest);
+      } else {
+        return new OMBucketSetPropertyRequest(omRequest);
       }
-      return new OMAllocateBlockRequest(omRequest);
-    case CreateKey:
-      keyArgs = omRequest.getCreateKeyRequest().getKeyArgs();
-      if (isBucketFSOptimized(keyArgs.getVolumeName(), keyArgs.getBucketName(),
-          ozoneManager)) {
-        return new OMKeyCreateRequestWithFSO(omRequest);
-      }
-      return new OMKeyCreateRequest(omRequest);
-    case CommitKey:
-      keyArgs = omRequest.getCommitKeyRequest().getKeyArgs();
-      if (isBucketFSOptimized(keyArgs.getVolumeName(), keyArgs.getBucketName(),
-          ozoneManager)) {
-        return new OMKeyCommitRequestWithFSO(omRequest);
-      }
-      return new OMKeyCommitRequest(omRequest);
-    case DeleteKey:
-      keyArgs = omRequest.getDeleteKeyRequest().getKeyArgs();
-      if (isBucketFSOptimized(keyArgs.getVolumeName(), keyArgs.getBucketName(),
-          ozoneManager)) {
-        return new OMKeyDeleteRequestWithFSO(omRequest);
-      }
-      return new OMKeyDeleteRequest(omRequest);
-    case DeleteKeys:
-      return new OMKeysDeleteRequest(omRequest);
-    case RenameKey:
-      keyArgs = omRequest.getRenameKeyRequest().getKeyArgs();
-      if (isBucketFSOptimized(keyArgs.getVolumeName(), keyArgs.getBucketName(),
-          ozoneManager)) {
-        return new OMKeyRenameRequestWithFSO(omRequest);
-      }
-      return new OMKeyRenameRequest(omRequest);
-    case RenameKeys:
-      return new OMKeysRenameRequest(omRequest);
-    case CreateDirectory:
-      keyArgs = omRequest.getCreateDirectoryRequest().getKeyArgs();
-      if (isBucketFSOptimized(keyArgs.getVolumeName(), keyArgs.getBucketName(),
-          ozoneManager)) {
-        return new OMDirectoryCreateRequestWithFSO(omRequest);
-      }
-      return new OMDirectoryCreateRequest(omRequest);
-    case CreateFile:
-      keyArgs = omRequest.getCreateFileRequest().getKeyArgs();
-      if (isBucketFSOptimized(keyArgs.getVolumeName(), keyArgs.getBucketName(),
-          ozoneManager)) {
-        return new OMFileCreateRequestWithFSO(omRequest);
-      }
-      return new OMFileCreateRequest(omRequest);
-    case PurgeKeys:
-      return new OMKeyPurgeRequest(omRequest);
-    case PurgePaths:
-      return new OMPathsPurgeRequestWithFSO(omRequest);
-    case InitiateMultiPartUpload:
-      keyArgs = omRequest.getInitiateMultiPartUploadRequest().getKeyArgs();
-      if (isBucketFSOptimized(keyArgs.getVolumeName(), keyArgs.getBucketName(),
-          ozoneManager)) {
-        return new S3InitiateMultipartUploadRequestWithFSO(omRequest);
-      }
-      return new S3InitiateMultipartUploadRequest(omRequest);
-    case CommitMultiPartUpload:
-      keyArgs = omRequest.getCommitMultiPartUploadRequest().getKeyArgs();
-      if (isBucketFSOptimized(keyArgs.getVolumeName(), keyArgs.getBucketName(),
-          ozoneManager)) {
-        return new S3MultipartUploadCommitPartRequestWithFSO(omRequest);
-      }
-      return new S3MultipartUploadCommitPartRequest(omRequest);
-    case AbortMultiPartUpload:
-      keyArgs = omRequest.getAbortMultiPartUploadRequest().getKeyArgs();
-      if (isBucketFSOptimized(keyArgs.getVolumeName(), keyArgs.getBucketName(),
-          ozoneManager)) {
-        return new S3MultipartUploadAbortRequestWithFSO(omRequest);
-      }
-      return new S3MultipartUploadAbortRequest(omRequest);
-    case CompleteMultiPartUpload:
-      keyArgs = omRequest.getCompleteMultiPartUploadRequest().getKeyArgs();
-      if (isBucketFSOptimized(keyArgs.getVolumeName(), keyArgs.getBucketName(),
-          ozoneManager)) {
-        return new S3MultipartUploadCompleteRequestWithFSO(omRequest);
-      }
-      return new S3MultipartUploadCompleteRequest(omRequest);
     case AddAcl:
     case RemoveAcl:
     case SetAcl:
@@ -284,12 +182,123 @@ public final class OzoneManagerRatisUtils {
       return new OMPrepareRequest(omRequest);
     case CancelPrepare:
       return new OMCancelPrepareRequest(omRequest);
+    case SetS3Secret:
+      return new OMSetSecretRequest(omRequest);
     case RevokeS3Secret:
       return new S3RevokeSecretRequest(omRequest);
+    case PurgeKeys:
+      return new OMKeyPurgeRequest(omRequest);
+    case PurgeDirectories:
+      return new OMDirectoriesPurgeRequestWithFSO(omRequest);
+    case CreateTenant:
+      ozoneManager.checkS3MultiTenancyEnabled();
+      return new OMTenantCreateRequest(omRequest);
+    case DeleteTenant:
+      ozoneManager.checkS3MultiTenancyEnabled();
+      return new OMTenantDeleteRequest(omRequest);
+    case TenantAssignUserAccessId:
+      ozoneManager.checkS3MultiTenancyEnabled();
+      return new OMTenantAssignUserAccessIdRequest(omRequest);
+    case TenantRevokeUserAccessId:
+      ozoneManager.checkS3MultiTenancyEnabled();
+      return new OMTenantRevokeUserAccessIdRequest(omRequest);
+    case TenantAssignAdmin:
+      ozoneManager.checkS3MultiTenancyEnabled();
+      return new OMTenantAssignAdminRequest(omRequest);
+    case TenantRevokeAdmin:
+      ozoneManager.checkS3MultiTenancyEnabled();
+      return new OMTenantRevokeAdminRequest(omRequest);
+    case SetRangerServiceVersion:
+      return new OMSetRangerServiceVersionRequest(omRequest);
+    case DeleteOpenKeys:
+      BucketLayout bktLayout = BucketLayout.DEFAULT;
+      if (omRequest.getDeleteOpenKeysRequest().hasBucketLayout()) {
+        bktLayout = BucketLayout.fromProto(
+            omRequest.getDeleteOpenKeysRequest().getBucketLayout());
+      }
+      return new OMOpenKeysDeleteRequest(omRequest, bktLayout);
+
+    /*
+     * Key requests that can have multiple variants based on the bucket layout
+     * should be created using {@link BucketLayoutAwareOMKeyRequestFactory}.
+     */
+    case CreateDirectory:
+      keyArgs = omRequest.getCreateDirectoryRequest().getKeyArgs();
+      volumeName = keyArgs.getVolumeName();
+      bucketName = keyArgs.getBucketName();
+      break;
+    case CreateFile:
+      keyArgs = omRequest.getCreateFileRequest().getKeyArgs();
+      volumeName = keyArgs.getVolumeName();
+      bucketName = keyArgs.getBucketName();
+      break;
+    case CreateKey:
+      keyArgs = omRequest.getCreateKeyRequest().getKeyArgs();
+      volumeName = keyArgs.getVolumeName();
+      bucketName = keyArgs.getBucketName();
+      break;
+    case AllocateBlock:
+      keyArgs = omRequest.getAllocateBlockRequest().getKeyArgs();
+      volumeName = keyArgs.getVolumeName();
+      bucketName = keyArgs.getBucketName();
+      break;
+    case CommitKey:
+      keyArgs = omRequest.getCommitKeyRequest().getKeyArgs();
+      volumeName = keyArgs.getVolumeName();
+      bucketName = keyArgs.getBucketName();
+      break;
+    case DeleteKey:
+      keyArgs = omRequest.getDeleteKeyRequest().getKeyArgs();
+      volumeName = keyArgs.getVolumeName();
+      bucketName = keyArgs.getBucketName();
+      break;
+    case DeleteKeys:
+      OzoneManagerProtocolProtos.DeleteKeyArgs deleteKeyArgs =
+          omRequest.getDeleteKeysRequest()
+              .getDeleteKeys();
+      volumeName = deleteKeyArgs.getVolumeName();
+      bucketName = deleteKeyArgs.getBucketName();
+      break;
+    case RenameKey:
+      keyArgs = omRequest.getRenameKeyRequest().getKeyArgs();
+      volumeName = keyArgs.getVolumeName();
+      bucketName = keyArgs.getBucketName();
+      break;
+    case RenameKeys:
+      OzoneManagerProtocolProtos.RenameKeysArgs renameKeysArgs =
+          omRequest.getRenameKeysRequest().getRenameKeysArgs();
+      volumeName = renameKeysArgs.getVolumeName();
+      bucketName = renameKeysArgs.getBucketName();
+      break;
+    case InitiateMultiPartUpload:
+      keyArgs = omRequest.getInitiateMultiPartUploadRequest().getKeyArgs();
+      volumeName = keyArgs.getVolumeName();
+      bucketName = keyArgs.getBucketName();
+      break;
+    case CommitMultiPartUpload:
+      keyArgs = omRequest.getCommitMultiPartUploadRequest().getKeyArgs();
+      volumeName = keyArgs.getVolumeName();
+      bucketName = keyArgs.getBucketName();
+      break;
+    case AbortMultiPartUpload:
+      keyArgs = omRequest.getAbortMultiPartUploadRequest().getKeyArgs();
+      volumeName = keyArgs.getVolumeName();
+      bucketName = keyArgs.getBucketName();
+      break;
+    case CompleteMultiPartUpload:
+      keyArgs = omRequest.getCompleteMultiPartUploadRequest().getKeyArgs();
+      volumeName = keyArgs.getVolumeName();
+      bucketName = keyArgs.getBucketName();
+      break;
+    case EchoRPC:
+      return new OMEchoRPCRequest(omRequest);
     default:
       throw new IllegalStateException("Unrecognized write command " +
           "type request" + cmdType);
     }
+
+    return BucketLayoutAwareOMKeyRequestFactory.createRequest(
+        volumeName, bucketName, omRequest, ozoneManager.getMetadataManager());
   }
 
   private static OMClientRequest getOMAclRequest(OMRequest omRequest,
@@ -302,11 +311,11 @@ public final class OzoneManagerRatisUtils {
       } else if (ObjectType.BUCKET == type) {
         return new OMBucketAddAclRequest(omRequest);
       } else if (ObjectType.KEY == type) {
-        OMKeyAddAclRequest aclReq = new OMKeyAddAclRequest(omRequest);
-        BucketLayout bucketLayout = aclReq.getBucketLayout(ozoneManager);
-        if (BucketLayout.FILE_SYSTEM_OPTIMIZED
-            .equals(bucketLayout)) {
-          return new OMKeyAddAclRequestWithFSO(omRequest);
+        OMKeyAddAclRequest aclReq =
+            new OMKeyAddAclRequest(omRequest, ozoneManager);
+        if (aclReq.getBucketLayout().isFileSystemOptimized()) {
+          return new OMKeyAddAclRequestWithFSO(omRequest,
+              aclReq.getBucketLayout());
         }
         return aclReq;
       } else {
@@ -319,11 +328,11 @@ public final class OzoneManagerRatisUtils {
       } else if (ObjectType.BUCKET == type) {
         return new OMBucketRemoveAclRequest(omRequest);
       } else if (ObjectType.KEY == type) {
-        OMKeyRemoveAclRequest aclReq = new OMKeyRemoveAclRequest(omRequest);
-        BucketLayout bucketLayout = aclReq.getBucketLayout(ozoneManager);
-        if (BucketLayout.FILE_SYSTEM_OPTIMIZED
-            .equals(bucketLayout)) {
-          return new OMKeyRemoveAclRequestWithFSO(omRequest);
+        OMKeyRemoveAclRequest aclReq =
+            new OMKeyRemoveAclRequest(omRequest, ozoneManager);
+        if (aclReq.getBucketLayout().isFileSystemOptimized()) {
+          return new OMKeyRemoveAclRequestWithFSO(omRequest,
+              aclReq.getBucketLayout());
         }
         return aclReq;
       } else {
@@ -336,11 +345,11 @@ public final class OzoneManagerRatisUtils {
       } else if (ObjectType.BUCKET == type) {
         return new OMBucketSetAclRequest(omRequest);
       } else if (ObjectType.KEY == type) {
-        OMKeySetAclRequest aclReq = new OMKeySetAclRequest(omRequest);
-        BucketLayout bucketLayout = aclReq.getBucketLayout(ozoneManager);
-        if (BucketLayout.FILE_SYSTEM_OPTIMIZED
-            .equals(bucketLayout)) {
-          return new OMKeySetAclRequestWithFSO(omRequest);
+        OMKeySetAclRequest aclReq =
+            new OMKeySetAclRequest(omRequest, ozoneManager);
+        if (aclReq.getBucketLayout().isFileSystemOptimized()) {
+          return new OMKeySetAclRequestWithFSO(omRequest,
+              aclReq.getBucketLayout());
         }
         return aclReq;
       } else {
@@ -402,16 +411,6 @@ public final class OzoneManagerRatisUtils {
             newDBlocation, OzoneManager.LOG);
   }
 
-  /**
-   * Returns enabled/disabled file system optimized path property. A true value
-   * represents FSO path is enabled, false represents disabled.
-   *
-   * @return true or false.
-   */
-  public static boolean isBucketFSOptimized() {
-    return isBucketFSOptimized;
-  }
-
   /**
    * Get the local directory where ratis logs will be stored.
    */
@@ -441,90 +440,13 @@ public final class OzoneManagerRatisUtils {
     return snapshotDir;
   }
 
-  public static void checkLeaderStatus(RaftServerStatus raftServerStatus,
-      RaftPeerId raftPeerId) throws ServiceException {
-    switch (raftServerStatus) {
-    case LEADER_AND_READY: return;
-
-    case LEADER_AND_NOT_READY: throw createLeaderNotReadyException(raftPeerId);
-
-    case NOT_LEADER: throw createNotLeaderException(raftPeerId);
-
-    default: throw new IllegalStateException(
-        "Unknown Ratis Server state: " + raftServerStatus);
-    }
-  }
-
-  private static ServiceException createNotLeaderException(
-      RaftPeerId raftPeerId) {
-
-    // TODO: Set suggest leaderID. Right now, client is not using suggest
-    // leaderID. Need to fix this.
-
-    OMNotLeaderException notLeaderException =
-        new OMNotLeaderException(raftPeerId);
-
-    LOG.debug(notLeaderException.getMessage());
-
-    return new ServiceException(notLeaderException);
-  }
-
-  private static ServiceException createLeaderNotReadyException(
-      RaftPeerId raftPeerId) {
-
-    OMLeaderNotReadyException leaderNotReadyException =
-        new OMLeaderNotReadyException(raftPeerId.toString() + " is Leader " +
-            "but not ready to process request yet.");
-
-    LOG.debug(leaderNotReadyException.getMessage());
-
-    return new ServiceException(leaderNotReadyException);
-  }
-
-  /**
-   * All the client requests are executed through
-   * OzoneManagerStateMachine#runCommand function and ensures sequential
-   * execution path.
-   * Below is the call trace to perform OM client request operation:
-   * OzoneManagerStateMachine#applyTransaction ->
-   * OzoneManagerStateMachine#runCommand ->
-   * OzoneManagerRequestHandler#handleWriteRequest ->
-   * OzoneManagerRatisUtils#createClientRequest ->
-   * OzoneManagerRatisUtils#getOmBucketInfo ->
-   * omMetadataManager().getBucketTable().get(buckKey)
-   */
-
-  private static OmBucketInfo getOmBucketInfo(OzoneManager ozoneManager,
-      OmBucketInfo buckInfo, String volName, String buckName) {
-    String buckKey =
-        ozoneManager.getMetadataManager().getBucketKey(volName, buckName);
+  public static void checkLeaderStatus(OzoneManager ozoneManager)
+      throws ServiceException {
     try {
-      buckInfo =
-          ozoneManager.getMetadataManager().getBucketTable().get(buckKey);
-    } catch (IOException e) {
-      LOG.debug("Failed to get the value for the key: " + buckKey);
+      ozoneManager.checkLeaderStatus();
+    } catch (OMNotLeaderException | OMLeaderNotReadyException e) {
+      LOG.debug(e.getMessage());
+      throw new ServiceException(e);
     }
-    return buckInfo;
   }
-
-  private static BucketLayout getBucketLayout(OmBucketInfo buckInfo,
-      String volName, String buckName) {
-    if (buckInfo != null) {
-      return buckInfo.getBucketLayout();
-    } else {
-      LOG.error("Bucket not found: {}/{} ", volName, buckName);
-      // TODO: Handle bucket validation
-    }
-    return BucketLayout.LEGACY;
-  }
-
-  private static boolean isBucketFSOptimized(String volName, String buckName,
-      OzoneManager ozoneManager) {
-    BucketLayout bucketLayout = null;
-    OmBucketInfo buckInfo = null;
-    buckInfo = getOmBucketInfo(ozoneManager, buckInfo, volName, buckName);
-    bucketLayout = getBucketLayout(buckInfo, volName, buckName);
-    return BucketLayout.FILE_SYSTEM_OPTIMIZED.equals(bucketLayout);
-  }
-
 }
