@@ -29,6 +29,7 @@ import java.util.Map;
 
 import com.google.common.base.Optional;
 import com.google.common.base.Strings;
+import org.apache.commons.lang3.StringUtils;
 import org.apache.hadoop.hdds.client.RatisReplicationConfig;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.hdds.utils.db.BatchOperation;
@@ -47,6 +48,7 @@ import org.apache.hadoop.ozone.om.helpers.OmKeyLocationInfoGroup;
 import org.apache.hadoop.ozone.om.helpers.OzoneFSUtils;
 import org.apache.hadoop.ozone.om.helpers.OzoneFileStatus;
 import org.apache.hadoop.ozone.om.helpers.BucketLayout;
+import org.apache.hadoop.ozone.om.helpers.RepeatedOmKeyInfo;
 import org.jetbrains.annotations.NotNull;
 import org.jetbrains.annotations.Nullable;
 import org.slf4j.Logger;
@@ -54,8 +56,9 @@ import org.slf4j.LoggerFactory;
 
 import javax.annotation.Nonnull;
 
+import static org.apache.hadoop.ozone.om.OzoneManagerUtils.getBucketLayout;
+
 import static org.apache.hadoop.ozone.om.exceptions.OMException.ResultCodes.BUCKET_NOT_FOUND;
-import static org.apache.hadoop.ozone.om.exceptions.OMException.ResultCodes.KEY_NOT_FOUND;
 import static org.apache.hadoop.ozone.om.exceptions.OMException.ResultCodes.NOT_A_FILE;
 import static org.apache.hadoop.ozone.om.exceptions.OMException.ResultCodes.VOLUME_NOT_FOUND;
 
@@ -69,12 +72,14 @@ public final class OMFileRequest {
 
   private OMFileRequest() {
   }
+
   /**
    * Verify any files exist in the given path in the specified volume/bucket.
-   * @param omMetadataManager
-   * @param volumeName
-   * @param bucketName
-   * @param keyPath
+   *
+   * @param omMetadataManager OMMetadataManager
+   * @param volumeName        Volume Name
+   * @param bucketName        Bucket Name
+   * @param keyPath           Key Path
    * @return true - if file exist in the given path, else false.
    * @throws IOException
    */
@@ -100,7 +105,9 @@ public final class OMFileRequest {
       String dbDirKeyName = omMetadataManager.getOzoneDirKey(volumeName,
           bucketName, pathName);
 
-      if (omMetadataManager.getKeyTable().isExist(dbKeyName)) {
+      if (omMetadataManager.getKeyTable(
+          getBucketLayout(omMetadataManager, volumeName, bucketName))
+          .isExist(dbKeyName)) {
         // Found a file in the given path.
         // Check if this is actual file or a file in the given path
         if (dbKeyName.equals(fileNameFromDetails)) {
@@ -108,15 +115,18 @@ public final class OMFileRequest {
         } else {
           result = OMDirectoryResult.FILE_EXISTS_IN_GIVENPATH;
         }
-      } else if (omMetadataManager.getKeyTable().isExist(dbDirKeyName)) {
+      } else if (omMetadataManager.getKeyTable(
+          getBucketLayout(omMetadataManager, volumeName, bucketName))
+          .isExist(dbDirKeyName)) {
         // Found a directory in the given path.
         // Check if this is actual directory or a directory in the given path
         if (dbDirKeyName.equals(dirNameFromDetails)) {
           result = OMDirectoryResult.DIRECTORY_EXISTS;
         } else {
           result = OMDirectoryResult.DIRECTORY_EXISTS_IN_GIVENPATH;
-          inheritAcls = omMetadataManager.getKeyTable().get(dbDirKeyName)
-              .getAcls();
+          inheritAcls = omMetadataManager.getKeyTable(
+              getBucketLayout(omMetadataManager, volumeName, bucketName))
+              .get(dbDirKeyName).getAcls();
           LOG.trace("Acls inherited from parent " + dbDirKeyName + " are : "
               + inheritAcls);
         }
@@ -178,6 +188,11 @@ public final class OMFileRequest {
     String bucketKey = omMetadataManager.getBucketKey(volumeName, bucketName);
     OmBucketInfo omBucketInfo =
             omMetadataManager.getBucketTable().get(bucketKey);
+
+    final long volumeId = omMetadataManager.getVolumeId(volumeName);
+    final long bucketId = omMetadataManager.getBucketId(volumeName,
+            bucketName);
+
     // by default, inherit bucket ACLs
     List<OzoneAcl> inheritAcls = omBucketInfo.getAcls();
 
@@ -193,7 +208,7 @@ public final class OMFileRequest {
         // Add all the sub-dirs to the missing list except the leaf element.
         // For example, /vol1/buck1/a/b/c/d/e/f/file1.txt.
         // Assume /vol1/buck1/a/b/c exists, then add d, e, f into missing list.
-        if(elements.hasNext()){
+        if (elements.hasNext()) {
           // skips leaf node.
           missing.add(fileName);
         }
@@ -204,7 +219,7 @@ public final class OMFileRequest {
       // 1. Do lookup on directoryTable. If not exists goto next step.
       // 2. Do look on keyTable. If not exists goto next step.
       // 3. Add 'sub-dir' to missing parents list
-      String dbNodeName = omMetadataManager.getOzonePathKey(
+      String dbNodeName = omMetadataManager.getOzonePathKey(volumeId, bucketId,
               lastKnownParentId, fileName);
       OmDirectoryInfo omDirInfo = omMetadataManager.getDirectoryTable().
               get(dbNodeName);
@@ -224,7 +239,8 @@ public final class OMFileRequest {
         // Get parentID from the lastKnownParent. For any files, directly under
         // the bucket, the parent is the bucketID. Say, "/vol1/buck1/file1"
         // TODO: Need to add UT for this case along with OMFileCreateRequest.
-        if (omMetadataManager.getKeyTable().isExist(dbNodeName)) {
+        if (omMetadataManager.getKeyTable(BucketLayout.FILE_SYSTEM_OPTIMIZED)
+            .isExist(dbNodeName)) {
           if (elements.hasNext()) {
             // Found a file in the given key name.
             result = OMDirectoryResult.FILE_EXISTS_IN_GIVENPATH;
@@ -266,7 +282,7 @@ public final class OMFileRequest {
    * Includes the list of missing intermediate directories and
    * the directory search result code.
    */
-  public static class OMPathInfoWithFSO extends OMPathInfo{
+  public static class OMPathInfoWithFSO extends OMPathInfo {
     private String leafNodeName;
     private long lastKnownParentId;
     private long leafNodeObjectId;
@@ -396,19 +412,21 @@ public final class OMFileRequest {
       OMMetadataManager omMetadataManager, String volumeName,
       String bucketName, Optional<OmKeyInfo> keyInfo,
       Optional<List<OmKeyInfo>> parentInfoList,
-      long index) {
+      long index) throws IOException {
     for (OmKeyInfo parentInfo : parentInfoList.get()) {
-      omMetadataManager.getKeyTable().addCacheEntry(
-          new CacheKey<>(omMetadataManager.getOzoneKey(volumeName, bucketName,
-              parentInfo.getKeyName())),
-          new CacheValue<>(Optional.of(parentInfo), index));
+      omMetadataManager.getKeyTable(
+          getBucketLayout(omMetadataManager, volumeName, bucketName))
+          .addCacheEntry(new CacheKey<>(omMetadataManager
+          .getOzoneKey(volumeName, bucketName, parentInfo.getKeyName())),
+              new CacheValue<>(Optional.of(parentInfo), index));
     }
 
     if (keyInfo.isPresent()) {
-      omMetadataManager.getKeyTable().addCacheEntry(
-          new CacheKey<>(omMetadataManager.getOzoneKey(volumeName, bucketName,
-                  keyInfo.get().getKeyName())),
-          new CacheValue<>(keyInfo, index));
+      omMetadataManager.getKeyTable(
+          getBucketLayout(omMetadataManager, volumeName, bucketName))
+          .addCacheEntry(new CacheKey<>(omMetadataManager
+          .getOzoneKey(volumeName, bucketName, keyInfo.get().getKeyName())),
+              new CacheValue<>(keyInfo, index));
     }
   }
 
@@ -416,26 +434,30 @@ public final class OMFileRequest {
    * Adding directory info to the Table cache.
    *
    * @param omMetadataManager  OM Metadata Manager
-   * @param dirInfo            directory info
-   * @param missingParentInfos list of the parents to be added to DB
+   * @param volumeId           ID of the Volume
+   * @param bucketId           ID of the Bucket
    * @param trxnLogIndex       transaction log index
+   * @param missingParentInfos list of the parents to be added to DB
+   * @param dirInfo            directory info
    */
   public static void addDirectoryTableCacheEntries(
           OMMetadataManager omMetadataManager,
-          Optional<OmDirectoryInfo> dirInfo,
+          long volumeId, long bucketId, long trxnLogIndex,
           Optional<List<OmDirectoryInfo>> missingParentInfos,
-          long trxnLogIndex) {
+          Optional<OmDirectoryInfo> dirInfo) {
+
     for (OmDirectoryInfo subDirInfo : missingParentInfos.get()) {
       omMetadataManager.getDirectoryTable().addCacheEntry(
               new CacheKey<>(omMetadataManager.getOzonePathKey(
-                      subDirInfo.getParentObjectID(), subDirInfo.getName())),
+                      volumeId, bucketId, subDirInfo.getParentObjectID(),
+                      subDirInfo.getName())),
               new CacheValue<>(Optional.of(subDirInfo), trxnLogIndex));
     }
 
     if (dirInfo.isPresent()) {
       omMetadataManager.getDirectoryTable().addCacheEntry(
               new CacheKey<>(omMetadataManager.getOzonePathKey(
-                      dirInfo.get().getParentObjectID(),
+                      volumeId, bucketId, dirInfo.get().getParentObjectID(),
                       dirInfo.get().getName())),
               new CacheValue<>(dirInfo, trxnLogIndex));
     }
@@ -479,8 +501,9 @@ public final class OMFileRequest {
    * @param trxnLogIndex      transaction log index
    */
   public static void addFileTableCacheEntry(
-          OMMetadataManager omMetadataManager, String dbFileKey,
-          OmKeyInfo omFileInfo, String fileName, long trxnLogIndex) {
+      OMMetadataManager omMetadataManager, String dbFileKey,
+      OmKeyInfo omFileInfo, String fileName, long trxnLogIndex)
+      throws IOException {
 
     // New key format for the fileTable.
     // For example, the user given key path is '/a/b/c/d/e/file1', then in DB
@@ -488,11 +511,31 @@ public final class OMFileRequest {
     omFileInfo.setKeyName(fileName);
     omFileInfo.setFileName(fileName);
 
-    omMetadataManager.getKeyTable().addCacheEntry(
-            new CacheKey<>(dbFileKey),
+    BucketLayout bucketLayout =
+        getBucketLayout(omMetadataManager, omFileInfo.getVolumeName(),
+            omFileInfo.getBucketName());
+
+    omMetadataManager.getKeyTable(bucketLayout)
+        .addCacheEntry(new CacheKey<>(dbFileKey),
             new CacheValue<>(Optional.of(omFileInfo), trxnLogIndex));
   }
 
+  /**
+   * Updating the list of OmKeyInfo eligible for deleting blocks.
+   *
+   * @param omMetadataManager OM Metadata Manager
+   * @param dbDeletedKey      Ozone key in deletion table
+   * @param keysToDelete      Repeated OMKeyInfos
+   * @param trxnLogIndex      transaction log index
+   */
+  public static void addDeletedTableCacheEntry(
+          OMMetadataManager omMetadataManager, String dbDeletedKey,
+          RepeatedOmKeyInfo keysToDelete, long trxnLogIndex) {
+    omMetadataManager.getDeletedTable().addCacheEntry(
+            new CacheKey<>(dbDeletedKey),
+            new CacheValue<>(Optional.of(keysToDelete), trxnLogIndex));
+  }
+
   /**
    * Adding omKeyInfo to open file table.
    *
@@ -503,12 +546,10 @@ public final class OMFileRequest {
    * @throws IOException DB failure
    */
   public static void addToOpenFileTable(OMMetadataManager omMetadataMgr,
-                                        BatchOperation batchOp,
-                                        OmKeyInfo omFileInfo,
-                                        long openKeySessionID)
-          throws IOException {
+      BatchOperation batchOp, OmKeyInfo omFileInfo, long openKeySessionID,
+      long volumeId, long bucketId) throws IOException {
 
-    String dbOpenFileKey = omMetadataMgr.getOpenFileName(
+    String dbOpenFileKey = omMetadataMgr.getOpenFileName(volumeId, bucketId,
             omFileInfo.getParentObjectID(), omFileInfo.getFileName(),
             openKeySessionID);
 
@@ -527,12 +568,12 @@ public final class OMFileRequest {
    * @throws IOException DB failure
    */
   public static String addToOpenFileTable(OMMetadataManager omMetadataMgr,
-      BatchOperation batchOp, OmKeyInfo omFileInfo, String uploadID)
-          throws IOException {
+      BatchOperation batchOp, OmKeyInfo omFileInfo, String uploadID,
+      long volumeId, long bucketId) throws IOException {
 
-    String multipartFileKey = omMetadataMgr.getMultipartKey(
-            omFileInfo.getParentObjectID(), omFileInfo.getFileName(),
-            uploadID);
+    String multipartFileKey = omMetadataMgr.getMultipartKey(volumeId,
+            bucketId, omFileInfo.getParentObjectID(),
+            omFileInfo.getFileName(), uploadID);
 
     omMetadataMgr.getOpenKeyTable(BucketLayout.FILE_SYSTEM_OPTIMIZED)
         .putWithBatch(batchOp, multipartFileKey, omFileInfo);
@@ -550,15 +591,14 @@ public final class OMFileRequest {
    * @throws IOException
    */
   public static String addToFileTable(OMMetadataManager omMetadataMgr,
-                                    BatchOperation batchOp,
-                                    OmKeyInfo omFileInfo)
-          throws IOException {
+      BatchOperation batchOp, OmKeyInfo omFileInfo, long volumeId,
+      long bucketId) throws IOException {
 
-    String dbFileKey = omMetadataMgr.getOzonePathKey(
+    String dbFileKey = omMetadataMgr.getOzonePathKey(volumeId, bucketId,
             omFileInfo.getParentObjectID(), omFileInfo.getFileName());
 
-    omMetadataMgr.getKeyTable().putWithBatch(batchOp,
-            dbFileKey, omFileInfo);
+    omMetadataMgr.getKeyTable(BucketLayout.FILE_SYSTEM_OPTIMIZED)
+        .putWithBatch(batchOp, dbFileKey, omFileInfo);
     return dbFileKey;
   }
 
@@ -584,7 +624,9 @@ public final class OMFileRequest {
           omMetadataMgr.getOpenKeyTable(BucketLayout.FILE_SYSTEM_OPTIMIZED)
               .get(dbOpenFileKey);
     } else {
-      dbOmKeyInfo = omMetadataMgr.getKeyTable().get(dbOpenFileKey);
+      dbOmKeyInfo =
+          omMetadataMgr.getKeyTable(BucketLayout.FILE_SYSTEM_OPTIMIZED)
+              .get(dbOpenFileKey);
     }
 
     // DB OMKeyInfo will store only fileName into keyName field. This
@@ -617,6 +659,7 @@ public final class OMFileRequest {
 
     Path keyPath = Paths.get(keyName);
     Iterator<Path> elements = keyPath.iterator();
+    final long volumeId = omMetadataMgr.getVolumeId(volumeName);
     String bucketKey = omMetadataMgr.getBucketKey(volumeName, bucketName);
     OmBucketInfo omBucketInfo =
             omMetadataMgr.getBucketTable().get(bucketKey);
@@ -632,6 +675,7 @@ public final class OMFileRequest {
       // 2. If there is no dir exists for the leaf node component 'file1.txt'
       // then do look it on fileTable.
       String dbNodeName = omMetadataMgr.getOzonePathKey(
+              volumeId, omBucketInfo.getObjectID(),
               lastKnownParentId, fileName);
       omDirInfo = omMetadataMgr.getDirectoryTable().get(dbNodeName);
 
@@ -686,8 +730,8 @@ public final class OMFileRequest {
     builder.setObjectID(dirInfo.getObjectID());
     builder.setUpdateID(dirInfo.getUpdateID());
     builder.setFileName(dirInfo.getName());
-    builder.setReplicationConfig(new RatisReplicationConfig(
-            HddsProtos.ReplicationFactor.ONE));
+    builder.setReplicationConfig(RatisReplicationConfig
+        .getInstance(HddsProtos.ReplicationFactor.ONE));
     builder.setOmKeyLocationInfos(Collections.singletonList(
             new OmKeyLocationInfoGroup(0, new ArrayList<>())));
     return builder.build();
@@ -715,7 +759,7 @@ public final class OMFileRequest {
    * @param keyInfo omKeyInfo
    * @return omDirectoryInfo object
    */
-  public static OmDirectoryInfo getDirectoryInfo(OmKeyInfo keyInfo){
+  public static OmDirectoryInfo getDirectoryInfo(OmKeyInfo keyInfo) {
     OmDirectoryInfo.Builder builder = new OmDirectoryInfo.Builder();
     builder.setParentObjectID(keyInfo.getParentObjectID());
     builder.setAcls(keyInfo.getAcls());
@@ -794,7 +838,7 @@ public final class OMFileRequest {
               "Failed to rename %s to %s, %s doesn't exist", fromKeyName,
               toKeyName, toKeyParentDir),
               OMException.ResultCodes.KEY_RENAME_ERROR);
-    } else if (toKeyParentDirStatus.isFile()){
+    } else if (toKeyParentDirStatus.isFile()) {
       throw new OMException(String.format(
               "Failed to rename %s to %s, %s is a file", fromKeyName, toKeyName,
               toKeyParentDir), OMException.ResultCodes.KEY_RENAME_ERROR);
@@ -805,15 +849,16 @@ public final class OMFileRequest {
   /**
    * Check if there are any sub path exist for the given user key path.
    *
-   * @param omKeyInfo om key path
+   * @param omKeyInfo       om key path
    * @param metaMgr   OMMetadataManager
    * @return true if there are any sub path, false otherwise
    * @throws IOException DB exception
    */
   public static boolean hasChildren(OmKeyInfo omKeyInfo,
-      OMMetadataManager metaMgr) throws IOException {
+                                    OMMetadataManager metaMgr)
+      throws IOException {
     return checkSubDirectoryExists(omKeyInfo, metaMgr) ||
-            checkSubFileExists(omKeyInfo, metaMgr);
+        checkSubFileExists(omKeyInfo, metaMgr);
   }
 
   private static boolean checkSubDirectoryExists(OmKeyInfo omKeyInfo,
@@ -836,25 +881,37 @@ public final class OMFileRequest {
       }
     }
 
+    final long volumeId = metaMgr.getVolumeId(omKeyInfo.getVolumeName());
+    final long bucketId = metaMgr.getBucketId(omKeyInfo.getVolumeName(),
+            omKeyInfo.getBucketName());
     // Check dirTable entries for any sub paths.
-    String seekDirInDB = metaMgr.getOzonePathKey(omKeyInfo.getObjectID(), "");
-    TableIterator<String, ? extends Table.KeyValue<String, OmDirectoryInfo>>
-            iterator = dirTable.iterator();
+    String seekDirInDB = metaMgr.getOzonePathKey(volumeId, bucketId,
+            omKeyInfo.getObjectID(), "");
+    try (TableIterator<String, ? extends
+        Table.KeyValue<String, OmDirectoryInfo>>
+            iterator = dirTable.iterator()) {
 
-    iterator.seek(seekDirInDB);
+      iterator.seek(seekDirInDB);
+
+      if (iterator.hasNext()) {
+        Table.KeyValue<String, OmDirectoryInfo> entry = iterator.next();
+        OmDirectoryInfo dirInfo = entry.getValue();
+        return isImmediateChild(dirInfo.getParentObjectID(),
+            omKeyInfo.getObjectID());
+      }
 
-    if (iterator.hasNext()) {
-      OmDirectoryInfo dirInfo = iterator.value().getValue();
-      return isImmediateChild(dirInfo.getParentObjectID(),
-              omKeyInfo.getObjectID());
     }
     return false; // no sub paths found
   }
 
   private static boolean checkSubFileExists(OmKeyInfo omKeyInfo,
-      OMMetadataManager metaMgr) throws IOException {
+                                            OMMetadataManager metaMgr)
+      throws IOException {
+    BucketLayout bucketLayout =
+        getBucketLayout(metaMgr, omKeyInfo.getVolumeName(),
+            omKeyInfo.getBucketName());
     // Check all fileTable cache for any sub paths.
-    Table fileTable = metaMgr.getKeyTable();
+    Table fileTable = metaMgr.getKeyTable(bucketLayout);
     Iterator<Map.Entry<CacheKey<String>, CacheValue<OmKeyInfo>>>
             cacheIter = fileTable.cacheIterator();
 
@@ -871,18 +928,23 @@ public final class OMFileRequest {
       }
     }
 
+    final long volumeId = metaMgr.getVolumeId(omKeyInfo.getVolumeName());
+    final long bucketId = metaMgr.getBucketId(omKeyInfo.getVolumeName(),
+            omKeyInfo.getBucketName());
     // Check fileTable entries for any sub paths.
-    String seekFileInDB = metaMgr.getOzonePathKey(
+    String seekFileInDB = metaMgr.getOzonePathKey(volumeId, bucketId,
             omKeyInfo.getObjectID(), "");
-    TableIterator<String, ? extends Table.KeyValue<String, OmKeyInfo>>
-            iterator = fileTable.iterator();
+    try (TableIterator<String, ? extends Table.KeyValue<String, OmKeyInfo>>
+            iterator = fileTable.iterator()) {
 
-    iterator.seek(seekFileInDB);
+      iterator.seek(seekFileInDB);
 
-    if (iterator.hasNext()) {
-      OmKeyInfo fileInfo = iterator.value().getValue();
-      return isImmediateChild(fileInfo.getParentObjectID(),
-              omKeyInfo.getObjectID()); // found a sub path file
+      if (iterator.hasNext()) {
+        Table.KeyValue<String, OmKeyInfo> entry = iterator.next();
+        OmKeyInfo fileInfo = entry.getValue();
+        return isImmediateChild(fileInfo.getParentObjectID(),
+            omKeyInfo.getObjectID()); // found a sub path file
+      }
     }
     return false; // no sub paths found
   }
@@ -891,6 +953,27 @@ public final class OMFileRequest {
     return parentId == ancestorId;
   }
 
+  /**
+   * Get parent id for the user given path.
+   *
+   * @param volumeId       volume id
+   * @param bucketId       bucket id
+   * @param pathComponents fie path elements
+   * @param keyName        user given key name
+   * @param omMetadataManager   om metadata manager
+   * @return lastKnownParentID
+   * @throws IOException DB failure or parent not exists in DirectoryTable
+   */
+  public static long getParentID(long volumeId, long bucketId,
+                                 Iterator<Path> pathComponents,
+                                 String keyName,
+                                 OMMetadataManager omMetadataManager)
+      throws IOException {
+
+    return getParentID(volumeId, bucketId, pathComponents, keyName,
+            omMetadataManager, null);
+  }
+
   /**
    * Get parent id for the user given path.
    *
@@ -901,22 +984,28 @@ public final class OMFileRequest {
    * @return lastKnownParentID
    * @throws IOException DB failure or parent not exists in DirectoryTable
    */
-  public static long getParentID(long bucketId, Iterator<Path> pathComponents,
-      String keyName, OMMetadataManager omMetadataManager) throws IOException {
+  public static long getParentID(long volumeId, long bucketId,
+      Iterator<Path> pathComponents, String keyName,
+      OMMetadataManager omMetadataManager, String errMsg)
+      throws IOException {
 
     long lastKnownParentId = bucketId;
 
     // If no sub-dirs then bucketID is the root/parent.
-    if(!pathComponents.hasNext()){
+    if (!pathComponents.hasNext()) {
       return bucketId;
     }
-
+    if (StringUtils.isBlank(errMsg)) {
+      errMsg = "Failed to find parent directory of " + keyName;
+    }
     OmDirectoryInfo omDirectoryInfo;
     while (pathComponents.hasNext()) {
       String nodeName = pathComponents.next().toString();
       boolean reachedLastPathComponent = !pathComponents.hasNext();
       String dbNodeName =
-              omMetadataManager.getOzonePathKey(lastKnownParentId, nodeName);
+              omMetadataManager.getOzonePathKey(volumeId, bucketId,
+                      lastKnownParentId, nodeName);
+
 
       omDirectoryInfo = omMetadataManager.
               getDirectoryTable().get(dbNodeName);
@@ -931,8 +1020,8 @@ public final class OMFileRequest {
         // One of the sub-dir doesn't exists in DB. Immediate parent should
         // exists for committing the key, otherwise will fail the operation.
         if (!reachedLastPathComponent) {
-          throw new OMException("Failed to find parent directory of "
-                  + keyName + " in DirectoryTable", KEY_NOT_FOUND);
+          throw new OMException(errMsg,
+              OMException.ResultCodes.DIRECTORY_NOT_FOUND);
         }
         break;
       }
@@ -941,6 +1030,30 @@ public final class OMFileRequest {
     return lastKnownParentId;
   }
 
+  /**
+   * Get parent ID for the user given keyName.
+   *
+   * @param omMetadataManager
+   * @param volumeName        - volume name.
+   * @param bucketName        - bucket name.
+   * @param keyName           - key name.
+   * @return
+   * @throws IOException
+   */
+  public static long getParentId(OMMetadataManager omMetadataManager,
+                                 String volumeName, String bucketName,
+                                 String keyName)
+      throws IOException {
+
+
+    final long volumeId = omMetadataManager.getVolumeId(volumeName);
+    final long bucketId = omMetadataManager.getBucketId(volumeName,
+            bucketName);
+    Iterator<Path> pathComponents = Paths.get(keyName).iterator();
+    return OMFileRequest.getParentID(volumeId, bucketId,
+            pathComponents, keyName, omMetadataManager);
+  }
+
   /**
    * Validates volume and bucket existence.
    *
