@@ -55,6 +55,8 @@ public abstract class OMKeyAclRequest extends OMClientRequest {
   private static final Logger LOG = LoggerFactory
       .getLogger(OMKeyAclRequest.class);
 
+  private BucketLayout bucketLayout = BucketLayout.DEFAULT;
+
   public OMKeyAclRequest(OMRequest omRequest) {
     super(omRequest);
   }
@@ -86,7 +88,7 @@ public abstract class OMKeyAclRequest extends OMClientRequest {
 
       // check Acl
       if (ozoneManager.getAclsEnabled()) {
-        checkAcls(ozoneManager, OzoneObj.ResourceType.VOLUME,
+        checkAcls(ozoneManager, OzoneObj.ResourceType.KEY,
             OzoneObj.StoreType.OZONE, IAccessAuthorizer.ACLType.WRITE_ACL,
             volume, bucket, key);
       }
@@ -95,7 +97,8 @@ public abstract class OMKeyAclRequest extends OMClientRequest {
               bucket);
 
       String dbKey = omMetadataManager.getOzoneKey(volume, bucket, key);
-      omKeyInfo = omMetadataManager.getKeyTable().get(dbKey);
+      omKeyInfo = omMetadataManager.getKeyTable(getBucketLayout())
+          .get(dbKey);
 
       if (omKeyInfo == null) {
         throw new OMException(OMException.ResultCodes.KEY_NOT_FOUND);
@@ -109,7 +112,8 @@ public abstract class OMKeyAclRequest extends OMClientRequest {
       if (getOmRequest().getAddAclRequest().hasObj() && operationResult) {
         modificationTime = getOmRequest().getAddAclRequest()
             .getModificationTime();
-      } else if (getOmRequest().getSetAclRequest().hasObj() && operationResult){
+      } else if (getOmRequest().getSetAclRequest().hasObj()
+          && operationResult) {
         modificationTime = getOmRequest().getSetAclRequest()
             .getModificationTime();
       } else if (getOmRequest().getRemoveAclRequest().hasObj()
@@ -120,9 +124,9 @@ public abstract class OMKeyAclRequest extends OMClientRequest {
       omKeyInfo.setModificationTime(modificationTime);
 
       // update cache.
-      omMetadataManager.getKeyTable().addCacheEntry(
-          new CacheKey<>(dbKey),
-          new CacheValue<>(Optional.of(omKeyInfo), trxnLogIndex));
+      omMetadataManager.getKeyTable(getBucketLayout())
+          .addCacheEntry(new CacheKey<>(dbKey),
+              new CacheValue<>(Optional.of(omKeyInfo), trxnLogIndex));
 
       omClientResponse = onSuccess(omResponse, omKeyInfo, operationResult);
       result = Result.SUCCESS;
@@ -153,9 +157,8 @@ public abstract class OMKeyAclRequest extends OMClientRequest {
    */
   abstract String getPath();
 
-  public BucketLayout getBucketLayout(OzoneManager ozoneManager) {
-    BucketLayout bucketLayout = BucketLayout.LEGACY;
-    OmBucketInfo buckInfo = null;
+  public void initializeBucketLayout(OzoneManager ozoneManager) {
+    OmBucketInfo buckInfo;
     try {
       ObjectParser objectParser = new ObjectParser(getPath(),
           OzoneManagerProtocolProtos.OzoneObj.ObjectType.KEY);
@@ -171,7 +174,8 @@ public abstract class OMKeyAclRequest extends OMClientRequest {
             ozoneManager.getMetadataManager().getBucketTable().get(buckKey);
         if (buckInfo == null) {
           LOG.error("Bucket not found: {}/{} ", volume, bucket);
-          return BucketLayout.LEGACY;
+          // defaulting to BucketLayout.DEFAULT
+          return;
         }
         bucketLayout = buckInfo.getBucketLayout();
       } catch (IOException e) {
@@ -180,8 +184,9 @@ public abstract class OMKeyAclRequest extends OMClientRequest {
     } catch (OMException ome) {
       LOG.error("Invalid Path: " + getPath(), ome);
       // Handle exception
+      // defaulting to BucketLayout.DEFAULT
+      return;
     }
-    return bucketLayout;
   }
 
   /**
@@ -217,7 +222,8 @@ public abstract class OMKeyAclRequest extends OMClientRequest {
    */
   OMClientResponse onFailure(OMResponse.Builder omResponse,
       IOException exception) {
-    return new OMKeyAclResponse(createErrorOMResponse(omResponse, exception));
+    return new OMKeyAclResponse(createErrorOMResponse(omResponse, exception),
+        getBucketLayout());
   }
 
   /**
@@ -237,5 +243,13 @@ public abstract class OMKeyAclRequest extends OMClientRequest {
    * @param trxnLogIndex
    */
   abstract boolean apply(OmKeyInfo omKeyInfo, long trxnLogIndex);
+
+  public void setBucketLayout(BucketLayout bucketLayout) {
+    this.bucketLayout = bucketLayout;
+  }
+
+  public BucketLayout getBucketLayout() {
+    return bucketLayout;
+  }
 }
 
