@@ -28,11 +28,19 @@ import org.apache.hadoop.ozone.OzoneConsts;
 import org.apache.hadoop.ozone.audit.AuditLogger;
 import org.apache.hadoop.ozone.audit.OMAction;
 import org.apache.hadoop.ozone.om.OzoneManager;
+import org.apache.hadoop.ozone.om.exceptions.OMException;
+import org.apache.hadoop.ozone.om.helpers.BucketLayout;
 import org.apache.hadoop.ozone.om.helpers.OmKeyInfo;
 import org.apache.hadoop.ozone.om.ratis.utils.OzoneManagerDoubleBufferHelper;
+import org.apache.hadoop.ozone.om.request.util.ObjectParser;
 import org.apache.hadoop.ozone.om.request.util.OmResponseUtil;
+import org.apache.hadoop.ozone.om.request.validation.RequestFeatureValidator;
+import org.apache.hadoop.ozone.om.request.validation.RequestProcessingPhase;
+import org.apache.hadoop.ozone.om.request.validation.ValidationCondition;
+import org.apache.hadoop.ozone.om.request.validation.ValidationContext;
 import org.apache.hadoop.ozone.om.response.key.acl.OMKeyAclResponse;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos;
+import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.Type;
 import org.apache.hadoop.ozone.security.acl.OzoneObj;
 import org.apache.hadoop.ozone.security.acl.OzoneObjInfo;
 import org.apache.hadoop.util.Time;
@@ -69,7 +77,7 @@ public class OMKeyAddAclRequest extends OMKeyAclRequest {
   private List<OzoneAcl> ozoneAcls;
   private OzoneObj obj;
 
-  public OMKeyAddAclRequest(OMRequest omRequest) {
+  public OMKeyAddAclRequest(OMRequest omRequest, OzoneManager ozoneManager) {
     super(omRequest);
     OzoneManagerProtocolProtos.AddAclRequest addAclRequest =
         getOmRequest().getAddAclRequest();
@@ -77,6 +85,8 @@ public class OMKeyAddAclRequest extends OMKeyAclRequest {
     path = obj.getPath();
     ozoneAcls = Lists.newArrayList(
         OzoneAcl.fromProtobuf(addAclRequest.getAcl()));
+
+    initializeBucketLayout(ozoneManager);
   }
 
   @Override
@@ -146,5 +156,37 @@ public class OMKeyAddAclRequest extends OMKeyAclRequest {
         omDoubleBufferHelper);
   }
 
+  /**
+   * Validates Add ACL requests.
+   * We do not want to allow older clients to create add ACL requests
+   * for keys that are present in buckets which use non LEGACY layouts.
+   *
+   * @param req - the request to validate
+   * @param ctx - the validation context
+   * @return the validated request
+   * @throws OMException if the request is invalid
+   */
+  @RequestFeatureValidator(
+      conditions = ValidationCondition.OLDER_CLIENT_REQUESTS,
+      processingPhase = RequestProcessingPhase.PRE_PROCESS,
+      requestType = Type.AddAcl
+  )
+  public static OMRequest blockAddAclWithBucketLayoutFromOldClient(
+      OMRequest req, ValidationContext ctx) throws IOException {
+    if (req.getAddAclRequest().hasObj()) {
+      OzoneObj obj = OzoneObjInfo.fromProtobuf(req.getAddAclRequest().getObj());
+      String path = obj.getPath();
+
+      ObjectParser objectParser = new ObjectParser(path,
+          OzoneManagerProtocolProtos.OzoneObj.ObjectType.KEY);
+
+      String volume = objectParser.getVolume();
+      String bucket = objectParser.getBucket();
+
+      BucketLayout bucketLayout = ctx.getBucketLayout(volume, bucket);
+      bucketLayout.validateSupportedOperation();
+    }
+    return req;
+  }
 }
 
