@@ -100,6 +100,16 @@ public class OMVolumeDeleteRequest extends OMVolumeRequest {
 
       OmVolumeArgs omVolumeArgs = getVolumeInfo(omMetadataManager, volume);
 
+      // Check reference count
+      final long volRefCount = omVolumeArgs.getRefCount();
+      if (volRefCount != 0L) {
+        LOG.debug("volume: {} has a non-zero ref count. won't delete", volume);
+        throw new OMException("Volume reference count is not zero (" +
+            volRefCount + "). Ozone features are enabled on this volume. " +
+            "Try `ozone tenant delete <tenantId>` first.",
+            OMException.ResultCodes.VOLUME_IS_REFERENCED);
+      }
+
       owner = omVolumeArgs.getOwnerName();
       acquiredUserLock = omMetadataManager.getLock().acquireWriteLock(USER_LOCK,
           owner);
