@@ -28,6 +28,8 @@ import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos
     .OMResponse;
 import org.apache.hadoop.hdds.utils.db.BatchOperation;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 /**
  * Interface for OM Responses, each OM response should implement this interface.
@@ -37,6 +39,9 @@ public abstract class OMClientResponse {
   private OMResponse omResponse;
   private CompletableFuture<Void> flushFuture = null;
 
+  private static final Logger LOG =
+      LoggerFactory.getLogger(OMClientResponse.class);
+
   public OMClientResponse(OMResponse omResponse) {
     Preconditions.checkNotNull(omResponse);
     this.omResponse = omResponse;
