@@ -30,6 +30,7 @@ import org.apache.hadoop.hdds.tracing.TracingUtil;
 import org.apache.hadoop.hdds.utils.ProtocolMessageMetrics;
 import org.apache.hadoop.ozone.OmUtils;
 import org.apache.hadoop.ozone.om.OzoneManager;
+import org.apache.hadoop.ozone.om.exceptions.OMException;
 import org.apache.hadoop.ozone.om.exceptions.OMLeaderNotReadyException;
 import org.apache.hadoop.ozone.om.exceptions.OMNotLeaderException;
 import org.apache.hadoop.ozone.om.protocolPB.OzoneManagerProtocolPB;
@@ -38,6 +39,8 @@ import org.apache.hadoop.ozone.om.ratis.OzoneManagerRatisServer;
 import org.apache.hadoop.ozone.om.ratis.OzoneManagerRatisServer.RaftServerStatus;
 import org.apache.hadoop.ozone.om.ratis.utils.OzoneManagerRatisUtils;
 import org.apache.hadoop.ozone.om.request.OMClientRequest;
+import org.apache.hadoop.ozone.om.request.validation.RequestValidations;
+import org.apache.hadoop.ozone.om.request.validation.ValidationContext;
 import org.apache.hadoop.ozone.om.response.OMClientResponse;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.OMRequest;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.OMResponse;
@@ -45,6 +48,7 @@ import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.OMRespo
 import com.google.protobuf.ProtocolMessageEnum;
 import com.google.protobuf.RpcController;
 import com.google.protobuf.ServiceException;
+import org.apache.hadoop.ozone.security.S3SecurityUtil;
 import org.apache.ratis.protocol.RaftPeerId;
 import org.apache.ratis.util.ExitUtils;
 import org.slf4j.Logger;
@@ -59,6 +63,9 @@ public class OzoneManagerProtocolServerSideTranslatorPB implements
     OzoneManagerProtocolPB {
   private static final Logger LOG = LoggerFactory
       .getLogger(OzoneManagerProtocolServerSideTranslatorPB.class);
+  private static final String OM_REQUESTS_PACKAGE = 
+      "org.apache.hadoop.ozone";
+  
   private final OzoneManagerRatisServer omRatisServer;
   private final RequestHandler handler;
   private final boolean isRatisEnabled;
@@ -67,6 +74,7 @@ public class OzoneManagerProtocolServerSideTranslatorPB implements
   private final AtomicLong transactionIndex;
   private final OzoneProtocolMessageDispatcher<OMRequest, OMResponse,
       ProtocolMessageEnum> dispatcher;
+  private final RequestValidations requestValidations;
 
   /**
    * Constructs an instance of the server handler.
@@ -108,43 +116,96 @@ public class OzoneManagerProtocolServerSideTranslatorPB implements
     }
     this.omRatisServer = ratisServer;
     dispatcher = new OzoneProtocolMessageDispatcher<>("OzoneProtocol",
-        metrics, LOG);
+        metrics, LOG, OMPBHelper::processForDebug, OMPBHelper::processForDebug);
+    // TODO: make this injectable for testing...
+    requestValidations =
+        new RequestValidations()
+            .fromPackage(OM_REQUESTS_PACKAGE)
+            .withinContext(
+                ValidationContext.of(ozoneManager.getVersionManager(),
+                    ozoneManager.getMetadataManager()))
+            .load();
   }
 
   /**
-   * Submit requests to Ratis server for OM HA implementation.
-   * TODO: Once HA is implemented fully, we should have only one server side
-   * translator for OM protocol.
+   * Submit mutating requests to Ratis server in OM, and process read requests.
    */
   @Override
   public OMResponse submitRequest(RpcController controller,
       OMRequest request) throws ServiceException {
+    OMRequest validatedRequest;
+    try {
+      validatedRequest = requestValidations.validateRequest(request);
+    } catch (Exception e) {
+      if (e instanceof OMException) {
+        return createErrorResponse(request, (OMException) e);
+      }
+      throw new ServiceException(e);
+    }
 
-    return dispatcher.processRequest(request, this::processRequest,
+    OMResponse response = 
+        dispatcher.processRequest(validatedRequest, this::processRequest,
         request.getCmdType(), request.getTraceID());
+    
+    return requestValidations.validateResponse(request, response);
   }
 
   private OMResponse processRequest(OMRequest request) throws
       ServiceException {
-    RaftServerStatus raftServerStatus;
-    if (isRatisEnabled) {
-      // Check if the request is a read only request
-      if (OmUtils.isReadOnly(request)) {
-        return submitReadRequestToOM(request);
-      } else {
-        checkLeaderStatus();
+    OMClientRequest omClientRequest = null;
+
+    boolean s3Auth = false;
+    try {
+      if (request.hasS3Authentication()) {
+        OzoneManager.setS3Auth(request.getS3Authentication());
         try {
-          OMClientRequest omClientRequest =
-              createClientRequest(request, ozoneManager);
-          request = omClientRequest.preExecute(ozoneManager);
+          s3Auth = true;
+          // If Request has S3Authentication validate S3 credentials
+          // if current OM is leader and then proceed with
+          // processing the request.
+          S3SecurityUtil.validateS3Credential(request, ozoneManager);
         } catch (IOException ex) {
-          // As some of the preExecute returns error. So handle here.
+          // If validate credentials fail return error OM Response.
           return createErrorResponse(request, ex);
         }
-        return submitRequestToRatis(request);
       }
-    } else {
-      return submitRequestDirectlyToOM(request);
+      if (isRatisEnabled) {
+        // Check if the request is a read only request
+        if (OmUtils.isReadOnly(request)) {
+          return submitReadRequestToOM(request);
+        } else {
+          // To validate credentials we have already verified leader status.
+          // This will skip of checking leader status again if request has
+          // S3Auth.
+          if (!s3Auth) {
+            OzoneManagerRatisUtils.checkLeaderStatus(ozoneManager);
+          }
+          try {
+            omClientRequest =
+                createClientRequest(request, ozoneManager);
+            // TODO: Note: Due to HDDS-6055, createClientRequest() could now
+            //  return null, which triggered the findbugs warning.
+            //  Added the assertion.
+            assert (omClientRequest != null);
+            request = omClientRequest.preExecute(ozoneManager);
+          } catch (IOException ex) {
+            // As some of the preExecute returns error. So handle here.
+            if (omClientRequest != null) {
+              omClientRequest.handleRequestFailure(ozoneManager);
+            }
+            return createErrorResponse(request, ex);
+          }
+          OMResponse response = submitRequestToRatis(request);
+          if (!response.getSuccess()) {
+            omClientRequest.handleRequestFailure(ozoneManager);
+          }
+          return response;
+        }
+      } else {
+        return submitRequestDirectlyToOM(request);
+      }
+    } finally {
+      OzoneManager.setS3Auth(null);
     }
   }
 
@@ -206,7 +267,8 @@ public class OzoneManagerProtocolServerSideTranslatorPB implements
   /**
    * Submits request directly to OM.
    */
-  private OMResponse submitRequestDirectlyToOM(OMRequest request) {
+  private OMResponse submitRequestDirectlyToOM(OMRequest request) throws
+      ServiceException {
     OMClientResponse omClientResponse = null;
     long index = 0L;
     try {
@@ -234,15 +296,11 @@ public class OzoneManagerProtocolServerSideTranslatorPB implements
       String errorMessage = "Got error during waiting for flush to be " +
           "completed for " + "request" + request.toString();
       ExitUtils.terminate(1, errorMessage, ex, LOG);
+      Thread.currentThread().interrupt();
     }
     return omClientResponse.getOMResponse();
   }
 
-  private void checkLeaderStatus() throws ServiceException {
-    OzoneManagerRatisUtils.checkLeaderStatus(omRatisServer.checkLeaderStatus(),
-        omRatisServer.getRaftPeerId());
-  }
-
   /**
    * Create OMResponse from the specified OMRequest and exception.
    *
@@ -271,4 +329,8 @@ public class OzoneManagerProtocolServerSideTranslatorPB implements
       ozoneManagerDoubleBuffer.stop();
     }
   }
+
+  public static Logger getLog() {
+    return LOG;
+  }
 }
