@@ -76,6 +76,9 @@ public class ScmBlockLocationTestingClient implements ScmBlockLocationProtocol {
   private final int failCallsFrequency;
   private int currentCall = 0;
 
+  // The number of blocks deleted by this client
+  private int numBlocksDeleted = 0;
+
   /**
    * If ClusterID or SCMID is blank a per instance ID is generated.
    *
@@ -137,7 +140,7 @@ public class ScmBlockLocationTestingClient implements ScmBlockLocationProtocol {
         .setState(Pipeline.PipelineState.OPEN)
         .setId(PipelineID.randomId())
         .setReplicationConfig(
-            new StandaloneReplicationConfig(ReplicationFactor.ONE))
+            StandaloneReplicationConfig.getInstance(ReplicationFactor.ONE))
         .setNodes(dns)
         .build();
     return pipeline;
@@ -155,6 +158,7 @@ public class ScmBlockLocationTestingClient implements ScmBlockLocationProtocol {
         switch (this.failCallsFrequency) {
         case 0:
           result = success;
+          numBlocksDeleted++;
           break;
         case 1:
           result = unknownFailure;
@@ -164,6 +168,7 @@ public class ScmBlockLocationTestingClient implements ScmBlockLocationProtocol {
             result = unknownFailure;
           } else {
             result = success;
+            numBlocksDeleted++;
           }
         }
         blockResultList.add(new DeleteBlockResult(blockKey, result));
@@ -194,6 +199,13 @@ public class ScmBlockLocationTestingClient implements ScmBlockLocationProtocol {
     return null;
   }
 
+  /**
+   * Return the number of blocks puesdo deleted by this testing client.
+   */
+  public int getNumberOfDeletedBlocks() {
+    return numBlocksDeleted;
+  }
+
   @Override
   public void close() throws IOException {
 
