@@ -28,6 +28,7 @@ import java.util.Arrays;
 import java.util.Collections;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import java.util.UUID;
 
 import com.google.common.base.Optional;
@@ -64,22 +65,26 @@ import org.apache.hadoop.ozone.om.helpers.OmMultipartUploadList;
 import org.apache.hadoop.ozone.om.helpers.OmMultipartUploadListParts;
 import org.apache.hadoop.ozone.om.helpers.OmVolumeArgs;
 import org.apache.hadoop.ozone.om.helpers.OzoneFileStatus;
-import org.apache.hadoop.ozone.om.request.TestOMRequestUtils;
+import org.apache.hadoop.ozone.om.protocol.OzoneManagerProtocol;
+import org.apache.hadoop.ozone.om.request.OMRequestTestUtils;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos;
-import org.apache.hadoop.ozone.security.OzoneBlockTokenSecretManager;
 import org.apache.ozone.test.GenericTestUtils;
 
 import org.apache.hadoop.util.Time;
+import org.apache.ratis.util.ExitUtils;
 import org.junit.After;
 import org.junit.Assert;
 import org.junit.Before;
+import org.junit.BeforeClass;
 import org.junit.Test;
 import org.mockito.Mockito;
 
+import static com.google.common.collect.Sets.newHashSet;
 import static java.util.Collections.emptyList;
 import static java.util.Collections.singletonList;
 import static java.util.Comparator.comparing;
 import static java.util.stream.Collectors.toList;
+import static org.mockito.Mockito.times;
 import static org.mockito.Mockito.verify;
 import static org.mockito.Mockito.when;
 
@@ -89,7 +94,7 @@ import static org.mockito.Mockito.when;
 public class TestKeyManagerUnit {
 
   private OzoneConfiguration configuration;
-  private OmMetadataManagerImpl metadataManager;
+  private OMMetadataManager metadataManager;
   private StorageContainerLocationProtocol containerClient;
   private KeyManagerImpl keyManager;
 
@@ -97,25 +102,35 @@ public class TestKeyManagerUnit {
   private File testDir;
   private ScmBlockLocationProtocol blockClient;
 
+  private OzoneManagerProtocol writeClient;
+  private OzoneManager om;
+
+  @BeforeClass
+  public static void setup() {
+    ExitUtils.disableSystemExit();
+  }
+  
   @Before
-  public void setup() throws IOException {
+  public void init() throws Exception {
     configuration = new OzoneConfiguration();
     testDir = GenericTestUtils.getRandomizedTestDir();
     configuration.set(HddsConfigKeys.OZONE_METADATA_DIRS,
         testDir.toString());
-    metadataManager = new OmMetadataManagerImpl(configuration);
     containerClient = Mockito.mock(StorageContainerLocationProtocol.class);
     blockClient = Mockito.mock(ScmBlockLocationProtocol.class);
-    keyManager = new KeyManagerImpl(
-        blockClient, containerClient, metadataManager, configuration,
-        "omtest", Mockito.mock(OzoneBlockTokenSecretManager.class));
 
+    OmTestManagers omTestManagers
+        = new OmTestManagers(configuration, blockClient, containerClient);
+    om = omTestManagers.getOzoneManager();
+    metadataManager = omTestManagers.getMetadataManager();
+    keyManager = (KeyManagerImpl)omTestManagers.getKeyManager();
+    writeClient = omTestManagers.getWriteClient();
     startDate = Instant.now();
   }
 
   @After
   public void cleanup() throws Exception {
-    metadataManager.stop();
+    om.stop();
     FileUtils.deleteDirectory(testDir);
   }
 
@@ -125,7 +140,7 @@ public class TestKeyManagerUnit {
     createBucket(metadataManager, "vol1", "bucket1");
 
     OmMultipartInfo omMultipartInfo =
-        initMultipartUpload(keyManager, "vol1", "bucket1", "dir/key1");
+        initMultipartUpload(writeClient, "vol1", "bucket1", "dir/key1");
 
     //WHEN
     OmMultipartUploadListParts omMultipartUploadListParts = keyManager
@@ -143,9 +158,9 @@ public class TestKeyManagerUnit {
     createBucket(metadataManager, "vol1", "bucket1");
     createBucket(metadataManager, "vol1", "bucket2");
 
-    initMultipartUpload(keyManager, "vol1", "bucket1", "dir/key1");
-    initMultipartUpload(keyManager, "vol1", "bucket1", "dir/key2");
-    initMultipartUpload(keyManager, "vol1", "bucket2", "dir/key1");
+    initMultipartUpload(writeClient, "vol1", "bucket1", "dir/key1");
+    initMultipartUpload(writeClient, "vol1", "bucket1", "dir/key2");
+    initMultipartUpload(writeClient, "vol1", "bucket2", "dir/key1");
 
     //WHEN
     OmMultipartUploadList omMultipartUploadList =
@@ -178,11 +193,11 @@ public class TestKeyManagerUnit {
     // Add few to cache and few to DB.
     addinitMultipartUploadToCache(volume, bucket, "dir/key1");
 
-    initMultipartUpload(keyManager, volume, bucket, "dir/key2");
+    initMultipartUpload(writeClient, volume, bucket, "dir/key2");
 
     addinitMultipartUploadToCache(volume, bucket, "dir/key3");
 
-    initMultipartUpload(keyManager, volume, bucket, "dir/key4");
+    initMultipartUpload(writeClient, volume, bucket, "dir/key4");
 
     //WHEN
     OmMultipartUploadList omMultipartUploadList =
@@ -201,12 +216,12 @@ public class TestKeyManagerUnit {
     // Same way add few to cache and few to DB.
     addinitMultipartUploadToCache(volume, bucket, "dir/ozonekey1");
 
-    initMultipartUpload(keyManager, volume, bucket, "dir/ozonekey2");
+    initMultipartUpload(writeClient, volume, bucket, "dir/ozonekey2");
 
-    OmMultipartInfo omMultipartInfo3 =addinitMultipartUploadToCache(volume,
+    OmMultipartInfo omMultipartInfo3 = addinitMultipartUploadToCache(volume,
         bucket, "dir/ozonekey3");
 
-    OmMultipartInfo omMultipartInfo4 = initMultipartUpload(keyManager,
+    OmMultipartInfo omMultipartInfo4 = initMultipartUpload(writeClient,
         volume, bucket, "dir/ozonekey4");
 
     omMultipartUploadList =
@@ -258,13 +273,13 @@ public class TestKeyManagerUnit {
     createBucket(metadataManager, "vol1", "bucket1");
     createBucket(metadataManager, "vol1", "bucket2");
 
-    initMultipartUpload(keyManager, "vol1", "bucket1", "dip/key1");
+    initMultipartUpload(writeClient, "vol1", "bucket1", "dip/key1");
 
-    initMultipartUpload(keyManager, "vol1", "bucket1", "dir/key1");
-    initMultipartUpload(keyManager, "vol1", "bucket1", "dir/key2");
-    initMultipartUpload(keyManager, "vol1", "bucket1", "key3");
+    initMultipartUpload(writeClient, "vol1", "bucket1", "dir/key1");
+    initMultipartUpload(writeClient, "vol1", "bucket1", "dir/key2");
+    initMultipartUpload(writeClient, "vol1", "bucket1", "key3");
 
-    initMultipartUpload(keyManager, "vol1", "bucket2", "dir/key1");
+    initMultipartUpload(writeClient, "vol1", "bucket2", "dir/key1");
 
     //WHEN
     OmMultipartUploadList omMultipartUploadList =
@@ -277,7 +292,7 @@ public class TestKeyManagerUnit {
     Assert.assertEquals("dir/key2", uploads.get(1).getKeyName());
   }
 
-  private void createBucket(OmMetadataManagerImpl omMetadataManager,
+  private void createBucket(OMMetadataManager omMetadataManager,
       String volume, String bucket)
       throws IOException {
     OmBucketInfo omBucketInfo = OmBucketInfo.newBuilder()
@@ -287,10 +302,10 @@ public class TestKeyManagerUnit {
         .setIsVersionEnabled(false)
         .setAcls(new ArrayList<>())
         .build();
-    TestOMRequestUtils.addBucketToOM(metadataManager, omBucketInfo);
+    OMRequestTestUtils.addBucketToOM(omMetadataManager, omBucketInfo);
   }
 
-  private OmMultipartInfo initMultipartUpload(KeyManagerImpl omtest,
+  private OmMultipartInfo initMultipartUpload(OzoneManagerProtocol omtest,
       String volume, String bucket, String key)
       throws IOException {
     OmKeyArgs key1 = new Builder()
@@ -298,10 +313,11 @@ public class TestKeyManagerUnit {
         .setBucketName(bucket)
         .setKeyName(key)
         .setReplicationConfig(
-            new RatisReplicationConfig(ReplicationFactor.THREE))
+            RatisReplicationConfig.getInstance(ReplicationFactor.THREE))
         .setAcls(new ArrayList<>())
         .build();
-    return omtest.initiateMultipartUpload(key1);
+    OmMultipartInfo omMultipartInfo = omtest.initiateMultipartUpload(key1);
+    return omMultipartInfo;
   }
 
   private OmMultipartInfo addinitMultipartUploadToCache(
@@ -313,7 +329,7 @@ public class TestKeyManagerUnit {
         .setUploadID(uploadID)
         .setCreationTime(Time.now())
         .setReplicationConfig(
-            new RatisReplicationConfig(ReplicationFactor.THREE))
+            RatisReplicationConfig.getInstance(ReplicationFactor.THREE))
         .setPartKeyInfoList(partKeyInfoMap)
         .build();
 
@@ -332,6 +348,100 @@ public class TestKeyManagerUnit {
             RandomUtils.nextInt()));
   }
 
+  @Test
+  @SuppressWarnings ({"unchecked", "varargs"})
+  public void testGetKeyInfo() throws IOException {
+    final DatanodeDetails dn1 = MockDatanodeDetails.randomDatanodeDetails();
+    final DatanodeDetails dn2 = MockDatanodeDetails.randomDatanodeDetails();
+    final DatanodeDetails dn3 = MockDatanodeDetails.randomDatanodeDetails();
+    final DatanodeDetails dn4 = MockDatanodeDetails.randomDatanodeDetails();
+    final long containerID = 1L;
+    Set<Long> containerIDs = newHashSet(containerID);
+
+    final Pipeline pipeline1 = Pipeline.newBuilder()
+        .setId(PipelineID.randomId())
+        .setReplicationConfig(
+            RatisReplicationConfig.getInstance(ReplicationFactor.THREE))
+        .setState(Pipeline.PipelineState.OPEN)
+        .setLeaderId(dn1.getUuid())
+        .setNodes(Arrays.asList(dn1, dn2, dn3))
+        .build();
+
+    final Pipeline pipeline2 = Pipeline.newBuilder()
+        .setId(PipelineID.randomId())
+        .setReplicationConfig(
+            RatisReplicationConfig.getInstance(ReplicationFactor.THREE))
+        .setState(Pipeline.PipelineState.OPEN)
+        .setLeaderId(dn1.getUuid())
+        .setNodes(Arrays.asList(dn2, dn3, dn4))
+        .build();
+
+    ContainerInfo ci = Mockito.mock(ContainerInfo.class);
+    when(ci.getContainerID()).thenReturn(1L);
+
+    // Setup SCM containerClient so that 1st call returns pipeline1 and
+    // 2nd call returns pipeline2.
+    when(containerClient.getContainerWithPipelineBatch(containerIDs))
+        .thenReturn(
+            singletonList(new ContainerWithPipeline(ci, pipeline1)),
+            singletonList(new ContainerWithPipeline(ci, pipeline2)));
+
+    insertVolume("volumeOne");
+
+    insertBucket("volumeOne", "bucketOne");
+
+    BlockID blockID1 = new BlockID(containerID, 1L);
+    insertKey(null, "volumeOne", "bucketOne", "keyOne", blockID1);
+    BlockID blockID2 = new BlockID(containerID, 2L);
+    insertKey(null, "volumeOne", "bucketOne", "keyTwo", blockID2);
+
+    // 1st call to get key1.
+    OmKeyArgs keyArgs = new Builder()
+        .setVolumeName("volumeOne")
+        .setBucketName("bucketOne")
+        .setKeyName("keyOne")
+        .build();
+    OmKeyInfo keyInfo = keyManager.getKeyInfo(keyArgs, "test");
+    final OmKeyLocationInfo blockLocation1 = keyInfo
+        .getLatestVersionLocations().getBlocksLatestVersionOnly().get(0);
+    Assert.assertEquals(blockID1, blockLocation1.getBlockID());
+    Assert.assertEquals(pipeline1, blockLocation1.getPipeline());
+    // Ensure SCM is called.
+    verify(containerClient, times(1))
+        .getContainerWithPipelineBatch(containerIDs);
+
+    // subsequent call to key2 in same container sound result no scm calls.
+    keyArgs = new Builder()
+        .setVolumeName("volumeOne")
+        .setBucketName("bucketOne")
+        .setKeyName("keyTwo")
+        .build();
+    OmKeyInfo keyInfo2 = keyManager.getKeyInfo(keyArgs, "test");
+    OmKeyLocationInfo blockLocation2 = keyInfo2
+        .getLatestVersionLocations().getBlocksLatestVersionOnly().get(0);
+    Assert.assertEquals(blockID2, blockLocation2.getBlockID());
+    Assert.assertEquals(pipeline1, blockLocation2.getPipeline());
+    // Ensure SCM is not called.
+    verify(containerClient, times(1))
+        .getContainerWithPipelineBatch(containerIDs);
+
+    // Yet, another call with forceCacheUpdate should trigger a call to SCM.
+    keyArgs = new Builder()
+        .setVolumeName("volumeOne")
+        .setBucketName("bucketOne")
+        .setKeyName("keyTwo")
+        .setForceUpdateContainerCacheFromSCM(true)
+        .build();
+    keyInfo2 = keyManager.getKeyInfo(keyArgs, "test");
+    blockLocation2 = keyInfo2
+        .getLatestVersionLocations().getBlocksLatestVersionOnly().get(0);
+    Assert.assertEquals(blockID2, blockLocation2.getBlockID());
+    Assert.assertEquals(pipeline2, blockLocation2.getPipeline());
+    // Ensure SCM is called.
+    verify(containerClient, times(2))
+        .getContainerWithPipelineBatch(containerIDs);
+  }
+
   @Test
   public void testLookupFileWithDnFailure() throws IOException {
     final DatanodeDetails dnOne = MockDatanodeDetails.randomDatanodeDetails();
@@ -345,7 +455,7 @@ public class TestKeyManagerUnit {
     final Pipeline pipelineOne = Pipeline.newBuilder()
         .setId(PipelineID.randomId())
         .setReplicationConfig(
-            new RatisReplicationConfig(ReplicationFactor.THREE))
+            RatisReplicationConfig.getInstance(ReplicationFactor.THREE))
         .setState(Pipeline.PipelineState.OPEN)
         .setLeaderId(dnOne.getUuid())
         .setNodes(Arrays.asList(dnOne, dnTwo, dnThree))
@@ -354,7 +464,7 @@ public class TestKeyManagerUnit {
     final Pipeline pipelineTwo = Pipeline.newBuilder()
         .setId(PipelineID.randomId())
         .setReplicationConfig(
-            new RatisReplicationConfig(ReplicationFactor.THREE))
+            RatisReplicationConfig.getInstance(ReplicationFactor.THREE))
         .setState(Pipeline.PipelineState.OPEN)
         .setLeaderId(dnFour.getUuid())
         .setNodes(Arrays.asList(dnFour, dnFive, dnSix))
@@ -371,41 +481,12 @@ public class TestKeyManagerUnit {
     when(containerClient.getContainerWithPipelineBatch(containerIDs))
         .thenReturn(cps);
 
-    final OmVolumeArgs volumeArgs = OmVolumeArgs.newBuilder()
-        .setVolume("volumeOne")
-        .setAdminName("admin")
-        .setOwnerName("admin")
-        .build();
-    TestOMRequestUtils.addVolumeToOM(metadataManager, volumeArgs);
+    insertVolume("volumeOne");
 
-    final OmBucketInfo bucketInfo = OmBucketInfo.newBuilder()
-          .setVolumeName("volumeOne")
-          .setBucketName("bucketOne")
-          .build();
-    TestOMRequestUtils.addBucketToOM(metadataManager, bucketInfo);
+    insertBucket("volumeOne", "bucketOne");
 
-    final OmKeyLocationInfo keyLocationInfo = new OmKeyLocationInfo.Builder()
-        .setBlockID(new BlockID(1L, 1L))
-        .setPipeline(pipelineOne)
-        .setOffset(0)
-        .setLength(256000)
-        .build();
-
-    final OmKeyInfo keyInfo = new OmKeyInfo.Builder()
-        .setVolumeName("volumeOne")
-        .setBucketName("bucketOne")
-        .setKeyName("keyOne")
-        .setOmKeyLocationInfos(singletonList(
-            new OmKeyLocationInfoGroup(0,
-                singletonList(keyLocationInfo))))
-        .setCreationTime(Time.now())
-        .setModificationTime(Time.now())
-        .setDataSize(256000)
-        .setReplicationConfig(
-                    new RatisReplicationConfig(ReplicationFactor.THREE))
-            .setAcls(Collections.emptyList())
-        .build();
-    TestOMRequestUtils.addKeyToOM(metadataManager, keyInfo);
+    insertKey(pipelineOne, "volumeOne", "bucketOne", "keyOne",
+        new BlockID(1L, 1L));
 
     final OmKeyArgs.Builder keyArgs = new OmKeyArgs.Builder()
         .setVolumeName("volumeOne")
@@ -431,6 +512,51 @@ public class TestKeyManagerUnit {
         .getNodes().contains(dnSix));
   }
 
+  private void insertKey(Pipeline pipeline, String volumeName,
+                         String bucketName, String keyName,
+                         BlockID blockID) throws IOException {
+    final OmKeyLocationInfo keyLocationInfo = new OmKeyLocationInfo.Builder()
+        .setBlockID(blockID)
+        .setPipeline(pipeline)
+        .setOffset(0)
+        .setLength(256000)
+        .build();
+
+    final OmKeyInfo keyInfo = new OmKeyInfo.Builder()
+        .setVolumeName(volumeName)
+        .setBucketName(bucketName)
+        .setKeyName(keyName)
+        .setOmKeyLocationInfos(singletonList(
+            new OmKeyLocationInfoGroup(0,
+                singletonList(keyLocationInfo))))
+        .setCreationTime(Time.now())
+        .setModificationTime(Time.now())
+        .setDataSize(256000)
+        .setReplicationConfig(
+                    RatisReplicationConfig.getInstance(ReplicationFactor.THREE))
+            .setAcls(Collections.emptyList())
+        .build();
+    OMRequestTestUtils.addKeyToOM(metadataManager, keyInfo);
+  }
+
+  private void insertBucket(String volumeName, String bucketName)
+      throws IOException {
+    final OmBucketInfo bucketInfo = OmBucketInfo.newBuilder()
+          .setVolumeName(volumeName)
+          .setBucketName(bucketName)
+          .build();
+    OMRequestTestUtils.addBucketToOM(metadataManager, bucketInfo);
+  }
+
+  private void insertVolume(String volumeName) throws IOException {
+    final OmVolumeArgs volumeArgs = OmVolumeArgs.newBuilder()
+        .setVolume(volumeName)
+        .setAdminName("admin")
+        .setOwnerName("admin")
+        .build();
+    OMRequestTestUtils.addVolumeToOM(metadataManager, volumeArgs);
+  }
+
   @Test
   public void listStatus() throws Exception {
     String volume = "vol";
@@ -438,10 +564,10 @@ public class TestKeyManagerUnit {
     String keyPrefix = "key";
     String client = "client.host";
 
-    TestOMRequestUtils.addVolumeToDB(volume, OzoneConsts.OZONE,
+    OMRequestTestUtils.addVolumeToDB(volume, OzoneConsts.OZONE,
         metadataManager);
 
-    TestOMRequestUtils.addBucketToDB(volume, bucket, metadataManager);
+    OMRequestTestUtils.addBucketToDB(volume, bucket, metadataManager);
 
     final Pipeline pipeline = MockPipeline.createPipeline(3);
     final List<String> nodes = pipeline.getNodes().stream()
@@ -471,14 +597,14 @@ public class TestKeyManagerUnit {
           .setCreationTime(Time.now())
           .setOmKeyLocationInfos(singletonList(
               new OmKeyLocationInfoGroup(0, new ArrayList<>())))
-          .setReplicationConfig(
-                      new RatisReplicationConfig(ReplicationFactor.THREE))
+          .setReplicationConfig(RatisReplicationConfig
+              .getInstance(ReplicationFactor.THREE))
           .setKeyName(keyPrefix + i)
           .setObjectID(i)
           .setUpdateID(i)
           .build();
       keyInfo.appendNewBlocks(singletonList(keyLocationInfo), false);
-      TestOMRequestUtils.addKeyToOM(metadataManager, keyInfo);
+      OMRequestTestUtils.addKeyToOM(metadataManager, keyInfo);
     }
 
     when(containerClient.getContainerWithPipelineBatch(containerIDs))
