@@ -22,6 +22,7 @@ import org.apache.hadoop.security.authentication.client.AuthenticationException;
 import org.junit.After;
 import org.junit.Before;
 import org.junit.Test;
+
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.io.PrintStream;
@@ -30,7 +31,12 @@ import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
 import static java.nio.charset.StandardCharsets.UTF_8;
-import static org.junit.Assert.*;
+import static org.apache.hadoop.hdds.cli.GenericCli.EXECUTION_ERROR_EXIT_CODE;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertTrue;
+import static picocli.CommandLine.ExitCode.OK;
+import static picocli.CommandLine.ExitCode.USAGE;
 
 /**
  * This class is used to test the CLI provided by OzoneManagerStarter, which is
@@ -63,89 +69,69 @@ public class TestOzoneManagerStarter {
 
   @Test
   public void testCallsStartWhenServerStarted() throws Exception {
-    executeCommand();
+    assertEquals(OK, executeCommand());
     assertTrue(mock.startCalled);
   }
 
   @Test
   public void testExceptionThrownWhenStartFails() throws Exception {
     mock.throwOnStart = true;
-    try {
-      executeCommand();
-      fail("Exception should have been thrown");
-    } catch (Exception e) {
-      assertTrue(true);
-    }
+    assertEquals(EXECUTION_ERROR_EXIT_CODE, executeCommand());
   }
 
   @Test
   public void testStartNotCalledWithInvalidParam() throws Exception {
-    executeCommand("--invalid");
+    assertEquals(USAGE, executeCommand("--invalid"));
     assertFalse(mock.startCalled);
   }
 
   @Test
   public void testPassingInitSwitchCallsInit() {
-    executeCommand("--init");
+    assertEquals(OK, executeCommand("--init"));
     assertTrue(mock.initCalled);
   }
 
   @Test
   public void testInitSwitchWithInvalidParamDoesNotRun() {
-    executeCommand("--init", "--invalid");
+    assertEquals(USAGE, executeCommand("--init", "--invalid"));
     assertFalse(mock.initCalled);
   }
 
   @Test
   public void testUnSuccessfulInitThrowsException() {
     mock.throwOnInit = true;
-    try {
-      executeCommand("--init");
-      fail("Exception show have been thrown");
-    } catch (Exception e) {
-      assertTrue(true);
-    }
+    assertEquals(EXECUTION_ERROR_EXIT_CODE, executeCommand("--init"));
   }
 
   @Test
   public void testInitThatReturnsFalseThrowsException() {
     mock.initStatus = false;
-    try {
-      executeCommand("--init");
-      fail("Exception show have been thrown");
-    } catch (Exception e) {
-      assertTrue(true);
-    }
+    assertEquals(EXECUTION_ERROR_EXIT_CODE, executeCommand("--init"));
   }
 
   @Test
   public void testCallsStartAndCancelPrepareWithUpgradeFlag() {
-    executeCommand("--upgrade");
+    assertEquals(OK, executeCommand("--upgrade"));
     assertTrue(mock.startAndCancelPrepareCalled);
   }
 
   @Test
   public void testUnsuccessfulUpgradeThrowsException() {
     mock.throwOnStartAndCancelPrepare = true;
-    try {
-      executeCommand("--upgrade");
-      fail("Exception show have been thrown");
-    } catch (Exception e) {
-      assertTrue(true);
-    }
+    assertEquals(EXECUTION_ERROR_EXIT_CODE, executeCommand("--upgrade"));
   }
 
   @Test
   public void testUsagePrintedOnInvalidInput()
       throws UnsupportedEncodingException {
-    executeCommand("--invalid");
+    assertEquals(USAGE, executeCommand("--invalid"));
     Pattern p = Pattern.compile("^Unknown option:.*--invalid.*\nUsage");
     Matcher m = p.matcher(errContent.toString(DEFAULT_ENCODING));
     assertTrue(m.find());
   }
 
-  private void executeCommand(String... args) {
-    new OzoneManagerStarter(mock).execute(args);
+  private int executeCommand(String... args) {
+    return new OzoneManagerStarter(mock).execute(args);
   }
 
   static class MockOMStarter implements OMStarterInterface {
@@ -178,8 +164,8 @@ public class TestOzoneManagerStarter {
     }
 
     @Override
-    public void bootstrap(OzoneConfiguration conf) throws IOException,
-        AuthenticationException {
+    public void bootstrap(OzoneConfiguration conf, boolean force)
+        throws IOException, AuthenticationException {
       //TODO: Add test for bootstrap
     }
 
