@@ -46,10 +46,12 @@ import org.apache.ozone.test.GenericTestUtils;
 import org.apache.ratis.protocol.RaftGroupId;
 import org.apache.ratis.server.protocol.TermIndex;
 import org.apache.ratis.statemachine.SnapshotInfo;
+import org.apache.ratis.util.ExitUtils;
 import org.apache.ratis.util.LifeCycle;
 import org.junit.After;
 import org.junit.Assert;
 import org.junit.Before;
+import org.junit.BeforeClass;
 import org.junit.Rule;
 import org.junit.Test;
 import org.junit.rules.TemporaryFolder;
@@ -82,6 +84,11 @@ public class TestOzoneManagerRatisServer {
   private SecurityConfig secConfig;
   private OMCertificateClient certClient;
 
+  @BeforeClass
+  public static void setup() {
+    ExitUtils.disableSystemExit();
+  }
+  
   @Before
   public void init() throws Exception {
     conf = new OzoneConfiguration();
@@ -112,6 +119,7 @@ public class TestOzoneManagerRatisServer {
     initialTermIndex = TermIndex.valueOf(0, 0);
     RatisSnapshotInfo omRatisSnapshotInfo = new RatisSnapshotInfo();
     when(ozoneManager.getSnapshotInfo()).thenReturn(omRatisSnapshotInfo);
+    when(ozoneManager.getConfiguration()).thenReturn(conf);
     secConfig = new SecurityConfig(conf);
     certClient = new OMCertificateClient(secConfig);
     omRatisServer = OzoneManagerRatisServer.newOMRatisServer(conf, ozoneManager,
@@ -185,7 +193,7 @@ public class TestOzoneManagerRatisServer {
       OmUtils.isReadOnly(request);
       assertFalse(cmdtype + " is not categorized in " +
               "OmUtils#isReadyOnly",
-          logCapturer.getOutput().contains("CmdType " + cmdtype +" is not " +
+          logCapturer.getOutput().contains("CmdType " + cmdtype + " is not " +
               "categorized as readOnly or not."));
       logCapturer.clearOutput();
     }
