@@ -34,13 +34,14 @@ import org.apache.hadoop.ozone.om.OMMetrics;
 import org.apache.hadoop.ozone.om.OmMetadataManagerImpl;
 import org.apache.hadoop.ozone.om.OzoneManager;
 import org.apache.hadoop.ozone.om.ResolvedBucket;
+import org.apache.hadoop.ozone.om.helpers.OzoneFSUtils;
+import org.apache.hadoop.ozone.om.helpers.BucketLayout;
+import org.apache.hadoop.ozone.om.helpers.OmKeyInfo;
 import org.apache.hadoop.ozone.om.helpers.OmBucketInfo;
 import org.apache.hadoop.ozone.om.helpers.OmDirectoryInfo;
-import org.apache.hadoop.ozone.om.helpers.OmKeyInfo;
-import org.apache.hadoop.ozone.om.helpers.OzoneFSUtils;
 import org.apache.hadoop.ozone.om.ratis.utils.OzoneManagerDoubleBufferHelper;
 import org.apache.hadoop.ozone.om.request.OMClientRequest;
-import org.apache.hadoop.ozone.om.request.TestOMRequestUtils;
+import org.apache.hadoop.ozone.om.request.OMRequestTestUtils;
 import org.apache.hadoop.ozone.om.response.OMClientResponse;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.CreateDirectoryRequest;
@@ -89,8 +90,7 @@ public class TestOMDirectoryCreateRequestWithFSO {
     OzoneConfiguration ozoneConfiguration = new OzoneConfiguration();
     ozoneConfiguration.set(OMConfigKeys.OZONE_OM_DB_DIRS,
             folder.newFolder().getAbsolutePath());
-    TestOMRequestUtils.configureFSOptimizedPaths(ozoneConfiguration,
-            true, OMConfigKeys.OZONE_OM_METADATA_LAYOUT_PREFIX);
+    OMRequestTestUtils.configureFSOptimizedPaths(ozoneConfiguration, true);
     omMetadataManager = new OmMetadataManagerImpl(ozoneConfiguration);
     when(ozoneManager.getMetrics()).thenReturn(omMetrics);
     when(ozoneManager.getMetadataManager()).thenReturn(omMetadataManager);
@@ -114,13 +114,14 @@ public class TestOMDirectoryCreateRequestWithFSO {
     String bucketName = "bucket1";
     String keyName = "a/b/c";
 
-    TestOMRequestUtils.addVolumeAndBucketToDB(volumeName, bucketName,
-            omMetadataManager);
+    OMRequestTestUtils.addVolumeAndBucketToDB(volumeName, bucketName,
+            omMetadataManager, getBucketLayout());
 
     OMRequest omRequest = createDirectoryRequest(volumeName, bucketName,
             keyName);
     OMDirectoryCreateRequestWithFSO omDirectoryCreateRequestWithFSO =
-            new OMDirectoryCreateRequestWithFSO(omRequest);
+        new OMDirectoryCreateRequestWithFSO(omRequest,
+            BucketLayout.FILE_SYSTEM_OPTIMIZED);
 
     OMRequest modifiedOmRequest =
             omDirectoryCreateRequestWithFSO.preExecute(ozoneManager);
@@ -137,24 +138,25 @@ public class TestOMDirectoryCreateRequestWithFSO {
     String keyName = createDirKey(dirs, 3);
 
     // Add volume and bucket entries to DB.
-    TestOMRequestUtils.addVolumeAndBucketToDB(volumeName, bucketName,
-            omMetadataManager);
+    OMRequestTestUtils.addVolumeAndBucketToDB(volumeName, bucketName,
+            omMetadataManager, getBucketLayout());
 
-    String bucketKey = omMetadataManager.getBucketKey(volumeName, bucketName);
-    OmBucketInfo omBucketInfo =
-            omMetadataManager.getBucketTable().get(bucketKey);
-    long bucketID = omBucketInfo.getObjectID();
+    final long volumeId = omMetadataManager.getVolumeId(volumeName);
+    final long bucketId = omMetadataManager.getBucketId(volumeName,
+            bucketName);
 
     OMRequest omRequest = createDirectoryRequest(volumeName, bucketName,
             keyName);
     OMDirectoryCreateRequestWithFSO omDirCreateRequestFSO =
-            new OMDirectoryCreateRequestWithFSO(omRequest);
+        new OMDirectoryCreateRequestWithFSO(omRequest,
+            BucketLayout.FILE_SYSTEM_OPTIMIZED);
 
     OMRequest modifiedOmReq =
         omDirCreateRequestFSO.preExecute(ozoneManager);
 
     omDirCreateRequestFSO =
-        new OMDirectoryCreateRequestWithFSO(modifiedOmReq);
+        new OMDirectoryCreateRequestWithFSO(modifiedOmReq,
+            BucketLayout.FILE_SYSTEM_OPTIMIZED);
 
     OMClientResponse omClientResponse =
             omDirCreateRequestFSO.validateAndUpdateCache(ozoneManager, 100L,
@@ -162,7 +164,7 @@ public class TestOMDirectoryCreateRequestWithFSO {
 
     Assert.assertTrue(omClientResponse.getOMResponse().getStatus()
             == OzoneManagerProtocolProtos.Status.OK);
-    verifyDirectoriesInDB(dirs, bucketID);
+    verifyDirectoriesInDB(dirs, volumeId, bucketId);
   }
 
   @Test
@@ -175,13 +177,15 @@ public class TestOMDirectoryCreateRequestWithFSO {
     OMRequest omRequest = createDirectoryRequest(volumeName, bucketName,
             keyName);
     OMDirectoryCreateRequestWithFSO omDirCreateRequestFSO =
-            new OMDirectoryCreateRequestWithFSO(omRequest);
+        new OMDirectoryCreateRequestWithFSO(omRequest,
+            BucketLayout.FILE_SYSTEM_OPTIMIZED);
 
     OMRequest modifiedOmRequest =
         omDirCreateRequestFSO.preExecute(ozoneManager);
 
     omDirCreateRequestFSO =
-        new OMDirectoryCreateRequestWithFSO(modifiedOmRequest);
+        new OMDirectoryCreateRequestWithFSO(modifiedOmRequest,
+            BucketLayout.FILE_SYSTEM_OPTIMIZED);
 
     OMClientResponse omClientResponse =
             omDirCreateRequestFSO.validateAndUpdateCache(ozoneManager, 100L,
@@ -206,12 +210,14 @@ public class TestOMDirectoryCreateRequestWithFSO {
     OMRequest omRequest = createDirectoryRequest(volumeName, bucketName,
             keyName);
     OMDirectoryCreateRequestWithFSO omDirCreateReqFSO =
-            new OMDirectoryCreateRequestWithFSO(omRequest);
+        new OMDirectoryCreateRequestWithFSO(omRequest,
+            BucketLayout.FILE_SYSTEM_OPTIMIZED);
 
     OMRequest modifiedOmReq = omDirCreateReqFSO.preExecute(ozoneManager);
 
-    omDirCreateReqFSO = new OMDirectoryCreateRequestWithFSO(modifiedOmReq);
-    TestOMRequestUtils.addVolumeToDB(volumeName, omMetadataManager);
+    omDirCreateReqFSO = new OMDirectoryCreateRequestWithFSO(modifiedOmReq,
+        BucketLayout.FILE_SYSTEM_OPTIMIZED);
+    OMRequestTestUtils.addVolumeToDB(volumeName, omMetadataManager);
 
     OMClientResponse omClientResponse =
             omDirCreateReqFSO.validateAndUpdateCache(ozoneManager, 100L,
@@ -234,35 +240,38 @@ public class TestOMDirectoryCreateRequestWithFSO {
     String keyName = createDirKey(dirs, 3);
 
     // Add volume and bucket entries to DB.
-    TestOMRequestUtils.addVolumeAndBucketToDB(volumeName, bucketName,
-            omMetadataManager);
+    OMRequestTestUtils.addVolumeAndBucketToDB(volumeName, bucketName,
+            omMetadataManager, getBucketLayout());
 
-    String bucketKey = omMetadataManager.getBucketKey(volumeName, bucketName);
-    OmBucketInfo omBucketInfo =
-            omMetadataManager.getBucketTable().get(bucketKey);
-    long bucketID = omBucketInfo.getObjectID();
+    final long volumeId = omMetadataManager.getVolumeId(volumeName);
+    final long bucketId = omMetadataManager.getBucketId(volumeName,
+            bucketName);
     int objID = 100;
 
     //1. Create root
     OmDirectoryInfo omDirInfo =
-            TestOMRequestUtils.createOmDirectoryInfo(dirs.get(0), objID++,
-                    bucketID);
-    TestOMRequestUtils.addDirKeyToDirTable(true, omDirInfo, 5000,
+            OMRequestTestUtils.createOmDirectoryInfo(dirs.get(0), objID++,
+                    bucketId);
+    OMRequestTestUtils.addDirKeyToDirTable(true, omDirInfo,
+            volumeName, bucketName, 5000,
             omMetadataManager);
     //2. Create sub-directory under root
-    omDirInfo = TestOMRequestUtils.createOmDirectoryInfo(dirs.get(1), objID++,
+    omDirInfo = OMRequestTestUtils.createOmDirectoryInfo(dirs.get(1), objID++,
             omDirInfo.getObjectID());
-    TestOMRequestUtils.addDirKeyToDirTable(true, omDirInfo, 5000,
+    OMRequestTestUtils.addDirKeyToDirTable(true, omDirInfo,
+            volumeName, bucketName, 5000,
             omMetadataManager);
 
     OMRequest omRequest = createDirectoryRequest(volumeName, bucketName,
             keyName);
     OMDirectoryCreateRequestWithFSO omDirCreateReqFSO =
-            new OMDirectoryCreateRequestWithFSO(omRequest);
+        new OMDirectoryCreateRequestWithFSO(omRequest,
+            BucketLayout.FILE_SYSTEM_OPTIMIZED);
 
     OMRequest modifiedOmReq = omDirCreateReqFSO.preExecute(ozoneManager);
 
-    omDirCreateReqFSO = new OMDirectoryCreateRequestWithFSO(modifiedOmReq);
+    omDirCreateReqFSO = new OMDirectoryCreateRequestWithFSO(modifiedOmReq,
+        BucketLayout.FILE_SYSTEM_OPTIMIZED);
 
     OMClientResponse omClientResponse =
             omDirCreateReqFSO.validateAndUpdateCache(ozoneManager, 100L,
@@ -272,7 +281,7 @@ public class TestOMDirectoryCreateRequestWithFSO {
             == OzoneManagerProtocolProtos.Status.OK);
 
     // Key should exist in DB and cache.
-    verifyDirectoriesInDB(dirs, bucketID);
+    verifyDirectoriesInDB(dirs, volumeId, bucketId);
   }
 
   @Test
@@ -284,26 +293,25 @@ public class TestOMDirectoryCreateRequestWithFSO {
     String keyName = createDirKey(dirs, 3);
 
     // Add volume and bucket entries to DB.
-    TestOMRequestUtils.addVolumeAndBucketToDB(volumeName, bucketName,
-            omMetadataManager);
+    OMRequestTestUtils.addVolumeAndBucketToDB(volumeName, bucketName,
+            omMetadataManager, getBucketLayout());
 
-    String bucketKey = omMetadataManager.getBucketKey(volumeName, bucketName);
-    OmBucketInfo omBucketInfo =
-            omMetadataManager.getBucketTable().get(bucketKey);
-    long bucketID = omBucketInfo.getObjectID();
+    final long volumeId = omMetadataManager.getVolumeId(volumeName);
+    final long bucketId = omMetadataManager.getBucketId(volumeName,
+            bucketName);
 
     // bucketID is the parent
-    long parentID = bucketID;
+    long parentID = bucketId;
 
     // add all the directories into DirectoryTable
     for (int indx = 0; indx < dirs.size(); indx++) {
       long objID = 100 + indx;
       long txnID = 5000 + indx;
       // for index=0, parentID is bucketID
-      OmDirectoryInfo omDirInfo = TestOMRequestUtils.createOmDirectoryInfo(
+      OmDirectoryInfo omDirInfo = OMRequestTestUtils.createOmDirectoryInfo(
               dirs.get(indx), objID, parentID);
-      TestOMRequestUtils.addDirKeyToDirTable(false, omDirInfo,
-              txnID, omMetadataManager);
+      OMRequestTestUtils.addDirKeyToDirTable(false, omDirInfo,
+              volumeName, bucketName, txnID, omMetadataManager);
 
       parentID = omDirInfo.getObjectID();
     }
@@ -311,11 +319,13 @@ public class TestOMDirectoryCreateRequestWithFSO {
     OMRequest omRequest = createDirectoryRequest(volumeName, bucketName,
             keyName);
     OMDirectoryCreateRequestWithFSO omDirCreateReqFSO =
-            new OMDirectoryCreateRequestWithFSO(omRequest);
+        new OMDirectoryCreateRequestWithFSO(omRequest,
+            BucketLayout.FILE_SYSTEM_OPTIMIZED);
 
     OMRequest modifiedOmReq = omDirCreateReqFSO.preExecute(ozoneManager);
 
-    omDirCreateReqFSO = new OMDirectoryCreateRequestWithFSO(modifiedOmReq);
+    omDirCreateReqFSO = new OMDirectoryCreateRequestWithFSO(modifiedOmReq,
+        BucketLayout.FILE_SYSTEM_OPTIMIZED);
 
     OMClientResponse omClientResponse =
             omDirCreateReqFSO.validateAndUpdateCache(ozoneManager, 100L,
@@ -328,8 +338,8 @@ public class TestOMDirectoryCreateRequestWithFSO {
             0, ozoneManager.getMetrics().getNumKeys());
 
     // Key should exist in DB and doesn't added to cache.
-    verifyDirectoriesInDB(dirs, bucketID);
-    verifyDirectoriesNotInCache(dirs, bucketID);
+    verifyDirectoriesInDB(dirs, volumeId, bucketId);
+    verifyDirectoriesNotInCache(dirs, volumeId, bucketId);
   }
 
   /**
@@ -345,8 +355,8 @@ public class TestOMDirectoryCreateRequestWithFSO {
     String keyName = createDirKey(dirs, 3);
 
     // Add volume and bucket entries to DB.
-    TestOMRequestUtils.addVolumeAndBucketToDB(volumeName, bucketName,
-            omMetadataManager);
+    OMRequestTestUtils.addVolumeAndBucketToDB(volumeName, bucketName,
+            omMetadataManager, getBucketLayout());
     String bucketKey = omMetadataManager.getBucketKey(volumeName, bucketName);
     OmBucketInfo omBucketInfo =
             omMetadataManager.getBucketTable().get(bucketKey);
@@ -358,10 +368,10 @@ public class TestOMDirectoryCreateRequestWithFSO {
       long objID = 100 + indx;
       long txnID = 5000 + indx;
       // for index=0, parentID is bucketID
-      OmDirectoryInfo omDirInfo = TestOMRequestUtils.createOmDirectoryInfo(
+      OmDirectoryInfo omDirInfo = OMRequestTestUtils.createOmDirectoryInfo(
               dirs.get(indx), objID, parentID);
-      TestOMRequestUtils.addDirKeyToDirTable(false, omDirInfo,
-              txnID, omMetadataManager);
+      OMRequestTestUtils.addDirKeyToDirTable(false, omDirInfo,
+              volumeName, bucketName, txnID, omMetadataManager);
 
       parentID = omDirInfo.getObjectID();
     }
@@ -370,24 +380,32 @@ public class TestOMDirectoryCreateRequestWithFSO {
     long txnID = 50000;
 
     // Add a file into the FileTable, this is to simulate "file exists" check.
-    OmKeyInfo omKeyInfo = TestOMRequestUtils.createOmKeyInfo(volumeName,
+    OmKeyInfo omKeyInfo = OMRequestTestUtils.createOmKeyInfo(volumeName,
             bucketName, keyName, HddsProtos.ReplicationType.RATIS,
             HddsProtos.ReplicationFactor.THREE, objID++);
-    String ozoneFileName = parentID + "/" + dirs.get(dirs.size() - 1);
+    final long volumeId = omMetadataManager.getVolumeId(volumeName);
+    final long bucketId = omBucketInfo.getObjectID();
+
+    final String ozoneFileName = omMetadataManager.getOzonePathKey(
+            volumeId, bucketId, parentID, dirs.get(dirs.size() - 1));
     ++txnID;
-    omMetadataManager.getKeyTable().addCacheEntry(new CacheKey<>(ozoneFileName),
+    omMetadataManager.getKeyTable(getBucketLayout())
+        .addCacheEntry(new CacheKey<>(ozoneFileName),
             new CacheValue<>(Optional.of(omKeyInfo), txnID));
-    omMetadataManager.getKeyTable().put(ozoneFileName, omKeyInfo);
+    omMetadataManager.getKeyTable(getBucketLayout())
+        .put(ozoneFileName, omKeyInfo);
 
     OMRequest omRequest = createDirectoryRequest(volumeName, bucketName,
             keyName);
     OMDirectoryCreateRequestWithFSO omDirCreateReqFSO =
-            new OMDirectoryCreateRequestWithFSO(omRequest);
+        new OMDirectoryCreateRequestWithFSO(omRequest,
+            BucketLayout.FILE_SYSTEM_OPTIMIZED);
 
     OMRequest modifiedOmReq =
             omDirCreateReqFSO.preExecute(ozoneManager);
 
-    omDirCreateReqFSO = new OMDirectoryCreateRequestWithFSO(modifiedOmReq);
+    omDirCreateReqFSO = new OMDirectoryCreateRequestWithFSO(modifiedOmReq,
+        BucketLayout.FILE_SYSTEM_OPTIMIZED);
 
     OMClientResponse omClientResponse =
             omDirCreateReqFSO.validateAndUpdateCache(ozoneManager, 100L,
@@ -400,7 +418,8 @@ public class TestOMDirectoryCreateRequestWithFSO {
             0, ozoneManager.getMetrics().getNumKeys());
 
     // Key should not exist in DB
-    Assert.assertNotNull(omMetadataManager.getKeyTable().get(ozoneFileName));
+    Assert.assertNotNull(
+        omMetadataManager.getKeyTable(getBucketLayout()).get(ozoneFileName));
     // Key should not exist in DB
     Assert.assertEquals("Wrong directories count!", 3,
             omMetadataManager.getDirectoryTable().getEstimatedKeyCount());
@@ -421,8 +440,8 @@ public class TestOMDirectoryCreateRequestWithFSO {
     String keyName = createDirKey(dirs, 3);
 
     // Add volume and bucket entries to DB.
-    TestOMRequestUtils.addVolumeAndBucketToDB(volumeName, bucketName,
-            omMetadataManager);
+    OMRequestTestUtils.addVolumeAndBucketToDB(volumeName, bucketName,
+            omMetadataManager, getBucketLayout());
     String bucketKey = omMetadataManager.getBucketKey(volumeName, bucketName);
     OmBucketInfo omBucketInfo =
             omMetadataManager.getBucketTable().get(bucketKey);
@@ -432,32 +451,39 @@ public class TestOMDirectoryCreateRequestWithFSO {
     long txnID = 5000;
 
     // for index=0, parentID is bucketID
-    OmDirectoryInfo omDirInfo = TestOMRequestUtils.createOmDirectoryInfo(
+    OmDirectoryInfo omDirInfo = OMRequestTestUtils.createOmDirectoryInfo(
             dirs.get(0), objID++, parentID);
-    TestOMRequestUtils.addDirKeyToDirTable(true, omDirInfo,
-            txnID, omMetadataManager);
+    OMRequestTestUtils.addDirKeyToDirTable(true, omDirInfo,
+            volumeName, bucketName, txnID, omMetadataManager);
     parentID = omDirInfo.getObjectID();
 
     // Add a key in second level.
-    OmKeyInfo omKeyInfo = TestOMRequestUtils.createOmKeyInfo(volumeName,
+    OmKeyInfo omKeyInfo = OMRequestTestUtils.createOmKeyInfo(volumeName,
             bucketName, keyName, HddsProtos.ReplicationType.RATIS,
             HddsProtos.ReplicationFactor.THREE, objID);
 
-    String ozoneKey = parentID + "/" + dirs.get(1);
+    final long volumeId = omMetadataManager.getVolumeId(volumeName);
+    final long bucketId = omBucketInfo.getObjectID();
+
+    final String ozoneKey = omMetadataManager.getOzonePathKey(
+            volumeId, bucketId, parentID, dirs.get(1));
     ++txnID;
-    omMetadataManager.getKeyTable().addCacheEntry(new CacheKey<>(ozoneKey),
+    omMetadataManager.getKeyTable(getBucketLayout())
+        .addCacheEntry(new CacheKey<>(ozoneKey),
             new CacheValue<>(Optional.of(omKeyInfo), txnID));
-    omMetadataManager.getKeyTable().put(ozoneKey, omKeyInfo);
+    omMetadataManager.getKeyTable(getBucketLayout()).put(ozoneKey, omKeyInfo);
 
     OMRequest omRequest = createDirectoryRequest(volumeName, bucketName,
             keyName);
     OMDirectoryCreateRequestWithFSO omDirCreateReqFSO =
-            new OMDirectoryCreateRequestWithFSO(omRequest);
+        new OMDirectoryCreateRequestWithFSO(omRequest,
+            BucketLayout.FILE_SYSTEM_OPTIMIZED);
 
     OMRequest modifiedOmReq =
             omDirCreateReqFSO.preExecute(ozoneManager);
 
-    omDirCreateReqFSO = new OMDirectoryCreateRequestWithFSO(modifiedOmReq);
+    omDirCreateReqFSO = new OMDirectoryCreateRequestWithFSO(modifiedOmReq,
+        BucketLayout.FILE_SYSTEM_OPTIMIZED);
 
     OMClientResponse omClientResponse =
             omDirCreateReqFSO.validateAndUpdateCache(ozoneManager, 100L,
@@ -472,7 +498,8 @@ public class TestOMDirectoryCreateRequestWithFSO {
             0, ozoneManager.getMetrics().getNumKeys());
 
     // Key should not exist in DB
-    Assert.assertTrue(omMetadataManager.getKeyTable().get(ozoneKey) != null);
+    Assert.assertTrue(
+        omMetadataManager.getKeyTable(getBucketLayout()).get(ozoneKey) != null);
     // Key should not exist in DB
     Assert.assertEquals("Wrong directories count!",
             1, omMetadataManager.getDirectoryTable().getEstimatedKeyCount());
@@ -486,21 +513,22 @@ public class TestOMDirectoryCreateRequestWithFSO {
     String keyName = createDirKey(dirs, 255);
 
     // Add volume and bucket entries to DB.
-    TestOMRequestUtils.addVolumeAndBucketToDB(volumeName, bucketName,
-            omMetadataManager);
-    String bucketKey = omMetadataManager.getBucketKey(volumeName, bucketName);
-    OmBucketInfo omBucketInfo =
-            omMetadataManager.getBucketTable().get(bucketKey);
-    long bucketID = omBucketInfo.getObjectID();
+    OMRequestTestUtils.addVolumeAndBucketToDB(volumeName, bucketName,
+            omMetadataManager, getBucketLayout());
+    final long volumeId = omMetadataManager.getVolumeId(volumeName);
+    final long bucketId = omMetadataManager.getBucketId(volumeName,
+            bucketName);
 
     OMRequest omRequest = createDirectoryRequest(volumeName, bucketName,
             OzoneFSUtils.addTrailingSlashIfNeeded(keyName));
     OMDirectoryCreateRequestWithFSO omDirCreateReqFSO =
-            new OMDirectoryCreateRequestWithFSO(omRequest);
+        new OMDirectoryCreateRequestWithFSO(omRequest,
+            BucketLayout.FILE_SYSTEM_OPTIMIZED);
 
     OMRequest modifiedOmReq = omDirCreateReqFSO.preExecute(ozoneManager);
 
-    omDirCreateReqFSO = new OMDirectoryCreateRequestWithFSO(modifiedOmReq);
+    omDirCreateReqFSO = new OMDirectoryCreateRequestWithFSO(modifiedOmReq,
+        BucketLayout.FILE_SYSTEM_OPTIMIZED);
 
     Assert.assertEquals(0L, omMetrics.getNumKeys());
     OMClientResponse omClientResponse =
@@ -510,7 +538,7 @@ public class TestOMDirectoryCreateRequestWithFSO {
     Assert.assertEquals(OzoneManagerProtocolProtos.Status.OK,
             omClientResponse.getOMResponse().getStatus());
 
-    verifyDirectoriesInDB(dirs, bucketID);
+    verifyDirectoriesInDB(dirs, volumeId, bucketId);
 
     Assert.assertEquals(dirs.size(), omMetrics.getNumKeys());
   }
@@ -523,17 +551,19 @@ public class TestOMDirectoryCreateRequestWithFSO {
     String keyName = createDirKey(dirs, 256);
 
     // Add volume and bucket entries to DB.
-    TestOMRequestUtils.addVolumeAndBucketToDB(volumeName, bucketName,
-            omMetadataManager);
+    OMRequestTestUtils.addVolumeAndBucketToDB(volumeName, bucketName,
+            omMetadataManager, getBucketLayout());
 
     OMRequest omRequest = createDirectoryRequest(volumeName, bucketName,
             OzoneFSUtils.addTrailingSlashIfNeeded(keyName));
     OMDirectoryCreateRequestWithFSO omDirCreateReqFSO =
-            new OMDirectoryCreateRequestWithFSO(omRequest);
+        new OMDirectoryCreateRequestWithFSO(omRequest,
+            BucketLayout.FILE_SYSTEM_OPTIMIZED);
 
     OMRequest modifiedOmReq = omDirCreateReqFSO.preExecute(ozoneManager);
 
-    omDirCreateReqFSO = new OMDirectoryCreateRequestWithFSO(modifiedOmReq);
+    omDirCreateReqFSO = new OMDirectoryCreateRequestWithFSO(modifiedOmReq,
+        BucketLayout.FILE_SYSTEM_OPTIMIZED);
 
     Assert.assertEquals(0L, omMetrics.getNumKeys());
     OMClientResponse omClientResponse =
@@ -557,21 +587,22 @@ public class TestOMDirectoryCreateRequestWithFSO {
     String keyName = createDirKey(dirs, 3);
 
     // Add volume and bucket entries to DB.
-    TestOMRequestUtils.addVolumeAndBucketToDB(volumeName, bucketName,
-            omMetadataManager);
-    String bucketKey = omMetadataManager.getBucketKey(volumeName, bucketName);
-    OmBucketInfo omBucketInfo =
-            omMetadataManager.getBucketTable().get(bucketKey);
-    long bucketID = omBucketInfo.getObjectID();
+    OMRequestTestUtils.addVolumeAndBucketToDB(volumeName, bucketName,
+            omMetadataManager, getBucketLayout());
+    final long volumeId = omMetadataManager.getVolumeId(volumeName);
+    final long bucketId = omMetadataManager.getBucketId(volumeName,
+            bucketName);
 
     OMRequest omRequest = createDirectoryRequest(volumeName, bucketName,
             OzoneFSUtils.addTrailingSlashIfNeeded(keyName));
     OMDirectoryCreateRequestWithFSO omDirCreateReqFSO =
-            new OMDirectoryCreateRequestWithFSO(omRequest);
+        new OMDirectoryCreateRequestWithFSO(omRequest,
+            BucketLayout.FILE_SYSTEM_OPTIMIZED);
 
     OMRequest modifiedOmReq = omDirCreateReqFSO.preExecute(ozoneManager);
 
-    omDirCreateReqFSO = new OMDirectoryCreateRequestWithFSO(modifiedOmReq);
+    omDirCreateReqFSO = new OMDirectoryCreateRequestWithFSO(modifiedOmReq,
+        BucketLayout.FILE_SYSTEM_OPTIMIZED);
 
     Assert.assertEquals(0L, omMetrics.getNumKeys());
     OMClientResponse omClientResponse =
@@ -581,7 +612,7 @@ public class TestOMDirectoryCreateRequestWithFSO {
     Assert.assertEquals(OzoneManagerProtocolProtos.Status.OK,
             omClientResponse.getOMResponse().getStatus());
 
-    verifyDirectoriesInDB(dirs, bucketID);
+    verifyDirectoriesInDB(dirs, volumeId, bucketId);
 
     Assert.assertEquals(dirs.size(), omMetrics.getNumKeys());
   }
@@ -601,15 +632,17 @@ public class TestOMDirectoryCreateRequestWithFSO {
     return buf.toString();
   }
 
-  private void verifyDirectoriesInDB(List<String> dirs, long bucketID)
+  private void verifyDirectoriesInDB(List<String> dirs,
+                                     long volumeId, long bucketId)
           throws IOException {
     // bucketID is the parent
-    long parentID = bucketID;
+    long parentID = bucketId;
     for (int indx = 0; indx < dirs.size(); indx++) {
       String dirName = dirs.get(indx);
       String dbKey = "";
       // for index=0, parentID is bucketID
-      dbKey = omMetadataManager.getOzonePathKey(parentID, dirName);
+      dbKey = omMetadataManager.getOzonePathKey(volumeId, bucketId,
+              parentID, dirName);
       OmDirectoryInfo omDirInfo =
               omMetadataManager.getDirectoryTable().get(dbKey);
       Assert.assertNotNull("Invalid directory!", omDirInfo);
@@ -620,15 +653,17 @@ public class TestOMDirectoryCreateRequestWithFSO {
     }
   }
 
-  private void verifyDirectoriesNotInCache(List<String> dirs, long bucketID)
+  private void verifyDirectoriesNotInCache(List<String> dirs,
+                                           long volumeId, long bucketId)
           throws IOException {
     // bucketID is the parent
-    long parentID = bucketID;
+    long parentID = bucketId;
     for (int indx = 0; indx < dirs.size(); indx++) {
       String dirName = dirs.get(indx);
       String dbKey = "";
       // for index=0, parentID is bucketID
-      dbKey = omMetadataManager.getOzonePathKey(parentID, dirName);
+      dbKey = omMetadataManager.getOzonePathKey(volumeId, bucketId,
+              parentID, dirName);
       CacheValue<OmDirectoryInfo> omDirInfoCacheValue =
               omMetadataManager.getDirectoryTable()
                       .getCacheValue(new CacheKey<>(dbKey));
@@ -654,4 +689,8 @@ public class TestOMDirectoryCreateRequestWithFSO {
             .setClientId(UUID.randomUUID().toString()).build();
   }
 
+  private BucketLayout getBucketLayout() {
+    return BucketLayout.FILE_SYSTEM_OPTIMIZED;
+  }
+
 }
