@@ -20,9 +20,9 @@ package org.apache.hadoop.ozone.om.request.volume;
 
 import java.util.UUID;
 
-import org.apache.commons.logging.LogFactory;
+import org.slf4j.LoggerFactory;
 import org.apache.hadoop.ozone.om.helpers.OmVolumeArgs;
-import org.apache.hadoop.ozone.om.request.TestOMRequestUtils;
+import org.apache.hadoop.ozone.om.request.OMRequestTestUtils;
 import org.apache.hadoop.ozone.om.response.OMClientResponse;
 import org.apache.ozone.test.GenericTestUtils;
 import org.junit.Assert;
@@ -44,7 +44,7 @@ public class TestOMVolumeSetQuotaRequest extends TestOMVolumeRequest {
     long quotaInBytes = 100L;
     long quotaInNamespace = 1000L;
     OMRequest originalRequest =
-        TestOMRequestUtils.createSetVolumePropertyRequest(volumeName,
+        OMRequestTestUtils.createSetVolumePropertyRequest(volumeName,
             quotaInBytes, quotaInNamespace);
 
     OMVolumeSetQuotaRequest omVolumeSetQuotaRequest =
@@ -62,11 +62,11 @@ public class TestOMVolumeSetQuotaRequest extends TestOMVolumeRequest {
     long quotaInBytes = 100L;
     long quotaInNamespace = 1000L;
 
-    TestOMRequestUtils.addUserToDB(volumeName, ownerName, omMetadataManager);
-    TestOMRequestUtils.addVolumeToDB(volumeName, ownerName, omMetadataManager);
+    OMRequestTestUtils.addUserToDB(volumeName, ownerName, omMetadataManager);
+    OMRequestTestUtils.addVolumeToDB(volumeName, ownerName, omMetadataManager);
 
     OMRequest originalRequest =
-        TestOMRequestUtils.createSetVolumePropertyRequest(volumeName,
+        OMRequestTestUtils.createSetVolumePropertyRequest(volumeName,
             quotaInBytes, quotaInNamespace);
 
     OMVolumeSetQuotaRequest omVolumeSetQuotaRequest =
@@ -108,7 +108,12 @@ public class TestOMVolumeSetQuotaRequest extends TestOMVolumeRequest {
         .getVolumeTable().get(volumeKey).getCreationTime();
     long modificationTime = omMetadataManager
         .getVolumeTable().get(volumeKey).getModificationTime();
-    Assert.assertTrue(modificationTime > creationTime);
+
+    // creationTime and modificationTime can be the same to the precision of a
+    // millisecond - since there is no time-consuming operation between
+    // OMRequestTestUtils.addVolumeToDB (sets creationTime) and
+    // preExecute (sets modificationTime).
+    Assert.assertTrue(modificationTime >= creationTime);
   }
 
   @Test
@@ -116,10 +121,10 @@ public class TestOMVolumeSetQuotaRequest extends TestOMVolumeRequest {
       throws Exception {
     String volumeName = UUID.randomUUID().toString();
     long quotaInBytes = 100L;
-    long quotaInNamespace= 100L;
+    long quotaInNamespace = 100L;
 
     OMRequest originalRequest =
-        TestOMRequestUtils.createSetVolumePropertyRequest(volumeName,
+        OMRequestTestUtils.createSetVolumePropertyRequest(volumeName,
             quotaInBytes, quotaInNamespace);
 
     OMVolumeSetQuotaRequest omVolumeSetQuotaRequest =
@@ -144,7 +149,7 @@ public class TestOMVolumeSetQuotaRequest extends TestOMVolumeRequest {
 
     // create request with owner set.
     OMRequest originalRequest =
-        TestOMRequestUtils.createSetVolumePropertyRequest(volumeName,
+        OMRequestTestUtils.createSetVolumePropertyRequest(volumeName,
             "user1");
 
     // Creating OMVolumeSetQuotaRequest with SetProperty request set with owner.
@@ -169,19 +174,21 @@ public class TestOMVolumeSetQuotaRequest extends TestOMVolumeRequest {
     String volumeName = UUID.randomUUID().toString();
     String bucketName = UUID.randomUUID().toString();
 
-    TestOMRequestUtils.addVolumeToDB(
+    OMRequestTestUtils.addVolumeToDB(
         volumeName, omMetadataManager, 10 * GB);
-    TestOMRequestUtils.addBucketToDB(
+    OMRequestTestUtils.addBucketToDB(
         volumeName, bucketName, omMetadataManager, 8 * GB);
     OMRequest originalRequest =
-        TestOMRequestUtils.createSetVolumePropertyRequest(volumeName,
+        OMRequestTestUtils.createSetVolumePropertyRequest(volumeName,
             5 * GB, 100L);
 
     OMVolumeSetQuotaRequest omVolumeSetQuotaRequest =
         new OMVolumeSetQuotaRequest(originalRequest);
 
-    GenericTestUtils.LogCapturer logs = GenericTestUtils.LogCapturer
-        .captureLogs(LogFactory.getLog(OMVolumeSetQuotaRequest.class));
+    GenericTestUtils.LogCapturer logs =
+            GenericTestUtils.LogCapturer.captureLogs(
+                    LoggerFactory.getLogger(OMVolumeSetQuotaRequest.class)
+            );
 
     OMClientResponse omClientResponse = omVolumeSetQuotaRequest
         .validateAndUpdateCache(ozoneManager, 1,
