@@ -18,16 +18,18 @@
 
 package org.apache.hadoop.ozone.om.response.key;
 
-import org.apache.hadoop.ozone.om.helpers.OmBucketInfo;
-import org.apache.hadoop.util.Time;
+import org.apache.hadoop.ozone.OmUtils;
+import org.apache.hadoop.ozone.om.helpers.RepeatedOmKeyInfo;
 import org.jetbrains.annotations.NotNull;
 import org.junit.Assert;
 import org.junit.Test;
 
 import org.apache.hadoop.ozone.om.helpers.OmKeyInfo;
-import org.apache.hadoop.ozone.om.request.TestOMRequestUtils;
+import org.apache.hadoop.ozone.om.request.OMRequestTestUtils;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos;
 
+import java.io.IOException;
+
 /**
  * Tests OMKeyCommitResponse.
  */
@@ -37,10 +39,6 @@ public class TestOMKeyCommitResponse extends TestOMKeyResponse {
   @Test
   public void testAddToDBBatch() throws Exception {
 
-    omBucketInfo = OmBucketInfo.newBuilder()
-        .setVolumeName(volumeName).setBucketName(bucketName)
-        .setCreationTime(Time.now()).build();
-
     OmKeyInfo omKeyInfo = getOmKeyInfo();
 
     OzoneManagerProtocolProtos.OMResponse omResponse =
@@ -60,7 +58,7 @@ public class TestOMKeyCommitResponse extends TestOMKeyResponse {
 
     String ozoneKey = getOzoneKey();
     OMKeyCommitResponse omKeyCommitResponse = getOmKeyCommitResponse(
-            omKeyInfo, omResponse, openKey, ozoneKey);
+            omKeyInfo, omResponse, openKey, ozoneKey, keysToDelete);
 
     omKeyCommitResponse.addToDBBatch(omMetadataManager, batchOperation);
 
@@ -70,17 +68,15 @@ public class TestOMKeyCommitResponse extends TestOMKeyResponse {
     // When key commit key is deleted from openKey table and added to keyTable.
     Assert.assertFalse(
         omMetadataManager.getOpenKeyTable(getBucketLayout()).isExist(openKey));
-    Assert.assertTrue(omMetadataManager.getKeyTable().isExist(ozoneKey));
+    Assert.assertTrue(
+        omMetadataManager.getKeyTable(getBucketLayout()).isExist(ozoneKey));
   }
 
   @Test
   public void testAddToDBBatchNoOp() throws Exception {
 
-    OmKeyInfo omKeyInfo = TestOMRequestUtils.createOmKeyInfo(volumeName,
+    OmKeyInfo omKeyInfo = OMRequestTestUtils.createOmKeyInfo(volumeName,
         bucketName, keyName, replicationType, replicationFactor);
-    omBucketInfo = OmBucketInfo.newBuilder()
-        .setVolumeName(volumeName).setBucketName(bucketName)
-        .setCreationTime(Time.now()).build();
 
     OzoneManagerProtocolProtos.OMResponse omResponse =
         OzoneManagerProtocolProtos.OMResponse.newBuilder().setCommitKeyResponse(
@@ -93,7 +89,7 @@ public class TestOMKeyCommitResponse extends TestOMKeyResponse {
     String ozoneKey = getOzoneKey();
 
     OMKeyCommitResponse omKeyCommitResponse = getOmKeyCommitResponse(
-            omKeyInfo, omResponse, openKey, ozoneKey);
+            omKeyInfo, omResponse, openKey, ozoneKey, null);
 
     // As during commit Key, entry will be already there in openKeyTable.
     // Adding it here.
@@ -112,17 +108,33 @@ public class TestOMKeyCommitResponse extends TestOMKeyResponse {
     // openKey table.
     Assert.assertTrue(
         omMetadataManager.getOpenKeyTable(getBucketLayout()).isExist(openKey));
-    Assert.assertFalse(omMetadataManager.getKeyTable().isExist(ozoneKey));
+    Assert.assertFalse(
+        omMetadataManager.getKeyTable(getBucketLayout()).isExist(ozoneKey));
+  }
+
+  @Test
+  public void testAddToDBBatchOnOverwrite() throws Exception {
+    OmKeyInfo omKeyInfo = getOmKeyInfo();
+    keysToDelete =
+            OmUtils.prepareKeyForDelete(omKeyInfo, null, 100, false);
+    Assert.assertNotNull(keysToDelete);
+    testAddToDBBatch();
+
+    RepeatedOmKeyInfo keysInDeleteTable =
+            omMetadataManager.getDeletedTable().get(getOzoneKey());
+    Assert.assertNotNull(keysInDeleteTable);
+    Assert.assertEquals(1, keysInDeleteTable.getOmKeyInfoList().size());
+
   }
 
   @NotNull
   protected void addKeyToOpenKeyTable() throws Exception {
-    TestOMRequestUtils.addKeyToTable(true, volumeName, bucketName, keyName,
+    OMRequestTestUtils.addKeyToTable(true, volumeName, bucketName, keyName,
             clientID, replicationType, replicationFactor, omMetadataManager);
   }
 
   @NotNull
-  protected String getOzoneKey() {
+  protected String getOzoneKey() throws IOException {
     Assert.assertNotNull(omBucketInfo);
     return omMetadataManager.getOzoneKey(volumeName,
             omBucketInfo.getBucketName(), keyName);
@@ -131,9 +143,9 @@ public class TestOMKeyCommitResponse extends TestOMKeyResponse {
   @NotNull
   protected OMKeyCommitResponse getOmKeyCommitResponse(OmKeyInfo omKeyInfo,
           OzoneManagerProtocolProtos.OMResponse omResponse, String openKey,
-          String ozoneKey) {
+          String ozoneKey, RepeatedOmKeyInfo deleteKeys) throws IOException {
     Assert.assertNotNull(omBucketInfo);
     return new OMKeyCommitResponse(omResponse, omKeyInfo, ozoneKey, openKey,
-            omBucketInfo);
+            omBucketInfo, deleteKeys);
   }
 }
