@@ -18,13 +18,20 @@
 
 package org.apache.hadoop.ozone.om.response.key;
 
+import java.io.IOException;
 import java.util.Random;
 import java.util.UUID;
 
+import com.google.common.base.Optional;
+import org.apache.hadoop.hdds.protocol.StorageType;
+import org.apache.hadoop.hdds.utils.db.cache.CacheKey;
+import org.apache.hadoop.hdds.utils.db.cache.CacheValue;
 import org.apache.hadoop.ozone.om.helpers.BucketLayout;
 import org.apache.hadoop.ozone.om.helpers.OmBucketInfo;
 import org.apache.hadoop.ozone.om.helpers.OmKeyInfo;
-import org.apache.hadoop.ozone.om.request.TestOMRequestUtils;
+import org.apache.hadoop.ozone.om.helpers.OmVolumeArgs;
+import org.apache.hadoop.ozone.om.helpers.RepeatedOmKeyInfo;
+import org.apache.hadoop.ozone.om.request.OMRequestTestUtils;
 import org.jetbrains.annotations.NotNull;
 import org.junit.After;
 import org.junit.Before;
@@ -59,6 +66,7 @@ public class TestOMKeyResponse {
   protected long clientID;
   protected Random random;
   protected long txnLogId = 100000L;
+  protected RepeatedOmKeyInfo keysToDelete;
 
   @Before
   public void setup() throws Exception {
@@ -75,17 +83,42 @@ public class TestOMKeyResponse {
     replicationType = HddsProtos.ReplicationType.RATIS;
     clientID = 1000L;
     random = new Random();
+    keysToDelete = null;
+
+    final OmVolumeArgs volumeArgs = OmVolumeArgs.newBuilder()
+            .setVolume(volumeName)
+            .setAdminName("admin")
+            .setOwnerName("owner")
+            .setObjectID(System.currentTimeMillis())
+            .build();
+
+    omMetadataManager.getVolumeTable().addCacheEntry(
+            new CacheKey<>(omMetadataManager.getVolumeKey(volumeName)),
+            new CacheValue<>(Optional.of(volumeArgs), 1));
+
+    omBucketInfo = OmBucketInfo.newBuilder()
+            .setVolumeName(volumeName)
+            .setBucketName(bucketName)
+            .setObjectID(System.currentTimeMillis())
+            .setStorageType(StorageType.DISK)
+            .setIsVersionEnabled(false)
+            .build();
+
+    omMetadataManager.getBucketTable().addCacheEntry(
+            new CacheKey<>(omMetadataManager.getBucketKey(
+                    volumeName, bucketName)),
+            new CacheValue<>(Optional.of(omBucketInfo), 1));
   }
 
   @NotNull
-  protected String getOpenKeyName() {
+  protected String getOpenKeyName()  throws IOException {
     return omMetadataManager.getOpenKey(volumeName, bucketName, keyName,
             clientID);
   }
 
   @NotNull
   protected OmKeyInfo getOmKeyInfo() {
-    return TestOMRequestUtils.createOmKeyInfo(volumeName, bucketName, keyName,
+    return OMRequestTestUtils.createOmKeyInfo(volumeName, bucketName, keyName,
             replicationType, replicationFactor);
   }
 
