@@ -18,24 +18,52 @@
 
 package org.apache.hadoop.ozone.om.response.key;
 
-import org.junit.Assert;
-import org.junit.Test;
+import org.apache.hadoop.ozone.om.helpers.BucketLayout;
+import org.apache.hadoop.ozone.om.helpers.OzoneFSUtils;
+import org.apache.hadoop.ozone.om.request.OMRequestTestUtils;
 
 import org.apache.hadoop.ozone.om.helpers.OmKeyInfo;
-import org.apache.hadoop.ozone.om.request.TestOMRequestUtils;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.Status;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.OMResponse;
+import org.junit.Assert;
+import org.junit.Test;
+import org.junit.runner.RunWith;
+import org.junit.runners.Parameterized;
+import org.junit.runners.Parameterized.Parameters;
 
+import java.util.Arrays;
+import java.util.Collection;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.UUID;
 
+import static org.apache.hadoop.ozone.om.request.OMRequestTestUtils.addBucketToDB;
+
 /**
  * Tests OMOpenKeysDeleteResponse.
  */
+@RunWith(Parameterized.class)
 public class TestOMOpenKeysDeleteResponse extends TestOMKeyResponse {
   private static final long KEY_LENGTH = 100;
+  private final BucketLayout bucketLayout;
+
+  public TestOMOpenKeysDeleteResponse(BucketLayout bucketLayout) {
+    this.bucketLayout = bucketLayout;
+  }
+
+  @Override
+  public BucketLayout getBucketLayout() {
+    return bucketLayout;
+  }
+
+  @Parameters
+  public static Collection<BucketLayout> bucketLayouts() {
+    return Arrays.asList(
+        BucketLayout.DEFAULT,
+        BucketLayout.FILE_SYSTEM_OPTIMIZED
+    );
+  }
 
   /**
    * Tests deleting a subset of keys from the open key table DB when the keys
@@ -43,8 +71,11 @@ public class TestOMOpenKeysDeleteResponse extends TestOMKeyResponse {
    */
   @Test
   public void testAddToDBBatchWithEmptyBlocks() throws Exception {
+    OMRequestTestUtils.addVolumeAndBucketToDB(volumeName, bucketName,
+            omMetadataManager, getBucketLayout());
     Map<String, OmKeyInfo> keysToDelete = addOpenKeysToDB(volumeName, 3);
     Map<String, OmKeyInfo> keysToKeep = addOpenKeysToDB(volumeName, 3);
+
     createAndCommitResponse(keysToDelete, Status.OK);
 
     for (String key: keysToDelete.keySet()) {
@@ -69,6 +100,8 @@ public class TestOMOpenKeysDeleteResponse extends TestOMKeyResponse {
    */
   @Test
   public void testAddToDBBatchWithNonEmptyBlocks() throws Exception {
+    OMRequestTestUtils.addVolumeAndBucketToDB(volumeName, bucketName,
+            omMetadataManager, getBucketLayout());
     Map<String, OmKeyInfo> keysToDelete = addOpenKeysToDB(volumeName, 3,
         KEY_LENGTH);
     Map<String, OmKeyInfo> keysToKeep = addOpenKeysToDB(volumeName, 3,
@@ -99,7 +132,10 @@ public class TestOMOpenKeysDeleteResponse extends TestOMKeyResponse {
    */
   @Test
   public void testAddToDBBatchWithErrorResponse() throws Exception {
+    OMRequestTestUtils.addVolumeAndBucketToDB(volumeName, bucketName,
+            omMetadataManager, getBucketLayout());
     Map<String, OmKeyInfo> keysToDelete = addOpenKeysToDB(volumeName, 3);
+
     createAndCommitResponse(keysToDelete, Status.INTERNAL_ERROR);
 
     for (String key: keysToDelete.keySet()) {
@@ -127,7 +163,7 @@ public class TestOMOpenKeysDeleteResponse extends TestOMKeyResponse {
         .build();
 
     OMOpenKeysDeleteResponse response = new OMOpenKeysDeleteResponse(omResponse,
-        keysToDelete, true);
+        keysToDelete, true, getBucketLayout());
 
     // Operations are only added to the batch by this method when status is OK.
     response.checkAndUpdateDB(omMetadataManager, batchOperation);
@@ -162,23 +198,37 @@ public class TestOMOpenKeysDeleteResponse extends TestOMKeyResponse {
     for (int i = 0; i < numKeys; i++) {
       String bucket = UUID.randomUUID().toString();
       String key = UUID.randomUUID().toString();
+      addBucketToDB(volume, bucket, omMetadataManager, getBucketLayout());
       long clientID = random.nextLong();
+      long parentID = random.nextLong();
 
-      OmKeyInfo omKeyInfo = TestOMRequestUtils.createOmKeyInfo(volume,
+      OmKeyInfo omKeyInfo = OMRequestTestUtils.createOmKeyInfo(volume,
           bucket, key, replicationType, replicationFactor);
 
       if (keyLength > 0) {
-        TestOMRequestUtils.addKeyLocationInfo(omKeyInfo, 0, keyLength);
+        OMRequestTestUtils.addKeyLocationInfo(omKeyInfo, 0, keyLength);
       }
 
-      String openKey = omMetadataManager.getOpenKey(volume, bucket,
-          key, clientID);
+      final String openKey;
 
       // Add to the open key table DB, not cache.
       // In a real execution, the open key would have been removed from the
       // cache by the request, and it would only remain in the DB.
-      TestOMRequestUtils.addKeyToTable(true, false, omKeyInfo,
-          clientID, 0L, omMetadataManager);
+      if (getBucketLayout().isFileSystemOptimized()) {
+        String file = OzoneFSUtils.getFileName(key);
+        final long volumeId = omMetadataManager.getVolumeId(volume);
+        final long bucketId = omMetadataManager.getBucketId(volume, bucket);
+        omKeyInfo.setFileName(file);
+        omKeyInfo.setParentObjectID(parentID);
+        OMRequestTestUtils.addFileToKeyTable(true, false, file, omKeyInfo,
+            clientID, 0L, omMetadataManager);
+        openKey = omMetadataManager.getOpenFileName(
+                volumeId, bucketId, parentID, file, clientID);
+      } else {
+        OMRequestTestUtils.addKeyToTable(true, false, omKeyInfo,
+            clientID, 0L, omMetadataManager);
+        openKey = omMetadataManager.getOpenKey(volume, bucket, key, clientID);
+      }
       Assert.assertTrue(omMetadataManager.getOpenKeyTable(getBucketLayout())
           .isExist(openKey));
 
