@@ -37,8 +37,11 @@ import org.apache.hadoop.io.Text;
 import org.apache.hadoop.ozone.OzoneConsts;
 import org.apache.hadoop.ozone.om.OMMetadataManager;
 import org.apache.hadoop.ozone.om.OmMetadataManagerImpl;
+import org.apache.hadoop.ozone.om.OzoneManager;
 import org.apache.hadoop.ozone.om.S3SecretManager;
 import org.apache.hadoop.ozone.om.S3SecretManagerImpl;
+import org.apache.hadoop.ozone.om.exceptions.OMLeaderNotReadyException;
+import org.apache.hadoop.ozone.om.exceptions.OMNotLeaderException;
 import org.apache.hadoop.ozone.om.helpers.S3SecretValue;
 import org.apache.hadoop.security.AccessControlException;
 import org.apache.hadoop.security.ssl.KeyStoreTestUtil;
@@ -55,12 +58,14 @@ import org.junit.Before;
 import org.junit.Rule;
 import org.junit.Test;
 import org.junit.rules.TemporaryFolder;
+import org.mockito.Mockito;
 
 /**
  * Test class for {@link OzoneDelegationTokenSecretManager}.
  */
 public class TestOzoneDelegationTokenSecretManager {
 
+  private OzoneManager om;
   private OzoneDelegationTokenSecretManager secretManager;
   private SecurityConfig securityConfig;
   private CertificateClient certificateClient;
@@ -87,11 +92,12 @@ public class TestOzoneDelegationTokenSecretManager {
     final Map<String, String> s3Secrets = new HashMap<>();
     s3Secrets.put("testuser1", s3Secret);
     s3Secrets.put("abc", "djakjahkd");
+    om = Mockito.mock(OzoneManager.class);
     OMMetadataManager metadataManager = new OmMetadataManagerImpl(conf);
     s3SecretManager = new S3SecretManagerImpl(conf, metadataManager) {
       @Override
       public S3SecretValue getS3Secret(String kerberosID) {
-        if(s3Secrets.containsKey(kerberosID)) {
+        if (s3Secrets.containsKey(kerberosID)) {
           return new S3SecretValue(kerberosID, s3Secrets.get(kerberosID));
         }
         return null;
@@ -99,7 +105,7 @@ public class TestOzoneDelegationTokenSecretManager {
 
       @Override
       public String getS3UserSecretString(String awsAccessKey) {
-        if(s3Secrets.containsKey(awsAccessKey)) {
+        if (s3Secrets.containsKey(awsAccessKey)) {
           return s3Secrets.get(awsAccessKey);
         }
         return null;
@@ -161,6 +167,39 @@ public class TestOzoneDelegationTokenSecretManager {
     secretManager.stop();
   }
 
+  @Test
+  public void testLeadershipCheckinRetrievePassword() throws Exception {
+    secretManager = createSecretManager(conf, tokenMaxLifetime,
+        expiryTime, tokenRemoverScanInterval);
+    Mockito.doThrow(new OMNotLeaderException("Not leader"))
+        .when(om).checkLeaderStatus();
+    OzoneTokenIdentifier identifier = new OzoneTokenIdentifier();
+    try {
+      secretManager.retrievePassword(identifier);
+    } catch (Exception e) {
+      Assert.assertEquals(SecretManager.InvalidToken.class, e.getClass());
+      Assert.assertEquals(OMNotLeaderException.class, e.getCause().getClass());
+    }
+
+    Mockito.doThrow(new OMLeaderNotReadyException("Leader not ready"))
+        .when(om).checkLeaderStatus();
+    try {
+      secretManager.retrievePassword(identifier);
+    } catch (Exception e) {
+      Assert.assertEquals(SecretManager.InvalidToken.class, e.getClass());
+      Assert.assertEquals(OMLeaderNotReadyException.class,
+          e.getCause().getClass());
+    }
+
+    Mockito.doNothing().when(om).checkLeaderStatus();
+    try {
+      secretManager.retrievePassword(identifier);
+    } catch (Exception e) {
+      Assert.assertEquals(SecretManager.InvalidToken.class, e.getClass());
+      Assert.assertNull(e.getCause());
+    }
+  }
+
   @Test
   public void testCreateToken() throws Exception {
     secretManager = createSecretManager(conf, tokenMaxLifetime,
@@ -322,7 +361,7 @@ public class TestOzoneDelegationTokenSecretManager {
     OzoneTokenIdentifier id = new OzoneTokenIdentifier();
     // set invalid om cert serial id
     id.setOmCertSerialId("1927393");
-    id.setMaxDate(Time.now() + 60*60*24);
+    id.setMaxDate(Time.now() + 60 * 60 * 24);
     id.setOwner(new Text("test"));
     Assert.assertFalse(secretManager.verifySignature(id, id.getBytes()));
   }
@@ -413,6 +452,7 @@ public class TestOzoneDelegationTokenSecretManager {
         .setTokenRenewInterval(expiry)
         .setTokenRemoverScanInterval(tokenRemoverScanTime)
         .setService(serviceRpcAdd)
+        .setOzoneManager(om)
         .setS3SecretManager(s3SecretManager)
         .setCertificateClient(certificateClient)
         .setOmServiceId(OzoneConsts.OM_SERVICE_ID_DEFAULT)
