@@ -21,25 +21,24 @@ import org.apache.commons.lang3.RandomUtils;
 import org.apache.hadoop.hdds.client.StandaloneReplicationConfig;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
-import org.apache.hadoop.hdds.scm.protocol.ScmBlockLocationProtocol;
+import org.apache.hadoop.hdds.server.OzoneAdmins;
 import org.apache.hadoop.hdds.utils.db.cache.CacheKey;
 import org.apache.hadoop.hdds.utils.db.cache.CacheValue;
 import org.apache.hadoop.ozone.OzoneAcl;
-import org.apache.hadoop.ozone.om.BucketManagerImpl;
-import org.apache.hadoop.ozone.om.IOzoneAcl;
-import org.apache.hadoop.ozone.om.KeyManagerImpl;
+import org.apache.hadoop.ozone.om.BucketManager;
+import org.apache.hadoop.ozone.om.KeyManager;
 import org.apache.hadoop.ozone.om.OMMetadataManager;
-import org.apache.hadoop.ozone.om.OmMetadataManagerImpl;
+import org.apache.hadoop.ozone.om.OmTestManagers;
 import org.apache.hadoop.ozone.om.PrefixManager;
-import org.apache.hadoop.ozone.om.PrefixManagerImpl;
-import org.apache.hadoop.ozone.om.VolumeManagerImpl;
+import org.apache.hadoop.ozone.om.VolumeManager;
 import org.apache.hadoop.ozone.om.exceptions.OMException;
 import org.apache.hadoop.ozone.om.helpers.OmBucketInfo;
 import org.apache.hadoop.ozone.om.helpers.OmKeyArgs;
 import org.apache.hadoop.ozone.om.helpers.OmVolumeArgs;
 import org.apache.hadoop.ozone.om.helpers.OpenKeySession;
 import org.apache.hadoop.ozone.om.helpers.OzoneAclUtil;
-import org.apache.hadoop.ozone.om.request.TestOMRequestUtils;
+import org.apache.hadoop.ozone.om.protocol.OzoneManagerProtocol;
+import org.apache.hadoop.ozone.om.request.OMRequestTestUtils;
 import org.apache.hadoop.ozone.security.acl.IAccessAuthorizer.ACLIdentityType;
 import org.apache.hadoop.ozone.security.acl.IAccessAuthorizer.ACLType;
 import org.apache.hadoop.security.UserGroupInformation;
@@ -77,9 +76,9 @@ import static org.apache.hadoop.ozone.security.acl.OzoneObj.ResourceType.KEY;
 import static org.apache.hadoop.ozone.security.acl.OzoneObj.ResourceType.PREFIX;
 import static org.apache.hadoop.ozone.security.acl.OzoneObj.ResourceType.VOLUME;
 import static org.apache.hadoop.ozone.security.acl.OzoneObj.StoreType.OZONE;
-import static org.junit.Assert.*;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertTrue;
-import static org.mockito.Mockito.mock;
 
 /**
  * Test class for {@link OzoneNativeAuthorizer}.
@@ -96,9 +95,10 @@ public class TestOzoneNativeAuthorizer {
   private ACLType parentDirGroupAcl;
   private boolean expectedAclResult;
 
-  private static KeyManagerImpl keyManager;
-  private static VolumeManagerImpl volumeManager;
-  private static BucketManagerImpl bucketManager;
+  private static OzoneManagerProtocol writeClient;
+  private static KeyManager keyManager;
+  private static VolumeManager volumeManager;
+  private static BucketManager bucketManager;
   private static PrefixManager prefixManager;
   private static OMMetadataManager metadataManager;
   private static OzoneNativeAuthorizer nativeAuthorizer;
@@ -150,17 +150,17 @@ public class TestOzoneNativeAuthorizer {
     ozConfig.set(OZONE_METADATA_DIRS, dir.toString());
     ozConfig.set(OZONE_ADMINISTRATORS, "om");
 
-    metadataManager = new OmMetadataManagerImpl(ozConfig);
-    volumeManager = new VolumeManagerImpl(metadataManager, ozConfig);
-    bucketManager = new BucketManagerImpl(metadataManager);
-    prefixManager = new PrefixManagerImpl(metadataManager, false);
-
-    keyManager = new KeyManagerImpl(mock(ScmBlockLocationProtocol.class),
-        metadataManager, ozConfig, "om1", null);
-
+    OmTestManagers omTestManagers =
+        new OmTestManagers(ozConfig);
+    metadataManager = omTestManagers.getMetadataManager();
+    volumeManager = omTestManagers.getVolumeManager();
+    bucketManager = omTestManagers.getBucketManager();
+    prefixManager = omTestManagers.getPrefixManager();
+    keyManager = omTestManagers.getKeyManager();
+    writeClient = omTestManagers.getWriteClient();
     nativeAuthorizer = new OzoneNativeAuthorizer(volumeManager, bucketManager,
         keyManager, prefixManager,
-        Collections.singletonList("om"));
+        new OzoneAdmins(Collections.singletonList("om")));
     adminUgi = UserGroupInformation.createUserForTesting("om",
         new String[]{"ozone"});
     testUgi = UserGroupInformation.createUserForTesting("testuser",
@@ -173,22 +173,22 @@ public class TestOzoneNativeAuthorizer {
         .setVolumeName(volume)
         .setBucketName(bucket)
         .setKeyName(keyName)
-        .setReplicationConfig(
-            new StandaloneReplicationConfig(HddsProtos.ReplicationFactor.ONE))
+        .setReplicationConfig(StandaloneReplicationConfig
+            .getInstance(HddsProtos.ReplicationFactor.ONE))
         .setDataSize(0)
         .setAcls(OzoneAclUtil.getAclList(testUgi.getUserName(),
             testUgi.getGroupNames(), ALL, ALL))
         .build();
 
     if (keyName.split(OZONE_URI_DELIMITER).length > 1) {
-      keyManager.createDirectory(keyArgs);
+      writeClient.createDirectory(keyArgs);
       key = key + OZONE_URI_DELIMITER;
     } else {
-      OpenKeySession keySession = keyManager.createFile(keyArgs, true, false);
+      OpenKeySession keySession = writeClient.createFile(keyArgs, true, false);
       keyArgs.setLocationInfoList(
           keySession.getKeyInfo().getLatestVersionLocations()
               .getLocationList());
-      keyManager.commitKey(keyArgs, keySession.getId());
+      writeClient.commitKey(keyArgs, keySession.getId());
     }
 
     keyObj = new OzoneObjInfo.Builder()
@@ -206,7 +206,7 @@ public class TestOzoneNativeAuthorizer {
         .setVolumeName(volumeName)
         .setBucketName(bucketName)
         .build();
-    TestOMRequestUtils.addBucketToOM(metadataManager, bucketInfo);
+    OMRequestTestUtils.addBucketToOM(metadataManager, bucketInfo);
     buckObj = new OzoneObjInfo.Builder()
         .setVolumeName(vol)
         .setBucketName(buck)
@@ -221,7 +221,7 @@ public class TestOzoneNativeAuthorizer {
         .setAdminName(adminUgi.getUserName())
         .setOwnerName(testUgi.getUserName())
         .build();
-    TestOMRequestUtils.addVolumeToOM(metadataManager, volumeArgs);
+    OMRequestTestUtils.addVolumeToOM(metadataManager, volumeArgs);
     volObj = new OzoneObjInfo.Builder()
         .setVolumeName(vol)
         .setResType(VOLUME)
@@ -232,10 +232,10 @@ public class TestOzoneNativeAuthorizer {
   @Test
   public void testCheckAccessForVolume() throws Exception {
     expectedAclResult = true;
-    resetAclsAndValidateAccess(volObj, USER, volumeManager);
-    resetAclsAndValidateAccess(volObj, GROUP, volumeManager);
-    resetAclsAndValidateAccess(volObj, WORLD, volumeManager);
-    resetAclsAndValidateAccess(volObj, ANONYMOUS, volumeManager);
+    resetAclsAndValidateAccess(volObj, USER, writeClient);
+    resetAclsAndValidateAccess(volObj, GROUP, writeClient);
+    resetAclsAndValidateAccess(volObj, WORLD, writeClient);
+    resetAclsAndValidateAccess(volObj, ANONYMOUS, writeClient);
   }
 
   @Test
@@ -251,10 +251,10 @@ public class TestOzoneNativeAuthorizer {
     setVolumeAcl(Arrays.asList(userAcl, groupAcl));
 
 
-    resetAclsAndValidateAccess(buckObj, USER, bucketManager);
-    resetAclsAndValidateAccess(buckObj, GROUP, bucketManager);
-    resetAclsAndValidateAccess(buckObj, WORLD, bucketManager);
-    resetAclsAndValidateAccess(buckObj, ANONYMOUS, bucketManager);
+    resetAclsAndValidateAccess(buckObj, USER, writeClient);
+    resetAclsAndValidateAccess(buckObj, GROUP, writeClient);
+    resetAclsAndValidateAccess(buckObj, WORLD, writeClient);
+    resetAclsAndValidateAccess(buckObj, ANONYMOUS, writeClient);
   }
 
   @Test
@@ -269,10 +269,10 @@ public class TestOzoneNativeAuthorizer {
     setVolumeAcl(Arrays.asList(userAcl, groupAcl));
     setBucketAcl(Arrays.asList(userAcl, groupAcl));
 
-    resetAclsAndValidateAccess(keyObj, USER, keyManager);
-    resetAclsAndValidateAccess(keyObj, GROUP, keyManager);
-    resetAclsAndValidateAccess(keyObj, WORLD, keyManager);
-    resetAclsAndValidateAccess(keyObj, ANONYMOUS, keyManager);
+    resetAclsAndValidateAccess(keyObj, USER, writeClient);
+    resetAclsAndValidateAccess(keyObj, GROUP, writeClient);
+    resetAclsAndValidateAccess(keyObj, WORLD, writeClient);
+    resetAclsAndValidateAccess(keyObj, ANONYMOUS, writeClient);
   }
 
   @Test
@@ -297,10 +297,10 @@ public class TestOzoneNativeAuthorizer {
     setBucketAcl(Arrays.asList(userAcl, groupAcl));
 
 
-    resetAclsAndValidateAccess(prefixObj, USER, prefixManager);
-    resetAclsAndValidateAccess(prefixObj, GROUP, prefixManager);
-    resetAclsAndValidateAccess(prefixObj, WORLD, prefixManager);
-    resetAclsAndValidateAccess(prefixObj, ANONYMOUS, prefixManager);
+    resetAclsAndValidateAccess(prefixObj, USER, writeClient);
+    resetAclsAndValidateAccess(prefixObj, GROUP, writeClient);
+    resetAclsAndValidateAccess(prefixObj, WORLD, writeClient);
+    resetAclsAndValidateAccess(prefixObj, ANONYMOUS, writeClient);
   }
 
 
@@ -347,12 +347,12 @@ public class TestOzoneNativeAuthorizer {
   }
 
   private void resetAclsAndValidateAccess(OzoneObj obj,
-      ACLIdentityType accessType, IOzoneAcl aclImplementor)
+      ACLIdentityType accessType, OzoneManagerProtocol aclImplementor)
       throws IOException {
     List<OzoneAcl> acls;
     String user = testUgi.getUserName();
     String group = (testUgi.getGroups().size() > 0) ?
-        testUgi.getGroups().get(0): "";
+        testUgi.getGroups().get(0) : "";
 
     RequestContext.Builder builder = new RequestContext.Builder()
         .setClientUgi(testUgi)
@@ -375,7 +375,7 @@ public class TestOzoneNativeAuthorizer {
       // Reset acls to only one right.
       if (obj.getResourceType() == VOLUME) {
         setVolumeAcl(Collections.singletonList(newAcl));
-      } else if (obj.getResourceType() == BUCKET){
+      } else if (obj.getResourceType() == BUCKET) {
         setBucketAcl(Collections.singletonList(newAcl));
       } else {
         aclImplementor.setAcl(obj, Collections.singletonList(newAcl));
@@ -401,7 +401,8 @@ public class TestOzoneNativeAuthorizer {
       String msg = "Acl to check:" + a1 + " accessType:" +
           accessType + " path:" + obj.getPath();
       if (a1.equals(CREATE) && obj.getResourceType().equals(VOLUME)) {
-        assertEquals(msg, nativeAuthorizer.getOzoneAdmins().contains(user),
+        assertEquals(msg, nativeAuthorizer.getOzoneAdmins()
+                         .getAdminUsernames().contains(user),
             nativeAuthorizer.checkAccess(obj,
                 builder.setAclRights(a1).build()));
       } else {
@@ -453,7 +454,7 @@ public class TestOzoneNativeAuthorizer {
           // only DB not cache.
           if (obj.getResourceType() == VOLUME) {
             addVolumeAcl(addAcl);
-          } else if (obj.getResourceType() == BUCKET){
+          } else if (obj.getResourceType() == BUCKET) {
             addBucketAcl(addAcl);
           } else {
             aclImplementor.addAcl(obj, addAcl);
@@ -522,7 +523,7 @@ public class TestOzoneNativeAuthorizer {
     allAcls.remove(NONE);
     RequestContext ctx = builder.build();
     boolean expectedResult = expectedAclResult;
-    if (nativeAuthorizer.getOzoneAdmins().contains(
+    if (nativeAuthorizer.getOzoneAdmins().getAdminUsernames().contains(
         ctx.getClientUgi().getUserName())) {
       expectedResult = true;
     }
