@@ -54,6 +54,7 @@ import org.slf4j.LoggerFactory;
 
 import java.io.FileNotFoundException;
 import java.io.IOException;
+import java.io.InputStream;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.util.ArrayList;
@@ -194,8 +195,11 @@ public class BasicRootedOzoneFileSystem extends FileSystem {
     statistics.incrementReadOps(1);
     LOG.trace("open() path: {}", path);
     final String key = pathToKey(path);
-    return new FSDataInputStream(
-        new OzoneFSInputStream(adapter.readFile(key), statistics));
+    return new FSDataInputStream(createFSInputStream(adapter.readFile(key)));
+  }
+
+  protected InputStream createFSInputStream(InputStream inputStream) {
+    return new OzoneFSInputStream(inputStream, statistics);
   }
 
   protected void incrementCounter(Statistic statistic) {
@@ -310,7 +314,7 @@ public class BasicRootedOzoneFileSystem extends FileSystem {
       throw new IOException("Cannot rename a key to a different bucket");
     }
     OzoneBucket bucket = adapterImpl.getBucket(ofsSrc, false);
-    if (OzoneFSUtils.isFSOptimizedBucket(bucket.getBucketLayout())) {
+    if (bucket.getBucketLayout().isFileSystemOptimized()) {
       return renameFSO(bucket, ofsSrc, ofsDst);
     }
 
@@ -468,6 +472,68 @@ public class BasicRootedOzoneFileSystem extends FileSystem {
     }
   }
 
+  /**
+   * To be used only by recursiveBucketDelete().
+   */
+  private class DeleteIteratorWithFSO extends OzoneListingIterator {
+    private final OzoneBucket bucket;
+    private final BasicRootedOzoneClientAdapterImpl adapterImpl;
+    private boolean recursive;
+    private Path f;
+    DeleteIteratorWithFSO(Path f, boolean recursive)
+        throws IOException {
+      super(f, true);
+      this.f = f;
+      this.recursive = recursive;
+      // Initialize bucket here to reduce number of RPC calls
+      OFSPath ofsPath = new OFSPath(f);
+      adapterImpl = (BasicRootedOzoneClientAdapterImpl) adapter;
+      this.bucket = adapterImpl.getBucket(ofsPath, false);
+      LOG.debug("Deleting bucket with name {} is via DeleteIteratorWithFSO.",
+          bucket.getName());
+    }
+
+    @Override
+    boolean processKeyPath(List<String> keyPathList) throws IOException {
+      LOG.trace("Deleting keys: {}", keyPathList);
+      boolean succeed = keyPathList.isEmpty();
+      if (recursive && !succeed) {
+        succeed = adapterImpl.deleteObjects(keyPathList);
+      } else {
+        // Non empty paths cannot be deleted when recursive flag is false
+        if (!keyPathList.isEmpty()) {
+          throw new PathIsNotEmptyDirectoryException(f.toString());
+        }
+      }
+      return succeed;
+    }
+  }
+
+  private class DeleteIteratorFactory {
+    private Path path;
+    private boolean recursive;
+    private OFSPath ofsPath;
+
+    DeleteIteratorFactory(Path f, boolean recursive) {
+      this.path = f;
+      this.recursive = recursive;
+      this.ofsPath = new OFSPath(f);
+    }
+
+    OzoneListingIterator getDeleteIterator()
+        throws IOException {
+      OzoneListingIterator deleteIterator;
+      if (ofsPath.isBucket() &&
+          isFSObucket(ofsPath.getVolumeName(), ofsPath.getBucketName())) {
+        deleteIterator = new DeleteIteratorWithFSO(path, recursive);
+      } else {
+        deleteIterator = new DeleteIterator(path, recursive);
+      }
+      return deleteIterator;
+    }
+  }
+
+
   /**
    * Deletes the children of the input dir path by iterating though the
    * DeleteIterator.
@@ -479,7 +545,8 @@ public class BasicRootedOzoneFileSystem extends FileSystem {
   private boolean innerDelete(Path f, boolean recursive) throws IOException {
     LOG.trace("delete() path:{} recursive:{}", f, recursive);
     try {
-      DeleteIterator iterator = new DeleteIterator(f, recursive);
+      OzoneListingIterator iterator =
+          new DeleteIteratorFactory(f, recursive).getDeleteIterator();
       return iterator.iterate();
     } catch (FileNotFoundException e) {
       if (LOG.isDebugEnabled()) {
@@ -489,6 +556,7 @@ public class BasicRootedOzoneFileSystem extends FileSystem {
     }
   }
 
+
   /**
    * {@inheritDoc}
    *
@@ -509,10 +577,6 @@ public class BasicRootedOzoneFileSystem extends FileSystem {
       return false;
     }
 
-    if (status == null) {
-      return false;
-    }
-
     String key = pathToKey(f);
     boolean result;
 
@@ -532,7 +596,7 @@ public class BasicRootedOzoneFileSystem extends FileSystem {
 
       if (!ofsPath.isVolume() && !ofsPath.isBucket()) {
         OzoneBucket bucket = adapterImpl.getBucket(ofsPath, false);
-        if (OzoneFSUtils.isFSOptimizedBucket(bucket.getBucketLayout())) {
+        if (bucket.getBucketLayout().isFileSystemOptimized()) {
           String ofsKeyPath = ofsPath.getNonKeyPathNoPrefixDelim() +
               OZONE_URI_DELIMITER + ofsPath.getKeyName();
           return adapterImpl.deleteObject(ofsKeyPath, recursive);
@@ -567,7 +631,19 @@ public class BasicRootedOzoneFileSystem extends FileSystem {
         }
       }
 
-      result = innerDelete(f, recursive);
+      boolean isBucketLink = false;
+      // check for bucket link
+      if (ofsPath.isBucket()) {
+        isBucketLink = adapterImpl.getBucket(ofsPath, false)
+            .isLink();
+      }
+
+      // if link, don't delete contents
+      if (isBucketLink) {
+        result = true;
+      } else {
+        result = innerDelete(f, recursive);
+      }
 
       // Handle delete bucket
       if (ofsPath.isBucket()) {
@@ -600,6 +676,14 @@ public class BasicRootedOzoneFileSystem extends FileSystem {
     return result;
   }
 
+  private boolean isFSObucket(String volumeName, String bucketName)
+      throws IOException {
+    OzoneVolume volume =
+        adapterImpl.getObjectStore().getVolume(volumeName);
+    OzoneBucket bucket = volume.getBucket(bucketName);
+    return bucket.getBucketLayout().isFileSystemOptimized();
+  }
+
   /**
    * Create a fake parent directory key if it does not already exist and no
    * other child of this parent directory exists.
@@ -783,12 +867,16 @@ public class BasicRootedOzoneFileSystem extends FileSystem {
     try {
       fileStatus = convertFileStatus(
           adapter.getFileStatus(key, uri, qualifiedPath, getUsername()));
-    } catch (OMException ex) {
-      if (ex.getResult().equals(OMException.ResultCodes.KEY_NOT_FOUND) ||
-          ex.getResult().equals(OMException.ResultCodes.BUCKET_NOT_FOUND) ||
-          ex.getResult().equals(OMException.ResultCodes.VOLUME_NOT_FOUND)) {
-        throw new FileNotFoundException("File not found. path:" + f);
+    } catch (IOException e) {
+      if (e instanceof OMException) {
+        OMException ex = (OMException) e;
+        if (ex.getResult().equals(OMException.ResultCodes.KEY_NOT_FOUND) ||
+            ex.getResult().equals(OMException.ResultCodes.BUCKET_NOT_FOUND) ||
+            ex.getResult().equals(OMException.ResultCodes.VOLUME_NOT_FOUND)) {
+          throw new FileNotFoundException("File not found. path:" + f);
+        }
       }
+      throw e;
     }
     return fileStatus;
   }
@@ -832,7 +920,8 @@ public class BasicRootedOzoneFileSystem extends FileSystem {
   @Override
   public FileChecksum getFileChecksum(Path f, long length) throws IOException {
     incrementCounter(Statistic.INVOCATION_GET_FILE_CHECKSUM);
-    return super.getFileChecksum(f, length);
+    String key = pathToKey(f);
+    return adapter.getFileChecksum(key, length);
   }
 
   @Override
@@ -876,6 +965,133 @@ public class BasicRootedOzoneFileSystem extends FileSystem {
     return super.listLocatedStatus(f);
   }
 
+  @Override
+  public RemoteIterator<FileStatus> listStatusIterator(Path f)
+      throws IOException {
+    return new OzoneFileStatusIterator<>(f);
+  }
+
+  /**
+   * A private class implementation for iterating list of file status.
+   *
+   * @param <T> the type of the file status.
+   */
+  private final class OzoneFileStatusIterator<T extends FileStatus>
+      implements RemoteIterator<T> {
+    private List<FileStatus> thisListing;
+    private int i;
+    private Path p;
+    private T curStat = null;
+    private String startPath = "";
+
+    /**
+     * Constructor to initialize OzoneFileStatusIterator.
+     * Get the first batch of entry for iteration.
+     *
+     * @param p path to file/directory.
+     * @throws IOException
+     */
+    private OzoneFileStatusIterator(Path p) throws IOException {
+      this.p = p;
+      // fetch the first batch of entries in the directory
+      thisListing = listFileStatus(p, startPath);
+      if (thisListing != null && !thisListing.isEmpty()) {
+        startPath = pathToKey(
+            thisListing.get(thisListing.size() - 1).getPath());
+        LOG.debug("Got {} file status, next start path {}",
+            thisListing.size(), startPath);
+      }
+      i = 0;
+    }
+
+    /**
+     * @return true if next entry exists false otherwise.
+     * @throws IOException
+     */
+    @Override
+    public boolean hasNext() throws IOException {
+      while (curStat == null && hasNextNoFilter()) {
+        T next;
+        FileStatus fileStat = thisListing.get(i++);
+        next = (T) (fileStat);
+        curStat = next;
+      }
+      return curStat != null;
+    }
+
+    /**
+     * Checks the next available entry from partial listing if not exhausted
+     * or fetches new batch for listing.
+     *
+     * @return true if next entry exists false otherwise.
+     * @throws IOException
+     */
+    private boolean hasNextNoFilter() throws IOException {
+      if (thisListing == null) {
+        return false;
+      }
+      if (i >= thisListing.size()) {
+        if (startPath != null && (thisListing.size() == LISTING_PAGE_SIZE ||
+            thisListing.size() == LISTING_PAGE_SIZE - 1)) {
+          // current listing is exhausted & fetch a new listing
+          thisListing = listFileStatus(p, startPath);
+          if (thisListing != null && !thisListing.isEmpty()) {
+            startPath = pathToKey(
+                thisListing.get(thisListing.size() - 1).getPath());
+            LOG.debug("Got {} file status, next start path {}",
+                thisListing.size(), startPath);
+          } else {
+            return false;
+          }
+          i = 0;
+        }
+      }
+      return (i < thisListing.size());
+    }
+
+    /**
+     * @return next entry.
+     * @throws IOException
+     */
+    @Override
+    public T next() throws IOException {
+      if (hasNext()) {
+        T tmp = curStat;
+        curStat = null;
+        return tmp;
+      }
+      throw new java.util.NoSuchElementException("No more entry in " + p);
+    }
+  }
+
+  /**
+   * Get all the file status for input path and startPath.
+   *
+   * @param f
+   * @param startPath
+   * @return list of file status.
+   * @throws IOException
+   */
+  private List<FileStatus> listFileStatus(Path f, String startPath)
+      throws IOException {
+    incrementCounter(Statistic.INVOCATION_LIST_STATUS, 1);
+    statistics.incrementReadOps(1);
+    LOG.trace("listFileStatus() path:{}", f);
+    List<FileStatus> statusList;
+    statusList =
+        adapter.listStatus(pathToKey(f), false, startPath,
+            LISTING_PAGE_SIZE, uri, workingDir, getUsername())
+            .stream()
+            .map(this::convertFileStatus)
+            .collect(Collectors.toList());
+
+    if (!statusList.isEmpty() && !startPath.isEmpty()) {
+      // Excluding the 1st file status element from list.
+      statusList.remove(0);
+    }
+    return statusList;
+  }
+
   /**
    * Turn a path (relative or otherwise) into an Ozone key.
    *
@@ -942,19 +1158,29 @@ public class BasicRootedOzoneFileSystem extends FileSystem {
     private final Path path;
     private final FileStatus status;
     private String pathKey;
-    private Iterator<BasicKeyInfo> keyIterator;
+    private Iterator<BasicKeyInfo> keyIterator = null;
+    private boolean isFSO;
 
-    OzoneListingIterator(Path path)
+    OzoneListingIterator(Path path, boolean isFSO)
         throws IOException {
       this.path = path;
       this.status = getFileStatus(path);
       this.pathKey = pathToKey(path);
-      if (status.isDirectory()) {
-        this.pathKey = addTrailingSlashIfNeeded(pathKey);
+      this.isFSO = isFSO;
+      if (!isFSO) {
+        if (status.isDirectory()) {
+          this.pathKey = addTrailingSlashIfNeeded(pathKey);
+        }
+        keyIterator = adapter.listKeys(pathKey);
       }
-      keyIterator = adapter.listKeys(pathKey);
     }
 
+    OzoneListingIterator(Path path) throws IOException {
+      // non FSO implementation
+      this(path, false);
+    }
+
+
     /**
      * The output of processKey determines if further iteration through the
      * keys should be done or not.
@@ -972,6 +1198,9 @@ public class BasicRootedOzoneFileSystem extends FileSystem {
      * stopped and returned with false indicating that all the keys could not
      * be processed successfully.
      *
+     * If isFSO is true, call is from DeleteIteratorWithFSO and the list of keys
+     * will only contain immediate children i.e top level dirs and files
+     *
      * @return true if all keys are processed successfully, false otherwise.
      * @throws IOException
      */
@@ -985,15 +1214,13 @@ public class BasicRootedOzoneFileSystem extends FileSystem {
         OFSPath ofsPath = new OFSPath(pathKey);
         String ofsPathPrefix =
             ofsPath.getNonKeyPathNoPrefixDelim() + OZONE_URI_DELIMITER;
-        while (keyIterator.hasNext()) {
-          BasicKeyInfo key = keyIterator.next();
-          // Convert key to full path before passing it to processKeyPath
-          // TODO: This conversion is redundant. But want to use only full path
-          //  outside AdapterImpl. - Maybe a refactor later.
-          String keyPath = ofsPathPrefix + key.getName();
-          LOG.trace("iterating key path: {}", keyPath);
-          if (!key.getName().equals("")) {
-            keyPathList.add(keyPath);
+        if (isFSO) {
+          FileStatus[] fileStatuses;
+          fileStatuses = listStatus(path);
+          for (FileStatus fileStatus : fileStatuses) {
+            String keyName =
+                new OFSPath(fileStatus.getPath().toString()).getKeyName();
+            keyPathList.add(ofsPathPrefix + keyName);
           }
           if (keyPathList.size() >= batchSize) {
             if (!processKeyPath(keyPathList)) {
@@ -1002,6 +1229,25 @@ public class BasicRootedOzoneFileSystem extends FileSystem {
               keyPathList.clear();
             }
           }
+        } else {
+          while (keyIterator.hasNext()) {
+            BasicKeyInfo key = keyIterator.next();
+            // Convert key to full path before passing it to processKeyPath
+            // TODO: This conversion is redundant. But want to use only
+            //  full path outside AdapterImpl. - Maybe a refactor later.
+            String keyPath = ofsPathPrefix + key.getName();
+            LOG.trace("iterating key path: {}", keyPath);
+            if (!key.getName().equals("")) {
+              keyPathList.add(keyPath);
+            }
+            if (keyPathList.size() >= batchSize) {
+              if (!processKeyPath(keyPathList)) {
+                return false;
+              } else {
+                keyPathList.clear();
+              }
+            }
+          }
         }
         if (keyPathList.size() > 0) {
           if (!processKeyPath(keyPathList)) {
