@@ -18,7 +18,6 @@
 
 package org.apache.hadoop.ozone.recon.api;
 
-import org.apache.hadoop.hdds.client.ReplicationConfig;
 import org.apache.hadoop.hdds.scm.pipeline.Pipeline;
 import org.apache.hadoop.hdds.scm.server.OzoneStorageContainerManager;
 import org.apache.hadoop.ozone.recon.MetricsServiceProviderFactory;
@@ -40,7 +39,6 @@ import java.io.IOException;
 import java.time.Instant;
 import java.util.ArrayList;
 import java.util.List;
-import java.util.Optional;
 import java.util.TreeMap;
 import java.util.UUID;
 
@@ -105,9 +103,7 @@ public class PipelineEndpoint {
               .setDatanodes(datanodes)
               .setDuration(duration)
               .setStatus(pipeline.getPipelineState())
-              .setReplicationFactor(ReplicationConfig
-                  .getLegacyFactor(pipeline.getReplicationConfig()).getNumber())
-          .setReplicationType(pipeline.getType().toString());
+              .setReplicationConfig(pipeline.getReplicationConfig());
       // If any metrics service providers like Prometheus
       // is configured, then query it for metrics and populate
       // leader election count and last leader election time
@@ -116,13 +112,13 @@ public class PipelineEndpoint {
         // ex. group id of 48981bf7-8bea-4fbd-9857-79df51ee872d
         // is group-79DF51EE872D
         String[] splits = pipelineId.toString().split("-");
-        String groupId = "group-" + splits[splits.length-1].toUpperCase();
-        Optional<Long> leaderElectionCount = getMetricValue(
-            "ratis_leader_election_electionCount", groupId);
-        leaderElectionCount.ifPresent(pipelineBuilder::setLeaderElections);
-        Optional<Long> leaderElectionTime = getMetricValue(
-            "ratis_leader_election_lastLeaderElectionTime", groupId);
-        leaderElectionTime.ifPresent(pipelineBuilder::setLastLeaderElection);
+        String groupId = "group-" + splits[splits.length - 1].toUpperCase();
+        Long leaderElectionCount = getElectionCountMetricValue(groupId);
+        pipelineBuilder.setLeaderElections(leaderElectionCount);
+        Long lastLeaderElectionElapsedTime
+                = getLastLeaderElectionElapsedTimeMetricValue(groupId,
+                pipeline.getLeaderId());
+        pipelineBuilder.setLastLeaderElection(lastLeaderElectionElapsedTime);
       }
 
       pipelinesList.add(pipelineBuilder.build());
@@ -133,25 +129,53 @@ public class PipelineEndpoint {
     return Response.ok(pipelinesResponse).build();
   }
 
-  private Optional<Long> getMetricValue(String metricName, String groupId) {
+  private Long getElectionCountMetricValue(String groupId) {
+    Long electionCount = 0L;
     String metricsQuery = String.format(
-        "query=%s{group=\"%s\"}", metricName, groupId);
+            "query=ratis_leader_election_electionCount{group=\"%s\"}", groupId);
     try {
       List<Metric> metrics = metricsServiceProvider.getMetricsInstant(
-          metricsQuery);
+              metricsQuery);
       if (!metrics.isEmpty()) {
-        TreeMap<Double, Double> values = (TreeMap<Double, Double>)
-            metrics.get(0).getValues();
-        if (!values.isEmpty()) {
-          return Optional.of(values.firstEntry().getValue().longValue());
+        for (Metric m : metrics) {
+          TreeMap<Double, Double> values = (TreeMap<Double, Double>)
+                  m.getValues();
+          if (!values.isEmpty()) {
+            electionCount += values.firstEntry().getValue().longValue();
+          }
         }
       }
     } catch (Exception ex) {
       if (LOG.isErrorEnabled()) {
-        LOG.error(String.format("Unable to get metrics value for %s",
-            metricName), ex);
+        LOG.error("Unable to get metrics value for " +
+                "ratis_leader_election_electionCount", ex);
       }
     }
-    return Optional.empty();
+    return electionCount;
+  }
+
+  private Long getLastLeaderElectionElapsedTimeMetricValue(String groupId,
+                                                           UUID uuid) {
+    String metricsQuery = String.format(
+            "query=ratis_leader_election_lastLeaderElectionElapsedTime{group=" +
+                    "\"%s\",exported_instance=\"%s\"}", groupId,
+            uuid.toString());
+    try {
+      List<Metric> metrics = metricsServiceProvider.getMetricsInstant(
+              metricsQuery);
+      if (!metrics.isEmpty()) {
+        TreeMap<Double, Double> values = (TreeMap<Double, Double>)
+                metrics.get(0).getValues();
+        if (!values.isEmpty()) {
+          return values.firstEntry().getValue().longValue();
+        }
+      }
+    } catch (Exception ex) {
+      if (LOG.isErrorEnabled()) {
+        LOG.error("Unable to get metrics value for " +
+                "ratis_leader_election_lastLeaderElectionElapsedTime", ex);
+      }
+    }
+    return 0L;
   }
 }
