@@ -30,14 +30,21 @@ public class ContainerHistory implements Serializable {
   private String datanodeHost;
   private long firstSeenTime;
   private long lastSeenTime;
+  private long bcsId;
 
   public ContainerHistory(long containerId, String datanodeUuid,
-      String datanodeHost, long firstSeenTime, long lastSeenTime) {
+                          String datanodeHost, long firstSeenTime,
+                          long lastSeenTime, long lastBcsId) {
     this.containerId = containerId;
     this.datanodeUuid = datanodeUuid;
     this.datanodeHost = datanodeHost;
     this.firstSeenTime = firstSeenTime;
     this.lastSeenTime = lastSeenTime;
+    this.bcsId = lastBcsId;
+  }
+
+  public long getLastBcsId() {
+    return bcsId;
   }
 
   public long getContainerId() {
