@@ -23,6 +23,7 @@ import org.apache.hadoop.hdds.utils.db.IntegerCodec;
 import org.apache.hadoop.hdds.utils.db.LongCodec;
 import org.apache.hadoop.ozone.recon.ReconServerConfigKeys;
 import org.apache.hadoop.ozone.recon.api.types.ContainerKeyPrefix;
+import org.apache.hadoop.ozone.recon.api.types.KeyPrefixContainer;
 import org.apache.hadoop.ozone.recon.codec.ContainerReplicaHistoryListCodec;
 import org.apache.hadoop.ozone.recon.codec.NSSummaryCodec;
 import org.apache.hadoop.ozone.recon.scm.ContainerReplicaHistoryList;
@@ -48,6 +49,15 @@ public class ReconDBDefinition implements DBDefinition {
           Integer.class,
           new IntegerCodec());
 
+  public static final DBColumnFamilyDefinition<KeyPrefixContainer, Integer>
+      KEY_CONTAINER =
+      new DBColumnFamilyDefinition<>(
+          "keyContainerTable",
+          KeyPrefixContainer.class,
+          new KeyPrefixContainerCodec(),
+          Integer.class,
+          new IntegerCodec());
+
   public static final DBColumnFamilyDefinition<Long, Long>
       CONTAINER_KEY_COUNT =
       new DBColumnFamilyDefinition<>(
@@ -74,6 +84,16 @@ public class ReconDBDefinition implements DBDefinition {
           NSSummary.class,
           new NSSummaryCodec());
 
+  // Container Replica History with bcsId tracking.
+  public static final DBColumnFamilyDefinition
+      <Long, ContainerReplicaHistoryList> REPLICA_HISTORY_V2 =
+      new DBColumnFamilyDefinition<Long, ContainerReplicaHistoryList>(
+          "replica_history_v2",
+          Long.class,
+          new LongCodec(),
+          ContainerReplicaHistoryList.class,
+          new ContainerReplicaHistoryListCodec());
+
   @Override
   public String getName() {
     return dbName;
@@ -87,6 +107,7 @@ public class ReconDBDefinition implements DBDefinition {
   @Override
   public DBColumnFamilyDefinition[] getColumnFamilies() {
     return new DBColumnFamilyDefinition[] {
-        CONTAINER_KEY, CONTAINER_KEY_COUNT, REPLICA_HISTORY, NAMESPACE_SUMMARY};
+        CONTAINER_KEY, KEY_CONTAINER, CONTAINER_KEY_COUNT, REPLICA_HISTORY,
+        NAMESPACE_SUMMARY, REPLICA_HISTORY_V2};
   }
 }
