@@ -23,20 +23,26 @@ import static org.apache.hadoop.ozone.om.OmMetadataManagerImpl.KEY_TABLE;
 import java.io.IOException;
 import java.time.Duration;
 import java.time.Instant;
+import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
 import java.util.Set;
 
 import org.apache.commons.lang3.tuple.ImmutablePair;
 import org.apache.commons.lang3.tuple.Pair;
+import org.apache.hadoop.hdds.utils.db.RDBBatchOperation;
 import org.apache.hadoop.ozone.om.OMMetadataManager;
+import org.apache.hadoop.ozone.om.helpers.BucketLayout;
 import org.apache.hadoop.ozone.om.helpers.OmKeyInfo;
 import org.apache.hadoop.ozone.om.helpers.OmKeyLocationInfo;
 import org.apache.hadoop.ozone.om.helpers.OmKeyLocationInfoGroup;
 import org.apache.hadoop.ozone.recon.api.types.ContainerKeyPrefix;
+import org.apache.hadoop.ozone.recon.api.types.KeyPrefixContainer;
 import org.apache.hadoop.ozone.recon.spi.ReconContainerMetadataManager;
 import org.apache.hadoop.hdds.utils.db.Table;
 import org.apache.hadoop.hdds.utils.db.TableIterator;
@@ -69,6 +75,13 @@ public class ContainerKeyMapperTask implements ReconOmTask {
   @Override
   public Pair<String, Boolean> reprocess(OMMetadataManager omMetadataManager) {
     long omKeyCount = 0;
+    // Maps the (container, key) -> count
+    Map<ContainerKeyPrefix, Integer> containerKeyMap = new HashMap<>();
+    // Maps the key -> containerId
+    Map<Long, Long> containerKeyCountMap = new HashMap<>();
+    // List of the deleted (container, key) pair's
+    List<ContainerKeyPrefix> deletedKeyCountList = new ArrayList<>();
+
     try {
       LOG.info("Starting a 'reprocess' run of ContainerKeyMapperTask.");
       Instant start = Instant.now();
@@ -77,13 +90,15 @@ public class ContainerKeyMapperTask implements ReconOmTask {
       reconContainerMetadataManager
               .reinitWithNewContainerDataFromOm(new HashMap<>());
 
-      Table<String, OmKeyInfo> omKeyInfoTable = omMetadataManager.getKeyTable();
+      Table<String, OmKeyInfo> omKeyInfoTable =
+          omMetadataManager.getKeyTable(getBucketLayout());
       try (TableIterator<String, ? extends Table.KeyValue<String, OmKeyInfo>>
                keyIter = omKeyInfoTable.iterator()) {
         while (keyIter.hasNext()) {
           Table.KeyValue<String, OmKeyInfo> kv = keyIter.next();
           OmKeyInfo omKeyInfo = kv.getValue();
-          writeOMKeyToContainerDB(kv.getKey(), omKeyInfo);
+          handlePutOMKeyEvent(kv.getKey(), omKeyInfo, containerKeyMap,
+              containerKeyCountMap, deletedKeyCountList);
           omKeyCount++;
         }
       }
@@ -97,6 +112,12 @@ public class ContainerKeyMapperTask implements ReconOmTask {
           ioEx);
       return new ImmutablePair<>(getTaskName(), false);
     }
+    try {
+      writeToTheDB(containerKeyMap, containerKeyCountMap, deletedKeyCountList);
+    } catch (IOException e) {
+      LOG.error("Unable to write Container Key Prefix data in Recon DB.", e);
+      return new ImmutablePair<>(getTaskName(), false);
+    }
     return new ImmutablePair<>(getTaskName(), true);
   }
 
@@ -114,6 +135,9 @@ public class ContainerKeyMapperTask implements ReconOmTask {
     Iterator<OMDBUpdateEvent> eventIterator = events.getIterator();
     int eventCount = 0;
     final Collection<String> taskTables = getTaskTables();
+    Map<ContainerKeyPrefix, Integer> containerKeyMap = new HashMap<>();
+    Map<Long, Long> containerKeyCountMap = new HashMap<>();
+    List<ContainerKeyPrefix> deletedKeyCountList = new ArrayList<>();
 
     while (eventIterator.hasNext()) {
       OMDBUpdateEvent<String, OmKeyInfo> omdbUpdateEvent = eventIterator.next();
@@ -126,22 +150,26 @@ public class ContainerKeyMapperTask implements ReconOmTask {
       try {
         switch (omdbUpdateEvent.getAction()) {
         case PUT:
-          writeOMKeyToContainerDB(updatedKey, updatedKeyValue);
+          handlePutOMKeyEvent(updatedKey, updatedKeyValue, containerKeyMap,
+              containerKeyCountMap, deletedKeyCountList);
           break;
 
         case DELETE:
-          deleteOMKeyFromContainerDB(updatedKey);
+          handleDeleteOMKeyEvent(updatedKey, containerKeyMap,
+              containerKeyCountMap, deletedKeyCountList);
           break;
 
         case UPDATE:
           if (omdbUpdateEvent.getOldValue() != null) {
-            deleteOMKeyFromContainerDB(
-                omdbUpdateEvent.getOldValue().getKeyName());
+            handleDeleteOMKeyEvent(
+                omdbUpdateEvent.getOldValue().getKeyName(), containerKeyMap,
+                containerKeyCountMap, deletedKeyCountList);
           } else {
             LOG.warn("Update event does not have the old Key Info for {}.",
                 updatedKey);
           }
-          writeOMKeyToContainerDB(updatedKey, updatedKeyValue);
+          handlePutOMKeyEvent(updatedKey, updatedKeyValue, containerKeyMap,
+              containerKeyCountMap, deletedKeyCountList);
           break;
 
         default: LOG.debug("Skipping DB update event : {}",
@@ -154,59 +182,146 @@ public class ContainerKeyMapperTask implements ReconOmTask {
         return new ImmutablePair<>(getTaskName(), false);
       }
     }
+    try {
+      writeToTheDB(containerKeyMap, containerKeyCountMap, deletedKeyCountList);
+    } catch (IOException e) {
+      LOG.error("Unable to write Container Key Prefix data in Recon DB.", e);
+      return new ImmutablePair<>(getTaskName(), false);
+    }
     LOG.info("{} successfully processed {} OM DB update event(s).",
         getTaskName(), eventCount);
     return new ImmutablePair<>(getTaskName(), true);
   }
 
+  private void writeToTheDB(Map<ContainerKeyPrefix, Integer> containerKeyMap,
+                            Map<Long, Long> containerKeyCountMap,
+                            List<ContainerKeyPrefix> deletedContainerKeyList)
+      throws IOException {
+    try (RDBBatchOperation rdbBatchOperation = new RDBBatchOperation()) {
+      containerKeyMap.keySet().forEach((ContainerKeyPrefix key) -> {
+        try {
+          reconContainerMetadataManager
+              .batchStoreContainerKeyMapping(rdbBatchOperation, key,
+                  containerKeyMap.get(key));
+        } catch (IOException e) {
+          LOG.error("Unable to write Container Key Prefix data in Recon DB.",
+              e);
+        }
+      });
+
+
+      containerKeyCountMap.keySet().forEach((Long key) -> {
+        try {
+          reconContainerMetadataManager
+              .batchStoreContainerKeyCounts(rdbBatchOperation, key,
+                  containerKeyCountMap.get(key));
+        } catch (IOException e) {
+          LOG.error("Unable to write Container Key Prefix data in Recon DB.",
+              e);
+        }
+      });
+
+      deletedContainerKeyList.forEach((ContainerKeyPrefix key) -> {
+        try {
+          reconContainerMetadataManager
+              .batchDeleteContainerMapping(rdbBatchOperation, key);
+        } catch (IOException e) {
+          LOG.error("Unable to write Container Key Prefix data in Recon DB.",
+              e);
+        }
+      });
+
+      reconContainerMetadataManager.commitBatchOperation(rdbBatchOperation);
+    }
+  }
+
   /**
-   * Delete an OM Key from Container DB and update containerID -> no. of keys
-   * count.
+   * Note to delete an OM Key and update the containerID -> no. of keys counts
+   * (we are preparing for batch deletion in these data structures).
    *
    * @param key key String.
+   * @param containerKeyMap we keep the added containerKeys in this map
+   *                        (in this batch)
+   * @param containerKeyCountMap we keep the containerKey counts in this map
+   * @param deletedContainerKeyList list of the deleted containerKeys
    * @throws IOException If Unable to write to container DB.
    */
-  private void  deleteOMKeyFromContainerDB(String key)
+  private void handleDeleteOMKeyEvent(String key,
+                                      Map<ContainerKeyPrefix, Integer>
+                                          containerKeyMap,
+                                      Map<Long, Long> containerKeyCountMap,
+                                      List<ContainerKeyPrefix>
+                                          deletedContainerKeyList)
       throws IOException {
 
-    TableIterator<ContainerKeyPrefix, ? extends
-        Table.KeyValue<ContainerKeyPrefix, Integer>> containerIterator =
-        reconContainerMetadataManager.getContainerTableIterator();
-
     Set<ContainerKeyPrefix> keysToBeDeleted = new HashSet<>();
+    try (TableIterator<KeyPrefixContainer, ? extends
+        Table.KeyValue<KeyPrefixContainer, Integer>> keyContainerIterator =
+             reconContainerMetadataManager.getKeyContainerTableIterator()) {
 
-    while (containerIterator.hasNext()) {
-      Table.KeyValue<ContainerKeyPrefix, Integer> keyValue =
-          containerIterator.next();
-      String keyPrefix = keyValue.getKey().getKeyPrefix();
-      if (keyPrefix.equals(key)) {
-        keysToBeDeleted.add(keyValue.getKey());
+      // Check if we have keys in this container in the DB
+      keyContainerIterator.seek(new KeyPrefixContainer(key));
+      while (keyContainerIterator.hasNext()) {
+        Table.KeyValue<KeyPrefixContainer, Integer> keyValue =
+            keyContainerIterator.next();
+        String keyPrefix = keyValue.getKey().getKeyPrefix();
+        if (keyPrefix.equals(key)) {
+          if (keyValue.getKey().getContainerId() != -1) {
+            keysToBeDeleted.add(keyValue.getKey().toContainerKeyPrefix());
+          }
+        } else {
+          break;
+        }
       }
     }
 
+    // Check if we have keys in this container in our containerKeyMap
+    containerKeyMap.keySet()
+        .forEach((ContainerKeyPrefix containerKeyPrefix) -> {
+          String keyPrefix = containerKeyPrefix.getKeyPrefix();
+          if (keyPrefix.equals(key)) {
+            keysToBeDeleted.add(containerKeyPrefix);
+          }
+        });
+
     for (ContainerKeyPrefix containerKeyPrefix : keysToBeDeleted) {
-      reconContainerMetadataManager.deleteContainerMapping(containerKeyPrefix);
+      deletedContainerKeyList.add(containerKeyPrefix);
+      // Remove the container-key prefix from the map if we previously added
+      // it in this batch (and now we delete it)
+      containerKeyMap.remove(containerKeyPrefix);
 
       // decrement count and update containerKeyCount.
       Long containerID = containerKeyPrefix.getContainerId();
-      long keyCount =
-          reconContainerMetadataManager.getKeyCountForContainer(containerID);
+      long keyCount;
+      if (containerKeyCountMap.containsKey(containerID)) {
+        keyCount = containerKeyCountMap.get(containerID);
+      } else {
+        keyCount = reconContainerMetadataManager
+            .getKeyCountForContainer(containerID);
+      }
       if (keyCount > 0) {
-        reconContainerMetadataManager.storeContainerKeyCount(containerID,
-            --keyCount);
+        containerKeyCountMap.put(containerID, --keyCount);
       }
     }
   }
 
   /**
-   * Write an OM key to container DB and update containerID -> no. of keys
-   * count.
+   * Note to add an OM key and update containerID -> no. of keys count.
    *
    * @param key key String
    * @param omKeyInfo omKeyInfo value
+   * @param containerKeyMap we keep the added containerKeys in this map
+   *                        (in this batch)
+   * @param containerKeyCountMap we keep the containerKey counts in this map
+   * @param deletedContainerKeyList list of the deleted containerKeys
    * @throws IOException if unable to write to recon DB.
    */
-  private void  writeOMKeyToContainerDB(String key, OmKeyInfo omKeyInfo)
+  private void handlePutOMKeyEvent(String key, OmKeyInfo omKeyInfo,
+                                   Map<ContainerKeyPrefix, Integer>
+                                       containerKeyMap,
+                                   Map<Long, Long> containerKeyCountMap,
+                                   List<ContainerKeyPrefix>
+                                       deletedContainerKeyList)
       throws IOException {
     long containerCountToIncrement = 0;
     for (OmKeyLocationInfoGroup omKeyLocationInfoGroup : omKeyInfo
@@ -218,27 +333,36 @@ public class ContainerKeyMapperTask implements ReconOmTask {
         ContainerKeyPrefix containerKeyPrefix = new ContainerKeyPrefix(
             containerId, key, keyVersion);
         if (reconContainerMetadataManager.getCountForContainerKeyPrefix(
-            containerKeyPrefix) == 0) {
+            containerKeyPrefix) == 0
+            && !containerKeyMap.containsKey(containerKeyPrefix)) {
           // Save on writes. No need to save same container-key prefix
           // mapping again.
-          reconContainerMetadataManager.storeContainerKeyMapping(
-              containerKeyPrefix, 1);
+          containerKeyMap.put(containerKeyPrefix, 1);
+          // Remove the container-key prefix from the deleted list if we
+          // previously deleted it in this batch (and now we add it again)
+          deletedContainerKeyList.remove(containerKeyPrefix);
+
 
           // check if container already exists and
           // increment the count of containers if it does not exist
-          if (!reconContainerMetadataManager.doesContainerExists(containerId)) {
+          if (!reconContainerMetadataManager.doesContainerExists(containerId)
+              && !containerKeyCountMap.containsKey(containerId)) {
             containerCountToIncrement++;
           }
 
           // update the count of keys for the given containerID
-          long keyCount = reconContainerMetadataManager
-                  .getKeyCountForContainer(containerId);
+          long keyCount;
+          if (containerKeyCountMap.containsKey(containerId)) {
+            keyCount = containerKeyCountMap.get(containerId);
+          } else {
+            keyCount = reconContainerMetadataManager
+                .getKeyCountForContainer(containerId);
+          }
 
           // increment the count and update containerKeyCount.
           // keyCount will be 0 if containerID is not found. So, there is no
           // need to initialize keyCount for the first time.
-          reconContainerMetadataManager.storeContainerKeyCount(containerId,
-              ++keyCount);
+          containerKeyCountMap.put(containerId, ++keyCount);
         }
       }
     }
@@ -249,4 +373,8 @@ public class ContainerKeyMapperTask implements ReconOmTask {
     }
   }
 
+  private BucketLayout getBucketLayout() {
+    return BucketLayout.DEFAULT;
+  }
+
 }
