@@ -41,6 +41,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.UUID;
+import java.util.concurrent.TimeoutException;
 import java.util.stream.Collectors;
 import javax.ws.rs.WebApplicationException;
 import javax.ws.rs.core.Response;
@@ -58,6 +59,7 @@ import org.apache.hadoop.hdds.scm.pipeline.Pipeline;
 import org.apache.hadoop.hdds.scm.pipeline.PipelineID;
 import org.apache.hadoop.hdds.scm.server.OzoneStorageContainerManager;
 import org.apache.hadoop.ozone.om.OMMetadataManager;
+import org.apache.hadoop.ozone.om.helpers.BucketLayout;
 import org.apache.hadoop.ozone.om.helpers.OmKeyLocationInfo;
 import org.apache.hadoop.ozone.om.helpers.OmKeyLocationInfoGroup;
 import org.apache.hadoop.ozone.recon.ReconTestInjector;
@@ -81,7 +83,6 @@ import org.apache.hadoop.ozone.recon.spi.impl.OzoneManagerServiceProviderImpl;
 import org.apache.hadoop.ozone.recon.spi.impl.StorageContainerServiceProviderImpl;
 import org.apache.hadoop.ozone.recon.tasks.ContainerKeyMapperTask;
 import org.apache.hadoop.hdds.utils.db.Table;
-import org.hadoop.ozone.recon.schema.ContainerSchemaDefinition;
 import org.hadoop.ozone.recon.schema.ContainerSchemaDefinition.UnHealthyContainerStates;
 import org.hadoop.ozone.recon.schema.tables.pojos.UnhealthyContainers;
 import org.junit.Assert;
@@ -227,7 +228,8 @@ public class TestContainerEndpoint {
     OMMetadataManager omMetadataManagerMock = mock(OMMetadataManager.class);
     Table tableMock = mock(Table.class);
     when(tableMock.getName()).thenReturn("KeyTable");
-    when(omMetadataManagerMock.getKeyTable()).thenReturn(tableMock);
+    when(omMetadataManagerMock.getKeyTable(getBucketLayout()))
+        .thenReturn(tableMock);
     ContainerKeyMapperTask containerKeyMapperTask  =
         new ContainerKeyMapperTask(reconContainerMetadataManager);
     containerKeyMapperTask.reprocess(reconOMMetadataManager);
@@ -407,8 +409,8 @@ public class TestContainerEndpoint {
   }
 
   @Test
-  public void testGetMissingContainers() throws IOException {
-    Response response = containerEndpoint.getMissingContainers();
+  public void testGetMissingContainers() throws IOException, TimeoutException {
+    Response response = containerEndpoint.getMissingContainers(1000);
 
     MissingContainersResponse responseObject =
         (MissingContainersResponse) response.getEntity();
@@ -416,35 +418,34 @@ public class TestContainerEndpoint {
     assertEquals(0, responseObject.getTotalCount());
     assertEquals(Collections.EMPTY_LIST, responseObject.getContainers());
 
-    // Add missing containers to the database
-    long missingSince = System.currentTimeMillis();
-    UnhealthyContainers missing = new UnhealthyContainers();
-    missing.setContainerId(1L);
-    missing.setInStateSince(missingSince);
-    missing.setActualReplicaCount(0);
-    missing.setExpectedReplicaCount(3);
-    missing.setReplicaDelta(3);
-    missing.setContainerState(
-        ContainerSchemaDefinition.UnHealthyContainerStates.MISSING.toString());
-    ArrayList<UnhealthyContainers> missingList =
-        new ArrayList<UnhealthyContainers>();
-    missingList.add(missing);
-    containerHealthSchemaManager.insertUnhealthyContainerRecords(missingList);
+    putContainerInfos(5);
+    uuid1 = newDatanode("host1", "127.0.0.1");
+    uuid2 = newDatanode("host2", "127.0.0.2");
+    uuid3 = newDatanode("host3", "127.0.0.3");
+    uuid4 = newDatanode("host4", "127.0.0.4");
+    createUnhealthyRecords(5, 0, 0, 0);
 
-    putContainerInfos(1);
-    // Add container history for id 1
-    final UUID u1 = newDatanode("host1", "127.0.0.1");
-    final UUID u2 = newDatanode("host2", "127.0.0.2");
-    final UUID u3 = newDatanode("host3", "127.0.0.3");
-    final UUID u4 = newDatanode("host4", "127.0.0.4");
-    reconContainerManager.upsertContainerHistory(1L, u1, 1L);
-    reconContainerManager.upsertContainerHistory(1L, u2, 2L);
-    reconContainerManager.upsertContainerHistory(1L, u3, 3L);
-    reconContainerManager.upsertContainerHistory(1L, u4, 4L);
+    Response responseWithLimit = containerEndpoint.getMissingContainers(3);
+    MissingContainersResponse responseWithLimitObject
+            = (MissingContainersResponse) responseWithLimit.getEntity();
+    assertEquals(3, responseWithLimitObject.getTotalCount());
+    MissingContainerMetadata containerWithLimit =
+            responseWithLimitObject.getContainers().stream().findFirst()
+                    .orElse(null);
+    Assert.assertNotNull(containerWithLimit);
 
-    response = containerEndpoint.getMissingContainers();
+    Collection<MissingContainerMetadata> recordsWithLimit
+            = responseWithLimitObject.getContainers();
+    List<MissingContainerMetadata> missingWithLimit
+            = new ArrayList<>(recordsWithLimit);
+    assertEquals(3, missingWithLimit.size());
+    assertEquals(1L, missingWithLimit.get(0).getContainerID());
+    assertEquals(2L, missingWithLimit.get(1).getContainerID());
+    assertEquals(3L, missingWithLimit.get(2).getContainerID());
+
+    response = containerEndpoint.getMissingContainers(1000);
     responseObject = (MissingContainersResponse) response.getEntity();
-    assertEquals(1, responseObject.getTotalCount());
+    assertEquals(5, responseObject.getTotalCount());
     MissingContainerMetadata container =
         responseObject.getContainers().stream().findFirst().orElse(null);
     Assert.assertNotNull(container);
@@ -453,7 +454,6 @@ public class TestContainerEndpoint {
     assertEquals(keyCount, container.getKeys());
     assertEquals(pipelineID.getId(), container.getPipelineID());
     assertEquals(3, container.getReplicas().size());
-    assertEquals(missingSince, container.getMissingSince());
 
     Set<String> datanodes = Collections.unmodifiableSet(
         new HashSet<>(Arrays.asList("host2", "host3", "host4")));
@@ -467,7 +467,7 @@ public class TestContainerEndpoint {
     return new ContainerInfo.Builder()
         .setContainerID(containerId)
         .setReplicationConfig(
-            new RatisReplicationConfig(ReplicationFactor.THREE))
+            RatisReplicationConfig.getInstance(ReplicationFactor.THREE))
         .setState(HddsProtos.LifeCycleState.OPEN)
         .setOwner("owner1")
         .setNumberOfKeys(keyCount)
@@ -475,7 +475,7 @@ public class TestContainerEndpoint {
         .build();
   }
 
-  void putContainerInfos(int num) throws IOException {
+  void putContainerInfos(int num) throws IOException, TimeoutException {
     for (int i = 1; i <= num; i++) {
       final ContainerInfo info = newContainerInfo(i);
       reconContainerManager.addNewContainer(
@@ -484,7 +484,7 @@ public class TestContainerEndpoint {
   }
 
   @Test
-  public void testUnhealthyContainers() throws IOException {
+  public void testUnhealthyContainers() throws IOException, TimeoutException {
     Response response = containerEndpoint.getUnhealthyContainers(1000, 1);
 
     UnhealthyContainersResponse responseObject =
@@ -578,7 +578,8 @@ public class TestContainerEndpoint {
   }
 
   @Test
-  public void testUnhealthyContainersFilteredResponse() throws IOException {
+  public void testUnhealthyContainersFilteredResponse()
+      throws IOException, TimeoutException {
     String missing =  UnHealthyContainerStates.MISSING.toString();
 
     Response response = containerEndpoint
@@ -631,7 +632,8 @@ public class TestContainerEndpoint {
   }
 
   @Test
-  public void testUnhealthyContainersPaging() throws IOException {
+  public void testUnhealthyContainersPaging()
+      throws IOException, TimeoutException {
     putContainerInfos(6);
     uuid1 = newDatanode("host1", "127.0.0.1");
     uuid2 = newDatanode("host2", "127.0.0.2");
@@ -668,12 +670,12 @@ public class TestContainerEndpoint {
     final UUID u2 = newDatanode("host2", "127.0.0.2");
     final UUID u3 = newDatanode("host3", "127.0.0.3");
     final UUID u4 = newDatanode("host4", "127.0.0.4");
-    reconContainerManager.upsertContainerHistory(1L, u1, 1L);
-    reconContainerManager.upsertContainerHistory(1L, u2, 2L);
-    reconContainerManager.upsertContainerHistory(1L, u3, 3L);
-    reconContainerManager.upsertContainerHistory(1L, u4, 4L);
+    reconContainerManager.upsertContainerHistory(1L, u1, 1L, 1L);
+    reconContainerManager.upsertContainerHistory(1L, u2, 2L, 1L);
+    reconContainerManager.upsertContainerHistory(1L, u3, 3L, 1L);
+    reconContainerManager.upsertContainerHistory(1L, u4, 4L, 1L);
 
-    reconContainerManager.upsertContainerHistory(1L, u1, 5L);
+    reconContainerManager.upsertContainerHistory(1L, u1, 5L, 1L);
 
     Response response = containerEndpoint.getReplicaHistoryForContainer(1L);
     List<ContainerHistory> histories =
@@ -753,9 +755,13 @@ public class TestContainerEndpoint {
     missingList.add(missing);
     containerHealthSchemaManager.insertUnhealthyContainerRecords(missingList);
 
-    reconContainerManager.upsertContainerHistory(cID, uuid1, 1L);
-    reconContainerManager.upsertContainerHistory(cID, uuid2, 2L);
-    reconContainerManager.upsertContainerHistory(cID, uuid3, 3L);
-    reconContainerManager.upsertContainerHistory(cID, uuid4, 4L);
+    reconContainerManager.upsertContainerHistory(cID, uuid1, 1L, 1L);
+    reconContainerManager.upsertContainerHistory(cID, uuid2, 2L, 1L);
+    reconContainerManager.upsertContainerHistory(cID, uuid3, 3L, 1L);
+    reconContainerManager.upsertContainerHistory(cID, uuid4, 4L, 1L);
+  }
+
+  private BucketLayout getBucketLayout() {
+    return BucketLayout.DEFAULT;
   }
 }
