@@ -24,32 +24,25 @@ import javax.inject.Inject;
 import javax.ws.rs.WebApplicationException;
 import javax.ws.rs.container.ContainerRequestContext;
 import javax.ws.rs.core.Context;
+import javax.ws.rs.core.Response;
 import java.io.IOException;
-import java.nio.charset.StandardCharsets;
-import java.security.PrivilegedExceptionAction;
 
+import com.google.common.annotations.VisibleForTesting;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
-import org.apache.hadoop.io.Text;
-import org.apache.hadoop.ozone.OzoneSecurityUtil;
 import org.apache.hadoop.ozone.client.OzoneClient;
-import org.apache.hadoop.ozone.client.OzoneClientFactory;
+import org.apache.hadoop.ozone.om.protocol.S3Auth;
 import org.apache.hadoop.ozone.s3.exception.OS3Exception;
+import org.apache.hadoop.ozone.s3.exception.S3ErrorTable;
 import org.apache.hadoop.ozone.s3.signature.SignatureInfo;
 import org.apache.hadoop.ozone.s3.signature.SignatureInfo.Version;
 import org.apache.hadoop.ozone.s3.signature.SignatureProcessor;
 import org.apache.hadoop.ozone.s3.signature.StringToSignProducer;
-import org.apache.hadoop.ozone.security.OzoneTokenIdentifier;
-import org.apache.hadoop.security.UserGroupInformation;
-import org.apache.hadoop.security.token.Token;
-
-import com.google.common.annotations.VisibleForTesting;
-import static org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.OMTokenProto.Type.S3AUTHINFO;
-import static org.apache.hadoop.ozone.s3.exception.S3ErrorTable.INTERNAL_ERROR;
-import static org.apache.hadoop.ozone.s3.exception.S3ErrorTable.MALFORMED_HEADER;
-import org.jetbrains.annotations.NotNull;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import static org.apache.hadoop.ozone.s3.exception.S3ErrorTable.ACCESS_DENIED;
+import static org.apache.hadoop.ozone.s3.exception.S3ErrorTable.INTERNAL_ERROR;
+
 /**
  * This class creates the OzoneClient for the Rest endpoints.
  */
@@ -67,33 +60,25 @@ public class OzoneClientProducer {
   @Inject
   private OzoneConfiguration ozoneConfiguration;
 
-  @Inject
-  private Text omService;
-
-  @Inject
-  private String omServiceID;
-
   @Context
   private ContainerRequestContext context;
 
   @Produces
-  public OzoneClient createClient() throws WebApplicationException,
+  public synchronized OzoneClient createClient() throws WebApplicationException,
       IOException {
+    ozoneConfiguration.set("ozone.om.group.rights", "NONE");
     client = getClient(ozoneConfiguration);
     return client;
   }
 
   @PreDestroy
   public void destroy() throws IOException {
-    client.close();
+    client.getObjectStore().getClientProxy().clearThreadLocalS3Auth();
   }
-
-  private OzoneClient getClient(OzoneConfiguration config)
-      throws WebApplicationException {
-    OzoneClient ozoneClient = null;
+  @Produces
+  public S3Auth getSignature() {
     try {
       SignatureInfo signatureInfo = signatureProcessor.parseSignature();
-
       String stringToSign = "";
       if (signatureInfo.getVersion() == Version.V4) {
         stringToSign =
@@ -101,74 +86,46 @@ public class OzoneClientProducer {
       }
 
       String awsAccessId = signatureInfo.getAwsAccessId();
-      validateAccessId(awsAccessId);
-
-      UserGroupInformation remoteUser =
-          UserGroupInformation.createRemoteUser(awsAccessId);
-      if (OzoneSecurityUtil.isSecurityEnabled(config)) {
-        LOG.debug("Creating s3 auth info for client.");
-
-        if (signatureInfo.getVersion() == Version.NONE) {
-          throw MALFORMED_HEADER;
-        }
-
-        OzoneTokenIdentifier identifier = new OzoneTokenIdentifier();
-        identifier.setTokenType(S3AUTHINFO);
-        identifier.setStrToSign(stringToSign);
-        identifier.setSignature(signatureInfo.getSignature());
-        identifier.setAwsAccessId(awsAccessId);
-        identifier.setOwner(new Text(awsAccessId));
-        if (LOG.isTraceEnabled()) {
-          LOG.trace("Adding token for service:{}", omService);
-        }
-        Token<OzoneTokenIdentifier> token = new Token(identifier.getBytes(),
-            identifier.getSignature().getBytes(StandardCharsets.UTF_8),
-            identifier.getKind(),
-            omService);
-        remoteUser.addToken(token);
-
+      // ONLY validate aws access id when needed.
+      if (awsAccessId == null || awsAccessId.equals("")) {
+        LOG.debug("Malformed s3 header. awsAccessID: {}", awsAccessId);
+        throw ACCESS_DENIED;
       }
-      ozoneClient =
-          remoteUser.doAs((PrivilegedExceptionAction<OzoneClient>) () -> {
-            return createOzoneClient();
-          });
+
+      // Note: userPrincipal is initialized to be the same value as accessId,
+      //  could be updated later in RpcClient#getS3Volume
+      return new S3Auth(stringToSign,
+          signatureInfo.getSignature(),
+          awsAccessId, awsAccessId);
     } catch (OS3Exception ex) {
-      if (LOG.isDebugEnabled()) {
-        LOG.debug("Error during Client Creation: ", ex);
-      }
+      LOG.debug("Error during Client Creation: ", ex);
       throw wrapOS3Exception(ex);
+    } catch (Exception e) {
+      // For any other critical errors during object creation throw Internal
+      // error.
+      LOG.debug("Error during Client Creation: ", e);
+      throw wrapOS3Exception(S3ErrorTable.newError(INTERNAL_ERROR, null, e));
+    }
+  }
+
+  private OzoneClient getClient(OzoneConfiguration config)
+      throws IOException {
+    OzoneClient ozoneClient = null;
+    try {
+      ozoneClient =
+          OzoneClientCache.getOzoneClientInstance(ozoneConfiguration);
     } catch (Exception e) {
       // For any other critical errors during object creation throw Internal
       // error.
       if (LOG.isDebugEnabled()) {
         LOG.debug("Error during Client Creation: ", e);
       }
-      throw wrapOS3Exception(INTERNAL_ERROR);
+      throw e;
     }
     return ozoneClient;
   }
 
-  @NotNull
-  @VisibleForTesting
-  OzoneClient createOzoneClient() throws IOException {
-    if (omServiceID == null) {
-      return OzoneClientFactory.getRpcClient(ozoneConfiguration);
-    } else {
-      // As in HA case, we need to pass om service ID.
-      return OzoneClientFactory.getRpcClient(omServiceID,
-          ozoneConfiguration);
-    }
-  }
-
-  // ONLY validate aws access id when needed.
-  private void validateAccessId(String awsAccessId) throws Exception {
-    if (awsAccessId == null || awsAccessId.equals("")) {
-      LOG.error("Malformed s3 header. awsAccessID: ", awsAccessId);
-      throw wrapOS3Exception(MALFORMED_HEADER);
-    }
-  }
-
-  public void setOzoneConfiguration(OzoneConfiguration config) {
+  public synchronized void setOzoneConfiguration(OzoneConfiguration config) {
     this.ozoneConfiguration = config;
   }
 
@@ -178,7 +135,9 @@ public class OzoneClientProducer {
   }
 
   private WebApplicationException wrapOS3Exception(OS3Exception os3Exception) {
-    return new WebApplicationException(os3Exception,
-        os3Exception.getHttpCode());
+    return new WebApplicationException(os3Exception.getErrorMessage(),
+        os3Exception,
+        Response.status(os3Exception.getHttpCode())
+            .entity(os3Exception.toXml()).build());
   }
 }
