@@ -20,6 +20,7 @@
 
 package org.apache.hadoop.ozone.s3.endpoint;
 
+import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.ozone.OzoneConsts;
 import org.apache.hadoop.ozone.client.OzoneClient;
 import org.apache.hadoop.ozone.client.OzoneClientStub;
@@ -62,6 +63,7 @@ public class TestPartUpload {
 
     REST.setHeaders(headers);
     REST.setClient(client);
+    REST.setOzoneConfiguration(new OzoneConfiguration());
   }
 
 
