@@ -25,6 +25,7 @@ import org.apache.hadoop.ozone.client.ObjectStore;
 import org.apache.hadoop.ozone.client.OzoneBucket;
 import org.apache.hadoop.ozone.client.OzoneClient;
 import org.apache.hadoop.ozone.client.OzoneVolume;
+import org.apache.hadoop.ozone.client.protocol.ClientProtocol;
 import org.apache.hadoop.ozone.om.exceptions.OMException;
 import org.apache.hadoop.ozone.s3.exception.OS3Exception;
 import org.junit.Assert;
@@ -41,6 +42,7 @@ import java.util.Map;
 
 import static java.net.HttpURLConnection.HTTP_FORBIDDEN;
 import static org.mockito.ArgumentMatchers.any;
+import static org.mockito.ArgumentMatchers.anyBoolean;
 import static org.mockito.ArgumentMatchers.anyLong;
 import static org.mockito.ArgumentMatchers.anyString;
 import static org.mockito.Mockito.doThrow;
@@ -48,12 +50,15 @@ import static org.mockito.Mockito.when;
 
 /**
  * Test operation permission check result.
+ * Test-cases in this class verify that permission denied errors from OM
+ * are handled correctly by S3 endpoints.
  */
 public class TestPermissionCheck {
   private OzoneConfiguration conf;
   private OzoneClient client;
-  private ObjectStore objectStore;
+  private ClientProtocol clientProtocol;
   private OzoneBucket bucket;
+  private ObjectStore objectStore;
   private OzoneVolume volume;
   private OMException exception;
   private HttpHeaders headers;
@@ -67,11 +72,14 @@ public class TestPermissionCheck {
     objectStore = Mockito.mock(ObjectStore.class);
     bucket = Mockito.mock(OzoneBucket.class);
     volume = Mockito.mock(OzoneVolume.class);
+    when(volume.getName()).thenReturn("s3Volume");
     exception = new OMException("Permission Denied",
         OMException.ResultCodes.PERMISSION_DENIED);
     Mockito.when(client.getObjectStore()).thenReturn(objectStore);
     Mockito.when(client.getConfiguration()).thenReturn(conf);
     headers = Mockito.mock(HttpHeaders.class);
+    clientProtocol = Mockito.mock(ClientProtocol.class);
+    Mockito.when(client.getProxy()).thenReturn(clientProtocol);
   }
 
   /**
@@ -79,7 +87,7 @@ public class TestPermissionCheck {
    */
   @Test
   public void testListS3Buckets() throws IOException {
-    doThrow(exception).when(objectStore).getVolume(anyString());
+    doThrow(exception).when(objectStore).getS3Volume();
     RootEndpoint rootEndpoint = new RootEndpoint();
     rootEndpoint.setClient(client);
 
@@ -166,7 +174,7 @@ public class TestPermissionCheck {
 
     try {
       bucketEndpoint.get("bucketName", null, null, null, 1000,
-          null, null, null, null, null, null, null);
+          null, null, null, null, null, null);
       Assert.fail("Should fail");
     } catch (Exception e) {
       Assert.assertTrue(e instanceof OS3Exception);
@@ -196,7 +204,7 @@ public class TestPermissionCheck {
 
   @Test
   public void testGetAcl() throws Exception {
-    Mockito.when(objectStore.getVolume(anyString())).thenReturn(volume);
+    Mockito.when(objectStore.getS3Volume()).thenReturn(volume);
     Mockito.when(objectStore.getS3Bucket(anyString())).thenReturn(bucket);
     doThrow(exception).when(bucket).getAcls();
 
@@ -211,16 +219,16 @@ public class TestPermissionCheck {
     bucketEndpoint.setClient(client);
     try {
       bucketEndpoint.get("bucketName", null, null, null, 1000,
-          null, null, null, null, null, "acl", null);
-    } catch (Exception e) {
-      Assert.assertTrue(e instanceof OS3Exception &&
-          ((OS3Exception)e).getHttpCode() == HTTP_FORBIDDEN);
+          null, null, null, null, "acl", null);
+      Assert.fail("Expected OS3Exception with FORBIDDEN http code.");
+    } catch (OS3Exception e) {
+      Assert.assertEquals(HTTP_FORBIDDEN, e.getHttpCode());
     }
   }
 
   @Test
   public void testSetAcl() throws Exception {
-    Mockito.when(objectStore.getVolume(anyString())).thenReturn(volume);
+    Mockito.when(objectStore.getS3Volume()).thenReturn(volume);
     Mockito.when(objectStore.getS3Bucket(anyString())).thenReturn(bucket);
     doThrow(exception).when(bucket).addAcl(any());
 
@@ -246,11 +254,14 @@ public class TestPermissionCheck {
    */
   @Test
   public void testGetKey() throws IOException {
-    Mockito.when(objectStore.getS3Bucket(anyString())).thenReturn(bucket);
-    doThrow(exception).when(bucket).getKey(anyString());
+    Mockito.when(objectStore.getS3Volume()).thenReturn(volume);
+    Mockito.when(client.getProxy()).thenReturn(clientProtocol);
+    doThrow(exception).when(clientProtocol)
+        .getKeyDetails(anyString(), anyString(), anyString());
     ObjectEndpoint objectEndpoint = new ObjectEndpoint();
     objectEndpoint.setClient(client);
     objectEndpoint.setHeaders(headers);
+    objectEndpoint.setOzoneConfiguration(conf);
 
     try {
       objectEndpoint.get("bucketName", "keyPath", null, 1000, "marker",
@@ -265,12 +276,14 @@ public class TestPermissionCheck {
 
   @Test
   public void testPutKey() throws IOException {
-    Mockito.when(objectStore.getS3Bucket(anyString())).thenReturn(bucket);
-    doThrow(exception).when(bucket)
-        .createKey(anyString(), anyLong(), any(), any(), any());
+    Mockito.when(objectStore.getS3Volume()).thenReturn(volume);
+    Mockito.when(volume.getBucket("bucketName")).thenReturn(bucket);
+    doThrow(exception).when(clientProtocol).createKey(
+            anyString(), anyString(), anyString(), anyLong(), any(), any());
     ObjectEndpoint objectEndpoint = new ObjectEndpoint();
     objectEndpoint.setClient(client);
     objectEndpoint.setHeaders(headers);
+    objectEndpoint.setOzoneConfiguration(conf);
 
     try {
       objectEndpoint.put("bucketName", "keyPath", 1024, 0, null, null);
@@ -283,11 +296,13 @@ public class TestPermissionCheck {
 
   @Test
   public void testDeleteKey() throws IOException {
-    Mockito.when(objectStore.getS3Bucket(anyString())).thenReturn(bucket);
-    doThrow(exception).when(bucket).deleteKey(anyString());
+    Mockito.when(objectStore.getS3Volume()).thenReturn(volume);
+    doThrow(exception).when(clientProtocol).deleteKey(anyString(), anyString(),
+        anyString(), anyBoolean());
     ObjectEndpoint objectEndpoint = new ObjectEndpoint();
     objectEndpoint.setClient(client);
     objectEndpoint.setHeaders(headers);
+    objectEndpoint.setOzoneConfiguration(conf);
 
     try {
       objectEndpoint.delete("bucketName", "keyPath", null);
@@ -301,11 +316,11 @@ public class TestPermissionCheck {
   @Test
   public void testMultiUploadKey() throws IOException {
     Mockito.when(objectStore.getS3Bucket(anyString())).thenReturn(bucket);
-    doThrow(exception).when(bucket)
-        .initiateMultipartUpload(anyString(), any(), any());
+    doThrow(exception).when(bucket).initiateMultipartUpload(anyString(), any());
     ObjectEndpoint objectEndpoint = new ObjectEndpoint();
     objectEndpoint.setClient(client);
     objectEndpoint.setHeaders(headers);
+    objectEndpoint.setOzoneConfiguration(conf);
 
     try {
       objectEndpoint.initializeMultipartUpload("bucketName", "keyPath");
