@@ -18,6 +18,25 @@
 
 package org.apache.hadoop.ozone.debug;
 
+import com.google.common.primitives.Longs;
+import com.google.gson.Gson;
+import com.google.gson.GsonBuilder;
+import org.apache.hadoop.hdds.cli.SubcommandWithParent;
+import org.apache.hadoop.hdds.conf.OzoneConfiguration;
+import org.apache.hadoop.hdds.utils.db.DBColumnFamilyDefinition;
+import org.apache.hadoop.hdds.utils.db.DBDefinition;
+import org.apache.hadoop.hdds.utils.db.FixedLengthStringUtils;
+import org.apache.hadoop.hdds.utils.db.managed.ManagedReadOptions;
+import org.apache.hadoop.hdds.utils.db.managed.ManagedRocksDB;
+import org.apache.hadoop.hdds.utils.db.managed.ManagedRocksIterator;
+import org.apache.hadoop.hdds.utils.db.managed.ManagedSlice;
+import org.apache.hadoop.ozone.OzoneConsts;
+import org.apache.hadoop.ozone.container.metadata.DatanodeSchemaThreeDBDefinition;
+import org.kohsuke.MetaInfServices;
+import org.rocksdb.ColumnFamilyDescriptor;
+import org.rocksdb.ColumnFamilyHandle;
+import picocli.CommandLine;
+
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.OutputStreamWriter;
@@ -31,22 +50,8 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.concurrent.Callable;
 
-import org.apache.hadoop.hdds.cli.SubcommandWithParent;
-import org.apache.hadoop.hdds.utils.db.DBColumnFamilyDefinition;
-import org.apache.hadoop.hdds.utils.db.DBDefinition;
-import org.apache.hadoop.ozone.OzoneConsts;
-
-import com.google.gson.Gson;
-import com.google.gson.GsonBuilder;
-import org.kohsuke.MetaInfServices;
-import org.rocksdb.ColumnFamilyDescriptor;
-import org.rocksdb.ColumnFamilyHandle;
-import org.rocksdb.RocksDB;
-import org.rocksdb.RocksIterator;
-import picocli.CommandLine;
-
 /**
- * Parser for scm.db file.
+ * Parser for scm.db, om.db or container db file.
  */
 @CommandLine.Command(
         name = "scan",
@@ -75,6 +80,16 @@ public class DBScanner implements Callable<Void>, SubcommandWithParent {
       description = "File to dump table scan data")
   private static String fileName;
 
+  @CommandLine.Option(names = {"--dnSchema", "-d"},
+      description = "Datanode DB Schema Version : V1/V2/V3",
+      defaultValue = "V2")
+  private static String dnDBSchemaVersion;
+
+  @CommandLine.Option(names = {"--container-id", "-cid"},
+      description = "Container ID when datanode DB Schema is V3",
+      defaultValue = "-1")
+  private static long containerId;
+
   @CommandLine.ParentCommand
   private RDBParser parent;
 
@@ -82,10 +97,9 @@ public class DBScanner implements Callable<Void>, SubcommandWithParent {
 
   private List<Object> scannedObjects;
 
-  private static List<Object> displayTable(RocksIterator iterator,
+  private static List<Object> displayTable(ManagedRocksIterator iterator,
       DBColumnFamilyDefinition dbColumnFamilyDefinition) throws IOException {
     List<Object> outputs = new ArrayList<>();
-    iterator.seekToFirst();
 
     Writer fileWriter = null;
     PrintWriter printWriter = null;
@@ -95,17 +109,29 @@ public class DBScanner implements Callable<Void>, SubcommandWithParent {
             new FileOutputStream(fileName), StandardCharsets.UTF_8);
         printWriter = new PrintWriter(fileWriter);
       }
-      while (iterator.isValid()) {
+
+      boolean schemaV3 = dnDBSchemaVersion != null &&
+          dnDBSchemaVersion.equals("V3");
+      while (iterator.get().isValid()) {
         StringBuilder result = new StringBuilder();
         if (withKey) {
           Object key = dbColumnFamilyDefinition.getKeyCodec()
-              .fromPersistedFormat(iterator.key());
+              .fromPersistedFormat(iterator.get().key());
           Gson gson = new GsonBuilder().setPrettyPrinting().create();
-          result.append(gson.toJson(key));
+          if (schemaV3) {
+            int index =
+                DatanodeSchemaThreeDBDefinition.getContainerKeyPrefixLength();
+            String cid = key.toString().substring(0, index);
+            String blockId = key.toString().substring(index);
+            result.append(gson.toJson(Longs.fromByteArray(
+                FixedLengthStringUtils.string2Bytes(cid)) + ": " + blockId));
+          } else {
+            result.append(gson.toJson(key));
+          }
           result.append(" -> ");
         }
         Object o = dbColumnFamilyDefinition.getValueCodec()
-            .fromPersistedFormat(iterator.value());
+            .fromPersistedFormat(iterator.get().value());
         outputs.add(o);
         Gson gson = new GsonBuilder().setPrettyPrinting().create();
         result.append(gson.toJson(o));
@@ -115,7 +141,7 @@ public class DBScanner implements Callable<Void>, SubcommandWithParent {
           System.out.println(result.toString());
         }
         limit--;
-        iterator.next();
+        iterator.get().next();
         if (limit == 0) {
           break;
         }
@@ -155,6 +181,18 @@ public class DBScanner implements Callable<Void>, SubcommandWithParent {
     DBScanner.fileName = name;
   }
 
+  public static void setContainerId(long id) {
+    DBScanner.containerId = id;
+  }
+
+  public static void setDnDBSchemaVersion(String version) {
+    DBScanner.dnDBSchemaVersion = version;
+  }
+
+  public static void setWithKey(boolean withKey) {
+    DBScanner.withKey = withKey;
+  }
+
   private static ColumnFamilyHandle getColumnFamilyHandle(
             byte[] name, List<ColumnFamilyHandle> columnFamilyHandles) {
     return columnFamilyHandles
@@ -172,7 +210,7 @@ public class DBScanner implements Callable<Void>, SubcommandWithParent {
   }
 
   private void constructColumnFamilyMap(DBDefinition dbDefinition) {
-    if (dbDefinition == null){
+    if (dbDefinition == null) {
       System.out.println("Incorrect Db Path");
       return;
     }
@@ -193,7 +231,7 @@ public class DBScanner implements Callable<Void>, SubcommandWithParent {
 
     final List<ColumnFamilyHandle> columnFamilyHandleList =
         new ArrayList<>();
-    RocksDB rocksDB = RocksDB.openReadOnly(parent.getDbPath(),
+    ManagedRocksDB rocksDB = ManagedRocksDB.openReadOnly(parent.getDbPath(),
             cfs, columnFamilyHandleList);
     this.printAppropriateTable(columnFamilyHandleList,
            rocksDB, parent.getDbPath());
@@ -202,16 +240,17 @@ public class DBScanner implements Callable<Void>, SubcommandWithParent {
 
   private void printAppropriateTable(
           List<ColumnFamilyHandle> columnFamilyHandleList,
-          RocksDB rocksDB, String dbPath) throws IOException {
+          ManagedRocksDB rocksDB, String dbPath) throws IOException {
     if (limit < 1 && limit != -1) {
       throw new IllegalArgumentException(
               "List length should be a positive number. Only allowed negative" +
                   " number is -1 which is to dump entire table");
     }
     dbPath = removeTrailingSlashIfNeeded(dbPath);
+    DBDefinitionFactory.setDnDBSchemaVersion(dnDBSchemaVersion);
     this.constructColumnFamilyMap(DBDefinitionFactory.
-            getDefinition(Paths.get(dbPath)));
-    if (this.columnFamilyMap !=null) {
+            getDefinition(Paths.get(dbPath), new OzoneConfiguration()));
+    if (this.columnFamilyMap != null) {
       if (!this.columnFamilyMap.containsKey(tableName)) {
         System.out.print("Table with name:" + tableName + " does not exist");
       } else {
@@ -224,7 +263,24 @@ public class DBScanner implements Callable<Void>, SubcommandWithParent {
         if (columnFamilyHandle == null) {
           throw new IllegalArgumentException("columnFamilyHandle is null");
         }
-        RocksIterator iterator = rocksDB.newIterator(columnFamilyHandle);
+        ManagedRocksIterator iterator;
+        if (containerId > 0 && dnDBSchemaVersion != null &&
+            dnDBSchemaVersion.equals("V3")) {
+          ManagedReadOptions readOptions = new ManagedReadOptions();
+          readOptions.setIterateUpperBound(new ManagedSlice(
+              FixedLengthStringUtils.string2Bytes(
+                  DatanodeSchemaThreeDBDefinition.getContainerKeyPrefix(
+                  containerId + 1))));
+          iterator = new ManagedRocksIterator(
+              rocksDB.get().newIterator(columnFamilyHandle, readOptions));
+          iterator.get().seek(FixedLengthStringUtils.string2Bytes(
+              DatanodeSchemaThreeDBDefinition.getContainerKeyPrefix(
+                  containerId)));
+        } else {
+          iterator = new ManagedRocksIterator(
+              rocksDB.get().newIterator(columnFamilyHandle));
+          iterator.get().seekToFirst();
+        }
         scannedObjects = displayTable(iterator, columnFamilyDefinition);
       }
     } else {
@@ -233,8 +289,8 @@ public class DBScanner implements Callable<Void>, SubcommandWithParent {
   }
 
   private String removeTrailingSlashIfNeeded(String dbPath) {
-    if(dbPath.endsWith(OzoneConsts.OZONE_URI_DELIMITER)){
-      dbPath = dbPath.substring(0, dbPath.length()-1);
+    if (dbPath.endsWith(OzoneConsts.OZONE_URI_DELIMITER)) {
+      dbPath = dbPath.substring(0, dbPath.length() - 1);
     }
     return dbPath;
   }
