@@ -52,6 +52,7 @@ import picocli.CommandLine.Option;
         HadoopFsValidator.class,
         SameKeyReader.class,
         S3KeyGenerator.class,
+        S3BucketGenerator.class,
         DatanodeChunkGenerator.class,
         DatanodeChunkValidator.class,
         DatanodeBlockPutter.class,
@@ -62,7 +63,12 @@ import picocli.CommandLine.Option;
         GeneratorScm.class,
         GeneratorDatanode.class,
         ClosedContainerReplicator.class,
-        StreamingGenerator.class},
+        StreamingGenerator.class,
+        SCMThroughputBenchmark.class,
+        OmBucketReadWriteFileOps.class,
+        OmBucketReadWriteKeyOps.class,
+        OmRPCLoadGenerator.class,
+    },
     versionProvider = HddsVersionProvider.class,
     mixinStandardHelpOptions = true)
 public class Freon extends GenericCli {
@@ -84,11 +90,11 @@ public class Freon extends GenericCli {
   private OzoneConfiguration conf;
 
   @Override
-  public void execute(String[] argv) {
+  public int execute(String[] argv) {
     conf = createOzoneConfiguration();
     HddsServerUtil.initializeMetrics(conf, "ozone-freon");
     TracingUtil.initTracing("freon", conf);
-    super.execute(argv);
+    return super.execute(argv);
   }
 
   public void stopHttpServer() {
