@@ -41,8 +41,7 @@ import java.util.function.LongSupplier;
 import org.apache.hadoop.hdds.HddsConfigKeys;
 import org.apache.hadoop.hdds.StringUtils;
 import org.apache.hadoop.hdds.cli.HddsVersionProvider;
-import org.apache.hadoop.hdds.client.ReplicationFactor;
-import org.apache.hadoop.hdds.client.ReplicationType;
+import org.apache.hadoop.hdds.client.ReplicationConfig;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.tracing.TracingUtil;
 import org.apache.hadoop.ozone.OzoneConsts;
@@ -72,6 +71,7 @@ import org.apache.commons.lang3.time.DurationFormatUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import picocli.CommandLine.Command;
+import picocli.CommandLine.Mixin;
 import picocli.CommandLine.Option;
 import picocli.CommandLine.ParentCommand;
 
@@ -86,6 +86,7 @@ import static org.apache.hadoop.ozone.conf.OzoneServiceConfig.DEFAULT_SHUTDOWN_H
     versionProvider = HddsVersionProvider.class,
     mixinStandardHelpOptions = true,
     showDefaultValues = true)
+@SuppressWarnings("java:S2245") // no need for secure random
 public final class RandomKeyGenerator implements Callable<Void> {
 
   @ParentCommand
@@ -157,6 +158,12 @@ public final class RandomKeyGenerator implements Callable<Void> {
   )
   private boolean validateWrites = false;
 
+  @Option(names = {"--num-of-validate-threads", "--numOfValidateThreads"},
+      description = "number of threads to be launched for validating keys." +
+          "Full name --numOfValidateThreads will be removed in later versions.",
+      defaultValue = "1")
+  private int numOfValidateThreads = 1;
+
   @Option(
       names = {"--buffer-size", "--bufferSize"},
       description = "Specifies the buffer size while writing. Full name " +
@@ -171,20 +178,8 @@ public final class RandomKeyGenerator implements Callable<Void> {
   )
   private String jsonDir;
 
-  @Option(
-      names = {"--replication-type", "--replicationType"},
-      description = "Replication type (STAND_ALONE, RATIS). Full name " +
-          "--replicationType will be removed in later versions.",
-      defaultValue = "STAND_ALONE"
-  )
-  private ReplicationType type = ReplicationType.STAND_ALONE;
-
-  @Option(
-      names = "--factor",
-      description = "Replication factor (ONE, THREE)",
-      defaultValue = "ONE"
-  )
-  private ReplicationFactor factor = ReplicationFactor.ONE;
+  @Mixin
+  private FreonReplicationOptions replication;
 
   @Option(
       names = "--om-service-id",
@@ -199,6 +194,8 @@ public final class RandomKeyGenerator implements Callable<Void> {
   )
   private boolean cleanObjects = false;
 
+  private ReplicationConfig replicationConfig;
+
   private int threadPoolSize;
 
   private OzoneClient ozoneClient;
@@ -231,9 +228,9 @@ public final class RandomKeyGenerator implements Callable<Void> {
   private AtomicInteger numberOfBucketsCleaned;
   private AtomicInteger numberOfVolumesCleaned;
 
-  private Long totalWritesValidated;
-  private Long writeValidationSuccessCount;
-  private Long writeValidationFailureCount;
+  private AtomicLong totalWritesValidated;
+  private AtomicLong writeValidationSuccessCount;
+  private AtomicLong writeValidationFailureCount;
 
   private BlockingQueue<KeyValidate> validationQueue;
   private ArrayList<Histogram> histograms = new ArrayList<>();
@@ -242,6 +239,7 @@ public final class RandomKeyGenerator implements Callable<Void> {
   private ProgressBar progressbar;
 
   RandomKeyGenerator() {
+    // for picocli
   }
 
   @VisibleForTesting
@@ -284,18 +282,19 @@ public final class RandomKeyGenerator implements Callable<Void> {
 
   @Override
   public Void call() throws Exception {
-    if (ozoneConfiguration != null) {
-      if (!ozoneConfiguration.getBoolean(
-          HddsConfigKeys.HDDS_CONTAINER_PERSISTDATA,
-          HddsConfigKeys.HDDS_CONTAINER_PERSISTDATA_DEFAULT)) {
-        LOG.info("Override validateWrites to false, because "
-            + HddsConfigKeys.HDDS_CONTAINER_PERSISTDATA + " is set to false.");
-        validateWrites = false;
-      }
-      init(ozoneConfiguration);
-    } else {
-      init(freon.createOzoneConfiguration());
+    if (ozoneConfiguration == null) {
+      ozoneConfiguration = freon.createOzoneConfiguration();
     }
+    if (!ozoneConfiguration.getBoolean(
+        HddsConfigKeys.HDDS_CONTAINER_PERSISTDATA,
+        HddsConfigKeys.HDDS_CONTAINER_PERSISTDATA_DEFAULT)) {
+      LOG.info("Override validateWrites to false, because "
+          + HddsConfigKeys.HDDS_CONTAINER_PERSISTDATA + " is set to false.");
+      validateWrites = false;
+    }
+    init(ozoneConfiguration);
+
+    replicationConfig = replication.fromParamsOrConfig(ozoneConfiguration);
 
     keyValueBuffer = StringUtils.string2Bytes(
         RandomStringUtils.randomAscii(bufferSize));
@@ -324,20 +323,23 @@ public final class RandomKeyGenerator implements Callable<Void> {
     LOG.info("Key size: {} bytes", keySize);
     LOG.info("Buffer size: {} bytes", bufferSize);
     LOG.info("validateWrites : {}", validateWrites);
+    LOG.info("Number of Validate Threads: {}", numOfValidateThreads);
     LOG.info("cleanObjects : {}", cleanObjects);
     for (int i = 0; i < numOfThreads; i++) {
       executor.execute(new ObjectCreator());
     }
 
-    Thread validator = null;
+    ExecutorService validateExecutor = null;
     if (validateWrites) {
-      totalWritesValidated = 0L;
-      writeValidationSuccessCount = 0L;
-      writeValidationFailureCount = 0L;
+      totalWritesValidated = new AtomicLong();
+      writeValidationSuccessCount = new AtomicLong();
+      writeValidationFailureCount = new AtomicLong();
 
       validationQueue = new LinkedBlockingQueue<>();
-      validator = new Thread(new Validator());
-      validator.start();
+      validateExecutor = Executors.newFixedThreadPool(numOfValidateThreads);
+      for (int i = 0; i < numOfValidateThreads; i++) {
+        validateExecutor.execute(new Validator());
+      }
       LOG.info("Data validation is enabled.");
     }
 
@@ -367,8 +369,17 @@ public final class RandomKeyGenerator implements Callable<Void> {
       progressbar.shutdown();
     }
 
-    if (validator != null) {
-      validator.join();
+    if (validateExecutor != null) {
+      while (!validationQueue.isEmpty()) {
+        try {
+          Thread.sleep(CHECK_INTERVAL_MILLIS);
+        } catch (InterruptedException e) {
+          throw e;
+        }
+      }
+      validateExecutor.shutdown();
+      validateExecutor.awaitTermination(Integer.MAX_VALUE,
+          TimeUnit.MILLISECONDS);
     }
     if (cleanObjects && exception == null) {
       doCleanObjects();
@@ -473,8 +484,8 @@ public final class RandomKeyGenerator implements Callable<Void> {
     out.println("Number of Volumes created: " + numberOfVolumesCreated);
     out.println("Number of Buckets created: " + numberOfBucketsCreated);
     out.println("Number of Keys added: " + numberOfKeysAdded);
-    out.println("Ratis replication factor: " + factor.name());
-    out.println("Ratis replication type: " + type.name());
+    out.println("Replication: " + replicationConfig.getReplication());
+    out.println("Replication type: " + replicationConfig.getReplicationType());
     out.println(
         "Average Time spent in volume creation: " + prettyAverageVolumeTime);
     out.println(
@@ -488,7 +499,7 @@ public final class RandomKeyGenerator implements Callable<Void> {
       out.println("Total number of writes validated: " +
           totalWritesValidated);
       out.println("Writes validated: " +
-          (100.0 * totalWritesValidated / numberOfKeysAdded.get())
+          (100.0 * totalWritesValidated.get() / numberOfKeysAdded.get())
           + " %");
       out.println("Successful validation: " +
           writeValidationSuccessCount);
@@ -625,7 +636,7 @@ public final class RandomKeyGenerator implements Callable<Void> {
    */
   @VisibleForTesting
   long getTotalKeysValidated() {
-    return totalWritesValidated;
+    return totalWritesValidated.get();
   }
 
   /**
@@ -635,7 +646,7 @@ public final class RandomKeyGenerator implements Callable<Void> {
    */
   @VisibleForTesting
   long getSuccessfulValidationCount() {
-    return writeValidationSuccessCount;
+    return writeValidationSuccessCount.get();
   }
 
   /**
@@ -645,7 +656,7 @@ public final class RandomKeyGenerator implements Callable<Void> {
    */
   @VisibleForTesting
   long getUnsuccessfulValidationCount() {
-    return validateWrites ? writeValidationFailureCount : 0;
+    return validateWrites ? writeValidationFailureCount.get() : 0;
   }
 
   /**
@@ -794,8 +805,8 @@ public final class RandomKeyGenerator implements Callable<Void> {
     try {
       try (AutoCloseable scope = TracingUtil.createActivatedSpan("createKey")) {
         long keyCreateStart = System.nanoTime();
-        try (OzoneOutputStream os = bucket.createKey(keyName, keySize, type,
-            factor, new HashMap<>())) {
+        try (OzoneOutputStream os = bucket.createKey(keyName, keySize,
+            replicationConfig, new HashMap<>())) {
           long keyCreationDuration = System.nanoTime() - keyCreateStart;
           histograms.get(FreonOps.KEY_CREATE.ordinal())
               .update(keyCreationDuration);
@@ -918,7 +929,7 @@ public final class RandomKeyGenerator implements Callable<Void> {
     private int numOfThreads;
     private String dataWritten;
     private String execTime;
-    private String replicationFactor;
+    private String replication;
     private String replicationType;
 
     private long keySize;
@@ -951,8 +962,8 @@ public final class RandomKeyGenerator implements Callable<Void> {
       this.keySize = RandomKeyGenerator.this.keySize;
       this.bufferSize = RandomKeyGenerator.this.bufferSize;
       this.jobStartTime = Time.formatTime(RandomKeyGenerator.this.jobStartTime);
-      this.replicationFactor = RandomKeyGenerator.this.factor.name();
-      this.replicationType = RandomKeyGenerator.this.type.name();
+      replicationType = replicationConfig.getReplicationType().name();
+      replication = replicationConfig.getReplication();
 
       long totalBytes =
           (long) numOfVolumes * numOfBuckets * numOfKeys * keySize;
@@ -1089,8 +1100,8 @@ public final class RandomKeyGenerator implements Callable<Void> {
       return execTime;
     }
 
-    public String getReplicationFactor() {
-      return replicationFactor;
+    public String getReplication() {
+      return replication;
     }
 
     public String getReplicationType() {
@@ -1189,11 +1200,11 @@ public final class RandomKeyGenerator implements Callable<Void> {
             OzoneInputStream is = kv.bucket.readKey(kv.keyName);
             dig.getMessageDigest().reset();
             byte[] curDigest = dig.digest(is);
-            totalWritesValidated++;
+            totalWritesValidated.getAndIncrement();
             if (MessageDigest.isEqual(kv.digest, curDigest)) {
-              writeValidationSuccessCount++;
+              writeValidationSuccessCount.getAndIncrement();
             } else {
-              writeValidationFailureCount++;
+              writeValidationFailureCount.getAndIncrement();
               LOG.warn("Data validation error for key {}/{}/{}",
                   kv.bucket.getVolumeName(), kv.bucket, kv.keyName);
               LOG.warn("Expected checksum: {}, Actual checksum: {}",
@@ -1210,53 +1221,8 @@ public final class RandomKeyGenerator implements Callable<Void> {
     }
   }
 
-  @VisibleForTesting
-  public void setNumOfVolumes(int numOfVolumes) {
-    this.numOfVolumes = numOfVolumes;
-  }
-
-  @VisibleForTesting
-  public void setNumOfBuckets(int numOfBuckets) {
-    this.numOfBuckets = numOfBuckets;
-  }
-
-  @VisibleForTesting
-  public void setNumOfKeys(int numOfKeys) {
-    this.numOfKeys = numOfKeys;
-  }
-
-  @VisibleForTesting
-  public void setNumOfThreads(int numOfThreads) {
-    this.numOfThreads = numOfThreads;
-  }
-
-  @VisibleForTesting
-  public void setKeySize(long keySize) {
-    this.keySize = keySize;
-  }
-
-  @VisibleForTesting
-  public void setType(ReplicationType type) {
-    this.type = type;
-  }
-
-  @VisibleForTesting
-  public void setFactor(ReplicationFactor factor) {
-    this.factor = factor;
-  }
-
-  @VisibleForTesting
-  public void setValidateWrites(boolean validateWrites) {
-    this.validateWrites = validateWrites;
-  }
-
   @VisibleForTesting
   public int getThreadPoolSize() {
     return threadPoolSize;
   }
-
-  @VisibleForTesting
-  public void setCleanObjects(boolean cleanObjects) {
-    this.cleanObjects = cleanObjects;
-  }
 }
